@@ -94,29 +94,45 @@ typedef struct {
 } barrelCallbackStruct;
 #endif
 
-#define BARREL_HIDE (-9)
-#define BARREL_PRACTICE_QUERY (-8)
-#define BARREL_RANDOMIZE_QUERY (-7)
-#define BARREL_SOLVE_MESSAGE (-6)
-#define BARREL_RESTORE (-5)
-#define BARREL_RESET (-4)
-#define BARREL_BLOCKED (-3)
-#define BARREL_SPACE (-2)
-#define BARREL_IGNORE (-1)
-#define BARREL_MOVED 0
-#define BARREL_SOLVED 1
-#define BARREL_CONTROL 2
-#define BARREL_PRACTICE 3
-#define BARREL_RANDOMIZE 4
-#define BARREL_ORIENT 5
-#define BARREL_PAIRS 6
-#define BARREL_DEC_X 7
-#define BARREL_INC_X 8
-#define BARREL_DEC_Y 9
-#define BARREL_INC_Y 10
-#define BARREL_COMPUTED 11
-#define BARREL_UNDO 12
-#define BARREL_SPEED 13
+#define PUZZLE_HIDE (-9)
+#define PUZZLE_PRACTICE_QUERY (-8)
+#define PUZZLE_RANDOMIZE_QUERY (-7)
+#define PUZZLE_SOLVE_MESSAGE (-6)
+#define PUZZLE_RESTORE (-5)
+#define PUZZLE_RESET (-4)
+#define PUZZLE_BLOCKED (-3)
+#define PUZZLE_SPACE (-2)
+#define PUZZLE_IGNORE (-1)
+#define PUZZLE_MOVED 0
+#define PUZZLE_SOLVED 1
+#define PUZZLE_CONTROL 2
+#define PUZZLE_PRACTICE 3
+#define PUZZLE_RANDOMIZE 4
+#define PUZZLE_ORIENTIZE 5
+#define PUZZLE_PAIRS 6
+#define PUZZLE_DEC_X 7
+#define PUZZLE_INC_X 8
+#define PUZZLE_DEC_Y 9
+#define PUZZLE_INC_Y 10
+#define PUZZLE_COMPUTED 11
+#define PUZZLE_UNDO 12
+#define PUZZLE_REDO 13
+#define PUZZLE_SPEED 14
+
+#define MENU_GET 0
+#define MENU_WRITE 1
+#define MENU_EXIT 2
+#define MENU_UNDO 3
+#define MENU_REDO 4
+#define MENU_CLEAR 5
+#define MENU_RANDOMIZE 6
+#define MENU_PRACTICE 7
+#define MENU_SOLVE 8
+#define MENU_ORIENTIZE 9
+#define MENU_PAIRS 10
+#define MENU_SPEED 11
+#define MENU_SLOW 12
+#define MENU_SOUND 13
 
 #define MINFACES 5
 #define MAXFACES 5
