@@ -8,7 +8,7 @@
 #  Taken from James G. Nourse's The Simple Solutions to Cubic Puzzles.
 #  Break ability taken from the X puzzle by Don Bennett, HP Labs
 #
-#  Copyright (c) 2003 - 2005    David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 2003 - 2006    David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -125,7 +125,7 @@ MoveTilePiece(BarrelWidget w,
         if (SolvingFlag && AbortSolvingFlag)
                 longjmp(solve_env, 1);
 #endif
-        MoveBarrelDelay(w, direction, tile, all);
+        MovePuzzleDelay(w, direction, tile, all);
 }
 
 static int
@@ -822,7 +822,7 @@ SolveRight(BarrelWidget w)
 void
 SolveSomeTiles(BarrelWidget w)
 {
-        SetBarrel(w, BARREL_RESET);
+        setPuzzle(w, PUZZLE_RESET);
 #ifdef JMP
 	if (!setjmp(solve_env))
 #endif
@@ -844,5 +844,5 @@ SolveSomeTiles(BarrelWidget w)
 #endif
 	SolvingFlag = False;
 	w->barrel.cheat = True; /* Assume the worst. */
-        SetBarrel(w, BARREL_COMPUTED);
+        setPuzzle(w, PUZZLE_COMPUTED);
 }
