@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 2003 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 2003 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -29,16 +29,17 @@
 #define IDM_WRITE	101
 #define IDM_EXIT	102
 #define IDM_HIDE	103
-#define IDM_CLEAR	200
-#define IDM_UNDO	201
-#define IDM_PRACTICE	202
-#define IDM_RANDOMIZE	203
-#define IDM_SOLVE	204
-#define IDM_ORIENT	205
-#define IDM_PAIRS	206
-#define IDM_SPEED	207
-#define IDM_SLOW	208
-#define IDM_SOUND	209
+#define IDM_UNDO	200
+#define IDM_REDO	201
+#define IDM_CLEAR	202
+#define IDM_PRACTICE	203
+#define IDM_RANDOMIZE	204
+#define IDM_SOLVE	205
+#define IDM_ORIENTIZE	206
+#define IDM_PAIRS	207
+#define IDM_SPEED	210
+#define IDM_SLOW	211
+#define IDM_SOUND	212
 #define IDM_LEFT	303
 #define IDM_RIGHT	301
 #define IDM_SHIFT_UP	300
