@@ -84,7 +84,7 @@ static BarrelRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Barrel Version 7.2.1\n"
+"Barrel Version 7.2.4\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -129,8 +129,22 @@ static const char featuresHelp[] = {
 "record should say \"practice\").  This is good for learning "
 "moves and experimenting.\n"
 "\n"
-"Click \"mouse-right\" button, or press \"R\" or \"r\" "
-"keys, to randomize the puzzle.\n"
+"Click \"mouse-right\" button, or press \"Z\" or \"z\" "
+"keys, to randomize the puzzle (this must be done first\n"
+"to set a new record).\n"
+"\n"
+"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
+"\n"
+"Press \"W\" or \"w\" keys to save (write) a puzzle.\n"
+"\n"
+"Press \"U\" or \"u\" keys to undo a move.\n"
+"\n"
+"Press \"R\" or \"r\" keys to redo a move.\n"
+"\n"
+"Press \"C\" or \"c\" keys to clear the puzzle.\n"
+"\n"
+"\"S\" or \"s\" keys reserved for the auto-solver "
+"(not implemented).\n"
 "\n"
 "Press \"O\" or \"o\" keys to toggle orient mode.  Slightly "
 "more challenging on (default is off).\n"
@@ -139,17 +153,6 @@ static const char featuresHelp[] = {
 "the tiles rotate in pairs.  This is considerably\n"
 "more challenging on (default is on).\n"
 "\n"
-"\"S\" or \"s\" keys reserved for the auto-solver "
-"(not implemented).\n"
-"\n"
-"Press \"U\" or \"u\" keys to undo a move.\n"
-"\n"
-"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
-"\n"
-"Press \"W\" or \"w\" keys to save a puzzle.\n"
-"\n"
-"Press \"C\" or \"c\" keys to clear a puzzle.\n"
-"\n"
 "Press \">\" or \".\" keys to speed up the movement of tiles.\n"
 "\n"
 "Press \"<\" or \",\" keys to slow down the movement of tiles.\n"
@@ -234,7 +237,7 @@ static char fileName[FILENAMELEN];
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap barrelIcon = None;
+static Pixmap icon = None;
 static Widget topLevel, barrel;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
@@ -472,7 +475,7 @@ Widget w
 	char *userName, *scoreFile;
 
 #ifdef WINVER
-	InitializeBarrel(w, brush);
+	InitializePuzzle(w, brush);
 
 	w->barrel.started = FALSE;
 	orient = w->barrel.orient;
@@ -535,7 +538,7 @@ Widget w
 #ifdef WINVER
 		(void) strncpy(userNameDsp, userName, USERNAMELEN);
 #else
-		char * login = getlogin();
+		char *login = getlogin();
 
 		if (login == NULL) {
 			(void) strcpy(userNameDsp, "");
@@ -560,10 +563,10 @@ Widget w
 
 #ifdef WINVER
 void
-SetBarrel(BarrelWidget w, int reason)
+setPuzzle(BarrelWidget w, int reason)
 #else
 static void
-CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
+CallbackPuzzle(Widget w, caddr_t clientData, barrelCallbackStruct *callData)
 #endif
 {
 #ifndef WINVER
@@ -585,7 +588,7 @@ CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-		case BARREL_HIDE:
+		case PUZZLE_HIDE:
 #ifdef WINVER
 			ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -595,24 +598,22 @@ CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
 #endif
 			break;
 #ifndef WINVER
-		case BARREL_PRACTICE_QUERY:
+		case PUZZLE_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(practiceDialog);
 #else
-			XtVaSetValues(barrel,
-				XtNmenu, 9, NULL); /* menu choice */
+			XtVaSetValues(barrel, XtNmenu, MENU_PRACTICE, NULL);
 #endif
 			break;
-		case BARREL_RANDOMIZE_QUERY:
+		case PUZZLE_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(randomizeDialog);
 #else
-			XtVaSetValues(barrel,
-				XtNmenu, 5, NULL); /* menu choice */
+			XtVaSetValues(barrel, XtNmenu, MENU_RANDOMIZE, NULL);
 #endif
 			break;
 #endif
-		case BARREL_SOLVE_MESSAGE:
+		case PUZZLE_SOLVE_MESSAGE:
 #ifdef WINVER
 			(void) MessageBox(w->core.hWnd, solveHelp,
 				"Auto-Solve", MB_OK);
@@ -624,7 +625,7 @@ CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case BARREL_RESTORE:
+		case PUZZLE_RESTORE:
 			if (practice) {
 				(void) strncpy(recordDsp, "practice", MESSAGELEN);
 #ifdef HAVE_MOTIF
@@ -633,30 +634,30 @@ CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
 			}
 			movesDsp = 0;
 			break;
-		case BARREL_RESET:
+		case PUZZLE_RESET:
 			movesDsp = 0;
 			break;
-		case BARREL_BLOCKED:
+		case PUZZLE_BLOCKED:
 			(void) strncpy(messageDsp, "Blocked", MESSAGELEN);
 			break;
-		case BARREL_SPACE:
+		case PUZZLE_SPACE:
 #if 0
 			/* Too annoying */
 			(void) strncpy(messageDsp, "A space can not slide",
 				MESSAGELEN);
 #endif
 			break;
-		case BARREL_IGNORE:
+		case PUZZLE_IGNORE:
 			(void) strncpy(messageDsp, "Randomize to start",
 				MESSAGELEN);
 			break;
-		case BARREL_MOVED:
+		case PUZZLE_MOVED:
 			movesDsp++;
 			SET_STARTED(w, True);
 			break;
-		case BARREL_CONTROL:
+		case PUZZLE_CONTROL:
 			return;
-		case BARREL_SOLVED:
+		case PUZZLE_SOLVED:
 			if (practice)
 				movesDsp = 0;
 			else if (cheat)
@@ -670,7 +671,7 @@ CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
 					MESSAGELEN);
 			SET_STARTED(w, False);
 			break;
-		case BARREL_PRACTICE:
+		case PUZZLE_PRACTICE:
 			movesDsp = 0;
 			practice = !practice;
 			if (!practice)
@@ -689,7 +690,7 @@ CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
 			XmToggleButtonSetState(practiceSwitch, practice, True);
 #endif
 			break;
-		case BARREL_RANDOMIZE:
+		case PUZZLE_RANDOMIZE:
 			movesDsp = 0;
 #ifdef WINVER
 			w->barrel.practice = False;
@@ -700,7 +701,7 @@ CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
 				XtNstart, False, NULL);
 #endif
 			break;
-		case BARREL_ORIENT:
+		case PUZZLE_ORIENTIZE:
 			movesDsp = 0;
 			orient = !orient;
 			PrintRecord(orient, pairs, practice);
@@ -713,7 +714,7 @@ CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case BARREL_PAIRS:
+		case PUZZLE_PAIRS:
 			movesDsp = 0;
 			pairs = !pairs;
 			PrintRecord(orient, pairs, practice);
@@ -727,7 +728,7 @@ CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
 #endif
 			break;
 #if 0
-		case BARREL_DEC_X:
+		case PUZZLE_DEC_X:
 			movesDsp = 0;
 			tiles--;
 			PrintRecord(tiles, faces, orient, pairs, practice);
@@ -742,7 +743,7 @@ CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case BARREL_INC_X:
+		case PUZZLE_INC_X:
 			movesDsp = 0;
 			tiles++;
 			PrintRecord(tiles, faces, orient, pairs, practice);
@@ -757,7 +758,7 @@ CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case BARREL_DEC_Y:
+		case PUZZLE_DEC_Y:
 			movesDsp = 0;
 			faces--;
 			PrintRecord(tiles, faces, orient, pairs, practice);
@@ -770,7 +771,7 @@ CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case BARREL_INC_Y:
+		case PUZZLE_INC_Y:
 			movesDsp = 0;
 			faces++;
 			PrintRecord(tiles, faces, orient, pairs, practice);
@@ -784,15 +785,19 @@ CallbackBarrel(Widget w, caddr_t clientData, barrelCallbackStruct * callData)
 			break;
 #endif
 #endif
-		case BARREL_COMPUTED:
+		case PUZZLE_COMPUTED:
 			SET_STARTED(w, False);
 			break;
-		case BARREL_UNDO:
+		case PUZZLE_UNDO:
 			movesDsp--;
 			SET_STARTED(w, True);
 			break;
+		case PUZZLE_REDO:
+			movesDsp++;
+			SET_STARTED(w, True);
+			break;
 #ifdef HAVE_MOTIF
-		case BARREL_SPEED:
+		case PUZZLE_SPEED:
 			{
 				int oldDelay, delay;
 
@@ -845,15 +850,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	if (GetFocus()) {
 		if (!widget.barrel.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			EnterBarrel(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			EnterPuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	} else {
 		if (widget.barrel.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			LeaveBarrel(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			LeavePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	}
@@ -862,29 +869,32 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			Initialize(&widget, brush);
 			break;
 		case WM_DESTROY:
-			DestroyBarrel(brush);
+			DestroyPuzzle(&widget, brush);
 			break;
 		case WM_SIZE:
-			ResizeBarrel(&widget);
+			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
 		case WM_PAINT:
 			widget.core.hDC = BeginPaint(hWnd, &paint);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ExposeBarrel(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ExposePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 			break;
 		case WM_RBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			RandomizeBarrel(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONDOWN:
 			mousePressed = True;
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			SelectBarrel(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			SelectPuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
@@ -893,31 +903,35 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				if (mousePressed) {
 					mousePressed = False;
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					ReleaseBarrel(&widget, LOWORD(lParam), HIWORD(lParam),
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					ReleasePuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)));
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 				}
 				break;
 			}
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			MotionBarrel(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			MotionPuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONUP:
 			mousePressed = False;
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ReleaseBarrel(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ReleasePuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 #if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
 		case WM_MOUSEWHEEL:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
 			{
 				int zDelta = ((short) HIWORD(wParam));
 				POINT cursor, origin;
@@ -926,10 +940,10 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				ClientToScreen(hWnd, &origin);
 				(void) GetCursorPos(&cursor);
 				if (zDelta > (WHEEL_DELTA >> 1)) {
-					MoveBarrelInput(&widget, cursor.x - origin.x, TOP,
+					MovePuzzleInput(&widget, cursor.x - origin.x, TOP,
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)));
 				} else if (zDelta < -(WHEEL_DELTA >> 1)) {
-					MoveBarrelInput(&widget, cursor.x - origin.x, BOTTOM,
+					MovePuzzleInput(&widget, cursor.x - origin.x, BOTTOM,
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)));
 				}
 			}
@@ -939,74 +953,86 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		case WM_COMMAND:
 			switch LOWORD((wParam)) {
 				case IDM_GET:
-					GetBarrel(&widget);
-					ResizeBarrel(&widget);
+					GetPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_WRITE:
-					WriteBarrel(&widget);
+					WritePuzzle(&widget);
 					break;
 				case IDM_EXIT:
-					DestroyBarrel(brush);
+					DestroyPuzzle(&widget, brush);
 					break;
 				case IDM_HIDE:
-					HideBarrel(&widget);
-					break;
-				case IDM_CLEAR:
-					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					ClearBarrel(&widget);
-					(void) ReleaseDC(hWnd, widget.core.hDC);
+					HidePuzzle(&widget);
 					break;
 				case IDM_UNDO:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					UndoBarrel(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					UndoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_REDO:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RedoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_CLEAR:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					ClearPuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_RANDOMIZE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					RandomizeBarrel(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RandomizePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_PRACTICE:
-					PracticeBarrel(&widget);
-					SizeBarrel(&widget);
+					PracticePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SOLVE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					SolveBarrel(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					SolvePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
-				case IDM_ORIENT:
-					OrientizeBarrel(&widget);
-					SizeBarrel(&widget);
+				case IDM_ORIENTIZE:
+					OrientizePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_PAIRS:
-					(void) PairsBarrel(&widget);
-					SizeBarrel(&widget);
+					(void) PairsPuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SPEED:
-					SpeedBarrel(&widget);
+					SpeedPuzzle(&widget);
 					break;
 				case IDM_SLOW:
-					SlowBarrel(&widget);
+					SlowPuzzle(&widget);
 					break;
 				case IDM_SOUND:
-					SoundBarrel(&widget);
+					SoundPuzzle(&widget);
 					break;
 				case IDM_LEFT:
 				case IDM_RIGHT:
 				case IDM_SHIFT_UP:
 				case IDM_SHIFT_DOWN:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					(void) MoveBarrelInput(&widget, 0,
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					(void) MovePuzzleInput(&widget, 0,
 							(int) LOWORD(wParam) - IDM_SHIFT_UP, !((int) wParam % 2));
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
@@ -1016,11 +1042,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
-						(void) MoveBarrelInput(&widget, cursor.x - origin.x,
+						(void) MovePuzzleInput(&widget, cursor.x - origin.x,
 								(int) LOWORD(wParam) - IDM_UP, FALSE);
 						(void) ReleaseDC(hWnd, widget.core.hDC);
 					}
@@ -1104,7 +1131,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
 #else
 
 static void
-Usage(char * programName)
+Usage(char *programName)
 {
 	unsigned int i;
 
@@ -1138,14 +1165,6 @@ static XrmOptionDescRec options[] =
 	{(char *) "-face3", (char *) "*barrel.faceColor3", XrmoptionSepArg, NULL},
 	{(char *) "-face4", (char *) "*barrel.faceColor4", XrmoptionSepArg, NULL},
 	{(char *) "-face5", (char *) "*barrel.faceColor5", XrmoptionSepArg, NULL},
-#if 0
-	{(char *) "-tiles", (char *) "*barrel.tiles", XrmoptionSepArg, NULL},
-	{(char *) "-faces", (char *) "*barrel.faces", XrmoptionSepArg, NULL},
-#endif
-	{(char *) "-orient", (char *) "*barrel.orient", XrmoptionNoArg, (char *) "TRUE"},
-	{(char *) "-noorient", (char *) "*barrel.orient", XrmoptionNoArg, (char *) "FALSE"},
-	{(char *) "-pairs", (char *) "*barrel.pairs", XrmoptionNoArg, (char *) "TRUE"},
-	{(char *) "-nopairs", (char *) "*barrel.pairs", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-install", (char *) "*barrel.install", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-noinstall", (char *) "*barrel.install", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-picture", (char *) "*barrel.picture", XrmoptionSepArg, NULL},
@@ -1157,6 +1176,14 @@ static XrmOptionDescRec options[] =
 	{(char *) "-base", (char *) "*barrel.base", XrmoptionSepArg, NULL},
 	{(char *) "-fn", (char *) "*barrel.font", XrmoptionSepArg, NULL},
 	{(char *) "-font", (char *) "*barrel.font", XrmoptionSepArg, NULL},
+#if 0
+	{(char *) "-tiles", (char *) "*barrel.tiles", XrmoptionSepArg, NULL},
+	{(char *) "-faces", (char *) "*barrel.faces", XrmoptionSepArg, NULL},
+#endif
+	{(char *) "-orient", (char *) "*barrel.orient", XrmoptionNoArg, (char *) "TRUE"},
+	{(char *) "-noorient", (char *) "*barrel.orient", XrmoptionNoArg, (char *) "FALSE"},
+	{(char *) "-pairs", (char *) "*barrel.pairs", XrmoptionNoArg, (char *) "TRUE"},
+	{(char *) "-nopairs", (char *) "*barrel.pairs", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-userName", (char *) "*barrel.userName", XrmoptionSepArg, NULL},
 	{(char *) "-scoreFile", (char *) "*barrel.scoreFile", XrmoptionSepArg, NULL},
 	{(char *) "-scores", (char *) "*barrel.scoreOnly", XrmoptionNoArg, (char *) "TRUE"},
@@ -1165,26 +1192,26 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackBarrelPractice(Widget w, XtPointer clientData,
-		XmAnyCallbackStruct * cbs)
+CallbackPuzzlePractice(Widget w, XtPointer clientData,
+		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(barrel, XtNmenu, 9, NULL); /* menu choice */
+		XtVaSetValues(barrel, XtNmenu, MENU_PRACTICE, NULL);
 	}
 }
 
 static void
-CallbackBarrelRandomize(Widget w, XtPointer clientData,
-		XmAnyCallbackStruct * cbs)
+CallbackPuzzleRandomize(Widget w, XtPointer clientData,
+		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(barrel, XtNmenu, 5, NULL); /* menu choice */
+		XtVaSetValues(barrel, XtNmenu, MENU_RANDOMIZE, NULL);
 	}
 }
 
 #if 0
 static void
-TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int tiles = cbs->value, faces, old;
 	Boolean orient, pairs, practice;
@@ -1208,7 +1235,7 @@ TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-FaceSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+FaceSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int tiles, faces = cbs->value, old;
 	Boolean orient, pairs, practice;
@@ -1233,7 +1260,7 @@ FaceSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 #endif
 
 static void
-OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	Boolean orient = cbs->set, pairs, practice, oldOrient;
 	char *picture;
@@ -1260,7 +1287,7 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
 }
 
 static void
-PairsToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+PairsToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	Boolean orient, pairs = cbs->set, practice, oldPairs;
 
@@ -1282,7 +1309,7 @@ PairsToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
 
 static void
 PracticeToggle(Widget w, XtPointer clientData,
-		XmToggleButtonCallbackStruct * cbs)
+		XmToggleButtonCallbackStruct *cbs)
 {
 	Boolean orient, pairs, practice = cbs->set, oldPract;
 
@@ -1307,7 +1334,7 @@ PracticeToggle(Widget w, XtPointer clientData,
 }
 
 static void
-SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MAXSPEED + MINSPEED - cbs->value - 1, oldDelay;
 
@@ -1323,9 +1350,9 @@ fileCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	if (val == 2)
+	if (val == MENU_EXIT)
 		exit(0);
-	XtVaSetValues(barrel, XtNmenu, val, NULL);
+	XtVaSetValues(barrel, XtNmenu, val, NULL); /* GWQ */
 }
 
 static void
@@ -1333,7 +1360,7 @@ playCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	XtVaSetValues(barrel, XtNmenu, val + 3, NULL); /* GWQ */
+	XtVaSetValues(barrel, XtNmenu, val + MENU_EXIT + 1, NULL);
 }
 
 static Widget
@@ -1432,8 +1459,9 @@ main(int argc, char **argv)
 	Widget messageRowCol;
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
-	XmString clearString, undoString, randomizeString, solveString;
-	XmString orientString, pairsString, practiceString;
+	XmString undoString, redoString;
+	XmString clearString, randomizeString, practiceString, solveString;
+	XmString orientString, pairsString;
 	XmString speedString, slowString, soundString;
 #endif
 
@@ -1443,7 +1471,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#if HAVE_XPM
+#ifdef HAVE_XPM
 	{
 		XpmAttributes xpmAttributes;
 		XpmColorSymbol transparentColor[1] = {{NULL,
@@ -1458,18 +1486,18 @@ main(int argc, char **argv)
 		transparentColor[0].pixel = bg;
 		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) barrel_xpm, &barrelIcon, NULL,
+			(char **) barrel_xpm, &icon, NULL,
 			&xpmAttributes);
 	}
-	if (barrelIcon == (Pixmap) NULL)
+	if (icon == (Pixmap) NULL)
 #endif
-		barrelIcon = XCreateBitmapFromData(XtDisplay(topLevel),
+		icon = XCreateBitmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
 			(char *) barrel_bits,
 			barrel_width, barrel_height);
 #ifdef HAVE_MOTIF
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, barrelIcon,
+		XtNiconPixmap, icon,
 		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
@@ -1496,35 +1524,38 @@ main(int argc, char **argv)
 	XmStringFree(getString);
 	XmStringFree(writeString);
 	XmStringFree(quitString);
-	clearString = XmStringCreateSimple((char *) "Clear");
 	undoString = XmStringCreateSimple((char *) "Undo");
-	randomizeString = XmStringCreateSimple((char *) "Randomize");
+	redoString = XmStringCreateSimple((char *) "Redo");
+	clearString = XmStringCreateSimple((char *) "Clear");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
+	practiceString = XmStringCreateSimple((char *) "Practice");
 	solveString = XmStringCreateSimple((char *) "Solve");
 	orientString = XmStringCreateSimple((char *) "Orientize");
 	pairsString = XmStringCreateSimple((char *) "PAirs");
-	practiceString = XmStringCreateSimple((char *) "Practice");
 	speedString = XmStringCreateSimple((char *) ">Speed");
 	slowString = XmStringCreateSimple((char *) "<Slow");
 	soundString = XmStringCreateSimple((char *) "@Sound");
 	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
-		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
+		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
 		XmVaPUSHBUTTON, orientString, 'O', NULL, NULL,
 		XmVaPUSHBUTTON, pairsString, 'A', NULL, NULL,
-		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
 		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
 		XmVaPUSHBUTTON, soundString, '@', NULL, NULL,
 		NULL);
-	XmStringFree(clearString);
 	XmStringFree(undoString);
+	XmStringFree(redoString);
+	XmStringFree(clearString);
 	XmStringFree(randomizeString);
+	XmStringFree(practiceString);
 	XmStringFree(solveString);
 	XmStringFree(orientString);
 	XmStringFree(pairsString);
-	XmStringFree(practiceString);
 	XmStringFree(speedString);
 	XmStringFree(slowString);
 	XmStringFree(soundString);
@@ -1571,11 +1602,11 @@ main(int argc, char **argv)
 	practiceDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to toggle the practice mode?",
 		(char *) "Practice Query",
-		(XtCallbackProc) CallbackBarrelPractice);
+		(XtCallbackProc) CallbackPuzzlePractice);
 	randomizeDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to randomize?",
 		(char *) "Randomize Query",
-		(XtCallbackProc) CallbackBarrelRandomize);
+		(XtCallbackProc) CallbackPuzzleRandomize);
 	mainPanel = XtCreateManagedWidget("mainPanel",
 		xmPanedWindowWidgetClass, menuBarPanel,
 		NULL, 0);
@@ -1710,13 +1741,13 @@ main(int argc, char **argv)
 		barrelWidgetClass, mainPanel, NULL, 0);
 #else
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, barrelIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 	barrel = XtCreateManagedWidget("barrel",
 		barrelWidgetClass, topLevel, NULL, 0);
 #endif
 	XtAddCallback(barrel, XtNselectCallback,
-		(XtCallbackProc) CallbackBarrel, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	Initialize(barrel);
 	XtRealizeWidget(topLevel);
 	XGrabButton(XtDisplay(barrel), (unsigned int) AnyButton, AnyModifier,
