@@ -65,37 +65,49 @@
 #endif
 #endif
 
-static Boolean SetValuesCubes(Widget current, Widget request, Widget renew);
-static void QuitCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void DestroyCubes(Widget old);
-static void ResizeCubes(CubesWidget w);
-static void SizeCubes(CubesWidget w);
-static void InitializeCubes(Widget request, Widget renew);
-static void ExposeCubes(Widget renew, XEvent * event, Region region);
-static void HideCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void SelectCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void ReleaseCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void RandomizeCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void RandomizeCubesMaybe(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void RandomizeCubes2(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void GetCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void WriteCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void ClearCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void UndoCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void SolveCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void SpeedCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void SlowCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void SoundCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void EnterCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void LeaveCubes(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void MoveCubesOut(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void MoveCubesTop(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void MoveCubesLeft(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void MoveCubesIn(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void MoveCubesRight(CubesWidget w, XEvent * event, char **args, int nArgs);
-static void MoveCubesBottom(CubesWidget w, XEvent * event, char **args, int nArgs);
+static Boolean SetValuesPuzzle(Widget current, Widget request, Widget renew);
+static void QuitPuzzle(CubesWidget w, XEvent *event, char **args, int nArgs);
+static void DestroyPuzzle(Widget old);
+static void ResizePuzzle(CubesWidget w);
+static void SizePuzzle(CubesWidget w);
+static void InitializePuzzle(Widget request, Widget renew);
+static void ExposePuzzle(Widget renew, XEvent *event, Region region);
+static void HidePuzzle(CubesWidget w, XEvent *event, char **args, int nArgs);
+static void SelectPuzzle(CubesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ReleasePuzzle(CubesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ClearPuzzleMaybe(CubesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ClearPuzzle2(CubesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void GetPuzzle(CubesWidget w, XEvent *event, char **args, int nArgs);
+static void WritePuzzle(CubesWidget w, XEvent *event, char **args, int nArgs);
+static void UndoPuzzle(CubesWidget w, XEvent *event, char **args, int nArgs);
+static void RedoPuzzle(CubesWidget w, XEvent *event, char **args, int nArgs);
+static void ClearPuzzle(CubesWidget w, XEvent *event, char **args, int nArgs);
+static void RandomizePuzzle(CubesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SolvePuzzle(CubesWidget w, XEvent *event, char **args, int nArgs);
+static void SpeedPuzzle(CubesWidget w, XEvent *event, char **args, int nArgs);
+static void SlowPuzzle(CubesWidget w, XEvent *event, char **args, int nArgs);
+static void SoundPuzzle(CubesWidget w, XEvent *event, char **args, int nArgs);
+static void EnterPuzzle(CubesWidget w, XEvent *event, char **args, int nArgs);
+static void LeavePuzzle(CubesWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzleOut(CubesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleTop(CubesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleLeft(CubesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleIn(CubesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleRight(CubesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleBottom(CubesWidget w,
+	XEvent *event, char **args, int nArgs);
 
-static char defaultTranslationsCubes[] =
+static char defaultTranslationsPuzzle[] =
 "<KeyPress>q: Quit()\n\
  Ctrl<KeyPress>C: Quit()\n\
  <KeyPress>osfCancel: Hide()\n\
@@ -132,15 +144,16 @@ static char defaultTranslationsCubes[] =
  <KeyPress>R14: MoveBottom()\n\
  <Btn1Down>: Select()\n\
  <Btn1Up>: Release()\n\
- <KeyPress>r: Randomize()\n\
- <Btn3Down>: RandomizeMaybe()\n\
- <Btn3Down>(2+): Randomize2()\n\
+ <Btn3Down>: ClearMaybe()\n\
+ <Btn3Down>(2+): Clear2()\n\
  <Btn4Down>: MoveTop()\n\
  <Btn5Down>: MoveBottom()\n\
  <KeyPress>g: Get()\n\
  <KeyPress>w: Write()\n\
- <KeyPress>c: Clear()\n\
  <KeyPress>u: Undo()\n\
+ <KeyPress>r: Redo()\n\
+ <KeyPress>c: Clear()\n\
+ <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
  <KeyPress>0x2E: Speed()\n\
  <KeyPress>0x3E: Speed()\n\
@@ -151,34 +164,35 @@ static char defaultTranslationsCubes[] =
  <LeaveWindow>: Leave()";
 
 
-static XtActionsRec actionsListCubes[] =
+static XtActionsRec actionsListPuzzle[] =
 {
-	{(char *) "Quit", (XtActionProc) QuitCubes},
-	{(char *) "Hide", (XtActionProc) HideCubes},
-	{(char *) "MoveOut", (XtActionProc) MoveCubesOut},
-	{(char *) "MoveTop", (XtActionProc) MoveCubesTop},
-	{(char *) "MoveLeft", (XtActionProc) MoveCubesLeft},
-	{(char *) "MoveIn", (XtActionProc) MoveCubesIn},
-	{(char *) "MoveRight", (XtActionProc) MoveCubesRight},
-	{(char *) "MoveBottom", (XtActionProc) MoveCubesBottom},
-	{(char *) "Select", (XtActionProc) SelectCubes},
-	{(char *) "Release", (XtActionProc) ReleaseCubes},
-	{(char *) "Randomize", (XtActionProc) RandomizeCubes},
-	{(char *) "RandomizeMaybe", (XtActionProc) RandomizeCubesMaybe},
-	{(char *) "Randomize2", (XtActionProc) RandomizeCubes2},
-	{(char *) "Get", (XtActionProc) GetCubes},
-	{(char *) "Write", (XtActionProc) WriteCubes},
-	{(char *) "Clear", (XtActionProc) ClearCubes},
-	{(char *) "Undo", (XtActionProc) UndoCubes},
-	{(char *) "Solve", (XtActionProc) SolveCubes},
-	{(char *) "Speed", (XtActionProc) SpeedCubes},
-	{(char *) "Slow", (XtActionProc) SlowCubes},
-	{(char *) "Sound", (XtActionProc) SoundCubes},
-	{(char *) "Enter", (XtActionProc) EnterCubes},
-	{(char *) "Leave", (XtActionProc) LeaveCubes}
+	{(char *) "Quit", (XtActionProc) QuitPuzzle},
+	{(char *) "Hide", (XtActionProc) HidePuzzle},
+	{(char *) "MoveOut", (XtActionProc) MovePuzzleOut},
+	{(char *) "MoveTop", (XtActionProc) MovePuzzleTop},
+	{(char *) "MoveLeft", (XtActionProc) MovePuzzleLeft},
+	{(char *) "MoveIn", (XtActionProc) MovePuzzleIn},
+	{(char *) "MoveRight", (XtActionProc) MovePuzzleRight},
+	{(char *) "MoveBottom", (XtActionProc) MovePuzzleBottom},
+	{(char *) "Select", (XtActionProc) SelectPuzzle},
+	{(char *) "Release", (XtActionProc) ReleasePuzzle},
+	{(char *) "ClearMaybe", (XtActionProc) ClearPuzzleMaybe},
+	{(char *) "Clear2", (XtActionProc) ClearPuzzle2},
+	{(char *) "Get", (XtActionProc) GetPuzzle},
+	{(char *) "Write", (XtActionProc) WritePuzzle},
+	{(char *) "Undo", (XtActionProc) UndoPuzzle},
+	{(char *) "Redo", (XtActionProc) RedoPuzzle},
+	{(char *) "Clear", (XtActionProc) ClearPuzzle},
+	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
+	{(char *) "Solve", (XtActionProc) SolvePuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
+	{(char *) "Enter", (XtActionProc) EnterPuzzle},
+	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
 
-static XtResource resourcesCubes[] =
+static XtResource resourcesPuzzle[] =
 {
 	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
 	 XtOffset(CubesWidget, core.width),
@@ -275,29 +289,29 @@ CubesClassRec cubesClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeCubes,	/* initialize */
+		(XtInitProc) InitializePuzzle,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
-		actionsListCubes,	/* actions */
-		XtNumber(actionsListCubes),	/* num actions */
-		resourcesCubes,	/* resources */
-		XtNumber(resourcesCubes),	/* num resources */
+		actionsListPuzzle,	/* actions */
+		XtNumber(actionsListPuzzle),	/* num actions */
+		resourcesPuzzle,	/* resources */
+		XtNumber(resourcesPuzzle),	/* num resources */
 		NULLQUARK,	/* xrm class */
 		TRUE,		/* compress motion */
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
-		(XtWidgetProc) DestroyCubes,	/* destroy */
-		(XtWidgetProc) ResizeCubes,	/* resize */
-		(XtExposeProc) ExposeCubes,	/* expose */
-		(XtSetValuesFunc) SetValuesCubes,	/* set values */
+		(XtWidgetProc) DestroyPuzzle,	/* destroy */
+		(XtWidgetProc) ResizePuzzle,	/* resize */
+		(XtExposeProc) ExposePuzzle,	/* expose */
+		(XtSetValuesFunc) SetValuesPuzzle,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
 		NULL,		/* accept focus */
 		XtVersion,	/* version */
 		NULL,		/* callback private */
-		defaultTranslationsCubes,	/* tm table */
+		defaultTranslationsPuzzle,	/* tm table */
 		NULL,		/* query geometry */
 		NULL,		/* display accelerator */
 		NULL		/* extension */
@@ -310,7 +324,7 @@ CubesClassRec cubesClassRec =
 WidgetClass cubesWidgetClass = (WidgetClass) & cubesClassRec;
 
 void
-SetCubes(CubesWidget w, int reason)
+setPuzzle(CubesWidget w, int reason)
 {
 	cubesCallbackStruct cb;
 
@@ -331,8 +345,8 @@ loadFont(CubesWidget w)
 		XUnloadFont(XtDisplay(w), w->cubes.fontInfo->fid);
 		XFreeFont(XtDisplay(w), w->cubes.fontInfo);
 	}
-	if ((w->cubes.fontInfo = XLoadQueryFont(display,
-			w->cubes.font)) == NULL) {
+	if (w->cubes.font && (w->cubes.fontInfo =
+			XLoadQueryFont(display, w->cubes.font)) == NULL) {
 		(void) sprintf(buf,
 			"Can not open %s font.\nAttempting %s font as alternate\n",
 			w->cubes.font, altfontname);
@@ -362,6 +376,9 @@ loadFont(CubesWidget w)
 #define LOGFILE "cubes.log"
 #endif
 
+static CubesStack undo = {NULL, NULL, NULL, 0};
+static CubesStack redo = {NULL, NULL, NULL, 0};
+
 static void
 CheckBlocks(CubesWidget w)
 {
@@ -434,6 +451,11 @@ ToPosition(CubesWidget w, int column, int row, int stack)
 	return (column + w->cubes.sizeX * row + w->cubes.sizeRect * stack);
 }
 
+static int
+ToInvPosition(CubesWidget w, int column, int row, int stack) {
+	return (row + w->cubes.sizeY * column + w->cubes.sizeRect * stack);
+}
+
 static int
 Column(CubesWidget w, int pos)
 {
@@ -798,8 +820,9 @@ AnimateSlide(CubesWidget w, int numBlocks, int dir, int fast, Boolean logMoves)
 	  for (aBlock = 0; aBlock < numBlocks; aBlock++) {
 	    posNext = BlockNFromSpace(w, aBlock + 1, dir);
 	    if (logMoves && inc == 0) {
-	      SetCubes(w, CUBES_MOVED);
-	      PutMove(dir);
+	      setPuzzle(w, PUZZLE_MOVED);
+	      setMove(&undo, dir);
+	      flushMoves(w, &redo, FALSE);
 	    }
 	    /* Calculate deltas */
 	    dx = CartesianX(w, posNext);
@@ -870,7 +893,7 @@ AnimateSlide(CubesWidget w, int numBlocks, int dir, int fast, Boolean logMoves)
 		break;
 	      }
 	    }
-	  }	
+	  }
 	  FLUSH(w);
 	  useTimer(&(w->cubes.oldTime), w->cubes.delay);
 	}
@@ -1066,7 +1089,7 @@ DrawFrame(CubesWidget w, Pixmap dr, Boolean focus)
 static void
 MoveNoBlocks(CubesWidget w)
 {
-	SetCubes(w, CUBES_IGNORE);
+	setPuzzle(w, PUZZLE_IGNORE);
 }
 
 static void
@@ -1187,11 +1210,12 @@ MoveBlocksDir(CubesWidget w, int direction, int fast)
 }
 
 int
-MoveCubesDir(CubesWidget w, const int direction, const int fast)
+MovePuzzleDir(CubesWidget w, const int direction, const int fast)
 {
 	if (MoveBlocksDir(w, direction, fast)) {
-		SetCubes(w, CUBES_MOVED);
-		PutMove(direction);
+		setPuzzle(w, PUZZLE_MOVED);
+		setMove(&undo, direction);
+		flushMoves(w, &redo, FALSE);
 		return TRUE;
 	}
 	return FALSE;
@@ -1202,62 +1226,62 @@ MoveCubesDir(CubesWidget w, const int direction, const int fast)
 static
 #endif
 int
-MoveCubes(CubesWidget w, const int direction, const int control)
+MovePuzzle(CubesWidget w, const int direction, const int control)
 {
 	int reason;
 
 	if (control) {
-		reason = CUBES_IGNORE;
+		reason = PUZZLE_IGNORE;
 		switch (direction) {
 			case TOP:
 				if (w->cubes.sizeY <= MINCUBES)
 					return FALSE;
-				reason = CUBES_DEC_Y;
+				reason = PUZZLE_DEC_Y;
 				break;
 			case RIGHT:
-				reason = CUBES_INC_X;
+				reason = PUZZLE_INC_X;
 				break;
 			case BOTTOM:
-				reason = CUBES_INC_Y;
+				reason = PUZZLE_INC_Y;
 				break;
 			case LEFT:
 				if (w->cubes.sizeX <= MINCUBES)
 					return FALSE;
-				reason = CUBES_DEC_X;
+				reason = PUZZLE_DEC_X;
 				break;
 			case INWARDS:
 				if (w->cubes.sizeZ <= MINCUBES)
 					return FALSE;
-				reason = CUBES_DEC_Z;
+				reason = PUZZLE_DEC_Z;
 				break;
 			case OUTWARDS:
-				reason = CUBES_INC_Z;
+				reason = PUZZLE_INC_Z;
 				break;
 			default:
 				{
 					char *buf;
 
-					intCat(&buf, "MoveCubes: direction ",
+					intCat(&buf, "MovePuzzle: direction ",
 						direction);
 					DISPLAY_WARNING(buf);
 					free(buf);
-					SetCubes(w, reason);
+					setPuzzle(w, reason);
 					return FALSE;
 				}
 		}
-		SetCubes(w, reason);
+		setPuzzle(w, reason);
 		return TRUE;
 	}
 	if (CheckSolved(w)) {
 		MoveNoBlocks(w);
 		return FALSE;
 	}
-	if (!MoveCubesDir(w, direction, NORMAL)) {
-		SetCubes(w, CUBES_BLOCKED);
+	if (!MovePuzzleDir(w, direction, NORMAL)) {
+		setPuzzle(w, PUZZLE_BLOCKED);
 		return FALSE;
 	}
 	if (CheckSolved(w)) {
-		SetCubes(w, CUBES_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 	return TRUE;
 }
@@ -1281,8 +1305,9 @@ static void
 DiscreteMoves(CubesWidget w, int from, int dir, int fast)
 {
 	MoveBlocks(w, from, fast);
-	SetCubes(w, CUBES_MOVED);
-	PutMove(dir);
+	setPuzzle(w, PUZZLE_MOVED);
+	setMove(&undo, dir);
+	flushMoves(w, &redo, FALSE);
 }
 
 static int
@@ -1328,13 +1353,13 @@ MovableBlock(CubesWidget w)
 	/* Order important if w->cubes.sizeX = 1 */
 	if (l % w->cubes.sizeX == 0 && k == Stack(w, w->cubes.spacePosition)) {
 		if (l == 0) {
-			return CUBES_SPACE;
+			return PUZZLE_SPACE;
 		}
 	} else if (!(l / w->cubes.sizeX == 0 &&
 			j == Row(w, w->cubes.spacePosition) &&
 			k == Stack(w, w->cubes.spacePosition)) &&
 			(m % w->cubes.sizeRect != 0)) {
-		return CUBES_BLOCKED;
+		return PUZZLE_BLOCKED;
 	}
 	return 0;
 }
@@ -1384,7 +1409,7 @@ SelectBlocks(CubesWidget w)
 						TOP, NORMAL);
 			}
 		} else {	/* (l == 0) */
-			intCat(&buf, "SelectBlocks: ", CUBES_SPACE);
+			intCat(&buf, "SelectBlocks: ", PUZZLE_SPACE);
 			DISPLAY_WARNING(buf);
 			free(buf);
 			return;
@@ -1436,20 +1461,20 @@ SelectBlocks(CubesWidget w)
 			}
 		}
 	} else {
-		intCat(&buf, "SelectBlocks: ", CUBES_BLOCKED);
+		intCat(&buf, "SelectBlocks: ", PUZZLE_BLOCKED);
 		DISPLAY_WARNING(buf);
 		free(buf);
 		return;
 	}
 	if (CheckSolved(w)) {
-		SetCubes(w, CUBES_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
 static void
 ResetBlocks(CubesWidget w)
 {
-	int i;
+	int i, j, switch1 = 0, switch2 = 0;
 
 	w->cubes.sizeRect = w->cubes.sizeX * w->cubes.sizeY;
 	w->cubes.sizeBlock = w->cubes.sizeRect * w->cubes.sizeZ;
@@ -1459,19 +1484,68 @@ ResetBlocks(CubesWidget w)
 			malloc(sizeof (int) * w->cubes.sizeBlock))) {
 		DISPLAY_ERROR("Not enough memory, exiting.");
 	}
-
 	if (startPosition)
 		free(startPosition);
 	if (!(startPosition = (int *)
 			malloc(sizeof (int) * w->cubes.sizeBlock))) {
 		DISPLAY_ERROR("Not enough memory, exiting.");
 	}
-
-	w->cubes.spacePosition = w->cubes.sizeBlock - 1;
+	/*w->cubes.spacePosition = w->cubes.sizeBlock - 1;
 	w->cubes.blockOfPosition[w->cubes.sizeBlock - 1] = 0;
 	for (i = 1; i < w->cubes.sizeBlock; i++)
-		w->cubes.blockOfPosition[i - 1] = i;
-	FlushMoves(w);
+		w->cubes.blockOfPosition[i - 1] = i;*/
+	/* makes puzzle too trivial
+	for (i = 0; i < w->cubes.sizeBlock; i++)
+		w->cubes.blockOfPosition[i] = w->cubes.sizeBlock - i;
+	w->cubes.spacePosition = 0;
+	w->cubes.blockOfPosition[0] = 0;*/
+	for (i = 0; i < w->cubes.sizeBlock; i++)  {
+		j = ToInvPosition(w, Column(w, i), Row(w, i), Stack(w, i)) + 1;
+		w->cubes.blockOfPosition[i] = j;
+		if (j == w->cubes.sizeBlock - 2)
+			switch1 = i;
+		else if (j == w->cubes.sizeBlock - 1)
+			switch2 = i;
+	}
+	w->cubes.spacePosition = w->cubes.sizeBlock - 1;
+	w->cubes.blockOfPosition[w->cubes.sizeBlock - 1] = 0;
+	if (((w->cubes.sizeX >> 1) & 1) == 1 &&
+			((w->cubes.sizeY >> 1) & 1) == 1 &&
+			(w->cubes.sizeZ & 1) == 1) {
+		i = w->cubes.blockOfPosition[switch1];
+		w->cubes.blockOfPosition[switch1] =
+			w->cubes.blockOfPosition[switch2];
+		w->cubes.blockOfPosition[switch2] = i;
+	}
+	if ((w->cubes.sizeX == 2 && w->cubes.sizeY == 2 &&
+			w->cubes.sizeZ == 1) ||
+			(w->cubes.sizeX == 2 && w->cubes.sizeY == 1 &&
+			w->cubes.sizeZ == 2) ||
+			(w->cubes.sizeX == 1 && w->cubes.sizeY == 2 &&
+			w->cubes.sizeZ == 2)) {
+		w->cubes.spacePosition = 0;
+		w->cubes.blockOfPosition[0] = 0;
+		w->cubes.blockOfPosition[1] = 3;
+		w->cubes.blockOfPosition[2] = 2;
+		w->cubes.blockOfPosition[3] = 1;
+	}
+	if ((w->cubes.sizeX > 1 && w->cubes.sizeY == 1 &&
+			w->cubes.sizeZ == 1) ||
+			(w->cubes.sizeX == 1 && w->cubes.sizeY > 1 &&
+			w->cubes.sizeZ == 1) ||
+			(w->cubes.sizeX == 1 && w->cubes.sizeY == 1 &&
+			w->cubes.sizeZ > 1)) {
+		int max = (w->cubes.sizeX > w->cubes.sizeY) ?
+			w->cubes.sizeX : (w->cubes.sizeY > w->cubes.sizeZ) ?
+			w->cubes.sizeY : w->cubes.sizeZ;
+
+		w->cubes.spacePosition = 0;
+		w->cubes.blockOfPosition[0] = 0;
+		for (i = 1; i < max; i++)
+			w->cubes.blockOfPosition[i] = i;
+	}
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	w->cubes.currentPosition = -1;
 	w->cubes.started = False;
 }
@@ -1519,15 +1593,16 @@ GetBlocks(CubesWidget w)
 		}
 #endif
 	}
-	FlushMoves(w);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &sizeX);
 	if (sizeX >= MINCUBES) {
 		for (i = w->cubes.sizeX; i < sizeX; i++) {
-			SetCubes(w, CUBES_INC_X);
+			setPuzzle(w, PUZZLE_INC_X);
 		}
 		for (i = w->cubes.sizeX; i > sizeX; i--) {
-			SetCubes(w, CUBES_DEC_X);
+			setPuzzle(w, PUZZLE_DEC_X);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizeX ");
@@ -1546,10 +1621,10 @@ GetBlocks(CubesWidget w)
 	(void) fscanf(fp, "%d", &sizeY);
 	if (sizeY >= MINCUBES) {
 		for (i = w->cubes.sizeY; i < sizeY; i++) {
-			SetCubes(w, CUBES_INC_Y);
+			setPuzzle(w, PUZZLE_INC_Y);
 		}
 		for (i = w->cubes.sizeY; i > sizeY; i--) {
-			SetCubes(w, CUBES_DEC_Y);
+			setPuzzle(w, PUZZLE_DEC_Y);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizeY ");
@@ -1568,10 +1643,10 @@ GetBlocks(CubesWidget w)
 	(void) fscanf(fp, "%d", &sizeZ);
 	if (sizeZ >= MINCUBES) {
 		for (i = w->cubes.sizeZ; i < sizeZ; i++) {
-			SetCubes(w, CUBES_INC_Z);
+			setPuzzle(w, PUZZLE_INC_Z);
 		}
 		for (i = w->cubes.sizeZ; i > sizeZ; i--) {
-			SetCubes(w, CUBES_DEC_Z);
+			setPuzzle(w, PUZZLE_DEC_Z);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizeZ ");
@@ -1590,10 +1665,10 @@ GetBlocks(CubesWidget w)
 #endif
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
-	ScanStartPosition(fp, w);
-	SetCubes(w, CUBES_RESTORE);
-	SetStartPosition(w);
-	ScanMoves(fp, w, moves);
+	scanStartPosition(fp, w);
+	setPuzzle(w, PUZZLE_RESTORE);
+	setStartPosition(w);
+	scanMoves(fp, w, moves);
 	(void) fclose(fp);
 	(void) printf("%s: sizeX %d, sizeY %d, sizeZ %d, moves %d.\n",
 		name, sizeX, sizeY, sizeZ, moves);
@@ -1647,37 +1722,29 @@ WriteBlocks(CubesWidget w)
 	(void) fprintf(fp, "sizeX%c %d\n", SYMBOL, w->cubes.sizeX);
 	(void) fprintf(fp, "sizeY%c %d\n", SYMBOL, w->cubes.sizeY);
 	(void) fprintf(fp, "sizeZ%c %d\n", SYMBOL, w->cubes.sizeZ);
-	(void) fprintf(fp, "moves%c %d\n", SYMBOL, NumMoves());
-	PrintStartPosition(fp, w);
-	PrintMoves(fp);
+	(void) fprintf(fp, "moves%c %d\n", SYMBOL,
+		numMoves(&undo));
+	printStartPosition(fp, w);
+	printMoves(fp, &undo);
 	(void) fclose(fp);
 	(void) printf("Saved to %s.\n", name);
 	free(lname);
 	free(fname);
 }
 
-static void
-ClearBlocks(CubesWidget w)
-{
-	if (w->cubes.currentPosition >= 0)
-		return;
-	ResetBlocks(w);
-	DrawAllBlocks(w);
-	SetCubes(w, CUBES_RESET);
-}
-
 static void
 UndoBlocks(CubesWidget w)
 {
-	if (MadeMoves() && w->cubes.currentPosition < 0) {
+	if (madeMoves(&undo) &&
+			w->cubes.currentPosition < 0) {
 		int direction;
 
-		GetMove(&direction);
+		getMove(&undo, &direction);
+		setMove(&redo, direction);
 		direction = (direction < 4) ? (direction + 2) % 4 :
 			((direction == INWARDS) ? OUTWARDS : INWARDS);
-
 		if (MoveBlocksDir(w, direction, DOUBLE)) {
-			SetCubes(w, CUBES_UNDO);
+			setPuzzle(w, PUZZLE_UNDO);
 		} else {
 			char *buf1, *buf2;
 
@@ -1690,6 +1757,39 @@ UndoBlocks(CubesWidget w)
 	}
 }
 
+static void
+RedoBlocks(CubesWidget w)
+{
+	if (madeMoves(&redo) &&
+			w->cubes.currentPosition < 0) {
+		int direction;
+
+		getMove(&redo, &direction);
+		setMove(&undo, direction);
+		if (MoveBlocksDir(w, direction, DOUBLE)) {
+			setPuzzle(w, PUZZLE_REDO);
+		} else {
+			char *buf1, *buf2;
+
+			intCat(&buf1, "Move ", direction);
+			stringCat(&buf2, buf1, " can not be made");
+			free(buf1);
+			DISPLAY_WARNING(buf2);
+			free(buf2);
+		}
+	}
+}
+
+static void
+ClearBlocks(CubesWidget w)
+{
+	if (w->cubes.currentPosition >= 0)
+		return;
+	ResetBlocks(w);
+	DrawAllBlocks(w);
+	setPuzzle(w, PUZZLE_RESET);
+}
+
 static void
 RandomizeBlocks(CubesWidget w)
 {
@@ -1729,23 +1829,24 @@ RandomizeBlocks(CubesWidget w)
 		s = Column(w, w->cubes.spacePosition);
 		e = NRAND(w->cubes.sizeX);
 		for (n = 0; n < e - s; n++)
-			(void) MoveCubesDir(w, LEFT, INSTANT);
+			(void) MovePuzzleDir(w, LEFT, INSTANT);
 		for (n = 0; n < s - e; n++)
-			(void) MoveCubesDir(w, RIGHT, INSTANT);
+			(void) MovePuzzleDir(w, RIGHT, INSTANT);
 		s = Row(w, w->cubes.spacePosition);
 		e = NRAND(w->cubes.sizeY);
 		for (n = 0; n < e - s; n++)
-			(void) MoveCubesDir(w, TOP, INSTANT);
+			(void) MovePuzzleDir(w, TOP, INSTANT);
 		for (n = 0; n < s - e; n++)
-			(void) MoveCubesDir(w, BOTTOM, INSTANT);
+			(void) MovePuzzleDir(w, BOTTOM, INSTANT);
 		s = Stack(w, w->cubes.spacePosition);
 		e = NRAND(w->cubes.sizeZ);
 		for (n = 0; n < e - s; n++)
-			(void) MoveCubesDir(w, OUTWARDS, INSTANT);
+			(void) MovePuzzleDir(w, OUTWARDS, INSTANT);
 		for (n = 0; n < s - e; n++)
-			(void) MoveCubesDir(w, INWARDS, INSTANT);
-		FlushMoves(w);
-		SetCubes(w, CUBES_RANDOMIZE);
+			(void) MovePuzzleDir(w, INWARDS, INSTANT);
+		flushMoves(w, &undo, TRUE);
+		flushMoves(w, &redo, FALSE);
+		setPuzzle(w, PUZZLE_RANDOMIZE);
 	}
 #if 0
 	/* Now move the space around randomly */
@@ -1754,7 +1855,7 @@ RandomizeBlocks(CubesWidget w)
 		int lastDirection = -1;
 		int randomDirection;
 
-		SetCubes(w, CUBES_RESET);
+		setPuzzle(w, PUZZLE_RESET);
 #ifdef DEBUG
 		big = 3;
 #endif
@@ -1770,18 +1871,19 @@ RandomizeBlocks(CubesWidget w)
 			    (w->cubes.sizeX == 1 && w->cubes.sizeY == 1) ||
 			    (w->cubes.sizeY == 1 && w->cubes.sizeZ == 1) ||
 			    (w->cubes.sizeZ == 1 && w->cubes.sizeX == 1)) {
-				if (MoveCubesDir(w, randomDirection, INSTANT))
+				if (MovePuzzleDir(w, randomDirection, INSTANT))
 					lastDirection = randomDirection;
 				else
 					big++;
 			}
 		}
-		FlushMoves(w);
-		SetCubes(w, CUBES_RANDOMIZE);
+		flushMoves(w, &undo, TRUE);
+		flushMoves(w, &redo, FALSE);
+		setPuzzle(w, PUZZLE_RANDOMIZE);
 	}
 #endif
 	if (CheckSolved(w)) {
-		SetCubes(w, CUBES_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
@@ -1794,7 +1896,7 @@ SolveBlocks(CubesWidget w)
 			w->cubes.sizeZ == 1) {
 		SolveSomeBlocks(w);
 	} else {
-		SetCubes(w, CUBES_SOLVE_MESSAGE);
+		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
 	}
 }
 
@@ -1805,7 +1907,7 @@ SpeedBlocks(CubesWidget w)
 	if (w->cubes.delay < 0)
 		w->cubes.delay = 0;
 #ifdef HAVE_MOTIF
-	SetCubes(w, CUBES_SPEED);
+	setPuzzle(w, PUZZLE_SPEED);
 #endif
 }
 
@@ -1814,7 +1916,7 @@ SlowBlocks(CubesWidget w)
 {
 	w->cubes.delay += 5;
 #ifdef HAVE_MOTIF
-	SetCubes(w, CUBES_SPEED);
+	setPuzzle(w, PUZZLE_SPEED);
 #endif
 }
 
@@ -1846,7 +1948,7 @@ darker(const int light)
 }
 
 static void
-SetValuesCubes(CubesWidget w)
+SetValuesPuzzle(CubesWidget w)
 {
 	struct tagColor {
 		int red, green, blue;
@@ -1920,8 +2022,17 @@ SetValuesCubes(CubesWidget w)
 }
 
 void
-DestroyCubes(HBRUSH brush)
+DestroyPuzzle(CubesWidget w, HBRUSH brush)
 {
+	if (w->core.memDC) {
+		if (w->cubes.bufferBlocks != NULL) {
+			DeleteObject(w->cubes.bufferBlocks);
+		}
+		DeleteDC(w->core.memDC);
+		w->core.memDC = NULL;
+	}
+	deleteMoves(&undo);
+	deleteMoves(&redo);
 	(void) DeleteObject(brush);
 	PostQuitMessage(0);
 }
@@ -2052,7 +2163,7 @@ SetAllColors(CubesWidget w)
 }
 
 static Boolean
-SetValuesCubes(Widget current, Widget request, Widget renew)
+SetValuesPuzzle(Widget current, Widget request, Widget renew)
 {
 	CubesWidget c = (CubesWidget) current, w = (CubesWidget) renew;
 	Boolean redraw = False;
@@ -2076,11 +2187,11 @@ SetValuesCubes(Widget current, Widget request, Widget renew)
 			w->cubes.sizeY != c->cubes.sizeY ||
 			w->cubes.sizeZ != c->cubes.sizeZ ||
 			w->cubes.base != c->cubes.base) {
-		SizeCubes(w);
+		SizePuzzle(w);
 		redraw = True;
 	} else if (w->cubes.offset.x != c->cubes.offset.x ||
 			w->cubes.offset.y != c->cubes.offset.y) {
-		ResizeCubes(w);
+		ResizePuzzle(w);
 		redraw = True;
 	}
 	if (w->cubes.delay != c->cubes.delay) {
@@ -2092,31 +2203,34 @@ SetValuesCubes(Widget current, Widget request, Widget renew)
 
 		w->cubes.menu = -1;
 		switch (menu) {
-		case 0:
+		case MENU_GET:
 			GetBlocks(w);
 			break;
-		case 1:
+		case MENU_WRITE:
 			WriteBlocks(w);
 			break;
-		case 3:
-			ClearBlocks(w);
-			break;
-		case 4:
+		case MENU_UNDO:
 			UndoBlocks(w);
 			break;
-		case 5:
+		case MENU_REDO:
+			RedoBlocks(w);
+			break;
+		case MENU_CLEAR:
+			ClearBlocks(w);
+			break;
+		case MENU_RANDOMIZE:
 			RandomizeBlocks(w);
 			break;
-		case 6:
+		case MENU_SOLVE:
 			SolveBlocks(w);
 			break;
-		case 7:
+		case MENU_SPEED:
 			SpeedBlocks(w);
 			break;
-		case 8:
+		case MENU_SLOW:
 			SlowBlocks(w);
 			break;
-		case 9:
+		case MENU_SOUND:
 			SoundBlocks(w);
 			break;
 		default:
@@ -2134,24 +2248,19 @@ SetValuesCubes(Widget current, Widget request, Widget renew)
 }
 
 static void
-QuitCubes(CubesWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
+DestroyPuzzle(Widget old)
 {
-#ifdef WINVER
-	if (w->cubes.freeDC) {
-		if (w->cubes.bufferBlocks != NULL) {
-			DeleteObject(w->cubes.bufferBlocks);
-		}
-		DeleteDC(w->core.memDC);
-		w->core.memDC = NULL;
-	}
-#else
+	CubesWidget w = (CubesWidget) old;
 	Display *display = XtDisplay(w);
 	int sel;
 
+	XtReleaseGC(old, w->cubes.blockGC);
+	XtReleaseGC(old, w->cubes.borderGC);
+	XtReleaseGC(old, w->cubes.blockBrighterGC);
+	XtReleaseGC(old, w->cubes.blockDarkerGC);
+	XtReleaseGC(old, w->cubes.frameGC);
+	XtReleaseGC(old, w->cubes.inverseGC);
+	XtRemoveCallbacks(old, XtNselectCallback, w->cubes.select);
 	if (w->cubes.colormap != None) {
 		XInstallColormap(display, w->cubes.oldColormap);
 		XFreeColormap(display, w->cubes.colormap);
@@ -2163,23 +2272,15 @@ QuitCubes(CubesWidget w
 		XUnloadFont(XtDisplay(w), w->cubes.fontInfo->fid);
 		XFreeFont(display, w->cubes.fontInfo);
 	}
-	XtCloseDisplay(display);
-#endif
-	exit(0);
+	deleteMoves(&undo);
+	deleteMoves(&redo);
 }
 
 static void
-DestroyCubes(Widget old)
+QuitPuzzle(CubesWidget w, XEvent *event, char **args, int nArgs)
 {
-	CubesWidget w = (CubesWidget) old;
-
-	XtReleaseGC(old, w->cubes.blockGC);
-	XtReleaseGC(old, w->cubes.borderGC);
-	XtReleaseGC(old, w->cubes.blockBrighterGC);
-	XtReleaseGC(old, w->cubes.blockDarkerGC);
-	XtReleaseGC(old, w->cubes.frameGC);
-	XtReleaseGC(old, w->cubes.inverseGC);
-	XtRemoveCallbacks(old, XtNselectCallback, w->cubes.select);
+	XtCloseDisplay(XtDisplay(w));
+	exit(0);
 }
 #endif
 
@@ -2281,7 +2382,7 @@ ResizeBlocks(CubesWidget w)
 }
 
 void
-ResizeCubes(CubesWidget w)
+ResizePuzzle(CubesWidget w)
 {
 #ifdef WINVER
 	RECT rect;
@@ -2332,17 +2433,17 @@ ResizeCubes(CubesWidget w)
 static
 #endif
 void
-SizeCubes(CubesWidget w)
+SizePuzzle(CubesWidget w)
 {
 	ResetBlocks(w);
-	ResizeCubes(w);
+	ResizePuzzle(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-InitializeCubes(
+InitializePuzzle(
 #ifdef WINVER
 CubesWidget w, HBRUSH brush
 #else
@@ -2351,7 +2452,7 @@ Widget request, Widget renew
 )
 {
 #ifdef WINVER
-	SetValuesCubes(w);
+	SetValuesPuzzle(w);
 	brush = CreateSolidBrush(w->cubes.inverseGC);
 	SETBACK(w->core.hWnd, brush);
 	(void) SRAND(time(NULL));
@@ -2372,27 +2473,29 @@ Widget request, Widget renew
 	w->cubes.blockDarkerGC = NULL;
 	w->cubes.frameGC = NULL;
 	w->cubes.inverseGC = NULL;
+	w->cubes.image = NULL;
 #endif
 	w->cubes.focus = False;
 	loadFont(w);
 	w->cubes.blockOfPosition = NULL;
 	CheckBlocks(w);
-	InitMoves();
+	newMoves(&undo);
+	newMoves(&redo);
 	w->cubes.numSlices = ((w->cubes.delay < MAXSLICES) ?
 		w->cubes.delay + 1 : MAXSLICES);
 	w->cubes.cheat = False;
-	SizeCubes(w);
+	SizePuzzle(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-ExposeCubes(
+ExposePuzzle(
 #ifdef WINVER
 CubesWidget w
 #else
-Widget renew, XEvent * event, Region region
+Widget renew, XEvent *event, Region region
 #endif
 )
 {
@@ -2412,24 +2515,24 @@ Widget renew, XEvent * event, Region region
 static
 #endif
 void
-HideCubes(CubesWidget w
+HidePuzzle(CubesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
-	SetCubes(w, CUBES_HIDE);
+	setPuzzle(w, PUZZLE_HIDE);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-SelectCubes(CubesWidget w
+SelectPuzzle(CubesWidget w
 #ifdef WINVER
 , const int x, const int y
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2452,15 +2555,15 @@ SelectCubes(CubesWidget w
 		rowType = MovableBlock(w);
 		if (rowType < 0) {
 			DrawBlock(w, w->cubes.currentPosition,
-				rowType == CUBES_SPACE, False, TRUE, 0, 0);
+				rowType == PUZZLE_SPACE, False, TRUE, 0, 0);
 			FLUSH(w);
 			Sleep(100);
 			DrawBlock(w, w->cubes.currentPosition,
 				True, True, TRUE, 0, 0);
-			if (rowType != CUBES_SPACE)
+			if (rowType != PUZZLE_SPACE)
 				DrawBlock(w, w->cubes.currentPosition,
 					False, False, FALSE, 0, 0);
-			SetCubes(w, rowType);
+			setPuzzle(w, rowType);
 			w->cubes.currentPosition = -1;
 			return;
 		}
@@ -2474,9 +2577,9 @@ SelectCubes(CubesWidget w
 static
 #endif
 void
-ReleaseCubes(CubesWidget w
+ReleasePuzzle(CubesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2489,23 +2592,9 @@ ReleaseCubes(CubesWidget w
 }
 
 #ifndef WINVER
-static
-#endif
-void
-RandomizeCubes(CubesWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	RandomizeBlocks(w);
-}
-
-#ifndef WINVER
-static
-void
-RandomizeCubesMaybe(CubesWidget w
-, XEvent * event, char **args, int nArgs
+static void
+ClearPuzzleMaybe(CubesWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 	if (!w->cubes.started) {
@@ -2513,21 +2602,20 @@ RandomizeCubesMaybe(CubesWidget w
 	}
 #ifdef HAVE_MOTIF
 	else {
-		SetCubes(w, CUBES_RANDOMIZE_QUERY);
+		setPuzzle(w, PUZZLE_CLEAR_QUERY);
 	}
 #endif
 }
 
-static
-void
-RandomizeCubes2(CubesWidget w
-, XEvent * event, char **args, int nArgs
+static void
+ClearPuzzle2(CubesWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 #ifdef HAVE_MOTIF
 	if (!w->cubes.started)
 #endif
-		RandomizeBlocks(w);
+		ClearBlocks(w);
 }
 #endif
 
@@ -2535,9 +2623,9 @@ RandomizeCubes2(CubesWidget w
 static
 #endif
 void
-GetCubes(CubesWidget w
+GetPuzzle(CubesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2548,9 +2636,9 @@ GetCubes(CubesWidget w
 static
 #endif
 void
-WriteCubes(CubesWidget w
+WritePuzzle(CubesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2561,22 +2649,9 @@ WriteCubes(CubesWidget w
 static
 #endif
 void
-ClearCubes(CubesWidget w
+UndoPuzzle(CubesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	ClearBlocks(w);
-}
-
-#ifndef WINVER
-static
-#endif
-void
-UndoCubes(CubesWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2587,9 +2662,48 @@ UndoCubes(CubesWidget w
 static
 #endif
 void
-SolveCubes(CubesWidget w
+RedoPuzzle(CubesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RedoBlocks(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+ClearPuzzle(CubesWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	ClearBlocks(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+RandomizePuzzle(CubesWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RandomizeBlocks(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+SolvePuzzle(CubesWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2600,9 +2714,9 @@ SolveCubes(CubesWidget w
 static
 #endif
 void
-SpeedCubes(CubesWidget w
+SpeedPuzzle(CubesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2613,9 +2727,9 @@ SpeedCubes(CubesWidget w
 static
 #endif
 void
-SlowCubes(CubesWidget w
+SlowPuzzle(CubesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2626,9 +2740,9 @@ SlowCubes(CubesWidget w
 static
 #endif
 void
-SoundCubes(CubesWidget w
+SoundPuzzle(CubesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2639,9 +2753,9 @@ SoundCubes(CubesWidget w
 static
 #endif
 void
-EnterCubes(CubesWidget w
+EnterPuzzle(CubesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2657,9 +2771,9 @@ EnterCubes(CubesWidget w
 static
 #endif
 void
-LeaveCubes(CubesWidget w
+LeavePuzzle(CubesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2673,44 +2787,44 @@ LeaveCubes(CubesWidget w
 
 #ifndef WINVER
 static void
-MoveCubesOut(CubesWidget w, XEvent * event, char **args, int nArgs
+MovePuzzleOut(CubesWidget w, XEvent *event, char **args, int nArgs
 )
 {
-	(void) MoveCubes(w, OUTWARDS, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, OUTWARDS, (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveCubesTop(CubesWidget w, XEvent * event, char **args, int nArgs
+MovePuzzleTop(CubesWidget w, XEvent *event, char **args, int nArgs
 )
 {
-	(void) MoveCubes(w, TOP, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, TOP, (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveCubesLeft(CubesWidget w, XEvent * event, char **args, int nArgs
+MovePuzzleLeft(CubesWidget w, XEvent *event, char **args, int nArgs
 )
 {
-	(void) MoveCubes(w, LEFT, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, LEFT, (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveCubesIn(CubesWidget w, XEvent * event, char **args, int nArgs
+MovePuzzleIn(CubesWidget w, XEvent *event, char **args, int nArgs
 )
 {
-	(void) MoveCubes(w, INWARDS, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, INWARDS, (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveCubesRight(CubesWidget w, XEvent * event, char **args, int nArgs
+MovePuzzleRight(CubesWidget w, XEvent *event, char **args, int nArgs
 )
 {
-	(void) MoveCubes(w, RIGHT, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, RIGHT, (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveCubesBottom(CubesWidget w, XEvent * event, char **args, int nArgs
+MovePuzzleBottom(CubesWidget w, XEvent *event, char **args, int nArgs
 )
 {
-	(void) MoveCubes(w, BOTTOM, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, BOTTOM, (int) (event->xkey.state & ControlMask));
 }
 #endif
