@@ -84,27 +84,40 @@ typedef struct {
 } cubesCallbackStruct;
 #endif
 
-#define CUBES_HIDE (-8)
-#define CUBES_RANDOMIZE_QUERY (-7)
-#define CUBES_SOLVE_MESSAGE (-6)
-#define CUBES_RESTORE (-5)
-#define CUBES_RESET (-4)
-#define CUBES_BLOCKED (-3)
-#define CUBES_SPACE (-2)
-#define CUBES_IGNORE (-1)
-#define CUBES_MOVED 0
-#define CUBES_SOLVED 1
-#define CUBES_PRACTICE 2
-#define CUBES_RANDOMIZE 3
-#define CUBES_DEC_X 4
-#define CUBES_INC_X 5
-#define CUBES_DEC_Y 6
-#define CUBES_INC_Y 7
-#define CUBES_DEC_Z 8
-#define CUBES_INC_Z 9
-#define CUBES_COMPUTED 10
-#define CUBES_UNDO 11
-#define CUBES_SPEED 12
+#define PUZZLE_HIDE (-8)
+#define PUZZLE_CLEAR_QUERY (-7)
+#define PUZZLE_SOLVE_MESSAGE (-6)
+#define PUZZLE_RESTORE (-5)
+#define PUZZLE_RESET (-4)
+#define PUZZLE_BLOCKED (-3)
+#define PUZZLE_SPACE (-2)
+#define PUZZLE_IGNORE (-1)
+#define PUZZLE_MOVED 0
+#define PUZZLE_SOLVED 1
+#define PUZZLE_PRACTICE 2
+#define PUZZLE_RANDOMIZE 3
+#define PUZZLE_DEC_X 4
+#define PUZZLE_INC_X 5
+#define PUZZLE_DEC_Y 6
+#define PUZZLE_INC_Y 7
+#define PUZZLE_DEC_Z 8
+#define PUZZLE_INC_Z 9
+#define PUZZLE_COMPUTED 10
+#define PUZZLE_UNDO 11
+#define PUZZLE_REDO 12
+#define PUZZLE_SPEED 13
+
+#define MENU_GET 0
+#define MENU_WRITE 1
+#define MENU_EXIT 2
+#define MENU_UNDO 3
+#define MENU_REDO 4
+#define MENU_CLEAR 5
+#define MENU_RANDOMIZE 6
+#define MENU_SOLVE 7
+#define MENU_SPEED 8
+#define MENU_SLOW 9
+#define MENU_SOUND 10
 
 #define MINCUBES 1
 #define DEFAULTCUBESX 4
