@@ -7,7 +7,7 @@
 #
 #  Taken from the X puzzle by Don Bennett, HP Labs
 #
-#  Copyright (c) 2003 - 2005    David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 2003 - 2006    David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -794,7 +794,7 @@ SolveSomeBlocks(CubesWidget w)
 		myFree();
 		myInitialize(w);
 	}
-        SetCubes(w, CUBES_RESET);
+        setPuzzle(w, PUZZLE_RESET);
 	/** determine the position we want to be in when
 	 ** we are done; This position will have the space in
 	 ** the center;  Then, we'll move the space back to
@@ -837,5 +837,5 @@ SolveSomeBlocks(CubesWidget w)
 		locked[i] = False;
 	SolvingFlag = False;
 	w->cubes.cheat = True; /* Assume the worst. */
-        SetCubes(w, CUBES_COMPUTED);
+        setPuzzle(w, PUZZLE_COMPUTED);
 }
