@@ -88,7 +88,7 @@ static CubesRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Cubes Version 7.2\n"
+"Cubes Version 7.2.4\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -133,23 +133,26 @@ static const char featuresHelp[] = {
 "the current box  or the bottom of the current\n"
 "box.\n"
 "\n"
-"Click \"mouse-right\" button, or press \"R\" or \"r\" "
-"keys, to randomize the puzzle.\n"
+"Click \"mouse-right\" button, or press \"C\" or \"c\" "
+"keys, to clear the puzzle.\n"
 "\n"
-"Press \"I\" or \"i\" keys to move a block in.\n"
+"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
 "\n"
-"Press \"O\" or \"o\" keys to move a block out.\n"
+"Press \"W\" or \"w\" keys to save (write) a puzzle.\n"
+"\n"
+"Press \"U\" or \"u\" keys to undo a move.\n"
+"\n"
+"Press \"R\" or \"r\" keys to redo a move.\n"
+"\n"
+"Press \"Z\" or \"z\" keys to randomize the puzzle.  New records "
+"can only be set from a cleared state.\n"
 "\n"
 "Press \"S\" or \"s\" keys to start the auto-solver.  Only "
 "works on lxmx1 where l > 3 and m > 3.\n"
 "\n"
-"Press \"U\" or \"u\" keys to undo a move.\n"
+"Press \"I\" or \"i\" keys to move a block in.\n"
 "\n"
-"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
-"\n"
-"Press \"W\" or \"w\" keys to save a puzzle.\n"
-"\n"
-"Press \"C\" or \"c\" keys to clear a puzzle.\n"
+"Press \"O\" or \"o\" keys to move a block out.\n"
 "\n"
 "Press \">\" or \".\" keys to speed up the movement of blocks.\n"
 "\n"
@@ -221,19 +224,20 @@ static Widget moves, record, message, blockX, blockY, blockZ, speed;
 static char buff[21];
 static Widget descriptionDialog, featuresDialog;
 static Widget optionsDialog, referencesDialog, aboutDialog;
-static Widget solveDialog, randomizeDialog;
+static Widget solveDialog, clearDialog;
 static Arg arg[3];
 #else
 static char titleDsp[TITLELEN] = "";
 #endif
 static char scoreFileName[FILENAMELEN] = SCOREFILE;
 static char fileName[FILENAMELEN];
+static Boolean randomized = False;
 #ifdef WINVER
 #define MAXPROGNAME 80
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap cubesIcon = None;
+static Pixmap icon = None;
 static Widget topLevel, cubes;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
@@ -508,7 +512,7 @@ Widget w
 	char *userName, *scoreFile;
 
 #ifdef WINVER
-	InitializeCubes(w, brush);
+	InitializePuzzle(w, brush);
 
 	sizeX = w->cubes.sizeX;
 	sizeY = w->cubes.sizeY;
@@ -570,7 +574,7 @@ Widget w
 #ifdef WINVER
 		(void) strncpy(userNameDsp, userName, USERNAMELEN);
 #else
-		char * login = getlogin();
+		char *login = getlogin();
 
 		if (login == NULL) {
 			(void) strcpy(userNameDsp, "");
@@ -595,10 +599,10 @@ Widget w
 
 #ifdef WINVER
 void
-SetCubes(CubesWidget w, int reason)
+setPuzzle(CubesWidget w, int reason)
 #else
 static void
-CallbackCubes(Widget w, caddr_t clientData, cubesCallbackStruct * callData)
+CallbackPuzzle(Widget w, caddr_t clientData, cubesCallbackStruct *callData)
 #endif
 {
 #ifndef WINVER
@@ -621,7 +625,7 @@ CallbackCubes(Widget w, caddr_t clientData, cubesCallbackStruct * callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-		case CUBES_HIDE:
+		case PUZZLE_HIDE:
 #ifdef WINVER
 			ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -631,16 +635,15 @@ CallbackCubes(Widget w, caddr_t clientData, cubesCallbackStruct * callData)
 #endif
 			break;
 #ifndef WINVER
-		case CUBES_RANDOMIZE_QUERY:
+		case PUZZLE_CLEAR_QUERY:
 #ifdef HAVE_MOTIF
-			XtManageChild(randomizeDialog);
+			XtManageChild(clearDialog);
 #else
-			XtVaSetValues(cubes,
-				XtNmenu, 5, NULL); /* menu choice */
+			XtVaSetValues(cubes, XtNmenu, MENU_CLEAR, NULL);
 #endif
 			break;
 #endif
-		case CUBES_SOLVE_MESSAGE:
+		case PUZZLE_SOLVE_MESSAGE:
 #ifdef WINVER
 			(void) MessageBox(w->core.hWnd, solveHelp,
 				"Auto-Solve", MB_OK);
@@ -653,33 +656,35 @@ CallbackCubes(Widget w, caddr_t clientData, cubesCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case CUBES_RESTORE:
-		case CUBES_RESET:
+		case PUZZLE_RESTORE:
+		case PUZZLE_RESET:
 			movesDsp = 0;
+			randomized = False;
 			break;
-		case CUBES_BLOCKED:
+		case PUZZLE_BLOCKED:
 			(void) strncpy(messageDsp, "Blocked", MESSAGELEN);
 			break;
-		case CUBES_SPACE:
+		case PUZZLE_SPACE:
 #if 0
 			/* Too annoying */
 			(void) strncpy(messageDsp, "A space can not slide",
 				MESSAGELEN);
 #endif
 			break;
-		case CUBES_IGNORE:
-			(void) strncpy(messageDsp, "Randomize to start",
+		case PUZZLE_IGNORE:
+			(void) strncpy(messageDsp, "Clear to start",
 				MESSAGELEN);
 			break;
-		case CUBES_MOVED:
+		case PUZZLE_MOVED:
 			movesDsp++;
 			SET_STARTED(w, True);
 			break;
-		case CUBES_SOLVED:
+		case PUZZLE_SOLVED:
 			if (cheat)
 				(void) sprintf(messageDsp,
 					"No cheating %s!!", userNameDsp);
-			else if (HandleSolved(movesDsp, sizeX, sizeY, sizeZ))
+			else if (!randomized && HandleSolved(movesDsp,
+					sizeX, sizeY, sizeZ))
 				(void) sprintf(messageDsp,
 					"Congratulations %s!!", userNameDsp);
 			else
@@ -687,11 +692,12 @@ CallbackCubes(Widget w, caddr_t clientData, cubesCallbackStruct * callData)
 					MESSAGELEN);
 			SET_STARTED(w, False);
 			break;
-		case CUBES_RANDOMIZE:
+		case PUZZLE_RANDOMIZE:
 			movesDsp = 0;
+			randomized = True;
 			SET_STARTED(w, False);
 			break;
-		case CUBES_DEC_X:
+		case PUZZLE_DEC_X:
 			movesDsp = 0;
 			sizeX--;
 			PrintRecord(sizeX, sizeY, sizeZ);
@@ -706,7 +712,7 @@ CallbackCubes(Widget w, caddr_t clientData, cubesCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case CUBES_INC_X:
+		case PUZZLE_INC_X:
 			movesDsp = 0;
 			sizeX++;
 			PrintRecord(sizeX, sizeY, sizeZ);
@@ -721,7 +727,7 @@ CallbackCubes(Widget w, caddr_t clientData, cubesCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case CUBES_DEC_Y:
+		case PUZZLE_DEC_Y:
 			movesDsp = 0;
 			sizeY--;
 			PrintRecord(sizeX, sizeY, sizeZ);
@@ -736,7 +742,7 @@ CallbackCubes(Widget w, caddr_t clientData, cubesCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case CUBES_INC_Y:
+		case PUZZLE_INC_Y:
 			movesDsp = 0;
 			sizeY++;
 			PrintRecord(sizeX, sizeY, sizeZ);
@@ -751,7 +757,7 @@ CallbackCubes(Widget w, caddr_t clientData, cubesCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case CUBES_DEC_Z:
+		case PUZZLE_DEC_Z:
 			movesDsp = 0;
 			sizeZ--;
 			PrintRecord(sizeX, sizeY, sizeZ);
@@ -766,7 +772,7 @@ CallbackCubes(Widget w, caddr_t clientData, cubesCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case CUBES_INC_Z:
+		case PUZZLE_INC_Z:
 			movesDsp = 0;
 			sizeZ++;
 			PrintRecord(sizeX, sizeY, sizeZ);
@@ -781,15 +787,19 @@ CallbackCubes(Widget w, caddr_t clientData, cubesCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case CUBES_COMPUTED:
+		case PUZZLE_COMPUTED:
 			SET_STARTED(w, False);
 			break;
-		case CUBES_UNDO:
+		case PUZZLE_UNDO:
 			movesDsp--;
 			SET_STARTED(w, True);
 			break;
+		case PUZZLE_REDO:
+			movesDsp++;
+			SET_STARTED(w, True);
+			break;
 #ifdef HAVE_MOTIF
-		case CUBES_SPEED:
+		case PUZZLE_SPEED:
 			{
 				int oldDelay, delay;
 
@@ -841,15 +851,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	if (GetFocus()) {
 		if (!widget.cubes.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			EnterCubes(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			EnterPuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	} else {
 		if (widget.cubes.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			LeaveCubes(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			LeavePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	}
@@ -858,40 +870,45 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			Initialize(&widget, brush);
 			break;
 		case WM_DESTROY:
-			DestroyCubes(brush);
+			DestroyPuzzle(&widget, brush);
 			break;
 		case WM_SIZE:
-			ResizeCubes(&widget);
+			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
 		case WM_PAINT:
 			widget.core.hDC = BeginPaint(hWnd, &paint);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ExposeCubes(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ExposePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 			break;
 		case WM_RBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			RandomizeCubes(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ClearPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			SelectCubes(&widget, LOWORD(lParam), HIWORD(lParam));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			SelectPuzzle(&widget, LOWORD(lParam), HIWORD(lParam));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONUP:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ReleaseCubes(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ReleasePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 #if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
 		case WM_MOUSEWHEEL:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
 			{
 				int zDelta = ((short) HIWORD(wParam));
 				POINT cursor, origin;
@@ -900,17 +917,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				ClientToScreen(hWnd, &origin);
 				(void) GetCursorPos(&cursor);
 				if (zDelta > (WHEEL_DELTA >> 1)) {
-					MoveCubes(&widget, TOP,
+					MovePuzzle(&widget, TOP,
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 					if (GetKeyState(VK_CONTROL) >> 1) {
-						SizeCubes(&widget);
+						SizePuzzle(&widget);
 						(void) InvalidateRect(hWnd, NULL, TRUE);
 					}
 				} else if (zDelta < -(WHEEL_DELTA >> 1)) {
-					MoveCubes(&widget, BOTTOM,
+					MovePuzzle(&widget, BOTTOM,
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 					if (GetKeyState(VK_CONTROL) >> 1) {
-						SizeCubes(&widget);
+						SizePuzzle(&widget);
 						(void) InvalidateRect(hWnd, NULL, TRUE);
 					}
 				}
@@ -921,51 +938,62 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		case WM_COMMAND:
 			switch (LOWORD(wParam)) {
 				case IDM_GET:
-					GetCubes(&widget);
-					ResizeCubes(&widget);
+					GetPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_WRITE:
-					WriteCubes(&widget);
+					WritePuzzle(&widget);
 					break;
 				case IDM_EXIT:
-					DestroyCubes(brush);
+					DestroyPuzzle(&widget, brush);
 					break;
 				case IDM_HIDE:
-					HideCubes(&widget);
-					break;
-				case IDM_CLEAR:
-					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					ClearCubes(&widget);
-					(void) ReleaseDC(hWnd, widget.core.hDC);
+					HidePuzzle(&widget);
 					break;
 				case IDM_UNDO:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					UndoCubes(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					UndoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_REDO:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RedoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_CLEAR:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					ClearPuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_RANDOMIZE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					RandomizeCubes(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RandomizePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_SOLVE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					SolveCubes(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					SolvePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_SPEED:
-					SpeedCubes(&widget);
+					SpeedPuzzle(&widget);
 					break;
 				case IDM_SLOW:
-					SlowCubes(&widget);
+					SlowPuzzle(&widget);
 					break;
 				case IDM_SOUND:
-					SoundCubes(&widget);
+					SoundPuzzle(&widget);
 					break;
 				case IDM_UP:
 				case IDM_RIGHT:
@@ -974,8 +1002,9 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				case IDM_IN:
 				case IDM_OUT:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					(void) MoveCubes(&widget, (int) LOWORD(wParam) - IDM_UP, FALSE);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					(void) MovePuzzle(&widget, (int) LOWORD(wParam) - IDM_UP, FALSE);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_DECY:
@@ -984,8 +1013,8 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				case IDM_DECX:
 				case IDM_DECZ:
 				case IDM_INCZ:
-					if (MoveCubes(&widget, (int) LOWORD(wParam) - IDM_DECY, TRUE)) {
-						SizeCubes(&widget);
+					if (MovePuzzle(&widget, (int) LOWORD(wParam) - IDM_DECY, TRUE)) {
+						SizePuzzle(&widget);
 						(void) InvalidateRect(hWnd, NULL, TRUE);
 					}
 					break;
@@ -1068,7 +1097,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
 #else
 
 static void
-Usage(char * programName)
+Usage(char *programName)
 {
 	unsigned int i;
 
@@ -1097,9 +1126,6 @@ static XrmOptionDescRec options[] =
 	{(char *) "-border", (char *) "*cubes.blockBorder", XrmoptionSepArg, NULL},
 	{(char *) "-frame", (char *) "*cubes.frameColor", XrmoptionSepArg, NULL},
 	{(char *) "-block", (char *) "*cubes.blockColor", XrmoptionSepArg, NULL},
-	{(char *) "-sizex", (char *) "*cubes.sizeX", XrmoptionSepArg, NULL},
-	{(char *) "-sizey", (char *) "*cubes.sizeY", XrmoptionSepArg, NULL},
-	{(char *) "-sizez", (char *) "*cubes.sizeZ", XrmoptionSepArg, NULL},
 	{(char *) "-install", (char *) "*cubes.install", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-noinstall", (char *) "*cubes.install", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-picture", (char *) "*cubes.picture", XrmoptionSepArg, NULL},
@@ -1111,6 +1137,9 @@ static XrmOptionDescRec options[] =
 	{(char *) "-base", (char *) "*cubes.base", XrmoptionSepArg, NULL},
 	{(char *) "-fn", (char *) "*cubes.font", XrmoptionSepArg, NULL},
 	{(char *) "-font", (char *) "*cubes.font", XrmoptionSepArg, NULL},
+	{(char *) "-sizex", (char *) "*cubes.sizeX", XrmoptionSepArg, NULL},
+	{(char *) "-sizey", (char *) "*cubes.sizeY", XrmoptionSepArg, NULL},
+	{(char *) "-sizez", (char *) "*cubes.sizeZ", XrmoptionSepArg, NULL},
 	{(char *) "-userName", (char *) "*cubes.userName", XrmoptionSepArg, NULL},
 	{(char *) "-scoreFile", (char *) "*cubes.scoreFile", XrmoptionSepArg, NULL},
 	{(char *) "-scores", (char *) "*cubes.scoreOnly", XrmoptionNoArg, (char *) "TRUE"},
@@ -1119,15 +1148,15 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackCubesRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzleClear(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(cubes, XtNmenu, 5, NULL); /* menu choice */
+		XtVaSetValues(cubes, XtNmenu, MENU_CLEAR, NULL);
 	}
 }
 
 static void
-BlockXSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+BlockXSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int sizeX = cbs->value, sizeY, sizeZ, oldX;
 
@@ -1148,7 +1177,7 @@ BlockXSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-BlockYSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+BlockYSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int sizeX, sizeY = cbs->value, sizeZ, oldY;
 
@@ -1169,7 +1198,7 @@ BlockYSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-BlockZSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+BlockZSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int sizeX, sizeY, sizeZ = cbs->value, oldZ;
 
@@ -1190,7 +1219,7 @@ BlockZSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MAXSPEED + MINSPEED - cbs->value - 1, oldDelay;
 
@@ -1206,9 +1235,9 @@ fileCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	if (val == 2)
+	if (val == MENU_EXIT)
 		exit(0);
-	XtVaSetValues(cubes, XtNmenu, val, NULL);
+	XtVaSetValues(cubes, XtNmenu, val, NULL); /* GWQ */
 }
 
 static void
@@ -1216,7 +1245,7 @@ playCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	XtVaSetValues(cubes, XtNmenu, val + 3, NULL); /* GWQ */
+	XtVaSetValues(cubes, XtNmenu, val + MENU_EXIT + 1, NULL);
 }
 
 static Widget
@@ -1315,7 +1344,8 @@ main(int argc, char **argv)
 	Widget movesRowCol, blocksRowCol, speedRowCol, messageRowCol;
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
-	XmString clearString, undoString, randomizeString, solveString;
+	XmString undoString, redoString;
+	XmString clearString, randomizeString, solveString;
 	XmString speedString, slowString, soundString;
 #endif
 
@@ -1325,7 +1355,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#if HAVE_XPM
+#ifdef HAVE_XPM
 	{
 		XpmAttributes xpmAttributes;
 		XpmColorSymbol transparentColor[1] = {{NULL,
@@ -1340,18 +1370,18 @@ main(int argc, char **argv)
 		transparentColor[0].pixel = bg;
 		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) cubes_xpm, &cubesIcon, NULL,
+			(char **) cubes_xpm, &icon, NULL,
 			&xpmAttributes);
 	}
-	if (cubesIcon == (Pixmap) NULL)
+	if (icon == (Pixmap) NULL)
 #endif
-		cubesIcon = XCreateBitmapFromData(XtDisplay(topLevel),
+		icon = XCreateBitmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
 			(char *) cubes_bits,
 			cubes_width, cubes_height);
 #ifdef HAVE_MOTIF
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, cubesIcon,
+		XtNiconPixmap, icon,
 		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
@@ -1378,24 +1408,27 @@ main(int argc, char **argv)
 	XmStringFree(getString);
 	XmStringFree(writeString);
 	XmStringFree(quitString);
-	clearString = XmStringCreateSimple((char *) "Clear");
 	undoString = XmStringCreateSimple((char *) "Undo");
-	randomizeString = XmStringCreateSimple((char *) "Randomize");
+	redoString = XmStringCreateSimple((char *) "Redo");
+	clearString = XmStringCreateSimple((char *) "Clear");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
 	solveString = XmStringCreateSimple((char *) "Solve");
 	speedString = XmStringCreateSimple((char *) ">Speed");
 	slowString = XmStringCreateSimple((char *) "<Slow");
 	soundString = XmStringCreateSimple((char *) "@Sound");
 	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
-		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
 		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
 		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
 		XmVaPUSHBUTTON, soundString, '@', NULL, NULL,
 		NULL);
-	XmStringFree(clearString);
 	XmStringFree(undoString);
+	XmStringFree(redoString);
+	XmStringFree(clearString);
 	XmStringFree(randomizeString);
 	XmStringFree(solveString);
 	XmStringFree(speedString);
@@ -1441,10 +1474,10 @@ main(int argc, char **argv)
 		(char *) "About");
 	solveDialog = createHelp(menuBar, (char *) solveHelp,
 		(char *) "Solve");
-	randomizeDialog = createQuery(topLevel,
-		(char *) "Are you sure you want to randomize?",
-		(char *) "Randomize Query",
-		(XtCallbackProc) CallbackCubesRandomize);
+	clearDialog = createQuery(topLevel,
+		(char *) "Are you sure you want to start over?",
+		(char *) "Clear Query",
+		(XtCallbackProc) CallbackPuzzleClear);
 	mainPanel = XtCreateManagedWidget("mainPanel",
 		xmPanedWindowWidgetClass, menuBarPanel,
 		NULL, 0);
@@ -1490,7 +1523,7 @@ main(int argc, char **argv)
 		(void) XtVaCreateManagedWidget("mouseRightText",
 			xmLabelGadgetClass, movesRowCol,
 			XtVaTypedArg, XmNlabelString,
-			XmRString, "Randomize", 10, NULL);
+			XmRString, "Clear", 6, NULL);
 		(void) XtVaCreateManagedWidget("mouseRight",
 			xmLabelGadgetClass, movesRowCol,
 			XmNlabelType, XmPIXMAP,
@@ -1566,13 +1599,13 @@ main(int argc, char **argv)
 		cubesWidgetClass, mainPanel, NULL, 0);
 #else
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, cubesIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 	cubes = XtCreateManagedWidget("cubes",
 		cubesWidgetClass, topLevel, NULL, 0);
 #endif
 	XtAddCallback(cubes, XtNselectCallback,
-		(XtCallbackProc) CallbackCubes, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	Initialize(cubes);
 	XtRealizeWidget(topLevel);
 	XGrabButton(XtDisplay(cubes), (unsigned int) AnyButton, AnyModifier,
