@@ -43,39 +43,36 @@
 #define MIN(a,b) (((int)(a)<(int)(b))?(int)(a):(int)(b))
 #define MAX(a,b) (((int)(a)>(int)(b))?(int)(a):(int)(b))
 
-typedef struct _DialPart
-{
-  Pixel indicatorColor;
-  Pixel foreground;
-  int angle;
-  int minimum, maximum;
-  int val;
-  int markers;
-  Position markerLength;
-  Position innerDiam, outerDiam;
-  XPoint indicator, center;
-  GC dialGC;
-  GC indicatorGC;
-  GC inverseGC;
-  XSegment segments[MAXSEGMENTS];
-  XtCallbackList select;
+typedef struct _DialPart {
+	Pixel indicatorColor;
+	Pixel foreground;
+	int angle;
+	int minimum, maximum;
+	int val;
+	int markers;
+	Boolean versionOnly;
+	Position markerLength;
+	Position innerDiam, outerDiam;
+	XPoint indicator, center;
+	GC dialGC;
+	GC indicatorGC;
+	GC inverseGC;
+	XSegment segments[MAXSEGMENTS];
+	XtCallbackList select;
 } DialPart;
 
-typedef struct _DialRec
-{
-  CorePart core;
-  DialPart dial;
+typedef struct _DialRec {
+	CorePart core;
+	DialPart dial;
 } DialRec;
 
-typedef struct _DialClassPart
-{
-  int ignore;
+typedef struct _DialClassPart {
+	int ignore;
 } DialClassPart;
 
-typedef struct _DialClassRec
-{
-  CoreClassPart core_class;
-  DialClassPart dial_class;
+typedef struct _DialClassRec {
+	CoreClassPart coreClass;
+	DialClassPart dialClass;
 } DialClassRec;
 
 extern DialClassRec dialClassRec;
