@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1995 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1995 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -30,6 +30,9 @@
 #include "DinoP.h"
 #include "Dino2dP.h"
 #include "Dino3dP.h"
+#ifdef HAVE_OPENGL
+#include "DinoGLP.h"
+#endif
 
 #ifdef WINVER
 #ifndef LOGPATH
@@ -63,9 +66,9 @@ static const char faceColorChar[MAXFACES] =
 #endif
 #endif
 
-static Boolean SetValuesDino(Widget current, Widget request, Widget renew);
-static void DestroyDino(Widget old);
-static void InitializeDino(Widget request, Widget renew);
+static Boolean SetValuesPuzzle(Widget current, Widget request, Widget renew);
+static void DestroyPuzzle(Widget old);
+static void InitializePuzzle(Widget request, Widget renew);
 
 DinoClassRec dinoClassRec =
 {
@@ -76,7 +79,7 @@ DinoClassRec dinoClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeDino,	/* initialize */
+		(XtInitProc) InitializePuzzle,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
 		NULL,		/* actions */
@@ -88,10 +91,10 @@ DinoClassRec dinoClassRec =
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
-		(XtWidgetProc) DestroyDino,	/* destroy */
+		(XtWidgetProc) DestroyPuzzle,	/* destroy */
 		NULL,		/* resize */
 		NULL,		/* expose */
-		(XtSetValuesFunc) SetValuesDino,	/* set values */
+		(XtSetValuesFunc) SetValuesPuzzle,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
@@ -111,7 +114,7 @@ DinoClassRec dinoClassRec =
 WidgetClass dinoWidgetClass = (WidgetClass) & dinoClassRec;
 
 void
-SetDino(DinoWidget w, int reason)
+setPuzzle(DinoWidget w, int reason)
 {
 	dinoCallbackStruct cb;
 
@@ -120,7 +123,7 @@ SetDino(DinoWidget w, int reason)
 }
 
 static void
-SetDinoMove(DinoWidget w, int reason, int face, int position, int direction,
+setPuzzleMove(DinoWidget w, int reason, int face, int position, int direction,
 		int style)
 {
 	dinoCallbackStruct cb;
@@ -146,8 +149,8 @@ loadFont(DinoWidget w)
 		XUnloadFont(XtDisplay(w), w->dino.fontInfo->fid);
 		XFreeFont(XtDisplay(w), w->dino.fontInfo);
 	}
-	if ((w->dino.fontInfo = XLoadQueryFont(display,
-			w->dino.font)) == NULL) {
+	if (w->dino.font && (w->dino.fontInfo =
+			XLoadQueryFont(display, w->dino.font)) == NULL) {
 		(void) sprintf(buf,
 			"Can not open %s font.\nAttempting %s font as alternate\n",
 			w->dino.font, altfontname);
@@ -177,6 +180,57 @@ loadFont(DinoWidget w)
 #define LOGFILE "dino.log"
 #endif
 
+static int facesToDirection[MAXFACES][MAXFACES][MAXORIENT] =
+{
+	{
+		{-3, -3, -3, -3},
+		{2, -1, 3, -3},
+		{-1, 2, -3, 1},
+		{1, -3, 0, -1},
+		{-1, -1, -1, -1},
+		{-1, 3, -3, 0}
+	},
+	{
+		{-3, 3, -1, 0},
+		{-3, -3, -3, -3},
+		{1, -3, 0, -1},
+		{-1, -1, -1, -1},
+		{-1, 2, -3, 1},
+		{2, -1, 3, -3}
+	},
+	{
+		{-3, 3, -1, 0},
+		{2, -1, 3, -3},
+		{-3, -3, -3, -3},
+		{1, -3, 0, -1},
+		{-1, 2, -3, 1},
+		{-1, -1, -1, -1}
+	},
+	{
+		{-3, 3, -1, 0},
+		{-1, -1, -1, -1},
+		{2, -1, 3, -3},
+		{-3, -3, -3, -3},
+		{-1, 2, -3, 1},
+		{1, -3, 0, -1}
+	},
+	{
+		{-1, -1, -1, -1},
+		{2, -1, 3, -3},
+		{-3, 3, -1, 0},
+		{1, -3, 0, -1},
+		{-3, -3, -3, -3},
+		{-1, 2, -3, 1}
+	},
+	{
+		{-1, 2, -3, 1},
+		{2, -1, 3, -3},
+		{-1, -1, -1, -1},
+		{1, -3, 0, -1},
+		{-3, 3, -1, 0},
+		{-3, -3, -3, -3}
+	}
+};
 static DinoLoc slideCorner[MAXFACES][MAXORIENT][MAXORIENT / 2] =
 {
 	{
@@ -265,7 +319,7 @@ static DinoLoc slideCorner[MAXFACES][MAXORIENT][MAXORIENT / 2] =
 	}
 };
 
-static int  oppFace[MAXFACES] =
+static int oppFace[MAXFACES] =
 {4, 3, 5, 1, 0, 2};
 
 static DinoCornerLoc oppCorner[MAXFACES][MAXORIENT] =
@@ -338,7 +392,7 @@ static DinoCornerLoc slideNextFace[MAXFACES][MAXORIENT] =
 		{1, HALF}}
 };
 
-static int  faceToRotate2[MAXFACES][MAXORIENT][2] =
+static int faceToRotate2[MAXFACES][MAXORIENT][2] =
 {
 	{
 		{3, 5},
@@ -372,7 +426,7 @@ static int  faceToRotate2[MAXFACES][MAXORIENT][2] =
 		{1, 4}}
 };
 
-static int  faceToRotate[MAXFACES][MAXORIENT] =
+static int faceToRotate[MAXFACES][MAXORIENT] =
 {
 	{3, 2, 1, 5},
 	{2, 4, 5, 0},
@@ -382,6 +436,9 @@ static int  faceToRotate[MAXFACES][MAXORIENT] =
 	{3, 0, 1, 4}
 };
 
+static DinoStack undo = {NULL, NULL, NULL, 0};
+static DinoStack redo = {NULL, NULL, NULL, 0};
+
 static void
 CheckPieces(DinoWidget w)
 {
@@ -428,7 +485,7 @@ GetStyle(DinoWidget w, int shift, int control, int alt)
 Boolean
 CheckSolved(DinoWidget w)
 {
-	int         face, position;
+	int face, position;
 	DinoCornerLoc test;
 
 	for (face = 0; face < MAXFACES; face++)
@@ -437,9 +494,9 @@ CheckSolved(DinoWidget w)
 				test.face = w->dino.cubeLoc[face][position].face;
 				test.rotation = w->dino.cubeLoc[face][position].rotation;
 			} else if (test.face !=		/*face */
-				   w->dino.cubeLoc[face][position].face ||
-				   (w->dino.orient && test.rotation !=	/*STRT - MAXORIENT */
-				    w->dino.cubeLoc[face][position].rotation))
+				  w->dino.cubeLoc[face][position].face ||
+				  (w->dino.orient && test.rotation !=	/*STRT - MAXORIENT */
+				  w->dino.cubeLoc[face][position].rotation))
 				return False;
 		}
 	return True;
@@ -449,7 +506,7 @@ CheckSolved(DinoWidget w)
 void
 PrintCube(DinoWidget w)
 {
-	int         face, position;
+	int face, position;
 
 	for (face = 0; face < MAXFACES; face++) {
 		for (position = 0; position < MAXORIENT; position++)
@@ -474,8 +531,13 @@ DrawTriangle(DinoWidget w, int face, int position, int offset)
 void
 DrawAllPieces(DinoWidget w)
 {
-	int         face, position;
+	int face, position;
 
+#ifdef HAVE_OPENGL
+	if (w->dino.dim == 4) {
+		DrawAllPiecesGL((DinoGLWidget) w);
+	}
+#endif
 	for (face = 0; face < MAXFACES; face++)
 		for (position = 0; position < MAXORIENT; position++)
 			DrawTriangle(w, face, position, FALSE);
@@ -488,18 +550,22 @@ DrawFrame(const DinoWidget w, const Boolean focus)
 		DrawFrame2D((Dino2DWidget) w, focus);
 	else if (w->dino.dim == 3)
 		DrawFrame3D((Dino3DWidget) w, focus);
+#ifdef HAVE_OPENGL
+	else if (w->dino.dim == 4)
+		DrawFrameGL((DinoGLWidget) w, focus);
+#endif
 }
 
 static void
 MoveNoPieces(DinoWidget w)
 {
-	SetDino(w, DINO_ILLEGAL);
+	setPuzzle(w, PUZZLE_ILLEGAL);
 }
 
 static void
 RotateFace(DinoWidget w, int face, int direction)
 {
-	int         side;
+	int side;
 
 	/* Read Face */
 	for (side = 0; side < MAXORIENT; side++)
@@ -513,6 +579,11 @@ RotateFace(DinoWidget w, int face, int direction)
 			(w->dino.cubeLoc[face][side].rotation + direction) % MAXORIENT;
 		DrawTriangle(w, face, side, FALSE);
 	}
+#ifdef HAVE_OPENGL
+	if (w->dino.dim == 4) {
+		DrawAllPiecesGL((DinoGLWidget) w);
+	}
+#endif
 }
 
 static void
@@ -533,12 +604,17 @@ WriteDiagonal(DinoWidget w, int face, int corner, int rotate, int h)
 	DrawTriangle(w, face, corner, FALSE);
 	w->dino.cubeLoc[face][(corner + 1) % MAXORIENT] = w->dino.spindleLoc[h][1];
 	DrawTriangle(w, face, (corner + 1) % MAXORIENT, FALSE);
+#ifdef HAVE_OPENGL
+	if (w->dino.dim == 4) {
+		DrawAllPiecesGL((DinoGLWidget) w);
+	}
+#endif
 }
 
 static void
 ReadFace(DinoWidget w, int face, int h)
 {
-	int         side;
+	int side;
 
 	for (side = 0; side < MAXORIENT; side++)
 		w->dino.rowLoc[h][side] = w->dino.cubeLoc[face][side];
@@ -547,7 +623,7 @@ ReadFace(DinoWidget w, int face, int h)
 static void
 WriteFace(DinoWidget w, int face, int rotate, int h)
 {
-	int         side, newSide;
+	int side, newSide;
 
 	for (side = 0; side < MAXORIENT; side++) {
 		newSide = (side + rotate) % MAXORIENT;
@@ -556,20 +632,25 @@ WriteFace(DinoWidget w, int face, int rotate, int h)
 			(w->dino.cubeLoc[face][newSide].rotation + rotate) % MAXORIENT;
 		DrawTriangle(w, face, (side + rotate) % MAXORIENT, FALSE);
 	}
+#ifdef HAVE_OPENGL
+	if (w->dino.dim == 4) {
+		DrawAllPiecesGL((DinoGLWidget) w);
+	}
+#endif
 }
 
 static void
 MoveEdges(DinoWidget w, int face, int corner, int direction)
 {
-	int         k, newFace, rotate, newCorner, newDirection;
+	int k, newFace, rotate, newCorner, newDirection;
 
 	ReadDiagonal((DinoWidget) w, face, corner, 0);
 	for (k = 1; k <= 2; k++) {
 		newFace = oppFace[face];
 		/*rotate = (((face == 1 || face == 3) ? 1 : 3) + 3 * direction) %
-		   MAXORIENT; */
+		  MAXORIENT; */
 		newCorner = ((((face == 1 || face == 3) + corner) % 2) ?
-			     (corner + 3) : (corner + 1)) % MAXORIENT;
+			(corner + 3) : (corner + 1)) % MAXORIENT;
 		rotate = (newCorner - corner + MAXORIENT) % MAXORIENT;
 		newDirection = (rotate + direction) % MAXORIENT;
 		if (k != 2)
@@ -584,7 +665,7 @@ MoveEdges(DinoWidget w, int face, int corner, int direction)
 static void
 MoveFaces(DinoWidget w, int f, int d, int rotate)
 {
-	int         k, face, newFace;
+	int k, face, newFace;
 
 	face = faceToRotate2[f][d][0];
 	ReadFace((DinoWidget) w, face, 0);
@@ -601,7 +682,7 @@ MoveFaces(DinoWidget w, int f, int d, int rotate)
 static void
 MoveInsideCorners(DinoWidget w, int face, int corner, int direction)
 {
-	int         newFace, newCorner, newDirection, dir, k;
+	int newFace, newCorner, newDirection, dir, k;
 
 	ReadDiagonal((DinoWidget) w, face, corner, 0);
 	for (k = 1; k <= MAXROTATE; k++) {
@@ -622,7 +703,7 @@ MoveInsideCorners(DinoWidget w, int face, int corner, int direction)
 static void
 MoveOutsideCorners(DinoWidget w, int face, int corner, int direction)
 {
-	int         newFace, newCorner, newDirection, dir, k;
+	int newFace, newCorner, newDirection, dir, k;
 
 	ReadDiagonal((DinoWidget) w, face, corner, 0);
 	for (k = 1; k <= MAXROTATE; k++) {
@@ -644,7 +725,7 @@ MoveOutsideCorners(DinoWidget w, int face, int corner, int direction)
 static void
 MovePieces(DinoWidget w, int face, int position, int direction, int style)
 {
-	int         corner, newCorner;
+	int corner, newCorner;
 
 	corner = (position - !((position + direction) % 2) + MAXORIENT) % MAXORIENT;
 	if (style == CORNER) {
@@ -664,7 +745,7 @@ MovePieces(DinoWidget w, int face, int position, int direction, int style)
 		MoveFaces(w, face, corner,
 			((face == 2 || face == 5) ? CCW : HALF) % MAXORIENT);
 	} else {
-		int         k, newFace, rotate, newDirection;
+		int k, newFace, rotate, newDirection;
 
 		RotateFace(w, faceToRotate[face][direction % MAXORIENT], CW);
 		RotateFace(w, faceToRotate[face][(direction + 2) % MAXORIENT], CCW);
@@ -685,14 +766,14 @@ MovePieces(DinoWidget w, int face, int position, int direction, int style)
 static void
 MoveControlCb(DinoWidget w, int face, int position, int direction, int style)
 {
-	int         newFace, newSide, newDirection, corner, newCorner;
+	int newFace, newSide, newDirection, corner, newCorner;
 
 	MovePieces(w, face, position, direction, style);
-	SetDinoMove(w, DINO_CONTROL, face, position, direction, style);
+	setPuzzleMove(w, PUZZLE_CONTROL, face, position, direction, style);
 	if (style == CORNER) {
 		newSide = (position + 2) % MAXORIENT;
 		MovePieces(w, face, newSide, direction, MIDDLE);
-		SetDinoMove(w, DINO_CONTROL, face, newSide, direction, MIDDLE);
+		setPuzzleMove(w, PUZZLE_CONTROL, face, newSide, direction, MIDDLE);
 		corner = (position - !((position + direction) % 2) + MAXORIENT) % MAXORIENT;
 		newFace = oppCorner[face][corner].face;
 		newCorner = oppCorner[face][corner].rotation;
@@ -700,12 +781,12 @@ MoveControlCb(DinoWidget w, int face, int position, int direction, int style)
 			(face != 1 && face != 3)) % 2) + !(newCorner % 2);
 		newSide = newCorner;
 		MovePieces(w, newFace, newSide, newDirection, CORNER);
-		SetDinoMove(w, DINO_CONTROL, newFace, newSide, newDirection,
+		setPuzzleMove(w, PUZZLE_CONTROL, newFace, newSide, newDirection,
 			CORNER);
 	} else if (style == MIDDLE) {
 		newSide = (position + 2) % MAXORIENT;
 		MovePieces(w, face, newSide, direction, CORNER);
-		SetDinoMove(w, DINO_CONTROL, face, newSide, direction,
+		setPuzzleMove(w, PUZZLE_CONTROL, face, newSide, direction,
 			CORNER);
 		corner = (position - !((position + direction) % 2) +
 			MAXORIENT) % MAXORIENT;
@@ -718,29 +799,30 @@ MoveControlCb(DinoWidget w, int face, int position, int direction, int style)
 		newDirection = 2 * (newDirection / 2) + !(newDirection % 2);
 		newSide = (newCorner + 2) % MAXORIENT;
 		MovePieces(w, newFace, newSide, newDirection, CORNER);
-		SetDinoMove(w, DINO_CONTROL, newFace, newSide, newDirection,
+		setPuzzleMove(w, PUZZLE_CONTROL, newFace, newSide, newDirection,
 			CORNER);
 	} else if (style == EDGE) {
 		newSide = (position + 2) % MAXORIENT;
 		MovePieces(w, face, newSide, direction, EDGE);
-		SetDinoMove(w, DINO_CONTROL, face, newSide, direction, EDGE);
+		setPuzzleMove(w, PUZZLE_CONTROL, face, newSide, direction, EDGE);
 	}
 }
 
 void
-MoveDino(DinoWidget w, int face, int position, int direction, int style,
+MovePuzzle(DinoWidget w, int face, int position, int direction, int style,
 		int control)
 {
 	if (control)
 		MoveControlCb(w, face, position, direction, style);
 	else {
 		MovePieces(w, face, position, direction, style);
-		SetDinoMove(w, DINO_MOVED, face, position, direction, style);
+		setPuzzleMove(w, PUZZLE_MOVED, face, position, direction, style);
 	}
-	PutMove(face, position, direction, style, control);
+	setMove(&undo, face, position, direction, style, control);
+	flushMoves(w, &redo, FALSE);
 }
 
-static      Boolean
+static Boolean
 SelectPieces(DinoWidget w, int x, int y, int *face, int *position)
 {
 	if (w->dino.dim == 2)
@@ -755,8 +837,10 @@ SelectPieces(DinoWidget w, int x, int y, int *face, int *position)
 static int
 CheckMoveDir(int position1, int position2, int *direction)
 {
+	if (position1 == position2)
+		return 2;
 	if (!((position1 - position2 + MAXORIENT) % 2))
-		return FALSE;
+		return 0;
 	switch (position1) {
 		case 0:
 			*direction = (position2 == 1) ? 1 : 2;
@@ -771,12 +855,12 @@ CheckMoveDir(int position1, int position2, int *direction)
 			*direction = (position2 == 0) ? 0 : 1;
 			break;
 		default:
-			return FALSE;
+			return 0;
 	}
-	return TRUE;
+	return 1;
 }
 
-static      Boolean
+static Boolean
 NarrowSelection(DinoWidget w, int *face, int *direction)
 {
 	if (w->dino.dim == 2)
@@ -786,7 +870,7 @@ NarrowSelection(DinoWidget w, int *face, int *direction)
 	return False;
 }
 
-static      Boolean
+static Boolean
 PositionPieces(DinoWidget w, int x, int y, int *face, int *position, int *direction)
 {
 	if (!SelectPieces(w, x, y, face, position))
@@ -795,10 +879,10 @@ PositionPieces(DinoWidget w, int x, int y, int *face, int *position, int *direct
 }
 
 void
-MoveDinoInput(DinoWidget w, int x, int y, int direction, int shift,
+MovePuzzleInput(DinoWidget w, int x, int y, int direction, int shift,
 		int control, int alt)
 {
-	int         face, position, style;
+	int face, position, style;
 
 	if (!w->dino.practice && !control && CheckSolved(w)) {
 		MoveNoPieces(w);
@@ -814,16 +898,16 @@ MoveDinoInput(DinoWidget w, int x, int y, int direction, int shift,
 	} else
 		style = GetStyle(w, shift, control, alt);
 	control = (control) ? 1 : 0;
-	MoveDino(w, face, position, direction, style, control);
+	MovePuzzle(w, face, position, direction, style, control);
 	if (!control && CheckSolved(w)) {
-		SetDino(w, DINO_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
 static void
 ResetPieces(DinoWidget w)
 {
-	int         face, position;
+	int face, position;
 
 	for (face = 0; face < MAXFACES; face++)
 		for (position = 0; position < MAXORIENT; position++) {
@@ -831,15 +915,18 @@ ResetPieces(DinoWidget w)
 			w->dino.cubeLoc[face][position].rotation = STRT -
 				MAXORIENT;
 		}
-	FlushMoves(w);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	w->dino.started = False;
+	w->dino.currentFace = PUZZLE_IGNORE;
+	w->dino.currentDirection = PUZZLE_IGNORE;
 }
 
 static void
 GetPieces(DinoWidget w)
 {
-	FILE       *fp;
-	int         c, mode, orient, practice, moves;
+	FILE *fp;
+	int c, mode, orient, practice, moves;
 	char *buf1 = NULL, *buf2 = NULL;
 	char *fname, *lname, *name;
 
@@ -878,18 +965,19 @@ GetPieces(DinoWidget w)
 		}
 #endif
 	}
-	FlushMoves(w);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &mode);
 	switch (mode) {
 		case PERIOD2:
-			SetDino(w, DINO_PERIOD2);
+			setPuzzle(w, PUZZLE_PERIOD2);
 			break;
 		case PERIOD3:
-			SetDino(w, DINO_PERIOD3);
+			setPuzzle(w, PUZZLE_PERIOD3);
 			break;
 		case BOTH:
-			SetDino(w, DINO_BOTH);
+			setPuzzle(w, PUZZLE_BOTH);
 			break;
 		default:
 			stringCat(&buf1, name, " corrupted: mode ");
@@ -909,24 +997,24 @@ GetPieces(DinoWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &orient);
 	if (w->dino.orient != (Boolean) orient) {
-		SetDino(w, DINO_ORIENT);
+		setPuzzle(w, PUZZLE_ORIENTIZE);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &practice);
 	if (w->dino.practice != (Boolean) practice) {
-		SetDino(w, DINO_PRACTICE);
+		setPuzzle(w, PUZZLE_PRACTICE);
 	}
 #ifdef WINVER
 	ResetPieces(w);
 #endif
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
-	ScanStartPosition(fp, w);
-	SetDino(w, DINO_RESTORE);
+	scanStartPosition(fp, w);
+	setPuzzle(w, PUZZLE_RESTORE);
 #ifdef WINVER
-	SetStartPosition(w);
+	setStartPosition(w);
 #endif
-	ScanMoves(fp, w, moves);
+	scanMoves(fp, w, moves);
 	(void) fclose(fp);
 	(void) printf("%s: mode %d, orient %d, practice %d, moves %d.\n",
 		name, mode, orient, practice, moves);
@@ -938,7 +1026,7 @@ GetPieces(DinoWidget w)
 static void
 WritePieces(DinoWidget w)
 {
-	FILE       *fp;
+	FILE *fp;
 	char *buf1 = NULL, *buf2 = NULL;
 	char *fname, *lname, *name;
 
@@ -981,28 +1069,26 @@ WritePieces(DinoWidget w)
 	(void) fprintf(fp, "orient%c %d\n", SYMBOL, (w->dino.orient) ? 1 : 0);
 	(void) fprintf(fp, "practice%c %d\n", SYMBOL,
 		(w->dino.practice) ? 1 : 0);
-	(void) fprintf(fp, "moves%c %d\n", SYMBOL, NumMoves());
-	PrintStartPosition(fp, w);
-	PrintMoves(fp);
+	(void) fprintf(fp, "moves%c %d\n", SYMBOL,
+		numMoves(&undo));
+	printStartPosition(fp, w);
+	printMoves(fp, &undo);
 	(void) fclose(fp);
 	(void) printf("Saved to %s.\n", name);
 	free(lname);
 	free(fname);
 }
 
-static void
-ClearPieces(DinoWidget w)
-{
-	SetDino(w, DINO_CLEAR);
-}
-
 static void
 UndoPieces(DinoWidget w)
 {
-	if (MadeMoves()) {
-		int         face, position, direction, style, control;
+	if (madeMoves(&undo)) {
+		int face, position, direction, style, control;
 
-		GetMove(&face, &position, &direction, &style, &control);
+		getMove(&undo,
+			&face, &position, &direction, &style, &control);
+		setMove(&redo,
+			face, position, direction, style, control);
 		direction = (direction < MAXORIENT) ?
 			(direction + MAXORIENT / 2) % MAXORIENT :
 			5 * MAXORIENT - direction;
@@ -1010,31 +1096,60 @@ UndoPieces(DinoWidget w)
 			MoveControlCb(w, face, position, direction, style);
 		else {
 			MovePieces(w, face, position, direction, style);
-			SetDinoMove(w, DINO_UNDO, face, position, direction,
+			setPuzzleMove(w, PUZZLE_UNDO, face, position, direction,
 				style);
 			if (CheckSolved(w)) {
-				SetDino(w, DINO_SOLVED);
+				setPuzzle(w, PUZZLE_SOLVED);
 			}
 		}
 	}
 }
 
+static void
+RedoPieces(DinoWidget w)
+{
+	if (madeMoves(&redo)) {
+		int face, position, direction, style, control;
+
+		getMove(&redo,
+			&face, &position, &direction, &style, &control);
+		setMove(&undo,
+			face, position, direction, style, control);
+		if (control)
+			MoveControlCb(w, face, position, direction, style);
+		else {
+			MovePieces(w, face, position, direction, style);
+			setPuzzleMove(w, PUZZLE_REDO, face, position, direction,
+				style);
+			if (CheckSolved(w)) {
+				setPuzzle(w, PUZZLE_SOLVED);
+			}
+		}
+	}
+}
+
+static void
+ClearPieces(DinoWidget w)
+{
+	setPuzzle(w, PUZZLE_CLEAR);
+}
+
 static void
 PracticePieces(DinoWidget w)
 {
-	SetDino(w, DINO_PRACTICE);
+	setPuzzle(w, PUZZLE_PRACTICE);
 }
 
 static void
 RandomizePieces(DinoWidget w)
 {
-	int         face, position, direction, style;
-	int         big = MAXORIENT * 3 + NRAND(2);
+	int face, position, direction, style;
+	int big = MAXORIENT * 3 + NRAND(2);
 
 	w->dino.cheat = False;
 	if (w->dino.practice)
 		PracticePieces(w);
-	SetDino(w, DINO_RESET);
+	setPuzzle(w, PUZZLE_RESET);
 
 #ifdef DEBUG
 	big = 3;
@@ -1051,12 +1166,13 @@ RandomizePieces(DinoWidget w)
 			style = NRAND(3);
 		else
 			style = NRAND(2);
-		MoveDino(w, face, position, direction, style, FALSE);
+		MovePuzzle(w, face, position, direction, style, FALSE);
 	}
-	FlushMoves(w);
-	SetDino(w, DINO_RANDOMIZE);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
+	setPuzzle(w, PUZZLE_RANDOMIZE);
 	if (CheckSolved(w)) {
-		SetDino(w, DINO_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
@@ -1066,26 +1182,31 @@ SolvePieces(DinoWidget w)
 	if (CheckSolved(w))
 		return;
 	{
-		SetDino(w, DINO_SOLVE_MESSAGE);
+		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
 	}
 }
 
 static void
 OrientizePieces(DinoWidget w)
 {
-	SetDino(w, DINO_ORIENT);
+	setPuzzle(w, PUZZLE_ORIENTIZE);
 }
 
+static void
+ViewPieces(DinoWidget w)
+{
+	setPuzzle(w, PUZZLE_VIEW);
+}
 
 #ifdef WINVER
 static void
-SetValuesDino(DinoWidget w)
+SetValuesPuzzle(DinoWidget w)
 {
 	struct tagColor {
-		int         red, green, blue;
+		int red, green, blue;
 	} color;
-	char        szBuf[80], buf[20], charbuf[2];
-	int         face;
+	char szBuf[80], buf[20], charbuf[2];
+	int face;
 
 	w->dino.mode = GetPrivateProfileInt(SECTION, "mode",
 		DEFAULTMODE, INIFILE);
@@ -1143,44 +1264,46 @@ SetValuesDino(DinoWidget w)
 }
 
 void
-DestroyDino(HBRUSH brush)
+DestroyPuzzle(HBRUSH brush)
 {
+	deleteMoves(&undo);
+	deleteMoves(&redo);
 	(void) DeleteObject(brush);
 	PostQuitMessage(0);
 }
 
 void
-ResizeDino(DinoWidget w)
+ResizePuzzle(DinoWidget w)
 {
 	if (w->dino.dim == 2)
-		ResizeDino2D((Dino2DWidget) w);
+		ResizePuzzle2D((Dino2DWidget) w);
 	else if (w->dino.dim == 3)
-		ResizeDino3D((Dino3DWidget) w);
+		ResizePuzzle3D((Dino3DWidget) w);
 }
 
 void
-SizeDino(DinoWidget w)
+SizePuzzle(DinoWidget w)
 {
 	ResetPieces(w);
-	ResizeDino(w);
+	ResizePuzzle(w);
 }
 
 void
-ExposeDino(DinoWidget w)
+ExposePuzzle(DinoWidget w)
 {
 	if (w->dino.dim == 2)
-		ExposeDino2D((Dino2DWidget) w);
+		ExposePuzzle2D((Dino2DWidget) w);
 	else if (w->dino.dim == 3)
-		ExposeDino3D((Dino3DWidget) w);
+		ExposePuzzle3D((Dino3DWidget) w);
 }
 
 #else
 static void
 GetColor(DinoWidget w, int face)
 {
-	XGCValues   values;
-	XtGCMask    valueMask;
-	XColor      colorCell, rgb;
+	XGCValues values;
+	XtGCMask valueMask;
+	XColor colorCell, rgb;
 
 	valueMask = GCForeground | GCBackground;
 	if (w->dino.reverse) {
@@ -1189,8 +1312,14 @@ GetColor(DinoWidget w, int face)
 		values.background = w->dino.background;
 	}
 	if (!w->dino.mono) {
-		if (XAllocNamedColor(XtDisplay(w),
-				  DefaultColormapOfScreen(XtScreen(w)),
+		if (!w->dino.faceName[face]) {
+			char *buf1;
+
+			intCat(&buf1, "Color name null for face ", face);
+			DISPLAY_WARNING(buf1);
+			free(buf1);
+		} else if (XAllocNamedColor(XtDisplay(w),
+				 DefaultColormapOfScreen(XtScreen(w)),
 				 w->dino.faceName[face], &colorCell, &rgb)) {
 			values.foreground = w->dino.faceColor[face] =
 				colorCell.pixel;
@@ -1227,9 +1356,9 @@ GetColor(DinoWidget w, int face)
 void
 SetAllColors(DinoWidget w)
 {
-	XGCValues   values;
-	XtGCMask    valueMask;
-	int         face;
+	XGCValues values;
+	XtGCMask valueMask;
+	int face;
 
 	valueMask = GCForeground | GCBackground;
 
@@ -1280,12 +1409,12 @@ SetAllColors(DinoWidget w)
 			w->dino.fontInfo->fid);
 }
 
-static      Boolean
-SetValuesDino(Widget current, Widget request, Widget renew)
+static Boolean
+SetValuesPuzzle(Widget current, Widget request, Widget renew)
 {
-	DinoWidget  c = (DinoWidget) current, w = (DinoWidget) renew;
-	Boolean     redraw = False, setColors = False;
-	int         face;
+	DinoWidget c = (DinoWidget) current, w = (DinoWidget) renew;
+	Boolean redraw = False, setColors = False;
+	int face;
 
 	for (face = 0; face < MAXFACES; face++) {
 		if (strcmp(w->dino.faceName[face], c->dino.faceName[face])) {
@@ -1319,64 +1448,65 @@ SetValuesDino(Widget current, Widget request, Widget renew)
 	}
 
 	if (w->dino.menu != -1) {
-		switch (w->dino.menu) {
-		case 0:
-			w->dino.menu = -1;
+		int menu = w->dino.menu;
+
+		w->dino.menu = -1;
+		switch (menu) {
+		case MENU_GET:
 			GetPieces(w);
 			break;
-		case 1:
-			w->dino.menu = -1;
+		case MENU_WRITE:
 			WritePieces(w);
 			break;
-		case 3:
-			w->dino.menu = -1;
-			ClearPieces(w);
-			break;
-		case 4:
-			w->dino.menu = -1;
+		case MENU_UNDO:
 			UndoPieces(w);
 			break;
-		case 5:
-			w->dino.menu = -1;
+		case MENU_REDO:
+			RedoPieces(w);
+			break;
+		case MENU_CLEAR:
+			ClearPieces(w);
+			break;
+		case MENU_RANDOMIZE:
 			RandomizePieces(w);
 			break;
-		case 6:
-			w->dino.menu = -1;
-			SolvePieces(w);
-			break;
-		case 7:
-			w->dino.menu = -1;
-			OrientizePieces(w);
-			break;
-		case 8:
-			w->dino.menu = -1;
+		case MENU_PRACTICE:
 			PracticePieces(w);
 			break;
+		case MENU_SOLVE:
+			SolvePieces(w);
+			break;
+		case MENU_ORIENTIZE:
+			OrientizePieces(w);
+			break;
+		case MENU_VIEW:
+			ViewPieces(w);
+			break;
 		default:
-			w->dino.menu = -1;
 			break;
 		}
 	}
-	if (w->dino.currentDirection == DINO_RESTORE) {
-		SetStartPosition(w);
-		w->dino.currentDirection = DINO_IGNORE;
-	} else if (w->dino.currentDirection == DINO_CLEAR) {
+	if (w->dino.currentDirection == PUZZLE_RESTORE) {
+		setStartPosition(w);
+		w->dino.currentDirection = PUZZLE_IGNORE;
+	} else if (w->dino.currentDirection == PUZZLE_CLEAR) {
 		ResetPieces(w);
 		redraw = TRUE;
-		w->dino.currentDirection = DINO_IGNORE;
-	} else if (w->dino.currentDirection != DINO_IGNORE) {
+		w->dino.currentDirection = PUZZLE_IGNORE;
+	} else if (w->dino.currentDirection > PUZZLE_IGNORE) {
 		MovePieces(w, w->dino.currentFace, w->dino.currentPosition,
-				w->dino.currentDirection, w->dino.style);
-		w->dino.currentDirection = DINO_IGNORE;
+			w->dino.currentDirection, w->dino.style);
+		w->dino.currentDirection = PUZZLE_IGNORE;
 	}
 	return redraw;
 }
 
 static void
-DestroyDino(Widget old)
+DestroyPuzzle(Widget old)
 {
-	DinoWidget  w = (DinoWidget) old;
-	int         face;
+	DinoWidget w = (DinoWidget) old;
+	Display *display = XtDisplay(w);
+	int face;
 
 	for (face = 0; face < MAXFACES; face++)
 		XtReleaseGC(old, w->dino.faceGC[face]);
@@ -1384,18 +1514,18 @@ DestroyDino(Widget old)
 	XtReleaseGC(old, w->dino.frameGC);
 	XtReleaseGC(old, w->dino.inverseGC);
 	XtRemoveCallbacks(old, XtNselectCallback, w->dino.select);
-}
-
-void
-QuitDino(DinoWidget w, XEvent * event, char **args, int nArgs)
-{
-	Display *display = XtDisplay(w);
-
 	if (w->dino.fontInfo) {
 		XUnloadFont(display, w->dino.fontInfo->fid);
 		XFreeFont(display, w->dino.fontInfo);
 	}
-	XtCloseDisplay(display);
+	deleteMoves(&undo);
+	deleteMoves(&redo);
+}
+
+void
+QuitPuzzle(DinoWidget w, XEvent *event, char **args, int nArgs)
+{
+	XtCloseDisplay(XtDisplay(w));
 	exit(0);
 }
 #endif
@@ -1404,7 +1534,7 @@ QuitDino(DinoWidget w, XEvent * event, char **args, int nArgs)
 static
 #endif
 void
-InitializeDino(
+InitializePuzzle(
 #ifdef WINVER
 DinoWidget w, HBRUSH brush
 #else
@@ -1413,7 +1543,7 @@ Widget request, Widget renew
 )
 {
 #ifdef WINVER
-	SetValuesDino(w);
+	SetValuesPuzzle(w);
 #else
 	DinoWidget w = (DinoWidget) renew;
 	int face;
@@ -1430,7 +1560,8 @@ Widget request, Widget renew
 	w->dino.focus = False;
 	loadFont(w);
 	CheckPieces(w);
-	InitMoves();
+	newMoves(&undo);
+	newMoves(&redo);
 	w->dino.cheat = False;
 	ResetPieces(w);
 #ifdef WINVER
@@ -1443,111 +1574,122 @@ Widget request, Widget renew
 }
 
 void
-HideDino(DinoWidget w
+HidePuzzle(DinoWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
-	SetDino(w, DINO_HIDE);
+	setPuzzle(w, PUZZLE_HIDE);
 }
 
 void
-SelectDino(DinoWidget w
+SelectPuzzle(DinoWidget w
 #ifdef WINVER
 , const int x, const int y, const int control
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
 #ifndef WINVER
-	int  x = event->xbutton.x, y = event->xbutton.y;
+	int x = event->xbutton.x, y = event->xbutton.y;
 	int control = (int) (event->xkey.state & ControlMask);
 #endif
 
-	if (SelectPieces(w, x, y,
-		       &(w->dino.currentFace), &(w->dino.currentPosition))) {
+	if (SelectPieces(w, x, y, &(w->dino.currentFace),
+			&(w->dino.currentPosition))) {
 		if (control || w->dino.practice || !CheckSolved(w))
 			DrawTriangle(w, w->dino.currentFace,
 				w->dino.currentPosition, TRUE);
 	} else {
-		w->dino.currentFace = DINO_IGNORE;
-		w->dino.currentDirection = DINO_IGNORE;
+		w->dino.currentFace = PUZZLE_IGNORE;
+		w->dino.currentDirection = PUZZLE_IGNORE;
 	}
 }
 
 void
-ReleaseDino(DinoWidget w
+ReleasePuzzle(DinoWidget w
 #ifdef WINVER
 , const int x, const int y, const int shift, const int control, const int alt
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
 #ifndef WINVER
-	int  x = event->xbutton.x, y = event->xbutton.y;
+	int x = event->xbutton.x, y = event->xbutton.y;
 	int shift = (int) (event->xbutton.state & (ShiftMask | LockMask));
 	int control = (int) (event->xkey.state & ControlMask);
-	int alt = (int) (event->xkey.state &
-		     (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask));
+	int alt = (int) (event->xkey.state & Mod1Mask);
 #endif
-	int         style, face, position, count = -1;
-	int         direction = 0;
+	int style, face, position, count = -1;
+	int direction = -1;
 
-	if (w->dino.currentFace == DINO_IGNORE)
+	if (w->dino.currentFace <= PUZZLE_IGNORE)
 		return;
 	DrawTriangle(w, w->dino.currentFace, w->dino.currentPosition, FALSE);
 	style = GetStyle(w, shift, control, alt);
 	if (!control && !w->dino.practice && CheckSolved(w))
 		MoveNoPieces(w);
-	else if (SelectPieces(w, x, y,
-		  &face, &position) && position != w->dino.currentPosition) {
-		if (face == w->dino.currentFace)
-			count = CheckMoveDir(w->dino.currentPosition, position,
-				&direction);
+	else if (SelectPieces(w, x, y, &face, &position)) {
+		if (face == w->dino.currentFace) {
+			count = CheckMoveDir(w->dino.currentPosition,
+				position, &direction);
+			/* count == 2 too annoying */
+			if (count == 2)
+				count = -1;
+		} else {
+			direction = facesToDirection[w->dino.currentFace][face][w->dino.currentPosition];
+			if (direction == PUZZLE_AMBIGUOUS &&
+					(style == CORNER || style == MIDDLE)) {
+				/* minimize ambiguity */
+				count = face;
+				face = w->dino.currentFace;
+				w->dino.currentFace = count;
+				w->dino.currentPosition = position;
+				direction = facesToDirection[w->dino.currentFace][face][w->dino.currentPosition];
+				if (direction >= 0)
+					direction = (direction + MAXORIENT / 2) % MAXORIENT;
+			}
+			count = (direction == PUZZLE_AMBIGUOUS) ? 2 : 1;
+			if (direction == PUZZLE_IGNORE)
+				count = 0;
+		}
 		if (count == 1) {
-			MoveDino(w, face, w->dino.currentPosition, direction,
+			MovePuzzle(w, w->dino.currentFace,
+				w->dino.currentPosition, direction,
 				style, (control) ? 1 : 0);
 			if (!control && CheckSolved(w)) {
-				SetDino(w, DINO_SOLVED);
+				setPuzzle(w, PUZZLE_SOLVED);
 			}
+		} else if (count == 2) {
+			setPuzzle(w, PUZZLE_AMBIGUOUS);
 		} else if (count == 0)
 			MoveNoPieces(w);
 	}
-	w->dino.currentFace = DINO_IGNORE;
-	w->dino.currentDirection = DINO_IGNORE;
-}
-
-void
-PracticeDino(DinoWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	PracticePieces(w);
+	w->dino.currentFace = PUZZLE_IGNORE;
+	w->dino.currentDirection = PUZZLE_IGNORE;
 }
 
 #ifndef WINVER
 void
-PracticeDinoMaybe(DinoWidget w
-, XEvent * event, char **args, int nArgs
+PracticePuzzleMaybe(DinoWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 	if (!w->dino.started)
 		PracticePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		SetDino(w, DINO_PRACTICE_QUERY);
+		setPuzzle(w, PUZZLE_PRACTICE_QUERY);
 	}
 #endif
 }
 
 void
-PracticeDino2(DinoWidget w
-, XEvent * event, char **args, int nArgs
+PracticePuzzle2(DinoWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 #ifdef HAVE_MOTIF
@@ -1555,36 +1697,24 @@ PracticeDino2(DinoWidget w
 #endif
 		PracticePieces(w);
 }
-#endif
 
 void
-RandomizeDino(DinoWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	RandomizePieces(w);
-}
-
-#ifndef WINVER
-void
-RandomizeDinoMaybe(DinoWidget w
-, XEvent * event, char **args, int nArgs
+RandomizePuzzleMaybe(DinoWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 	if (!w->dino.started)
 		RandomizePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		SetDino(w, DINO_RANDOMIZE_QUERY);
+		setPuzzle(w, PUZZLE_RANDOMIZE_QUERY);
 	}
 #endif
 }
 
 void
-RandomizeDino2(DinoWidget w
-, XEvent * event, char **args, int nArgs
+RandomizePuzzle2(DinoWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 #ifdef HAVE_MOTIF
@@ -1595,9 +1725,9 @@ RandomizeDino2(DinoWidget w
 #endif
 
 void
-GetDino(DinoWidget w
+GetPuzzle(DinoWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1605,9 +1735,9 @@ GetDino(DinoWidget w
 }
 
 void
-WriteDino(DinoWidget w
+WritePuzzle(DinoWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1615,19 +1745,9 @@ WriteDino(DinoWidget w
 }
 
 void
-ClearDino(DinoWidget w
+UndoPuzzle(DinoWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	ClearPieces(w);
-}
-
-void
-UndoDino(DinoWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1635,9 +1755,39 @@ UndoDino(DinoWidget w
 }
 
 void
-SolveDino(DinoWidget w
+RedoPuzzle(DinoWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RedoPieces(w);
+}
+
+void
+ClearPuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	ClearPieces(w);
+}
+
+void
+RandomizePuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RandomizePieces(w);
+}
+
+void
+SolvePuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1645,9 +1795,19 @@ SolveDino(DinoWidget w
 }
 
 void
-OrientizeDino(DinoWidget w
+PracticePuzzle(DinoWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	PracticePieces(w);
+}
+
+void
+OrientizePuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1656,34 +1816,42 @@ OrientizeDino(DinoWidget w
 
 #ifndef WINVER
 void
-Period2ModeDino(DinoWidget w
-, XEvent * event, char **args, int nArgs
+Period2ModePuzzle(DinoWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
-	SetDino(w, DINO_PERIOD2);
+	setPuzzle(w, PUZZLE_PERIOD2);
 }
 
 void
-Period3ModeDino(DinoWidget w
-, XEvent * event, char **args, int nArgs
+Period3ModePuzzle(DinoWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
-	SetDino(w, DINO_PERIOD3);
+	setPuzzle(w, PUZZLE_PERIOD3);
 }
 
 void
-BothModeDino(DinoWidget w
-, XEvent * event, char **args, int nArgs
+BothModePuzzle(DinoWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
-	SetDino(w, DINO_BOTH);
+	setPuzzle(w, PUZZLE_BOTH);
 }
 #endif
 
 void
-EnterDino(DinoWidget w
+ViewPuzzle(DinoWidget w
+, XEvent *event, char **args, int nArgs
+)
+{
+	ViewPieces(w);
+}
+
+void
+EnterPuzzle(DinoWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1692,9 +1860,9 @@ EnterDino(DinoWidget w
 }
 
 void
-LeaveDino(DinoWidget w
+LeavePuzzle(DinoWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1704,14 +1872,14 @@ LeaveDino(DinoWidget w
 
 #ifdef WINVER
 void
-PeriodModeDino(DinoWidget w, const int mode)
+PeriodModePuzzle(DinoWidget w, const int mode)
 {
-	SetDino(w, mode + DINO_PERIOD2);
+	setPuzzle(w, mode + PUZZLE_PERIOD2);
 }
 
 void
-DimDino(DinoWidget w)
+DimPuzzle(DinoWidget w)
 {
-	SetDino(w, DINO_DIM);
+	setPuzzle(w, PUZZLE_DIM);
 }
 #endif
