@@ -45,6 +45,8 @@
 #define XtNfaceColor4 ((char *) "faceColor4")
 #define XtNfaceColor5 ((char *) "faceColor5")
 #define XtNpieceBorder ((char *) "pieceBorder")
+#define XtNdelay ((char *) "delay")
+#define XtNview ((char *) "view")
 #define XtNmode ((char *) "mode")
 #define XtNorient ((char *) "orient")
 #define XtNpractice ((char *) "practice")
@@ -62,6 +64,8 @@
 #define XtNselectCallback ((char *) "selectCallback")
 #define XtCMono ((char *) "Mono")
 #define XtCReverse ((char *) "Reverse")
+#define XtCDelay ((char *) "Delay")
+#define XtCView ((char *) "View")
 #define XtCMode ((char *) "Mode")
 #define XtCOrient ((char *) "Orient")
 #define XtCPractice ((char *) "Practice")
@@ -84,35 +88,52 @@ typedef struct {
 } dinoCallbackStruct;
 #endif
 
-#define DINO_HIDE (-9)
-#define DINO_PRACTICE_QUERY (-8)
-#define DINO_RANDOMIZE_QUERY (-7)
-#define DINO_SOLVE_MESSAGE (-6)
-#define DINO_RESTORE (-5)
-#define DINO_CLEAR (-4)
-#define DINO_RESET (-3)
-#define DINO_ILLEGAL (-2)
-#define DINO_IGNORE (-1)
-#define DINO_MOVED 0
-#define DINO_CONTROL 1
-#define DINO_SOLVED 2
-#define DINO_PRACTICE 3
-#define DINO_RANDOMIZE 4
-#define DINO_DEC 5
-#define DINO_ORIENT 6
-#define DINO_INC 7
-#define DINO_PERIOD2 8
-#define DINO_PERIOD3 9
-#define DINO_BOTH 10
-#define DINO_COMPUTED 11
-#define DINO_UNDO 12
-#define DINO_DIM 13
+#define PUZZLE_HIDE (-10)
+#define PUZZLE_PRACTICE_QUERY (-9)
+#define PUZZLE_RANDOMIZE_QUERY (-8)
+#define PUZZLE_SOLVE_MESSAGE (-7)
+#define PUZZLE_RESTORE (-6)
+#define PUZZLE_CLEAR (-5)
+#define PUZZLE_RESET (-4)
+#define PUZZLE_AMBIGUOUS (-3)
+#define PUZZLE_ILLEGAL (-2)
+#define PUZZLE_IGNORE (-1)
+#define PUZZLE_MOVED 0
+#define PUZZLE_CONTROL 1
+#define PUZZLE_SOLVED 2
+#define PUZZLE_PRACTICE 3
+#define PUZZLE_RANDOMIZE 4
+#define PUZZLE_DEC 5
+#define PUZZLE_ORIENTIZE 6
+#define PUZZLE_INC 7
+#define PUZZLE_PERIOD2 8
+#define PUZZLE_PERIOD3 9
+#define PUZZLE_BOTH 10
+#define PUZZLE_COMPUTED 11
+#define PUZZLE_UNDO 12
+#define PUZZLE_REDO 13
+#define PUZZLE_DIM 14
+#define PUZZLE_VIEW 15
+
+#define MENU_GET 0
+#define MENU_WRITE 1
+#define MENU_EXIT 2
+#define MENU_UNDO 3
+#define MENU_REDO 4
+#define MENU_CLEAR 5
+#define MENU_RANDOMIZE 6
+#define MENU_PRACTICE 7
+#define MENU_SOLVE 8
+#define MENU_ORIENTIZE 9
+#define MENU_STICKY 10
+#define MENU_VIEW 11
 
 #define PERIOD2 2
 #define PERIOD3 3
 #define BOTH 4
 #define MAXMODES 3
 #define MAXFACES 6
+#define MAXVIEWS 2
 #define DEFAULTMODE PERIOD3
 #define DEFAULTORIENT FALSE
 #define DEFAULTPRACTICE FALSE
