@@ -29,20 +29,29 @@
 #include "Dino2dP.h"
 
 #ifndef WINVER
-static Boolean SetValuesDino2D(Widget current, Widget request, Widget renew);
-static void ResizeDino2D(Dino2DWidget w);
-static void InitializeDino2D(Widget request, Widget renew);
-static void ExposeDino2D(Widget renew, XEvent * event, Region region);
-static void MoveDino2DTl(Dino2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino2DTop(Dino2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino2DTr(Dino2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino2DLeft(Dino2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino2DRight(Dino2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino2DBl(Dino2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino2DBottom(Dino2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino2DBr(Dino2DWidget w, XEvent * event, char **args, int nArgs);
+static Boolean SetValuesPuzzle2D(Widget current, Widget request, Widget renew);
+static void ResizePuzzle2D(Dino2DWidget w);
+static void InitializePuzzle2D(Widget request, Widget renew);
+static void ExposePuzzle2D(Widget renew,
+	XEvent *event, Region region);
+static void MovePuzzle2DTl(Dino2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DTop(Dino2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DTr(Dino2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DLeft(Dino2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DRight(Dino2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DBl(Dino2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DBottom(Dino2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DBr(Dino2DWidget w,
+	XEvent *event, char **args, int nArgs);
 
-static char defaultTranslationsDino2D[] =
+static char defaultTranslationsPuzzle2D[] =
 "<KeyPress>q: Quit()\n\
  Ctrl<KeyPress>C: Quit()\n\
  <KeyPress>osfCancel: Hide()\n\
@@ -73,7 +82,7 @@ static char defaultTranslationsDino2D[] =
  <KeyPress>R12: MoveRight()\n\
  <KeyPress>End: MoveBl()\n\
  <KeyPress>KP_1: MoveBl()\n\
- <KeyPress>R1: MoveBl()\n\
+ <KeyPress>R13: MoveBl()\n\
  <KeyPress>Down: MoveBottom()\n\
  <KeyPress>osfDown: MoveBottom()\n\
  <KeyPress>KP_Down: MoveBottom()\n\
@@ -81,63 +90,67 @@ static char defaultTranslationsDino2D[] =
  <KeyPress>R14: MoveBottom()\n\
  <KeyPress>Next: MoveBr()\n\
  <KeyPress>KP_3: MoveBr()\n\
- <KeyPress>R3: MoveBr()\n\
+ <KeyPress>R15: MoveBr()\n\
  <Btn1Down>: Select()\n\
  <Btn1Up>: Release()\n\
- <KeyPress>p: Practice()\n\
  <Btn2Down>: PracticeMaybe()\n\
  <Btn2Down>(2+): Practice2()\n\
- <KeyPress>r: Randomize()\n\
  <Btn3Down>: RandomizeMaybe()\n\
  <Btn3Down>(2+): Randomize2()\n\
  <Btn4Down>: MoveTop()\n\
  <Btn5Down>: MoveBottom()\n\
  <KeyPress>g: Get()\n\
  <KeyPress>w: Write()\n\
- <KeyPress>c: Clear()\n\
  <KeyPress>u: Undo()\n\
+ <KeyPress>r: Redo()\n\
+ <KeyPress>c: Clear()\n\
+ <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
+ <KeyPress>p: Practice()\n\
  <KeyPress>o: Orientize()\n\
  <KeyPress>2: Period2()\n\
  <KeyPress>3: Period3()\n\
  <KeyPress>b: Both()\n\
+ <KeyPress>v: View()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
-static XtActionsRec actionsListDino2D[] =
+static XtActionsRec actionsListPuzzle2D[] =
 {
-	{(char *) "Quit", (XtActionProc) QuitDino},
-	{(char *) "Hide", (XtActionProc) HideDino},
-	{(char *) "MoveTl", (XtActionProc) MoveDino2DTl},
-	{(char *) "MoveTop", (XtActionProc) MoveDino2DTop},
-	{(char *) "MoveTr", (XtActionProc) MoveDino2DTr},
-	{(char *) "MoveLeft", (XtActionProc) MoveDino2DLeft},
-	{(char *) "MoveRight", (XtActionProc) MoveDino2DRight},
-	{(char *) "MoveBl", (XtActionProc) MoveDino2DBl},
-	{(char *) "MoveBottom", (XtActionProc) MoveDino2DBottom},
-	{(char *) "MoveBr", (XtActionProc) MoveDino2DBr},
-	{(char *) "Select", (XtActionProc) SelectDino},
-	{(char *) "Release", (XtActionProc) ReleaseDino},
-	{(char *) "Practice", (XtActionProc) PracticeDino},
-	{(char *) "PracticeMaybe", (XtActionProc) PracticeDinoMaybe},
-	{(char *) "Practice2", (XtActionProc) PracticeDino2},
-	{(char *) "Randomize", (XtActionProc) RandomizeDino},
-	{(char *) "RandomizeMaybe", (XtActionProc) RandomizeDinoMaybe},
-	{(char *) "Randomize2", (XtActionProc) RandomizeDino2},
-	{(char *) "Get", (XtActionProc) GetDino},
-	{(char *) "Write", (XtActionProc) WriteDino},
-	{(char *) "Clear", (XtActionProc) ClearDino},
-	{(char *) "Undo", (XtActionProc) UndoDino},
-	{(char *) "Solve", (XtActionProc) SolveDino},
-	{(char *) "Orientize", (XtActionProc) OrientizeDino},
-	{(char *) "Period2", (XtActionProc) Period2ModeDino},
-	{(char *) "Period3", (XtActionProc) Period3ModeDino},
-	{(char *) "Both", (XtActionProc) BothModeDino},
-	{(char *) "Enter", (XtActionProc) EnterDino},
-	{(char *) "Leave", (XtActionProc) LeaveDino}
+	{(char *) "Quit", (XtActionProc) QuitPuzzle},
+	{(char *) "Hide", (XtActionProc) HidePuzzle},
+	{(char *) "MoveTl", (XtActionProc) MovePuzzle2DTl},
+	{(char *) "MoveTop", (XtActionProc) MovePuzzle2DTop},
+	{(char *) "MoveTr", (XtActionProc) MovePuzzle2DTr},
+	{(char *) "MoveLeft", (XtActionProc) MovePuzzle2DLeft},
+	{(char *) "MoveRight", (XtActionProc) MovePuzzle2DRight},
+	{(char *) "MoveBl", (XtActionProc) MovePuzzle2DBl},
+	{(char *) "MoveBottom", (XtActionProc) MovePuzzle2DBottom},
+	{(char *) "MoveBr", (XtActionProc) MovePuzzle2DBr},
+	{(char *) "Select", (XtActionProc) SelectPuzzle},
+	{(char *) "Release", (XtActionProc) ReleasePuzzle},
+	{(char *) "PracticeMaybe", (XtActionProc) PracticePuzzleMaybe},
+	{(char *) "Practice2", (XtActionProc) PracticePuzzle2},
+	{(char *) "RandomizeMaybe", (XtActionProc) RandomizePuzzleMaybe},
+	{(char *) "Randomize2", (XtActionProc) RandomizePuzzle2},
+	{(char *) "Get", (XtActionProc) GetPuzzle},
+	{(char *) "Write", (XtActionProc) WritePuzzle},
+	{(char *) "Undo", (XtActionProc) UndoPuzzle},
+	{(char *) "Redo", (XtActionProc) RedoPuzzle},
+	{(char *) "Clear", (XtActionProc) ClearPuzzle},
+	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
+	{(char *) "Solve", (XtActionProc) SolvePuzzle},
+	{(char *) "Practice", (XtActionProc) PracticePuzzle},
+	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
+	{(char *) "Period2", (XtActionProc) Period2ModePuzzle},
+	{(char *) "Period3", (XtActionProc) Period3ModePuzzle},
+	{(char *) "Both", (XtActionProc) BothModePuzzle},
+	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Enter", (XtActionProc) EnterPuzzle},
+	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
 
-static XtResource resourcesDino2D[] =
+static XtResource resourcesPuzzle2D[] =
 {
 	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
 	 XtOffset(DinoWidget, core.width),
@@ -240,29 +253,29 @@ Dino2DClassRec dino2dClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeDino2D,	/* initialize */
+		(XtInitProc) InitializePuzzle2D,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
-		actionsListDino2D,	/* actions */
-		XtNumber(actionsListDino2D),	/* num actions */
-		resourcesDino2D,	/* resources */
-		XtNumber(resourcesDino2D),	/* num resources */
+		actionsListPuzzle2D,	/* actions */
+		XtNumber(actionsListPuzzle2D),	/* num actions */
+		resourcesPuzzle2D,	/* resources */
+		XtNumber(resourcesPuzzle2D),	/* num resources */
 		NULLQUARK,	/* xrm class */
 		TRUE,		/* compress motion */
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
 		NULL,		/* destroy */
-		(XtWidgetProc) ResizeDino2D,	/* resize */
-		(XtExposeProc) ExposeDino2D,	/* expose */
-		(XtSetValuesFunc) SetValuesDino2D,	/* set values */
+		(XtWidgetProc) ResizePuzzle2D,	/* resize */
+		(XtExposeProc) ExposePuzzle2D,	/* expose */
+		(XtSetValuesFunc) SetValuesPuzzle2D,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
 		XtInheritAcceptFocus,	/* accept focus */
 		XtVersion,	/* version */
 		NULL,		/* callback private */
-		defaultTranslationsDino2D,	/* tm table */
+		defaultTranslationsPuzzle2D,	/* tm table */
 		NULL,		/* query geometry */
 		NULL,		/* display accelerator */
 		NULL		/* extension */
@@ -278,9 +291,9 @@ Dino2DClassRec dino2dClassRec =
 WidgetClass dino2dWidgetClass = (WidgetClass) & dino2dClassRec;
 #endif
 
-static int  planeToCube[MAXRECT] =
+static int planeToCube[MAXRECT] =
 {6, 0, 6, 1, 2, 3, 6, 4, 6, 6, 5, 6};
-static int  cubeToPlane[MAXFACES] =
+static int cubeToPlane[MAXFACES] =
 {1, 3, 4, 5, 7, 10};
 static Point triangleUnit[MAXORIENT][4] =
 {
@@ -304,7 +317,7 @@ static Point triangleUnit[MAXORIENT][4] =
 		{1, 1},
 		{-1, 1},
 		{0, -2}}
-},          letterUnit[MAXORIENT] =
+}, letterUnit[MAXORIENT] =
 {
 	{1, 0},
 	{2, 1},
@@ -321,38 +334,38 @@ DrawOrientLine(Dino2DWidget w, int orient, int dx, int dy, GC borderGC)
 	switch (orient) {
 		case TOP:
 			DRAWLINE(w, dr, borderGC,
-				  dx,
-				  dy - w->dino.delta,
-				  dx,
-			  dy - w->dino.delta - w->dino.orientLineLength / 2);
+				dx,
+				dy - w->dino.delta,
+				dx,
+			dy - w->dino.delta - w->dino.orientLineLength / 2);
 			return;
 		case RIGHT:
 			DRAWLINE(w, dr, borderGC,
-				  dx + w->dino.delta,
-				  dy,
-			   dx + w->dino.delta + w->dino.orientLineLength / 2,
-				  dy);
+				dx + w->dino.delta,
+				dy,
+			dx + w->dino.delta + w->dino.orientLineLength / 2,
+				dy);
 			return;
 		case BOTTOM:
 			DRAWLINE(w, dr, borderGC,
-				  dx,
-				  dy + w->dino.delta,
-				  dx,
-			  dy + w->dino.delta + w->dino.orientLineLength / 2);
+				dx,
+				dy + w->dino.delta,
+				dx,
+			dy + w->dino.delta + w->dino.orientLineLength / 2);
 			return;
 		case LEFT:
 			DRAWLINE(w, dr, borderGC,
-				  dx - w->dino.delta,
-				  dy,
-			   dx - w->dino.delta - w->dino.orientLineLength / 2,
-				  dy);
+				dx - w->dino.delta,
+				dy,
+			dx - w->dino.delta - w->dino.orientLineLength / 2,
+				dy);
 			return;
 		default:
 			{
 				char *buf;
 
 				intCat(&buf, "DrawOrientLine: orient ",
-					 orient);
+					orient);
 				DISPLAY_WARNING(buf);
 				free(buf);
 			}
@@ -362,8 +375,8 @@ DrawOrientLine(Dino2DWidget w, int orient, int dx, int dy, GC borderGC)
 void
 DrawTriangle2D(Dino2DWidget w, int face, int position, int offset)
 {
-	GC          faceGC, borderGC;
-	int         dx, dy, letterX, letterY, newSide, orient;
+	GC faceGC, borderGC;
+	int dx, dy, letterX, letterY, newSide, orient;
 
 	orient = w->dino.cubeLoc[face][position].rotation;
 	if (w->dino.vertical || face != MAXFACES - 1) {
@@ -396,16 +409,17 @@ DrawTriangle2D(Dino2DWidget w, int face, int position, int offset)
 	POLYGON((DinoWidget) w, dr, faceGC, borderGC,
 		triangleList[newSide], 3, True, False);
 	if (w->dino.mono) {
-		char        buf[2];
+		char buf[2];
 
 		buf[0] =
 #ifdef WINVER
-		 	w->dino.faceChar[w->dino.cubeLoc
-			 	[face][position].face];
+			w->dino.faceChar[w->dino.cubeLoc
+				[face][position].face];
 #else
 			w->dino.faceName[w->dino.cubeLoc
 				[face][position].face][0];
 #endif
+		buf[1] = '\0';
 		if (offset) {
 			borderGC = w->dino.borderGC;
 		} else {
@@ -415,15 +429,16 @@ DrawTriangle2D(Dino2DWidget w, int face, int position, int offset)
 	}
 	if (w->dino.orient)
 		DrawOrientLine(w, orient + 2 * MAXORIENT,
-			       letterX - w->dino.letterOffset.x, letterY - w->dino.letterOffset.y,
-			       borderGC);
+			letterX - w->dino.letterOffset.x + 1,
+			letterY - w->dino.letterOffset.y,
+			borderGC);
 }
 
 void
 DrawFrame2D(Dino2DWidget w, Boolean focus)
 {
-	int         i;
-	Point       pos[MAXXY + 1], letters;
+	int i;
+	Point pos[MAXXY + 1], letters;
 	GC gc = (focus) ? w->dino.frameGC : w->dino.borderGC;
 
 	for (i = 0; i <= MAXXY; i++) {
@@ -437,13 +452,13 @@ DrawFrame2D(Dino2DWidget w, Boolean focus)
 	letters.x = pos[0].x + w->dino2d.viewLength / 2 - w->dino.delta;
 	letters.y = pos[0].y + w->dino2d.viewLength / 2;
 	DRAWTEXT(w, dr, w->dino.borderGC,
-		    (int) (letters.x + 5 * w->dino.letterOffset.x),
-		    (int) (letters.y + w->dino.letterOffset.y), "Front", 5);
+			(int) (letters.x + 5 * w->dino.letterOffset.x),
+			(int) (letters.y + w->dino.letterOffset.y), "Front", 5);
 	letters.x = pos[2].x + w->dino2d.viewLength / 2 - w->dino.delta;
 	letters.y = pos[2].y + w->dino2d.viewLength / 2;
 	DRAWTEXT(w, dr, w->dino.borderGC,
-		    (int) (letters.x + 4 * w->dino.letterOffset.x),
-		    (int) (letters.y + w->dino.letterOffset.y), "Back", 4);
+			(int) (letters.x + 4 * w->dino.letterOffset.x),
+			(int) (letters.y + w->dino.letterOffset.y), "Back", 4);
 	if (w->dino.vertical) {
 		DRAWLINE(w, dr, gc, pos[1].x, pos[0].y, pos[1].x, pos[4].y);
 		DRAWLINE(w, dr, gc, pos[2].x, pos[0].y, pos[2].x, pos[4].y);
@@ -462,9 +477,9 @@ DrawFrame2D(Dino2DWidget w, Boolean focus)
 static void
 ResizePieces(Dino2DWidget w)
 {
-	int         i, j;
+	int i, j;
 
-	w->dino.orientLineLength = w->dino2d.diamondLength / 3;
+	w->dino.orientLineLength = w->dino2d.diamondLength / 4 - 1;
 	w->dino.letterOffset.x = -2;
 	w->dino.letterOffset.y = 4;
 	for (i = 0; i < MAXORIENT; i++) {
@@ -517,7 +532,7 @@ ResizePieces(Dino2DWidget w)
 Boolean
 SelectPieces2D(Dino2DWidget w, int x, int y, int *face, int *position)
 {
-	int         faceX, faceY, i, j;
+	int faceX, faceY, i, j;
 
 	x -= w->dino.puzzleOffset.x;
 	y -= w->dino.puzzleOffset.y;
@@ -536,8 +551,8 @@ SelectPieces2D(Dino2DWidget w, int x, int y, int *face, int *position)
 	else
 		*position = MAXORIENT;
 	if ((faceX != 1 && faceY != 1) ||
-	    (faceX >= 3 && w->dino.vertical) ||
-	    (faceY >= 3 && !w->dino.vertical))
+			(faceX >= 3 && w->dino.vertical) ||
+			(faceY >= 3 && !w->dino.vertical))
 		return FALSE;
 	*face = planeToCube[faceX + faceY * MAXX];
 	if (*position != MAXORIENT && faceX == 3) {
@@ -545,7 +560,7 @@ SelectPieces2D(Dino2DWidget w, int x, int y, int *face, int *position)
 		*position = (*position + HALF) % MAXORIENT;
 	} else
 		*position %= MAXORIENT;
-	return TRUE;
+	return True;
 }
 
 Boolean
@@ -561,14 +576,14 @@ NarrowSelection2D(Dino2DWidget w, int *face, int *direction)
 }
 
 #ifndef WINVER
-static      Boolean
-SetValuesDino2D(Widget current, Widget request, Widget renew)
+static Boolean
+SetValuesPuzzle2D(Widget current, Widget request, Widget renew)
 {
 	Dino2DWidget c = (Dino2DWidget) current, w = (Dino2DWidget) renew;
-	Boolean     redraw = False;
+	Boolean redraw = False;
 
 	if (w->dino2d.diamondLength != c->dino2d.diamondLength) {
-		ResizeDino2D(w);
+		ResizePuzzle2D(w);
 		redraw = TRUE;
 	}
 	return (redraw);
@@ -579,10 +594,10 @@ SetValuesDino2D(Widget current, Widget request, Widget renew)
 static
 #endif
 void
-ResizeDino2D(Dino2DWidget w)
+ResizePuzzle2D(Dino2DWidget w)
 {
 #ifdef WINVER
-	RECT        rect;
+	RECT rect;
 
 	/* Determine size of client area */
 	(void) GetClientRect(w->core.hWnd, &rect);
@@ -620,7 +635,7 @@ ResizeDino2D(Dino2DWidget w)
 static
 #endif
 void
-InitializeDino2D(
+InitializePuzzle2D(
 #ifdef WINVER
 Dino2DWidget w
 #else
@@ -634,18 +649,18 @@ Widget request, Widget renew
 	SetAllColors((DinoWidget) w);
 #endif
 	w->dino.dim = 2;
-	ResizeDino2D(w);
+	ResizePuzzle2D(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-ExposeDino2D(
+ExposePuzzle2D(
 #ifdef WINVER
 Dino2DWidget w
 #else
-Widget renew, XEvent * event, Region region
+Widget renew, XEvent *event, Region region
 #endif
 )
 {
@@ -663,82 +678,74 @@ Widget renew, XEvent * event, Region region
 
 #ifndef WINVER
 static void
-MoveDino2DTl(Dino2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DTl(Dino2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, TL,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, TL,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino2DTop(Dino2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DTop(Dino2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, TOP,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, TOP,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino2DTr(Dino2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DTr(Dino2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, TR,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, TR,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino2DLeft(Dino2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DLeft(Dino2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino2DRight(Dino2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DRight(Dino2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino2DBl(Dino2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DBl(Dino2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, BL,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, BL,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino2DBottom(Dino2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DBottom(Dino2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino2DBr(Dino2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DBr(Dino2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, BR,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, BR,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 #endif
