@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1995 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1995 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -71,9 +71,9 @@ extern Boolean NarrowSelection2D(Dino2DWidget w, int *face, int *direction);
 extern void DrawFrame2D(Dino2DWidget w, Boolean focus);
 extern void DrawTriangle2D(Dino2DWidget w, int face, int position, int offset);
 #ifdef WINVER
-extern void ResizeDino2D(Dino2DWidget w);
-extern void InitializeDino2D(Dino2DWidget w);
-extern void ExposeDino2D(Dino2DWidget w);
+extern void ResizePuzzle2D(Dino2DWidget w);
+extern void InitializePuzzle2D(Dino2DWidget w);
+extern void ExposePuzzle2D(Dino2DWidget w);
 #endif
 
 #endif /* _Dino2dP_h */
