@@ -29,22 +29,32 @@
 #include "Dino3dP.h"
 
 #ifndef WINVER
-static Boolean SetValuesDino3D(Widget current, Widget request, Widget renew);
-static void ResizeDino3D(Dino3DWidget w);
-static void InitializeDino3D(Widget request, Widget renew);
-static void ExposeDino3D(Widget renew, XEvent * event, Region region);
-static void MoveDino3DCcw(Dino3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino3DTl(Dino3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino3DTop(Dino3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino3DTr(Dino3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino3DLeft(Dino3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino3DCw(Dino3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino3DRight(Dino3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino3DBl(Dino3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino3DBottom(Dino3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveDino3DBr(Dino3DWidget w, XEvent * event, char **args, int nArgs);
+static Boolean SetValuesPuzzle3D(Widget current, Widget request, Widget renew);
+static void ResizePuzzle3D(Dino3DWidget w);
+static void InitializePuzzle3D(Widget request, Widget renew);
+static void ExposePuzzle3D(Widget renew, XEvent *event, Region region);
+static void MovePuzzle3DCcw(Dino3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DTl(Dino3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DTop(Dino3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DTr(Dino3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DLeft(Dino3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DCw(Dino3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DRight(Dino3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DBl(Dino3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DBottom(Dino3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DBr(Dino3DWidget w,
+	XEvent *event, char **args, int nArgs);
 
-static char defaultTranslationsDino3D[] =
+static char defaultTranslationsPuzzle3D[] =
 "<KeyPress>q: Quit()\n\
  Ctrl<KeyPress>C: Quit()\n\
  <KeyPress>osfCancel: Hide()\n\
@@ -91,62 +101,66 @@ static char defaultTranslationsDino3D[] =
  <KeyPress>R15: MoveBr()\n\
  <Btn1Down>: Select()\n\
  <Btn1Up>: Release()\n\
- <KeyPress>p: Practice()\n\
  <Btn2Down>: PracticeMaybe()\n\
  <Btn2Down>(2+): Practice2()\n\
- <KeyPress>r: Randomize()\n\
  <Btn3Down>: RandomizeMaybe()\n\
  <Btn3Down>(2+): Randomize2()\n\
  <Btn4Down>: MoveTop()\n\
  <Btn5Down>: MoveBottom()\n\
  <KeyPress>g: Get()\n\
  <KeyPress>w: Write()\n\
- <KeyPress>c: Clear()\n\
  <KeyPress>u: Undo()\n\
+ <KeyPress>r: Redo()\n\
+ <KeyPress>c: Clear()\n\
+ <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
+ <KeyPress>p: Practice()\n\
  <KeyPress>o: Orientize()\n\
  <KeyPress>2: Period2()\n\
  <KeyPress>3: Period3()\n\
  <KeyPress>b: Both()\n\
+ <KeyPress>v: View()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
-static XtActionsRec actionsListDino3D[] =
+static XtActionsRec actionsListPuzzle3D[] =
 {
-	{(char *) "Quit", (XtActionProc) QuitDino},
-	{(char *) "Hide", (XtActionProc) HideDino},
-	{(char *) "MoveCcw", (XtActionProc) MoveDino3DCcw},
-	{(char *) "MoveTl", (XtActionProc) MoveDino3DTl},
-	{(char *) "MoveTop", (XtActionProc) MoveDino3DTop},
-	{(char *) "MoveTr", (XtActionProc) MoveDino3DTr},
-	{(char *) "MoveLeft", (XtActionProc) MoveDino3DLeft},
-	{(char *) "MoveCw", (XtActionProc) MoveDino3DCw},
-	{(char *) "MoveRight", (XtActionProc) MoveDino3DRight},
-	{(char *) "MoveBl", (XtActionProc) MoveDino3DBl},
-	{(char *) "MoveBottom", (XtActionProc) MoveDino3DBottom},
-	{(char *) "MoveBr", (XtActionProc) MoveDino3DBr},
-	{(char *) "Select", (XtActionProc) SelectDino},
-	{(char *) "Release", (XtActionProc) ReleaseDino},
-	{(char *) "Practice", (XtActionProc) PracticeDino},
-	{(char *) "PracticeMaybe", (XtActionProc) PracticeDinoMaybe},
-	{(char *) "Practice2", (XtActionProc) PracticeDino2},
-	{(char *) "Randomize", (XtActionProc) RandomizeDino},
-	{(char *) "RandomizeMaybe", (XtActionProc) RandomizeDinoMaybe},
-	{(char *) "Randomize2", (XtActionProc) RandomizeDino2},
-	{(char *) "Get", (XtActionProc) GetDino},
-	{(char *) "Write", (XtActionProc) WriteDino},
-	{(char *) "Clear", (XtActionProc) ClearDino},
-	{(char *) "Undo", (XtActionProc) UndoDino},
-	{(char *) "Solve", (XtActionProc) SolveDino},
-	{(char *) "Orientize", (XtActionProc) OrientizeDino},
-	{(char *) "Period2", (XtActionProc) Period2ModeDino},
-	{(char *) "Period3", (XtActionProc) Period3ModeDino},
-	{(char *) "Both", (XtActionProc) BothModeDino},
-	{(char *) "Enter", (XtActionProc) EnterDino},
-	{(char *) "Leave", (XtActionProc) LeaveDino}
+	{(char *) "Quit", (XtActionProc) QuitPuzzle},
+	{(char *) "Hide", (XtActionProc) HidePuzzle},
+	{(char *) "MoveCcw", (XtActionProc) MovePuzzle3DCcw},
+	{(char *) "MoveTl", (XtActionProc) MovePuzzle3DTl},
+	{(char *) "MoveTop", (XtActionProc) MovePuzzle3DTop},
+	{(char *) "MoveTr", (XtActionProc) MovePuzzle3DTr},
+	{(char *) "MoveLeft", (XtActionProc) MovePuzzle3DLeft},
+	{(char *) "MoveCw", (XtActionProc) MovePuzzle3DCw},
+	{(char *) "MoveRight", (XtActionProc) MovePuzzle3DRight},
+	{(char *) "MoveBl", (XtActionProc) MovePuzzle3DBl},
+	{(char *) "MoveBottom", (XtActionProc) MovePuzzle3DBottom},
+	{(char *) "MoveBr", (XtActionProc) MovePuzzle3DBr},
+	{(char *) "Select", (XtActionProc) SelectPuzzle},
+	{(char *) "Release", (XtActionProc) ReleasePuzzle},
+	{(char *) "PracticeMaybe", (XtActionProc) PracticePuzzleMaybe},
+	{(char *) "Practice2", (XtActionProc) PracticePuzzle2},
+	{(char *) "RandomizeMaybe", (XtActionProc) RandomizePuzzleMaybe},
+	{(char *) "Randomize2", (XtActionProc) RandomizePuzzle2},
+	{(char *) "Get", (XtActionProc) GetPuzzle},
+	{(char *) "Write", (XtActionProc) WritePuzzle},
+	{(char *) "Undo", (XtActionProc) UndoPuzzle},
+	{(char *) "Redo", (XtActionProc) RedoPuzzle},
+	{(char *) "Clear", (XtActionProc) ClearPuzzle},
+	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
+	{(char *) "Solve", (XtActionProc) SolvePuzzle},
+	{(char *) "Practice", (XtActionProc) PracticePuzzle},
+	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
+	{(char *) "Period2", (XtActionProc) Period2ModePuzzle},
+	{(char *) "Period3", (XtActionProc) Period3ModePuzzle},
+	{(char *) "Both", (XtActionProc) BothModePuzzle},
+	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Enter", (XtActionProc) EnterPuzzle},
+	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
 
-static XtResource resourcesDino3D[] =
+static XtResource resourcesPuzzle3D[] =
 {
 	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
 	 XtOffset(DinoWidget, core.width),
@@ -249,29 +263,29 @@ Dino3DClassRec dino3dClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeDino3D,	/* initialize */
+		(XtInitProc) InitializePuzzle3D,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
-		actionsListDino3D,	/* actions */
-		XtNumber(actionsListDino3D),	/* num actions */
-		resourcesDino3D,	/* resources */
-		XtNumber(resourcesDino3D),	/* num resources */
+		actionsListPuzzle3D,	/* actions */
+		XtNumber(actionsListPuzzle3D),	/* num actions */
+		resourcesPuzzle3D,	/* resources */
+		XtNumber(resourcesPuzzle3D),	/* num resources */
 		NULLQUARK,	/* xrm class */
 		TRUE,		/* compress motion */
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
 		NULL,		/* destroy */
-		(XtWidgetProc) ResizeDino3D,	/* resize */
-		(XtExposeProc) ExposeDino3D,	/* expose */
-		(XtSetValuesFunc) SetValuesDino3D,	/* set values */
+		(XtWidgetProc) ResizePuzzle3D,	/* resize */
+		(XtExposeProc) ExposePuzzle3D,	/* expose */
+		(XtSetValuesFunc) SetValuesPuzzle3D,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
 		XtInheritAcceptFocus,	/* accept focus */
 		XtVersion,	/* version */
 		NULL,		/* callback private */
-		defaultTranslationsDino3D,	/* tm table */
+		defaultTranslationsPuzzle3D,	/* tm table */
 		NULL,		/* query geometry */
 		NULL,		/* display accelerator */
 		NULL		/* extension */
@@ -303,34 +317,42 @@ DiamondOffset3D(Dino3DWidget w, int face, int *dx, int *dy)
 	if (w->dino.vertical) {
 		switch (face) {
 			case 0:
-				*dx = (2 * faceLoc3D[0][0].x + faceLoc3D[0][1].x +
-				       faceLoc3D[0][2].x) / 2;
-				*dy = (2 * faceLoc3D[0][0].y + faceLoc3D[0][1].y) / 2;
+				*dx = (2 * faceLoc3D[0][0].x +
+					faceLoc3D[0][1].x +
+					faceLoc3D[0][2].x) / 2;
+				*dy = (2 * faceLoc3D[0][0].y +
+					faceLoc3D[0][1].y) / 2;
 				break;
 			case 1:
 				*dx = faceLoc3D[1][0].x + faceLoc3D[1][1].x;
 				*dy = faceLoc3D[1][0].y;
 				break;
 			case 2:
-				*dx = (2 * faceLoc3D[2][0].x + faceLoc3D[2][1].x +
-				       faceLoc3D[2][2].x) / 2;
-				*dy = (2 * faceLoc3D[2][0].y + faceLoc3D[2][1].y +
-				       faceLoc3D[2][2].y) / 2;
+				*dx = (2 * faceLoc3D[2][0].x +
+					faceLoc3D[2][1].x +
+					faceLoc3D[2][2].x) / 2;
+				*dy = (2 * faceLoc3D[2][0].y +
+					faceLoc3D[2][1].y +
+					faceLoc3D[2][2].y) / 2;
 				break;
 			case 3:
 				*dx = faceLoc3D[3][0].x + faceLoc3D[3][1].x;
 				*dy = faceLoc3D[3][0].y;
 				break;
 			case 4:
-				*dx = (2 * faceLoc3D[4][0].x + faceLoc3D[4][1].x +
-				       faceLoc3D[4][2].x) / 2;
-				*dy = (2 * faceLoc3D[4][0].y + faceLoc3D[4][1].y +
-				       faceLoc3D[4][2].y) / 2;
+				*dx = (2 * faceLoc3D[4][0].x +
+					faceLoc3D[4][1].x +
+					faceLoc3D[4][2].x) / 2;
+				*dy = (2 * faceLoc3D[4][0].y +
+					faceLoc3D[4][1].y +
+				faceLoc3D[4][2].y) / 2;
 				break;
 			case 5:
-				*dx = (2 * faceLoc3D[5][0].x + faceLoc3D[5][1].x +
-				       faceLoc3D[5][2].x) / 2;
-				*dy = (2 * faceLoc3D[5][0].y + faceLoc3D[5][1].y) / 2;
+				*dx = (2 * faceLoc3D[5][0].x +
+					faceLoc3D[5][1].x +
+					faceLoc3D[5][2].x) / 2;
+				*dy = (2 * faceLoc3D[5][0].y +
+					faceLoc3D[5][1].y) / 2;
 				break;
 			default:
 			{
@@ -349,30 +371,38 @@ DiamondOffset3D(Dino3DWidget w, int face, int *dx, int *dy)
 				*dy = faceLoc3D[0][0].y + faceLoc3D[0][1].y;
 				break;
 			case 1:
-				*dx = (2 * faceLoc3D[1][0].x + faceLoc3D[1][1].x +
-				       faceLoc3D[1][2].x) / 2;
-				*dy = (2 * faceLoc3D[1][0].y + faceLoc3D[1][1].y +
-				       faceLoc3D[1][2].y) / 2;
+				*dx = (2 * faceLoc3D[1][0].x +
+					faceLoc3D[1][1].x +
+					faceLoc3D[1][2].x) / 2;
+				*dy = (2 * faceLoc3D[1][0].y +
+					faceLoc3D[1][1].y +
+					faceLoc3D[1][2].y) / 2;
 				break;
 			case 2:
-				*dx = (2 * faceLoc3D[2][0].x + faceLoc3D[2][1].x) / 2;
-				*dy = (2 * faceLoc3D[2][0].y + faceLoc3D[2][1].y +
-				       faceLoc3D[2][2].y) / 2;
+				*dx = (2 * faceLoc3D[2][0].x +
+					faceLoc3D[2][1].x) / 2;
+				*dy = (2 * faceLoc3D[2][0].y +
+					faceLoc3D[2][1].y +
+					faceLoc3D[2][2].y) / 2;
 				break;
 			case 3:
-				*dx = (2 * faceLoc3D[3][0].x + faceLoc3D[3][1].x) / 2;
-				*dy = (2 * faceLoc3D[3][0].y + faceLoc3D[3][1].y +
-				       faceLoc3D[3][2].y) / 2;
+				*dx = (2 * faceLoc3D[3][0].x +
+					faceLoc3D[3][1].x) / 2;
+				*dy = (2 * faceLoc3D[3][0].y +
+					faceLoc3D[3][1].y +
+					faceLoc3D[3][2].y) / 2;
 				break;
 			case 4:
 				*dx = faceLoc3D[4][0].x;
 				*dy = faceLoc3D[4][0].y + faceLoc3D[4][1].y;
 				break;
 			case 5:
-				*dx = (2 * faceLoc3D[5][0].x + faceLoc3D[5][1].x +
-				       faceLoc3D[5][2].x) / 2;
-				*dy = (2 * faceLoc3D[5][0].y + faceLoc3D[5][1].y +
-				       faceLoc3D[5][2].y) / 2;
+				*dx = (2 * faceLoc3D[5][0].x +
+					faceLoc3D[5][1].x +
+					faceLoc3D[5][2].x) / 2;
+				*dy = (2 * faceLoc3D[5][0].y +
+					faceLoc3D[5][1].y +
+					faceLoc3D[5][2].y) / 2;
 				break;
 			default:
 			{
@@ -389,7 +419,7 @@ DiamondOffset3D(Dino3DWidget w, int face, int *dx, int *dy)
 static void
 CubeOffset3D(Dino3DWidget w, int face, int side, int *dx, int *dy)
 {
-	int         corner;
+	int corner;
 
 	DiamondOffset3D(w, face, dx, dy);
 	for (corner = 1; corner <= side; corner++) {
@@ -459,8 +489,8 @@ MapFrom3D(Dino3DWidget w, int face, int side, int *corner)
 void
 DrawTriangle3D(Dino3DWidget w, int face, int position, int offset)
 {
-	GC          faceGC, borderGC;
-	int         corner = 0, dx, dy, letterX, letterY;
+	GC faceGC, borderGC;
+	int corner = 0, dx, dy, letterX, letterY;
 
 	MapTo3D(w, face, position, &corner);
 	CubeOffset3D(w, face, corner, &dx, &dy);
@@ -482,7 +512,7 @@ DrawTriangle3D(Dino3DWidget w, int face, int position, int offset)
 	POLYGON((DinoWidget) w, dr, faceGC, borderGC,
 		triangleLoc3D[face][corner], 3, True, False);
 	if (w->dino.mono) {
-		char        buf[2];
+		char buf[2];
 
 		buf[0] =
 #ifdef WINVER
@@ -499,52 +529,52 @@ DrawTriangle3D(Dino3DWidget w, int face, int position, int offset)
 			borderGC = w->dino.inverseGC;
 		}
 		DRAWTEXT(w, dr, borderGC,
-			    letterX + w->dino.letterOffset.x, letterY + w->dino.letterOffset.y,
-			    buf, 1);
+			letterX + w->dino.letterOffset.x, letterY + w->dino.letterOffset.y,
+			buf, 1);
 	}
 	if (w->dino.orient) {
 		DRAWLINE(w, dr, borderGC,
-			  letterX +
-			  orientTriangle[face][w->dino.cubeLoc[face][position].rotation][0].x,
-			  letterY +
-			  orientTriangle[face][w->dino.cubeLoc[face][position].rotation][0].y,
-			  letterX +
-			  orientTriangle[face][w->dino.cubeLoc[face][position].rotation][1].x,
-			  letterY +
-			  orientTriangle[face][w->dino.cubeLoc[face][position].rotation][1].y);
+			letterX +
+			orientTriangle[face][w->dino.cubeLoc[face][position].rotation][0].x,
+			letterY +
+			orientTriangle[face][w->dino.cubeLoc[face][position].rotation][0].y,
+			letterX +
+			orientTriangle[face][w->dino.cubeLoc[face][position].rotation][1].x,
+			letterY +
+			orientTriangle[face][w->dino.cubeLoc[face][position].rotation][1].y);
 	}
 }
 
 void
 DrawFrame3D(Dino3DWidget w, Boolean focus)
 {
-	int         face, dx, dy;
+	int face, dx, dy;
 	GC gc = (focus) ? w->dino.frameGC : w->dino.borderGC;
 
 	dx = w->dino3d.viewSize.x + w->dino.puzzleOffset.x;
 	dy = w->dino3d.viewSize.y + w->dino.puzzleOffset.y;
 	if (w->dino.vertical) {
 		DRAWLINE(w, dr, w->dino.frameGC,
-			  0, dy, dx + w->dino.puzzleOffset.x + 1, dy);
+			0, dy, dx + w->dino.puzzleOffset.x + 1, dy);
 		DRAWTEXT(w, dr, w->dino.borderGC,
-			    (int) (2 * w->dino.delta),
-			  (int) (3 * w->dino.delta + w->dino.letterOffset.y),
-			    "Front", 5);
+			(int) (2 * w->dino.delta),
+			(int) (3 * w->dino.delta + w->dino.letterOffset.y),
+			"Front", 5);
 		DRAWTEXT(w, dr, w->dino.borderGC, (int)
-			    (-4 * w->dino.delta + 2 * 6 * w->dino.letterOffset.x + w->core.width),
-			    (int) (-w->dino.delta - 2 * w->dino.letterOffset.y + w->core.height),
-			    "Back", 4);
+			(-4 * w->dino.delta + 2 * 6 * w->dino.letterOffset.x + w->core.width),
+			(int) (-w->dino.delta - 2 * w->dino.letterOffset.y + w->core.height),
+			"Back", 4);
 	} else {
 		DRAWLINE(w, dr, w->dino.frameGC,
-			  dx, 0, dx, dy + w->dino.puzzleOffset.y + 1);
+			dx, 0, dx, dy + w->dino.puzzleOffset.y + 1);
 		DRAWTEXT(w, dr, w->dino.borderGC,
-			    (int) (2 * w->dino.delta),
-			  (int) (3 * w->dino.delta + w->dino.letterOffset.y),
-			    "Front", 5);
+			(int) (2 * w->dino.delta),
+			(int) (3 * w->dino.delta + w->dino.letterOffset.y),
+			"Front", 5);
 		DRAWTEXT(w, dr, w->dino.borderGC, (int)
-			    (-4 * w->dino.delta + 2 * 6 * w->dino.letterOffset.x + w->core.width),
-			    (int) (-w->dino.delta - 2 * w->dino.letterOffset.y + w->core.height),
-			    "Back", 4);
+			(-4 * w->dino.delta + 2 * 6 * w->dino.letterOffset.x + w->core.width),
+			(int) (-w->dino.delta - 2 * w->dino.letterOffset.y + w->core.height),
+			"Back", 4);
 	}
 	for (face = 0; face < MAXFACES; face++) {
 		POLYLINE((DinoWidget) w, dr, gc,
@@ -555,11 +585,11 @@ DrawFrame3D(Dino3DWidget w, Boolean focus)
 static void
 ResizePieces(Dino3DWidget w)
 {
-	int         face, orient, side, corner;
-	Point       subcubeLoc3D[MAXFACES][MAXORIENT];
-	Point       orientCubeLoc3D[2][MAXFACES][MAXORIENT];
-	Point       subdiamondLoc3D[MAXFACES][MAXORIENT];
-	Point       orientDiamondLoc3D[2][MAXFACES][MAXORIENT];
+	int face, orient, side, corner;
+	Point subcubeLoc3D[MAXFACES][MAXORIENT];
+	Point orientCubeLoc3D[2][MAXFACES][MAXORIENT];
+	Point subdiamondLoc3D[MAXFACES][MAXORIENT];
+	Point orientDiamondLoc3D[2][MAXFACES][MAXORIENT];
 
 	w->dino.letterOffset.x = -2;
 	w->dino.letterOffset.y = 3;
@@ -670,17 +700,17 @@ ResizePieces(Dino3DWidget w)
 				4 * faceLoc3D[face][orient].y * w->dino.delta /
 				w->dino3d.faceSize.y;
 			orientCubeLoc3D[0][face][orient].x = (faceLoc3D[face][orient].x -
-			      5 * faceLoc3D[face][orient].x * w->dino.delta /
-						   w->dino3d.faceSize.x) / 3;
+				5 * faceLoc3D[face][orient].x * w->dino.delta /
+				w->dino3d.faceSize.x) / 3;
 			orientCubeLoc3D[0][face][orient].y = (faceLoc3D[face][orient].y -
-			      5 * faceLoc3D[face][orient].y * w->dino.delta /
-						   w->dino3d.faceSize.y) / 3;
+				5 * faceLoc3D[face][orient].y * w->dino.delta /
+				w->dino3d.faceSize.y) / 3;
 			orientCubeLoc3D[1][face][orient].x = (faceLoc3D[face][orient].x -
-			      7 * faceLoc3D[face][orient].x * w->dino.delta /
-						   w->dino3d.faceSize.x) / 6;
+				7 * faceLoc3D[face][orient].x * w->dino.delta /
+				w->dino3d.faceSize.x) / 6;
 			orientCubeLoc3D[1][face][orient].y = (faceLoc3D[face][orient].y -
-			      7 * faceLoc3D[face][orient].y * w->dino.delta /
-						   w->dino3d.faceSize.y) / 6;
+				7 * faceLoc3D[face][orient].y * w->dino.delta /
+				w->dino3d.faceSize.y) / 6;
 		}
 
 		MapFrom3D(w, face, 1, &corner);
@@ -690,55 +720,55 @@ ResizePieces(Dino3DWidget w)
 		orientTriangle[face][corner][1].y = -orientCubeLoc3D[1][face][2].y / 2;
 		for (orient = 0; orient < MAXORIENT - 1; orient++) {
 			diamondLoc3D[face][orient].x = (cubeLoc3D[face][orient].x +
-					  cubeLoc3D[face][orient + 1].x) / 2;
+				cubeLoc3D[face][orient + 1].x) / 2;
 			diamondLoc3D[face][orient].y = (cubeLoc3D[face][orient].y +
-					  cubeLoc3D[face][orient + 1].y) / 2;
+				cubeLoc3D[face][orient + 1].y) / 2;
 			subdiamondLoc3D[face][orient].x = (subcubeLoc3D[face][orient].x +
-				       subcubeLoc3D[face][orient + 1].x) / 2;
+				subcubeLoc3D[face][orient + 1].x) / 2;
 			subdiamondLoc3D[face][orient].y = (subcubeLoc3D[face][orient].y +
-				       subcubeLoc3D[face][orient + 1].y) / 2;
+				subcubeLoc3D[face][orient + 1].y) / 2;
 			tinyDiamondLoc3D[face][orient + 1].x = diamondLoc3D[face][orient].x -
 				subdiamondLoc3D[face][orient].x;
 			tinyDiamondLoc3D[face][orient + 1].y = diamondLoc3D[face][orient].y -
 				subdiamondLoc3D[face][orient].y;
 			orientDiamondLoc3D[0][face][orient].x =
 				(orientCubeLoc3D[0][face][orient].x +
-				 orientCubeLoc3D[0][face][orient + 1].x) / 2;
+				orientCubeLoc3D[0][face][orient + 1].x) / 2;
 			orientDiamondLoc3D[0][face][orient].y =
 				(orientCubeLoc3D[0][face][orient].y +
-				 orientCubeLoc3D[0][face][orient + 1].y) / 2;
+				orientCubeLoc3D[0][face][orient + 1].y) / 2;
 			orientDiamondLoc3D[1][face][orient].x =
 				(orientCubeLoc3D[1][face][orient].x +
-				 orientCubeLoc3D[1][face][orient + 1].x) / 2;
+				orientCubeLoc3D[1][face][orient + 1].x) / 2;
 			orientDiamondLoc3D[1][face][orient].y =
 				(orientCubeLoc3D[1][face][orient].y +
-				 orientCubeLoc3D[1][face][orient + 1].y) / 2;
+				orientCubeLoc3D[1][face][orient + 1].y) / 2;
 		}
 		/* Its a parallelagram so take advantage of that */
 		diamondLoc3D[face][orient].x = (cubeLoc3D[face][MAXORIENT - 1].x -
-				       cubeLoc3D[face][MAXORIENT / 2].x) / 2;
+			cubeLoc3D[face][MAXORIENT / 2].x) / 2;
 		diamondLoc3D[face][orient].y = (cubeLoc3D[face][MAXORIENT - 1].y -
-				       cubeLoc3D[face][MAXORIENT / 2].y) / 2;
+			cubeLoc3D[face][MAXORIENT / 2].y) / 2;
 		subdiamondLoc3D[face][orient].x = (subcubeLoc3D[face][MAXORIENT - 1].x -
-				    subcubeLoc3D[face][MAXORIENT / 2].x) / 2;
+			subcubeLoc3D[face][MAXORIENT / 2].x) / 2;
 		subdiamondLoc3D[face][orient].y = (subcubeLoc3D[face][MAXORIENT - 1].y -
-				    subcubeLoc3D[face][MAXORIENT / 2].y) / 2;
+			subcubeLoc3D[face][MAXORIENT / 2].y) / 2;
 		tinyDiamondLoc3D[face][0].x = diamondLoc3D[face][orient].x -
 			subdiamondLoc3D[face][orient].x;
 		tinyDiamondLoc3D[face][0].y = diamondLoc3D[face][orient].y -
 			subdiamondLoc3D[face][orient].y;
 		orientDiamondLoc3D[0][face][orient].x =
 			(orientCubeLoc3D[0][face][MAXORIENT - 1].x -
-			 orientCubeLoc3D[0][face][MAXORIENT / 2].x) / 2;
+			orientCubeLoc3D[0][face][MAXORIENT / 2].x) / 2;
 		orientDiamondLoc3D[0][face][orient].y =
 			(orientCubeLoc3D[0][face][MAXORIENT - 1].y -
-			 orientCubeLoc3D[0][face][MAXORIENT / 2].y) / 2;
+			orientCubeLoc3D[0][face][MAXORIENT / 2].y) / 2;
 		orientDiamondLoc3D[1][face][orient].x =
 			(orientCubeLoc3D[1][face][MAXORIENT - 1].x -
-			 orientCubeLoc3D[1][face][MAXORIENT / 2].x) / 2;
+			orientCubeLoc3D[1][face][MAXORIENT / 2].x) / 2;
 		orientDiamondLoc3D[1][face][orient].y =
 			(orientCubeLoc3D[1][face][MAXORIENT - 1].y -
-			 orientCubeLoc3D[1][face][MAXORIENT / 2].y) / 2;
+			orientCubeLoc3D[1][face][MAXORIENT / 2].y) / 2;
 
 		MapFrom3D(w, face, 1, &corner);
 		orientTriangle[face][corner][0].x = -orientCubeLoc3D[0][face][2].x / 2;
@@ -773,10 +803,10 @@ ResizePieces(Dino3DWidget w)
 		for (orient = 0; orient < MAXORIENT; orient++) {
 			letter3DList[face][orient].x =
 				(2 * triangleLoc3D[face][orient][1].x +
-				 triangleLoc3D[face][orient][2].x) / 3;
+				triangleLoc3D[face][orient][2].x) / 3;
 			letter3DList[face][orient].y =
 				(2 * triangleLoc3D[face][orient][1].y +
-				 triangleLoc3D[face][orient][2].y) / 3;
+				triangleLoc3D[face][orient][2].y) / 3;
 			triangleLoc3D[face][orient][3].x =
 				-triangleLoc3D[face][orient][1].x - triangleLoc3D[face][orient][2].x;
 			triangleLoc3D[face][orient][3].y =
@@ -808,8 +838,8 @@ ResizePieces(Dino3DWidget w)
 Boolean
 SelectPieces3D(Dino3DWidget w, int x, int y, int *face, int *position)
 {
-	int         u, v, front, tl, ur, ul, found, corner, dx, dy;
-        int 	    x_1, y_1, x_2, y_2;
+	int u, v, front, tl, ur, ul, found, corner, dx, dy;
+	int x_1, y_1, x_2, y_2;
 
 	x_1 = x;
 	y_1 = y;
@@ -863,7 +893,7 @@ SelectPieces3D(Dino3DWidget w, int x, int y, int *face, int *position)
 	found = 0;
 	DiamondOffset3D(w, *face, &dx, &dy);
 	for (corner = 0; corner < MAXORIENT; corner++) {
-		int         ok = 0;
+		int ok = 0;
 
 		x_1 = dx + triangleLoc3D[*face][corner][1].x;
 		y_1 = dy + triangleLoc3D[*face][corner][1].y;
@@ -947,14 +977,14 @@ NarrowSelection3D(Dino3DWidget w, int *face, int *direction)
 }
 
 #ifndef WINVER
-static      Boolean
-SetValuesDino3D(Widget current, Widget request, Widget renew)
+static Boolean
+SetValuesPuzzle3D(Widget current, Widget request, Widget renew)
 {
 	Dino3DWidget c = (Dino3DWidget) current, w = (Dino3DWidget) renew;
-	Boolean     redraw = False;
+	Boolean redraw = False;
 
 	if (w->dino3d.cubeSize.x != c->dino3d.cubeSize.x) {
-		ResizeDino3D(w);
+		ResizePuzzle3D(w);
 		redraw = True;
 	}
 	return (redraw);
@@ -965,30 +995,29 @@ SetValuesDino3D(Widget current, Widget request, Widget renew)
 static
 #endif
 void
-ResizeDino3D(Dino3DWidget w)
+ResizePuzzle3D(Dino3DWidget w)
 {
-	Point      tempSize;
+	Point tempSize;
 #ifdef WINVER
-	RECT        rect;
+	RECT rect;
 
 	/* Determine size of client area */
 	(void) GetClientRect(w->core.hWnd, &rect);
 	w->core.width = rect.right;
 	w->core.height = rect.bottom;
 #endif
-
 	w->dino.delta = 4;
 	w->dino.vertical = (w->core.height >= w->core.width);
 	if (w->dino.vertical) {
 		tempSize.y = w->core.height / MAXVIEWS;
 		tempSize.x = w->core.width;
 		if (tempSize.x >= DIVIDE(tempSize.y)) {
-			w->dino3d.cubeSize.y = MAX((tempSize.y - 3 * w->dino.delta) / 2 -
-						   w->dino.delta - 2, 0);
+			w->dino3d.cubeSize.y = MAX((tempSize.y -
+				3 * w->dino.delta) / 2 - w->dino.delta - 2, 0);
 			w->dino3d.cubeSize.x = DIVIDE(w->dino3d.cubeSize.y);
 		} else {
-			w->dino3d.cubeSize.x = MAX((tempSize.x - 2 * w->dino.delta - 7) / 2 -
-						   w->dino.delta, 0);
+			w->dino3d.cubeSize.x = MAX((tempSize.x -
+				2 * w->dino.delta - 7) / 2 - w->dino.delta, 0);
 			w->dino3d.cubeSize.y = MULTIPLY(w->dino3d.cubeSize.x);
 		}
 		w->dino3d.cubeDiagonal = w->dino3d.cubeSize.x / 2;
@@ -1003,12 +1032,12 @@ ResizeDino3D(Dino3DWidget w)
 		tempSize.x = w->core.width / MAXVIEWS;
 		tempSize.y = w->core.height;
 		if (tempSize.y >= DIVIDE(tempSize.x)) {
-			w->dino3d.cubeSize.x = MAX((tempSize.x - 3 * w->dino.delta) / 2 -
-						   w->dino.delta - 2, 0);
+			w->dino3d.cubeSize.x = MAX((tempSize.x -
+				3 * w->dino.delta) / 2 - w->dino.delta - 2, 0);
 			w->dino3d.cubeSize.y = DIVIDE(w->dino3d.cubeSize.x);
 		} else {
-			w->dino3d.cubeSize.y = MAX((tempSize.y - 2 * w->dino.delta - 7) / 2 -
-						   w->dino.delta, 0);
+			w->dino3d.cubeSize.y = MAX((tempSize.y -
+				2 * w->dino.delta - 7) / 2 - w->dino.delta, 0);
 			w->dino3d.cubeSize.x = MULTIPLY(w->dino3d.cubeSize.y);
 		}
 		w->dino3d.cubeDiagonal = w->dino3d.cubeSize.y / 2;
@@ -1030,7 +1059,7 @@ ResizeDino3D(Dino3DWidget w)
 static
 #endif
 void
-InitializeDino3D(
+InitializePuzzle3D(
 #ifdef WINVER
 Dino3DWidget w
 #else
@@ -1044,18 +1073,18 @@ Widget request, Widget renew
 	SetAllColors((DinoWidget) w);
 #endif
 	w->dino.dim = 3;
-	ResizeDino3D(w);
+	ResizePuzzle3D(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-ExposeDino3D(
+ExposePuzzle3D(
 #ifdef WINVER
 Dino3DWidget w
 #else
-Widget renew, XEvent * event, Region region
+Widget renew, XEvent *event, Region region
 #endif
 )
 {
@@ -1066,109 +1095,99 @@ Widget renew, XEvent * event, Region region
 		return;
 #endif
 	FILLRECTANGLE(w, dr, w->dino.inverseGC,
-			0, 0, w->core.width, w->core.height);
+		0, 0, w->core.width, w->core.height);
 	DrawFrame3D(w, w->dino.focus);
 	DrawAllPieces((DinoWidget) w);
 }
 
 #ifndef WINVER
 static void
-MoveDino3DTl(Dino3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DTl(Dino3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, TL,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, TL,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino3DTop(Dino3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DTop(Dino3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, TOP,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, TOP,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino3DTr(Dino3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DTr(Dino3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, TR,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, TR,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino3DLeft(Dino3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DLeft(Dino3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino3DCw(Dino3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DCw(Dino3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, CW,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, CW,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino3DRight(Dino3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DRight(Dino3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino3DBl(Dino3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DBl(Dino3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, BL,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, BL,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino3DBottom(Dino3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DBottom(Dino3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino3DBr(Dino3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DBr(Dino3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, BR,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, BR,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveDino3DCcw(Dino3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DCcw(Dino3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveDinoInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, CCW,
-		      (int) (event->xkey.state & (ShiftMask | LockMask)),
-		      (int) (event->xkey.state & ControlMask),
-		      (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+	MovePuzzleInput((DinoWidget) w, event->xbutton.x, event->xbutton.y, CCW,
+		(int) (event->xkey.state & (ShiftMask | LockMask)),
+		(int) (event->xkey.state & ControlMask),
+		(int) (event->xkey.state & Mod1Mask));
 }
 #endif
