@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1995 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1995 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -50,7 +50,6 @@ extern Dino3DClassRec dino3dClassRec;
 #define DOWN 0
 #define UP 1
 #define COORD3D 12
-#define MAXVIEWS 2
 #define MAXORIENT 4
 
 #define SQRT_3 1.732050808
@@ -75,9 +74,9 @@ extern Boolean NarrowSelection3D(Dino3DWidget w, int *face, int *direction);
 extern void DrawFrame3D(Dino3DWidget w, Boolean focus);
 extern void DrawTriangle3D(Dino3DWidget w, int face, int position, int offset);
 #ifdef WINVER
-extern void ResizeDino3D(Dino3DWidget w);
-extern void InitializeDino3D(Dino3DWidget w);
-extern void ExposeDino3D(Dino3DWidget w);
+extern void ResizePuzzle3D(Dino3DWidget w);
+extern void InitializePuzzle3D(Dino3DWidget w);
+extern void ExposePuzzle3D(Dino3DWidget w);
 #endif
 
 #endif /* _Dino3dP_h */
