@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1995 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1995 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -33,16 +33,16 @@
 #include "Dino.h"
 
 #ifdef WINVER
-extern void DestroyDino(HBRUSH brush);
-extern void ResizeDino(DinoWidget w);
-extern void SizeDino(DinoWidget w);
-extern void InitializeDino(DinoWidget w, HBRUSH brush);
-extern void ExposeDino(DinoWidget w);
-extern void PeriodModeDino(DinoWidget w, const int mode);
-extern void DimDino(DinoWidget w);
-extern int  MoveDinoDir(DinoWidget w, const int direction);
-extern void SetDino(DinoWidget w, int reason);
-extern void SetDinoMove(DinoWidget w, int reason, int face, int position,
+extern void DestroyPuzzle(HBRUSH brush);
+extern void ResizePuzzle(DinoWidget w);
+extern void SizePuzzle(DinoWidget w);
+extern void InitializePuzzle(DinoWidget w, HBRUSH brush);
+extern void ExposePuzzle(DinoWidget w);
+extern void PeriodModePuzzle(DinoWidget w, const int mode);
+extern void DimPuzzle(DinoWidget w);
+extern int  MovePuzzleDir(DinoWidget w, const int direction);
+extern void setPuzzle(DinoWidget w, int reason);
+extern void setPuzzleMove(DinoWidget w, int reason, int face, int position,
 	int direction, int style);
 
 #else
@@ -59,7 +59,7 @@ typedef struct _DinoClassRec {
 
 extern DinoClassRec dinoClassRec;
 
-extern void QuitDino(DinoWidget w, XEvent * event, char **args, int nArgs);
+extern void QuitPuzzle(DinoWidget w, XEvent *event, char **args, int nArgs);
 #endif
 
 #define SYMBOL ':'
@@ -84,14 +84,31 @@ extern void QuitDino(DinoWidget w, XEvent * event, char **args, int nArgs);
 #define EDGE 2
 #define FACE 3
 
-typedef struct _DinoLoc {
-	int         face, side, dir;
-} DinoLoc;
-
 typedef struct _DinoCornerLoc {
 	int         face, rotation;
 } DinoCornerLoc;
 
+typedef struct _MoveRecord {
+#if 0
+	int face, position, direction, style, control;
+#endif
+	unsigned long int packed;  /* This makes assumptions on the data. */
+} MoveRecord;
+
+typedef struct _MoveStack {
+	MoveRecord move;
+	struct _MoveStack *previous, *next;
+} MoveStack;
+
+typedef struct _DinoStack {
+	MoveStack *currMove, *lastMove, *firstMove;
+	int count;
+} DinoStack;
+
+typedef struct _DinoLoc {
+	int face, side, dir;
+} DinoLoc;
+
 typedef struct _DinoPart {
 	DinoCornerLoc cubeLoc[MAXFACES][MAXORIENT];
 	DinoCornerLoc faceLoc[MAXORIENT];
@@ -102,6 +119,8 @@ typedef struct _DinoPart {
 	Boolean     practice, orient, vertical, mono, reverse, focus;
 	Boolean     scoreOnly, versionOnly;
 	int         dim; /* This allows us to reuse code between 2d and 3d */
+	int         degreeTurn;
+	int         delay;
 	int         mode, style;
 	Position    delta;
 	Position    orientLineLength;
@@ -113,7 +132,7 @@ typedef struct _DinoPart {
 	char        userName[81], scoreFile[81];
 	char        faceChar[MAXFACES];
 #else
-	int         menu;
+	int         menu, view;
 	char       *userName, *scoreFile, *font;
 	XFontStruct *fontInfo;
 	char       *faceName[MAXFACES];
@@ -133,108 +152,116 @@ typedef struct _RowNext {
 	int         face, direction, side_face;
 } RowNext;
 
-extern void HideDino(
+extern void HidePuzzle(
 #ifdef WINVER
 const DinoWidget w
 #else
-DinoWidget w, XEvent * event, char **args, int nArgs
+DinoWidget w, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void SelectDino(DinoWidget w
+extern void SelectPuzzle(DinoWidget w
 #ifdef WINVER
 , const int x, const int y, const int control
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void ReleaseDino(DinoWidget w
+extern void ReleasePuzzle(DinoWidget w
 #ifdef WINVER
 , const int x, const int y, const int shift, const int control, const int alt
 #else
-, XEvent * event, char **args, int nArgs
-#endif
-);
-extern void PracticeDino(DinoWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
 #ifndef WINVER
-extern void PracticeDinoMaybe(DinoWidget w
-, XEvent * event, char **args, int nArgs
+extern void PracticePuzzleMaybe(DinoWidget w
+, XEvent *event, char **args, int nArgs
 );
-extern void PracticeDino2(DinoWidget w
-, XEvent * event, char **args, int nArgs
+extern void PracticePuzzle2(DinoWidget w
+, XEvent *event, char **args, int nArgs
+);
+extern void RandomizePuzzleMaybe(DinoWidget w
+, XEvent *event, char **args, int nArgs
+);
+extern void RandomizePuzzle2(DinoWidget w
+, XEvent *event, char **args, int nArgs
 );
 #endif
-extern void RandomizeDino(DinoWidget w
+extern void GetPuzzle(DinoWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void WritePuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void UndoPuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void RedoPuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void ClearPuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void RandomizePuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void SolvePuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void PracticePuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void OrientizePuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void ViewPuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void EnterPuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void LeavePuzzle(DinoWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 );
 #ifndef WINVER
-extern void RandomizeDinoMaybe(DinoWidget w
-, XEvent * event, char **args, int nArgs
+extern void Period2ModePuzzle(DinoWidget w
+, XEvent *event, char **args, int nArgs
 );
-extern void RandomizeDino2(DinoWidget w
-, XEvent * event, char **args, int nArgs
+extern void Period3ModePuzzle(DinoWidget w
+, XEvent *event, char **args, int nArgs
 );
-#endif
-extern void GetDino(DinoWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-);
-extern void WriteDino(DinoWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-);
-extern void ClearDino(DinoWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-);
-extern void UndoDino(DinoWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-);
-extern void SolveDino(DinoWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-);
-extern void OrientizeDino(DinoWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-);
-extern void EnterDino(DinoWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-);
-extern void LeaveDino(DinoWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-);
-#ifndef WINVER
-extern void Period2ModeDino(DinoWidget w
-, XEvent * event, char **args, int nArgs
-);
-extern void Period3ModeDino(DinoWidget w
-, XEvent * event, char **args, int nArgs
-);
-extern void BothModeDino(DinoWidget w
-, XEvent * event, char **args, int nArgs
+extern void BothModePuzzle(DinoWidget w
+, XEvent *event, char **args, int nArgs
 );
 #endif
 
-extern void MoveDinoInput(DinoWidget w, int x, int y, int direction, int shift,
+extern void MovePuzzleInput(DinoWidget w, int x, int y, int direction, int shift,
 	int control, int alt);
-extern void MoveDino(DinoWidget w, int face, int position, int direction,
+extern void MovePuzzle(DinoWidget w, int face, int position, int direction,
 	int style, int control);
 extern void SetAllColors(DinoWidget w);
 #if 0
@@ -244,19 +271,20 @@ extern void SolvePieces(DinoWidget w);
 #endif
 extern void DrawAllPieces(DinoWidget w);
 extern Boolean CheckSolved(DinoWidget w);
-extern void InitMoves(void);
-extern void PutMove(int face, int position, int direction, int style,
-	int control);
-extern void GetMove(int *face, int *position, int *direction, int *style,
-	int *control);
-extern int  MadeMoves(void);
-extern void FlushMoves(DinoWidget w);
-extern int  NumMoves(void);
-extern void ScanMoves(FILE * fp, DinoWidget w, int moves);
-extern void PrintMoves(FILE * fp);
-extern void ScanStartPosition(FILE * fp, DinoWidget w);
-extern void PrintStartPosition(FILE * fp, DinoWidget w);
-extern void SetStartPosition(DinoWidget w);
+extern void newMoves(DinoStack *s);
+extern void deleteMoves(DinoStack *s);
+extern void setMove(DinoStack *s,
+	int face, int position, int direction, int style, int control);
+extern void getMove(DinoStack *s,
+	int *face, int *position, int *direction, int *style, int *control);
+extern int madeMoves(DinoStack *s);
+extern void flushMoves(DinoWidget w, DinoStack *s, Boolean undo);
+extern int numMoves(DinoStack *s);
+extern void scanMoves(FILE *fp, DinoWidget w, int moves);
+extern void printMoves(FILE *fp, DinoStack *s);
+extern void scanStartPosition(FILE *fp, DinoWidget w);
+extern void printStartPosition(FILE *fp, DinoWidget w);
+extern void setStartPosition(DinoWidget w);
 
 #ifdef DEBUG
 extern void PrintCube(DinoWidget w);
