@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1996 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1996 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -29,20 +29,21 @@
 #define IDM_WRITE	101
 #define IDM_EXIT	102
 #define IDM_HIDE	103
-#define IDM_CLEAR	200
-#define IDM_UNDO	201
-#define IDM_RANDOMIZE	202
-#define IDM_PRACTICE	203
-#define IDM_SOLVE	204
-#define IDM_ORIENT	205
-#define IDM_DIM 206
-#define IDM_TR	300
-#define IDM_BR	301
-#define IDM_BL	302
-#define IDM_TL	303
-#define IDM_CW	305
-#define IDM_CCW	307
-#define IDM_TOP	308
+#define IDM_UNDO	200
+#define IDM_REDO	201
+#define IDM_CLEAR	202
+#define IDM_RANDOMIZE	203
+#define IDM_PRACTICE	204
+#define IDM_SOLVE	205
+#define IDM_ORIENTIZE	206
+#define IDM_DIM 	207
+#define IDM_TR		300
+#define IDM_BR		301
+#define IDM_BL		302
+#define IDM_TL		303
+#define IDM_CW		305
+#define IDM_CCW		307
+#define IDM_TOP		308
 #define IDM_RIGHT	309
 #define IDM_BOTTOM	310
 #define IDM_LEFT	311
