@@ -72,6 +72,9 @@ static DinoRec widget;
 #include "Dino.h"
 #include "Dino2d.h"
 #include "Dino3d.h"
+#ifdef HAVE_OPENGL
+#include "DinoGL.h"
+#endif
 #ifdef HAVE_XPM
 #include <X11/xpm.h>
 #include "icons/dino.xpm"
@@ -88,7 +91,7 @@ static DinoRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Dino Version 7.2.1\n"
+"Dino Version 7.2.4\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -100,9 +103,10 @@ static const char optionsHelp[] = {
 "[-display [{host}]:[{vs}]] [-[no]mono] [-[no]{reverse|rv}]\n"
 "[-{foreground|fg} {color}] [-{background|bg} {color}]\n"
 "[-face{0|1|2|3|4|5} {color}] [-{border|bd} {color}]\n"
-"[-{font|fn} {fontname}] [-{mode {int} | both}]\n"
-"[-[no]orient] [-[no]practice] [-userName {string}]\n"
-"[-scoreFile {filename}] [-scores] [-version]\n"
+"[-{font|fn} {fontname}] [-view {int}]\n"
+"[-{mode {int} | both}] [-[no]orient] [-[no]practice]\n"
+"[-userName {string}] [-scoreFile {filename}] [-scores]\n"
+"[-version]\n"
 };
 #endif
 
@@ -131,10 +135,23 @@ static const char featuresHelp[] = {
 "record should say \"practice\").  This is good for learning "
 "moves and experimenting.\n"
 "\n"
-"Click \"mouse-right\" button, or press \"R\" or \"r\" "
+"Click \"mouse-right\" button, or press \"Z\" or \"z\" "
 "keys, to randomize the puzzle (this must be done first\n"
 "to set a new record).\n"
 "\n"
+"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
+"\n"
+"Press \"W\" or \"w\" keys to save (write) a puzzle.\n"
+"\n"
+"Press \"U\" or \"u\" keys to undo a move.\n"
+"\n"
+"Press \"R\" or \"r\" keys to redo a move.\n"
+"\n"
+"Press \"C\" or \"c\" keys to clear the puzzle.\n"
+"\n"
+"\"S\" or \"s\" keys reserved for the auto-solver "
+"(not implemented).\n"
+"\n"
 "Press \"O\" or \"o\" keys to toggle the orient mode.  One "
 "has to orient the faces in orient mode, besides\n"
 "getting all the faces to be the same color.  To do this "
@@ -146,16 +163,7 @@ static const char featuresHelp[] = {
 "Press \"2\", \"3\", \"B\", or \"b\" keys (not the keypad "
 "2, 3) to change modes to Period 2, Period 3, or Both.\n"
 "\n"
-"\"S\" or \"s\" keys reserved for the auto-solver "
-"(not implemented).\n"
-"\n"
-"Press \"U\" or \"u\" keys to undo a move.\n"
-"\n"
-"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
-"\n"
-"Press \"W\" or \"w\" keys to save a puzzle.\n"
-"\n"
-"Press \"C\" or \"c\" keys to clear a puzzle.\n"
+"Press \"V\" or \"v\" keys to change the view of the cube.\n"
 "\n"
 "Press \"Esc\" key to hide program.\n"
 "\n"
@@ -226,6 +234,7 @@ static char titleDsp[TITLELEN] = "";
 #endif
 static char scoreFileName[FILENAMELEN] = SCOREFILE;
 static char fileName[FILENAMELEN];
+static Boolean randomized = False;
 #ifdef WINVER
 #define MAXPROGNAME 80
 static char progDsp[MAXPROGNAME] = TITLE;
@@ -245,9 +254,15 @@ static Widget solveDialog, practiceDialog, randomizeDialog;
 static Arg arg[3];
 #else
 static Widget shell;
+#ifdef HAVE_OPENGL
+static Widget shellGL;
 #endif
-static Pixmap dinoIcon = None;
+#endif
+static Pixmap icon = None;
 static Widget topLevel, dino2d, dino3d;
+#ifdef HAVE_OPENGL
+static Widget dinoGL;
+#endif
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
 
@@ -466,7 +481,7 @@ MakeEquivalent(String *userName, String *scoreFile, int *mode,
 		exit(0);
 	}
 	XtVaSetValues(dino2d,
-		XtNdirection, DINO_IGNORE,
+		XtNdirection, PUZZLE_IGNORE,
 		XtNstart, False, NULL);
 	XtVaSetValues(dino3d,
 		XtNuserName, *userName,
@@ -475,7 +490,7 @@ MakeEquivalent(String *userName, String *scoreFile, int *mode,
 		XtNpractice, *practice,
 		XtNmono, mono,
 		XtNreverse, reverse,
-		XtNdirection, DINO_IGNORE,
+		XtNdirection, PUZZLE_IGNORE,
 		XtNstart, False,
 		XtNforeground, foreground,
 		XtNbackground, background,
@@ -486,6 +501,25 @@ MakeEquivalent(String *userName, String *scoreFile, int *mode,
 		XtNfaceColor3, faceColor[3],
 		XtNfaceColor4, faceColor[4],
 		XtNfaceColor5, faceColor[5], NULL);
+#ifdef HAVE_OPENGL
+	XtVaSetValues(dinoGL,
+		XtNuserName, *userName,
+		XtNorient, *orient,
+		XtNpractice, *practice,
+		XtNmono, mono,
+		XtNreverse, reverse,
+		XtNdirection, PUZZLE_IGNORE,
+		XtNstart, False,
+		XtNforeground, foreground,
+		XtNbackground, background,
+		XtNpieceBorder, pieceBorder,
+		XtNfaceColor0, faceColor[0],
+		XtNfaceColor1, faceColor[1],
+		XtNfaceColor2, faceColor[2],
+		XtNfaceColor3, faceColor[3],
+		XtNfaceColor4, faceColor[4],
+		XtNfaceColor5, faceColor[5], NULL);
+#endif
 }
 #endif
 
@@ -507,11 +541,22 @@ PrintStatus(char *msg, int nMoves
 		(void) strcpy(mb, "both");
 	else
 		(void) sprintf(mb, "%d", mode);
-	(void) sprintf(titleDsp, "%s%dd.%s: (%d/%s) - %s",
-		progDsp, dim, mb, nMoves, recordDsp, msg);
+#ifdef HAVE_OPENGL
+	if (dim == 4)
+		(void) sprintf(titleDsp, "%sGL.%s: (%d/%s) - %s",
+			progDsp, mb, nMoves, recordDsp, msg);
+	else
+#endif
+		(void) sprintf(titleDsp, "%s%dd.%s: (%d/%s) - %s",
+			progDsp, dim, mb, nMoves, recordDsp, msg);
 #ifdef WINVER
 	SetWindowText(widget.core.hWnd, (LPSTR) titleDsp);
 #else
+#ifdef HAVE_OPENGL
+	if (dim == 4)
+		XtVaSetValues(XtParent(dinoGL), XtNtitle, titleDsp, NULL);
+	else
+#endif
 	XtVaSetValues(XtParent((dim == 2) ? dino2d : dino3d),
 		XtNtitle, titleDsp, NULL);
 #endif
@@ -558,7 +603,7 @@ void
 #ifdef WINVER
 	int dim;
 
-	InitializeDino(w, brush);
+	InitializePuzzle(w, brush);
 
 	mode = w->dino.mode;
 	orient = w->dino.orient;
@@ -588,7 +633,7 @@ void
 #ifdef WINVER
 		(void) strncpy(userNameDsp, userName, USERNAMELEN);
 #else
-		char * login = getlogin();
+		char *login = getlogin();
 
 		if (login == NULL) {
 			(void) strcpy(userNameDsp, "");
@@ -612,40 +657,74 @@ void
 #else
 	PrintStatus(messageDsp, movesDsp, 2, mode);
 	PrintStatus(messageDsp, movesDsp, 3, mode);
+#ifdef HAVE_OPENGL
+	PrintStatus(messageDsp, movesDsp, 4, mode);
+#endif
 #endif
 #endif
 }
 
 #ifdef WINVER
 void
-SetDino(DinoWidget w, int reason)
+setPuzzle(DinoWidget w, int reason)
 #else
 static void
-CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
+CallbackPuzzle(Widget w, caddr_t clientData, dinoCallbackStruct *callData)
 #endif
 {
 	int mode;
 	Boolean orient, practice, start, cheat;
 #ifdef WINVER
-	int dim;
+	int dim = 0;
 #else
 	int reason = callData->reason;
-	Widget otherw;
+	Widget otherw1 = (Widget) NULL;
+#ifdef HAVE_OPENGL
+	Widget otherw2 = (Widget) NULL;
+#endif
 #ifndef HAVE_MOTIF
-	int dim, otherdim;
+	int dim = 0, otherdim1 = 0;
+#ifdef HAVE_OPENGL
+	int otherdim2 = 0;
+#endif
 #endif
 
 	if (w == dino2d) {
-		otherw = dino3d;
-#ifndef HAVE_MOTIF
-		dim = 2, otherdim = 3;
+		otherw1 = dino3d;
+#ifdef HAVE_OPENGL
+		otherw2 = dinoGL;
 #endif
-	} else {	/* (w == dino3d) */
-		otherw = dino2d;
 #ifndef HAVE_MOTIF
-		dim = 3, otherdim = 2;
+		dim = 2;
+		otherdim1 = 3;
+#ifdef HAVE_OPENGL
+		otherdim2 = 4;
+#endif
+#endif
+	} else if (w == dino3d) {
+		otherw1 = dino2d;
+#ifdef HAVE_OPENGL
+		otherw2 = dinoGL;
+#endif
+#ifndef HAVE_MOTIF
+		dim = 3;
+		otherdim1 = 2;
+#ifdef HAVE_OPENGL
+		otherdim2 = 4;
+#endif
 #endif
 	}
+#ifdef HAVE_OPENGL
+	else if (w == dinoGL) {
+		otherw1 = dino2d;
+		otherw2 = dino3d;
+#ifndef HAVE_MOTIF
+		dim = 4;
+		otherdim1 = 2;
+		otherdim2 = 3;
+#endif
+	}
+#endif
 #endif
 	(void) strcpy(messageDsp, "");
 #ifdef WINVER
@@ -664,7 +743,7 @@ CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-		case DINO_HIDE:
+		case PUZZLE_HIDE:
 #ifdef WINVER
 			ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -675,28 +754,31 @@ CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
 			(void) XIconifyWindow(XtDisplay(shell),
 				XtWindow(shell),
 				XScreenNumberOfScreen(XtScreen(shell)));
+#ifdef HAVE_OPENGL
+			(void) XIconifyWindow(XtDisplay(shellGL),
+				XtWindow(shellGL),
+				XScreenNumberOfScreen(XtScreen(shellGL)));
+#endif
 #endif
 #endif
 			break;
 #ifndef WINVER
-		case DINO_PRACTICE_QUERY:
+		case PUZZLE_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(practiceDialog);
 #else
-			XtVaSetValues(dino2d,
-				XtNmenu, 8, NULL); /* menu choice */
+			XtVaSetValues(dino2d, XtNmenu, MENU_PRACTICE, NULL);
 #endif
 			break;
-		case DINO_RANDOMIZE_QUERY:
+		case PUZZLE_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(randomizeDialog);
 #else
-			XtVaSetValues(dino2d,
-				XtNmenu, 5, NULL); /* menu choice */
+			XtVaSetValues(dino2d, XtNmenu, MENU_RANDOMIZE, NULL);
 #endif
 			break;
 #endif
-		case DINO_SOLVE_MESSAGE:
+		case PUZZLE_SOLVE_MESSAGE:
 #ifdef WINVER
 			(void) MessageBox(w->core.hWnd, solveHelp,
 				"Auto-Solve", MB_OK);
@@ -708,7 +790,7 @@ CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case DINO_RESTORE:
+		case PUZZLE_RESTORE:
 			if (practice) {
 				(void) strncpy(recordDsp, "practice",
 					MESSAGELEN);
@@ -717,51 +799,83 @@ CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
 #endif
 			}
 			movesDsp = 0;
+			randomized = False;
 #ifndef WINVER
-			XtVaSetValues(otherw,
-				XtNdirection, DINO_RESTORE, NULL);
+#ifdef HAVE_OPENGL
+			XtVaSetValues(otherw2,
+				XtNdirection, PUZZLE_RESTORE, NULL);
+#endif
+			XtVaSetValues(otherw1,
+				XtNdirection, PUZZLE_RESTORE, NULL);
 			XtVaSetValues(w,
-				XtNdirection, DINO_RESTORE, NULL);
+				XtNdirection, PUZZLE_RESTORE, NULL);
 #endif
 			break;
-		case DINO_CLEAR:
+		case PUZZLE_CLEAR:
 			movesDsp = 0;
+			randomized = False;
 #ifndef WINVER
-			XtVaSetValues(otherw, XtNdirection, DINO_CLEAR, NULL);
-			XtVaSetValues(w, XtNdirection, DINO_CLEAR, NULL);
+#ifdef HAVE_OPENGL
+			XtVaSetValues(otherw2,
+				XtNdirection, PUZZLE_CLEAR, NULL);
+#endif
+			XtVaSetValues(otherw1,
+				XtNdirection, PUZZLE_CLEAR, NULL);
+			XtVaSetValues(w,
+				XtNdirection, PUZZLE_CLEAR, NULL);
 #endif
 			break;
-		case DINO_RESET:
+		case PUZZLE_RESET:
 			movesDsp = 0;
+			randomized = False;
 			break;
-		case DINO_ILLEGAL:
-			if (practice || start)
+		 case PUZZLE_AMBIGUOUS:
+			(void) strncpy(messageDsp, "Ambiguous move",
+				MESSAGELEN);
+			break;
+		case PUZZLE_ILLEGAL:
+			if (practice || randomized)
 				(void) strncpy(messageDsp, "Illegal move",
 					MESSAGELEN);
 			else
-				(void) strncpy(messageDsp, "Randomize to start",
-					MESSAGELEN);
+				(void) strncpy(messageDsp,
+					"Randomize to start", MESSAGELEN);
 			break;
 #ifndef WINVER
-		case DINO_MOVED:
+		case PUZZLE_MOVED:
 			movesDsp++;
-			XtVaSetValues(otherw,
+#ifdef HAVE_OPENGL
+			XtVaSetValues(otherw2,
 				XtNstart, True,
 				XtNface, callData->face,
 				XtNpos, callData->position,
 				XtNdirection, callData->direction,
 				XtNstyle, callData->style, NULL);
-			SET_STARTED(w, False);
+#endif
+			XtVaSetValues(otherw1,
+				XtNstart, True,
+				XtNface, callData->face,
+				XtNpos, callData->position,
+				XtNdirection, callData->direction,
+				XtNstyle, callData->style, NULL);
+			SET_STARTED(w, True);
 			break;
-		case DINO_CONTROL:
-			XtVaSetValues(otherw,
+		case PUZZLE_CONTROL:
+#ifdef HAVE_OPENGL
+			XtVaSetValues(otherw2,
 				XtNface, callData->face,
 				XtNpos, callData->position,
 				XtNdirection, callData->direction,
 				XtNstyle, callData->style, NULL);
-			return;
 #endif
-		case DINO_SOLVED:
+			XtVaSetValues(otherw1,
+				XtNface, callData->face,
+				XtNpos, callData->position,
+				XtNdirection, callData->direction,
+				XtNstyle, callData->style, NULL);
+			break;
+#endif
+		case PUZZLE_SOLVED:
 			if (practice)
 				movesDsp = 0;
 			else if (cheat)
@@ -775,11 +889,16 @@ CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
 					MESSAGELEN);
 			SET_STARTED(w, False);
 #ifndef WINVER
-			SET_STARTED(otherw, False);
+#ifdef HAVE_OPENGL
+			SET_STARTED(otherw2, False);
 #endif
+			SET_STARTED(otherw1, False);
+#endif
+			randomized = False;
 			break;
-		case DINO_PRACTICE:
+		case PUZZLE_PRACTICE:
 			movesDsp = 0;
+			randomized = False;
 			practice = !practice;
 			if (!practice)
 				(void) strncpy(messageDsp, "Randomize to start",
@@ -792,7 +911,12 @@ CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
 			XtVaSetValues(w,
 				XtNpractice, practice,
 				XtNstart, False, NULL);
-			XtVaSetValues(otherw,
+#ifdef HAVE_OPENGL
+			XtVaSetValues(otherw2,
+				XtNpractice, practice,
+				XtNstart, False, NULL);
+#endif
+			XtVaSetValues(otherw1,
 				XtNpractice, practice,
 				XtNstart, False, NULL);
 #ifdef HAVE_MOTIF
@@ -800,8 +924,9 @@ CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case DINO_RANDOMIZE:
+		case PUZZLE_RANDOMIZE:
 			movesDsp = 0;
+			randomized = True;
 #ifdef WINVER
 			w->dino.practice = False;
 			w->dino.started = False;
@@ -809,12 +934,30 @@ CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
 			XtVaSetValues(w,
 				XtNpractice, False,
 				XtNstart, False, NULL);
-			XtVaSetValues(otherw,
+#ifdef HAVE_OPENGL
+			XtVaSetValues(otherw2,
+				XtNpractice, False,
+				XtNstart, False, NULL);
+#endif
+			XtVaSetValues(otherw1,
 				XtNpractice, False,
 				XtNstart, False, NULL);
 #endif
 			break;
-		case DINO_ORIENT:
+#ifdef HAVE_OPENGL
+		case PUZZLE_VIEW:
+			{
+				int view;
+
+				XtVaGetValues(dinoGL,
+					XtNview, &view, NULL);
+				view = (view + 1) % (2 * MAXVIEWS);
+				XtVaSetValues(dinoGL,
+					XtNview, view, NULL);
+			}
+			break;
+#endif
+		case PUZZLE_ORIENTIZE:
 			movesDsp = 0;
 			orient = !orient;
 			PrintRecord(mode, orient, practice);
@@ -823,41 +966,60 @@ CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
 #else
 			XtVaSetValues(w,
 				XtNorient, orient, NULL);
-			XtVaSetValues(otherw,
+#ifdef HAVE_OPENGL
+			XtVaSetValues(otherw2,
+				XtNorient, orient, NULL);
+#endif
+			XtVaSetValues(otherw1,
 				XtNorient, orient, NULL);
 #ifdef HAVE_MOTIF
 			XmToggleButtonSetState(orientSwitch, orient, True);
 #endif
 #endif
 			break;
-		case DINO_PERIOD2:
-		case DINO_PERIOD3:
-		case DINO_BOTH:
+		case PUZZLE_PERIOD2:
+		case PUZZLE_PERIOD3:
+		case PUZZLE_BOTH:
 			movesDsp = 0;
-			mode = reason - DINO_PERIOD2 + PERIOD2;
+			mode = reason - PUZZLE_PERIOD2 + PERIOD2;
 			PrintRecord(mode, orient, practice);
 #ifdef WINVER
 			w->dino.mode = mode;
 #else
 			XtVaSetValues(w,
 				XtNmode, mode, NULL);
-			XtVaSetValues(otherw,
+#ifdef HAVE_OPENGL
+			XtVaSetValues(otherw2,
+				XtNmode, mode, NULL);
+#endif
+			XtVaSetValues(otherw1,
 				XtNmode, mode, NULL);
 #ifdef HAVE_MOTIF
 			XmToggleButtonSetState(modes[mode - PERIOD2], True, True);
 #endif
 #endif
 			break;
-		case DINO_COMPUTED:
+		case PUZZLE_COMPUTED:
 			SET_STARTED(w, False);
 #ifndef WINVER
-			SET_STARTED(otherw, False);
+#ifdef HAVE_OPENGL
+			SET_STARTED(otherw2, False);
+#endif
+			SET_STARTED(otherw1, False);
 #endif
 			break;
 #ifndef WINVER
-		case DINO_UNDO:
+		case PUZZLE_UNDO:
 			movesDsp--;
-			XtVaSetValues(otherw,
+#ifdef HAVE_OPENGL
+			XtVaSetValues(otherw2,
+				XtNstart, True,
+				XtNface, callData->face,
+				XtNpos, callData->position,
+				XtNdirection, callData->direction,
+				XtNstyle, callData->style, NULL);
+#endif
+			XtVaSetValues(otherw1,
 				XtNstart, True,
 				XtNface, callData->face,
 				XtNpos, callData->position,
@@ -866,10 +1028,34 @@ CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
 			SET_STARTED(w, False);
 			break;
 #endif
+#ifndef WINVER
+		case PUZZLE_REDO:
+			movesDsp++;
+#ifdef HAVE_OPENGL
+			XtVaSetValues(otherw2,
+				XtNstart, True,
+				XtNface, callData->face,
+				XtNpos, callData->position,
+				XtNdirection, callData->direction,
+				XtNstyle, callData->style, NULL);
+#endif
+			XtVaSetValues(otherw1,
+				XtNstart, True,
+				XtNface, callData->face,
+				XtNpos, callData->position,
+				XtNdirection, callData->direction,
+				XtNstyle, callData->style, NULL);
+			SET_STARTED(w, True);
+			break;
+#endif
 #ifdef WINVER
-		case DINO_DIM:
+		case PUZZLE_DIM:
 			dim++;
+#ifdef HAVE_OPENGL
+			if (dim > 4)
+#else
 			if (dim > 3)
+#endif
 				dim = 2;
 			w->dino.dim = dim;
 			break;
@@ -882,19 +1068,22 @@ CallbackDino(Widget w, caddr_t clientData, dinoCallbackStruct * callData)
 	PrintStatus(messageDsp, movesDsp);
 #else
 	PrintStatus(messageDsp, movesDsp, dim, mode);
-	PrintStatus(messageDsp, movesDsp, otherdim, mode);
+	PrintStatus(messageDsp, movesDsp, otherdim1, mode);
+#ifdef HAVE_OPENGL
+	PrintStatus(messageDsp, movesDsp, otherdim2, mode);
+#endif
 #endif
 #endif
 }
 
 #ifdef WINVER
 void
-SetDinoMove(DinoWidget w, int reason, int face, int position, int direction,
+setPuzzleMove(DinoWidget w, int reason, int face, int position, int direction,
 		int style)
 {
 	int mode;
 	/* Boolean orient, practice, cheat; */
-	int dim;
+	int dim = 0;
 
 	(void) strcpy(messageDsp, "");
 	mode = w->dino.mode;
@@ -905,16 +1094,20 @@ SetDinoMove(DinoWidget w, int reason, int face, int position, int direction,
 #endif
 	dim = w->dino.dim;
 	switch (reason) {
-		case DINO_MOVED:
+		case PUZZLE_MOVED:
 			movesDsp++;
 			SET_STARTED(w, True);
 			break;
-		case DINO_CONTROL:
+		case PUZZLE_CONTROL:
 			break;
-		case DINO_UNDO:
+		case PUZZLE_UNDO:
 			movesDsp--;
 			SET_STARTED(w, True);
 			break;
+		case PUZZLE_REDO:
+			movesDsp++;
+			SET_STARTED(w, True);
+			break;
 	}
 	PrintStatus(messageDsp, movesDsp, dim, mode);
 }
@@ -944,15 +1137,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	if (GetFocus()) {
 		if (!widget.dino.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			EnterDino(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			EnterPuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	} else {
 		if (widget.dino.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			LeaveDino(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			LeavePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	}
@@ -961,35 +1156,39 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			Initialize(&widget, brush);
 			break;
 		case WM_DESTROY:
-			DestroyDino(brush);
+			DestroyPuzzle(brush);
 			break;
 		case WM_SIZE:
-			ResizeDino(&widget);
+			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
 		case WM_PAINT:
 			widget.core.hDC = BeginPaint(hWnd, &paint);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ExposeDino(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ExposePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 			break;
 		case WM_RBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			RandomizeDino(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			SelectDino(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			SelectPuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONUP:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ReleaseDino(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ReleasePuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0,
 				(GetKeyState(VK_TAB) >> 1) ? 1 : 0);
@@ -998,7 +1197,8 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
 		case WM_MOUSEWHEEL:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
 			{
 				int zDelta = ((short) HIWORD(wParam));
 				POINT cursor, origin;
@@ -1007,14 +1207,14 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				ClientToScreen(hWnd, &origin);
 				(void) GetCursorPos(&cursor);
 				if (zDelta > (WHEEL_DELTA >> 1)) {
-					MoveDinoInput(&widget,
+					MovePuzzleInput(&widget,
 						cursor.x - origin.x, cursor.y - origin.y,
 						TOP,
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0,
 				(GetKeyState(VK_TAB) >> 1) ? 1 : 0);
 				} else if (zDelta < -(WHEEL_DELTA >> 1)) {
-					MoveDinoInput(&widget,
+					MovePuzzleInput(&widget,
 						cursor.x - origin.x, cursor.y - origin.y,
 						BOTTOM,
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
@@ -1028,55 +1228,65 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		case WM_COMMAND:
 			switch (LOWORD(wParam)) {
 				case IDM_GET:
-					GetDino(&widget);
-					ResizeDino(&widget);
+					GetPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_WRITE:
-					WriteDino(&widget);
+					WritePuzzle(&widget);
 					break;
 				case IDM_EXIT:
-					DestroyDino(brush);
+					DestroyPuzzle(brush);
 					break;
 				case IDM_HIDE:
-					HideDino(&widget);
-					break;
-				case IDM_CLEAR:
-					ClearDino(&widget);
-					SizeDino(&widget);
-					(void) InvalidateRect(hWnd, NULL, TRUE);
+					HidePuzzle(&widget);
 					break;
 				case IDM_UNDO:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					UndoDino(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					UndoPuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
+				case IDM_REDO:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RedoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_CLEAR:
+					ClearPuzzle(&widget);
+					SizePuzzle(&widget);
+					(void) InvalidateRect(hWnd, NULL, TRUE);
+					break;
 				case IDM_RANDOMIZE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					RandomizeDino(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RandomizePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_PRACTICE:
-					PracticeDino(&widget);
-					SizeDino(&widget);
+					PracticePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SOLVE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					SolveDino(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					SolvePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
-				case IDM_ORIENT:
-					OrientizeDino(&widget);
-					SizeDino(&widget);
+				case IDM_ORIENTIZE:
+					OrientizePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_DIM:
-					(void) DimDino(&widget);
-					ResizeDino(&widget);
+					(void) DimPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SHIFT_TR:
@@ -1104,12 +1314,13 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
 						shift = shift || (GetKeyState(VK_CAPITAL) & 1);
-						(void) MoveDinoInput(&widget,
+						(void) MovePuzzleInput(&widget,
 							cursor.x - origin.x, cursor.y - origin.y,
 							(int) LOWORD(wParam) - ((shift) ? IDM_SHIFT_TR : IDM_TR),
 							shift, FALSE, FALSE);
@@ -1141,11 +1352,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
-						(void) MoveDinoInput(&widget,
+						(void) MovePuzzleInput(&widget,
 							cursor.x - origin.x, cursor.y - origin.y,
 							(int) LOWORD(wParam) - ((shift) ? IDM_SHIFTCONTROL_TR : IDM_CONTROL_TR),
 							shift, TRUE, FALSE);
@@ -1177,11 +1389,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
-						(void) MoveDinoInput(&widget,
+						(void) MovePuzzleInput(&widget,
 							cursor.x - origin.x, cursor.y - origin.y,
 							(int) LOWORD(wParam) - ((shift) ? IDM_CONTROLALT_TR : IDM_ALT_TR),
 							FALSE, shift, TRUE);
@@ -1191,8 +1404,8 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				case IDM_PERIOD2:
 				case IDM_PERIOD3:
 				case IDM_BOTH:
-					PeriodModeDino(&widget, (int) LOWORD(wParam) - IDM_PERIOD2);
-					SizeDino(&widget);
+					PeriodModePuzzle(&widget, (int) LOWORD(wParam) - IDM_PERIOD2);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_ABOUT:
@@ -1269,7 +1482,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
 
 #else
 static void
-Usage(char * programName)
+Usage(char *programName)
 {
 	unsigned int i;
 
@@ -1302,14 +1515,15 @@ static XrmOptionDescRec options[] =
 	{(char *) "-face3", (char *) "*dino.faceColor3", XrmoptionSepArg, NULL},
 	{(char *) "-face4", (char *) "*dino.faceColor4", XrmoptionSepArg, NULL},
 	{(char *) "-face5", (char *) "*dino.faceColor5", XrmoptionSepArg, NULL},
+	{(char *) "-fn", (char *) "*dino.font", XrmoptionSepArg, NULL},
+	{(char *) "-font", (char *) "*dino.font", XrmoptionSepArg, NULL},
+	{(char *) "-view", (char *) "*dino.view", XrmoptionSepArg, NULL},
 	{(char *) "-mode", (char *) "*dino.mode", XrmoptionSepArg, NULL},
 	{(char *) "-both", (char *) "*dino.mode", XrmoptionNoArg, (char *) "4"},
 	{(char *) "-orient", (char *) "*dino.orient", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-noorient", (char *) "*dino.orient", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-practice", (char *) "*dino.practice", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-nopractice", (char *) "*dino.practice", XrmoptionNoArg, (char *) "FALSE"},
-	{(char *) "-fn", (char *) "*dino.font", XrmoptionSepArg, NULL},
-	{(char *) "-font", (char *) "*dino.font", XrmoptionSepArg, NULL},
 	{(char *) "-userName", (char *) "*dino.userName", XrmoptionSepArg, NULL},
 	{(char *) "-scoreFile", (char *) "*dino.scoreFile", XrmoptionSepArg, NULL},
 	{(char *) "-scores", (char *) "*dino.scoreOnly", XrmoptionNoArg, (char *) "TRUE"},
@@ -1318,22 +1532,22 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackDinoPractice(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzlePractice(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(dino2d, XtNmenu, 8, NULL); /* menu choice */
+		XtVaSetValues(dino2d, XtNmenu, MENU_PRACTICE, NULL);
 	}
 }
 
 static void
-CallbackDinoRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzleRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(dino2d, XtNmenu, 5, NULL); /* menu choice */
+		XtVaSetValues(dino2d, XtNmenu, MENU_RANDOMIZE, NULL);
 	}
 }
 static void
-ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct * cbs)
+ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct *cbs)
 {
 	Boolean orient, practice;
 
@@ -1345,6 +1559,10 @@ ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct * cbs)
 			XtNmode, mode + PERIOD2, NULL);
 		XtVaSetValues(dino3d,
 			XtNmode, mode + PERIOD2, NULL);
+#ifdef HAVE_OPENGL
+		XtVaSetValues(dinoGL,
+			XtNmode, mode + PERIOD2, NULL);
+#endif
 		movesDsp = 0;
 		(void) sprintf(buff, "%d", movesDsp);
 		PrintState(moves, buff);
@@ -1355,7 +1573,7 @@ ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct * cbs)
 }
 
 static void
-OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	int mode;
 	Boolean orient = cbs->set, practice;
@@ -1367,6 +1585,10 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
 		XtNorient, orient, NULL);
 	XtVaSetValues(dino3d,
 		XtNorient, orient, NULL);
+#ifdef HAVE_OPENGL
+	XtVaSetValues(dinoGL,
+		XtNorient, orient, NULL);
+#endif
 	movesDsp = 0;
 	(void) sprintf(buff, "%d", movesDsp);
 	PrintState(moves, buff);
@@ -1376,7 +1598,7 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
 }
 
 static void
-PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	int mode;
 	Boolean orient, practice = cbs->set;
@@ -1387,6 +1609,11 @@ PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cb
 	XtVaSetValues(dino3d,
 		XtNpractice, practice,
 		XtNstart, False, NULL);
+#ifdef HAVE_OPENGL
+	XtVaSetValues(dinoGL,
+		XtNpractice, practice,
+		XtNstart, False, NULL);
+#endif
 	XtVaGetValues(dino2d,
 		XtNmode, &mode,
 		XtNpractice, &orient, NULL);
@@ -1406,9 +1633,9 @@ fileCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	if (val == 2)
+	if (val == MENU_EXIT)
 		exit(0);
-	XtVaSetValues(dino2d, XtNmenu, val, NULL);
+	XtVaSetValues(dino2d, XtNmenu, val, NULL); /* GWQ */
 }
 
 static void
@@ -1416,7 +1643,7 @@ playCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	XtVaSetValues(dino2d, XtNmenu, val + 3, NULL); /* GWQ */
+	XtVaSetValues(dino2d, XtNmenu, val + MENU_EXIT + 1, NULL);
 }
 
 static Widget
@@ -1515,8 +1742,9 @@ main(int argc, char **argv)
 	Widget messageRowCol;
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
-	XmString clearString, undoString, randomizeString, solveString;
-	XmString orientString, practiceString;
+	XmString undoString, redoString;
+	XmString clearString, randomizeString, practiceString, solveString;
+	XmString orientString, viewString;
 	unsigned int i;
 #endif
 
@@ -1526,7 +1754,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#if HAVE_XPM
+#ifdef HAVE_XPM
 	{
 		XpmAttributes xpmAttributes;
 		XpmColorSymbol transparentColor[1] = {{NULL,
@@ -1541,17 +1769,17 @@ main(int argc, char **argv)
 		transparentColor[0].pixel = bg;
 		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) dino_xpm, &dinoIcon, NULL,
+			(char **) dino_xpm, &icon, NULL,
 			&xpmAttributes);
 	}
-	if (dinoIcon == (Pixmap) NULL)
+	if (icon == (Pixmap) NULL)
 #endif
-		dinoIcon = XCreateBitmapFromData(XtDisplay(topLevel),
+		icon = XCreateBitmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
 			(char *) dino_bits, dino_width, dino_height);
 #ifdef HAVE_MOTIF
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, dinoIcon,
+		XtNiconPixmap, icon,
 		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
@@ -1578,26 +1806,32 @@ main(int argc, char **argv)
 	XmStringFree(getString);
 	XmStringFree(writeString);
 	XmStringFree(quitString);
-	clearString = XmStringCreateSimple((char *) "Clear");
 	undoString = XmStringCreateSimple((char *) "Undo");
-	randomizeString = XmStringCreateSimple((char *) "Randomize");
+	redoString = XmStringCreateSimple((char *) "Redo");
+	clearString = XmStringCreateSimple((char *) "Clear");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
+	practiceString = XmStringCreateSimple((char *) "Practice");
 	solveString = XmStringCreateSimple((char *) "(Solve)");
 	orientString = XmStringCreateSimple((char *) "Orientize");
-	practiceString = XmStringCreateSimple((char *) "Practice");
+	viewString = XmStringCreateSimple((char *) "View");
 	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
-		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
+		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
 		XmVaPUSHBUTTON, orientString, 'O', NULL, NULL,
-		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
+		XmVaPUSHBUTTON, viewString, 'V', NULL, NULL,
 		NULL);
-	XmStringFree(clearString);
 	XmStringFree(undoString);
+	XmStringFree(redoString);
+	XmStringFree(clearString);
 	XmStringFree(randomizeString);
+	XmStringFree(practiceString);
 	XmStringFree(solveString);
 	XmStringFree(orientString);
-	XmStringFree(practiceString);
+	XmStringFree(viewString);
 	pullDownMenu = XmCreatePulldownMenu(menuBar,
 		(char *) "helpPullDown", NULL, 0);
 	widget = XtVaCreateManagedWidget("Help",
@@ -1641,11 +1875,11 @@ main(int argc, char **argv)
 	practiceDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to toggle the practice mode?",
 		(char *) "Practice Query",
-		(XtCallbackProc) CallbackDinoPractice);
+		(XtCallbackProc) CallbackPuzzlePractice);
 	randomizeDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to randomize?",
 		(char *) "Randomize Query",
-		(XtCallbackProc) CallbackDinoRandomize);
+		(XtCallbackProc) CallbackPuzzleRandomize);
 	mainPanel = XtCreateManagedWidget("mainPanel",
 		xmPanedWindowWidgetClass, menuBarPanel,
 		NULL, 0);
@@ -1708,6 +1942,7 @@ main(int argc, char **argv)
 		XtVaTypedArg, XmNlabelString, XmRString, "Record", 7, NULL);
 	record = XtVaCreateManagedWidget("0",
 		xmLabelWidgetClass, movesRowCol, NULL);
+
 	labelRowCol = XtVaCreateManagedWidget("labelRowCol",
 		xmRowColumnWidgetClass, controlPanel, NULL);
 	(void) XtVaCreateManagedWidget("PeriodText",
@@ -1739,7 +1974,6 @@ main(int argc, char **argv)
 		xmToggleButtonWidgetClass, switchRowCol, NULL);
 	XtAddCallback(practiceSwitch, XmNvalueChangedCallback,
 		(XtCallbackProc) PracticeToggle, (XtPointer) NULL);
-
 	messageRowCol = XtVaCreateManagedWidget("messageRowCol",
 		xmRowColumnWidgetClass, controlPanel,
 		XmNnumColumns, 1,
@@ -1753,33 +1987,57 @@ main(int argc, char **argv)
 	XtVaSetValues(dino2d,
 		XtNheight, 200, NULL);
 	XtAddCallback(dino2d, XtNselectCallback,
-		(XtCallbackProc) CallbackDino, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	dino3d = XtCreateManagedWidget("dino",
 		dino3dWidgetClass, mainPanel, NULL, 0);
 	XtVaSetValues(dino3d,
 		XtNheight, 200, NULL);
+#ifdef HAVE_OPENGL
+	dinoGL = XtCreateManagedWidget("dino",
+		dinoGLWidgetClass, mainPanel, NULL, 0);
+	XtVaSetValues(dinoGL,
+		XtNheight, 200, NULL);
+#endif
 #else
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, dinoIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 	shell = XtCreateApplicationShell(argv[0],
 		topLevelShellWidgetClass, NULL, 0);
 	XtVaSetValues(shell,
-		XtNiconPixmap, dinoIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
+#ifdef HAVE_OPENGL
+	shellGL = XtCreateApplicationShell(argv[0],
+		topLevelShellWidgetClass, NULL, 0);
+	XtVaSetValues(shellGL,
+		XtNiconPixmap, icon,
+		XtNinput, True, NULL);
+#endif
 	dino2d = XtCreateManagedWidget("dino",
 		dino2dWidgetClass, topLevel, NULL, 0);
 	XtAddCallback(dino2d, XtNselectCallback,
-		(XtCallbackProc) CallbackDino, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	dino3d = XtCreateManagedWidget("dino",
 		dino3dWidgetClass, shell, NULL, 0);
+#ifdef HAVE_OPENGL
+	dinoGL = XtCreateManagedWidget("dino",
+		dinoGLWidgetClass, shellGL, NULL, 0);
+#endif
 #endif
 	XtAddCallback(dino3d, XtNselectCallback,
-		(XtCallbackProc) CallbackDino, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
+#ifdef HAVE_OPENGL
+	XtAddCallback(dinoGL, XtNselectCallback,
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
+#endif
 	Initialize();
 	XtRealizeWidget(topLevel);
 #ifndef HAVE_MOTIF
 	XtRealizeWidget(shell);
+#ifdef HAVE_OPENGL
+	XtRealizeWidget(shellGL);
+#endif
 #endif
 	XGrabButton(XtDisplay(dino2d), (unsigned int) AnyButton, AnyModifier,
 		XtWindow(dino2d), TRUE, (unsigned int) (ButtonPressMask |
@@ -1791,6 +2049,13 @@ main(int argc, char **argv)
 		ButtonMotionMask | ButtonReleaseMask),
 		GrabModeAsync, GrabModeAsync, XtWindow(dino3d),
 		XCreateFontCursor(XtDisplay(dino3d), XC_crosshair));
+#ifdef HAVE_OPENGL
+	XGrabButton(XtDisplay(dinoGL), (unsigned int) AnyButton, AnyModifier,
+		XtWindow(dinoGL), TRUE, (unsigned int) (ButtonPressMask |
+		ButtonMotionMask | ButtonReleaseMask),
+		GrabModeAsync, GrabModeAsync, XtWindow(dinoGL),
+		XCreateFontCursor(XtDisplay(dinoGL), XC_crosshair));
+#endif
 	XtMainLoop();
 
 #ifdef VMS
