@@ -66,41 +66,69 @@
 #endif
 #endif
 
-static Boolean SetValuesHexagons(Widget current, Widget request, Widget renew);
-static void QuitHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void DestroyHexagons(Widget old);
-static void ResizeHexagons(HexagonsWidget w);
-static void SizeHexagons(HexagonsWidget w);
-static void InitializeHexagons(Widget request, Widget renew);
-static void ExposeHexagons(Widget renew, XEvent * event, Region region);
-static void HideHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void SelectHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void ReleaseHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void RandomizeHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void RandomizeHexagonsMaybe(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void RandomizeHexagons2(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void GetHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void WriteHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void ClearHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void UndoHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void SolveHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void ModeHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void SpeedHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void SlowHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void SoundHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void EnterHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void LeaveHexagons(HexagonsWidget w, XEvent * event, char **args, int nArgs);
+static Boolean SetValuesPuzzle(Widget current, Widget request, Widget renew);
+static void QuitPuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void DestroyPuzzle(Widget old);
+static void ResizePuzzle(HexagonsWidget w);
+static void SizePuzzle(HexagonsWidget w);
+static void InitializePuzzle(Widget request, Widget renew);
+static void ExposePuzzle(Widget renew, XEvent *event, Region region);
+static void HidePuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SelectPuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ReleasePuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ClearPuzzleMaybe(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ClearPuzzle2(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void GetPuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void WritePuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void UndoPuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void RedoPuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ClearPuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void RandomizePuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SolvePuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ModePuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SpeedPuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SlowPuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SoundPuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void EnterPuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void LeavePuzzle(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
 
-static void MoveHexagonsTl(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void MoveHexagonsTop(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void MoveHexagonsTr(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void MoveHexagonsLeft(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void MoveHexagonsRight(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void MoveHexagonsBl(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void MoveHexagonsBottom(HexagonsWidget w, XEvent * event, char **args, int nArgs);
-static void MoveHexagonsBr(HexagonsWidget w, XEvent * event, char **args, int nArgs);
+static void MovePuzzleTl(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleTop(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleTr(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleLeft(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleRight(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleBl(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleBottom(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleBr(HexagonsWidget w,
+	XEvent *event, char **args, int nArgs);
 
-static char defaultTranslationsHexagons[] =
+static char defaultTranslationsPuzzle[] =
 "<KeyPress>q: Quit()\n\
  Ctrl<KeyPress>C: Quit()\n\
  <KeyPress>osfCancel: Hide()\n\
@@ -142,15 +170,16 @@ static char defaultTranslationsHexagons[] =
  <KeyPress>R15: MoveBr()\n\
  <Btn1Down>: Select()\n\
  <Btn1Up>: Release()\n\
- <KeyPress>r: Randomize()\n\
- <Btn3Down>: RandomizeMaybe()\n\
- <Btn3Down>(2+): Randomize2()\n\
+ <Btn3Down>: ClearMaybe()\n\
+ <Btn3Down>(2+): Clear2()\n\
  <Btn4Down>: MoveTop()\n\
  <Btn5Down>: MoveBottom()\n\
  <KeyPress>g: Get()\n\
  <KeyPress>w: Write()\n\
- <KeyPress>c: Clear()\n\
  <KeyPress>u: Undo()\n\
+ <KeyPress>r: Redo()\n\
+ <KeyPress>c: Clear()\n\
+ <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
  <KeyPress>o: Mode()\n\
  <KeyPress>0x2E: Speed()\n\
@@ -161,37 +190,38 @@ static char defaultTranslationsHexagons[] =
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
-static XtActionsRec actionsListHexagons[] =
+static XtActionsRec actionsListPuzzle[] =
 {
-	{(char *) "Quit", (XtActionProc) QuitHexagons},
-	{(char *) "Hide", (XtActionProc) HideHexagons},
-	{(char *) "MoveTl", (XtActionProc) MoveHexagonsTl},
-	{(char *) "MoveTop", (XtActionProc) MoveHexagonsTop},
-	{(char *) "MoveTr", (XtActionProc) MoveHexagonsTr},
-	{(char *) "MoveLeft", (XtActionProc) MoveHexagonsLeft},
-	{(char *) "MoveRight", (XtActionProc) MoveHexagonsRight},
-	{(char *) "MoveBl", (XtActionProc) MoveHexagonsBl},
-	{(char *) "MoveBottom", (XtActionProc) MoveHexagonsBottom},
-	{(char *) "MoveBr", (XtActionProc) MoveHexagonsBr},
-	{(char *) "Select", (XtActionProc) SelectHexagons},
-	{(char *) "Release", (XtActionProc) ReleaseHexagons},
-	{(char *) "Randomize", (XtActionProc) RandomizeHexagons},
-	{(char *) "RandomizeMaybe", (XtActionProc) RandomizeHexagonsMaybe},
-	{(char *) "Randomize2", (XtActionProc) RandomizeHexagons2},
-	{(char *) "Get", (XtActionProc) GetHexagons},
-	{(char *) "Write", (XtActionProc) WriteHexagons},
-	{(char *) "Clear", (XtActionProc) ClearHexagons},
-	{(char *) "Undo", (XtActionProc) UndoHexagons},
-	{(char *) "Solve", (XtActionProc) SolveHexagons},
-	{(char *) "Mode", (XtActionProc) ModeHexagons},
-	{(char *) "Speed", (XtActionProc) SpeedHexagons},
-	{(char *) "Slow", (XtActionProc) SlowHexagons},
-	{(char *) "Sound", (XtActionProc) SoundHexagons},
-	{(char *) "Enter", (XtActionProc) EnterHexagons},
-	{(char *) "Leave", (XtActionProc) LeaveHexagons}
+	{(char *) "Quit", (XtActionProc) QuitPuzzle},
+	{(char *) "Hide", (XtActionProc) HidePuzzle},
+	{(char *) "MoveTl", (XtActionProc) MovePuzzleTl},
+	{(char *) "MoveTop", (XtActionProc) MovePuzzleTop},
+	{(char *) "MoveTr", (XtActionProc) MovePuzzleTr},
+	{(char *) "MoveLeft", (XtActionProc) MovePuzzleLeft},
+	{(char *) "MoveRight", (XtActionProc) MovePuzzleRight},
+	{(char *) "MoveBl", (XtActionProc) MovePuzzleBl},
+	{(char *) "MoveBottom", (XtActionProc) MovePuzzleBottom},
+	{(char *) "MoveBr", (XtActionProc) MovePuzzleBr},
+	{(char *) "Select", (XtActionProc) SelectPuzzle},
+	{(char *) "Release", (XtActionProc) ReleasePuzzle},
+	{(char *) "ClearMaybe", (XtActionProc) ClearPuzzleMaybe},
+	{(char *) "Clear2", (XtActionProc) ClearPuzzle2},
+	{(char *) "Get", (XtActionProc) GetPuzzle},
+	{(char *) "Write", (XtActionProc) WritePuzzle},
+	{(char *) "Undo", (XtActionProc) UndoPuzzle},
+	{(char *) "Redo", (XtActionProc) RedoPuzzle},
+	{(char *) "Clear", (XtActionProc) ClearPuzzle},
+	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
+	{(char *) "Solve", (XtActionProc) SolvePuzzle},
+	{(char *) "Mode", (XtActionProc) ModePuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
+	{(char *) "Enter", (XtActionProc) EnterPuzzle},
+	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
 
-static XtResource resourcesHexagons[] =
+static XtResource resourcesPuzzle[] =
 {
 	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
 	 XtOffset(HexagonsWidget, core.width),
@@ -285,29 +315,29 @@ HexagonsClassRec hexagonsClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeHexagons,	/* initialize */
+		(XtInitProc) InitializePuzzle,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
-		actionsListHexagons,	/* actions */
-		XtNumber(actionsListHexagons),	/* num actions */
-		resourcesHexagons,	/* resources */
-		XtNumber(resourcesHexagons),	/* num resources */
+		actionsListPuzzle,	/* actions */
+		XtNumber(actionsListPuzzle),	/* num actions */
+		resourcesPuzzle,	/* resources */
+		XtNumber(resourcesPuzzle),	/* num resources */
 		NULLQUARK,	/* xrm class */
 		TRUE,		/* compress motion */
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
-		(XtWidgetProc) DestroyHexagons,		/* destroy */
-		(XtWidgetProc) ResizeHexagons,	/* resize */
-		(XtExposeProc) ExposeHexagons,	/* expose */
-		(XtSetValuesFunc) SetValuesHexagons,	/* set values */
+		(XtWidgetProc) DestroyPuzzle,	/* destroy */
+		(XtWidgetProc) ResizePuzzle,	/* resize */
+		(XtExposeProc) ExposePuzzle,	/* expose */
+		(XtSetValuesFunc) SetValuesPuzzle,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
 		NULL,		/* accept focus */
 		XtVersion,	/* version */
 		NULL,		/* callback private */
-		defaultTranslationsHexagons,	/* tm table */
+		defaultTranslationsPuzzle,	/* tm table */
 		NULL,		/* query geometry */
 		NULL,		/* display accelerator */
 		NULL		/* extension */
@@ -320,7 +350,7 @@ HexagonsClassRec hexagonsClassRec =
 WidgetClass hexagonsWidgetClass = (WidgetClass) & hexagonsClassRec;
 
 void
-SetHexagons(HexagonsWidget w, int reason)
+setPuzzle(HexagonsWidget w, int reason)
 {
 	hexagonsCallbackStruct cb;
 
@@ -341,8 +371,8 @@ loadFont(HexagonsWidget w)
 		XUnloadFont(XtDisplay(w), w->hexagons.fontInfo->fid);
 		XFreeFont(XtDisplay(w), w->hexagons.fontInfo);
 	}
-	if ((w->hexagons.fontInfo = XLoadQueryFont(display,
-			w->hexagons.font)) == NULL) {
+	if (w->hexagons.font && (w->hexagons.fontInfo =
+			XLoadQueryFont(display, w->hexagons.font)) == NULL) {
 		(void) sprintf(buf,
 			"Can not open %s font.\nAttempting %s font as alternate\n",
 			w->hexagons.font, altfontname);
@@ -393,6 +423,9 @@ static Point hexagonUnit[MAXORIENT][7] =
 };
 static Point hexagonList[MAXORIENT][7];
 
+static HexagonsStack undo = {NULL, NULL, NULL, 0};
+static HexagonsStack redo = {NULL, NULL, NULL, 0};
+
 static void
 CheckTiles(HexagonsWidget w)
 {
@@ -856,7 +889,7 @@ DrawTile(HexagonsWidget w, int pos, Boolean blank, Boolean erase,
 		tileGC = w->hexagons.tileGC;
 		borderGC = w->hexagons.borderGC;
 	}
-	
+
 	row = Row(w, pos);
 	dx = CartesianX(w, pos, row) + offsetX + pressedOffset;
 	dy = CartesianY(w, row) + offsetY + pressedOffset;
@@ -1130,16 +1163,16 @@ MovableCornerTiles(HexagonsWidget w, int direction,
 static int
 MovableNoCornTile(HexagonsWidget w)
 {
-	int rowType = HEXAGONS_BLOCKED, l;
+	int rowType = PUZZLE_BLOCKED, l;
 
 	/* Are the spaces in a "row" with the mouse click?
 	   (If two, then one clicked on a space). */
 	for (l = 0; l < ROWTYPES; l++) {
 		if (w->hexagons.currentRow[l] == w->hexagons.spaceRow[l]) {
-			if (rowType == HEXAGONS_BLOCKED) {
+			if (rowType == PUZZLE_BLOCKED) {
 				rowType = l;
 			} else {
-				return HEXAGONS_SPACE;
+				return PUZZLE_SPACE;
 			}
 		}
 	}
@@ -1264,8 +1297,9 @@ AnimateSlide(HexagonsWidget w, int dir, int fast, Boolean logMoves)
 	}
 	for (aTile = 0; aTile < numTiles; aTile++) {
 		if (logMoves) {
-			SetHexagons(w, HEXAGONS_MOVED);
-			PutMove(direction);
+			setPuzzle(w, PUZZLE_MOVED);
+			setMove(&undo, direction);
+			flushMoves(w, &redo, FALSE);
 		}
 		posNext = TileNFromSpace(w, aTile + 1, rowType, !dir);
 	}
@@ -1310,7 +1344,7 @@ ResetTiles(HexagonsWidget w)
 		DISPLAY_ERROR("Not enough memory, exiting.");
 	}
 
-	w->hexagons.spacePosition[HIGH] = w->hexagons.sizeSize - 1;
+	/*w->hexagons.spacePosition[HIGH] = w->hexagons.sizeSize - 1;
 	if (w->hexagons.corners) {
 		w->hexagons.spaceRow[HIGH] = 2 * w->hexagons.sizeY - 2;
 		if (w->hexagons.sizeX > 1 || w->hexagons.sizeY > 1) {
@@ -1327,8 +1361,51 @@ ResetTiles(HexagonsWidget w)
 	w->hexagons.tileOfPosition[w->hexagons.sizeSize - 1] = 0;
 	for (i = 1; i < w->hexagons.sizeSize - ((w->hexagons.corners) ? 1 : 0);
 			i++)
-		w->hexagons.tileOfPosition[i - 1] = i;
-	FlushMoves(w);
+		w->hexagons.tileOfPosition[i - 1] = i;*/
+	if (w->hexagons.corners) {
+		if (w->hexagons.sizeX > 1 || w->hexagons.sizeY > 1) {
+			w->hexagons.spacePosition[HIGH] = 1;
+			w->hexagons.spaceRow[HIGH] =
+				(w->hexagons.sizeX == 1) ? 1 : 0;
+			w->hexagons.spacePosition[LOW] = 0;
+			w->hexagons.spaceRow[LOW] = 0;
+			w->hexagons.tileOfPosition[0] = -1;
+			w->hexagons.tileOfPosition[1] = 0;
+		} else {
+			w->hexagons.spacePosition[HIGH] = 0;
+			w->hexagons.spaceRow[HIGH] = 0;
+			w->hexagons.tileOfPosition[0] = 0;
+		}
+	} else {
+		w->hexagons.spacePosition[HIGH] = 0;
+		w->hexagons.spaceRow[ROW] = w->hexagons.spaceRow[TRBL] = 0;
+		w->hexagons.spaceRow[TLBR] = w->hexagons.sizeX - 1;
+		w->hexagons.tileOfPosition[0] = 0;
+	}
+	if (w->hexagons.corners && w->hexagons.sizeX == 1 &&
+			w->hexagons.sizeY == 3) {
+		w->hexagons.tileOfPosition[2] = 1;
+		w->hexagons.tileOfPosition[3] = 5;
+		w->hexagons.tileOfPosition[4] = 4;
+		w->hexagons.tileOfPosition[5] = 2;
+		w->hexagons.tileOfPosition[6] = 6;
+		w->hexagons.tileOfPosition[7] = 3;
+		w->hexagons.tileOfPosition[8] = 7;
+	} else if (w->hexagons.corners && w->hexagons.sizeX == 2 &&
+			w->hexagons.sizeY == 2) {
+		w->hexagons.tileOfPosition[2] = 4;
+		w->hexagons.tileOfPosition[3] = 3;
+		w->hexagons.tileOfPosition[4] = 1;
+		w->hexagons.tileOfPosition[5] = 5;
+		w->hexagons.tileOfPosition[6] = 2;
+	} else {
+		for (i = 0; i < w->hexagons.sizeSize - 1 -
+				((w->hexagons.corners) ? 1 : 0); i++)
+			w->hexagons.tileOfPosition[i + 1 +
+				((w->hexagons.corners) ? 1 : 0)] = i + 1;
+	}
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	w->hexagons.currentPosition = -1;
 	w->hexagons.started = FALSE;
 }
@@ -1453,7 +1530,7 @@ DrawFrame(HexagonsWidget w, Pixmap dr, Boolean all, Boolean focus)
 static void
 MoveNoTiles(HexagonsWidget w)
 {
-	SetHexagons(w, HEXAGONS_IGNORE);
+	setPuzzle(w, PUZZLE_IGNORE);
 }
 
 static void
@@ -1766,11 +1843,12 @@ MoveTilesDir(HexagonsWidget w, int direction, int fast)
 }
 
 int
-MoveHexagonsDir(HexagonsWidget w, const int direction, const int fast)
+MovePuzzleDir(HexagonsWidget w, const int direction, const int fast)
 {
 	if (MoveTilesDir(w, direction, fast)) {
-		SetHexagons(w, HEXAGONS_MOVED);
-		PutMove(direction);
+		setPuzzle(w, PUZZLE_MOVED);
+		setMove(&undo, direction);
+		flushMoves(w, &redo, FALSE);
 		return TRUE;
 	}
 	return FALSE;
@@ -1780,12 +1858,12 @@ MoveHexagonsDir(HexagonsWidget w, const int direction, const int fast)
 static
 #endif
 int
-MoveHexagons(HexagonsWidget w, const int direction, const int control)
+MovePuzzle(HexagonsWidget w, const int direction, const int control)
 {
 	int reason;
 
 	if (control) {
-		reason = HEXAGONS_IGNORE;
+		reason = PUZZLE_IGNORE;
 		switch (direction) {
 			case TL:
 			case TOP:
@@ -1793,33 +1871,33 @@ MoveHexagons(HexagonsWidget w, const int direction, const int control)
 				if (w->hexagons.sizeY <= MINTILES +
 						w->hexagons.corners)
 					return FALSE;
-				reason = HEXAGONS_DEC_Y;
+				reason = PUZZLE_DEC_Y;
 				break;
 			case RIGHT:
-				reason = HEXAGONS_INC_X;
+				reason = PUZZLE_INC_X;
 				break;
 			case BL:
 			case BOTTOM:
 			case BR:
-				reason = HEXAGONS_INC_Y;
+				reason = PUZZLE_INC_Y;
 				break;
 			case LEFT:
 				if (w->hexagons.sizeX <= MINTILES)
 					return FALSE;
-				reason = HEXAGONS_DEC_X;
+				reason = PUZZLE_DEC_X;
 				break;
 			default:
 				{
 					char *buf;
 
 					intCat(&buf,
-						"MoveHexagons: direction ",
+						"MovePuzzle: direction ",
 						direction);
 					DISPLAY_WARNING(buf);
 					free(buf);
 				}
 		}
-		SetHexagons(w, reason);
+		setPuzzle(w, reason);
 		return TRUE;
 	}
 	if (CheckSolved(w)) {
@@ -1829,12 +1907,12 @@ MoveHexagons(HexagonsWidget w, const int direction, const int control)
 	if (direction == TOP || direction == BOTTOM) {
 		return FALSE;
 	}
-	if (!MoveHexagonsDir(w, direction, NORMAL)) {
-		SetHexagons(w, HEXAGONS_BLOCKED);
+	if (!MovePuzzleDir(w, direction, NORMAL)) {
+		setPuzzle(w, PUZZLE_BLOCKED);
 		return FALSE;
 	}
 	if (CheckSolved(w)) {
-		SetHexagons(w, HEXAGONS_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 	return TRUE;
 }
@@ -1871,16 +1949,19 @@ DiscreteNoCornMoves(HexagonsWidget w, int dir, int fast)
 			 w->hexagons.spaceRow[ROW] - 1;
 		MoveNoCornTiles(w, TileNFromSpace(w, 1, rowType, HIGH),
 			orient, fast);
-		SetHexagons(w, HEXAGONS_MOVED);
+		setPuzzle(w, PUZZLE_MOVED);
 		switch (rowType) {
 		case TLBR:
-			PutMove(BR);
+			setMove(&undo, BR);
+			flushMoves(w, &redo, FALSE);
 			return;
 		case TRBL:
-			PutMove(BL);
+			setMove(&undo, BL);
+			flushMoves(w, &redo, FALSE);
 			return;
 		case ROW:
-			PutMove(RIGHT);
+			setMove(&undo, RIGHT);
+			flushMoves(w, &redo, FALSE);
 			return;
 		default:
 			{
@@ -1897,16 +1978,19 @@ DiscreteNoCornMoves(HexagonsWidget w, int dir, int fast)
 			w->hexagons.spaceRow[ROW] + 1;
 		MoveNoCornTiles(w, TileNFromSpace(w, 1, rowType, LOW),
 			orient, fast);
-		SetHexagons(w, HEXAGONS_MOVED);
+		setPuzzle(w, PUZZLE_MOVED);
 		switch (rowType) {
 		case TLBR:
-			PutMove(TL);
+			setMove(&undo, TL);
+			flushMoves(w, &redo, FALSE);
 			return;
 		case TRBL:
-			PutMove(TR);
+			setMove(&undo, TR);
+			flushMoves(w, &redo, FALSE);
 			return;
 		case ROW:
-			PutMove(LEFT);
+			setMove(&undo, LEFT);
+			flushMoves(w, &redo, FALSE);
 			return;
 		default:
 			{
@@ -1971,7 +2055,7 @@ MovableCornerTile(HexagonsWidget w)
 	/* (Clicked on a space?) */
 	if (w->hexagons.currentPosition == w->hexagons.spacePosition[LOW] ||
 	    w->hexagons.currentPosition == w->hexagons.spacePosition[HIGH]) {
-		return HEXAGONS_SPACE;
+		return PUZZLE_SPACE;
 	}
 	if (FindTileTriangle(w,
 		w->hexagons.currentPosition, w->hexagons.spacePosition[HIGH],
@@ -1979,7 +2063,7 @@ MovableCornerTile(HexagonsWidget w)
 		    w->hexagons.spaceRow[HIGH], w->hexagons.spaceRow[LOW])) {
 		return 0;
 	}
-	return HEXAGONS_BLOCKED;
+	return PUZZLE_BLOCKED;
 }
 
 static void
@@ -1998,14 +2082,15 @@ SelectCornerTiles(HexagonsWidget w, int space)
 	}
 	orient = (w->hexagons.spacePosition[HIGH] <
 		w->hexagons.spacePosition[LOW]) ? !space : space;
-	PutMove(FindDir(w,
+	setMove(&undo, FindDir(w,
 		w->hexagons.currentPosition, w->hexagons.spacePosition[orient],
 		w->hexagons.currentRow[ROW], w->hexagons.spaceRow[orient]));
+	flushMoves(w, &redo, FALSE);
 	MoveCornerTiles(w, w->hexagons.currentPosition,
 			w->hexagons.currentRow[ROW], orient, NORMAL);
-	SetHexagons(w, HEXAGONS_MOVED);
+	setPuzzle(w, PUZZLE_MOVED);
 	if (CheckSolved(w)) {
-		SetHexagons(w, HEXAGONS_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
@@ -2047,7 +2132,7 @@ SelectNoCornTiles(HexagonsWidget w)
 		}
 	}
 	if (CheckSolved(w)) {
-		SetHexagons(w, HEXAGONS_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
@@ -2094,7 +2179,7 @@ RandomizeTiles(HexagonsWidget w)
 		int lastDirection = -1;
 		int randomDirection;
 
-		SetHexagons(w, HEXAGONS_RESET);
+		setPuzzle(w, PUZZLE_RESET);
 #ifdef DEBUG
 		big = 3;
 #endif
@@ -2106,17 +2191,18 @@ RandomizeTiles(HexagonsWidget w)
 			Sleep(1);
 #endif
 			if ((randomDirection + COORD / 2) % COORD != lastDirection) {
-				if (MoveHexagonsDir(w, randomDirection, INSTANT))
+				if (MovePuzzleDir(w, randomDirection, INSTANT))
 					lastDirection = randomDirection;
 				else
 					big++;
 			}
 		}
-		FlushMoves(w);
-		SetHexagons(w, HEXAGONS_RANDOMIZE);
+		flushMoves(w, &undo, TRUE);
+		flushMoves(w, &redo, FALSE);
+		setPuzzle(w, PUZZLE_RANDOMIZE);
 	}
 	if (CheckSolved(w)) {
-		SetHexagons(w, HEXAGONS_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
@@ -2163,20 +2249,21 @@ GetTiles(HexagonsWidget w)
 		}
 #endif
 	}
-	FlushMoves(w);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &corners);
 	if (w->hexagons.corners != (Boolean) corners) {
-		SetHexagons(w, HEXAGONS_CORNERS);
+		setPuzzle(w, PUZZLE_CORNERS);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &sizeX);
 	if (sizeX >= MINTILES) {
 		for (i = w->hexagons.sizeX; i < sizeX; i++) {
-			SetHexagons(w, HEXAGONS_INC_X);
+			setPuzzle(w, PUZZLE_INC_X);
 		}
 		for (i = w->hexagons.sizeX; i > sizeX; i--) {
-			SetHexagons(w, HEXAGONS_DEC_X);
+			setPuzzle(w, PUZZLE_DEC_X);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizeX ");
@@ -2196,10 +2283,10 @@ GetTiles(HexagonsWidget w)
 	(void) fscanf(fp, "%d", &sizeY);
 	if (sizeY >= MINTILES) {
 		for (i = w->hexagons.sizeY; i < sizeY; i++) {
-			SetHexagons(w, HEXAGONS_INC_Y);
+			setPuzzle(w, PUZZLE_INC_Y);
 		}
 		for (i = w->hexagons.sizeY; i > sizeY; i--) {
-			SetHexagons(w, HEXAGONS_DEC_Y);
+			setPuzzle(w, PUZZLE_DEC_Y);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizeY ");
@@ -2219,10 +2306,10 @@ GetTiles(HexagonsWidget w)
 #endif
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
-	ScanStartPosition(fp, w);
-	SetHexagons(w, HEXAGONS_RESTORE);
-	SetStartPosition(w);
-	ScanMoves(fp, w, moves);
+	scanStartPosition(fp, w);
+	setPuzzle(w, PUZZLE_RESTORE);
+	setStartPosition(w);
+	scanMoves(fp, w, moves);
 	(void) fclose(fp);
 	(void) printf("%s: corners %d, sizeX %d, sizeY %d, moves %d.\n",
 		name, corners, sizeX, sizeY, moves);
@@ -2277,35 +2364,28 @@ WriteTiles(HexagonsWidget w)
 		(w->hexagons.corners) ? 1 : 0);
 	(void) fprintf(fp, "sizeX%c %d\n", SYMBOL, w->hexagons.sizeX);
 	(void) fprintf(fp, "sizeY%c %d\n", SYMBOL, w->hexagons.sizeY);
-	(void) fprintf(fp, "moves%c %d\n", SYMBOL, NumMoves());
-	PrintStartPosition(fp, w);
-	PrintMoves(fp);
+	(void) fprintf(fp, "moves%c %d\n", SYMBOL,
+		numMoves(&undo));
+	printStartPosition(fp, w);
+	printMoves(fp, &undo);
 	(void) fclose(fp);
 	(void) printf("Saved to %s.\n", name);
 	free(lname);
 	free(fname);
 }
 
-static void
-ClearTiles(HexagonsWidget w)
-{
-	if (w->hexagons.currentPosition >= 0)
-		return;
-	ResetTiles(w);
-	DrawAllTiles(w);
-	SetHexagons(w, HEXAGONS_RESET);
-}
-
 static void
 UndoTiles(HexagonsWidget w)
 {
-	if (MadeMoves() && w->hexagons.currentPosition < 0) {
+	if (madeMoves(&undo) &&
+			 w->hexagons.currentPosition < 0) {
 		int direction;
 
-		GetMove(&direction);
+		getMove(&undo, &direction);
+		setMove(&redo, direction);
 		direction = (direction + (COORD / 2)) % COORD;
 		if (MoveTilesDir(w, direction, DOUBLE)) {
-			SetHexagons(w, HEXAGONS_UNDO);
+			setPuzzle(w, PUZZLE_UNDO);
 		} else {
 			char *buf1, *buf2;
 
@@ -2318,13 +2398,46 @@ UndoTiles(HexagonsWidget w)
 	}
 }
 
+static void
+RedoTiles(HexagonsWidget w)
+{
+	if (madeMoves(&redo) &&
+			 w->hexagons.currentPosition < 0) {
+		int direction;
+
+		getMove(&redo, &direction);
+		setMove(&undo, direction);
+		if (MoveTilesDir(w, direction, DOUBLE)) {
+			setPuzzle(w, PUZZLE_REDO);
+		} else {
+			char *buf1, *buf2;
+
+			intCat(&buf1, "Move ", direction);
+			stringCat(&buf2, buf1, " can not be made");
+			free(buf1);
+			DISPLAY_WARNING(buf2);
+			free(buf2);
+		}
+	}
+}
+
+static void
+ClearTiles(HexagonsWidget w)
+{
+	if (w->hexagons.currentPosition >= 0)
+		return;
+	ResetTiles(w);
+	DrawAllTiles(w);
+	setPuzzle(w, PUZZLE_RESET);
+}
+
 static void
 SolveTiles(HexagonsWidget w)
 {
 	if (CheckSolved(w) || w->hexagons.currentPosition >= 0)
 		return;
 	{
-		SetHexagons(w, HEXAGONS_SOLVE_MESSAGE);
+		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
 	}
 }
 
@@ -2332,7 +2445,7 @@ static void
 ModeTiles(HexagonsWidget w)
 {
 	if (w->hexagons.corners || w->hexagons.sizeY > 1) {
-		SetHexagons(w, HEXAGONS_CORNERS);
+		setPuzzle(w, PUZZLE_CORNERS);
 	}
 }
 
@@ -2343,7 +2456,7 @@ SpeedTiles(HexagonsWidget w)
 	if (w->hexagons.delay < 0)
 		w->hexagons.delay = 0;
 #ifdef HAVE_MOTIF
-	SetHexagons(w, HEXAGONS_SPEED);
+	setPuzzle(w, PUZZLE_SPEED);
 #endif
 }
 
@@ -2352,7 +2465,7 @@ SlowTiles(HexagonsWidget w)
 {
 	w->hexagons.delay += 5;
 #ifdef HAVE_MOTIF
-	SetHexagons(w, HEXAGONS_SPEED);
+	setPuzzle(w, PUZZLE_SPEED);
 #endif
 }
 
@@ -2384,7 +2497,7 @@ darker(const int light)
 }
 
 static void
-SetValuesHexagons(HexagonsWidget w)
+SetValuesPuzzle(HexagonsWidget w)
 {
 	struct tagColor {
 		int red, green, blue;
@@ -2442,8 +2555,17 @@ SetValuesHexagons(HexagonsWidget w)
 }
 
 void
-DestroyHexagons(HBRUSH brush)
+DestroyPuzzle(HexagonsWidget w, HBRUSH brush)
 {
+	if (w->core.memDC) {
+		if (w->hexagons.bufferTiles != NULL) {
+			DeleteObject(w->hexagons.bufferTiles);
+		}
+		DeleteDC(w->core.memDC);
+		w->core.memDC = NULL;
+	}
+	deleteMoves(&undo);
+	deleteMoves(&redo);
 	(void) DeleteObject(brush);
 	PostQuitMessage(0);
 }
@@ -2572,7 +2694,7 @@ SetAllColors(HexagonsWidget w)
 }
 
 static Boolean
-SetValuesHexagons(Widget current, Widget request, Widget renew)
+SetValuesPuzzle(Widget current, Widget request, Widget renew)
 {
 	HexagonsWidget c = (HexagonsWidget) current, w = (HexagonsWidget) renew;
 	Boolean redraw = FALSE;
@@ -2596,11 +2718,11 @@ SetValuesHexagons(Widget current, Widget request, Widget renew)
 			w->hexagons.sizeY != c->hexagons.sizeY ||
 			w->hexagons.corners != c->hexagons.corners ||
 			w->hexagons.base != c->hexagons.base) {
-		SizeHexagons(w);
+		SizePuzzle(w);
 		redraw = TRUE;
 	} else if (w->hexagons.offset.x != c->hexagons.offset.x ||
 			w->hexagons.offset.y != c->hexagons.offset.y) {
-		ResizeHexagons(w);
+		ResizePuzzle(w);
 		redraw = TRUE;
 	}
 	if (w->hexagons.menu != -1) {
@@ -2608,34 +2730,37 @@ SetValuesHexagons(Widget current, Widget request, Widget renew)
 
 		w->hexagons.menu = -1;
 		switch (menu) {
-		case 0:
+		case MENU_GET:
 			GetTiles(w);
 			break;
-		case 1:
+		case MENU_WRITE:
 			WriteTiles(w);
 			break;
-		case 3:
-			ClearTiles(w);
-			break;
-		case 4:
+		case MENU_UNDO:
 			UndoTiles(w);
 			break;
-		case 5:
+		case MENU_REDO:
+			RedoTiles(w);
+			break;
+		case MENU_CLEAR:
+			ClearTiles(w);
+			break;
+		case MENU_RANDOMIZE:
 			RandomizeTiles(w);
 			break;
-		case 6:
+		case MENU_SOLVE:
 			SolveTiles(w);
 			break;
-		case 7:
+		case MENU_MODE:
 			ModeTiles(w);
 			break;
-		case 8:
+		case MENU_SPEED:
 			SpeedTiles(w);
 			break;
-		case 9:
+		case MENU_SLOW:
 			SlowTiles(w);
 			break;
-		case 10:
+		case MENU_SOUND:
 			SoundTiles(w);
 			break;
 		default:
@@ -2651,24 +2776,19 @@ SetValuesHexagons(Widget current, Widget request, Widget renew)
 }
 
 static void
-QuitHexagons(HexagonsWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
+DestroyPuzzle(Widget old)
 {
-#ifdef WINVER
-	if (w->hexagons.freeDC) {
-		if (w->hexagons.bufferTiles != NULL) {
-			DeleteObject(w->hexagons.bufferTiles);
-		}
-		DeleteDC(w->core.memDC);
-		w->core.memDC = NULL;
-	}
-#else
+	HexagonsWidget w = (HexagonsWidget) old;
 	Display *display = XtDisplay(w);
 	int sel;
 
+	XtReleaseGC(old, w->hexagons.tileGC);
+	XtReleaseGC(old, w->hexagons.borderGC);
+	XtReleaseGC(old, w->hexagons.tileBrighterGC);
+	XtReleaseGC(old, w->hexagons.tileDarkerGC);
+	XtReleaseGC(old, w->hexagons.frameGC);
+	XtReleaseGC(old, w->hexagons.inverseGC);
+	XtRemoveCallbacks(old, XtNselectCallback, w->hexagons.select);
 	if (w->hexagons.colormap != None) {
 		XInstallColormap(display, w->hexagons.oldColormap);
 		XFreeColormap(display, w->hexagons.colormap);
@@ -2680,23 +2800,15 @@ QuitHexagons(HexagonsWidget w
 		XUnloadFont(display, w->hexagons.fontInfo->fid);
 		XFreeFont(display, w->hexagons.fontInfo);
 	}
-	XtCloseDisplay(display);
-#endif
-	exit(0);
+	deleteMoves(&undo);
+	deleteMoves(&redo);
 }
 
 static void
-DestroyHexagons(Widget old)
+QuitPuzzle(HexagonsWidget w, XEvent *event, char **args, int nArgs)
 {
-	HexagonsWidget w = (HexagonsWidget) old;
-
-	XtReleaseGC(old, w->hexagons.tileGC);
-	XtReleaseGC(old, w->hexagons.borderGC);
-	XtReleaseGC(old, w->hexagons.tileBrighterGC);
-	XtReleaseGC(old, w->hexagons.tileDarkerGC);
-	XtReleaseGC(old, w->hexagons.frameGC);
-	XtReleaseGC(old, w->hexagons.inverseGC);
-	XtRemoveCallbacks(old, XtNselectCallback, w->hexagons.select);
+	XtCloseDisplay(XtDisplay(w));
+	exit(0);
 }
 #endif
 
@@ -2812,7 +2924,7 @@ ResizeTiles(HexagonsWidget w)
 static
 #endif
 void
-ResizeHexagons(HexagonsWidget w)
+ResizePuzzle(HexagonsWidget w)
 {
 	const double sqrt_3 = 1.73205080756887729352744634150587237;
 	Point tempSize;
@@ -2864,17 +2976,17 @@ ResizeHexagons(HexagonsWidget w)
 static
 #endif
 void
-SizeHexagons(HexagonsWidget w)
+SizePuzzle(HexagonsWidget w)
 {
 	ResetTiles(w);
-	ResizeHexagons(w);
+	ResizePuzzle(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-InitializeHexagons(
+InitializePuzzle(
 #ifdef WINVER
 HexagonsWidget w, HBRUSH brush
 #else
@@ -2883,7 +2995,7 @@ Widget request, Widget renew
 )
 {
 #ifdef WINVER
-	SetValuesHexagons(w);
+	SetValuesPuzzle(w);
 	brush = CreateSolidBrush(w->hexagons.inverseGC);
 	SETBACK(w->core.hWnd, brush);
 	(void) SRAND(time(NULL));
@@ -2907,27 +3019,29 @@ XSynchronize(XtDisplay(w), True);
 	w->hexagons.tileDarkerGC = NULL;
 	w->hexagons.frameGC = NULL;
 	w->hexagons.inverseGC = NULL;
+	w->hexagons.image = NULL;
 #endif
 	w->hexagons.focus = False;
 	loadFont(w);
 	w->hexagons.tileOfPosition = NULL;
 	CheckTiles(w);
-	InitMoves();
+	newMoves(&undo);
+	newMoves(&redo);;
 	w->hexagons.numSlices = ((w->hexagons.delay < MAXSLICES) ?
 		w->hexagons.delay + 1 : MAXSLICES);
 	w->hexagons.cheat = False;
-	SizeHexagons(w);
+	SizePuzzle(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-ExposeHexagons(
+ExposePuzzle(
 #ifdef WINVER
 HexagonsWidget w
 #else
-Widget renew, XEvent * event, Region region
+Widget renew, XEvent *event, Region region
 #endif
 )
 {
@@ -2947,24 +3061,24 @@ Widget renew, XEvent * event, Region region
 static
 #endif
 void
-HideHexagons(HexagonsWidget w
+HidePuzzle(HexagonsWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
-	SetHexagons(w, HEXAGONS_HIDE);
+	setPuzzle(w, PUZZLE_HIDE);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-SelectHexagons(HexagonsWidget w
+SelectPuzzle(HexagonsWidget w
 #ifdef WINVER
 , const int x, const int y
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2989,15 +3103,15 @@ SelectHexagons(HexagonsWidget w
 			rowType = MovableNoCornTile(w);
 		if (rowType < 0) {
 			DrawTile(w, w->hexagons.currentPosition,
-				rowType == HEXAGONS_SPACE, False, TRUE, 0, 0);
+				rowType == PUZZLE_SPACE, False, TRUE, 0, 0);
 			FLUSH(w);
 			Sleep(100);
 			DrawTile(w, w->hexagons.currentPosition,
 				True, True, TRUE, 0, 0);
-			if (rowType != HEXAGONS_SPACE)
+			if (rowType != PUZZLE_SPACE)
 				DrawTile(w, w->hexagons.currentPosition,
 					False, False, FALSE, 0, 0);
-			SetHexagons(w, rowType);
+			setPuzzle(w, rowType);
 			w->hexagons.currentPosition = -1;
 			return;
 		}
@@ -3011,11 +3125,11 @@ SelectHexagons(HexagonsWidget w
 static
 #endif
 void
-ReleaseHexagons(HexagonsWidget w
+ReleasePuzzle(HexagonsWidget w
 #ifdef WINVER
 , const int x, const int y
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -3050,43 +3164,30 @@ ReleaseHexagons(HexagonsWidget w
 	w->hexagons.currentPosition = -1;
 }
 
-#ifndef WINVER
-static
-#endif
-void
-RandomizeHexagons(HexagonsWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	RandomizeTiles(w);
-}
-
 #ifndef WINVER
 static void
-RandomizeHexagonsMaybe(HexagonsWidget w
-, XEvent * event, char **args, int nArgs
+ClearPuzzleMaybe(HexagonsWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 	if (!w->hexagons.started)
-		RandomizeTiles(w);
+		ClearTiles(w);
 #ifdef HAVE_MOTIF
 	else {
-		SetHexagons(w, HEXAGONS_RANDOMIZE_QUERY);
+		setPuzzle(w, PUZZLE_CLEAR_QUERY);
 	}
 #endif
 }
 
 static void
-RandomizeHexagons2(HexagonsWidget w
-, XEvent * event, char **args, int nArgs
+ClearPuzzle2(HexagonsWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 #ifdef HAVE_MOTIF
 	if (!w->hexagons.started)
 #endif
-		RandomizeTiles(w);
+		ClearTiles(w);
 }
 #endif
 
@@ -3094,9 +3195,9 @@ RandomizeHexagons2(HexagonsWidget w
 static
 #endif
 void
-GetHexagons(HexagonsWidget w
+GetPuzzle(HexagonsWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -3107,9 +3208,9 @@ GetHexagons(HexagonsWidget w
 static
 #endif
 void
-WriteHexagons(HexagonsWidget w
+WritePuzzle(HexagonsWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -3120,22 +3221,9 @@ WriteHexagons(HexagonsWidget w
 static
 #endif
 void
-ClearHexagons(HexagonsWidget w
+UndoPuzzle(HexagonsWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	ClearTiles(w);
-}
-
-#ifndef WINVER
-static
-#endif
-void
-UndoHexagons(HexagonsWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -3146,9 +3234,48 @@ UndoHexagons(HexagonsWidget w
 static
 #endif
 void
-SolveHexagons(HexagonsWidget w
+RedoPuzzle(HexagonsWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RedoTiles(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+ClearPuzzle(HexagonsWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	ClearTiles(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+RandomizePuzzle(HexagonsWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RandomizeTiles(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+SolvePuzzle(HexagonsWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -3159,9 +3286,9 @@ SolveHexagons(HexagonsWidget w
 static
 #endif
 void
-ModeHexagons(HexagonsWidget w
+ModePuzzle(HexagonsWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -3172,9 +3299,9 @@ ModeHexagons(HexagonsWidget w
 static
 #endif
 void
-SpeedHexagons(HexagonsWidget w
+SpeedPuzzle(HexagonsWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -3185,9 +3312,9 @@ SpeedHexagons(HexagonsWidget w
 static
 #endif
 void
-SlowHexagons(HexagonsWidget w
+SlowPuzzle(HexagonsWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -3198,9 +3325,9 @@ SlowHexagons(HexagonsWidget w
 static
 #endif
 void
-SoundHexagons(HexagonsWidget w
+SoundPuzzle(HexagonsWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -3211,9 +3338,9 @@ SoundHexagons(HexagonsWidget w
 static
 #endif
 void
-EnterHexagons(HexagonsWidget w
+EnterPuzzle(HexagonsWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -3225,9 +3352,9 @@ EnterHexagons(HexagonsWidget w
 static
 #endif
 void
-LeaveHexagons(HexagonsWidget w
+LeavePuzzle(HexagonsWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -3237,50 +3364,50 @@ LeaveHexagons(HexagonsWidget w
 
 #ifndef WINVER
 static void
-MoveHexagonsTl(HexagonsWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleTl(HexagonsWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveHexagons(w, TL, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, TL, (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveHexagonsTop(HexagonsWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleTop(HexagonsWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveHexagons(w, TOP, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, TOP, (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveHexagonsTr(HexagonsWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleTr(HexagonsWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveHexagons(w, TR, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, TR, (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveHexagonsLeft(HexagonsWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleLeft(HexagonsWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveHexagons(w, LEFT, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, LEFT, (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveHexagonsRight(HexagonsWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleRight(HexagonsWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveHexagons(w, RIGHT, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, RIGHT, (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveHexagonsBl(HexagonsWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleBl(HexagonsWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveHexagons(w, BL, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, BL, (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveHexagonsBottom(HexagonsWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleBottom(HexagonsWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveHexagons(w, BOTTOM, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, BOTTOM, (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveHexagonsBr(HexagonsWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleBr(HexagonsWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveHexagons(w, BR, (int) (event->xkey.state & ControlMask));
+	(void) MovePuzzle(w, BR, (int) (event->xkey.state & ControlMask));
 }
 #endif
