@@ -85,26 +85,40 @@ typedef struct {
 } hexagonsCallbackStruct;
 #endif
 
-#define HEXAGONS_HIDE (-8)
-#define HEXAGONS_RANDOMIZE_QUERY (-7)
-#define HEXAGONS_SOLVE_MESSAGE (-6)
-#define HEXAGONS_RESTORE (-5)
-#define HEXAGONS_RESET (-4)
-#define HEXAGONS_BLOCKED (-3)
-#define HEXAGONS_SPACE (-2)
-#define HEXAGONS_IGNORE (-1)
-#define HEXAGONS_MOVED 0
-#define HEXAGONS_SOLVED 1
-#define HEXAGONS_PRACTICE 2
-#define HEXAGONS_RANDOMIZE 3
-#define HEXAGONS_DEC_X 4
-#define HEXAGONS_INC_X 5
-#define HEXAGONS_DEC_Y 6
-#define HEXAGONS_INC_Y 7
-#define HEXAGONS_CORNERS 8
-#define HEXAGONS_COMPUTED 9
-#define HEXAGONS_UNDO 10
-#define HEXAGONS_SPEED 11
+#define PUZZLE_HIDE (-8)
+#define PUZZLE_CLEAR_QUERY (-7)
+#define PUZZLE_SOLVE_MESSAGE (-6)
+#define PUZZLE_RESTORE (-5)
+#define PUZZLE_RESET (-4)
+#define PUZZLE_BLOCKED (-3)
+#define PUZZLE_SPACE (-2)
+#define PUZZLE_IGNORE (-1)
+#define PUZZLE_MOVED 0
+#define PUZZLE_SOLVED 1
+#define PUZZLE_PRACTICE 2
+#define PUZZLE_RANDOMIZE 3
+#define PUZZLE_DEC_X 4
+#define PUZZLE_INC_X 5
+#define PUZZLE_DEC_Y 6
+#define PUZZLE_INC_Y 7
+#define PUZZLE_CORNERS 8
+#define PUZZLE_COMPUTED 9
+#define PUZZLE_UNDO 10
+#define PUZZLE_REDO 11
+#define PUZZLE_SPEED 12
+
+#define MENU_GET 0
+#define MENU_WRITE 1
+#define MENU_EXIT 2
+#define MENU_UNDO 3
+#define MENU_REDO 4
+#define MENU_CLEAR 5
+#define MENU_RANDOMIZE 6
+#define MENU_SOLVE 7
+#define MENU_MODE 8
+#define MENU_SPEED 9
+#define MENU_SLOW 10
+#define MENU_SOUND 11
 
 #define MINTILES 1
 #define DEFAULTTILESX 3
