@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1994 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -33,29 +33,28 @@
 #include "Hexagons.h"
 
 #ifdef WINVER
-extern void DestroyHexagons(HBRUSH brush);
-extern void ResizeHexagons(HexagonsWidget w);
-extern void SizeHexagons(HexagonsWidget w);
-extern void InitializeHexagons(HexagonsWidget w, HBRUSH brush);
-extern void ExposeHexagons(HexagonsWidget w);
-extern void HideHexagons(HexagonsWidget w);
-extern void SelectHexagons(HexagonsWidget w, const int x, const int y);
-extern void ReleaseHexagons(HexagonsWidget w, const int x, const int y);
-extern void RandomizeHexagons(HexagonsWidget w);
-extern void RandomizeHexagonsMaybe(HexagonsWidget w);
-extern void RandomizeHexagons2(HexagonsWidget w);
-extern void ClearHexagons(HexagonsWidget w);
-extern void GetHexagons(HexagonsWidget w);
-extern void WriteHexagons(HexagonsWidget w);
-extern void UndoHexagons(HexagonsWidget w);
-extern void SolveHexagons(HexagonsWidget w);
-extern void ModeHexagons(HexagonsWidget w);
-extern void SpeedHexagons(HexagonsWidget w);
-extern void SlowHexagons(HexagonsWidget w);
-extern void SoundHexagons(HexagonsWidget w);
-extern void EnterHexagons(HexagonsWidget w);
-extern void LeaveHexagons(HexagonsWidget w);
-extern int MoveHexagons(HexagonsWidget w, const int direction,
+extern void DestroyPuzzle(HexagonsWidget w, HBRUSH brush);
+extern void ResizePuzzle(HexagonsWidget w);
+extern void SizePuzzle(HexagonsWidget w);
+extern void InitializePuzzle(HexagonsWidget w, HBRUSH brush);
+extern void ExposePuzzle(HexagonsWidget w);
+extern void HidePuzzle(HexagonsWidget w);
+extern void SelectPuzzle(HexagonsWidget w, const int x, const int y);
+extern void ReleasePuzzle(HexagonsWidget w, const int x, const int y);
+extern void GetPuzzle(HexagonsWidget w);
+extern void WritePuzzle(HexagonsWidget w);
+extern void UndoPuzzle(HexagonsWidget w);
+extern void RedoPuzzle(HexagonsWidget w);
+extern void ClearPuzzle(HexagonsWidget w);
+extern void RandomizePuzzle(HexagonsWidget w);
+extern void SolvePuzzle(HexagonsWidget w);
+extern void ModePuzzle(HexagonsWidget w);
+extern void SpeedPuzzle(HexagonsWidget w);
+extern void SlowPuzzle(HexagonsWidget w);
+extern void SoundPuzzle(HexagonsWidget w);
+extern void EnterPuzzle(HexagonsWidget w);
+extern void LeavePuzzle(HexagonsWidget w);
+extern int MovePuzzle(HexagonsWidget w, const int direction,
 	const int control);
 
 #else
@@ -102,6 +101,23 @@ extern HexagonsClassRec hexagonsClassRec;
 #define DOUBLE 2
 #define INSTANT 3
 
+typedef struct _MoveRecord {
+#if 0
+	int direction;
+#endif
+	unsigned char packed;   /* This makes assumptions on the data. */
+} MoveRecord;
+
+typedef struct _MoveStack {
+	MoveRecord move;
+	struct _MoveStack *previous, *next;
+} MoveStack;
+
+typedef struct _HexagonsStack {
+	MoveStack *currMove, *lastMove, *firstMove;
+	int count;
+} HexagonsStack;
+
 typedef struct _HexagonsPart {
 	int        *tileOfPosition, spacePosition[SPACES];
 	int         spaceRow[ROWTYPES];
@@ -140,8 +156,8 @@ typedef struct _HexagonsRec {
 } HexagonsRec;
 
 extern int *startPosition;
-extern void SetHexagons(HexagonsWidget w, int reason);
-extern int MoveHexagonsDir(HexagonsWidget w, const int direction,
+extern void setPuzzle(HexagonsWidget w, int reason);
+extern int MovePuzzleDir(HexagonsWidget w, const int direction,
 	const int fast);
 
 #if 0
@@ -150,17 +166,18 @@ extern void SolveSomeTiles(HexagonsWidget w);
 #endif
 extern void DrawAllTiles(const HexagonsWidget w);
 extern Boolean CheckSolved(const HexagonsWidget w);
-extern void InitMoves(void);
-extern void PutMove(int direction);
-extern void GetMove(int *direction);
-extern int MadeMoves(void);
-extern void FlushMoves(HexagonsWidget w);
-extern int NumMoves(void);
-extern void ScanMoves(FILE * fp, HexagonsWidget w, int moves);
-extern void PrintMoves(FILE * fp);
-extern void ScanStartPosition(FILE * fp, HexagonsWidget w);
-extern void PrintStartPosition(FILE * fp, HexagonsWidget w);
-extern void SetStartPosition(HexagonsWidget w);
+extern void newMoves(HexagonsStack *s);
+extern void deleteMoves(HexagonsStack *s);
+extern void setMove(HexagonsStack *s, int direction);
+extern void getMove(HexagonsStack *s, int *direction);
+extern int madeMoves(HexagonsStack *s);
+extern void flushMoves(HexagonsWidget w, HexagonsStack *s, Boolean undo);
+extern int numMoves(HexagonsStack *s);
+extern void scanMoves(FILE *fp, HexagonsWidget w, int moves);
+extern void printMoves(FILE *fp, HexagonsStack *s);
+extern void scanStartPosition(FILE *fp, HexagonsWidget w);
+extern void printStartPosition(FILE *fp, HexagonsWidget w);
+extern void setStartPosition(HexagonsWidget w);
 extern int Row(HexagonsWidget w, const int pos);
 extern int TrBl(HexagonsWidget w, const int pos, const int posRow);
 extern int TlBr(HexagonsWidget w, const int pos, const int posRow);
