@@ -89,7 +89,7 @@ static HexagonsRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Hexagons Version 7.2\n"
+"Hexagons Version 7.2.4\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -133,22 +133,25 @@ static const char featuresHelp[] = {
 "move a tile.  One can move more than one tile at a time\n"
 "and one is not constrained by parity.\n"
 "\n"
-"Click \"mouse-right\" button, or press \"R\" or \"r\" "
-"keys, to randomize the puzzle.\n"
+"Click \"mouse-right\" button, or press \"C\" or \"c\" "
+"keys, to clear the puzzle.\n"
 "\n"
-"Press \"O\" or \"o\" keys to toggle corners mode.  "
-"(Default is on).\n"
+"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
+"\n"
+"Press \"W\" or \"w\" keys to save (write) a puzzle.\n"
+"\n"
+"Press \"U\" or \"u\" keys to undo a move.\n"
+"\n"
+"Press \"R\" or \"r\" keys to redo a move.\n"
+"\n"
+"Press \"Z\" or \"z\" keys to randomize the puzzle.  New records "
+"can only be set from a cleared state.\n"
 "\n"
 "\"S\" or \"s\" keys reserved for the auto-solver "
 "(not implemented).\n"
 "\n"
-"Press \"U\" or \"u\" keys to undo a move.\n"
-"\n"
-"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
-"\n"
-"Press \"W\" or \"w\" keys to save a puzzle.\n"
-"\n"
-"Press \"C\" or \"c\" keys to clear a puzzle.\n"
+"Press \"O\" or \"o\" keys to toggle corners mode.  "
+"(Default is on).\n"
 "\n"
 "Press \">\" or \".\" keys to speed up the movement of tiles.\n"
 "\n"
@@ -222,19 +225,20 @@ static Widget moves, record, message, cornersSwitch, tileX, tileY;
 static char buff[21];
 static Widget descriptionDialog, featuresDialog;
 static Widget optionsDialog, referencesDialog, aboutDialog;
-static Widget solveDialog, randomizeDialog;
+static Widget solveDialog, clearDialog;
 static Arg arg[3];
 #else
 static char titleDsp[TITLELEN] = "";
 #endif
 static char scoreFileName[FILENAMELEN] = SCOREFILE;
 static char fileName[FILENAMELEN];
+static Boolean randomized = False;
 #ifdef WINVER
 #define MAXPROGNAME 80
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap hexagonsIcon = None;
+static Pixmap icon = None;
 static Widget topLevel, hexagons;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
@@ -471,7 +475,7 @@ Widget w
 	char *userName, *scoreFile;
 
 #ifdef WINVER
-	InitializeHexagons(w, brush);
+	InitializePuzzle(w, brush);
 
 	sizeX = w->hexagons.sizeX;
 	sizeY = w->hexagons.sizeY;
@@ -533,7 +537,7 @@ Widget w
 #ifdef WINVER
 		(void) strncpy(userNameDsp, userName, USERNAMELEN);
 #else
-		char * login = getlogin();
+		char *login = getlogin();
 
 		if (login == NULL) {
 			(void) strcpy(userNameDsp, "");
@@ -558,11 +562,11 @@ Widget w
 
 #ifdef WINVER
 void
-SetHexagons(HexagonsWidget w, int reason)
+setPuzzle(HexagonsWidget w, int reason)
 #else
 static void
-CallbackHexagons(Widget w, caddr_t clientData,
-		hexagonsCallbackStruct * callData)
+CallbackPuzzle(Widget w, caddr_t clientData,
+		hexagonsCallbackStruct *callData)
 #endif
 {
 #ifndef WINVER
@@ -585,7 +589,7 @@ CallbackHexagons(Widget w, caddr_t clientData,
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-		case HEXAGONS_HIDE:
+		case PUZZLE_HIDE:
 #ifdef WINVER
 			ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -595,16 +599,15 @@ CallbackHexagons(Widget w, caddr_t clientData,
 #endif
 			break;
 #ifndef WINVER
-		case HEXAGONS_RANDOMIZE_QUERY:
+		case PUZZLE_CLEAR_QUERY:
 #ifdef HAVE_MOTIF
-			XtManageChild(randomizeDialog);
+			XtManageChild(clearDialog);
 #else
-			XtVaSetValues(hexagons,
-				XtNmenu, 5, NULL); /* menu choice */
+			XtVaSetValues(hexagons, XtNmenu, MENU_CLEAR, NULL);
 #endif
 			break;
 #endif
-		case HEXAGONS_SOLVE_MESSAGE:
+		case PUZZLE_SOLVE_MESSAGE:
 #ifdef WINVER
 			(void) MessageBox(w->core.hWnd, solveHelp,
 				"Auto-Solve", MB_OK);
@@ -616,33 +619,35 @@ CallbackHexagons(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case HEXAGONS_RESTORE:
-		case HEXAGONS_RESET:
+		case PUZZLE_RESTORE:
+		case PUZZLE_RESET:
 			movesDsp = 0;
+			randomized = False;
 			break;
-		case HEXAGONS_BLOCKED:
+		case PUZZLE_BLOCKED:
 			(void) strncpy(messageDsp, "Blocked", MESSAGELEN);
 			break;
-		case HEXAGONS_SPACE:
+		case PUZZLE_SPACE:
 #if 0
 			/* Too annoying */
 			(void) sprintf(messageDsp, "%s can not slide",
 				(corners) ? "Spaces" : "A space");
 #endif
 			break;
-		case HEXAGONS_IGNORE:
-			(void) strncpy(messageDsp, "Randomize to start",
+		case PUZZLE_IGNORE:
+			(void) strncpy(messageDsp, "Clear to start",
 				MESSAGELEN);
 			break;
-		case HEXAGONS_MOVED:
+		case PUZZLE_MOVED:
 			movesDsp++;
 			SET_STARTED(w, True);
 			break;
-		case HEXAGONS_SOLVED:
+		case PUZZLE_SOLVED:
 			if (cheat)
 				(void) sprintf(messageDsp,
 					"No cheating %s!!", userNameDsp);
-			else if (HandleSolved(movesDsp, sizeX, sizeY, corners))
+			else if (!randomized && HandleSolved(movesDsp,
+					sizeX, sizeY, corners))
 				(void) sprintf(messageDsp,
 					"Congratulations %s!!", userNameDsp);
 			else
@@ -650,11 +655,12 @@ CallbackHexagons(Widget w, caddr_t clientData,
 					MESSAGELEN);
 			SET_STARTED(w, False);
 			break;
-		case HEXAGONS_RANDOMIZE:
+		case PUZZLE_RANDOMIZE:
 			movesDsp = 0;
+			randomized = True;
 			SET_STARTED(w, False);
 			break;
-		case HEXAGONS_DEC_X:
+		case PUZZLE_DEC_X:
 			movesDsp = 0;
 			sizeX--;
 			PrintRecord(sizeX, sizeY, corners);
@@ -669,7 +675,7 @@ CallbackHexagons(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case HEXAGONS_INC_X:
+		case PUZZLE_INC_X:
 			movesDsp = 0;
 			sizeX++;
 			PrintRecord(sizeX, sizeY, corners);
@@ -684,7 +690,7 @@ CallbackHexagons(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case HEXAGONS_DEC_Y:
+		case PUZZLE_DEC_Y:
 			movesDsp = 0;
 			sizeY--;
 			PrintRecord(sizeX, sizeY, corners);
@@ -699,7 +705,7 @@ CallbackHexagons(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case HEXAGONS_INC_Y:
+		case PUZZLE_INC_Y:
 			movesDsp = 0;
 			sizeY++;
 			PrintRecord(sizeX, sizeY, corners);
@@ -714,7 +720,7 @@ CallbackHexagons(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case HEXAGONS_CORNERS:
+		case PUZZLE_CORNERS:
 			movesDsp = 0;
 			corners = !corners;
 			PrintRecord(sizeX, sizeY, corners);
@@ -727,15 +733,19 @@ CallbackHexagons(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case HEXAGONS_COMPUTED:
+		case PUZZLE_COMPUTED:
 			SET_STARTED(w, False);
 			break;
-		case HEXAGONS_UNDO:
+		case PUZZLE_UNDO:
 			movesDsp--;
 			SET_STARTED(w, True);
 			break;
+		case PUZZLE_REDO:
+			movesDsp++;
+			SET_STARTED(w, True);
+			break;
 #if defined(HAVE_MOTIF) && defined(ANIMATE)
-		case HEXAGONS_SPEED:
+		case PUZZLE_SPEED:
 			{
 				int oldDelay, delay;
 
@@ -788,15 +798,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	if (GetFocus()) {
 		if (!widget.hexagons.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			EnterHexagons(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			EnterPuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	} else {
 		if (widget.hexagons.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			LeaveHexagons(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			LeavePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	}
@@ -805,40 +817,45 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			Initialize(&widget, brush);
 			break;
 		case WM_DESTROY:
-			DestroyHexagons(brush);
+			DestroyPuzzle(&widget, brush);
 			break;
 		case WM_SIZE:
-			ResizeHexagons(&widget);
+			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
 		case WM_PAINT:
 			widget.core.hDC = BeginPaint(hWnd, &paint);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ExposeHexagons(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ExposePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 			break;
 		case WM_RBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			RandomizeHexagons(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ClearPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			SelectHexagons(&widget, LOWORD(lParam), HIWORD(lParam));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			SelectPuzzle(&widget, LOWORD(lParam), HIWORD(lParam));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONUP:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ReleaseHexagons(&widget, LOWORD(lParam), HIWORD(lParam));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ReleasePuzzle(&widget, LOWORD(lParam), HIWORD(lParam));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 #if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
 		case WM_MOUSEWHEEL:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
 			{
 				int zDelta = ((short) HIWORD(wParam));
 				POINT cursor, origin;
@@ -847,17 +864,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				ClientToScreen(hWnd, &origin);
 				(void) GetCursorPos(&cursor);
 				if (zDelta > (WHEEL_DELTA >> 1)) {
-					MoveHexagons(&widget, TOP,
+					MovePuzzle(&widget, TOP,
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 					if (GetKeyState(VK_CONTROL) >> 1) {
-						SizeHexagons(&widget);
+						SizePuzzle(&widget);
 						(void) InvalidateRect(hWnd, NULL, TRUE);
 					}
 				} else if (zDelta < -(WHEEL_DELTA >> 1)) {
-					MoveHexagons(&widget, BOTTOM,
+					MovePuzzle(&widget, BOTTOM,
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 					if (GetKeyState(VK_CONTROL) >> 1) {
-						SizeHexagons(&widget);
+						SizePuzzle(&widget);
 						(void) InvalidateRect(hWnd, NULL, TRUE);
 					}
 				}
@@ -868,56 +885,67 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		case WM_COMMAND:
 			switch (LOWORD(wParam)) {
 				case IDM_GET:
-					GetHexagons(&widget);
-					ResizeHexagons(&widget);
+					GetPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_WRITE:
-					WriteHexagons(&widget);
+					WritePuzzle(&widget);
 					break;
 				case IDM_EXIT:
-					DestroyHexagons(brush);
+					DestroyPuzzle(&widget, brush);
 					break;
 				case IDM_HIDE:
-					HideHexagons(&widget);
-					break;
-				case IDM_CLEAR:
-					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					ClearHexagons(&widget);
-					(void) ReleaseDC(hWnd, widget.core.hDC);
+					HidePuzzle(&widget);
 					break;
 				case IDM_UNDO:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					UndoHexagons(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					UndoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_REDO:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RedoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_CLEAR:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					ClearPuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_RANDOMIZE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					RandomizeHexagons(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RandomizePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_SOLVE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					SolveHexagons(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					SolvePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_CORNERS:
-					ModeHexagons(&widget);
-					SizeHexagons(&widget);
+					ModePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SPEED:
-					SpeedHexagons(&widget);
+					SpeedPuzzle(&widget);
 					break;
 				case IDM_SLOW:
-					SlowHexagons(&widget);
+					SlowPuzzle(&widget);
 					break;
 				case IDM_SOUND:
-					SoundHexagons(&widget);
+					SoundPuzzle(&widget);
 					break;
 				case IDM_TR:
 				case IDM_RIGHT:
@@ -926,8 +954,9 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				case IDM_LEFT:
 				case IDM_TL:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					(void) MoveHexagons(&widget,
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					(void) MovePuzzle(&widget,
 						(int) LOWORD(wParam) - IDM_TR, FALSE);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
@@ -935,8 +964,8 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				case IDM_INCX:
 				case IDM_INCY:
 				case IDM_DECX:
-					if (MoveHexagons(&widget, (int) LOWORD(wParam) - IDM_DECY, TRUE)) {
-						SizeHexagons(&widget);
+					if (MovePuzzle(&widget, (int) LOWORD(wParam) - IDM_DECY, TRUE)) {
+						SizePuzzle(&widget);
 						(void) InvalidateRect(hWnd, NULL, TRUE);
 					}
 					break;
@@ -1015,7 +1044,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
 #else
 
 static void
-Usage(char * programName)
+Usage(char *programName)
 {
 	unsigned int i;
 
@@ -1044,10 +1073,6 @@ static XrmOptionDescRec options[] =
 	{(char *) "-border", (char *) "*hexagons.tileBorder", XrmoptionSepArg, NULL},
 	{(char *) "-tile", (char *) "*hexagons.tileColor", XrmoptionSepArg, NULL},
 	{(char *) "-border", (char *) "*hexagons.tileBorder", XrmoptionSepArg, NULL},
-	{(char *) "-sizex", (char *) "*hexagons.sizeX", XrmoptionSepArg, NULL},
-	{(char *) "-sizey", (char *) "*hexagons.sizeY", XrmoptionSepArg, NULL},
-	{(char *) "-corners", (char *) "*hexagons.corners", XrmoptionNoArg, (char *) "TRUE"},
-	{(char *) "-nocorners", (char *) "*hexagons.corners", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-install", (char *) "*hexagons.install", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-noinstall", (char *) "*hexagons.install", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-picture", (char *) "*hexagons.picture", XrmoptionSepArg, NULL},
@@ -1058,6 +1083,10 @@ static XrmOptionDescRec options[] =
 	{(char *) "-base", (char *) "*hexagons.base", XrmoptionSepArg, NULL},
 	{(char *) "-fn", (char *) "*hexagons.font", XrmoptionSepArg, NULL},
 	{(char *) "-font", (char *) "*hexagons.font", XrmoptionSepArg, NULL},
+	{(char *) "-sizex", (char *) "*hexagons.sizeX", XrmoptionSepArg, NULL},
+	{(char *) "-sizey", (char *) "*hexagons.sizeY", XrmoptionSepArg, NULL},
+	{(char *) "-corners", (char *) "*hexagons.corners", XrmoptionNoArg, (char *) "TRUE"},
+	{(char *) "-nocorners", (char *) "*hexagons.corners", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-userName", (char *) "*hexagons.userName", XrmoptionSepArg, NULL},
 	{(char *) "-scoreFile", (char *) "*hexagons.scoreFile", XrmoptionSepArg, NULL},
 	{(char *) "-scores", (char *) "*hexagons.scoreOnly", XrmoptionNoArg, (char *) "TRUE"},
@@ -1066,16 +1095,16 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackHexagonsRandomize(Widget w, XtPointer clientData,
-		XmAnyCallbackStruct * cbs)
+CallbackPuzzleClear(Widget w, XtPointer clientData,
+		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(hexagons, XtNmenu, 5, NULL); /* menu choice */
+		XtVaSetValues(hexagons, XtNmenu, MENU_CLEAR, NULL);
 	}
 }
 
 static void
-TileXSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+TileXSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int sizeX = cbs->value, sizeY, oldX;
 	Boolean corners;
@@ -1097,7 +1126,7 @@ TileXSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-TileYSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+TileYSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int sizeX, sizeY = cbs->value, oldY;
 	Boolean corners;
@@ -1124,7 +1153,7 @@ TileYSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 
 static void
 CornersToggle(Widget w, XtPointer clientData,
-		XmToggleButtonCallbackStruct * cbs)
+		XmToggleButtonCallbackStruct *cbs)
 {
 	int sizeX, sizeY;
 	Boolean corners = cbs->set, oldCorn;
@@ -1151,7 +1180,7 @@ CornersToggle(Widget w, XtPointer clientData,
 
 #ifdef ANIMATE
 static void
-SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MAXSPEED + MINSPEED - cbs->value - 1, oldDelay;
 
@@ -1168,16 +1197,16 @@ fileCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	if (val == 2)
+	if (val == MENU_EXIT)
 		exit(0);
-	XtVaSetValues(hexagons, XtNmenu, val, NULL);
+	XtVaSetValues(hexagons, XtNmenu, val, NULL); /* GWQ */
 }
 
 static void
 playCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
-	XtVaSetValues(hexagons, XtNmenu, val + 3, NULL); /* GWQ */
+	XtVaSetValues(hexagons, XtNmenu, val + MENU_EXIT + 1, NULL);
 }
 
 static Widget
@@ -1275,7 +1304,8 @@ main(int argc, char **argv)
 	Widget movesRowCol, tilesRowCol, speedRowCol, messageRowCol;
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
-	XmString clearString, undoString, randomizeString, solveString;
+	XmString undoString, redoString;
+	XmString clearString, randomizeString, solveString;
 	XmString modeString;
 	XmString speedString, slowString, soundString;
 #endif
@@ -1287,7 +1317,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#if HAVE_XPM
+#ifdef HAVE_XPM
 	{
 		XpmAttributes xpmAttributes;
 		XpmColorSymbol transparentColor[1] = {{NULL,
@@ -1302,18 +1332,18 @@ main(int argc, char **argv)
 		transparentColor[0].pixel = bg;
 		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) hexagons_xpm, &hexagonsIcon, NULL,
+			(char **) hexagons_xpm, &icon, NULL,
 			&xpmAttributes);
 	}
-	if (hexagonsIcon == (Pixmap) NULL)
+	if (icon == (Pixmap) NULL)
 #endif
-		hexagonsIcon = XCreateBitmapFromData(XtDisplay(topLevel),
+		icon = XCreateBitmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
 			(char *) hexagons_bits,
 			hexagons_width, hexagons_height);
 #ifdef HAVE_MOTIF
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, hexagonsIcon,
+		XtNiconPixmap, icon,
 		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
@@ -1340,26 +1370,29 @@ main(int argc, char **argv)
 	XmStringFree(getString);
 	XmStringFree(writeString);
 	XmStringFree(quitString);
-	clearString = XmStringCreateSimple((char *) "Clear");
 	undoString = XmStringCreateSimple((char *) "Undo");
-	randomizeString = XmStringCreateSimple((char *) "Randomize");
+	redoString = XmStringCreateSimple((char *) "Redo");
+	clearString = XmStringCreateSimple((char *) "Clear");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
 	solveString = XmStringCreateSimple((char *) "(Solve)");
 	modeString = XmStringCreateSimple((char *) "Mode");
 	speedString = XmStringCreateSimple((char *) ">Speed");
 	slowString = XmStringCreateSimple((char *) "<Slow");
 	soundString = XmStringCreateSimple((char *) "@Sound");
 	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
-		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
 		XmVaPUSHBUTTON, modeString, 'M', NULL, NULL,
 		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
 		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
 		XmVaPUSHBUTTON, soundString, '@', NULL, NULL,
 		NULL);
-	XmStringFree(clearString);
 	XmStringFree(undoString);
+	XmStringFree(redoString);
+	XmStringFree(clearString);
 	XmStringFree(randomizeString);
 	XmStringFree(solveString);
 	XmStringFree(modeString);
@@ -1406,10 +1439,10 @@ main(int argc, char **argv)
 		(char *) "About");
 	solveDialog = createHelp(menuBar, (char *) solveHelp,
 		(char *) "Solve");
-	randomizeDialog = createQuery(topLevel,
-		(char *) "Are you sure you want to randomize?",
-		(char *) "Randomize Query",
-		(XtCallbackProc) CallbackHexagonsRandomize);
+	clearDialog = createQuery(topLevel,
+		(char *) "Are you sure you want to start over?",
+		(char *) "Clear Query",
+		(XtCallbackProc) CallbackPuzzleClear);
 	mainPanel = XtCreateManagedWidget("mainPanel",
 		xmPanedWindowWidgetClass, menuBarPanel,
 		NULL, 0);
@@ -1455,7 +1488,7 @@ main(int argc, char **argv)
 		(void) XtVaCreateManagedWidget("mouseRightText",
 			xmLabelGadgetClass, movesRowCol,
 			XtVaTypedArg, XmNlabelString,
-			XmRString, "Randomize", 10, NULL);
+			XmRString, "Clear", 6, NULL);
 		(void) XtVaCreateManagedWidget("mouseRight",
 			xmLabelGadgetClass, movesRowCol,
 			XmNlabelType, XmPIXMAP,
@@ -1528,13 +1561,13 @@ main(int argc, char **argv)
 		hexagonsWidgetClass, mainPanel, NULL, 0);
 #else
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, hexagonsIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 	hexagons = XtCreateManagedWidget("hexagons",
 		hexagonsWidgetClass, topLevel, NULL, 0);
 #endif
 	XtAddCallback(hexagons, XtNselectCallback,
-		(XtCallbackProc) CallbackHexagons, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	Initialize(hexagons);
 	XtRealizeWidget(topLevel);
 	XGrabButton(XtDisplay(hexagons), (unsigned int) AnyButton, AnyModifier,
