@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1996 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1996 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -29,12 +29,13 @@
 #define IDM_WRITE	101
 #define IDM_EXIT	102
 #define IDM_HIDE	103
-#define IDM_CLEAR	200
-#define IDM_UNDO	201
-#define IDM_RANDOMIZE	202
-#define IDM_PRACTICE	203
-#define IDM_SOLVE	204
-#define IDM_ORIENT	205
+#define IDM_UNDO	200
+#define IDM_REDO	201
+#define IDM_CLEAR	202
+#define IDM_RANDOMIZE	203
+#define IDM_PRACTICE	204
+#define IDM_SOLVE	205
+#define IDM_ORIENTIZE	206
 #define IDM_TOP		300
 #define IDM_TTR		301
 #define IDM_TR		302
