@@ -87,7 +87,7 @@ static MballRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Mball Version 7.2\n"
+"Mball Version 7.2.4\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -138,13 +138,22 @@ static const char featuresHelp[] = {
 "record should say \"practice\").  This is good for learning "
 "moves and experimenting.\n"
 "\n"
-"Click \"mouse-right\" button, or press \"R\" or \"r\" "
+"Click \"mouse-right\" button, or press \"Z\" or \"z\" "
 "keys, to randomize the puzzle (this must be done first\n"
 "to set a new record).\n"
 "\n"
-"Press \"I\" or \"i\" keys to increase the number of rings.\n"
+"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
 "\n"
-"Press \"D\" or \"d\" keys to decrease the number of rings.\n"
+"Press \"W\" or \"w\" keys to save (write) a puzzle.\n"
+"\n"
+"Press \"U\" or \"u\" keys to undo a move.\n"
+"\n"
+"Press \"R\" or \"r\" keys to redo a move.\n"
+"\n"
+"Press \"C\" or \"c\" keys to clear the puzzle.\n"
+"\n"
+"\"S\" or \"s\" keys reserved for the auto-solver "
+"(not implemented).\n"
 "\n"
 "Press \"O\" or \"o\" keys to toggle the orient mode.  One "
 "has to orient the wedges in orient mode, besides\n"
@@ -152,23 +161,16 @@ static const char featuresHelp[] = {
 "one has to get the numbers to be on the same side\n"
 "of the ball in clockwise order.\n"
 "\n"
+"Press \"I\" or \"i\" keys to increase the number of rings.\n"
+"\n"
+"Press \"D\" or \"d\" keys to decrease the number of rings.\n"
+"\n"
 "Press \"2\", \"4\", \"6\", \"8\", \"0\", or \"=\" keys "
 "(not the keypad 2, 4, 6, 8, 0) to change to 2, 4, 6, 8, 10,\n"
 "or 12 wedges, respectively.  Note: if there were odd "
 "number of wedges, there would be no 180 degree turn and\n"
 "therefore the puzzle would be inoperable.\n"
 "\n"
-"\"S\" or \"s\" keys reserved for the auto-solver "
-"(not implemented).\n"
-"\n"
-"Press \"U\" or \"u\" keys to undo a move.\n"
-"\n"
-"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
-"\n"
-"Press \"W\" or \"w\" keys to save a puzzle.\n"
-"\n"
-"Press \"C\" or \"c\" keys to clear a puzzle.\n"
-"\n"
 "Press \"Esc\" key to hide program.\n"
 "\n"
 "Press \"Q\", \"q\", or \"CTRL-C\" keys to kill program.\n"
@@ -237,12 +239,13 @@ static char titleDsp[TITLELEN] = "";
 #endif
 static char scoreFileName[FILENAMELEN] = SCOREFILE;
 static char fileName[FILENAMELEN];
+static Boolean randomized = False;
 #ifdef WINVER
 #define MAXPROGNAME 80
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap mballIcon = None;
+static Pixmap icon = None;
 static Widget topLevel, mball;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
@@ -489,7 +492,7 @@ Widget w
 	char *userName, *scoreFile;
 
 #ifdef WINVER
-	InitializeMball(w, brush);
+	InitializePuzzle(w, brush);
 
 	wedges = w->mball.wedges;
 	rings = w->mball.rings;
@@ -544,7 +547,7 @@ Widget w
 #ifdef WINVER
 		(void) strncpy(userNameDsp, userName, USERNAMELEN);
 #else
-		char * login = getlogin();
+		char *login = getlogin();
 
 		if (login == NULL) {
 			(void) strcpy(userNameDsp, "");
@@ -569,10 +572,10 @@ Widget w
 
 #ifdef WINVER
 void
-SetMball(MballWidget w, int reason)
+setPuzzle(MballWidget w, int reason)
 #else
 static void
-CallbackMball(Widget w, caddr_t clientData, mballCallbackStruct * callData)
+CallbackPuzzle(Widget w, caddr_t clientData, mballCallbackStruct *callData)
 #endif
 {
 #ifndef WINVER
@@ -599,7 +602,7 @@ CallbackMball(Widget w, caddr_t clientData, mballCallbackStruct * callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-		case MBALL_HIDE:
+		case PUZZLE_HIDE:
 #ifdef WINVER
 			ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -609,24 +612,22 @@ CallbackMball(Widget w, caddr_t clientData, mballCallbackStruct * callData)
 #endif
 			break;
 #ifndef WINVER
-		case MBALL_PRACTICE_QUERY:
+		case PUZZLE_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(practiceDialog);
 #else
-			XtVaSetValues(mball,
-				XtNmenu, 10, NULL); /* menu choice */
+			XtVaSetValues(mball, XtNmenu, MENU_PRACTICE, NULL);
 #endif
 			break;
-		case MBALL_RANDOMIZE_QUERY:
+		case PUZZLE_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(randomizeDialog);
 #else
-			XtVaSetValues(mball,
-				XtNmenu, 5, NULL); /* menu choice */
+			XtVaSetValues(mball, XtNmenu, MENU_RANDOMIZE, NULL);
 #endif
 			break;
 #endif
-		case MBALL_SOLVE_MESSAGE:
+		case PUZZLE_SOLVE_MESSAGE:
 #ifdef WINVER
 			(void) MessageBox(w->core.hWnd, solveHelp,
 				"Auto-Solve", MB_OK);
@@ -638,7 +639,7 @@ CallbackMball(Widget w, caddr_t clientData, mballCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case MBALL_RESTORE:
+		case PUZZLE_RESTORE:
 			if (practice) {
 				(void) strncpy(recordDsp, "practice",
 					MESSAGELEN);
@@ -647,29 +648,31 @@ CallbackMball(Widget w, caddr_t clientData, mballCallbackStruct * callData)
 #endif
 			}
 			movesDsp = 0;
+			randomized = False;
 			break;
-		case MBALL_RESET:
+		case PUZZLE_RESET:
 			movesDsp = 0;
+			randomized = False;
 			break;
-		case MBALL_AMBIGUOUS:
+		case PUZZLE_AMBIGUOUS:
 			(void) strncpy(messageDsp, "Ambiguous move",
 				MESSAGELEN);
 			break;
-		case MBALL_ILLEGAL:
-			if (practice || start)
+		case PUZZLE_ILLEGAL:
+			if (practice || randomized)
 				(void) strncpy(messageDsp, "Illegal move",
 					MESSAGELEN);
 			else
-				(void) strncpy(messageDsp, "Randomize to start",
-					MESSAGELEN);
+				(void) strncpy(messageDsp,
+					"Randomize to start", MESSAGELEN);
 			break;
-		case MBALL_MOVED:
+		case PUZZLE_MOVED:
 			movesDsp++;
 			SET_STARTED(w, True);
 			break;
-		case MBALL_CONTROL:
+		case PUZZLE_CONTROL:
 			return;
-		case MBALL_SOLVED:
+		case PUZZLE_SOLVED:
 			if (practice)
 				movesDsp = 0;
 			else if (cheat)
@@ -682,9 +685,11 @@ CallbackMball(Widget w, caddr_t clientData, mballCallbackStruct * callData)
 				(void) strncpy(messageDsp, "Solved!",
 					MESSAGELEN);
 			SET_STARTED(w, False);
+			randomized = False;
 			break;
-		case MBALL_PRACTICE:
+		case PUZZLE_PRACTICE:
 			movesDsp = 0;
+			randomized = False;
 			practice = !practice;
 			if (!practice)
 				(void) strncpy(messageDsp, "Randomize to start",
@@ -702,8 +707,9 @@ CallbackMball(Widget w, caddr_t clientData, mballCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case MBALL_RANDOMIZE:
+		case PUZZLE_RANDOMIZE:
 			movesDsp = 0;
+			randomized = True;
 #ifdef WINVER
 			w->mball.practice = False;
 			w->mball.started = False;
@@ -713,7 +719,7 @@ CallbackMball(Widget w, caddr_t clientData, mballCallbackStruct * callData)
 				XtNstart, False, NULL);
 #endif
 			break;
-		case MBALL_DEC:
+		case PUZZLE_DEC:
 			movesDsp = 0;
 			rings--;
 			PrintRecord(wedges, rings, orient, practice);
@@ -728,7 +734,7 @@ CallbackMball(Widget w, caddr_t clientData, mballCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case MBALL_ORIENT:
+		case PUZZLE_ORIENTIZE:
 			movesDsp = 0;
 			orient = !orient;
 			PrintRecord(wedges, rings, orient, practice);
@@ -741,7 +747,7 @@ CallbackMball(Widget w, caddr_t clientData, mballCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case MBALL_INC:
+		case PUZZLE_INC:
 			movesDsp = 0;
 			rings++;
 			PrintRecord(wedges, rings, orient, practice);
@@ -756,14 +762,14 @@ CallbackMball(Widget w, caddr_t clientData, mballCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case MBALL_WEDGE2:
-		case MBALL_WEDGE4:
-		case MBALL_WEDGE6:
-		case MBALL_WEDGE8:
-		case MBALL_WEDGE10:
-		case MBALL_WEDGE12:
+		case PUZZLE_WEDGE2:
+		case PUZZLE_WEDGE4:
+		case PUZZLE_WEDGE6:
+		case PUZZLE_WEDGE8:
+		case PUZZLE_WEDGE10:
+		case PUZZLE_WEDGE12:
 			movesDsp = 0;
-			wedges = 2 * (reason - MBALL_WEDGE2) +
+			wedges = 2 * (reason - PUZZLE_WEDGE2) +
 				MINWEDGES;
 			PrintRecord(wedges, rings, orient, practice);
 #ifdef WINVER
@@ -776,13 +782,17 @@ CallbackMball(Widget w, caddr_t clientData, mballCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case MBALL_COMPUTED:
+		case PUZZLE_COMPUTED:
 			SET_STARTED(w, False);
 			break;
-		case MBALL_UNDO:
+		case PUZZLE_UNDO:
 			movesDsp--;
 			SET_STARTED(w, True);
 			break;
+		case PUZZLE_REDO:
+			movesDsp++;
+			SET_STARTED(w, True);
+			break;
 	}
 	PrintStatus(messageDsp, movesDsp
 #ifndef HAVE_MOTIF
@@ -816,15 +826,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	if (GetFocus()) {
 		if (!widget.mball.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			EnterMball(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			EnterPuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	} else {
 		if (widget.mball.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			LeaveMball(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			LeavePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	}
@@ -833,42 +845,47 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			Initialize(&widget, brush);
 			break;
 		case WM_DESTROY:
-			DestroyMball(brush);
+			DestroyPuzzle(brush);
 			break;
 		case WM_SIZE:
-			ResizeMball(&widget);
+			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
 		case WM_PAINT:
 			widget.core.hDC = BeginPaint(hWnd, &paint);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ExposeMball(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ExposePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 			break;
 		case WM_RBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			RandomizeMball(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			SelectMball(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			SelectPuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONUP:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ReleaseMball(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ReleasePuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 #if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
 		case WM_MOUSEWHEEL:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
 			{
 				int zDelta = ((short) HIWORD(wParam));
 				POINT cursor, origin;
@@ -877,12 +894,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				ClientToScreen(hWnd, &origin);
 				(void) GetCursorPos(&cursor);
 				if (zDelta > (WHEEL_DELTA >> 1)) {
-					MoveMballInput(&widget,
+					MovePuzzleInput(&widget,
 						cursor.x - origin.x, cursor.y - origin.y,
 						TOP,
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 				} else if (zDelta < -(WHEEL_DELTA >> 1)) {
-					MoveMballInput(&widget,
+					MovePuzzleInput(&widget,
 						cursor.x - origin.x, cursor.y - origin.y,
 						BOTTOM,
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
@@ -894,51 +911,62 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		case WM_COMMAND:
 			switch (LOWORD(wParam)) {
 				case IDM_GET:
-					GetMball(&widget);
-					ResizeMball(&widget);
+					GetPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_WRITE:
-					WriteMball(&widget);
+					WritePuzzle(&widget);
 					break;
 				case IDM_EXIT:
-					DestroyMball(brush);
+					DestroyPuzzle(brush);
 					break;
 				case IDM_HIDE:
-					HideMball(&widget);
-					break;
-				case IDM_CLEAR:
-					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					ClearMball(&widget);
-					(void) ReleaseDC(hWnd, widget.core.hDC);
+					HidePuzzle(&widget);
 					break;
 				case IDM_UNDO:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					UndoMball(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					UndoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_REDO:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RedoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_CLEAR:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					ClearPuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_RANDOMIZE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					RandomizeMball(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RandomizePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_PRACTICE:
-					PracticeMball(&widget);
-					SizeMball(&widget);
+					PracticePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SOLVE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					SolveMball(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					SolvePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
-				case IDM_ORIENT:
-					OrientizeMball(&widget);
-					SizeMball(&widget);
+				case IDM_ORIENTIZE:
+					OrientizePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_TOP:
@@ -959,11 +987,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
-						(void) MoveMballInput(&widget,
+						(void) MovePuzzleInput(&widget,
 							cursor.x - origin.x, cursor.y - origin.y,
 							(int) LOWORD(wParam) - IDM_TOP, FALSE);
 						(void) ReleaseDC(hWnd, widget.core.hDC);
@@ -988,11 +1017,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
-						(void) MoveMballInput(&widget,
+						(void) MovePuzzleInput(&widget,
 							cursor.x - origin.x, cursor.y - origin.y,
 							(int) LOWORD(wParam) - IDM_CONTROL_TOP, TRUE);
 						(void) ReleaseDC(hWnd, widget.core.hDC);
@@ -1000,14 +1030,14 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 
 					break;
 				case IDM_DEC:
-					if (DecrementMball(&widget)) {
-						SizeMball(&widget);
+					if (DecrementPuzzle(&widget)) {
+						SizePuzzle(&widget);
 						(void) InvalidateRect(hWnd, NULL, TRUE);
 					}
 					break;
 				case IDM_INC:
-					IncrementMball(&widget);
-					SizeMball(&widget);
+					IncrementPuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_WEDGE2:
@@ -1016,8 +1046,8 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				case IDM_WEDGE8:
 				case IDM_WEDGE10:
 				case IDM_WEDGE12:
-					WedgeMball(&widget, (int) LOWORD(wParam) - IDM_WEDGE2);
-					SizeMball(&widget);
+					WedgePuzzle(&widget, (int) LOWORD(wParam) - IDM_WEDGE2);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_ABOUT:
@@ -1095,7 +1125,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
 #else
 
 static void
-Usage(char * programName)
+Usage(char *programName)
 {
 	unsigned int i;
 
@@ -1134,6 +1164,8 @@ static XrmOptionDescRec options[] =
 	{(char *) "-wedge9", (char *) "*mball.wedgeColor9", XrmoptionSepArg, NULL},
 	{(char *) "-wedge10", (char *) "*mball.wedgeColor10", XrmoptionSepArg, NULL},
 	{(char *) "-wedge11", (char *) "*mball.wedgeColor11", XrmoptionSepArg, NULL},
+	{(char *) "-fn", (char *) "*mball.font", XrmoptionSepArg, NULL},
+	{(char *) "-font", (char *) "*mball.font", XrmoptionSepArg, NULL},
 	{(char *) "-wedges", (char *) "*mball.wedges", XrmoptionSepArg, NULL},
 	{(char *) "-rings", (char *) "*mball.rings", XrmoptionSepArg, NULL},
 	{(char *) "-orient", (char *) "*mball.orient", XrmoptionNoArg, (char *) "TRUE"},
@@ -1141,8 +1173,6 @@ static XrmOptionDescRec options[] =
 	{(char *) "-practice", (char *) "*mball.practice", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-nopractice", (char *) "*mball.practice", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-base", (char *) "*mball.base", XrmoptionSepArg, NULL},
-	{(char *) "-fn", (char *) "*mball.font", XrmoptionSepArg, NULL},
-	{(char *) "-font", (char *) "*mball.font", XrmoptionSepArg, NULL},
 	{(char *) "-userName", (char *) "*mball.userName", XrmoptionSepArg, NULL},
 	{(char *) "-scoreFile", (char *) "*mball.scoreFile", XrmoptionSepArg, NULL},
 	{(char *) "-scores", (char *) "*mball.scoreOnly", XrmoptionNoArg, (char *) "TRUE"},
@@ -1151,23 +1181,23 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackMballPractice(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzlePractice(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(mball, XtNmenu, 10, NULL); /* menu choice */
+		XtVaSetValues(mball, XtNmenu, MENU_PRACTICE, NULL);
 	}
 }
 
 static void
-CallbackMballRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzleRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(mball, XtNmenu, 5, NULL); /* menu choice */
+		XtVaSetValues(mball, XtNmenu, MENU_RANDOMIZE, NULL);
 	}
 }
 
 static void
-WedgeToggle(Widget w, int bit, XmToggleButtonCallbackStruct * cbs)
+WedgeToggle(Widget w, int bit, XmToggleButtonCallbackStruct *cbs)
 {
 	int wedges, rings;
 	Boolean orient, practice;
@@ -1190,7 +1220,7 @@ WedgeToggle(Widget w, int bit, XmToggleButtonCallbackStruct * cbs)
 }
 
 static void
-RingSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+RingSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int wedges, rings = cbs->value, old;
 	Boolean orient, practice;
@@ -1213,7 +1243,7 @@ RingSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	int wedges, rings;
 	Boolean orient = cbs->set, practice, oldOrient;
@@ -1236,7 +1266,7 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
 }
 
 static void
-PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	int wedges, rings;
 	Boolean orient, practice = cbs->set, oldPract;
@@ -1267,9 +1297,9 @@ fileCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	if (val == 2)
+	if (val == MENU_EXIT)
 		exit(0);
-	XtVaSetValues(mball, XtNmenu, val, NULL);
+	XtVaSetValues(mball, XtNmenu, val, NULL); /* GWQ */
 }
 
 static void
@@ -1277,7 +1307,7 @@ playCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	XtVaSetValues(mball, XtNmenu, val + 3, NULL); /* GWQ */
+	XtVaSetValues(mball, XtNmenu, val + MENU_EXIT + 1, NULL);
 }
 
 static Widget
@@ -1376,9 +1406,9 @@ main(int argc, char **argv)
 	Widget messageRowCol;
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
-	XmString clearString, undoString, randomizeString, solveString;
-	XmString incrementString, decrementString;
-	XmString orientString, practiceString;
+	XmString undoString, redoString;
+	XmString clearString, randomizeString, practiceString, solveString;
+	XmString orientString,incrementString, decrementString;
 	unsigned int i;
 #endif
 
@@ -1388,7 +1418,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#if HAVE_XPM
+#ifdef HAVE_XPM
 	{
 		XpmAttributes xpmAttributes;
 		XpmColorSymbol transparentColor[1] = {{NULL,
@@ -1403,18 +1433,18 @@ main(int argc, char **argv)
 		transparentColor[0].pixel = bg;
 		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) mball_xpm, &mballIcon, NULL,
+			(char **) mball_xpm, &icon, NULL,
 			&xpmAttributes);
 	}
-	if (mballIcon == (Pixmap) NULL)
+	if (icon == (Pixmap) NULL)
 #endif
-		mballIcon = XCreateBitmapFromData(XtDisplay(topLevel),
+		icon = XCreateBitmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
 			(char *) mball_bits,
 			mball_width, mball_height);
 #ifdef HAVE_MOTIF
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, mballIcon,
+		XtNiconPixmap, icon,
 		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
@@ -1441,32 +1471,35 @@ main(int argc, char **argv)
 	XmStringFree(getString);
 	XmStringFree(writeString);
 	XmStringFree(quitString);
-	clearString = XmStringCreateSimple((char *) "Clear");
 	undoString = XmStringCreateSimple((char *) "Undo");
-	randomizeString = XmStringCreateSimple((char *) "Randomize");
+	redoString = XmStringCreateSimple((char *) "Redo");
+	clearString = XmStringCreateSimple((char *) "Clear");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
+	practiceString = XmStringCreateSimple((char *) "Practice");
 	solveString = XmStringCreateSimple((char *) "(Solve)");
+	orientString = XmStringCreateSimple((char *) "Orientize");
 	incrementString = XmStringCreateSimple((char *) "Increment");
 	decrementString = XmStringCreateSimple((char *) "Decrement");
-	orientString = XmStringCreateSimple((char *) "Orientize");
-	practiceString = XmStringCreateSimple((char *) "Practice");
 	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
-		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
+		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
+		XmVaPUSHBUTTON, orientString, 'O', NULL, NULL,
 		XmVaPUSHBUTTON, incrementString, 'I', NULL, NULL,
 		XmVaPUSHBUTTON, decrementString, 'D', NULL, NULL,
-		XmVaPUSHBUTTON, orientString, 'O', NULL, NULL,
-		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		NULL);
-	XmStringFree(clearString);
 	XmStringFree(undoString);
+	XmStringFree(redoString);
+	XmStringFree(clearString);
 	XmStringFree(randomizeString);
+	XmStringFree(practiceString);
 	XmStringFree(solveString);
 	XmStringFree(incrementString);
 	XmStringFree(decrementString);
 	XmStringFree(orientString);
-	XmStringFree(practiceString);
 	pullDownMenu = XmCreatePulldownMenu(menuBar,
 		(char *) "helpPullDown", NULL, 0);
 	widget = XtVaCreateManagedWidget("Help",
@@ -1510,11 +1543,11 @@ main(int argc, char **argv)
 	practiceDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to toggle the practice mode?",
 		(char *) "Practice Query",
-		(XtCallbackProc) CallbackMballPractice);
+		(XtCallbackProc) CallbackPuzzlePractice);
 	randomizeDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to randomize?",
 		(char *) "Randomize Query",
-		(XtCallbackProc) CallbackMballRandomize);
+		(XtCallbackProc) CallbackPuzzleRandomize);
 	mainPanel = XtCreateManagedWidget("mainPanel",
 		xmPanedWindowWidgetClass, menuBarPanel,
 		NULL, 0);
@@ -1630,13 +1663,13 @@ main(int argc, char **argv)
 		mballWidgetClass, mainPanel, NULL, 0);
 #else
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, mballIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 	mball = XtCreateManagedWidget("mball",
 		mballWidgetClass, topLevel, NULL, 0);
 #endif
 	XtAddCallback(mball, XtNselectCallback,
-		(XtCallbackProc) CallbackMball, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	Initialize(mball);
 	XtRealizeWidget(topLevel);
 	XGrabButton(XtDisplay(mball), (unsigned int) AnyButton, AnyModifier,
