@@ -96,29 +96,45 @@ typedef struct {
 } mlinkCallbackStruct;
 #endif
 
-#define MLINK_HIDE (-9)
-#define MLINK_PRACTICE_QUERY (-8)
-#define MLINK_RANDOMIZE_QUERY (-7)
-#define MLINK_SOLVE_MESSAGE (-6)
-#define MLINK_RESTORE (-5)
-#define MLINK_RESET (-4)
-#define MLINK_BLOCKED (-3)
-#define MLINK_SPACE (-2)
-#define MLINK_IGNORE (-1)
-#define MLINK_MOVED 0
-#define MLINK_SOLVED 1
-#define MLINK_CONTROL 2
-#define MLINK_PRACTICE 3
-#define MLINK_RANDOMIZE 4
-#define MLINK_ORIENT 5
-#define MLINK_MIDDLE 6
-#define MLINK_DEC_X 7
-#define MLINK_INC_X 8
-#define MLINK_DEC_Y 9
-#define MLINK_INC_Y 10
-#define MLINK_COMPUTED 11
-#define MLINK_UNDO 12
-#define MLINK_SPEED 13
+#define PUZZLE_HIDE (-9)
+#define PUZZLE_PRACTICE_QUERY (-8)
+#define PUZZLE_RANDOMIZE_QUERY (-7)
+#define PUZZLE_SOLVE_MESSAGE (-6)
+#define PUZZLE_RESTORE (-5)
+#define PUZZLE_RESET (-4)
+#define PUZZLE_BLOCKED (-3)
+#define PUZZLE_SPACE (-2)
+#define PUZZLE_IGNORE (-1)
+#define PUZZLE_MOVED 0
+#define PUZZLE_SOLVED 1
+#define PUZZLE_CONTROL 2
+#define PUZZLE_PRACTICE 3
+#define PUZZLE_RANDOMIZE 4
+#define PUZZLE_ORIENTIZE 5
+#define PUZZLE_MIDDLE 6
+#define PUZZLE_DEC_X 7
+#define PUZZLE_INC_X 8
+#define PUZZLE_DEC_Y 9
+#define PUZZLE_INC_Y 10
+#define PUZZLE_COMPUTED 11
+#define PUZZLE_UNDO 12
+#define PUZZLE_REDO 13
+#define PUZZLE_SPEED 14
+
+#define MENU_GET 0
+#define MENU_WRITE 1
+#define MENU_EXIT 2
+#define MENU_UNDO 3
+#define MENU_REDO 4
+#define MENU_CLEAR 5
+#define MENU_RANDOMIZE 6
+#define MENU_PRACTICE 7
+#define MENU_SOLVE 8
+#define MENU_ORIENTIZE 9
+#define MENU_MIDDLE 10
+#define MENU_SPEED 11
+#define MENU_SLOW 12
+#define MENU_SOUND 13
 
 #define MINFACES 1
 #define MAXFACES 8
