@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1994 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -33,39 +33,36 @@
 #include "Mlink.h"
 
 #ifdef WINVER
-extern void DestroyMlink(HBRUSH brush);
-extern void ResizeMlink(MlinkWidget w);
-extern void SizeMlink(MlinkWidget w);
-extern void InitializeMlink(MlinkWidget w, HBRUSH brush);
-extern void ExposeMlink(MlinkWidget w);
-extern void HideMlink(MlinkWidget w);
-extern void SelectMlink(MlinkWidget w,
+extern void DestroyPuzzle(MlinkWidget w, HBRUSH brush);
+extern void ResizePuzzle(MlinkWidget w);
+extern void SizePuzzle(MlinkWidget w);
+extern void InitializePuzzle(MlinkWidget w, HBRUSH brush);
+extern void ExposePuzzle(MlinkWidget w);
+extern void HidePuzzle(MlinkWidget w);
+extern void SelectPuzzle(MlinkWidget w,
 	const int x, const int y, const int shift);
-extern void MotionMlink(MlinkWidget w,
+extern void MotionPuzzle(MlinkWidget w,
 	const int x, const int y, const int shift);
-extern void ReleaseMlink(MlinkWidget w,
+extern void ReleasePuzzle(MlinkWidget w,
 	const int x, const int y, const int shift);
-extern void PracticeMlink(MlinkWidget w);
-extern void PracticeMlinkMaybe(MlinkWidget w);
-extern void PracticeMlink2(MlinkWidget w);
-extern void RandomizeMlink(MlinkWidget w);
-extern void RandomizeMlinkMaybe(MlinkWidget w);
-extern void RandomizeMlink2(MlinkWidget w);
-extern void ClearMlink(MlinkWidget w);
-extern void GetMlink(MlinkWidget w);
-extern void WriteMlink(MlinkWidget w);
-extern void UndoMlink(MlinkWidget w);
-extern void SolveMlink(MlinkWidget w);
-extern void OrientizeMlink(MlinkWidget w);
-extern void MiddleMlink(MlinkWidget w);
-extern void SpeedMlink(MlinkWidget w);
-extern void SlowMlink(MlinkWidget w);
-extern void SoundMlink(MlinkWidget w);
-extern void EnterMlink(MlinkWidget w);
-extern void LeaveMlink(MlinkWidget w);
-extern void MoveMlinkInput(MlinkWidget w,
+extern void PracticePuzzle(MlinkWidget w);
+extern void RandomizePuzzle(MlinkWidget w);
+extern void GetPuzzle(MlinkWidget w);
+extern void WritePuzzle(MlinkWidget w);
+extern void UndoPuzzle(MlinkWidget w);
+extern void RedoPuzzle(MlinkWidget w);
+extern void ClearPuzzle(MlinkWidget w);
+extern void SolvePuzzle(MlinkWidget w);
+extern void OrientizePuzzle(MlinkWidget w);
+extern void MiddlePuzzle(MlinkWidget w);
+extern void SpeedPuzzle(MlinkWidget w);
+extern void SlowPuzzle(MlinkWidget w);
+extern void SoundPuzzle(MlinkWidget w);
+extern void EnterPuzzle(MlinkWidget w);
+extern void LeavePuzzle(MlinkWidget w);
+extern void MovePuzzleInput(MlinkWidget w,
 	int x, int direction, int shift, int control);
-extern int  MoveMlinkDir(MlinkWidget w, const int direction);
+extern int  MovePuzzleDir(MlinkWidget w, const int direction);
 
 #else
 
@@ -103,6 +100,24 @@ extern MlinkClassRec mlinkClassRec;
 #define DOUBLE 2
 #define INSTANT 3
 
+typedef struct _MoveRecord {
+#if 0
+	int direction, control;
+#endif
+	unsigned char packed;	/* This makes assumptions on the data. */
+        int tile;		/* Do not make assumptions on this one. */
+} MoveRecord;
+
+typedef struct _MoveStack {
+	MoveRecord move;
+	struct _MoveStack *previous, *next;
+} MoveStack;
+
+typedef struct _MlinkStack {
+	MoveStack *currMove, *lastMove, *firstMove;
+	int count;
+} MlinkStack;
+
 typedef struct _MlinkPart {
 	int        *tileOfPosition, spacePosition;
 	int         currentTile, currentFace, currentRef;
@@ -145,26 +160,27 @@ typedef struct _MlinkRec {
 } MlinkRec;
 
 extern int *startPosition;
-extern void SetMlink(MlinkWidget w, int reason);
-extern Boolean MoveMlink(MlinkWidget w,
+extern void setPuzzle(MlinkWidget w, int reason);
+extern Boolean MovePuzzle(MlinkWidget w,
 	const int direction, const int tile, const int shift,
 	const Boolean motion, const int fast);
-extern void MoveMlinkDelay(MlinkWidget w,
+extern void MovePuzzleDelay(MlinkWidget w,
 	const int direction, const int tile, const Boolean all);
 
 extern void SolveSomeTiles(MlinkWidget w);
 extern void DrawAllTiles(const MlinkWidget w);
 extern Boolean CheckSolved(const MlinkWidget w);
-extern void InitMoves(void);
-extern void PutMove(int direction, int tile, int control);
-extern void GetMove(int *direction, int *tile, int *control);
-extern int  MadeMoves(void);
-extern void FlushMoves(MlinkWidget w);
-extern int  NumMoves(void);
-extern void ScanMoves(FILE * fp, MlinkWidget w, int moves);
-extern void PrintMoves(FILE * fp);
-extern void ScanStartPosition(FILE * fp, MlinkWidget w);
-extern void PrintStartPosition(FILE * fp, MlinkWidget w);
-extern void SetStartPosition(MlinkWidget w);
+extern void newMoves(MlinkStack *s);
+extern void deleteMoves(MlinkStack *s);
+extern void setMove(MlinkStack *s, int direction, int tile, int control);
+extern void getMove(MlinkStack *s, int *direction, int *tile, int *control);
+extern int madeMoves(MlinkStack *s);
+extern void flushMoves(MlinkWidget w, MlinkStack *s, Boolean undo);
+extern int numMoves(MlinkStack *s);
+extern void scanMoves(FILE *fp, MlinkWidget w, int moves);
+extern void printMoves(FILE *fp, MlinkStack *s);
+extern void scanStartPosition(FILE *fp, MlinkWidget w);
+extern void printStartPosition(FILE *fp, MlinkWidget w);
+extern void setStartPosition(MlinkWidget w);
 
 #endif /* _MlinkP_h */
