@@ -139,7 +139,7 @@ MoveTilePiece(MlinkWidget w,
         if (SolvingFlag && AbortSolvingFlag)
                 longjmp(solve_env, 1);
 #endif
-        MoveMlinkDelay(w, direction, tile, all);
+        MovePuzzleDelay(w, direction, tile, all);
 }
 
 static int
@@ -1801,7 +1801,7 @@ SolveRight(MlinkWidget w)
 void
 SolveSomeTiles(MlinkWidget w)
 {
-	SetMlink(w, MLINK_RESET);
+	setPuzzle(w, PUZZLE_RESET);
 	if (SolvingFlag)
 		return;
 
@@ -1826,5 +1826,5 @@ SolveSomeTiles(MlinkWidget w)
 #endif
 	SolvingFlag = False;
 	w->mlink.cheat = True; /* Assume the worst. */
-	SetMlink(w, MLINK_COMPUTED);
+	setPuzzle(w, PUZZLE_COMPUTED);
 }
