@@ -86,7 +86,7 @@ static MlinkRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Mlink Version 7.2.1\n"
+"Mlink Version 7.2.4\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -128,8 +128,22 @@ static const char featuresHelp[] = {
 "record should say \"practice\").  This is good for learning "
 "moves and experimenting.\n"
 "\n"
-"Click \"mouse-right\" button, or press \"R\" or \"r\" "
-"keys, to randomize the puzzle.\n"
+"Click \"mouse-right\" button, or press \"Z\" or \"z\" "
+"keys, to randomize the puzzle (this must be done first\n"
+"to set a new record).\n"
+"\n"
+"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
+"\n"
+"Press \"W\" or \"w\" keys to save (write) a puzzle.\n"
+"\n"
+"Press \"U\" or \"u\" keys to undo a move.\n"
+"\n"
+"Press \"R\" or \"r\" keys to redo a move.\n"
+"\n"
+"Press \"C\" or \"c\" keys to clear the puzzle.\n"
+"\n"
+"\"S\" or \"s\" keys reserved for the auto-solver "
+"(not implemented).\n"
 "\n"
 "Press \"O\" or \"o\" keys to toggle orient mode.  Slightly "
 "more challenging on (default is off).\n"
@@ -138,17 +152,6 @@ static const char featuresHelp[] = {
 "the middle tiles rotate together.  This is considerably\n"
 "more challenging on (default is on).\n"
 "\n"
-"\"S\" or \"s\" keys reserved for the auto-solver "
-"(not implemented).\n"
-"\n"
-"Press \"U\" or \"u\" keys to undo a move.\n"
-"\n"
-"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
-"\n"
-"Press \"W\" or \"w\" keys to save a puzzle.\n"
-"\n"
-"Press \"C\" or \"c\" keys to clear a puzzle.\n"
-"\n"
 "Press \">\" or \".\" keys to speed up the movement of tiles.\n"
 "\n"
 "Press \"<\" or \",\" keys to slow down the movement of tiles.\n"
@@ -232,7 +235,7 @@ static char fileName[FILENAMELEN];
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap mlinkIcon = None;
+static Pixmap icon = None;
 static Widget topLevel, mlink;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
@@ -487,7 +490,7 @@ Widget w
 	char *userName, *scoreFile;
 
 #ifdef WINVER
-	InitializeMlink(w, brush);
+	InitializePuzzle(w, brush);
 
 	tiles = w->mlink.tiles;
 	faces = w->mlink.faces;
@@ -551,7 +554,7 @@ Widget w
 #ifdef WINVER
 		(void) strncpy(userNameDsp, userName, USERNAMELEN);
 #else
-		char * login = getlogin();
+		char *login = getlogin();
 
 		if (login == NULL) {
 			(void) strcpy(userNameDsp, "");
@@ -576,10 +579,10 @@ Widget w
 
 #ifdef WINVER
 void
-SetMlink(MlinkWidget w, int reason)
+setPuzzle(MlinkWidget w, int reason)
 #else
 static void
-CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
+CallbackPuzzle(Widget w, caddr_t clientData, mlinkCallbackStruct *callData)
 #endif
 {
 #ifndef WINVER
@@ -606,7 +609,7 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-		case MLINK_HIDE:
+		case PUZZLE_HIDE:
 #ifdef WINVER
 			ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -616,24 +619,22 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 #endif
 			break;
 #ifndef WINVER
-		case MLINK_PRACTICE_QUERY:
+		case PUZZLE_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(practiceDialog);
 #else
-			XtVaSetValues(mlink,
-				XtNmenu, 9, NULL); /* menu choice */
+			XtVaSetValues(mlink, XtNmenu, MENU_PRACTICE, NULL);
 #endif
 			break;
-		case MLINK_RANDOMIZE_QUERY:
+		case PUZZLE_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(randomizeDialog);
 #else
-			XtVaSetValues(mlink,
-				XtNmenu, 5, NULL); /* menu choice */
+			XtVaSetValues(mlink, XtNmenu, MENU_RANDOMIZE, NULL);
 #endif
 			break;
 #endif
-		case MLINK_SOLVE_MESSAGE:
+		case PUZZLE_SOLVE_MESSAGE:
 #ifdef WINVER
 			(void) MessageBox(w->core.hWnd, solveHelp,
 				"Auto-Solve", MB_OK);
@@ -645,7 +646,7 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case MLINK_RESTORE:
+		case PUZZLE_RESTORE:
 			if (practice) {
 				(void) strncpy(recordDsp, "practice", MESSAGELEN);
 #ifdef HAVE_MOTIF
@@ -654,30 +655,30 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 			}
 			movesDsp = 0;
 			break;
-		case MLINK_RESET:
+		case PUZZLE_RESET:
 			movesDsp = 0;
 			break;
-		case MLINK_BLOCKED:
+		case PUZZLE_BLOCKED:
 			(void) strncpy(messageDsp, "Blocked", MESSAGELEN);
 			break;
-		case MLINK_SPACE:
+		case PUZZLE_SPACE:
 #if 0
 			/* Too annoying */
 			(void) strncpy(messageDsp, "A space can not slide",
 				MESSAGELEN);
 #endif
 			break;
-		case MLINK_IGNORE:
+		case PUZZLE_IGNORE:
 			(void) strncpy(messageDsp, "Randomize to start",
 				MESSAGELEN);
 			break;
-		case MLINK_MOVED:
+		case PUZZLE_MOVED:
 			movesDsp++;
 			SET_STARTED(w, True);
 			break;
-		case MLINK_CONTROL:
+		case PUZZLE_CONTROL:
 			return;
-		case MLINK_SOLVED:
+		case PUZZLE_SOLVED:
 			if (practice)
 				movesDsp = 0;
 			else if (cheat)
@@ -692,7 +693,7 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 					MESSAGELEN);
 			SET_STARTED(w, False);
 			break;
-		case MLINK_PRACTICE:
+		case PUZZLE_PRACTICE:
 			movesDsp = 0;
 			practice = !practice;
 			if (!practice)
@@ -711,7 +712,7 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 			XmToggleButtonSetState(practiceSwitch, practice, True);
 #endif
 			break;
-		case MLINK_RANDOMIZE:
+		case PUZZLE_RANDOMIZE:
 			movesDsp = 0;
 #ifdef WINVER
 			w->mlink.practice = False;
@@ -722,7 +723,7 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 				XtNstart, False, NULL);
 #endif
 			break;
-		case MLINK_ORIENT:
+		case PUZZLE_ORIENTIZE:
 			movesDsp = 0;
 			orient = !orient;
 			PrintRecord(tiles, faces, orient, middle, practice);
@@ -735,7 +736,7 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case MLINK_MIDDLE:
+		case PUZZLE_MIDDLE:
 			movesDsp = 0;
 			middle = !middle;
 			PrintRecord(tiles, faces, orient, middle, practice);
@@ -748,7 +749,7 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case MLINK_DEC_X:
+		case PUZZLE_DEC_X:
 			movesDsp = 0;
 			tiles--;
 			PrintRecord(tiles, faces, orient, middle, practice);
@@ -763,7 +764,7 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case MLINK_INC_X:
+		case PUZZLE_INC_X:
 			movesDsp = 0;
 			tiles++;
 			PrintRecord(tiles, faces, orient, middle, practice);
@@ -778,7 +779,7 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case MLINK_DEC_Y:
+		case PUZZLE_DEC_Y:
 			movesDsp = 0;
 			faces--;
 			PrintRecord(tiles, faces, orient, middle, practice);
@@ -791,7 +792,7 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case MLINK_INC_Y:
+		case PUZZLE_INC_Y:
 			movesDsp = 0;
 			faces++;
 			PrintRecord(tiles, faces, orient, middle, practice);
@@ -804,15 +805,19 @@ CallbackMlink(Widget w, caddr_t clientData, mlinkCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case MLINK_COMPUTED:
+		case PUZZLE_COMPUTED:
 			SET_STARTED(w, False);
 			break;
-		case MLINK_UNDO:
+		case PUZZLE_UNDO:
 			movesDsp--;
 			SET_STARTED(w, True);
 			break;
+		case PUZZLE_REDO:
+			movesDsp++;
+			SET_STARTED(w, True);
+			break;
 #ifdef HAVE_MOTIF
-		case MLINK_SPEED:
+		case PUZZLE_SPEED:
 			{
 				int oldDelay, delay;
 
@@ -865,15 +870,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	if (GetFocus()) {
 		if (!widget.mlink.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			EnterMlink(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			EnterPuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	} else {
 		if (widget.mlink.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			LeaveMlink(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			LeavePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	}
@@ -882,29 +889,32 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			Initialize(&widget, brush);
 			break;
 		case WM_DESTROY:
-			DestroyMlink(brush);
+			DestroyPuzzle(&widget, brush);
 			break;
 		case WM_SIZE:
-			ResizeMlink(&widget);
+			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
 		case WM_PAINT:
 			widget.core.hDC = BeginPaint(hWnd, &paint);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ExposeMlink(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ExposePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 			break;
 		case WM_RBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			RandomizeMlink(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONDOWN:
 			mousePressed = True;
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			SelectMlink(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			SelectPuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
@@ -912,30 +922,34 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			if (!(wParam & MK_LBUTTON)) {
 				mousePressed = False;
 				widget.core.hDC = GetDC(hWnd);
-				(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-				ReleaseMlink(&widget, LOWORD(lParam), HIWORD(lParam),
+				(void) SelectObject(widget.core.hDC,
+					GetStockObject(NULL_PEN));
+				ReleasePuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 					((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)));
 				(void) ReleaseDC(hWnd, widget.core.hDC);
 				break;
 			}
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			MotionMlink(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			MotionPuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONUP:
 			mousePressed = False;
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ReleaseMlink(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ReleasePuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 #if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
 		case WM_MOUSEWHEEL:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
 			{
 				int zDelta = ((short) HIWORD(wParam));
 				POINT cursor, origin;
@@ -944,19 +958,19 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				ClientToScreen(hWnd, &origin);
 				(void) GetCursorPos(&cursor);
 				if (zDelta > (WHEEL_DELTA >> 1)) {
-					MoveMlinkInput(&widget, cursor.x - origin.x, TOP,
+					MovePuzzleInput(&widget, cursor.x - origin.x, TOP,
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 					if (GetKeyState(VK_CONTROL) >> 1) {
-						SizeMlink(&widget);
+						SizePuzzle(&widget);
 						(void) InvalidateRect(hWnd, NULL, TRUE);
 					}
 				} else if (zDelta < -(WHEEL_DELTA >> 1)) {
-					MoveMlinkInput(&widget, cursor.x - origin.x, BOTTOM,
+					MovePuzzleInput(&widget, cursor.x - origin.x, BOTTOM,
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 					if (GetKeyState(VK_CONTROL) >> 1) {
-						SizeMlink(&widget);
+						SizePuzzle(&widget);
 						(void) InvalidateRect(hWnd, NULL, TRUE);
 					}
 				}
@@ -967,74 +981,86 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		case WM_COMMAND:
 			switch (LOWORD(wParam)) {
 				case IDM_GET:
-					GetMlink(&widget);
-					ResizeMlink(&widget);
+					GetPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_WRITE:
-					WriteMlink(&widget);
+					WritePuzzle(&widget);
 					break;
 				case IDM_EXIT:
-					DestroyMlink(brush);
+					DestroyPuzzle(&widget, brush);
 					break;
 				case IDM_HIDE:
-					HideMlink(&widget);
-					break;
-				case IDM_CLEAR:
-					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					ClearMlink(&widget);
-					(void) ReleaseDC(hWnd, widget.core.hDC);
+					HidePuzzle(&widget);
 					break;
 				case IDM_UNDO:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					UndoMlink(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					UndoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_REDO:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RedoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_CLEAR:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					ClearPuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_RANDOMIZE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					RandomizeMlink(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RandomizePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_PRACTICE:
-					(void) PracticeMlink(&widget);
-					SizeMlink(&widget);
+					(void) PracticePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SOLVE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					SolveMlink(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					SolvePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
-				case IDM_ORIENT:
-					OrientizeMlink(&widget);
-					SizeMlink(&widget);
+				case IDM_ORIENTIZE:
+					OrientizePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_MIDDLE:
-					(void) MiddleMlink(&widget);
-					SizeMlink(&widget);
+					(void) MiddlePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SPEED:
-					SpeedMlink(&widget);
+					SpeedPuzzle(&widget);
 					break;
 				case IDM_SLOW:
-					SlowMlink(&widget);
+					SlowPuzzle(&widget);
 					break;
 				case IDM_SOUND:
-					SoundMlink(&widget);
+					SoundPuzzle(&widget);
 					break;
 				case IDM_LEFT:
 				case IDM_RIGHT:
 				case IDM_SHIFT_UP:
 				case IDM_SHIFT_DOWN:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					(void) MoveMlinkInput(&widget, 0,
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					(void) MovePuzzleInput(&widget, 0,
 							(int) LOWORD(wParam) - IDM_SHIFT_UP, !((int) wParam % 2), FALSE);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
@@ -1044,11 +1070,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
-						(void) MoveMlinkInput(&widget, cursor.x - origin.x,
+						(void) MovePuzzleInput(&widget, cursor.x - origin.x,
 								(int) LOWORD(wParam) - IDM_UP, FALSE, FALSE);
 						(void) ReleaseDC(hWnd, widget.core.hDC);
 					}
@@ -1057,9 +1084,9 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				case IDM_INCX:
 				case IDM_INCY:
 				case IDM_DECX:
-					MoveMlinkInput(&widget, 0,
+					MovePuzzleInput(&widget, 0,
 						(int) LOWORD(wParam) - IDM_DECY, FALSE, TRUE);
-					SizeMlink(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_ABOUT:
@@ -1141,7 +1168,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
 #else
 
 static void
-Usage(char * programName)
+Usage(char *programName)
 {
 	unsigned int i;
 
@@ -1177,12 +1204,6 @@ static XrmOptionDescRec options[] =
 	{(char *) "-face5", (char *) "*mlink.faceColor5", XrmoptionSepArg, NULL},
 	{(char *) "-face6", (char *) "*mlink.faceColor6", XrmoptionSepArg, NULL},
 	{(char *) "-face7", (char *) "*mlink.faceColor7", XrmoptionSepArg, NULL},
-	{(char *) "-tiles", (char *) "*mlink.tiles", XrmoptionSepArg, NULL},
-	{(char *) "-faces", (char *) "*mlink.faces", XrmoptionSepArg, NULL},
-	{(char *) "-orient", (char *) "*mlink.orient", XrmoptionNoArg, (char *) "TRUE"},
-	{(char *) "-noorient", (char *) "*mlink.orient", XrmoptionNoArg, (char *) "FALSE"},
-	{(char *) "-middle", (char *) "*mlink.middle", XrmoptionNoArg, (char *) "TRUE"},
-	{(char *) "-nomiddle", (char *) "*mlink.middle", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-install", (char *) "*mlink.install", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-noinstall", (char *) "*mlink.install", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-picture", (char *) "*mlink.picture", XrmoptionSepArg, NULL},
@@ -1194,6 +1215,12 @@ static XrmOptionDescRec options[] =
 	{(char *) "-base", (char *) "*mlink.base", XrmoptionSepArg, NULL},
 	{(char *) "-fn", (char *) "*mlink.font", XrmoptionSepArg, NULL},
 	{(char *) "-font", (char *) "*mlink.font", XrmoptionSepArg, NULL},
+	{(char *) "-tiles", (char *) "*mlink.tiles", XrmoptionSepArg, NULL},
+	{(char *) "-faces", (char *) "*mlink.faces", XrmoptionSepArg, NULL},
+	{(char *) "-orient", (char *) "*mlink.orient", XrmoptionNoArg, (char *) "TRUE"},
+	{(char *) "-noorient", (char *) "*mlink.orient", XrmoptionNoArg, (char *) "FALSE"},
+	{(char *) "-middle", (char *) "*mlink.middle", XrmoptionNoArg, (char *) "TRUE"},
+	{(char *) "-nomiddle", (char *) "*mlink.middle", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-userName", (char *) "*mlink.userName", XrmoptionSepArg, NULL},
 	{(char *) "-scoreFile", (char *) "*mlink.scoreFile", XrmoptionSepArg, NULL},
 	{(char *) "-scores", (char *) "*mlink.scoreOnly", XrmoptionNoArg, (char *) "TRUE"},
@@ -1202,25 +1229,25 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackMlinkPractice(Widget w, XtPointer clientData,
-		XmAnyCallbackStruct * cbs)
+CallbackPuzzlePractice(Widget w, XtPointer clientData,
+		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(mlink, XtNmenu, 9, NULL); /* menu choice */
+		XtVaSetValues(mlink, XtNmenu, MENU_PRACTICE, NULL);
 	}
 }
 
 static void
-CallbackMlinkRandomize(Widget w, XtPointer clientData,
-		XmAnyCallbackStruct * cbs)
+CallbackPuzzleRandomize(Widget w, XtPointer clientData,
+		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(mlink, XtNmenu, 5, NULL); /* menu choice */
+		XtVaSetValues(mlink, XtNmenu, MENU_RANDOMIZE, NULL);
 	}
 }
 
 static void
-TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int tiles = cbs->value, faces, old;
 	Boolean orient, middle, practice;
@@ -1244,7 +1271,7 @@ TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-FaceSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+FaceSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int tiles, faces = cbs->value, old;
 	Boolean orient, middle, practice;
@@ -1268,7 +1295,7 @@ FaceSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	int tiles, faces;
 	Boolean orient = cbs->set, middle, practice, oldOrient;
@@ -1298,7 +1325,7 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
 }
 
 static void
-MiddleToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+MiddleToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	int tiles, faces;
 	Boolean orient, middle = cbs->set, practice, oldMid;
@@ -1323,7 +1350,7 @@ MiddleToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
 
 static void
 PracticeToggle(Widget w, XtPointer clientData,
-		XmToggleButtonCallbackStruct * cbs)
+		XmToggleButtonCallbackStruct *cbs)
 {
 	int tiles, faces;
 	Boolean orient, middle, practice = cbs->set, oldPract;
@@ -1351,7 +1378,7 @@ PracticeToggle(Widget w, XtPointer clientData,
 }
 
 static void
-SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MAXSPEED + MINSPEED - cbs->value - 1, oldDelay;
 
@@ -1367,16 +1394,16 @@ fileCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	if (val == 2)
+	if (val == MENU_EXIT)
 		exit(0);
-	XtVaSetValues(mlink, XtNmenu, val, NULL);
+	XtVaSetValues(mlink, XtNmenu, val, NULL); /* GWQ */
 }
 
 static void
 playCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
-	XtVaSetValues(mlink, XtNmenu, val + 3, NULL); /* GWQ */
+	XtVaSetValues(mlink, XtNmenu, val + MENU_EXIT + 1, NULL);
 }
 
 static Widget
@@ -1475,8 +1502,9 @@ main(int argc, char **argv)
 	Widget messageRowCol;
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
-	XmString clearString, undoString, randomizeString, solveString;
-	XmString orientString, middleString, practiceString;
+	XmString undoString, redoString;
+	XmString clearString, randomizeString, practiceString, solveString;
+	XmString orientString, middleString;
 	XmString speedString, slowString, soundString;
 #endif
 
@@ -1486,7 +1514,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#if HAVE_XPM
+#ifdef HAVE_XPM
 	{
 		XpmAttributes xpmAttributes;
 		XpmColorSymbol transparentColor[1] = {{NULL,
@@ -1501,18 +1529,18 @@ main(int argc, char **argv)
 		transparentColor[0].pixel = bg;
 		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) mlink_xpm, &mlinkIcon, NULL,
+			(char **) mlink_xpm, &icon, NULL,
 			&xpmAttributes);
 	}
-	if (mlinkIcon == (Pixmap) NULL)
+	if (icon == (Pixmap) NULL)
 #endif
-		mlinkIcon = XCreateBitmapFromData(XtDisplay(topLevel),
+		icon = XCreateBitmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
 			(char *) mlink_bits,
 			mlink_width, mlink_height);
 #ifdef HAVE_MOTIF
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, mlinkIcon,
+		XtNiconPixmap, icon,
 		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
@@ -1539,35 +1567,38 @@ main(int argc, char **argv)
 	XmStringFree(getString);
 	XmStringFree(writeString);
 	XmStringFree(quitString);
-	clearString = XmStringCreateSimple((char *) "Clear");
 	undoString = XmStringCreateSimple((char *) "Undo");
-	randomizeString = XmStringCreateSimple((char *) "Randomize");
+	redoString = XmStringCreateSimple((char *) "Redo");
+	clearString = XmStringCreateSimple((char *) "Clear");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
+	practiceString = XmStringCreateSimple((char *) "Practice");
 	solveString = XmStringCreateSimple((char *) "Solve");
 	orientString = XmStringCreateSimple((char *) "Orientize");
 	middleString = XmStringCreateSimple((char *) "Middle");
-	practiceString = XmStringCreateSimple((char *) "Practice");
 	speedString = XmStringCreateSimple((char *) ">Speed");
 	slowString = XmStringCreateSimple((char *) "<Slow");
 	soundString = XmStringCreateSimple((char *) "@Sound");
 	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
-		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
+		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
 		XmVaPUSHBUTTON, orientString, 'O', NULL, NULL,
 		XmVaPUSHBUTTON, middleString, 'M', NULL, NULL,
-		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
 		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
 		XmVaPUSHBUTTON, soundString, '@', NULL, NULL,
 		NULL);
-	XmStringFree(clearString);
 	XmStringFree(undoString);
+	XmStringFree(redoString);
+	XmStringFree(clearString);
 	XmStringFree(randomizeString);
+	XmStringFree(practiceString);
 	XmStringFree(solveString);
 	XmStringFree(orientString);
 	XmStringFree(middleString);
-	XmStringFree(practiceString);
 	XmStringFree(speedString);
 	XmStringFree(slowString);
 	XmStringFree(soundString);
@@ -1614,11 +1645,11 @@ main(int argc, char **argv)
 	practiceDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to toggle the practice mode?",
 		(char *) "Practice Query",
-		(XtCallbackProc) CallbackMlinkPractice);
+		(XtCallbackProc) CallbackPuzzlePractice);
 	randomizeDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to randomize?",
 		(char *) "Randomize Query",
-		(XtCallbackProc) CallbackMlinkRandomize);
+		(XtCallbackProc) CallbackPuzzleRandomize);
 	mainPanel = XtCreateManagedWidget("mainPanel",
 		xmPanedWindowWidgetClass, menuBarPanel,
 		NULL, 0);
@@ -1751,13 +1782,13 @@ main(int argc, char **argv)
 		mlinkWidgetClass, mainPanel, NULL, 0);
 #else
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, mlinkIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 	mlink = XtCreateManagedWidget("mlink",
 		mlinkWidgetClass, topLevel, NULL, 0);
 #endif
 	XtAddCallback(mlink, XtNselectCallback,
-		(XtCallbackProc) CallbackMlink, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	Initialize(mlink);
 	XtRealizeWidget(topLevel);
 	XGrabButton(XtDisplay(mlink), (unsigned int) AnyButton, AnyModifier,
