@@ -63,49 +63,67 @@ static char faceColorChar[MAXFACES] =
 #endif
 #endif
 
-static void ResizeOct(OctWidget w);
-static void SizeOct(OctWidget w);
-static Boolean SetValuesOct(Widget current, Widget request, Widget renew);
-static void DestroyOct(Widget old);
-static void QuitOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void InitializeOct(Widget request, Widget renew);
-static void ExposeOct(Widget renew, XEvent * event, Region region);
-static void HideOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void PracticeOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void PracticeOctMaybe(OctWidget w, XEvent * event, char **args, int nArgs);
-static void PracticeOct2(OctWidget w, XEvent * event, char **args, int nArgs);
-static void RandomizeOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void RandomizeOctMaybe(OctWidget w, XEvent * event, char **args, int nArgs);
-static void RandomizeOct2(OctWidget w, XEvent * event, char **args, int nArgs);
-static void GetOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void WriteOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void ClearOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void UndoOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void SolveOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void IncrementOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void DecrementOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void OrientizeOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void StickyModeOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void Period3ModeOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void Period4ModeOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void BothModeOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void EnterOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void LeaveOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void MoveOctTl(OctWidget w, XEvent * event, char **args, int nArgs);
-static void MoveOctTop(OctWidget w, XEvent * event, char **args, int nArgs);
-static void MoveOctTr(OctWidget w, XEvent * event, char **args, int nArgs);
-static void MoveOctLeft(OctWidget w, XEvent * event, char **args, int nArgs);
-static void MoveOctCw(OctWidget w, XEvent * event, char **args, int nArgs);
-static void MoveOctRight(OctWidget w, XEvent * event, char **args, int nArgs);
-static void MoveOctBl(OctWidget w, XEvent * event, char **args, int nArgs);
-static void MoveOctBottom(OctWidget w, XEvent * event, char **args, int nArgs);
-static void MoveOctBr(OctWidget w, XEvent * event, char **args, int nArgs);
-static void MoveOctCcw(OctWidget w, XEvent * event, char **args, int nArgs);
-static void MoveOctInput(OctWidget w, int x, int y, int direction, int shift, int control);
-static void SelectOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void ReleaseOct(OctWidget w, XEvent * event, char **args, int nArgs);
+static void ResizePuzzle(OctWidget w);
+static void SizePuzzle(OctWidget w);
+static Boolean SetValuesPuzzle(Widget current, Widget request, Widget renew);
+static void DestroyPuzzle(Widget old);
+static void QuitPuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
+static void InitializePuzzle(Widget request, Widget renew);
+static void ExposePuzzle(Widget renew, XEvent *event, Region region);
+static void HidePuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
+static void PracticePuzzleMaybe(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void PracticePuzzle2(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void RandomizePuzzle(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void RandomizePuzzle2(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void GetPuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
+static void WritePuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
+static void UndoPuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
+static void RedoPuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
+static void ClearPuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
+static void RandomizePuzzleMaybe(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SolvePuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
+static void PracticePuzzle(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void IncrementPuzzle(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void DecrementPuzzle(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void OrientizePuzzle(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void StickyModePuzzle(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void Period3ModePuzzle(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void Period4ModePuzzle(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void BothModePuzzle(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void EnterPuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
+static void LeavePuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzleTl(OctWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzleTop(OctWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzleTr(OctWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzleLeft(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleCw(OctWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzleRight(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleBl(OctWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzleBottom(OctWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleBr(OctWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzleCcw(OctWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzleInput(OctWidget w,
+	int x, int y, int direction, int shift, int control);
+static void SelectPuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
+static void ReleasePuzzle(OctWidget w, XEvent *event, char **args, int nArgs);
 
-static char defaultTranslationsOct[] =
+static char defaultTranslationsPuzzle[] =
 "<KeyPress>q: Quit()\n\
  Ctrl<KeyPress>C: Quit()\n\
  <KeyPress>osfCancel: Hide()\n\
@@ -152,17 +170,20 @@ static char defaultTranslationsOct[] =
  <KeyPress>R15: MoveBr()\n\
  <Btn1Down>: Select()\n\
  <Btn1Up>: Release()\n\
- <KeyPress>p: Practice()\n\
- <KeyPress>r: Randomize()\n\
+ <Btn2Down>: PracticeMaybe()\n\
+ <Btn2Down>(2+): Practice2()\n\
  <Btn3Down>: RandomizeMaybe()\n\
  <Btn3Down>(2+): Randomize2()\n\
  <Btn4Down>: MoveTop()\n\
  <Btn5Down>: MoveBottom()\n\
  <KeyPress>g: Get()\n\
  <KeyPress>w: Write()\n\
- <KeyPress>c: Clear()\n\
  <KeyPress>u: Undo()\n\
+ <KeyPress>r: Redo()\n\
+ <KeyPress>c: Clear()\n\
+ <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
+ <KeyPress>p: Practice()\n\
  <KeyPress>i: Increment()\n\
  <KeyPress>d: Decrement()\n\
  <KeyPress>o: Orientize()\n\
@@ -173,45 +194,46 @@ static char defaultTranslationsOct[] =
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
-static XtActionsRec actionsListOct[] =
+static XtActionsRec actionsListPuzzle[] =
 {
-	{(char *) "Quit", (XtActionProc) QuitOct},
-	{(char *) "Hide", (XtActionProc) HideOct},
-	{(char *) "MoveCcw", (XtActionProc) MoveOctCcw},
-	{(char *) "MoveTl", (XtActionProc) MoveOctTl},
-	{(char *) "MoveTop", (XtActionProc) MoveOctTop},
-	{(char *) "MoveTr", (XtActionProc) MoveOctTr},
-	{(char *) "MoveLeft", (XtActionProc) MoveOctLeft},
-	{(char *) "MoveCw", (XtActionProc) MoveOctCw},
-	{(char *) "MoveRight", (XtActionProc) MoveOctRight},
-	{(char *) "MoveBl", (XtActionProc) MoveOctBl},
-	{(char *) "MoveBottom", (XtActionProc) MoveOctBottom},
-	{(char *) "MoveBr", (XtActionProc) MoveOctBr},
-	{(char *) "Select", (XtActionProc) SelectOct},
-	{(char *) "Release", (XtActionProc) ReleaseOct},
-	{(char *) "Practice", (XtActionProc) PracticeOct},
-	{(char *) "PracticeMaybe", (XtActionProc) PracticeOctMaybe},
-	{(char *) "Practice2", (XtActionProc) PracticeOct2},
-	{(char *) "Randomize", (XtActionProc) RandomizeOct},
-	{(char *) "RandomizeMaybe", (XtActionProc) RandomizeOctMaybe},
-	{(char *) "Randomize2", (XtActionProc) RandomizeOct2},
-	{(char *) "Get", (XtActionProc) GetOct},
-	{(char *) "Write", (XtActionProc) WriteOct},
-	{(char *) "Clear", (XtActionProc) ClearOct},
-	{(char *) "Undo", (XtActionProc) UndoOct},
-	{(char *) "Solve", (XtActionProc) SolveOct},
-	{(char *) "Increment", (XtActionProc) IncrementOct},
-	{(char *) "Decrement", (XtActionProc) DecrementOct},
-	{(char *) "Orientize", (XtActionProc) OrientizeOct},
-	{(char *) "Period3", (XtActionProc) Period3ModeOct},
-	{(char *) "Period4", (XtActionProc) Period4ModeOct},
-	{(char *) "Both", (XtActionProc) BothModeOct},
-	{(char *) "Sticky", (XtActionProc) StickyModeOct},
-	{(char *) "Enter", (XtActionProc) EnterOct},
-	{(char *) "Leave", (XtActionProc) LeaveOct}
+	{(char *) "Quit", (XtActionProc) QuitPuzzle},
+	{(char *) "Hide", (XtActionProc) HidePuzzle},
+	{(char *) "MoveCcw", (XtActionProc) MovePuzzleCcw},
+	{(char *) "MoveTl", (XtActionProc) MovePuzzleTl},
+	{(char *) "MoveTop", (XtActionProc) MovePuzzleTop},
+	{(char *) "MoveTr", (XtActionProc) MovePuzzleTr},
+	{(char *) "MoveLeft", (XtActionProc) MovePuzzleLeft},
+	{(char *) "MoveCw", (XtActionProc) MovePuzzleCw},
+	{(char *) "MoveRight", (XtActionProc) MovePuzzleRight},
+	{(char *) "MoveBl", (XtActionProc) MovePuzzleBl},
+	{(char *) "MoveBottom", (XtActionProc) MovePuzzleBottom},
+	{(char *) "MoveBr", (XtActionProc) MovePuzzleBr},
+	{(char *) "Select", (XtActionProc) SelectPuzzle},
+	{(char *) "Release", (XtActionProc) ReleasePuzzle},
+	{(char *) "PracticeMaybe", (XtActionProc) PracticePuzzleMaybe},
+	{(char *) "Practice2", (XtActionProc) PracticePuzzle2},
+	{(char *) "RandomizeMaybe", (XtActionProc) RandomizePuzzleMaybe},
+	{(char *) "Randomize2", (XtActionProc) RandomizePuzzle2},
+	{(char *) "Get", (XtActionProc) GetPuzzle},
+	{(char *) "Write", (XtActionProc) WritePuzzle},
+	{(char *) "Undo", (XtActionProc) UndoPuzzle},
+	{(char *) "Redo", (XtActionProc) RedoPuzzle},
+	{(char *) "Clear", (XtActionProc) ClearPuzzle},
+	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
+	{(char *) "Solve", (XtActionProc) SolvePuzzle},
+	{(char *) "Practice", (XtActionProc) PracticePuzzle},
+	{(char *) "Increment", (XtActionProc) IncrementPuzzle},
+	{(char *) "Decrement", (XtActionProc) DecrementPuzzle},
+	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
+	{(char *) "Period3", (XtActionProc) Period3ModePuzzle},
+	{(char *) "Period4", (XtActionProc) Period4ModePuzzle},
+	{(char *) "Both", (XtActionProc) BothModePuzzle},
+	{(char *) "Sticky", (XtActionProc) StickyModePuzzle},
+	{(char *) "Enter", (XtActionProc) EnterPuzzle},
+	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
 
-static XtResource resourcesOct[] =
+static XtResource resourcesPuzzle[] =
 {
 	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
 	 XtOffset(OctWidget, core.width),
@@ -315,29 +337,29 @@ OctClassRec octClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeOct,	/* initialize */
+		(XtInitProc) InitializePuzzle,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
-		actionsListOct,	/* actions */
-		XtNumber(actionsListOct),	/* num actions */
-		resourcesOct,	/* resources */
-		XtNumber(resourcesOct),		/* num resources */
+		actionsListPuzzle,	/* actions */
+		XtNumber(actionsListPuzzle),	/* num actions */
+		resourcesPuzzle,	/* resources */
+		XtNumber(resourcesPuzzle),		/* num resources */
 		NULLQUARK,	/* xrm class */
 		TRUE,		/* compress motion */
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
-		(XtWidgetProc) DestroyOct,	/* destroy */
-		(XtWidgetProc) ResizeOct,	/* resize */
-		(XtExposeProc) ExposeOct,	/* expose */
-		(XtSetValuesFunc) SetValuesOct,		/* set values */
+		(XtWidgetProc) DestroyPuzzle,	/* destroy */
+		(XtWidgetProc) ResizePuzzle,	/* resize */
+		(XtExposeProc) ExposePuzzle,	/* expose */
+		(XtSetValuesFunc) SetValuesPuzzle,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
 		NULL,		/* accept focus */
 		XtVersion,	/* version */
 		NULL,		/* callback private */
-		defaultTranslationsOct,		/* tm table */
+		defaultTranslationsPuzzle,	/* tm table */
 		NULL,		/* query geometry */
 		NULL,		/* display accelerator */
 		NULL		/* extension */
@@ -350,7 +372,7 @@ OctClassRec octClassRec =
 WidgetClass octWidgetClass = (WidgetClass) & octClassRec;
 
 void
-SetOct(OctWidget w, int reason)
+setPuzzle(OctWidget w, int reason)
 {
 	octCallbackStruct cb;
 
@@ -371,8 +393,8 @@ loadFont(OctWidget w)
 		XUnloadFont(XtDisplay(w), w->oct.fontInfo->fid);
 		XFreeFont(XtDisplay(w), w->oct.fontInfo);
 	}
-	if ((w->oct.fontInfo = XLoadQueryFont(display,
-			w->oct.font)) == NULL) {
+	if (w->oct.font && (w->oct.fontInfo =
+			XLoadQueryFont(display, w->oct.font)) == NULL) {
 		(void) sprintf(buf,
 			"Can not open %s font.\nAttempting %s font as alternate\n",
 			w->oct.font, altfontname);
@@ -578,6 +600,9 @@ static Point triangleList[MAXSIDES][4], letterList[MAXSIDES];
 
 static Pixmap dr = 0; /* dummy for future double buffering */
 
+static OctStack undo = {NULL, NULL, NULL, 0};
+static OctStack redo = {NULL, NULL, NULL, 0};
+
 static void
 CheckPieces(OctWidget w)
 {
@@ -999,7 +1024,7 @@ DrawFrame(OctWidget w, Boolean focus)
 static void
 MoveNoPieces(OctWidget w)
 {
-	SetOct(w, OCT_ILLEGAL);
+	setPuzzle(w, PUZZLE_ILLEGAL);
 }
 
 static void
@@ -1320,7 +1345,7 @@ MoveControlCb(OctWidget w, int face, int direction, int style)
 				rtt.trbl = i;
 				rtt.tlbr = i;
 				MovePieces(w, face, rtt, direction, style);
-				SetOct(w, OCT_CONTROL);
+				setPuzzle(w, PUZZLE_CONTROL);
 		} else		/* (style == PERIOD4) */
 			for (i = 0; i < 3; i++) {
 				if (i < 2) {
@@ -1354,7 +1379,7 @@ MoveControlCb(OctWidget w, int face, int direction, int style)
 								(direction + 6) % MAXFACES, style);
 					}
 				}
-				SetOct(w, OCT_CONTROL);
+				setPuzzle(w, PUZZLE_CONTROL);
 			}
 	} else {
 		if (style == PERIOD3)
@@ -1363,7 +1388,7 @@ MoveControlCb(OctWidget w, int face, int direction, int style)
 				rtt.trbl = i;
 				rtt.tlbr = i;
 				MovePieces(w, face, rtt, direction, style);
-				SetOct(w, OCT_CONTROL);
+				setPuzzle(w, PUZZLE_CONTROL);
 		} else		/* (style == PERIOD4) */
 			for (i = 0; i < 2 * w->oct.size - 1; i++) {
 				if (i < w->oct.size) {
@@ -1393,13 +1418,13 @@ MoveControlCb(OctWidget w, int face, int direction, int style)
 								(direction + 6) % MAXFACES, style);
 					}
 				}
-				SetOct(w, OCT_CONTROL);
+				setPuzzle(w, PUZZLE_CONTROL);
 			}
 	}
 }
 
 void
-MoveOct(OctWidget w, int face, int position, int direction, int style, int control)
+MovePuzzle(OctWidget w, int face, int position, int direction, int style, int control)
 {
 	if (control)
 		MoveControlCb(w, face, direction, style);
@@ -1408,9 +1433,10 @@ MoveOct(OctWidget w, int face, int position, int direction, int style, int contr
 
 		ToRTT(position, &rtt);
 		MovePieces(w, face, rtt, direction, style);
-		SetOct(w, OCT_MOVED);
+		setPuzzle(w, PUZZLE_MOVED);
 	}
-	PutMove(face, position, direction, style, control);
+	setMove(&undo, face, position, direction, style, control);
+	flushMoves(w, &redo, FALSE);
 }
 
 static Boolean
@@ -1562,7 +1588,7 @@ PositionPieces(OctWidget w, int x, int y, int style, int *face, RTT * rtt, int *
 static
 #endif
 void
-MoveOctInput(OctWidget w, int x, int y, int direction, int shift, int control)
+MovePuzzleInput(OctWidget w, int x, int y, int direction, int shift, int control)
 {
 	int style, face;
 	RTT rtt;
@@ -1581,9 +1607,9 @@ MoveOctInput(OctWidget w, int x, int y, int direction, int shift, int control)
 	if (!PositionPieces(w, x, y, style, &face, &rtt, &direction))
 		return;
 	control = (control) ? 1 : 0;
-	MoveOct(w, face, ToPosition(rtt), direction, style, control);
+	MovePuzzle(w, face, ToPosition(rtt), direction, style, control);
 	if (!control && CheckSolved(w)) {
-		SetOct(w, OCT_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
@@ -1624,10 +1650,12 @@ ResetPieces(OctWidget w)
 	for (face = 0; face < MAXFACES; face++)
 		for (position = 0; position < w->oct.sizeSize; position++) {
 			w->oct.facetLoc[face][position].face = face;
-			w->oct.facetLoc[face][position].rotation = 3 * face % MAXORIENT;
+			w->oct.facetLoc[face][position].rotation =
+				3 * face % MAXORIENT;
 		}
-	FlushMoves(w);
-	w->oct.currentFace = -1;
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
+	w->oct.currentFace = PUZZLE_IGNORE;
 	w->oct.started = False;
 }
 
@@ -1703,15 +1731,16 @@ GetPieces(OctWidget w)
 		}
 #endif
 	}
-	FlushMoves(w);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &size);
 	if (size >= MINFACETS) {
 		for (i = w->oct.size; i < size; i++) {
-			SetOct(w, OCT_INC);
+			setPuzzle(w, PUZZLE_INC);
 		}
 		for (i = w->oct.size; i > size; i--) {
-			SetOct(w, OCT_DEC);
+			setPuzzle(w, PUZZLE_DEC);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: size ");
@@ -1730,13 +1759,13 @@ GetPieces(OctWidget w)
 	(void) fscanf(fp, "%d", &mode);
 	switch (mode) {
 		case PERIOD3:
-			SetOct(w, OCT_PERIOD3);
+			setPuzzle(w, PUZZLE_PERIOD3);
 			break;
 		case PERIOD4:
-			SetOct(w, OCT_PERIOD4);
+			setPuzzle(w, PUZZLE_PERIOD4);
 			break;
 		case BOTH:
-			SetOct(w, OCT_BOTH);
+			setPuzzle(w, PUZZLE_BOTH);
 			break;
 		default:
 			stringCat(&buf1, name, " corrupted: mode ");
@@ -1756,27 +1785,27 @@ GetPieces(OctWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &sticky);
 	if (w->oct.sticky != (Boolean) sticky) {
-		SetOct(w, OCT_STICKY);
+		setPuzzle(w, PUZZLE_STICKY);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &orient);
 	if (w->oct.orient != (Boolean) orient) {
-		SetOct(w, OCT_ORIENT);
+		setPuzzle(w, PUZZLE_ORIENTIZE);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &practice);
 	if (w->oct.practice != (Boolean) practice) {
-		SetOct(w, OCT_PRACTICE);
+		setPuzzle(w, PUZZLE_PRACTICE);
 	}
 #ifdef WINVER
 	ResetPieces(w);
 #endif
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
-	ScanStartPosition(fp, w);
-	SetOct(w, OCT_RESTORE);
-	SetStartPosition(w);
-	ScanMoves(fp, w, moves);
+	scanStartPosition(fp, w);
+	setPuzzle(w, PUZZLE_RESTORE);
+	setStartPosition(w);
+	scanMoves(fp, w, moves);
 	(void) fclose(fp);
 	(void) printf("%s: size %d, mode %d, sticky %d, orient %d",
 		name, size, mode, sticky, orient);
@@ -1834,32 +1863,27 @@ WritePieces(OctWidget w)
 	(void) fprintf(fp, "orient%c %d\n", SYMBOL, (w->oct.orient) ? 1 : 0);
 	(void) fprintf(fp, "practice%c %d\n", SYMBOL,
 		(w->oct.practice) ? 1 : 0);
-	(void) fprintf(fp, "moves%c %d\n", SYMBOL, NumMoves());
-	PrintStartPosition(fp, w);
-	PrintMoves(fp);
+	(void) fprintf(fp, "moves%c %d\n", SYMBOL,
+		numMoves(&undo));
+	printStartPosition(fp, w);
+	printMoves(fp, &undo);
 	(void) fclose(fp);
 	(void) printf("Saved to %s.\n", name);
 	free(lname);
 	free(fname);
 }
 
-static void
-ClearPieces(OctWidget w)
-{
-	if (w->oct.currentFace >= 0)
-		return;
-	ResetPieces(w);
-	DrawAllPieces(w);
-	SetOct(w, OCT_RESET);
-}
-
 static void
 UndoPieces(OctWidget w)
 {
-	if (MadeMoves() && w->oct.currentFace < 0) {
+	if (madeMoves(&undo) &&
+			w->oct.currentFace <= PUZZLE_IGNORE) {
 		int face, position, direction, style, control;
 
-		GetMove(&face, &position, &direction, &style, &control);
+		getMove(&undo,
+			&face, &position, &direction, &style, &control);
+		setMove(&redo,
+			face, position, direction, style, control);
 		direction = (direction < COORD) ?
 			(direction + MAXSIDES) % MAXFACES : 3 * COORD - direction;
 		if (control)
@@ -1869,18 +1893,54 @@ UndoPieces(OctWidget w)
 
 			ToRTT(position, &rtt);
 			MovePieces(w, face, rtt, direction, style);
-			SetOct(w, OCT_UNDO);
+			setPuzzle(w, PUZZLE_UNDO);
 			if (CheckSolved(w)) {
-				SetOct(w, OCT_SOLVED);
+				setPuzzle(w, PUZZLE_SOLVED);
 			}
 		}
 	}
 }
 
+static void
+RedoPieces(OctWidget w)
+{
+	if (madeMoves(&redo) &&
+			w->oct.currentFace <= PUZZLE_IGNORE) {
+		int face, position, direction, style, control;
+
+		getMove(&redo,
+			&face, &position, &direction, &style, &control);
+		setMove(&undo,
+			face, position, direction, style, control);
+		if (control)
+			MoveControlCb(w, face, direction, style);
+		else {
+			RTT rtt;
+
+			ToRTT(position, &rtt);
+			MovePieces(w, face, rtt, direction, style);
+			setPuzzle(w, PUZZLE_REDO);
+			if (CheckSolved(w)) {
+				setPuzzle(w, PUZZLE_SOLVED);
+			}
+		}
+	}
+}
+
+static void
+ClearPieces(OctWidget w)
+{
+	if (w->oct.currentFace > PUZZLE_IGNORE)
+		return;
+	ResetPieces(w);
+	DrawAllPieces(w);
+	setPuzzle(w, PUZZLE_RESET);
+}
+
 static void
 PracticePieces(OctWidget w)
 {
-	SetOct(w, OCT_PRACTICE);
+	setPuzzle(w, PUZZLE_PRACTICE);
 }
 
 static void
@@ -1889,7 +1949,7 @@ RandomizePieces(OctWidget w)
 	int randomDirection, face, position, style;
 	int big = w->oct.sizeSize * 3 + NRAND(2);
 
-	if (w->oct.currentFace >= 0)
+	if (w->oct.currentFace > PUZZLE_IGNORE)
 		return;
 	if (big > 1000)
 		big = 1000;
@@ -1897,7 +1957,7 @@ RandomizePieces(OctWidget w)
 		PracticePieces(w);
 	if (w->oct.sticky)
 		big /= 3;
-	SetOct(w, OCT_RESET);
+	setPuzzle(w, PUZZLE_RESET);
 
 #ifdef DEBUG
 	big = 3;
@@ -1963,30 +2023,31 @@ RandomizePieces(OctWidget w)
 			}
 			position = NRAND(w->oct.sizeSize);
 		}
-		MoveOct(w, face, position, randomDirection, style, FALSE);
-		SetOct(w, OCT_MOVED);
+		MovePuzzle(w, face, position, randomDirection, style, FALSE);
+		setPuzzle(w, PUZZLE_MOVED);
 	}
-	FlushMoves(w);
-	SetOct(w, OCT_RANDOMIZE);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
+	setPuzzle(w, PUZZLE_RANDOMIZE);
 	if (CheckSolved(w)) {
-		SetOct(w, OCT_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
 static void
 SolvePieces(OctWidget w)
 {
-	if (CheckSolved(w) && w->oct.currentFace >= 0)
+	if (CheckSolved(w) && w->oct.currentFace > PUZZLE_IGNORE)
 		return;
 	{
-		SetOct(w, OCT_SOLVE_MESSAGE);
+		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
 	}
 }
 
 static void
 IncrementPieces(OctWidget w)
 {
-	SetOct(w, OCT_INC);
+	setPuzzle(w, PUZZLE_INC);
 }
 
 static Boolean
@@ -1994,25 +2055,25 @@ DecrementPieces(OctWidget w)
 {
 	if (w->oct.size <= MINFACETS)
 		return False;
-	SetOct(w, OCT_DEC);
+	setPuzzle(w, PUZZLE_DEC);
 	return True;
 }
 
 static void
 OrientizePieces(OctWidget w)
 {
-	SetOct(w, OCT_ORIENT);
+	setPuzzle(w, PUZZLE_ORIENTIZE);
 }
 
 static void
 StickyPieces(OctWidget w)
 {
-	SetOct(w, OCT_STICKY);
+	setPuzzle(w, PUZZLE_STICKY);
 }
 
 #ifdef WINVER
 static void
-SetValuesOct(OctWidget w)
+SetValuesPuzzle(OctWidget w)
 {
 	struct tagColor {
 		int red, green, blue;
@@ -2079,8 +2140,10 @@ SetValuesOct(OctWidget w)
 }
 
 void
-DestroyOct(HBRUSH brush)
+DestroyPuzzle(HBRUSH brush)
 {
+	deleteMoves(&undo);
+	deleteMoves(&redo);
 	(void) DeleteObject(brush);
 	PostQuitMessage(0);
 }
@@ -2190,7 +2253,7 @@ SetAllColors(OctWidget w)
 }
 
 static Boolean
-SetValuesOct(Widget current, Widget request, Widget renew)
+SetValuesPuzzle(Widget current, Widget request, Widget renew)
 {
 	OctWidget c = (OctWidget) current, w = (OctWidget) renew;
 	Boolean redraw = False, setColors = False;
@@ -2226,61 +2289,55 @@ SetValuesOct(Widget current, Widget request, Widget renew)
 	if (w->oct.size != c->oct.size ||
 			w->oct.mode != c->oct.mode ||
 			w->oct.sticky != c->oct.sticky) {
-		SizeOct(w);
+		SizePuzzle(w);
 		redraw = True;
 	}
 	if (w->oct.facetLength != c->oct.facetLength) {
-		ResizeOct(w);
+		ResizePuzzle(w);
 		redraw = True;
 	}
 	if (w->oct.menu != -1) {
-		switch (w->oct.menu) {
-		case 0:
-			w->oct.menu = -1;
+		int menu = w->oct.menu;
+
+		w->oct.menu = -1;
+		switch (menu) {
+		case MENU_GET:
 			GetPieces(w);
 			break;
-		case 1:
-			w->oct.menu = -1;
+		case MENU_WRITE:
 			WritePieces(w);
 			break;
-		case 3:
-			w->oct.menu = -1;
-			ClearPieces(w);
-			break;
-		case 4:
-			w->oct.menu = -1;
+		case MENU_UNDO:
 			UndoPieces(w);
 			break;
-		case 5:
-			w->oct.menu = -1;
+		case MENU_REDO:
+			RedoPieces(w);
+			break;
+		case MENU_CLEAR:
+			ClearPieces(w);
+			break;
+		case MENU_RANDOMIZE:
 			RandomizePieces(w);
 			break;
-		case 6:
-			w->oct.menu = -1;
-			SolvePieces(w);
-			break;
-		case 7:
-			w->oct.menu = -1;
-			IncrementPieces(w);
-			break;
-		case 8:
-			w->oct.menu = -1;
-			(void) DecrementPieces(w);
-			break;
-		case 9:
-			w->oct.menu = -1;
-			OrientizePieces(w);
-			break;
-		case 10:
-			w->oct.menu = -1;
-			StickyPieces(w);
-			break;
-		case 11:
-			w->oct.menu = -1;
+		case MENU_PRACTICE:
 			PracticePieces(w);
 			break;
+		case MENU_SOLVE:
+			SolvePieces(w);
+			break;
+		case MENU_ORIENTIZE:
+			OrientizePieces(w);
+			break;
+		case MENU_STICKY:
+			StickyPieces(w);
+			break;
+		case MENU_INCREMENT:
+			IncrementPieces(w);
+			break;
+		case MENU_DECREMENT:
+			(void) DecrementPieces(w);
+			break;
 		default:
-			w->oct.menu = -1;
 			break;
 		}
 	}
@@ -2288,22 +2345,10 @@ SetValuesOct(Widget current, Widget request, Widget renew)
 }
 
 static void
-QuitOct(OctWidget w, XEvent * event, char **args, int nArgs)
-{
-	Display *display = XtDisplay(w);
-
-	if (w->oct.fontInfo) {
-		XUnloadFont(display, w->oct.fontInfo->fid);
-		XFreeFont(display, w->oct.fontInfo);
-	}
-	XtCloseDisplay(display);
-	exit(0);
-}
-
-static void
-DestroyOct(Widget old)
+DestroyPuzzle(Widget old)
 {
 	OctWidget w = (OctWidget) old;
+	Display *display = XtDisplay(w);
 	int face;
 
 	for (face = 0; face < MAXFACES; face++)
@@ -2312,6 +2357,19 @@ DestroyOct(Widget old)
 	XtReleaseGC(old, w->oct.frameGC);
 	XtReleaseGC(old, w->oct.inverseGC);
 	XtRemoveCallbacks(old, XtNselectCallback, w->oct.select);
+	if (w->oct.fontInfo) {
+		XUnloadFont(display, w->oct.fontInfo->fid);
+		XFreeFont(display, w->oct.fontInfo);
+	}
+	deleteMoves(&undo);
+	deleteMoves(&redo);
+}
+
+static void
+QuitPuzzle(OctWidget w, XEvent *event, char **args, int nArgs)
+{
+	XtCloseDisplay(XtDisplay(w));
+	exit(0);
 }
 #endif
 
@@ -2319,7 +2377,7 @@ DestroyOct(Widget old)
 static
 #endif
 void
-ResizeOct(OctWidget w)
+ResizePuzzle(OctWidget w)
 {
 	int tempLength;
 #ifdef WINVER
@@ -2357,17 +2415,17 @@ ResizeOct(OctWidget w)
 static
 #endif
 void
-SizeOct(OctWidget w)
+SizePuzzle(OctWidget w)
 {
 	ResetPieces(w);
-	ResizeOct(w);
+	ResizePuzzle(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-InitializeOct(
+InitializePuzzle(
 #ifdef WINVER
 OctWidget w, HBRUSH brush
 #else
@@ -2377,7 +2435,7 @@ Widget request, Widget renew
 {
 	int face, orient;
 #ifdef WINVER
-	SetValuesOct(w);
+	SetValuesPuzzle(w);
 #else
 	OctWidget w = (OctWidget) renew;
 
@@ -2398,9 +2456,10 @@ Widget request, Widget renew
 		w->oct.rowLoc[orient] = NULL;
 	w->oct.faceLoc = NULL;
 	CheckPieces(w);
-	InitMoves();
+	newMoves(&undo);
+	newMoves(&redo);
 	w->oct.cheat = False;
-	SizeOct(w);
+	SizePuzzle(w);
 #ifdef WINVER
 	brush = CreateSolidBrush(w->oct.inverseGC);
 	SETBACK(w->core.hWnd, brush);
@@ -2415,11 +2474,11 @@ Widget request, Widget renew
 static
 #endif
 void
-ExposeOct(
+ExposePuzzle(
 #ifdef WINVER
 OctWidget w
 #else
-Widget renew, XEvent * event, Region region
+Widget renew, XEvent *event, Region region
 #endif
 )
 {
@@ -2438,24 +2497,24 @@ Widget renew, XEvent * event, Region region
 static
 #endif
 void
-HideOct(OctWidget w
+HidePuzzle(OctWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
-	SetOct(w, OCT_HIDE);
+	setPuzzle(w, PUZZLE_HIDE);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-SelectOct(OctWidget w
+SelectPuzzle(OctWidget w
 #ifdef WINVER
 , const int x, const int y, const int control
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2468,21 +2527,22 @@ SelectOct(OctWidget w
 	if (SelectPieces(w, x, y, &(w->oct.currentFace), &rtt)) {
 		w->oct.currentPosition = ToPosition(rtt);
 		if (control || w->oct.practice || !CheckSolved(w)) {
-			DrawTriangle(w, w->oct.currentFace, w->oct.currentPosition, TRUE);
+			DrawTriangle(w, w->oct.currentFace,
+				w->oct.currentPosition, TRUE);
 		}
 	} else
-		w->oct.currentFace = -1;
+		w->oct.currentFace = PUZZLE_IGNORE;
 }
 
 #ifndef WINVER
 static
 #endif
 void
-ReleaseOct(OctWidget w
+ReleasePuzzle(OctWidget w
 #ifdef WINVER
 , const int x, const int y, const int shift, const int control
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2494,7 +2554,7 @@ ReleaseOct(OctWidget w
 	int control = (int) (event->xkey.state & ControlMask);
 #endif
 
-	if (w->oct.currentFace < 0)
+	if (w->oct.currentFace <= PUZZLE_IGNORE)
 		return;
 	DrawTriangle(w, w->oct.currentFace, w->oct.currentPosition, FALSE);
 	if (!control && !w->oct.practice && CheckSolved(w))
@@ -2514,50 +2574,38 @@ ReleaseOct(OctWidget w
 			count = CheckMoveDir(w, rtt1, rtt, face, &direction);
 		}
 		if (count == 1 && NarrowSelection(style, &face, &direction)) {
-			MoveOct(w, face, w->oct.currentPosition,
+			MovePuzzle(w, face, w->oct.currentPosition,
 				direction, style, (control) ? 1 : 0);
 			if (!control && CheckSolved(w)) {
-				SetOct(w, OCT_SOLVED);
+				setPuzzle(w, PUZZLE_SOLVED);
 			}
 		} else if (count == 2) {
-			SetOct(w, OCT_AMBIGUOUS);
-		} else if (!count)
+			/* count == 3 too annoying */
+			setPuzzle(w, PUZZLE_AMBIGUOUS);
+		} else if (count == 0)
 			MoveNoPieces(w);
 	}
-	w->oct.currentFace = -1;
-}
-
-#ifndef WINVER
-static
-#endif
-void
-PracticeOct(OctWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	PracticePieces(w);
+	w->oct.currentFace = PUZZLE_IGNORE;
 }
 
 #ifndef WINVER
 static void
-PracticeOctMaybe(OctWidget w
-, XEvent * event, char **args, int nArgs
+PracticePuzzleMaybe(OctWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 	if (!w->oct.started)
 		PracticePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		SetOct(w, OCT_PRACTICE_QUERY);
+		setPuzzle(w, PUZZLE_PRACTICE_QUERY);
 	}
 #endif
 }
 
 static void
-PracticeOct2(OctWidget w
-, XEvent * event, char **args, int nArgs
+PracticePuzzle2(OctWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 #ifdef HAVE_MOTIF
@@ -2567,37 +2615,24 @@ PracticeOct2(OctWidget w
 }
 #endif
 
-#ifndef WINVER
-static
-#endif
-void
-RandomizeOct(OctWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	RandomizePieces(w);
-}
-
 #ifndef WINVER
 static void
-RandomizeOctMaybe(OctWidget w
-, XEvent * event, char **args, int nArgs
+RandomizePuzzleMaybe(OctWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 	if (!w->oct.started)
 		RandomizePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		SetOct(w, OCT_RANDOMIZE_QUERY);
+		setPuzzle(w, PUZZLE_RANDOMIZE_QUERY);
 	}
 #endif
 }
 
 static void
-RandomizeOct2(OctWidget w
-, XEvent * event, char **args, int nArgs
+RandomizePuzzle2(OctWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 #ifdef HAVE_MOTIF
@@ -2611,9 +2646,9 @@ RandomizeOct2(OctWidget w
 static
 #endif
 void
-GetOct(OctWidget w
+GetPuzzle(OctWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2624,9 +2659,9 @@ GetOct(OctWidget w
 static
 #endif
 void
-WriteOct(OctWidget w
+WritePuzzle(OctWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2637,22 +2672,9 @@ WriteOct(OctWidget w
 static
 #endif
 void
-ClearOct(OctWidget w
+UndoPuzzle(OctWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	ClearPieces(w);
-}
-
-#ifndef WINVER
-static
-#endif
-void
-UndoOct(OctWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2663,9 +2685,48 @@ UndoOct(OctWidget w
 static
 #endif
 void
-SolveOct(OctWidget w
+RedoPuzzle(OctWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RedoPieces(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+ClearPuzzle(OctWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	ClearPieces(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+RandomizePuzzle(OctWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RandomizePieces(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+SolvePuzzle(OctWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2676,9 +2737,22 @@ SolveOct(OctWidget w
 static
 #endif
 void
-IncrementOct(OctWidget w
+PracticePuzzle(OctWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	PracticePieces(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+IncrementPuzzle(OctWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2690,9 +2764,9 @@ Boolean
 #else
 static void
 #endif
-DecrementOct(OctWidget w
+DecrementPuzzle(OctWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2708,9 +2782,9 @@ DecrementOct(OctWidget w
 static
 #endif
 void
-OrientizeOct(OctWidget w
+OrientizePuzzle(OctWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2721,9 +2795,9 @@ OrientizeOct(OctWidget w
 static
 #endif
 void
-StickyModeOct(OctWidget w
+StickyModePuzzle(OctWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2732,27 +2806,27 @@ StickyModeOct(OctWidget w
 
 #ifndef WINVER
 static void
-Period3ModeOct(OctWidget w
-, XEvent * event, char **args, int nArgs
+Period3ModePuzzle(OctWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
-	SetOct(w, OCT_PERIOD3);
+	setPuzzle(w, PUZZLE_PERIOD3);
 }
 
 static void
-Period4ModeOct(OctWidget w
-, XEvent * event, char **args, int nArgs
+Period4ModePuzzle(OctWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
-	SetOct(w, OCT_PERIOD4);
+	setPuzzle(w, PUZZLE_PERIOD4);
 }
 
 static void
-BothModeOct(OctWidget w
-, XEvent * event, char **args, int nArgs
+BothModePuzzle(OctWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
-	SetOct(w, OCT_BOTH);
+	setPuzzle(w, PUZZLE_BOTH);
 }
 #endif
 
@@ -2760,9 +2834,9 @@ BothModeOct(OctWidget w
 static
 #endif
 void
-EnterOct(OctWidget w
+EnterPuzzle(OctWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2774,9 +2848,9 @@ EnterOct(OctWidget w
 static
 #endif
 void
-LeaveOct(OctWidget w
+LeavePuzzle(OctWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2786,89 +2860,89 @@ LeaveOct(OctWidget w
 
 #ifdef WINVER
 void
-PeriodModeOct(OctWidget w, const int mode)
+PeriodModePuzzle(OctWidget w, const int mode)
 {
-	SetOct(w, mode + OCT_PERIOD3);
+	setPuzzle(w, mode + PUZZLE_PERIOD3);
 }
 
 #else
 
 static void
-MoveOctCcw(OctWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleCcw(OctWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveOctInput(w, event->xbutton.x, event->xbutton.y, CCW,
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, CCW,
 		(int) (event->xbutton.state & (ShiftMask | LockMask)),
 		(int) (event->xbutton.state & ControlMask));
 }
 
 static void
-MoveOctTl(OctWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleTl(OctWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveOctInput(w, event->xbutton.x, event->xbutton.y, TL,
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, TL,
 		(int) (event->xbutton.state & (ShiftMask | LockMask)),
 		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveOctTop(OctWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleTop(OctWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveOctInput(w, event->xbutton.x, event->xbutton.y, TOP,
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, TOP,
 		(int) (event->xbutton.state & (ShiftMask | LockMask)),
 		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveOctTr(OctWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleTr(OctWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveOctInput(w, event->xbutton.x, event->xbutton.y, TR,
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, TR,
 		(int) (event->xbutton.state & (ShiftMask | LockMask)),
 		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveOctLeft(OctWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleLeft(OctWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveOctInput(w, event->xbutton.x, event->xbutton.y, LEFT,
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, LEFT,
 		(int) (event->xbutton.state & (ShiftMask | LockMask)),
 		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveOctCw(OctWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleCw(OctWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveOctInput(w, event->xbutton.x, event->xbutton.y, CW,
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, CW,
 		(int) (event->xbutton.state & (ShiftMask | LockMask)),
 		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveOctRight(OctWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleRight(OctWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveOctInput(w, event->xbutton.x, event->xbutton.y, RIGHT,
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, RIGHT,
 		(int) (event->xbutton.state & (ShiftMask | LockMask)),
 		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveOctBl(OctWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleBl(OctWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveOctInput(w, event->xbutton.x, event->xbutton.y, BL,
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, BL,
 		(int) (event->xbutton.state & (ShiftMask | LockMask)),
 		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveOctBottom(OctWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleBottom(OctWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveOctInput(w, event->xbutton.x, event->xbutton.y, BOTTOM,
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, BOTTOM,
 		(int) (event->xbutton.state & (ShiftMask | LockMask)),
 		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveOctBr(OctWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleBr(OctWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveOctInput(w, event->xbutton.x, event->xbutton.y, BR,
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, BR,
 		(int) (event->xbutton.state & (ShiftMask | LockMask)),
 		(int) (event->xkey.state & ControlMask));
 }
