@@ -81,28 +81,44 @@ typedef struct {
 } octCallbackStruct;
 #endif
 
-#define OCT_HIDE (-8)
-#define OCT_PRACTICE_QUERY (-7)
-#define OCT_RANDOMIZE_QUERY (-6)
-#define OCT_SOLVE_MESSAGE (-5)
-#define OCT_RESTORE (-4)
-#define OCT_RESET (-3)
-#define OCT_AMBIGUOUS (-2)
-#define OCT_ILLEGAL (-1)
-#define OCT_MOVED 0
-#define OCT_CONTROL 1
-#define OCT_SOLVED 2
-#define OCT_PRACTICE 3
-#define OCT_RANDOMIZE 4
-#define OCT_DEC 5
-#define OCT_ORIENT 6
-#define OCT_INC 7
-#define OCT_PERIOD3 8
-#define OCT_PERIOD4 9
-#define OCT_BOTH 10
-#define OCT_STICKY 11
-#define OCT_COMPUTED 12
-#define OCT_UNDO 13
+#define PUZZLE_HIDE (-9)
+#define PUZZLE_PRACTICE_QUERY (-8)
+#define PUZZLE_RANDOMIZE_QUERY (-7)
+#define PUZZLE_SOLVE_MESSAGE (-6)
+#define PUZZLE_RESTORE (-5)
+#define PUZZLE_RESET (-4)
+#define PUZZLE_AMBIGUOUS (-3)
+#define PUZZLE_ILLEGAL (-2)
+#define PUZZLE_IGNORE (-1)
+#define PUZZLE_MOVED 0
+#define PUZZLE_CONTROL 1
+#define PUZZLE_SOLVED 2
+#define PUZZLE_PRACTICE 3
+#define PUZZLE_RANDOMIZE 4
+#define PUZZLE_DEC 5
+#define PUZZLE_ORIENTIZE 6
+#define PUZZLE_INC 7
+#define PUZZLE_PERIOD3 8
+#define PUZZLE_PERIOD4 9
+#define PUZZLE_BOTH 10
+#define PUZZLE_STICKY 11
+#define PUZZLE_COMPUTED 12
+#define PUZZLE_UNDO 13
+#define PUZZLE_REDO 14
+
+#define MENU_GET 0
+#define MENU_WRITE 1
+#define MENU_EXIT 2
+#define MENU_UNDO 3
+#define MENU_REDO 4
+#define MENU_CLEAR 5
+#define MENU_RANDOMIZE 6
+#define MENU_PRACTICE 7
+#define MENU_SOLVE 8
+#define MENU_ORIENTIZE 9
+#define MENU_STICKY 10
+#define MENU_INCREMENT 11
+#define MENU_DECREMENT 12
 
 #define MINFACETS 1
 #define PERIOD3 3
