@@ -90,7 +90,7 @@ static OctRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Oct Version 7.2.1\n"
+"Oct Version 7.2.4\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -144,13 +144,22 @@ static const char featuresHelp[] = {
 "record should say \"practice\").  This is good for learning "
 "moves and experimenting.\n"
 "\n"
-"Click \"mouse-right\" button, or press \"R\" or \"r\" "
+"Click \"mouse-right\" button, or press \"Z\" or \"z\" "
 "keys, to randomize the puzzle (this must be done first\n"
 "to set a new record).\n"
 "\n"
-"Press \"I\" or \"i\" keys to increase the number of \"facets\".\n"
+"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
 "\n"
-"Press \"D\" or \"d\" keys to decrease the number of \"facets\".\n"
+"Press \"W\" or \"w\" keys to save (write) a puzzle.\n"
+"\n"
+"Press \"U\" or \"u\" keys to undo a move.\n"
+"\n"
+"Press \"R\" or \"r\" keys to redo a move.\n"
+"\n"
+"Press \"C\" or \"c\" keys to clear the puzzle.\n"
+"\n"
+"Press \"S\" or \"s\" keys reserved for the auto-solver "
+"(not implemented).\n"
 "\n"
 "Press \"O\" or \"o\" keys to toggle the orient mode.  One "
 "has to orient the faces in orient mode, besides\n"
@@ -175,16 +184,9 @@ static const char featuresHelp[] = {
 "Beware, the \"Sticky\" mode is a hack and much could be done "
 "to improve its look.\n"
 "\n"
-"Press \"S\" or \"s\" keys reserved for the auto-solver "
-"(not implemented).\n"
+"Press \"I\" or \"i\" keys to increase the number of \"facets\".\n"
 "\n"
-"Press \"U\" or \"u\" keys to undo a move.\n"
-"\n"
-"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
-"\n"
-"Press \"W\" or \"w\" keys to save a puzzle.\n"
-"\n"
-"Press \"C\" or \"c\" keys to clear a puzzle.\n"
+"Press \"D\" or \"d\" keys to decrease the number of \"facets\".\n"
 "\n"
 "Press \"Esc\" key to hide program.\n"
 "\n"
@@ -260,12 +262,13 @@ static char titleDsp[TITLELEN] = "";
 #endif
 static char scoreFileName[FILENAMELEN] = SCOREFILE;
 static char fileName[FILENAMELEN];
+static Boolean randomized = False;
 #ifdef WINVER
 #define MAXPROGNAME 80
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap octIcon = None;
+static Pixmap icon = None;
 static Widget topLevel, oct;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
@@ -524,7 +527,7 @@ Widget w
 	char *userName, *scoreFile;
 
 #ifdef WINVER
-	InitializeOct(w, brush);
+	InitializePuzzle(w, brush);
 
 	size = w->oct.size;
 	mode = w->oct.mode;
@@ -582,7 +585,7 @@ Widget w
 #ifdef WINVER
 		(void) strncpy(userNameDsp, userName, USERNAMELEN);
 #else
-		char * login = getlogin();
+		char *login = getlogin();
 
 		if (login == NULL) {
 			(void) strcpy(userNameDsp, "");
@@ -608,10 +611,10 @@ Widget w
 
 #ifdef WINVER
 void
-SetOct(OctWidget w, int reason)
+setPuzzle(OctWidget w, int reason)
 #else
 static void
-CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
+CallbackPuzzle(Widget w, caddr_t clientData, octCallbackStruct *callData)
 #endif
 {
 #ifndef WINVER
@@ -640,7 +643,7 @@ CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-		case OCT_HIDE:
+		case PUZZLE_HIDE:
 #ifdef WINVER
 			ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -650,23 +653,22 @@ CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
 #endif
 			break;
 #ifndef WINVER
-		case OCT_PRACTICE_QUERY:
+		case PUZZLE_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(practiceDialog);
 #else
-			XtVaSetValues(oct,
-				XtNmenu, 11, NULL); /* menu choice */
+			XtVaSetValues(oct, XtNmenu, MENU_PRACTICE, NULL);
 #endif
 			break;
-		case OCT_RANDOMIZE_QUERY:
+		case PUZZLE_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(randomizeDialog);
 #else
-			XtVaSetValues(oct, XtNmenu, 5, NULL); /* menu choice */
+			XtVaSetValues(oct, XtNmenu, MENU_RANDOMIZE, NULL);
 #endif
 			break;
 #endif
-		case OCT_SOLVE_MESSAGE:
+		case PUZZLE_SOLVE_MESSAGE:
 #ifdef WINVER
 			(void) MessageBox(w->core.hWnd, solveHelp,
 				"Auto-Solve", MB_OK);
@@ -678,7 +680,7 @@ CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case OCT_RESTORE:
+		case PUZZLE_RESTORE:
 			if (practice) {
 				(void) strncpy(recordDsp, "practice",
 					MESSAGELEN);
@@ -687,29 +689,31 @@ CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
 #endif
 			}
 			movesDsp = 0;
+			randomized = False;
 			break;
-		case OCT_RESET:
+		case PUZZLE_RESET:
 			movesDsp = 0;
+			randomized = False;
 			break;
-		case OCT_AMBIGUOUS:
+		case PUZZLE_AMBIGUOUS:
 			(void) strncpy(messageDsp, "Ambiguous move",
 				MESSAGELEN);
 			break;
-		case OCT_ILLEGAL:
-			if (practice || start)
+		case PUZZLE_ILLEGAL:
+			if (practice || randomized)
 				(void) strncpy(messageDsp, "Illegal move",
 					MESSAGELEN);
 			else
-				(void) strncpy(messageDsp, "Randomize to start",
-					MESSAGELEN);
+				(void) strncpy(messageDsp,
+					"Randomize to start", MESSAGELEN);
 			break;
-		case OCT_MOVED:
+		case PUZZLE_MOVED:
 			movesDsp++;
 			SET_STARTED(w, True);
 			break;
-		case OCT_CONTROL:
+		case PUZZLE_CONTROL:
 			return;
-		case OCT_SOLVED:
+		case PUZZLE_SOLVED:
 			if (practice)
 				movesDsp = 0;
 			else if (cheat)
@@ -722,9 +726,11 @@ CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
 				(void) strncpy(messageDsp, "Solved!",
 					MESSAGELEN);
 			SET_STARTED(w, False);
+			randomized = False;
 			break;
-		case OCT_PRACTICE:
+		case PUZZLE_PRACTICE:
 			movesDsp = 0;
+			randomized = False;
 			practice = !practice;
 			if (!practice)
 				(void) strncpy(messageDsp, "Randomize to start",
@@ -742,8 +748,9 @@ CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case OCT_RANDOMIZE:
+		case PUZZLE_RANDOMIZE:
 			movesDsp = 0;
+			randomized = True;
 #ifdef WINVER
 			w->oct.practice = False;
 			w->oct.started = False;
@@ -753,7 +760,7 @@ CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
 				XtNstart, False, NULL);
 #endif
 			break;
-		case OCT_DEC:
+		case PUZZLE_DEC:
 			if (!sticky) {
 				movesDsp = 0;
 				size--;
@@ -771,7 +778,7 @@ CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
 #endif
 			}
 			break;
-		case OCT_ORIENT:
+		case PUZZLE_ORIENTIZE:
 			movesDsp = 0;
 			orient = !orient;
 			PrintRecord(size, mode, orient, sticky, practice);
@@ -784,7 +791,7 @@ CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case OCT_INC:
+		case PUZZLE_INC:
 			if (!sticky) {
 				movesDsp = 0;
 				size++;
@@ -802,11 +809,11 @@ CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
 #endif
 			}
 			break;
-		case OCT_PERIOD3:
-		case OCT_PERIOD4:
-		case OCT_BOTH:
+		case PUZZLE_PERIOD3:
+		case PUZZLE_PERIOD4:
+		case PUZZLE_BOTH:
 			movesDsp = 0;
-			mode = reason - OCT_PERIOD3 + PERIOD3;
+			mode = reason - PUZZLE_PERIOD3 + PERIOD3;
 			PrintRecord(size, mode, orient, sticky, practice);
 #ifdef WINVER
 			w->oct.mode = mode;
@@ -817,7 +824,7 @@ CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case OCT_STICKY:
+		case PUZZLE_STICKY:
 			movesDsp = 0;
 			sticky = !sticky;
 			if (sticky)
@@ -838,13 +845,17 @@ CallbackOct(Widget w, caddr_t clientData, octCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case OCT_COMPUTED:
+		case PUZZLE_COMPUTED:
 			SET_STARTED(w, False);
 			break;
-		case OCT_UNDO:
+		case PUZZLE_UNDO:
 			movesDsp--;
 			SET_STARTED(w, True);
 			break;
+		case PUZZLE_REDO:
+			movesDsp++;
+			SET_STARTED(w, True);
+			break;
 	}
 	PrintStatus(messageDsp, movesDsp
 #ifndef HAVE_MOTIF
@@ -879,15 +890,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	if (GetFocus()) {
 		if (!widget.oct.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			EnterOct(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			EnterPuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	} else {
 		if (widget.oct.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			LeaveOct(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			LeavePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	}
@@ -896,35 +909,39 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			Initialize(&widget, brush);
 			break;
 		case WM_DESTROY:
-			DestroyOct(brush);
+			DestroyPuzzle(brush);
 			break;
 		case WM_SIZE:
-			ResizeOct(&widget);
+			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
 		case WM_PAINT:
 			widget.core.hDC = BeginPaint(hWnd, &paint);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ExposeOct(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ExposePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 			break;
 		case WM_RBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			RandomizeOct(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			SelectOct(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			SelectPuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONUP:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ReleaseOct(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ReleasePuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
@@ -932,7 +949,8 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
 		case WM_MOUSEWHEEL:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
 			{
 				int zDelta = ((short) HIWORD(wParam));
 				POINT cursor, origin;
@@ -941,13 +959,13 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				ClientToScreen(hWnd, &origin);
 				(void) GetCursorPos(&cursor);
 				if (zDelta > (WHEEL_DELTA >> 1)) {
-					MoveOctInput(&widget,
+					MovePuzzleInput(&widget,
 						cursor.x - origin.x, cursor.y - origin.y,
 						TOP,
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 				} else if (zDelta < -(WHEEL_DELTA >> 1)) {
-					MoveOctInput(&widget,
+					MovePuzzleInput(&widget,
 						cursor.x - origin.x, cursor.y - origin.y,
 						BOTTOM,
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
@@ -960,56 +978,67 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		case WM_COMMAND:
 			switch (LOWORD(wParam)) {
 				case IDM_GET:
-					GetOct(&widget);
-					ResizeOct(&widget);
+					GetPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_WRITE:
-					WriteOct(&widget);
+					WritePuzzle(&widget);
 					break;
 				case IDM_EXIT:
-					DestroyOct(brush);
+					DestroyPuzzle(brush);
 					break;
 				case IDM_HIDE:
-					HideOct(&widget);
-					break;
-				case IDM_CLEAR:
-					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					ClearOct(&widget);
-					(void) ReleaseDC(hWnd, widget.core.hDC);
+					HidePuzzle(&widget);
 					break;
 				case IDM_UNDO:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					UndoOct(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					UndoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_REDO:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RedoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_CLEAR:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					ClearPuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_RANDOMIZE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					RandomizeOct(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RandomizePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_PRACTICE:
-					PracticeOct(&widget);
-					SizeOct(&widget);
+					PracticePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SOLVE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					SolveOct(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					SolvePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
-				case IDM_ORIENT:
-					OrientizeOct(&widget);
-					SizeOct(&widget);
+				case IDM_ORIENTIZE:
+					OrientizePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_STICKY:
-					(void) StickyModeOct(&widget);
-					SizeOct(&widget);
+					(void) StickyModePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SHIFT_TOP:
@@ -1037,11 +1066,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
-						(void) MoveOctInput(&widget,
+						(void) MovePuzzleInput(&widget,
 							cursor.x - origin.x, cursor.y - origin.y,
 							(int) LOWORD(wParam) - ((shift) ? IDM_SHIFT_TOP : IDM_TOP),
 							shift, FALSE);
@@ -1074,11 +1104,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
-						(void) MoveOctInput(&widget,
+						(void) MovePuzzleInput(&widget,
 							cursor.x - origin.x, cursor.y - origin.y,
 							(int) LOWORD(wParam) - ((shift) ? IDM_SHIFTCONTROL_TOP : IDM_CONTROL_TOP),
 							shift, TRUE);
@@ -1087,21 +1118,21 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 
 					break;
 				case IDM_DEC:
-					if (DecrementOct(&widget)) {
-						SizeOct(&widget);
+					if (DecrementPuzzle(&widget)) {
+						SizePuzzle(&widget);
 						(void) InvalidateRect(hWnd, NULL, TRUE);
 					}
 					break;
 				case IDM_INC:
-					IncrementOct(&widget);
-					SizeOct(&widget);
+					IncrementPuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_PERIOD3:
 				case IDM_PERIOD4:
 				case IDM_BOTH:
-					PeriodModeOct(&widget, (int) LOWORD(wParam) - IDM_PERIOD3);
-					SizeOct(&widget);
+					PeriodModePuzzle(&widget, (int) LOWORD(wParam) - IDM_PERIOD3);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_ABOUT:
@@ -1179,7 +1210,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
 #else
 
 static void
-Usage(char * programName)
+Usage(char *programName)
 {
 	unsigned int i;
 
@@ -1214,6 +1245,8 @@ static XrmOptionDescRec options[] =
 	{(char *) "-face5", (char *) "*oct.faceColor5", XrmoptionSepArg, NULL},
 	{(char *) "-face6", (char *) "*oct.faceColor6", XrmoptionSepArg, NULL},
 	{(char *) "-face7", (char *) "*oct.faceColor7", XrmoptionSepArg, NULL},
+	{(char *) "-fn", (char *) "*oct.font", XrmoptionSepArg, NULL},
+	{(char *) "-font", (char *) "*oct.font", XrmoptionSepArg, NULL},
 	{(char *) "-size", (char *) "*oct.size", XrmoptionSepArg, NULL},
 	{(char *) "-sticky", (char *) "*oct.sticky", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-mode", (char *) "*oct.mode", XrmoptionSepArg, NULL},
@@ -1222,8 +1255,6 @@ static XrmOptionDescRec options[] =
 	{(char *) "-noorient", (char *) "*oct.orient", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-practice", (char *) "*oct.practice", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-nopractice", (char *) "*oct.practice", XrmoptionNoArg, (char *) "FALSE"},
-	{(char *) "-fn", (char *) "*oct.font", XrmoptionSepArg, NULL},
-	{(char *) "-font", (char *) "*oct.font", XrmoptionSepArg, NULL},
 	{(char *) "-userName", (char *) "*oct.userName", XrmoptionSepArg, NULL},
 	{(char *) "-scoreFile", (char *) "*oct.scoreFile", XrmoptionSepArg, NULL},
 	{(char *) "-scores", (char *) "*oct.scoreOnly", XrmoptionNoArg, (char *) "TRUE"},
@@ -1232,22 +1263,22 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackOctPractice(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzlePractice(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(oct, XtNmenu, 11, NULL); /* menu choice */
+		XtVaSetValues(oct, XtNmenu, MENU_PRACTICE, NULL);
 	}
 }
 
 static void
-CallbackOctRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzleRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(oct, XtNmenu, 5, NULL); /* menu choice */
+		XtVaSetValues(oct, XtNmenu, MENU_RANDOMIZE, NULL);
 	}
 }
 static void
-FacetSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+FacetSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int size = cbs->value, mode;
 	Boolean orient, sticky, practice;
@@ -1274,7 +1305,7 @@ FacetSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct * cbs)
+ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct *cbs)
 {
 	int size;
 	Boolean orient, sticky, practice;
@@ -1297,7 +1328,7 @@ ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct * cbs)
 }
 
 static void
-OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	int size, mode;
 	Boolean orient = cbs->set, sticky, practice;
@@ -1318,7 +1349,7 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
 }
 
 static void
-StickyToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+StickyToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	int size, mode;
 	Boolean orient, sticky = cbs->set, practice;
@@ -1350,7 +1381,7 @@ StickyToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
 
 static void
 PracticeToggle(Widget w, XtPointer clientData,
-		XmToggleButtonCallbackStruct * cbs)
+		XmToggleButtonCallbackStruct *cbs)
 {
 	int size, mode;
 	Boolean orient, sticky, practice = cbs->set;
@@ -1379,9 +1410,9 @@ fileCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	if (val == 2)
+	if (val == MENU_EXIT)
 		exit(0);
-	XtVaSetValues(oct, XtNmenu, val, NULL);
+	XtVaSetValues(oct, XtNmenu, val, NULL); /* GWQ */
 }
 
 static void
@@ -1389,7 +1420,7 @@ playCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	XtVaSetValues(oct, XtNmenu, val + 3, NULL); /* GWQ */
+	XtVaSetValues(oct, XtNmenu, val + MENU_EXIT + 1, NULL);
 }
 
 static Widget
@@ -1488,9 +1519,10 @@ main(int argc, char **argv)
 	Widget switchRowCol, messageRowCol;
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
-	XmString clearString, undoString, randomizeString, solveString;
+	XmString undoString, redoString;
+	XmString clearString, randomizeString, practiceString, solveString;
+	XmString orientizeString, stickyString;
 	XmString incrementString, decrementString;
-	XmString orientizeString, stickyString, practiceString;
 	unsigned int i;
 #endif
 
@@ -1500,7 +1532,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#if HAVE_XPM
+#ifdef HAVE_XPM
 	{
 		XpmAttributes xpmAttributes;
 		XpmColorSymbol transparentColor[1] = {{NULL,
@@ -1515,18 +1547,18 @@ main(int argc, char **argv)
 		transparentColor[0].pixel = bg;
 		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) oct_xpm, &octIcon, NULL,
+			(char **) oct_xpm, &icon, NULL,
 			&xpmAttributes);
 	}
-	if (octIcon == (Pixmap) NULL)
+	if (icon == (Pixmap) NULL)
 #endif
-		octIcon = XCreateBitmapFromData(XtDisplay(topLevel),
+		icon = XCreateBitmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
 			(char *) oct_bits,
 			oct_width, oct_height);
 #ifdef HAVE_MOTIF
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, octIcon,
+		XtNiconPixmap, icon,
 		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
@@ -1553,35 +1585,38 @@ main(int argc, char **argv)
 	XmStringFree(getString);
 	XmStringFree(writeString);
 	XmStringFree(quitString);
-	clearString = XmStringCreateSimple((char *) "Clear");
 	undoString = XmStringCreateSimple((char *) "Undo");
-	randomizeString = XmStringCreateSimple((char *) "Randomize");
+	redoString = XmStringCreateSimple((char *) "Redo");
+	clearString = XmStringCreateSimple((char *) "Clear");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
+	practiceString = XmStringCreateSimple((char *) "Practice");
 	solveString = XmStringCreateSimple((char *) "(Solve)");
-	incrementString = XmStringCreateSimple((char *) "Increment");
-	decrementString = XmStringCreateSimple((char *) "Decrement");
 	orientizeString = XmStringCreateSimple((char *) "Orientize");
 	stickyString = XmStringCreateSimple((char *) "StickY");
-	practiceString = XmStringCreateSimple((char *) "Practice");
+	incrementString = XmStringCreateSimple((char *) "Increment");
+	decrementString = XmStringCreateSimple((char *) "Decrement");
 	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
-		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
+		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
-		XmVaPUSHBUTTON, incrementString, 'I', NULL, NULL,
-		XmVaPUSHBUTTON, decrementString, 'D', NULL, NULL,
 		XmVaPUSHBUTTON, orientizeString, 'O', NULL, NULL,
 		XmVaPUSHBUTTON, stickyString, 'Y', NULL, NULL,
-		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
+		XmVaPUSHBUTTON, incrementString, 'I', NULL, NULL,
+		XmVaPUSHBUTTON, decrementString, 'D', NULL, NULL,
 		NULL);
-	XmStringFree(clearString);
 	XmStringFree(undoString);
+	XmStringFree(redoString);
+	XmStringFree(clearString);
 	XmStringFree(randomizeString);
 	XmStringFree(solveString);
-	XmStringFree(incrementString);
-	XmStringFree(decrementString);
+	XmStringFree(practiceString);
 	XmStringFree(orientizeString);
 	XmStringFree(stickyString);
-	XmStringFree(practiceString);
+	XmStringFree(incrementString);
+	XmStringFree(decrementString);
 	pullDownMenu = XmCreatePulldownMenu(menuBar,
 		(char *) "helpPullDown", NULL, 0);
 	widget = XtVaCreateManagedWidget("Help",
@@ -1625,11 +1660,11 @@ main(int argc, char **argv)
 	practiceDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to toggle the practice mode?",
 		(char *) "Practice Query",
-		(XtCallbackProc) CallbackOctPractice);
+		(XtCallbackProc) CallbackPuzzlePractice);
 	randomizeDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to randomize?",
 		(char *) "Randomize Query",
-		(XtCallbackProc) CallbackOctRandomize);
+		(XtCallbackProc) CallbackPuzzleRandomize);
 	mainPanel = XtCreateManagedWidget("mainPanel",
 		xmPanedWindowWidgetClass, menuBarPanel,
 		NULL, 0);
@@ -1751,13 +1786,13 @@ main(int argc, char **argv)
 		octWidgetClass, mainPanel, NULL, 0);
 #else
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, octIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 	oct = XtCreateManagedWidget("oct",
 		octWidgetClass, topLevel, NULL, 0);
 #endif
 	XtAddCallback(oct, XtNselectCallback,
-		(XtCallbackProc) CallbackOct, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	Initialize(oct);
 	XtRealizeWidget(topLevel);
 	XGrabButton(XtDisplay(oct), (unsigned int) AnyButton, AnyModifier,
