@@ -57,38 +57,39 @@ static char pyramidColorChar[MAXSTACKS - 1] =
 #endif
 #endif
 
-static Boolean SetValuesPanex(Widget current, Widget request, Widget renew);
-static void QuitPanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void DestroyPanex(Widget old);
-static void ResizePanex(PanexWidget w);
-static void SizePanex(PanexWidget w);
-static void InitializePanex(Widget request, Widget renew);
-static void ExposePanex(Widget renew, XEvent * event, Region region);
-static void HidePanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void SelectPanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void MotionPanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void ReleasePanex(PanexWidget w, XEvent * event, char **args,
-	int nArgs);
-static void GetPanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void WritePanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void ClearPanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void ClearPanexMaybe(PanexWidget w, XEvent * event, char **args,
-	int nArgs);
-static void ClearPanex2(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void UndoPanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void SolvePanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void SpeedPanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void SlowPanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void SoundPanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void IncrementPanex(PanexWidget w, XEvent * event, char **args,
-	int nArgs);
-static void DecrementPanex(PanexWidget w, XEvent * event, char **args,
-	int nArgs);
-static void ModePanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void EnterPanex(PanexWidget w, XEvent * event, char **args, int nArgs);
-static void LeavePanex(PanexWidget w, XEvent * event, char **args, int nArgs);
+static Boolean SetValuesPuzzle(Widget current, Widget request, Widget renew);
+static void QuitPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void DestroyPuzzle(Widget old);
+static void ResizePuzzle(PanexWidget w);
+static void SizePuzzle(PanexWidget w);
+static void InitializePuzzle(Widget request, Widget renew);
+static void ExposePuzzle(Widget renew, XEvent *event, Region region);
+static void HidePuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void SelectPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void MotionPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void ReleasePuzzle(PanexWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ClearPuzzleMaybe(PanexWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ClearPuzzle2(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void GetPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void WritePuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void UndoPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void RedoPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void ClearPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void SolvePuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void SpeedPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void SlowPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void SoundPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void IncrementPuzzle(PanexWidget w,
+	XEvent *event, char **args, int nArgs);
+static void DecrementPuzzle(PanexWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ModePuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void EnterPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
+static void LeavePuzzle(PanexWidget w, XEvent *event, char **args, int nArgs);
 
-static char defaultTranslationsPanex[] =
+static char defaultTranslationsPuzzle[] =
 "<KeyPress>q: Quit()\n\
  Ctrl<KeyPress>C: Quit()\n\
  <KeyPress>osfCancel: Hide()\n\
@@ -99,13 +100,13 @@ static char defaultTranslationsPanex[] =
  <Btn1Down>: Select()\n\
  <Btn1Motion>: Motion()\n\
  <Btn1Up>: Release()\n\
- <KeyPress>r: Clear()\n\
- <KeyPress>c: Clear()\n\
  <Btn3Down>: ClearMaybe()\n\
  <Btn3Down>(2+): Clear2()\n\
  <KeyPress>g: Get()\n\
  <KeyPress>w: Write()\n\
  <KeyPress>u: Undo()\n\
+ <KeyPress>r: Redo()\n\
+ <KeyPress>c: Clear()\n\
  <KeyPress>s: Solve()\n\
  <KeyPress>0x2E: Speed()\n\
  <KeyPress>0x3E: Speed()\n\
@@ -118,31 +119,32 @@ static char defaultTranslationsPanex[] =
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
-static XtActionsRec actionsListPanex[] =
+static XtActionsRec actionsListPuzzle[] =
 {
-	{(char *) "Quit", (XtActionProc) QuitPanex},
-	{(char *) "Hide", (XtActionProc) HidePanex},
-	{(char *) "Select", (XtActionProc) SelectPanex},
-	{(char *) "Motion", (XtActionProc) MotionPanex},
-	{(char *) "Release", (XtActionProc) ReleasePanex},
-	{(char *) "Clear", (XtActionProc) ClearPanex},
-	{(char *) "ClearMaybe", (XtActionProc) ClearPanexMaybe},
-	{(char *) "Clear2", (XtActionProc) ClearPanex2},
-	{(char *) "Get", (XtActionProc) GetPanex},
-	{(char *) "Write", (XtActionProc) WritePanex},
-	{(char *) "Undo", (XtActionProc) UndoPanex},
-	{(char *) "Solve", (XtActionProc) SolvePanex},
-	{(char *) "Speed", (XtActionProc) SpeedPanex},
-	{(char *) "Slow", (XtActionProc) SlowPanex},
-	{(char *) "Sound", (XtActionProc) SoundPanex},
-	{(char *) "Increment", (XtActionProc) IncrementPanex},
-	{(char *) "Decrement", (XtActionProc) DecrementPanex},
-	{(char *) "Mode", (XtActionProc) ModePanex},
-	{(char *) "Enter", (XtActionProc) EnterPanex},
-	{(char *) "Leave", (XtActionProc) LeavePanex}
+	{(char *) "Quit", (XtActionProc) QuitPuzzle},
+	{(char *) "Hide", (XtActionProc) HidePuzzle},
+	{(char *) "Select", (XtActionProc) SelectPuzzle},
+	{(char *) "Motion", (XtActionProc) MotionPuzzle},
+	{(char *) "Release", (XtActionProc) ReleasePuzzle},
+	{(char *) "ClearMaybe", (XtActionProc) ClearPuzzleMaybe},
+	{(char *) "Clear2", (XtActionProc) ClearPuzzle2},
+	{(char *) "Get", (XtActionProc) GetPuzzle},
+	{(char *) "Write", (XtActionProc) WritePuzzle},
+	{(char *) "Undo", (XtActionProc) UndoPuzzle},
+	{(char *) "Redo", (XtActionProc) RedoPuzzle},
+	{(char *) "Clear", (XtActionProc) ClearPuzzle},
+	{(char *) "Solve", (XtActionProc) SolvePuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
+	{(char *) "Increment", (XtActionProc) IncrementPuzzle},
+	{(char *) "Decrement", (XtActionProc) DecrementPuzzle},
+	{(char *) "Mode", (XtActionProc) ModePuzzle},
+	{(char *) "Enter", (XtActionProc) EnterPuzzle},
+	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
 
-static XtResource resourcesPanex[] =
+static XtResource resourcesPuzzle[] =
 {
 	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
 	 XtOffset(PanexWidget, core.width),
@@ -229,29 +231,29 @@ PanexClassRec panexClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializePanex,	/* initialize */
+		(XtInitProc) InitializePuzzle,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
-		actionsListPanex,	/* actions */
-		XtNumber(actionsListPanex),	/* num actions */
-		resourcesPanex,	/* resources */
-		XtNumber(resourcesPanex),	/* num resources */
+		actionsListPuzzle,	/* actions */
+		XtNumber(actionsListPuzzle),	/* num actions */
+		resourcesPuzzle,	/* resources */
+		XtNumber(resourcesPuzzle),	/* num resources */
 		NULLQUARK,	/* xrm class */
 		TRUE,		/* compress motion */
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
-		(XtWidgetProc) DestroyPanex,	/* destroy */
-		(XtWidgetProc) ResizePanex,	/* resize */
-		(XtExposeProc) ExposePanex,	/* expose */
-		(XtSetValuesFunc) SetValuesPanex,	/* set values */
+		(XtWidgetProc) DestroyPuzzle,	/* destroy */
+		(XtWidgetProc) ResizePuzzle,	/* resize */
+		(XtExposeProc) ExposePuzzle,	/* expose */
+		(XtSetValuesFunc) SetValuesPuzzle,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
 		NULL,		/* accept focus */
 		XtVersion,	/* version */
 		NULL,		/* callback private */
-		defaultTranslationsPanex,	/* tm table */
+		defaultTranslationsPuzzle,	/* tm table */
 		NULL,		/* query geometry */
 		NULL,		/* display accelerator */
 		NULL		/* extension */
@@ -264,7 +266,7 @@ PanexClassRec panexClassRec =
 WidgetClass panexWidgetClass = (WidgetClass) & panexClassRec;
 
 void
-SetPanex(PanexWidget w, int reason)
+setPuzzle(PanexWidget w, int reason)
 {
 	panexCallbackStruct cb;
 
@@ -285,8 +287,8 @@ loadFont(PanexWidget w)
 		XUnloadFont(XtDisplay(w), w->panex.fontInfo->fid);
 		XFreeFont(XtDisplay(w), w->panex.fontInfo);
 	}
-	if ((w->panex.fontInfo = XLoadQueryFont(display,
-			w->panex.font)) == NULL) {
+	if (w->panex.font && (w->panex.fontInfo =
+			XLoadQueryFont(display, w->panex.font)) == NULL) {
 		(void) sprintf(buf,
 			"Can not open %s font.\nAttempting %s font as alternate\n",
 			w->panex.font, altfontname);
@@ -338,6 +340,9 @@ static Point trapazoidUnit[5] =
 
 #define MULTDELAY 48
 
+static PanexStack undo = {NULL, NULL, NULL, 0};
+static PanexStack redo = {NULL, NULL, NULL, 0};
+
 static void
 CheckTiles(PanexWidget w)
 {
@@ -905,7 +910,7 @@ DrawFrame(PanexWidget w, Pixmap dr, Boolean focus)
 static void
 MoveNoTiles(PanexWidget w)
 {
-	SetPanex(w, PANEX_IGNORE);
+	setPuzzle(w, PUZZLE_IGNORE);
 }
 
 int
@@ -1161,7 +1166,8 @@ ResetTiles(PanexWidget w)
 			for (loc = 1; loc <= w->panex.tiles; loc++)
 				w->panex.tileOfPosition[stack][loc] = i;
 	}
-	FlushMoves(w);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	w->panex.started = False;
 	w->panex.cheat = False;
 }
@@ -1209,15 +1215,16 @@ GetTiles(PanexWidget w)
 		}
 #endif
 	}
-	FlushMoves(w);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &tiles);
 	if (tiles >= MINTILES) {
 		for (i = w->panex.tiles; i < tiles; i++) {
-			SetPanex(w, PANEX_INC);
+			setPuzzle(w, PUZZLE_INC);
 		}
 		for (i = w->panex.tiles; i > tiles; i--) {
-			SetPanex(w, PANEX_DEC);
+			setPuzzle(w, PUZZLE_DEC);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: tiles ");
@@ -1236,7 +1243,7 @@ GetTiles(PanexWidget w)
 	(void) fscanf(fp, "%d", &mode);
 	if (mode >= HANOI && mode <= PANEX) {
 		if (w->panex.mode != mode) {
-			SetPanex(w, PANEX_MODE);
+			setPuzzle(w, PUZZLE_MODE);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: mode ");
@@ -1258,10 +1265,10 @@ GetTiles(PanexWidget w)
 #endif
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
-	ScanStartPosition(fp, w);
-	SetPanex(w, PANEX_RESTORE);
-	SetStartPosition(w);
-	ScanMoves(fp, w, moves);
+	scanStartPosition(fp, w);
+	setPuzzle(w, PUZZLE_RESTORE);
+	setStartPosition(w);
+	scanMoves(fp, w, moves);
 	(void) fclose(fp);
 	if (mode == 0)
 		(void) printf("%s: mode hanoi, tiles %d, moves %d.\n",
@@ -1318,34 +1325,25 @@ WriteTiles(PanexWidget w)
 	}
 	(void) fprintf(fp, "tiles%c %d\n", SYMBOL, w->panex.tiles);
 	(void) fprintf(fp, "mode%c %d\n", SYMBOL, w->panex.mode);
-	(void) fprintf(fp, "moves%c %d\n", SYMBOL, NumMoves());
-	PrintStartPosition(fp, w);
-	PrintMoves(fp);
+	(void) fprintf(fp, "moves%c %d\n", SYMBOL,
+		numMoves(&undo));
+	printStartPosition(fp, w);
+	printMoves(fp, &undo);
 	(void) fclose(fp);
 	(void) printf("Saved to %s.\n", name);
 	free(lname);
 	free(fname);
 }
 
-static void
-ClearTiles(PanexWidget w)
-{
-	if (w->panex.currentStack >= 0)
-		return;
-	EraseFrame(w, 0, w->panex.focus);
-	ResetTiles(w);
-	DrawSlots(w, 0);
-	DrawAllTiles(w);
-	SetPanex(w, PANEX_RESET);
-}
-
 static void
 UndoTiles(PanexWidget w)
 {
-	if (MadeMoves() && w->panex.currentStack < 0) {
+	if (madeMoves(&undo) &&
+			w->panex.currentStack < 0) {
 		int fromStack, fromPosition, toStack;
 
-		GetMove(&toStack, &fromStack);
+		getMove(&undo, &toStack, &fromStack);
+		setMove(&redo, toStack, fromStack);
 		if ((fromPosition = TopOfStack(w, fromStack, 0)) < 0 ||
 				MoveTile(w, fromStack, fromPosition, toStack,
 					DOUBLE) < 0) {
@@ -1361,11 +1359,52 @@ UndoTiles(PanexWidget w)
 			DISPLAY_WARNING(buf2);
 			free(buf2);
 		} else {
-			SetPanex(w, PANEX_UNDO);
+			setPuzzle(w, PUZZLE_UNDO);
 		}
 	}
 }
 
+static void
+RedoTiles(PanexWidget w)
+{
+	if (madeMoves(&redo) &&
+			w->panex.currentStack < 0) {
+		int fromStack, fromPosition, toStack;
+
+		getMove(&redo, &fromStack, &toStack);
+		setMove(&undo, fromStack, toStack);
+		if ((fromPosition = TopOfStack(w, fromStack, 0)) < 0 ||
+				MoveTile(w, fromStack, fromPosition, toStack,
+					DOUBLE) < 0) {
+			char *buf1, *buf2;
+
+			intCat(&buf1, "Move from ", fromStack);
+			stringCat(&buf2, buf1, " to ");
+			free(buf1);
+			intCat(&buf1, buf2, toStack);
+			free(buf2);
+			stringCat(&buf2, buf1, " can not be made");
+			free(buf1);
+			DISPLAY_WARNING(buf2);
+			free(buf2);
+		} else {
+			setPuzzle(w, PUZZLE_REDO);
+		}
+	}
+}
+
+static void
+ClearTiles(PanexWidget w)
+{
+	if (w->panex.currentStack >= 0)
+		return;
+	EraseFrame(w, 0, w->panex.focus);
+	ResetTiles(w);
+	DrawSlots(w, 0);
+	DrawAllTiles(w);
+	setPuzzle(w, PUZZLE_RESET);
+}
+
 static void
 SolveTiles(PanexWidget w)
 {
@@ -1374,7 +1413,7 @@ SolveTiles(PanexWidget w)
 	ResetTiles(w);
 	DrawSlots(w, 0);
 	DrawAllTiles(w);
-	SetPanex(w, PANEX_RESET);
+	setPuzzle(w, PUZZLE_RESET);
 	SolveTilesFromStart(w);
 }
 
@@ -1385,7 +1424,7 @@ SpeedTiles(PanexWidget w)
 	if (w->panex.delay < 0)
 		w->panex.delay = 0;
 #ifdef HAVE_MOTIF
-	SetPanex(w, PANEX_SPEED);
+	setPuzzle(w, PUZZLE_SPEED);
 #endif
 }
 
@@ -1394,7 +1433,7 @@ SlowTiles(PanexWidget w)
 {
 	w->panex.delay += 5;
 #ifdef HAVE_MOTIF
-	SetPanex(w, PANEX_SPEED);
+	setPuzzle(w, PUZZLE_SPEED);
 #endif
 }
 
@@ -1407,7 +1446,7 @@ SoundTiles(PanexWidget w)
 static void
 IncrementTiles(PanexWidget w)
 {
-	SetPanex(w, PANEX_INC);
+	setPuzzle(w, PUZZLE_INC);
 }
 
 static void
@@ -1415,25 +1454,26 @@ DecrementTiles(PanexWidget w)
 {
 	if (w->panex.tiles <= MINTILES)
 		return;
-	SetPanex(w, PANEX_DEC);
+	setPuzzle(w, PUZZLE_DEC);
 }
 
 static void
 ModeTiles(PanexWidget w)
 {
-	SetPanex(w, PANEX_MODE);
+	setPuzzle(w, PUZZLE_MODE);
 }
 
 int
-MovePanex(PanexWidget w, int fromStack, int fromPosition, int toStack,
+MovePuzzle(PanexWidget w, int fromStack, int fromPosition, int toStack,
 		int fast)
 {
 	int	 toPosition;
 
 	if ((toPosition = MoveTile(w, fromStack, fromPosition, toStack,
 			fast)) >= 0) {
-		SetPanex(w, PANEX_MOVED);
-		PutMove(fromStack, toStack);
+		setPuzzle(w, PUZZLE_MOVED);
+		setMove(&undo, fromStack, toStack);
+		flushMoves(w, &redo, FALSE);
 	}
 	return toPosition;
 }
@@ -1461,7 +1501,7 @@ darker(const int light)
 
 
 static void
-SetValuesPanex(PanexWidget w)
+SetValuesPuzzle(PanexWidget w)
 {
 	struct tagColor {
 		int red, green, blue;
@@ -1540,8 +1580,10 @@ SetValuesPanex(PanexWidget w)
 }
 
 void
-DestroyPanex(HBRUSH brush)
+DestroyPuzzle(HBRUSH brush)
 {
+	deleteMoves(&undo);
+	deleteMoves(&redo);
 	(void) DeleteObject(brush);
 	PostQuitMessage(0);
 }
@@ -1734,7 +1776,7 @@ SetAllColors(PanexWidget w)
 }
 
 static Boolean
-SetValuesPanex(Widget current, Widget request, Widget renew)
+SetValuesPuzzle(Widget current, Widget request, Widget renew)
 {
 	PanexWidget c = (PanexWidget) current, w = (PanexWidget) renew;
 	Boolean redraw = False;
@@ -1769,7 +1811,7 @@ SetValuesPanex(Widget current, Widget request, Widget renew)
 	}
 	if (w->panex.tiles != c->panex.tiles) {
 		EraseFrame(c, 0, False);
-		SizePanex(w);
+		SizePuzzle(w);
 		redraw = True;
 	} else if (w->panex.mode != c->panex.mode) {
 		ResetTiles(w);
@@ -1777,7 +1819,7 @@ SetValuesPanex(Widget current, Widget request, Widget renew)
 	} else if (w->panex.puzzleOffset.x != c->panex.puzzleOffset.x ||
 			w->panex.puzzleOffset.y != c->panex.puzzleOffset.y) {
 		EraseFrame(c, 0, False);
-		ResizePanex(w);
+		ResizePuzzle(w);
 		redraw = True;
 	}
 	if (w->panex.delay != c->panex.delay) {
@@ -1789,37 +1831,40 @@ SetValuesPanex(Widget current, Widget request, Widget renew)
 
 		w->panex.menu = -1;
 		switch (menu) {
-		case 0:
+		case MENU_GET:
 			GetTiles(w);
 			break;
-		case 1:
+		case MENU_WRITE:
 			WriteTiles(w);
 			break;
-		case 3:
-			ClearTiles(w);
-			break;
-		case 4:
+		case MENU_UNDO:
 			UndoTiles(w);
 			break;
-		case 5:
+		case MENU_REDO:
+			RedoTiles(w);
+			break;
+		case MENU_CLEAR:
+			ClearTiles(w);
+			break;
+		case MENU_SOLVE:
 			SolveTiles(w);
 			break;
-		case 6:
-			IncrementTiles(w);
-			break;
-		case 7:
-			DecrementTiles(w);
-			break;
-		case 8:
+		case MENU_MODE:
 			ModeTiles(w);
 			break;
-		case 9:
+		case MENU_INCREMENT:
+			IncrementTiles(w);
+			break;
+		case MENU_DECREMENT:
+			DecrementTiles(w);
+			break;
+		case MENU_SPEED:
 			SpeedTiles(w);
 			break;
-		case 10:
+		case MENU_SLOW:
 			SlowTiles(w);
 			break;
-		case 11:
+		case MENU_SOUND:
 			SoundTiles(w);
 			break;
 
@@ -1837,22 +1882,10 @@ SetValuesPanex(Widget current, Widget request, Widget renew)
 }
 
 static void
-QuitPanex(PanexWidget w, XEvent * event, char **args, int nArgs)
-{
-	Display *display = XtDisplay(w);
-
-	if (w->panex.fontInfo) {
-		XUnloadFont(display, w->panex.fontInfo->fid);
-		XFreeFont(display, w->panex.fontInfo);
-	}
-	XtCloseDisplay(display);
-	exit(0);
-}
-
-static void
-DestroyPanex(Widget old)
+DestroyPuzzle(Widget old)
 {
 	PanexWidget w = (PanexWidget) old;
+	Display *display = XtDisplay(w);
 	int pyramid;
 
 	for (pyramid = 0; pyramid <= w->panex.mode; pyramid++)
@@ -1864,6 +1897,19 @@ DestroyPanex(Widget old)
 	XtReleaseGC(old, w->panex.frameGC);
 	XtReleaseGC(old, w->panex.inverseGC);
 	XtRemoveCallbacks(old, XtNselectCallback, w->panex.select);
+	if (w->panex.fontInfo) {
+		XUnloadFont(display, w->panex.fontInfo->fid);
+		XFreeFont(display, w->panex.fontInfo);
+	}
+	deleteMoves(&undo);
+	deleteMoves(&redo);
+}
+
+static void
+QuitPuzzle(PanexWidget w, XEvent *event, char **args, int nArgs)
+{
+	XtCloseDisplay(XtDisplay(w));
+	exit(0);
 }
 #endif
 
@@ -1919,7 +1965,7 @@ ResizeTiles(PanexWidget w)
 static
 #endif
 void
-ResizePanex(PanexWidget w)
+ResizePuzzle(PanexWidget w)
 {
 #ifdef WINVER
 	RECT rect;
@@ -1951,17 +1997,17 @@ ResizePanex(PanexWidget w)
 static
 #endif
 void
-SizePanex(PanexWidget w)
+SizePuzzle(PanexWidget w)
 {
 	ResetTiles(w);
-	ResizePanex(w);
+	ResizePuzzle(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-InitializePanex(
+InitializePuzzle(
 #ifdef WINVER
 PanexWidget w, HBRUSH brush
 #else
@@ -1971,7 +2017,7 @@ Widget request, Widget renew
 {
 	int stack;
 #ifdef WINVER
-	SetValuesPanex(w);
+	SetValuesPuzzle(w);
 	brush = CreateSolidBrush(w->panex.inverseGC);
 	SETBACK(w->core.hWnd, brush);
 	(void) SRAND(time(NULL));
@@ -2001,18 +2047,19 @@ Widget request, Widget renew
 		w->panex.positionOfTile[stack] = NULL;
 	}
 	CheckTiles(w);
-	InitMoves();
+	newMoves(&undo);
+	newMoves(&redo);;
 	w->panex.numSlices = ((w->panex.delay < MAXSLICES) ?
 		w->panex.delay + 1 : MAXSLICES);
-	SizePanex(w);
+	SizePuzzle(w);
 }
 
 void
-ExposePanex(
+ExposePuzzle(
 #ifdef WINVER
 PanexWidget w
 #else
-Widget renew, XEvent * event, Region region
+Widget renew, XEvent *event, Region region
 #endif
 )
 {
@@ -2032,24 +2079,24 @@ Widget renew, XEvent * event, Region region
 static
 #endif
 void
-HidePanex(PanexWidget w
+HidePuzzle(PanexWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
-	SetPanex(w, PANEX_HIDE);
+	setPuzzle(w, PUZZLE_HIDE);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-SelectPanex(PanexWidget w
+SelectPuzzle(PanexWidget w
 #ifdef WINVER
 , const int x
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2083,11 +2130,11 @@ SelectPanex(PanexWidget w
 static
 #endif
 void
-MotionPanex(PanexWidget w
+MotionPuzzle(PanexWidget w
 #ifdef WINVER
 , const int x
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2098,7 +2145,6 @@ MotionPanex(PanexWidget w
 
 	if (w->panex.currentStack < 0)
 		return;
-	
 	if ((toStack = SelectTile(w, x)) >= 0 &&
 			toStack != w->panex.currentStack) {
 		if (MoveTileToLimbo(w, w->panex.currentStack,
@@ -2132,11 +2178,11 @@ MotionPanex(PanexWidget w
 static
 #endif
 void
-ReleasePanex(PanexWidget w
+ReleasePuzzle(PanexWidget w
 #ifdef WINVER
 , const int x
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2159,38 +2205,46 @@ ReleasePanex(PanexWidget w
 				w->panex.currentPosition, toStack,
 				NORMAL)) >= 0) {
 			if (w->panex.previousStack != toStack) {
-				SetPanex(w, PANEX_MOVED);
+				setPuzzle(w, PUZZLE_MOVED);
 				if (w->panex.previousStack >= 0) {
-				  PutMove(w->panex.previousStack, toStack);
+				  setMove(&undo,
+				    w->panex.previousStack, toStack);
+				  flushMoves(w, &redo, FALSE);
 				} else {
-				  PutMove(w->panex.currentStack, toStack);
+				  setMove(&undo,
+				    w->panex.currentStack, toStack);
+				  flushMoves(w, &redo, FALSE);
 				}
 				if (CheckSolved(w)) {
-					SetPanex(w, PANEX_SOLVED);
+					setPuzzle(w, PUZZLE_SOLVED);
 				}
 			}
 		} else if (toPosition == -1) {
-		  SetPanex(w, PANEX_BLOCKED);
+		  setPuzzle(w, PUZZLE_BLOCKED);
 		  if (w->panex.previousStack != -1) {
 		    if ((toPosition = MoveTile(w, w->panex.currentStack,
 			w->panex.currentPosition,
 			w->panex.currentStack, NORMAL)) >= 0) {
 		      if (w->panex.previousStack != toStack &&
 			  w->panex.previousStack != w->panex.currentStack) {
-				SetPanex(w, PANEX_MOVED);
+				setPuzzle(w, PUZZLE_MOVED);
 				if (w->panex.previousStack >= 0) {
-				  PutMove(w->panex.previousStack, toStack);
+				  setMove(&undo,
+				    w->panex.previousStack, toStack);
+				  flushMoves(w, &redo, FALSE);
 				} else {
-				  PutMove(w->panex.currentStack, toStack);
+				  setMove(&undo,
+				    w->panex.currentStack, toStack);
+				  flushMoves(w, &redo, FALSE);
 				}
 				if (CheckSolved(w)) {
-					SetPanex(w, PANEX_SOLVED);
+					setPuzzle(w, PUZZLE_SOLVED);
 				}
 			}
 		    }
 		  }
 		} else {
-		  SetPanex(w, PANEX_ILLEGAL);
+		  setPuzzle(w, PUZZLE_ILLEGAL);
 		  if (w->panex.previousStack != -1) {
 		    toPosition = MoveTile(w, w->panex.currentStack,
 			w->panex.currentPosition,
@@ -2202,40 +2256,10 @@ ReleasePanex(PanexWidget w
 	w->panex.currentStack = -1;
 }
 
-void
-GetPanex(PanexWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	GetTiles(w);
-}
-
-void
-WritePanex(PanexWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	WriteTiles(w);
-}
-
-void
-ClearPanex(PanexWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	ClearTiles(w);
-}
-
 #ifndef WINVER
 void
-ClearPanexMaybe(PanexWidget w
-, XEvent * event, char **args, int nArgs
+ClearPuzzleMaybe(PanexWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 	if (!w->panex.started) {
@@ -2244,14 +2268,14 @@ ClearPanexMaybe(PanexWidget w
 #ifdef HAVE_MOTIF
 	else {
 		/* Check if one really wants to destroy current state. */
-		SetPanex(w, PANEX_RESET_QUERY);
+		setPuzzle(w, PUZZLE_CLEAR_QUERY);
 	}
 #endif
 }
 
 void
-ClearPanex2(PanexWidget w
-, XEvent * event, char **args, int nArgs
+ClearPuzzle2(PanexWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 #ifndef HAVE_MOTIF
@@ -2261,9 +2285,29 @@ ClearPanex2(PanexWidget w
 #endif
 
 void
-UndoPanex(PanexWidget w
+GetPuzzle(PanexWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	GetTiles(w);
+}
+
+void
+WritePuzzle(PanexWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	WriteTiles(w);
+}
+
+void
+UndoPuzzle(PanexWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2271,9 +2315,29 @@ UndoPanex(PanexWidget w
 }
 
 void
-SolvePanex(PanexWidget w
+RedoPuzzle(PanexWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RedoTiles(w);
+}
+
+void
+ClearPuzzle(PanexWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	ClearTiles(w);
+}
+
+void
+SolvePuzzle(PanexWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2281,9 +2345,9 @@ SolvePanex(PanexWidget w
 }
 
 void
-ModePanex(PanexWidget w
+ModePuzzle(PanexWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2294,9 +2358,9 @@ ModePanex(PanexWidget w
 static
 #endif
 void
-SpeedPanex(PanexWidget w
+SpeedPuzzle(PanexWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2307,9 +2371,9 @@ SpeedPanex(PanexWidget w
 static
 #endif
 void
-SlowPanex(PanexWidget w
+SlowPuzzle(PanexWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2320,9 +2384,9 @@ SlowPanex(PanexWidget w
 static
 #endif
 void
-SoundPanex(PanexWidget w
+SoundPuzzle(PanexWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2330,9 +2394,9 @@ SoundPanex(PanexWidget w
 }
 
 void
-IncrementPanex(PanexWidget w
+IncrementPuzzle(PanexWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2340,9 +2404,9 @@ IncrementPanex(PanexWidget w
 }
 
 void
-DecrementPanex(PanexWidget w
+DecrementPuzzle(PanexWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2350,9 +2414,9 @@ DecrementPanex(PanexWidget w
 }
 
 void
-EnterPanex(PanexWidget w
+EnterPuzzle(PanexWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2361,9 +2425,9 @@ EnterPanex(PanexWidget w
 }
 
 void
-LeavePanex(PanexWidget w
+LeavePuzzle(PanexWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
