@@ -76,22 +76,37 @@ typedef struct {
 } panexCallbackStruct;
 #endif
 
-#define PANEX_HIDE (-8)
-#define PANEX_RESET_QUERY (-7)
-#define PANEX_RESTORE (-6)
-#define PANEX_RESET (-5)
-#define PANEX_ILLEGAL (-4)
-#define PANEX_BLOCKED (-3)
-#define PANEX_SPACE (-2)
-#define PANEX_IGNORE (-1)
-#define PANEX_MOVED 0
-#define PANEX_SOLVED 1
-#define PANEX_DEC 2
-#define PANEX_INC 3
-#define PANEX_MODE 4
-#define PANEX_COMPUTED 5
-#define PANEX_UNDO 6
-#define PANEX_SPEED 7
+#define PUZZLE_HIDE (-8)
+#define PUZZLE_CLEAR_QUERY (-7)
+#define PUZZLE_RESTORE (-6)
+#define PUZZLE_RESET (-5)
+#define PUZZLE_ILLEGAL (-4)
+#define PUZZLE_BLOCKED (-3)
+#define PUZZLE_SPACE (-2)
+#define PUZZLE_IGNORE (-1)
+#define PUZZLE_MOVED 0
+#define PUZZLE_SOLVED 1
+#define PUZZLE_DEC 2
+#define PUZZLE_INC 3
+#define PUZZLE_MODE 4
+#define PUZZLE_COMPUTED 5
+#define PUZZLE_UNDO 6
+#define PUZZLE_REDO 7
+#define PUZZLE_SPEED 8
+
+#define MENU_GET 0
+#define MENU_WRITE 1
+#define MENU_EXIT 2
+#define MENU_UNDO 3
+#define MENU_REDO 4
+#define MENU_CLEAR 5
+#define MENU_SOLVE 6
+#define MENU_MODE 7
+#define MENU_INCREMENT 8
+#define MENU_DECREMENT 9
+#define MENU_SPEED 10
+#define MENU_SLOW 11
+#define MENU_SOUND 12
 
 #define MINTILES 1
 #define DEFAULTTILES 10
