@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1996 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1996 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -33,28 +33,29 @@
 #include "Panex.h"
 
 #ifdef WINVER
-extern void DestroyPanex(HBRUSH brush);
-extern void ResizePanex(PanexWidget w);
-extern void SizePanex(PanexWidget w);
-extern void InitializePanex(PanexWidget w, HBRUSH brush);
-extern void ExposePanex(PanexWidget w);
-extern void HidePanex(PanexWidget w);
-extern void SelectPanex(PanexWidget w, const int x);
-extern void MotionPanex(PanexWidget w, const int x);
-extern void ReleasePanex(PanexWidget w, const int x);
-extern void ClearPanex(PanexWidget w);
-extern void GetPanex(PanexWidget w);
-extern void WritePanex(PanexWidget w);
-extern void UndoPanex(PanexWidget w);
-extern void SolvePanex(PanexWidget w);
-extern void ModePanex(PanexWidget w);
-extern void SpeedPanex(PanexWidget w);
-extern void SlowPanex(PanexWidget w);
-extern void SoundPanex(PanexWidget w);
-extern void DecrementPanex(PanexWidget w);
-extern void IncrementPanex(PanexWidget w);
-extern void EnterPanex(PanexWidget w);
-extern void LeavePanex(PanexWidget w);
+extern void DestroyPuzzle(HBRUSH brush);
+extern void ResizePuzzle(PanexWidget w);
+extern void SizePuzzle(PanexWidget w);
+extern void InitializePuzzle(PanexWidget w, HBRUSH brush);
+extern void ExposePuzzle(PanexWidget w);
+extern void HidePuzzle(PanexWidget w);
+extern void SelectPuzzle(PanexWidget w, const int x);
+extern void MotionPuzzle(PanexWidget w, const int x);
+extern void ReleasePuzzle(PanexWidget w, const int x);
+extern void GetPuzzle(PanexWidget w);
+extern void WritePuzzle(PanexWidget w);
+extern void UndoPuzzle(PanexWidget w);
+extern void RedoPuzzle(PanexWidget w);
+extern void ClearPuzzle(PanexWidget w);
+extern void SolvePuzzle(PanexWidget w);
+extern void ModePuzzle(PanexWidget w);
+extern void SpeedPuzzle(PanexWidget w);
+extern void SlowPuzzle(PanexWidget w);
+extern void SoundPuzzle(PanexWidget w);
+extern void DecrementPuzzle(PanexWidget w);
+extern void IncrementPuzzle(PanexWidget w);
+extern void EnterPuzzle(PanexWidget w);
+extern void LeavePuzzle(PanexWidget w);
 
 #else
 
@@ -93,8 +94,25 @@ extern PanexClassRec panexClassRec;
 #define DOUBLE 2
 #define INSTANT 3
 
+typedef struct _MoveRecord {
+#if 0
+	int from, to;
+#endif
+	unsigned char packed;	/* This makes assumptions on the data. */
+} MoveRecord;
+
+typedef struct _MoveStack {
+	MoveRecord move;
+	struct _MoveStack *previous, *next;
+} MoveStack;
+
+typedef struct _PanexStack {
+	MoveStack *currMove, *lastMove, *firstMove;
+	int count;
+} PanexStack;
+
 typedef struct _PanexLoc {
-	int         stack, loc;
+	int stack, loc;
 } PanexLoc;
 
 typedef struct _PanexPart {
@@ -136,8 +154,8 @@ typedef struct _PanexRec {
 } PanexRec;
 
 extern PanexLoc *startLoc[MAXSTACKS - 1];
-extern void SetPanex(PanexWidget w, int reason);
-extern int  MovePanex(PanexWidget w, int fromStack, int fromPosition,
+extern void setPuzzle(PanexWidget w, int reason);
+extern int  MovePuzzle(PanexWidget w, int fromStack, int fromPosition,
 	int toStack, int fast);
 
 extern void SolveTilesFromStart(PanexWidget w);
@@ -148,13 +166,17 @@ extern Boolean CheckSolved(PanexWidget w);
 extern void InitMoves(void);
 extern void PutMove(int from, int to);
 extern void GetMove(int *from, int *to);
-extern int  MadeMoves(void);
-extern void FlushMoves(PanexWidget w);
-extern int  NumMoves(void);
-extern void ScanMoves(FILE * fp, PanexWidget w, int moves);
-extern void PrintMoves(FILE * fp);
-extern void ScanStartPosition(FILE * fp, PanexWidget w);
-extern void PrintStartPosition(FILE * fp, PanexWidget w);
-extern void SetStartPosition(PanexWidget w);
+extern void newMoves(PanexStack *s);
+extern void deleteMoves(PanexStack *s);
+extern void setMove(PanexStack *s, int from, int to);
+extern void getMove(PanexStack *s, int *from, int *to);
+extern int madeMoves(PanexStack *s);
+extern void flushMoves(PanexWidget w, PanexStack *s, Boolean undo);
+extern int numMoves(PanexStack *s);
+extern void scanMoves(FILE *fp, PanexWidget w, int moves);
+extern void printMoves(FILE *fp, PanexStack *s);
+extern void scanStartPosition(FILE *fp, PanexWidget w);
+extern void printStartPosition(FILE *fp, PanexWidget w);
+extern void setStartPosition(PanexWidget w);
 
 #endif /* _PanexP_h */
