@@ -16,7 +16,7 @@
 #  Dubrovsky.
 #  Break ability taken from the X puzzle by Don Bennett, HP Labs
 #
-#  Copyright (c) 1996 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1996 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -131,7 +131,7 @@ MoveATile(PanexWidget w, int fromStack, int toStack)
 		longjmp(solve_env, 1);
 #endif
 	if ((fromPosition = TopOfStack(w, fromStack, 0)) < 0 ||
-	    MovePanex(w, fromStack, fromPosition, toStack, NORMAL) < 0) {
+	    MovePuzzle(w, fromStack, fromPosition, toStack, NORMAL) < 0) {
 		(void) printf("move from %d to %d can not be made.",
 			      fromStack, toStack);
 		return False;
@@ -201,7 +201,7 @@ SolveHanoiMode(PanexWidget w)
 #endif
 	SolvingFlag = False;
 	w->panex.cheat = True; /* Assume the worst. */
-	SetPanex(w, PANEX_COMPUTED);
+	setPuzzle(w, PUZZLE_COMPUTED);
 }
 
 #ifdef DEBUG
@@ -1312,7 +1312,7 @@ SolvePanexMode(PanexWidget w)
 #endif
 	SolvingFlag = False;
 	w->panex.cheat = True; /* Assume the worst. */
-	SetPanex(w, PANEX_COMPUTED);
+	setPuzzle(w, PUZZLE_COMPUTED);
 #ifdef DEBUG
 	counter = 0;
 #endif
