@@ -1,11 +1,11 @@
 /*-
 # MS WINDOWS-BASED PANEX(tm)
 #
-#  wpanexi.h
+#  wpanex.h
 #
 ###
 #
-#  Copyright (c) 1996 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1996 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -29,13 +29,14 @@
 #define IDM_WRITE	101
 #define IDM_EXIT	102
 #define IDM_HIDE	103
-#define IDM_CLEAR	200
-#define IDM_UNDO	201
-#define IDM_SOLVE	202
-#define IDM_MODE	203
-#define IDM_SPEED       204
-#define IDM_SLOW        205
-#define IDM_SOUND       206
+#define IDM_UNDO	200
+#define IDM_REDO	201
+#define IDM_CLEAR	202
+#define IDM_SOLVE	205
+#define IDM_MODE	206
+#define IDM_SPEED       210
+#define IDM_SLOW        211
+#define IDM_SOUND       212
 #define IDM_DEC		400
 #define IDM_INC		401
 #define IDM_ABOUT	500
