@@ -86,7 +86,7 @@ static PanexRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Panex Version 7.2\n"
+"Panex Version 7.2.4\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -129,8 +129,19 @@ static const char featuresHelp[] = {
 "another column to move the tile to that column.  It will "
 "not move if blocked.\n"
 "\n"
-"Click \"mouse-right\" button, or press \"C\", \"c\", \"R\","
-"or \"r\" keys, to clear/reset the puzzle.\n"
+"Click \"mouse-right\" button, or press \"C\" or \"c\" keys, "
+"to clear the puzzle.\n"
+"\n"
+"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
+"\n"
+"Press \"W\" or \"w\" keys to save (write) a puzzle.\n"
+"\n"
+"Press \"U\" or \"u\" keys to undo a move.\n"
+"\n"
+"Press \"R\" or \"r\" keys to redo a move.\n"
+"\n"
+"Press \"S\" or \"s\" keys to auto-solve.  Unfortunately, its "
+"only implemented from the starting position.\n"
 "\n"
 "Press \"M\" or \"m\" keys to toggle between hanoi "
 "(one pyramid column) and panex (two pyramid columns) modes\n"
@@ -145,15 +156,6 @@ static const char featuresHelp[] = {
 "\n"
 "Press \"D\" or \"d\" keys to decrease the number of tiles.\n"
 "\n"
-"Press \"S\" or \"s\" keys to auto-solve.  Unfortunately, its "
-"only implemented from the starting position.\n"
-"\n"
-"Press \"U\" or \"u\" keys to undo a move.\n"
-"\n"
-"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
-"\n"
-"Press \"W\" or \"w\" keys to save a puzzle.\n"
-"\n"
 "Press \">\" or \".\" keys to speed up the movement of tiles.\n"
 "\n"
 "Press \"<\" or \",\" keys to slow down the movement of tiles.\n"
@@ -231,7 +233,7 @@ static char fileName[FILENAMELEN];
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap panexIcon = None;
+static Pixmap icon = None;
 static Widget topLevel, panex;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
@@ -353,7 +355,7 @@ WriteRecords(void)
 		return;
 	}
 	{
-#if HAVE_FCNTL_H
+#ifdef HAVE_FCNTL_H
 		int lfd;
 		char lockFile[FILENAMELEN];
 
@@ -459,7 +461,7 @@ Widget w
 	char *userName, *scoreFile;
 
 #ifdef WINVER
-	InitializePanex(w, brush);
+	InitializePuzzle(w, brush);
 
 	tiles = w->panex.tiles;
 	mode = w->panex.mode;
@@ -514,7 +516,7 @@ Widget w
 #ifdef WINVER
 		(void) strncpy(userNameDsp, userName, USERNAMELEN);
 #else
-		char * login = getlogin();
+		char *login = getlogin();
 
 		if (login == NULL) {
 			(void) strcpy(userNameDsp, "");
@@ -539,10 +541,10 @@ Widget w
 
 #ifdef WINVER
 void
-SetPanex(PanexWidget w, int reason)
+setPuzzle(PanexWidget w, int reason)
 #else
 static void
-CallbackPanex(Widget w, caddr_t clientData, panexCallbackStruct * callData)
+CallbackPuzzle(Widget w, caddr_t clientData, panexCallbackStruct *callData)
 #endif
 {
 #ifndef WINVER
@@ -563,7 +565,7 @@ CallbackPanex(Widget w, caddr_t clientData, panexCallbackStruct * callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-		case PANEX_HIDE:
+		case PUZZLE_HIDE:
 #ifdef WINVER
 			ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -573,41 +575,40 @@ CallbackPanex(Widget w, caddr_t clientData, panexCallbackStruct * callData)
 #endif
 			break;
 #ifndef WINVER
-		case PANEX_RESET_QUERY:
+		case PUZZLE_CLEAR_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(clearDialog);
 #else
-			XtVaSetValues(panex,
-				XtNmenu, 3, NULL); /* menu choice */
+			XtVaSetValues(panex, XtNmenu, MENU_CLEAR, NULL);
 #endif
 			break;
 #endif
-		case PANEX_RESTORE:
-		case PANEX_RESET:
+		case PUZZLE_RESTORE:
+		case PUZZLE_RESET:
 			movesDsp = 0;
 			break;
-		case PANEX_ILLEGAL:
+		case PUZZLE_ILLEGAL:
 			(void) strncpy(messageDsp, "Illegal move", MESSAGELEN);
 			break;
-		case PANEX_BLOCKED:
+		case PUZZLE_BLOCKED:
 			(void) strncpy(messageDsp, "Blocked", MESSAGELEN);
 			break;
-		case PANEX_SPACE:
+		case PUZZLE_SPACE:
 #if 0
 			/* Too annoying */
 			(void) strncpy(messageDsp, "Spaces can not slide",
 				MESSAGELEN));
 #endif
 			break;
-		case PANEX_IGNORE:
+		case PUZZLE_IGNORE:
 			(void) strncpy(messageDsp, "Clear to start",
 				MESSAGELEN);
 			break;
-		case PANEX_MOVED:
+		case PUZZLE_MOVED:
 			movesDsp++;
 			SET_STARTED(w, True);
 			break;
-		case PANEX_SOLVED:
+		case PUZZLE_SOLVED:
 			if (cheat)
 				(void) sprintf(messageDsp,
 					"No cheating %s!!", userNameDsp);
@@ -619,7 +620,7 @@ CallbackPanex(Widget w, caddr_t clientData, panexCallbackStruct * callData)
 					MESSAGELEN);
 			SET_STARTED(w, False);
 			break;
-		case PANEX_DEC:
+		case PUZZLE_DEC:
 			movesDsp = 0;
 			tiles--;
 			PrintRecord(tiles, mode);
@@ -634,7 +635,7 @@ CallbackPanex(Widget w, caddr_t clientData, panexCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case PANEX_INC:
+		case PUZZLE_INC:
 			movesDsp = 0;
 			tiles++;
 			PrintRecord(tiles, mode);
@@ -649,7 +650,7 @@ CallbackPanex(Widget w, caddr_t clientData, panexCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case PANEX_MODE:
+		case PUZZLE_MODE:
 			movesDsp = 0;
 			mode = (mode == PANEX) ? HANOI : PANEX;
 			PrintRecord(tiles, mode);
@@ -663,15 +664,19 @@ CallbackPanex(Widget w, caddr_t clientData, panexCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case PANEX_COMPUTED:
+		case PUZZLE_COMPUTED:
 			SET_STARTED(w, False);
 			break;
-		case PANEX_UNDO:
+		case PUZZLE_UNDO:
 			movesDsp--;
 			SET_STARTED(w, True);
 			break;
+		case PUZZLE_REDO:
+			movesDsp++;
+			SET_STARTED(w, True);
+			break;
 #ifdef HAVE_MOTIF
-		case PANEX_SPEED:
+		case PUZZLE_SPEED:
 			{
 				int oldDelay, delay;
 
@@ -724,15 +729,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	if (GetFocus()) {
 		if (!widget.panex.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			EnterPanex(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			EnterPuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	} else {
 		if (widget.panex.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			LeavePanex(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			LeavePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	}
@@ -741,22 +748,24 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			Initialize(&widget, brush);
 			break;
 		case WM_DESTROY:
-			DestroyPanex(brush);
+			DestroyPuzzle(brush);
 			break;
 		case WM_SIZE:
-			ResizePanex(&widget);
+			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
 		case WM_PAINT:
 			widget.core.hDC = BeginPaint(hWnd, &paint);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ExposePanex(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ExposePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 			break;
 		case WM_RBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ClearPanex(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ClearPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONDOWN:
@@ -764,7 +773,7 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
-			SelectPanex(&widget, LOWORD(lParam));
+			SelectPuzzle(&widget, LOWORD(lParam));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_MOUSEMOVE:
@@ -773,7 +782,7 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
-			MotionPanex(&widget, LOWORD(lParam));
+			MotionPuzzle(&widget, LOWORD(lParam));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONUP:
@@ -781,63 +790,73 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			widget.core.hDC = GetDC(hWnd);
 			(void) SelectObject(widget.core.hDC,
 				GetStockObject(NULL_PEN));
-			ReleasePanex(&widget, LOWORD(lParam));
+			ReleasePuzzle(&widget, LOWORD(lParam));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_COMMAND:
 			switch (LOWORD(wParam)) {
 				case IDM_GET:
-					GetPanex(&widget);
-					ResizePanex(&widget);
+					GetPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_WRITE:
-					WritePanex(&widget);
+					WritePuzzle(&widget);
 					break;
 				case IDM_EXIT:
-					DestroyPanex(brush);
+					DestroyPuzzle(brush);
 					break;
 				case IDM_HIDE:
-					HidePanex(&widget);
-					break;
-				case IDM_CLEAR:
-					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					ClearPanex(&widget);
-					(void) ReleaseDC(hWnd, widget.core.hDC);
+					HidePuzzle(&widget);
 					break;
 				case IDM_UNDO:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					UndoPanex(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					UndoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_REDO:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RedoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_CLEAR:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					ClearPuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_SOLVE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					SolvePanex(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					SolvePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_MODE:
-					(void) ModePanex(&widget);
-					SizePanex(&widget);
+					(void) ModePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_DEC:
 				case IDM_INC:
 					((LOWORD(wParam) == IDM_DEC) ?
-						DecrementPanex(&widget) : IncrementPanex(&widget));
-					SizePanex(&widget);
+						DecrementPuzzle(&widget) : IncrementPuzzle(&widget));
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SPEED:
-					SpeedPanex(&widget);
+					SpeedPuzzle(&widget);
 					break;
 				case IDM_SLOW:
-					SlowPanex(&widget);
+					SlowPuzzle(&widget);
 					break;
 				case IDM_SOUND:
-					SoundPanex(&widget);
+					SoundPuzzle(&widget);
 					break;
 				case IDM_ABOUT:
 					(void) DialogBox(widget.core.hInstance,
@@ -914,7 +933,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
 #else
 
 static void
-Usage(char * programName)
+Usage(char *programName)
 {
 	unsigned int i;
 
@@ -944,16 +963,16 @@ static XrmOptionDescRec options[] =
 	{(char *) "-tile", (char *) "*panex.tileColor", XrmoptionSepArg, NULL},
 	{(char *) "-pyramid0", (char *) "*panex.pyramidColor0", XrmoptionSepArg, NULL},
 	{(char *) "-pyramid1", (char *) "*panex.pyramidColor1", XrmoptionSepArg, NULL},
-	{(char *) "-tiles", (char *) "*panex.tiles", XrmoptionSepArg, NULL},
-	{(char *) "-mode", (char *) "*panex.mode", XrmoptionSepArg, NULL},
-	{(char *) "-hanoi", (char *) "*panex.mode", XrmoptionNoArg, (char *) "0"},
-	{(char *) "-panex", (char *) "*panex.mode", XrmoptionNoArg, (char *) "1"},
 	{(char *) "-delay", (char *) "*panex.delay", XrmoptionSepArg, NULL},
 	{(char *) "-sound", (char *) "*panex.sound", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-nosound", (char *) "*panex.sound", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-moveSound", (char *) "*panex.moveSound", XrmoptionSepArg, NULL},
 	{(char *) "-fn", (char *) "*panex.font", XrmoptionSepArg, NULL},
 	{(char *) "-font", (char *) "*panex.font", XrmoptionSepArg, NULL},
+	{(char *) "-tiles", (char *) "*panex.tiles", XrmoptionSepArg, NULL},
+	{(char *) "-mode", (char *) "*panex.mode", XrmoptionSepArg, NULL},
+	{(char *) "-hanoi", (char *) "*panex.mode", XrmoptionNoArg, (char *) "0"},
+	{(char *) "-panex", (char *) "*panex.mode", XrmoptionNoArg, (char *) "1"},
 	{(char *) "-userName", (char *) "*panex.userName", XrmoptionSepArg, NULL},
 	{(char *) "-scoreFile", (char *) "*panex.scoreFile", XrmoptionSepArg, NULL},
 	{(char *) "-scores", (char *) "*panex.scoreOnly", XrmoptionNoArg, (char *) "TRUE"},
@@ -962,15 +981,15 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackPanexClear(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzleClear(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(panex, XtNmenu, 3, NULL); /* menu choice */
+		XtVaSetValues(panex, XtNmenu, MENU_CLEAR, NULL);
 	}
 }
 
 static void
-TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int tiles = cbs->value, mode, old;
 
@@ -990,7 +1009,7 @@ TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct * cbs)
+ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct *cbs)
 {
 	int tiles, old;
 
@@ -1010,7 +1029,7 @@ ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct * cbs)
 }
 
 static void
-SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MAXSPEED + MINSPEED - cbs->value - 1, oldDelay;
 
@@ -1026,9 +1045,9 @@ fileCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	if (val == 2)
+	if (val == MENU_EXIT)
 		exit(0);
-	XtVaSetValues(panex, XtNmenu, val, NULL);
+	XtVaSetValues(panex, XtNmenu, val, NULL); /* GWQ */
 }
 
 static void
@@ -1036,7 +1055,7 @@ playCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	XtVaSetValues(panex, XtNmenu, val + 3, NULL); /* GWQ */
+	XtVaSetValues(panex, XtNmenu, val + MENU_EXIT + 1, NULL);
 }
 
 static Widget
@@ -1130,12 +1149,13 @@ main(int argc, char **argv)
 {
 #ifdef HAVE_MOTIF
 	Widget menuBar, pullDownMenu, widget;
-	Widget panel, mainPanel, controlPanel;
+	Widget menuBarPanel, mainPanel, controlPanel;
 	Widget movesRowCol, radioRowCol, tileRowCol, messageRowCol;
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
-	XmString clearString, undoString, solveString;
-	XmString incrementString, decrementString, mode;
+	XmString undoString, redoString;
+	XmString clearString, solveString;
+	XmString mode, incrementString, decrementString;
 	XmString speedString, slowString, soundString;
 	unsigned int i;
 #endif
@@ -1147,7 +1167,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#if HAVE_XPM
+#ifdef HAVE_XPM
 	{
 		XpmAttributes xpmAttributes;
 		XpmColorSymbol transparentColor[1] = {{NULL,
@@ -1162,27 +1182,27 @@ main(int argc, char **argv)
 		transparentColor[0].pixel = bg;
 		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) panex_xpm, &panexIcon, NULL,
+			(char **) panex_xpm, &icon, NULL,
 			&xpmAttributes);
 	}
-	if (panexIcon == (Pixmap) NULL)
+	if (icon == (Pixmap) NULL)
 #endif
-		panexIcon = XCreateBitmapFromData(XtDisplay(topLevel),
+		icon = XCreateBitmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
 			(char *) panex_bits,
 			panex_width, panex_height);
 #ifdef HAVE_MOTIF
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, panexIcon,
+		XtNiconPixmap, icon,
 		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
-	panel = XtVaCreateManagedWidget("panel",
+	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
 		XmNseparatorOn, False,
 		XmNsashWidth, 1,
 		XmNsashHeight, 1, NULL);
 	fileString = XmStringCreateSimple((char *) "File");
 	playString = XmStringCreateSimple((char *) "Play");
-	menuBar = XmVaCreateSimpleMenuBar(panel, (char *) "menuBar",
+	menuBar = XmVaCreateSimpleMenuBar(menuBarPanel, (char *) "menuBar",
 		XmVaCASCADEBUTTON, fileString, 'F',
 		XmVaCASCADEBUTTON, playString, 'P',
 		NULL);
@@ -1200,32 +1220,35 @@ main(int argc, char **argv)
 	XmStringFree(getString);
 	XmStringFree(writeString);
 	XmStringFree(quitString);
-	clearString = XmStringCreateSimple((char *) "Clear");
 	undoString = XmStringCreateSimple((char *) "Undo");
+	redoString = XmStringCreateSimple((char *) "Redo");
+	clearString = XmStringCreateSimple((char *) "Clear");
 	solveString = XmStringCreateSimple((char *) "Solve");
+	mode = XmStringCreateSimple((char *) "Mode");
 	incrementString = XmStringCreateSimple((char *) "Increment");
 	decrementString = XmStringCreateSimple((char *) "Decrement");
-	mode = XmStringCreateSimple((char *) "Mode");
 	speedString = XmStringCreateSimple((char *) ">Speed");
 	slowString = XmStringCreateSimple((char *) "<Slow");
 	soundString = XmStringCreateSimple((char *) "@Sound");
 	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
-		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
+		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
+		XmVaPUSHBUTTON, mode, 'M', NULL, NULL,
 		XmVaPUSHBUTTON, incrementString, 'I', NULL, NULL,
 		XmVaPUSHBUTTON, decrementString, 'D', NULL, NULL,
-		XmVaPUSHBUTTON, mode, 'M', NULL, NULL,
 		XmVaPUSHBUTTON, speedString, '>', NULL, NULL,
 		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
 		XmVaPUSHBUTTON, soundString, '@', NULL, NULL,
 		NULL);
-	XmStringFree(clearString);
 	XmStringFree(undoString);
+	XmStringFree(redoString);
+	XmStringFree(clearString);
 	XmStringFree(solveString);
+	XmStringFree(mode);
 	XmStringFree(incrementString);
 	XmStringFree(decrementString);
-	XmStringFree(mode);
 	XmStringFree(speedString);
 	XmStringFree(slowString);
 	XmStringFree(soundString);
@@ -1270,9 +1293,9 @@ main(int argc, char **argv)
 	clearDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to start over?",
 		(char *) "Clear Query",
-		(XtCallbackProc) CallbackPanexClear);
+		(XtCallbackProc) CallbackPuzzleClear);
 	mainPanel = XtCreateManagedWidget("mainPanel",
-		xmPanedWindowWidgetClass, panel,
+		xmPanedWindowWidgetClass, menuBarPanel,
 		NULL, 0);
 	controlPanel = XtVaCreateManagedWidget("controlPanel",
 		xmPanedWindowWidgetClass, mainPanel,
@@ -1382,13 +1405,13 @@ main(int argc, char **argv)
 		panexWidgetClass, mainPanel, NULL, 0);
 #else
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, panexIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 	panex = XtCreateManagedWidget("panex",
 		panexWidgetClass, topLevel, NULL, 0);
 #endif
 	XtAddCallback(panex, XtNselectCallback,
-		(XtCallbackProc) CallbackPanex, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	Initialize(panex);
 	XtRealizeWidget(topLevel);
 	XGrabButton(XtDisplay(panex), (unsigned int) AnyButton, AnyModifier,
