@@ -79,28 +79,44 @@ typedef struct {
 } pyraminxCallbackStruct;
 #endif
 
-#define PYRAMINX_HIDE (-8)
-#define PYRAMINX_PRACTICE_QUERY (-7)
-#define PYRAMINX_RANDOMIZE_QUERY (-6)
-#define PYRAMINX_SOLVE_MESSAGE (-5)
-#define PYRAMINX_RESTORE (-4)
-#define PYRAMINX_RESET (-3)
-#define PYRAMINX_AMBIGUOUS (-2)
-#define PYRAMINX_ILLEGAL (-1)
-#define PYRAMINX_MOVED 0
-#define PYRAMINX_CONTROL 1
-#define PYRAMINX_SOLVED 2
-#define PYRAMINX_PRACTICE 3
-#define PYRAMINX_RANDOMIZE 4
-#define PYRAMINX_DEC 5
-#define PYRAMINX_ORIENT 6
-#define PYRAMINX_INC 7
-#define PYRAMINX_PERIOD2 8
-#define PYRAMINX_PERIOD3 9
-#define PYRAMINX_BOTH 10
-#define PYRAMINX_STICKY 11
-#define PYRAMINX_COMPUTED 12
-#define PYRAMINX_UNDO 13
+#define PUZZLE_HIDE (-9)
+#define PUZZLE_PRACTICE_QUERY (-8)
+#define PUZZLE_RANDOMIZE_QUERY (-7)
+#define PUZZLE_SOLVE_MESSAGE (-6)
+#define PUZZLE_RESTORE (-5)
+#define PUZZLE_RESET (-4)
+#define PUZZLE_AMBIGUOUS (-3)
+#define PUZZLE_ILLEGAL (-2)
+#define PUZZLE_IGNORE (-1)
+#define PUZZLE_MOVED 0
+#define PUZZLE_CONTROL 1
+#define PUZZLE_SOLVED 2
+#define PUZZLE_PRACTICE 3
+#define PUZZLE_RANDOMIZE 4
+#define PUZZLE_DEC 5
+#define PUZZLE_ORIENTIZE 6
+#define PUZZLE_INC 7
+#define PUZZLE_PERIOD2 8
+#define PUZZLE_PERIOD3 9
+#define PUZZLE_BOTH 10
+#define PUZZLE_STICKY 11
+#define PUZZLE_COMPUTED 12
+#define PUZZLE_UNDO 13
+#define PUZZLE_REDO 14
+
+#define MENU_GET 0
+#define MENU_WRITE 1
+#define MENU_EXIT 2
+#define MENU_UNDO 3
+#define MENU_REDO 4
+#define MENU_CLEAR 5
+#define MENU_RANDOMIZE 6
+#define MENU_PRACTICE 7
+#define MENU_SOLVE 8
+#define MENU_ORIENTIZE 9
+#define MENU_STICKY 10
+#define MENU_INCREMENT 11
+#define MENU_DECREMENT 12
 
 #define MINFACETS 1
 #define PERIOD2 2
