@@ -90,7 +90,7 @@ static PyraminxRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Pyraminx Version 7.2.1\n"
+"Pyraminx Version 7.2.4\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -154,13 +154,22 @@ static const char featuresHelp[] = {
 "record should say \"practice\").  This is good for learning "
 "moves and experimenting.\n"
 "\n"
-"Click \"mouse-right\" button, or press \"R\" or \"r\" "
+"Click \"mouse-right\" button, or press \"Z\" or \"z\" "
 "keys, to randomize the puzzle (this must be done first\n"
 "to set a new record).\n"
 "\n"
-"Press \"I\" or \"i\" keys to increase the number of \"facets\".\n"
+"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
 "\n"
-"Press \"D\" or \"d\" keys to decrease the number of \"facets\".\n"
+"Press \"W\" or \"w\" keys to save (write) a puzzle.\n"
+"\n"
+"Press \"U\" or \"u\" keys to undo a move.\n"
+"\n"
+"Press \"R\" or \"r\" keys to undo a move.\n"
+"\n"
+"Press \"C\" or \"c\" keys to clear the puzzle.\n"
+"\n"
+"Press \"S\" or \"s\" keys to start auto-solver.  Only works on "
+"1x1x1, 2x2x2, and 3x3x3 pyrmaminxs in Period 3 mode.\n"
 "\n"
 "Press \"O\" or \"o\" keys to toggle the orient mode.  One "
 "has to orient the faces in orient mode, besides\n"
@@ -185,16 +194,9 @@ static const char featuresHelp[] = {
 "Beware, the \"Sticky\" mode is a hack and much could be done "
 "to improve its look.\n"
 "\n"
-"Press \"S\" or \"s\" keys to start auto-solver.  Only works on "
-"1x1x1, 2x2x2, and 3x3x3 pyrmaminxs in Period 3 mode.\n"
+"Press \"I\" or \"i\" keys to increase the number of \"facets\".\n"
 "\n"
-"Press \"U\" or \"u\" keys to undo a move.\n"
-"\n"
-"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
-"\n"
-"Press \"W\" or \"w\" keys to save a puzzle.\n"
-"\n"
-"Press \"C\" or \"c\" keys to clear a puzzle.\n"
+"Press \"D\" or \"d\" keys to decrease the number of \"facets\".\n"
 "\n"
 "Press \"Esc\" key to hide program.\n"
 "\n"
@@ -281,12 +283,13 @@ static char titleDsp[TITLELEN] = "";
 #endif
 static char scoreFileName[FILENAMELEN] = SCOREFILE;
 static char fileName[FILENAMELEN];
+static Boolean randomized = False;
 #ifdef WINVER
 #define MAXPROGNAME 80
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap pyraminxIcon = None;
+static Pixmap icon = None;
 static Widget topLevel, pyraminx;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
@@ -547,7 +550,7 @@ Widget w
 	char *userName, *scoreFile;
 
 #ifdef WINVER
-	InitializePyraminx(w, brush);
+	InitializePuzzle(w, brush);
 
 	size = w->pyraminx.size;
 	mode = w->pyraminx.mode;
@@ -612,7 +615,7 @@ Widget w
 #ifdef WINVER
 		(void) strncpy(userNameDsp, userName, USERNAMELEN);
 #else
-		char * login = getlogin();
+		char *login = getlogin();
 
 		if (login == NULL) {
 			(void) strcpy(userNameDsp, "");
@@ -638,11 +641,11 @@ Widget w
 
 #ifdef WINVER
 void
-SetPyraminx(PyraminxWidget w, int reason)
+setPuzzle(PyraminxWidget w, int reason)
 #else
 static void
-CallbackPyraminx(Widget w, caddr_t clientData,
-		pyraminxCallbackStruct * callData)
+CallbackPuzzle(Widget w, caddr_t clientData,
+		pyraminxCallbackStruct *callData)
 #endif
 {
 #ifndef WINVER
@@ -671,7 +674,7 @@ CallbackPyraminx(Widget w, caddr_t clientData,
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-		case PYRAMINX_HIDE:
+		case PUZZLE_HIDE:
 #ifdef WINVER
 			ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -681,24 +684,22 @@ CallbackPyraminx(Widget w, caddr_t clientData,
 #endif
 			break;
 #ifndef WINVER
-		case PYRAMINX_PRACTICE_QUERY:
+		case PUZZLE_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(practiceDialog);
 #else
-			XtVaSetValues(pyraminx,
-				XtNmenu, 11, NULL); /* menu choice */
+			XtVaSetValues(pyraminx, XtNmenu, MENU_PRACTICE, NULL);
 #endif
 			break;
-		case PYRAMINX_RANDOMIZE_QUERY:
+		case PUZZLE_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(randomizeDialog);
 #else
-			XtVaSetValues(pyraminx,
-				XtNmenu, 5, NULL); /* menu choice */
+			XtVaSetValues(pyraminx, XtNmenu, MENU_RANDOMIZE, NULL);
 #endif
 			break;
 #endif
-		case PYRAMINX_SOLVE_MESSAGE:
+		case PUZZLE_SOLVE_MESSAGE:
 #ifdef WINVER
 			(void) MessageBox(w->core.hWnd, solveHelp,
 				"Auto-Solve", MB_OK);
@@ -710,7 +711,7 @@ CallbackPyraminx(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case PYRAMINX_RESTORE:
+		case PUZZLE_RESTORE:
 			if (practice) {
 				(void) strncpy(recordDsp, "practice",
 					MESSAGELEN);
@@ -719,29 +720,31 @@ CallbackPyraminx(Widget w, caddr_t clientData,
 #endif
 			}
 			movesDsp = 0;
+			randomized = False;
 			break;
-		case PYRAMINX_RESET:
+		case PUZZLE_RESET:
 			movesDsp = 0;
+			randomized = False;
 			break;
-		case PYRAMINX_AMBIGUOUS:
+		case PUZZLE_AMBIGUOUS:
 			(void) strncpy(messageDsp, "Ambiguous move",
 				MESSAGELEN);
 			break;
-		case PYRAMINX_ILLEGAL:
-			if (practice || start)
+		case PUZZLE_ILLEGAL:
+			if (practice || randomized)
 				(void) strncpy(messageDsp, "Illegal move",
 					MESSAGELEN);
 			else
-				(void) strncpy(messageDsp, "Randomize to start",
-					MESSAGELEN);
+				(void) strncpy(messageDsp,
+					"Randomize to start", MESSAGELEN);
 			break;
-		case PYRAMINX_MOVED:
+		case PUZZLE_MOVED:
 			movesDsp++;
 			SET_STARTED(w, True);
 			break;
-		case PYRAMINX_CONTROL:
+		case PUZZLE_CONTROL:
 			return;
-		case PYRAMINX_SOLVED:
+		case PUZZLE_SOLVED:
 			if (practice)
 				movesDsp = 0;
 			else if (cheat)
@@ -754,9 +757,11 @@ CallbackPyraminx(Widget w, caddr_t clientData,
 				(void) strncpy(messageDsp, "Solved!",
 					MESSAGELEN);
 			SET_STARTED(w, False);
+			randomized = False;
 			break;
-		case PYRAMINX_PRACTICE:
+		case PUZZLE_PRACTICE:
 			movesDsp = 0;
+			randomized = False;
 			practice = !practice;
 			if (!practice)
 				(void) strncpy(messageDsp, "Randomize to start",
@@ -774,8 +779,9 @@ CallbackPyraminx(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case PYRAMINX_RANDOMIZE:
+		case PUZZLE_RANDOMIZE:
 			movesDsp = 0;
+			randomized = True;
 #ifdef WINVER
 			w->pyraminx.practice = False;
 			w->pyraminx.started = False;
@@ -785,7 +791,7 @@ CallbackPyraminx(Widget w, caddr_t clientData,
 				XtNstart, False, NULL);
 #endif
 			break;
-		case PYRAMINX_DEC:
+		case PUZZLE_DEC:
 			if (!sticky) {
 				movesDsp = 0;
 				size--;
@@ -803,7 +809,7 @@ CallbackPyraminx(Widget w, caddr_t clientData,
 #endif
 			}
 			break;
-		case PYRAMINX_ORIENT:
+		case PUZZLE_ORIENTIZE:
 			movesDsp = 0;
 			orient = !orient;
 			PrintRecord(size, mode, orient, sticky, practice);
@@ -816,7 +822,7 @@ CallbackPyraminx(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case PYRAMINX_INC:
+		case PUZZLE_INC:
 			if (!sticky) {
 				movesDsp = 0;
 				size++;
@@ -834,11 +840,11 @@ CallbackPyraminx(Widget w, caddr_t clientData,
 #endif
 			}
 			break;
-		case PYRAMINX_PERIOD2:
-		case PYRAMINX_PERIOD3:
-		case PYRAMINX_BOTH:
+		case PUZZLE_PERIOD2:
+		case PUZZLE_PERIOD3:
+		case PUZZLE_BOTH:
 			movesDsp = 0;
-			mode = reason - PYRAMINX_PERIOD2 + PERIOD2;
+			mode = reason - PUZZLE_PERIOD2 + PERIOD2;
 			PrintRecord(size, mode, orient, sticky, practice);
 #ifdef WINVER
 			w->pyraminx.mode = mode;
@@ -849,7 +855,7 @@ CallbackPyraminx(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case PYRAMINX_STICKY:
+		case PUZZLE_STICKY:
 			movesDsp = 0;
 			sticky = !sticky;
 			if (sticky)
@@ -870,10 +876,10 @@ CallbackPyraminx(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case PYRAMINX_COMPUTED:
+		case PUZZLE_COMPUTED:
 			SET_STARTED(w, False);
 			break;
-		case PYRAMINX_UNDO:
+		case PUZZLE_UNDO:
 			movesDsp--;
 			SET_STARTED(w, True);
 			break;
@@ -911,15 +917,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	if (GetFocus()) {
 		if (!widget.pyraminx.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			EnterPyraminx(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			EnterPuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	} else {
 		if (widget.pyraminx.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			LeavePyraminx(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			LeavePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	}
@@ -928,35 +936,39 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			Initialize(&widget, brush);
 			break;
 		case WM_DESTROY:
-			DestroyPyraminx(brush);
+			DestroyPuzzle(brush);
 			break;
 		case WM_SIZE:
-			ResizePyraminx(&widget);
+			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
 		case WM_PAINT:
 			widget.core.hDC = BeginPaint(hWnd, &paint);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ExposePyraminx(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ExposePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 			break;
 		case WM_RBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			RandomizePyraminx(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			SelectPyraminx(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			SelectPuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONUP:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ReleasePyraminx(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ReleasePuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
@@ -964,7 +976,8 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
 		case WM_MOUSEWHEEL:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+					GetStockObject(NULL_PEN));
 			{
 				int zDelta = ((short) HIWORD(wParam));
 				POINT cursor, origin;
@@ -973,13 +986,13 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				ClientToScreen(hWnd, &origin);
 				(void) GetCursorPos(&cursor);
 				if (zDelta > (WHEEL_DELTA >> 1)) {
-					MovePyraminxInput(&widget,
+					MovePuzzleInput(&widget,
 						cursor.x - origin.x, cursor.y - origin.y,
 						TOP,
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 				} else if (zDelta < -(WHEEL_DELTA >> 1)) {
-					MovePyraminxInput(&widget,
+					MovePuzzleInput(&widget,
 						cursor.x - origin.x, cursor.y - origin.y,
 						BOTTOM,
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
@@ -992,56 +1005,67 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		case WM_COMMAND:
 			switch (LOWORD(wParam)) {
 				case IDM_GET:
-					GetPyraminx(&widget);
-					ResizePyraminx(&widget);
+					GetPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_WRITE:
-					WritePyraminx(&widget);
+					WritePuzzle(&widget);
 					break;
 				case IDM_EXIT:
-					DestroyPyraminx(brush);
+					DestroyPuzzle(brush);
 					break;
 				case IDM_HIDE:
-					HidePyraminx(&widget);
-					break;
-				case IDM_CLEAR:
-					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					ClearPyraminx(&widget);
-					(void) ReleaseDC(hWnd, widget.core.hDC);
+					HidePuzzle(&widget);
 					break;
 				case IDM_UNDO:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					UndoPyraminx(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					UndoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_REDO:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RedoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_CLEAR:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					ClearPuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_RANDOMIZE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					RandomizePyraminx(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RandomizePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_PRACTICE:
-					PracticePyraminx(&widget);
-					SizePyraminx(&widget);
+					PracticePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SOLVE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					SolvePyraminx(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					SolvePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
-				case IDM_ORIENT:
-					OrientizePyraminx(&widget);
-					SizePyraminx(&widget);
+				case IDM_ORIENTIZE:
+					OrientizePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_STICKY:
-					(void) StickyModePyraminx(&widget);
-					SizePyraminx(&widget);
+					(void) StickyModePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SHIFT_TOP:
@@ -1065,12 +1089,13 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
 						shift = shift || (GetKeyState(VK_CAPITAL) & 1);
-						(void) MovePyraminxInput(&widget,
+						(void) MovePuzzleInput(&widget,
 							cursor.x - origin.x, cursor.y - origin.y,
 							(int) LOWORD(wParam) - ((shift) ? IDM_SHIFT_TOP : IDM_TOP),
 							shift, FALSE);
@@ -1099,11 +1124,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
-						(void) MovePyraminxInput(&widget,
+						(void) MovePuzzleInput(&widget,
 							cursor.x - origin.x, cursor.y - origin.y,
 							(int) LOWORD(wParam) - ((shift) ? IDM_SHIFTCONTROL_TOP : IDM_CONTROL_TOP),
 							shift, TRUE);
@@ -1112,21 +1138,21 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 
 					break;
 				case IDM_DEC:
-					if (DecrementPyraminx(&widget)) {
-						SizePyraminx(&widget);
+					if (DecrementPuzzle(&widget)) {
+						SizePuzzle(&widget);
 						(void) InvalidateRect(hWnd, NULL, TRUE);
 					}
 					break;
 				case IDM_INC:
-					IncrementPyraminx(&widget);
-					SizePyraminx(&widget);
+					IncrementPuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_PERIOD2:
 				case IDM_PERIOD3:
 				case IDM_BOTH:
-					PeriodModePyraminx(&widget, (int) LOWORD(wParam) - IDM_PERIOD2);
-					SizePyraminx(&widget);
+					PeriodModePuzzle(&widget, (int) LOWORD(wParam) - IDM_PERIOD2);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_ABOUT:
@@ -1204,7 +1230,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
 #else
 
 static void
-Usage(char * programName)
+Usage(char *programName)
 {
 	unsigned int i;
 
@@ -1235,6 +1261,9 @@ static XrmOptionDescRec options[] =
 	{(char *) "-face1", (char *) "*pyraminx.faceColor1", XrmoptionSepArg, NULL},
 	{(char *) "-face2", (char *) "*pyraminx.faceColor2", XrmoptionSepArg, NULL},
 	{(char *) "-face3", (char *) "*pyraminx.faceColor3", XrmoptionSepArg, NULL},
+	{(char *) "-delay", (char *) "*pyraminx.delay", XrmoptionSepArg, NULL},
+	{(char *) "-fn", (char *) "*pyraminx.font", XrmoptionSepArg, NULL},
+	{(char *) "-font", (char *) "*pyraminx.font", XrmoptionSepArg, NULL},
 	{(char *) "-size", (char *) "*pyraminx.size", XrmoptionSepArg, NULL},
 	{(char *) "-sticky", (char *) "*pyraminx.sticky", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-mode", (char *) "*pyraminx.mode", XrmoptionSepArg, NULL},
@@ -1243,9 +1272,6 @@ static XrmOptionDescRec options[] =
 	{(char *) "-noorient", (char *) "*pyraminx.orient", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-practice", (char *) "*pyraminx.practice", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-nopractice", (char *) "*pyraminx.practice", XrmoptionNoArg, (char *) "FALSE"},
-	{(char *) "-delay", (char *) "*pyraminx.delay", XrmoptionSepArg, NULL},
-	{(char *) "-fn", (char *) "*pyraminx.font", XrmoptionSepArg, NULL},
-	{(char *) "-font", (char *) "*pyraminx.font", XrmoptionSepArg, NULL},
 	{(char *) "-userName", (char *) "*pyraminx.userName", XrmoptionSepArg, NULL},
 	{(char *) "-scoreFile", (char *) "*pyraminx.scoreFile", XrmoptionSepArg, NULL},
 	{(char *) "-scores", (char *) "*pyraminx.scoreOnly", XrmoptionNoArg, (char *) "TRUE"},
@@ -1254,22 +1280,22 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackPyraminxPractice(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzlePractice(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(pyraminx, XtNmenu, 11, NULL); /* menu choice */
+		XtVaSetValues(pyraminx, XtNmenu, MENU_PRACTICE, NULL);
 	}
 }
 
 static void
-CallbackPyraminxRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzleRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(pyraminx, XtNmenu, 5, NULL); /* menu choice */
+		XtVaSetValues(pyraminx, XtNmenu, MENU_RANDOMIZE, NULL);
 	}
 }
 static void
-FacetSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+FacetSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int size = cbs->value, mode;
 	Boolean orient, sticky, practice;
@@ -1296,7 +1322,7 @@ FacetSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct * cbs)
+ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct *cbs)
 {
 	int size;
 	Boolean orient, sticky, practice;
@@ -1319,7 +1345,7 @@ ModeToggle(Widget w, int mode, XmToggleButtonCallbackStruct * cbs)
 }
 
 static void
-OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	int size, mode;
 	Boolean orient = cbs->set, sticky, practice;
@@ -1340,7 +1366,7 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
 }
 
 static void
-StickyToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+StickyToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	int size, mode;
 	Boolean orient, sticky = cbs->set, practice;
@@ -1372,7 +1398,7 @@ StickyToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
 
 static void
 PracticeToggle(Widget w, XtPointer clientData,
-		XmToggleButtonCallbackStruct * cbs)
+		XmToggleButtonCallbackStruct *cbs)
 {
 	int size, mode;
 	Boolean orient, sticky, practice = cbs->set;
@@ -1397,7 +1423,7 @@ PracticeToggle(Widget w, XtPointer clientData,
 }
 
 static void
-SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MULTSPEED * (MAXSPEED + MINSPEED - cbs->value - 1), oldDelay;
 
@@ -1413,9 +1439,9 @@ fileCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	if (val == 2)
+	if (val == MENU_EXIT)
 		exit(0);
-	XtVaSetValues(pyraminx, XtNmenu, val, NULL);
+	XtVaSetValues(pyraminx, XtNmenu, val, NULL); /* GWQ */
 }
 
 static void
@@ -1423,7 +1449,7 @@ playCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	XtVaSetValues(pyraminx, XtNmenu, val + 3, NULL); /* GWQ */
+	XtVaSetValues(pyraminx, XtNmenu, val + MENU_EXIT + 1, NULL);
 }
 
 static Widget
@@ -1522,9 +1548,10 @@ main(int argc, char **argv)
 	Widget switchRowCol, messageRowCol;
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
-	XmString clearString, undoString, randomizeString, solveString;
+	XmString undoString, redoString;
+	XmString clearString, randomizeString, practiceString, solveString;
+	XmString orientizeString, stickyString;
 	XmString incrementString, decrementString;
-	XmString orientizeString, stickyString, practiceString;
 	unsigned int i;
 #endif
 
@@ -1534,7 +1561,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#if HAVE_XPM
+#ifdef HAVE_XPM
 	{
 		XpmAttributes xpmAttributes;
 		XpmColorSymbol transparentColor[1] = {{NULL,
@@ -1549,18 +1576,18 @@ main(int argc, char **argv)
 		transparentColor[0].pixel = bg;
 		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) pyraminx_xpm, &pyraminxIcon, NULL,
+			(char **) pyraminx_xpm, &icon, NULL,
 			&xpmAttributes);
 	}
-	if (pyraminxIcon == (Pixmap) NULL)
+	if (icon == (Pixmap) NULL)
 #endif
-		pyraminxIcon = XCreateBitmapFromData(XtDisplay(topLevel),
+		icon = XCreateBitmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
 			(char *) pyraminx_bits,
 			pyraminx_width, pyraminx_height);
 #ifdef HAVE_MOTIF
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, pyraminxIcon,
+		XtNiconPixmap, icon,
 		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
@@ -1587,35 +1614,38 @@ main(int argc, char **argv)
 	XmStringFree(getString);
 	XmStringFree(writeString);
 	XmStringFree(quitString);
-	clearString = XmStringCreateSimple((char *) "Clear");
 	undoString = XmStringCreateSimple((char *) "Undo");
-	randomizeString = XmStringCreateSimple((char *) "Randomize");
+	redoString = XmStringCreateSimple((char *) "Redo");
+	clearString = XmStringCreateSimple((char *) "Clear");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
+	practiceString = XmStringCreateSimple((char *) "Practice");
 	solveString = XmStringCreateSimple((char *) "Solve");
-	incrementString = XmStringCreateSimple((char *) "Increment");
-	decrementString = XmStringCreateSimple((char *) "Decrement");
 	orientizeString = XmStringCreateSimple((char *) "Orientize");
 	stickyString = XmStringCreateSimple((char *) "StickY");
-	practiceString = XmStringCreateSimple((char *) "Practice");
+	incrementString = XmStringCreateSimple((char *) "Increment");
+	decrementString = XmStringCreateSimple((char *) "Decrement");
 	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
-		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
+		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
-		XmVaPUSHBUTTON, incrementString, 'I', NULL, NULL,
-		XmVaPUSHBUTTON, decrementString, 'D', NULL, NULL,
 		XmVaPUSHBUTTON, orientizeString, 'O', NULL, NULL,
 		XmVaPUSHBUTTON, stickyString, 'Y', NULL, NULL,
-		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
+		XmVaPUSHBUTTON, incrementString, 'I', NULL, NULL,
+		XmVaPUSHBUTTON, decrementString, 'D', NULL, NULL,
 		NULL);
-	XmStringFree(clearString);
 	XmStringFree(undoString);
+	XmStringFree(redoString);
+	XmStringFree(clearString);
 	XmStringFree(randomizeString);
+	XmStringFree(practiceString);
 	XmStringFree(solveString);
-	XmStringFree(incrementString);
-	XmStringFree(decrementString);
 	XmStringFree(orientizeString);
 	XmStringFree(stickyString);
-	XmStringFree(practiceString);
+	XmStringFree(incrementString);
+	XmStringFree(decrementString);
 	pullDownMenu = XmCreatePulldownMenu(menuBar,
 		(char *) "helpPullDown", NULL, 0);
 	widget = XtVaCreateManagedWidget("Help",
@@ -1659,11 +1689,11 @@ main(int argc, char **argv)
 	practiceDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to toggle the practice mode?",
 		(char *) "Practice Query",
-		(XtCallbackProc) CallbackPyraminxPractice);
+		(XtCallbackProc) CallbackPuzzlePractice);
 	randomizeDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to randomize?",
 		(char *) "Randomize Query",
-		(XtCallbackProc) CallbackPyraminxRandomize);
+		(XtCallbackProc) CallbackPuzzleRandomize);
 	mainPanel = XtCreateManagedWidget("mainPanel",
 		xmPanedWindowWidgetClass, menuBarPanel,
 		NULL, 0);
@@ -1797,13 +1827,13 @@ main(int argc, char **argv)
 		pyraminxWidgetClass, mainPanel, NULL, 0);
 #else
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, pyraminxIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 	pyraminx = XtCreateManagedWidget("pyraminx",
 		pyraminxWidgetClass, topLevel, NULL, 0);
 #endif
 	XtAddCallback(pyraminx, XtNselectCallback,
-		(XtCallbackProc) CallbackPyraminx, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	Initialize(pyraminx);
 	XtRealizeWidget(topLevel);
 	XGrabButton(XtDisplay(pyraminx), (unsigned int) AnyButton, AnyModifier,
