@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1994 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -66,9 +66,9 @@ static const char faceColorChar[MAXFACES] =
 #endif
 #endif
 
-static Boolean SetValuesRubik(Widget current, Widget request, Widget renew);
-static void DestroyRubik(Widget old);
-static void InitializeRubik(Widget request, Widget renew);
+static Boolean SetValuesPuzzle(Widget current, Widget request, Widget renew);
+static void DestroyPuzzle(Widget old);
+static void InitializePuzzle(Widget request, Widget renew);
 
 RubikClassRec rubikClassRec =
 {
@@ -79,7 +79,7 @@ RubikClassRec rubikClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeRubik,	/* initialize */
+		(XtInitProc) InitializePuzzle,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
 		NULL,		/* actions */
@@ -91,10 +91,10 @@ RubikClassRec rubikClassRec =
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
-		(XtWidgetProc) DestroyRubik,	/* destroy */
+		(XtWidgetProc) DestroyPuzzle,	/* destroy */
 		NULL,		/* resize */
 		NULL,		/* expose */
-		(XtSetValuesFunc) SetValuesRubik,	/* set values */
+		(XtSetValuesFunc) SetValuesPuzzle,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
@@ -114,7 +114,7 @@ RubikClassRec rubikClassRec =
 WidgetClass rubikWidgetClass = (WidgetClass) & rubikClassRec;
 
 void
-SetRubik(RubikWidget w, int reason)
+setPuzzle(RubikWidget w, int reason)
 {
 	rubikCallbackStruct cb;
 
@@ -123,7 +123,7 @@ SetRubik(RubikWidget w, int reason)
 }
 
 static void
-SetRubikMove(RubikWidget w, int reason, int face, int position, int direction,
+setPuzzleMove(RubikWidget w, int reason, int face, int position, int direction,
 		Boolean control, int fast)
 {
 	rubikCallbackStruct cb;
@@ -150,8 +150,8 @@ loadFont(RubikWidget w)
 		XUnloadFont(XtDisplay(w), w->rubik.fontInfo->fid);
 		XFreeFont(XtDisplay(w), w->rubik.fontInfo);
 	}
-	if ((w->rubik.fontInfo = XLoadQueryFont(display,
-			w->rubik.font)) == NULL) {
+	if (w->rubik.font && (w->rubik.fontInfo =
+			XLoadQueryFont(display, w->rubik.font)) == NULL) {
 		(void) sprintf(buf,
 			"Can not open %s font.\nAttempting %s font as alternate\n",
 			w->rubik.font, altfontname);
@@ -181,40 +181,55 @@ loadFont(RubikWidget w)
 #define LOGFILE "rubik.log"
 #endif
 
+static int facesToDirection[MAXFACES][MAXFACES] =
+{
+	{-1, 3, 2, 1, -3, 0},
+	{0, -1, 1, -3, 2, 3},
+	{0, 3, -1, 1, 2, -3},
+	{0, -3, 3, -1, 2, 1},
+	{-3, 3, 0, 1, -1, 2},
+	{2, 3, -3, 1, 0, -1}
+};
 static RubikLoc slideNextRow[MAXFACES][MAXORIENT] =
 {
 	{
 		{5, TOP},
 		{3, RIGHT},
 		{2, TOP},
-		{1, LEFT}},
+		{1, LEFT}
+	},
 	{
 		{0, RIGHT},
 		{2, TOP},
 		{4, LEFT},
-		{5, BOTTOM}},
+		{5, BOTTOM}
+	},
 	{
 		{0, TOP},
 		{3, TOP},
 		{4, TOP},
-		{1, TOP}},
+		{1, TOP}
+	},
 	{
 		{0, LEFT},
 		{5, BOTTOM},
 		{4, RIGHT},
-		{2, TOP}},
+		{2, TOP}
+	},
 	{
 		{2, TOP},
 		{3, LEFT},
 		{5, TOP},
-		{1, RIGHT}},
+		{1, RIGHT}
+	},
 	{
 		{4, TOP},
 		{3, BOTTOM},
 		{0, TOP},
-		{1, BOTTOM}}
+		{1, BOTTOM}
+	}
 };
-static int  rowToRotate[MAXFACES][MAXORIENT] =
+static int rowToRotate[MAXFACES][MAXORIENT] =
 {
 	{3, 2, 1, 5},
 	{2, 4, 5, 0},
@@ -224,6 +239,9 @@ static int  rowToRotate[MAXFACES][MAXORIENT] =
 	{3, 0, 1, 4}
 };
 
+static RubikStack undo = {NULL, NULL, NULL, 0};
+static RubikStack redo = {NULL, NULL, NULL, 0};
+
 static void
 CheckPieces(RubikWidget w)
 {
@@ -438,7 +456,7 @@ DrawFrame(const RubikWidget w, const Boolean focus)
 static void
 MoveNoPieces(RubikWidget w)
 {
-	SetRubik(w, RUBIK_ILLEGAL);
+	setPuzzle(w, PUZZLE_ILLEGAL);
 }
 
 static void
@@ -743,20 +761,21 @@ MoveAllPieces(RubikWidget w, const int face, const int position,
 }
 
 void
-MoveRubik(RubikWidget w, const int face, const int position,
+MovePuzzle(RubikWidget w, const int face, const int position,
 		const int direction, const Boolean control, int fast)
 {
-	SetRubikMove(w, RUBIK_MOVED, face, position, direction,
+	setPuzzleMove(w, PUZZLE_MOVED, face, position, direction,
 		MoveAllPieces(w, face, position, direction, control, fast),
 		fast);
-	PutMove(face, position, direction, control);
+	setMove(&undo, face, position, direction, control);
+	flushMoves(w, &redo, FALSE);
 }
 
 void
-MoveRubikDelay(RubikWidget w, const int face, const int position,
+MovePuzzleDelay(RubikWidget w, const int face, const int position,
 		const int direction, const Boolean control)
 {
-	MoveRubik(w, face, position, direction, control, NORMAL);
+	MovePuzzle(w, face, position, direction, control, NORMAL);
 	Sleep((unsigned int) w->rubik.delay);
 }
 
@@ -815,7 +834,7 @@ PositionPieces(RubikWidget w, int x, int y, int *face, int *position, int *direc
 }
 
 void
-MoveRubikInput(RubikWidget w, int x, int y, int direction, int control)
+MovePuzzleInput(RubikWidget w, int x, int y, int direction, int control)
 {
 	int         face, position;
 
@@ -826,9 +845,9 @@ MoveRubikInput(RubikWidget w, int x, int y, int direction, int control)
 	if (!PositionPieces(w, x, y, &face, &position, &direction))
 		return;
 	control = (control) ? 1 : 0;
-	MoveRubik(w, face, position, direction, control, NORMAL);
+	MovePuzzle(w, face, position, direction, control, NORMAL);
 	if (!control && CheckSolved(w, w->rubik.orient)) {
-		SetRubik(w, RUBIK_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
@@ -864,8 +883,11 @@ ResetPieces(RubikWidget w)
 			DISPLAY_ERROR("Not enough memory, exiting.");
 		}
 	}
-	FlushMoves(w);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	w->rubik.started = False;
+	w->rubik.currentFace = PUZZLE_IGNORE;
+	w->rubik.currentDirection = PUZZLE_IGNORE;
 }
 
 static void
@@ -911,15 +933,16 @@ GetPieces(RubikWidget w)
 		}
 #endif
 	}
-	FlushMoves(w);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &sizex);
 	if (sizex >= MINFACETS) {
 		for (i = w->rubik.sizex; i < sizex; i++) {
-			SetRubik(w, RUBIK_INCX);
+			setPuzzle(w, PUZZLE_INCX);
 		}
 		for (i = w->rubik.sizex; i > sizex; i--) {
-			SetRubik(w, RUBIK_DECX);
+			setPuzzle(w, PUZZLE_DECX);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizex ");
@@ -938,10 +961,10 @@ GetPieces(RubikWidget w)
 	(void) fscanf(fp, "%d", &sizey);
 	if (sizey >= MINFACETS) {
 		for (i = w->rubik.sizey; i < sizey; i++) {
-			SetRubik(w, RUBIK_INCY);
+			setPuzzle(w, PUZZLE_INCY);
 		}
 		for (i = w->rubik.sizey; i > sizey; i--) {
-			SetRubik(w, RUBIK_DECY);
+			setPuzzle(w, PUZZLE_DECY);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizey ");
@@ -960,10 +983,10 @@ GetPieces(RubikWidget w)
 	(void) fscanf(fp, "%d", &sizez);
 	if (sizez >= MINFACETS) {
 		for (i = w->rubik.sizez; i < sizez; i++) {
-			SetRubik(w, RUBIK_INCZ);
+			setPuzzle(w, PUZZLE_INCZ);
 		}
 		for (i = w->rubik.sizez; i > sizez; i--) {
-			SetRubik(w, RUBIK_DECZ);
+			setPuzzle(w, PUZZLE_DECZ);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: sizez ");
@@ -981,24 +1004,24 @@ GetPieces(RubikWidget w)
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &orient);
 	if (w->rubik.orient != (Boolean) orient) {
-		SetRubik(w, RUBIK_ORIENT);
+		setPuzzle(w, PUZZLE_ORIENTIZE);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &practice);
 	if (w->rubik.practice != (Boolean) practice) {
-		SetRubik(w, RUBIK_PRACTICE);
+		setPuzzle(w, PUZZLE_PRACTICE);
 	}
 #ifdef WINVER
 	ResetPieces(w);
 #endif
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
-	ScanStartPosition(fp, w);
-	SetRubik(w, RUBIK_RESTORE);
+	scanStartPosition(fp, w);
+	setPuzzle(w, PUZZLE_RESTORE);
 #ifdef WINVER
-	SetStartPosition(w);
+	setStartPosition(w);
 #endif
-	ScanMoves(fp, w, moves);
+	scanMoves(fp, w, moves);
 	(void) fclose(fp);
 	(void) printf("%s: sizex %d, sizey %d, sizez %d, orient %d, practice %d, moves %d.\n",
 		name, sizex, sizey, sizez, orient, practice, moves);
@@ -1056,44 +1079,67 @@ WritePieces(RubikWidget w)
 		(w->rubik.orient) ? 1 : 0);
 	(void) fprintf(fp, "practice%c %d\n", SYMBOL,
 		(w->rubik.practice) ? 1 : 0);
-	(void) fprintf(fp, "moves%c %d\n", SYMBOL, NumMoves());
-	PrintStartPosition(fp, w);
-	PrintMoves(fp);
+	(void) fprintf(fp, "moves%c %d\n", SYMBOL,
+		numMoves(&undo));
+	printStartPosition(fp, w);
+	printMoves(fp, &undo);
 	(void) fclose(fp);
 	(void) printf("Saved to %s.\n", name);
 	free(lname);
 	free(fname);
 }
 
-static void
-ClearPieces(RubikWidget w)
-{
-	SetRubik(w, RUBIK_CLEAR);
-}
-
 static void
 UndoPieces(RubikWidget w)
 {
-	if (MadeMoves()) {
+	if (madeMoves(&undo)) {
 		int         face, position, direction, control;
 
-		GetMove(&face, &position, &direction, &control);
+		getMove(&undo,
+			&face, &position, &direction, &control);
+                setMove(&redo,
+                        face, position, direction, control);
 		direction = (direction < MAXORIENT) ?
 			(direction + MAXORIENT / 2) % MAXORIENT :
 			3 * MAXORIENT - direction;
-		SetRubikMove(w, RUBIK_UNDO, face, position, direction,
+		setPuzzleMove(w, PUZZLE_UNDO, face, position, direction,
 			MoveAllPieces(w, face, position, direction, control,
 			DOUBLE), DOUBLE);
 		if (!control && CheckSolved(w, w->rubik.orient)) {
-			SetRubik(w, RUBIK_SOLVED);
+			setPuzzle(w, PUZZLE_SOLVED);
 		}
 	}
 }
 
+static void
+RedoPieces(RubikWidget w)
+{
+	if (madeMoves(&redo)) {
+		int         face, position, direction, control;
+
+		getMove(&redo,
+			&face, &position, &direction, &control);
+                setMove(&undo,
+                        face, position, direction, control);
+		setPuzzleMove(w, PUZZLE_REDO, face, position, direction,
+			MoveAllPieces(w, face, position, direction, control,
+			DOUBLE), DOUBLE);
+		if (!control && CheckSolved(w, w->rubik.orient)) {
+			setPuzzle(w, PUZZLE_SOLVED);
+		}
+	}
+}
+
+static void
+ClearPieces(RubikWidget w)
+{
+	setPuzzle(w, PUZZLE_CLEAR);
+}
+
 static void
 PracticePieces(RubikWidget w)
 {
-	SetRubik(w, RUBIK_PRACTICE);
+	setPuzzle(w, PUZZLE_PRACTICE);
 }
 
 static void
@@ -1108,7 +1154,7 @@ RandomizePieces(RubikWidget w)
 		big = 1000;
 	if (w->rubik.practice)
 		PracticePieces(w);
-	SetRubik(w, RUBIK_RESET);
+	setPuzzle(w, PUZZLE_RESET);
 #ifdef DEBUG
 	big = 3;
 #endif
@@ -1116,12 +1162,13 @@ RandomizePieces(RubikWidget w)
 		face = NRAND(MAXFACES);
 		position = NRAND(sizeFace(w, face));
 		direction = NRAND(MAXORIENT);
-		MoveRubik(w, face, position, direction, FALSE, INSTANT);
+		MovePuzzle(w, face, position, direction, FALSE, INSTANT);
 	}
-	FlushMoves(w);
-	SetRubik(w, RUBIK_RANDOMIZE);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
+	setPuzzle(w, PUZZLE_RANDOMIZE);
 	if (CheckSolved(w, w->rubik.orient)) {
-		SetRubik(w, RUBIK_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
@@ -1134,19 +1181,47 @@ SolvePieces(RubikWidget w)
 	    (w->rubik.sizex == 3 && w->rubik.sizey == 3 && w->rubik.sizez == 3))
 		SolveSomePieces(w);
 	else {
-		SetRubik(w, RUBIK_SOLVE_MESSAGE);
+		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
 	}
 }
 
 static void
 OrientizePieces(RubikWidget w)
 {
-	SetRubik(w, RUBIK_ORIENT);
+	setPuzzle(w, PUZZLE_ORIENTIZE);
+}
+
+static void
+IncrementPieces(RubikWidget w)
+{
+	setPuzzle(w, PUZZLE_INCX);
+	setPuzzle(w, PUZZLE_INCY);
+	setPuzzle(w, PUZZLE_INCZ);
+}
+
+static void
+DecrementPieces(RubikWidget w)
+{
+	if (w->rubik.sizex > MINFACETS) {
+		setPuzzle(w, PUZZLE_DECX);
+	}
+	if (w->rubik.sizey > MINFACETS) {
+		setPuzzle(w, PUZZLE_DECY);
+	}
+	if (w->rubik.sizez > MINFACETS) {
+		setPuzzle(w, PUZZLE_DECZ);
+	}
+}
+
+static void
+ViewPieces(RubikWidget w)
+{
+	setPuzzle(w, PUZZLE_VIEW);
 }
 
 #ifdef WINVER
 static void
-SetValuesRubik(RubikWidget w)
+SetValuesPuzzle(RubikWidget w)
 {
 	struct tagColor {
 		int         red, green, blue;
@@ -1214,35 +1289,37 @@ SetValuesRubik(RubikWidget w)
 }
 
 void
-DestroyRubik(HBRUSH brush)
+DestroyPuzzle(HBRUSH brush)
 {
+	deleteMoves(&undo);
+	deleteMoves(&redo);
 	(void) DeleteObject(brush);
 	PostQuitMessage(0);
 }
 
 void
-ResizeRubik(RubikWidget w)
+ResizePuzzle(RubikWidget w)
 {
 	if (w->rubik.dim == 2)
-		ResizeRubik2D((Rubik2DWidget) w);
+		ResizePuzzle2D((Rubik2DWidget) w);
 	else if (w->rubik.dim == 3)
-		ResizeRubik3D((Rubik3DWidget) w);
+		ResizePuzzle3D((Rubik3DWidget) w);
 }
 
 void
-SizeRubik(RubikWidget w)
+SizePuzzle(RubikWidget w)
 {
 	ResetPieces(w);
-	ResizeRubik(w);
+	ResizePuzzle(w);
 }
 
 void
-ExposeRubik(RubikWidget w)
+ExposePuzzle(RubikWidget w)
 {
 	if (w->rubik.dim == 2)
-		ExposeRubik2D((Rubik2DWidget) w);
+		ExposePuzzle2D((Rubik2DWidget) w);
 	else if (w->rubik.dim == 3)
-		ExposeRubik3D((Rubik3DWidget) w);
+		ExposePuzzle3D((Rubik3DWidget) w);
 }
 
 #else
@@ -1260,7 +1337,13 @@ GetColor(RubikWidget w, int face)
 		values.background = w->rubik.background;
 	}
 	if (!w->rubik.mono || w->rubik.dim == 4) {
-		if (XAllocNamedColor(XtDisplay(w),
+		if (!w->rubik.faceName[face]) {
+			char *buf1;
+
+			intCat(&buf1, "Color name null for face ", face);
+			DISPLAY_WARNING(buf1);
+			free(buf1);
+		} else if (XAllocNamedColor(XtDisplay(w),
 				DefaultColormapOfScreen(XtScreen(w)),
 				w->rubik.faceName[face], &colorCell, &rgb)) {
 			values.foreground = w->rubik.faceColor[face] =
@@ -1352,7 +1435,7 @@ SetAllColors(RubikWidget w)
 }
 
 static Boolean
-SetValuesRubik(Widget current, Widget request, Widget renew)
+SetValuesPuzzle(Widget current, Widget request, Widget renew)
 {
 	RubikWidget c = (RubikWidget) current, w = (RubikWidget) renew;
 	Boolean     redraw = False, setColors = False;
@@ -1372,8 +1455,7 @@ SetValuesRubik(Widget current, Widget request, Widget renew)
 		loadFont(w);
 		SetAllColors(w);
 		redraw = True;
-	}
-	if (w->rubik.background != c->rubik.background ||
+	} else if (w->rubik.background != c->rubik.background ||
 			w->rubik.foreground != c->rubik.foreground ||
 			setColors) {
 		SetAllColors(w);
@@ -1387,65 +1469,72 @@ SetValuesRubik(Widget current, Widget request, Widget renew)
 		redraw = True;
 	}
 	if (w->rubik.menu != -1) {
-		switch (w->rubik.menu) {
-		case 0:
-			w->rubik.menu = -1;
+		int menu = w->rubik.menu;
+
+		w->rubik.menu = -1;
+		switch (menu) {
+		case MENU_GET:
 			GetPieces(w);
 			break;
-		case 1:
-			w->rubik.menu = -1;
+		case MENU_WRITE:
 			WritePieces(w);
 			break;
-		case 3:
-			w->rubik.menu = -1;
-			ClearPieces(w);
-			break;
-		case 4:
-			w->rubik.menu = -1;
+		case MENU_UNDO:
 			UndoPieces(w);
 			break;
-		case 5:
-			w->rubik.menu = -1;
+		case MENU_REDO:
+			RedoPieces(w);
+			break;
+		case MENU_CLEAR:
+			ClearPieces(w);
+			break;
+		case MENU_RANDOMIZE:
 			RandomizePieces(w);
 			break;
-		case 6:
-			w->rubik.menu = -1;
-			SolvePieces(w);
-			break;
-		case 7:
-			w->rubik.menu = -1;
-			OrientizePieces(w);
-			break;
-		case 8:
-			w->rubik.menu = -1;
+		case MENU_PRACTICE:
 			PracticePieces(w);
 			break;
+		case MENU_SOLVE:
+			SolvePieces(w);
+			break;
+		case MENU_ORIENTIZE:
+			OrientizePieces(w);
+			break;
+		case MENU_INCREMENT:
+			IncrementPieces(w);
+			break;
+		case MENU_DECREMENT:
+			(void) DecrementPieces(w);
+			break;
+		case MENU_VIEW:
+			ViewPieces(w);
+			break;
 		default:
-			w->rubik.menu = -1;
 			break;
 		}
 	}
-	if (w->rubik.currentDirection == RUBIK_RESTORE) {
-		SetStartPosition(w);
-		w->rubik.currentDirection = RUBIK_IGNORE;
-	} else if (w->rubik.currentDirection == RUBIK_CLEAR) {
+	if (w->rubik.currentDirection == PUZZLE_RESTORE) {
+		setStartPosition(w);
+		w->rubik.currentDirection = PUZZLE_IGNORE;
+	} else if (w->rubik.currentDirection == PUZZLE_CLEAR) {
 		ResetPieces(w);
 		redraw = True;
-		w->rubik.currentDirection = RUBIK_IGNORE;
-	} else if (w->rubik.currentDirection != RUBIK_IGNORE) {
-		(void) MoveAllPieces(w, w->rubik.currentFace, w->rubik.currentPosition,
-			w->rubik.currentDirection, w->rubik.currentControl,
-			w->rubik.currentFast);
-		w->rubik.currentDirection = RUBIK_IGNORE;
+		w->rubik.currentDirection = PUZZLE_IGNORE;
+	} else if (w->rubik.currentDirection > PUZZLE_IGNORE) {
+		(void) MoveAllPieces(w, w->rubik.currentFace,
+			w->rubik.currentPosition, w->rubik.currentDirection,
+			w->rubik.currentControl, w->rubik.currentFast);
+		w->rubik.currentDirection = PUZZLE_IGNORE;
 	}
 	return redraw;
 }
 
 static void
-DestroyRubik(Widget old)
+DestroyPuzzle(Widget old)
 {
 	RubikWidget w = (RubikWidget) old;
-	int         face;
+	Display *display = XtDisplay(w);
+	int face;
 
 	for (face = 0; face < MAXFACES; face++)
 		XtReleaseGC(old, w->rubik.faceGC[face]);
@@ -1453,18 +1542,18 @@ DestroyRubik(Widget old)
 	XtReleaseGC(old, w->rubik.frameGC);
 	XtReleaseGC(old, w->rubik.inverseGC);
 	XtRemoveCallbacks(old, XtNselectCallback, w->rubik.select);
-}
-
-void
-QuitRubik(RubikWidget w, XEvent * event, char **args, int nArgs)
-{
-	Display *display = XtDisplay(w);
-
 	if (w->rubik.fontInfo) {
 		XUnloadFont(display, w->rubik.fontInfo->fid);
 		XFreeFont(display, w->rubik.fontInfo);
 	}
-	XtCloseDisplay(display);
+	deleteMoves(&undo);
+	deleteMoves(&redo);
+}
+
+void
+QuitPuzzle(RubikWidget w, XEvent *event, char **args, int nArgs)
+{
+	XtCloseDisplay(XtDisplay(w));
 	exit(0);
 }
 #endif
@@ -1473,7 +1562,7 @@ QuitRubik(RubikWidget w, XEvent * event, char **args, int nArgs)
 static
 #endif
 void
-InitializeRubik(
+InitializePuzzle(
 #ifdef WINVER
 RubikWidget w, HBRUSH brush
 #else
@@ -1483,7 +1572,7 @@ Widget request, Widget renew
 {
 	int         face, orient;
 #ifdef WINVER
-	SetValuesRubik(w);
+	SetValuesPuzzle(w);
 #else
 	RubikWidget w = (RubikWidget) renew;
 
@@ -1503,7 +1592,8 @@ Widget request, Widget renew
 	for (orient = 0; orient < MAXORIENT; orient++)
 		w->rubik.rowLoc[orient] = NULL;
 	CheckPieces(w);
-	InitMoves();
+	newMoves(&undo);
+	newMoves(&redo);
 	w->rubik.cheat = False;
 	ResetPieces(w);
 #ifdef WINVER
@@ -1516,21 +1606,21 @@ Widget request, Widget renew
 }
 
 void
-HideRubik(RubikWidget w
+HidePuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
-	SetRubik(w, RUBIK_HIDE);
+	setPuzzle(w, PUZZLE_HIDE);
 }
 
 void
-SelectRubik(RubikWidget w
+SelectPuzzle(RubikWidget w
 #ifdef WINVER
 , const int x, const int y, const int control
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1545,72 +1635,76 @@ SelectRubik(RubikWidget w
 			DrawSquare(w, w->rubik.currentFace, w->rubik.currentPosition,
 				   TRUE);
 	} else {
-		w->rubik.currentFace = RUBIK_IGNORE;
-		w->rubik.currentDirection = RUBIK_IGNORE;
+		w->rubik.currentFace = PUZZLE_IGNORE;
+		w->rubik.currentDirection = PUZZLE_IGNORE;
 	}
 }
 
 void
-ReleaseRubik(RubikWidget w
+ReleasePuzzle(RubikWidget w
 #ifdef WINVER
 , const int x, const int y, const int control
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
-	int         face, position, count = -1, direction = 0;
+	int         face, position, count = -1, direction = -1;
 #ifndef WINVER
 	int  x = event->xbutton.x, y = event->xbutton.y;
 	int control = (int) (event->xkey.state & ControlMask);
 #endif
 
-	if (w->rubik.currentFace == RUBIK_IGNORE)
+	if (w->rubik.currentFace <= PUZZLE_IGNORE)
 		return;
 	DrawSquare(w, w->rubik.currentFace, w->rubik.currentPosition, FALSE);
 	if (!control && !w->rubik.practice && CheckSolved(w, w->rubik.orient))
 		MoveNoPieces(w);
 	else if (SelectPieces(w, x, y, &face, &position)) {
-		if (face == w->rubik.currentFace)
-			count = CheckMoveDir(w, face, w->rubik.currentPosition, position, &direction);
+		if (face == w->rubik.currentFace) {
+			if (position != w->rubik.currentPosition)
+				count = CheckMoveDir(w, face,
+					w->rubik.currentPosition, position,
+					&direction);
+		} else {
+			direction = facesToDirection[w->rubik.currentFace][face];
+			count = (direction == PUZZLE_AMBIGUOUS) ? 2 : 1;
+			/* can look closer to minimize this, but then
+			must also be able to handle double moves */
+		}
 		if (count == 1) {
-			MoveRubik(w, face, w->rubik.currentPosition, direction, (control) ? 1 : 0, NORMAL);
+			MovePuzzle(w, face, w->rubik.currentPosition,
+				direction, (control) ? 1 : 0, NORMAL);
 			if (!control && CheckSolved(w, w->rubik.orient)) {
-				SetRubik(w, RUBIK_SOLVED);
+				setPuzzle(w, PUZZLE_SOLVED);
 			}
+		} else if (count == 2) {
+			setPuzzle(w, PUZZLE_AMBIGUOUS);
 		} else if (count == 0)
 			MoveNoPieces(w);
 	}
-}
-
-void
-PracticeRubik(RubikWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	PracticePieces(w);
+	w->rubik.currentFace = PUZZLE_IGNORE;
+	w->rubik.currentDirection = PUZZLE_IGNORE;
 }
 
 #ifndef WINVER
 void
-PracticeRubikMaybe(RubikWidget w
-, XEvent * event, char **args, int nArgs
+PracticePuzzleMaybe(RubikWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 	if (!w->rubik.started)
 		PracticePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		SetRubik(w, RUBIK_PRACTICE_QUERY);
+		setPuzzle(w, PUZZLE_PRACTICE_QUERY);
 	}
 #endif
 }
 
 void
-PracticeRubik2(RubikWidget w
-, XEvent * event, char **args, int nArgs
+PracticePuzzle2(RubikWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 #ifdef HAVE_MOTIF
@@ -1618,36 +1712,24 @@ PracticeRubik2(RubikWidget w
 #endif
 		PracticePieces(w);
 }
-#endif
 
 void
-RandomizeRubik(RubikWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	RandomizePieces(w);
-}
-
-#ifndef WINVER
-void
-RandomizeRubikMaybe(RubikWidget w
-, XEvent * event, char **args, int nArgs
+RandomizePuzzleMaybe(RubikWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 	if (!w->rubik.started)
 		RandomizePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		SetRubik(w, RUBIK_RANDOMIZE_QUERY);
+		setPuzzle(w, PUZZLE_RANDOMIZE_QUERY);
 	}
 #endif
 }
 
 void
-RandomizeRubik2(RubikWidget w
-, XEvent * event, char **args, int nArgs
+RandomizePuzzle2(RubikWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 #ifdef HAVE_MOTIF
@@ -1658,9 +1740,9 @@ RandomizeRubik2(RubikWidget w
 #endif
 
 void
-GetRubik(RubikWidget w
+GetPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1668,9 +1750,9 @@ GetRubik(RubikWidget w
 }
 
 void
-WriteRubik(RubikWidget w
+WritePuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1678,19 +1760,9 @@ WriteRubik(RubikWidget w
 }
 
 void
-ClearRubik(RubikWidget w
+UndoPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	ClearPieces(w);
-}
-
-void
-UndoRubik(RubikWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1698,9 +1770,39 @@ UndoRubik(RubikWidget w
 }
 
 void
-SolveRubik(RubikWidget w
+RedoPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RedoPieces(w);
+}
+
+void
+ClearPuzzle(RubikWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	ClearPieces(w);
+}
+
+void
+RandomizePuzzle(RubikWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RandomizePieces(w);
+}
+
+void
+SolvePuzzle(RubikWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1708,9 +1810,19 @@ SolveRubik(RubikWidget w
 }
 
 void
-OrientizeRubik(RubikWidget w
+PracticePuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	PracticePieces(w);
+}
+
+void
+OrientizePuzzle(RubikWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1718,39 +1830,39 @@ OrientizeRubik(RubikWidget w
 }
 
 void
-DecrementRubik(RubikWidget w
+DecrementPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
-	if (w->rubik.sizex > MINFACETS) {
-		SetRubik(w, RUBIK_DECX);
-	}
-	if (w->rubik.sizey > MINFACETS) {
-		SetRubik(w, RUBIK_DECY);
-	}
-	if (w->rubik.sizez > MINFACETS) {
-		SetRubik(w, RUBIK_DECZ);
-	}
+	DecrementPieces(w);
 }
 
 void
-IncrementRubik(RubikWidget w
+IncrementPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
-	SetRubik(w, RUBIK_INCX);
-	SetRubik(w, RUBIK_INCY);
-	SetRubik(w, RUBIK_INCZ);
+	IncrementPieces(w);
 }
 
 void
-EnterRubik(RubikWidget w
+ViewPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	ViewPieces(w);
+}
+
+void
+EnterPuzzle(RubikWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1759,9 +1871,9 @@ EnterRubik(RubikWidget w
 }
 
 void
-LeaveRubik(RubikWidget w
+LeavePuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1771,85 +1883,85 @@ LeaveRubik(RubikWidget w
 
 #ifdef WINVER
 void
-IncrementAxisRubik(RubikWidget w, char axis)
+IncrementAxisPuzzle(RubikWidget w, char axis)
 {
 	switch (axis) {
 		case 'X':
 			if (w->rubik.sizex > MINFACETS)
-				SetRubik(w, RUBIK_DECX);
+				setPuzzle(w, PUZZLE_DECX);
 			return;
 		case 'x':
-			SetRubik(w, RUBIK_INCX);
+			setPuzzle(w, PUZZLE_INCX);
 			return;
 		case 'Y':
 			if (w->rubik.sizey > MINFACETS)
-				SetRubik(w, RUBIK_DECY);
+				setPuzzle(w, PUZZLE_DECY);
 			return;
 		case 'y':
-			SetRubik(w, RUBIK_INCY);
+			setPuzzle(w, PUZZLE_INCY);
 			return;
 		case 'Z':
 			if (w->rubik.sizez > MINFACETS)
-				SetRubik(w, RUBIK_DECZ);
+				setPuzzle(w, PUZZLE_DECZ);
 			return;
 		case 'z':
-			SetRubik(w, RUBIK_INCZ);
+			setPuzzle(w, PUZZLE_INCZ);
 			return;
 	}
 	return;
 }
 
 void
-DimRubik(RubikWidget w)
+DimPuzzle(RubikWidget w)
 {
-	SetRubik(w, RUBIK_DIM);
+	setPuzzle(w, PUZZLE_DIM);
 }
 
 #else
 void
-IncrementXRubik(RubikWidget w, XEvent * event, char **args, int nArgs)
+IncrementXPuzzle(RubikWidget w, XEvent *event, char **args, int nArgs)
 {
 	if (event->xbutton.state & (ShiftMask | LockMask)) {
 		if (w->rubik.sizex <= MINFACETS)
 			return;
-		SetRubik(w, RUBIK_DECX);
+		setPuzzle(w, PUZZLE_DECX);
 	} else
-		SetRubik(w, RUBIK_INCX);
+		setPuzzle(w, PUZZLE_INCX);
 }
 
 void
-IncrementYRubik(RubikWidget w , XEvent * event, char **args, int nArgs)
+IncrementYPuzzle(RubikWidget w , XEvent *event, char **args, int nArgs)
 {
 	if (event->xbutton.state & (ShiftMask | LockMask)) {
 		if (w->rubik.sizey <= MINFACETS)
 			return;
-		SetRubik(w, RUBIK_DECY);
+		setPuzzle(w, PUZZLE_DECY);
 	} else
-		SetRubik(w, RUBIK_INCY);
+		setPuzzle(w, PUZZLE_INCY);
 }
 
 void
-IncrementZRubik(RubikWidget w , XEvent * event, char **args, int nArgs)
+IncrementZPuzzle(RubikWidget w , XEvent *event, char **args, int nArgs)
 {
 	if (event->xbutton.state & (ShiftMask | LockMask)) {
 		if (w->rubik.sizez <= MINFACETS)
 			return;
-		SetRubik(w, RUBIK_DECZ);
+		setPuzzle(w, PUZZLE_DECZ);
 	} else
-		SetRubik(w, RUBIK_INCZ);
+		setPuzzle(w, PUZZLE_INCZ);
 }
 
 void
-MoveRubikCcw(RubikWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleCcw(RubikWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput(w, event->xbutton.x, event->xbutton.y, CCW,
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, CCW,
 		       (int) (event->xbutton.state & ControlMask));
 }
 
 void
-MoveRubikCw(RubikWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleCw(RubikWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput(w, event->xbutton.x, event->xbutton.y, CW,
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, CW,
 		       (int) (event->xkey.state & ControlMask));
 }
 #endif
