@@ -46,6 +46,7 @@
 #define XtNfaceColor5 ((char *) "faceColor5")
 #define XtNpieceBorder ((char *) "pieceBorder")
 #define XtNdelay ((char *) "delay")
+#define XtNview ((char *) "view")
 #define XtNsizex ((char *) "sizex")
 #define XtNsizey ((char *) "sizey")
 #define XtNsizez ((char *) "sizez")
@@ -67,6 +68,7 @@
 #define XtCMono ((char *) "Mono")
 #define XtCReverse ((char *) "Reverse")
 #define XtCDelay ((char *) "Delay")
+#define XtCView ((char *) "View")
 #define XtCSizeX ((char *) "SizeX")
 #define XtCSizeY ((char *) "SizeY")
 #define XtCSizeZ ((char *) "SizeZ")
@@ -94,32 +96,50 @@ typedef struct {
 } rubikCallbackStruct;
 #endif
 
-#define RUBIK_HIDE (-9)
-#define RUBIK_PRACTICE_QUERY (-8)
-#define RUBIK_RANDOMIZE_QUERY (-7)
-#define RUBIK_SOLVE_MESSAGE (-6)
-#define RUBIK_RESTORE (-5)
-#define RUBIK_CLEAR (-4)
-#define RUBIK_RESET (-3)
-#define RUBIK_ILLEGAL (-2)
-#define RUBIK_IGNORE (-1)
-#define RUBIK_MOVED 0
-#define RUBIK_SOLVED 2
-#define RUBIK_PRACTICE 3
-#define RUBIK_RANDOMIZE 4
-#define RUBIK_ORIENT 5
-#define RUBIK_COMPUTED 6
-#define RUBIK_UNDO 7
-#define RUBIK_DECX 8
-#define RUBIK_DECY 9
-#define RUBIK_DECZ 10
-#define RUBIK_INCX 11
-#define RUBIK_INCY 12
-#define RUBIK_INCZ 13
-#define RUBIK_DIM 14
+#define PUZZLE_HIDE (-10)
+#define PUZZLE_PRACTICE_QUERY (-9)
+#define PUZZLE_RANDOMIZE_QUERY (-8)
+#define PUZZLE_SOLVE_MESSAGE (-7)
+#define PUZZLE_RESTORE (-6)
+#define PUZZLE_CLEAR (-5)
+#define PUZZLE_RESET (-4)
+#define PUZZLE_AMBIGUOUS (-3)
+#define PUZZLE_ILLEGAL (-2)
+#define PUZZLE_IGNORE (-1)
+#define PUZZLE_MOVED 0
+#define PUZZLE_SOLVED 2
+#define PUZZLE_PRACTICE 3
+#define PUZZLE_RANDOMIZE 4
+#define PUZZLE_DECX 5
+#define PUZZLE_DECY 6
+#define PUZZLE_DECZ 7
+#define PUZZLE_INCX 8
+#define PUZZLE_INCY 9
+#define PUZZLE_INCZ 10
+#define PUZZLE_ORIENTIZE 11
+#define PUZZLE_COMPUTED 12
+#define PUZZLE_UNDO 13
+#define PUZZLE_REDO 14
+#define PUZZLE_DIM 15
+#define PUZZLE_VIEW 16
+
+#define MENU_GET 0
+#define MENU_WRITE 1
+#define MENU_EXIT 2
+#define MENU_UNDO 3
+#define MENU_REDO 4
+#define MENU_CLEAR 5
+#define MENU_RANDOMIZE 6
+#define MENU_PRACTICE 7
+#define MENU_SOLVE 8
+#define MENU_ORIENTIZE 9
+#define MENU_INCREMENT 10
+#define MENU_DECREMENT 11
+#define MENU_VIEW 12
 
 #define MAXORIENT 4
 #define MAXFACES 6
+#define MAXVIEWS 2
 #define MINFACETS 1
 #define DEFAULTFACETS 3
 #define DEFAULTORIENT FALSE
