@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1994 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -29,20 +29,20 @@
 #include "Rubik2dP.h"
 
 #ifndef WINVER
-static Boolean SetValuesRubik2D(Widget current, Widget request, Widget renew);
-static void ResizeRubik2D(Rubik2DWidget w);
-static void InitializeRubik2D(Widget request, Widget renew);
-static void ExposeRubik2D(Widget renew, XEvent * event, Region region);
-static void MoveRubik2DTop(Rubik2DWidget w,
-			   XEvent * event, char **args, int nArgs);
-static void MoveRubik2DLeft(Rubik2DWidget w,
-			    XEvent * event, char **args, int nArgs);
-static void MoveRubik2DRight(Rubik2DWidget w,
-			     XEvent * event, char **args, int nArgs);
-static void MoveRubik2DBottom(Rubik2DWidget w,
-			      XEvent * event, char **args, int nArgs);
+static Boolean SetValuesPuzzle2D(Widget current, Widget request, Widget renew);
+static void ResizePuzzle2D(Rubik2DWidget w);
+static void InitializePuzzle2D(Widget request, Widget renew);
+static void ExposePuzzle2D(Widget renew, XEvent *event, Region region);
+static void MovePuzzle2DTop(Rubik2DWidget w,
+			   XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DLeft(Rubik2DWidget w,
+			    XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DRight(Rubik2DWidget w,
+			     XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DBottom(Rubik2DWidget w,
+			      XEvent *event, char **args, int nArgs);
 
-static char defaultTranslationsRubik2D[] =
+static char defaultTranslationsPuzzle2D[] =
 "<KeyPress>q: Quit()\n\
  Ctrl<KeyPress>C: Quit()\n\
  <KeyPress>osfCancel: Hide()\n\
@@ -77,62 +77,66 @@ static char defaultTranslationsRubik2D[] =
  <KeyPress>R14: MoveBottom()\n\
  <Btn1Down>: Select()\n\
  <Btn1Up>: Release()\n\
- <KeyPress>p: Practice()\n\
  <Btn2Down>: PracticeMaybe()\n\
  <Btn2Down>(2+): Practice2()\n\
- <KeyPress>r: Randomize()\n\
  <Btn3Down>: RandomizeMaybe()\n\
  <Btn3Down>(2+): Randomize2()\n\
  <Btn4Down>: MoveTop()\n\
  <Btn5Down>: MoveBottom()\n\
  <KeyPress>g: Get()\n\
  <KeyPress>w: Write()\n\
- <KeyPress>c: Clear()\n\
  <KeyPress>u: Undo()\n\
+ <KeyPress>r: Redo()\n\
+ <KeyPress>c: Clear()\n\
+ <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
+ <KeyPress>p: Practice()\n\
+ <KeyPress>o: Orientize()\n\
  <KeyPress>d: Decrement()\n\
  <KeyPress>i: Increment()\n\
  <KeyPress>x: IncrementX()\n\
  <KeyPress>y: IncrementY()\n\
- <KeyPress>z: IncrementZ()\n\
- <KeyPress>o: Orientize()\n\
+ <KeyPress>f: IncrementZ()\n\
+ <KeyPress>v: View()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
-static XtActionsRec actionsListRubik2D[] =
+static XtActionsRec actionsListPuzzle2D[] =
 {
-	{(char *) "Quit", (XtActionProc) QuitRubik},
-	{(char *) "Hide", (XtActionProc) HideRubik},
-	{(char *) "MoveCcw", (XtActionProc) MoveRubikCcw},
-	{(char *) "MoveTop", (XtActionProc) MoveRubik2DTop},
-	{(char *) "MoveLeft", (XtActionProc) MoveRubik2DLeft},
-	{(char *) "MoveCw", (XtActionProc) MoveRubikCw},
-	{(char *) "MoveRight", (XtActionProc) MoveRubik2DRight},
-	{(char *) "MoveBottom", (XtActionProc) MoveRubik2DBottom},
-	{(char *) "Select", (XtActionProc) SelectRubik},
-	{(char *) "Release", (XtActionProc) ReleaseRubik},
-	{(char *) "Practice", (XtActionProc) PracticeRubik},
-	{(char *) "PracticeMaybe", (XtActionProc) PracticeRubikMaybe},
-	{(char *) "Practice2", (XtActionProc) PracticeRubik2},
-	{(char *) "Randomize", (XtActionProc) RandomizeRubik},
-	{(char *) "RandomizeMaybe", (XtActionProc) RandomizeRubikMaybe},
-	{(char *) "Randomize2", (XtActionProc) RandomizeRubik2},
-	{(char *) "Get", (XtActionProc) GetRubik},
-	{(char *) "Write", (XtActionProc) WriteRubik},
-	{(char *) "Clear", (XtActionProc) ClearRubik},
-	{(char *) "Undo", (XtActionProc) UndoRubik},
-	{(char *) "Solve", (XtActionProc) SolveRubik},
-	{(char *) "Decrement", (XtActionProc) DecrementRubik},
-	{(char *) "Increment", (XtActionProc) IncrementRubik},
-	{(char *) "IncrementX", (XtActionProc) IncrementXRubik},
-	{(char *) "IncrementY", (XtActionProc) IncrementYRubik},
-	{(char *) "IncrementZ", (XtActionProc) IncrementZRubik},
-	{(char *) "Orientize", (XtActionProc) OrientizeRubik},
-	{(char *) "Enter", (XtActionProc) EnterRubik},
-        {(char *) "Leave", (XtActionProc) LeaveRubik}
+	{(char *) "Quit", (XtActionProc) QuitPuzzle},
+	{(char *) "Hide", (XtActionProc) HidePuzzle},
+	{(char *) "MoveCcw", (XtActionProc) MovePuzzleCcw},
+	{(char *) "MoveTop", (XtActionProc) MovePuzzle2DTop},
+	{(char *) "MoveLeft", (XtActionProc) MovePuzzle2DLeft},
+	{(char *) "MoveCw", (XtActionProc) MovePuzzleCw},
+	{(char *) "MoveRight", (XtActionProc) MovePuzzle2DRight},
+	{(char *) "MoveBottom", (XtActionProc) MovePuzzle2DBottom},
+	{(char *) "Select", (XtActionProc) SelectPuzzle},
+	{(char *) "Release", (XtActionProc) ReleasePuzzle},
+	{(char *) "PracticeMaybe", (XtActionProc) PracticePuzzleMaybe},
+	{(char *) "Practice2", (XtActionProc) PracticePuzzle2},
+	{(char *) "RandomizeMaybe", (XtActionProc) RandomizePuzzleMaybe},
+	{(char *) "Randomize2", (XtActionProc) RandomizePuzzle2},
+	{(char *) "Get", (XtActionProc) GetPuzzle},
+	{(char *) "Write", (XtActionProc) WritePuzzle},
+	{(char *) "Undo", (XtActionProc) UndoPuzzle},
+	{(char *) "Redo", (XtActionProc) RedoPuzzle},
+	{(char *) "Clear", (XtActionProc) ClearPuzzle},
+	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
+	{(char *) "Solve", (XtActionProc) SolvePuzzle},
+	{(char *) "Practice", (XtActionProc) PracticePuzzle},
+	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
+	{(char *) "Decrement", (XtActionProc) DecrementPuzzle},
+	{(char *) "Increment", (XtActionProc) IncrementPuzzle},
+	{(char *) "IncrementX", (XtActionProc) IncrementXPuzzle},
+	{(char *) "IncrementY", (XtActionProc) IncrementYPuzzle},
+	{(char *) "IncrementZ", (XtActionProc) IncrementZPuzzle},
+	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Enter", (XtActionProc) EnterPuzzle},
+        {(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
 
-static XtResource resourcesRubik2D[] =
+static XtResource resourcesPuzzle2D[] =
 {
 	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
 	 XtOffset(RubikWidget, core.width),
@@ -247,29 +251,29 @@ Rubik2DClassRec rubik2dClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeRubik2D,		/* initialize */
+		(XtInitProc) InitializePuzzle2D,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
-		actionsListRubik2D,	/* actions */
-		XtNumber(actionsListRubik2D),	/* num actions */
-		resourcesRubik2D,	/* resources */
-		XtNumber(resourcesRubik2D),	/* num resources */
+		actionsListPuzzle2D,	/* actions */
+		XtNumber(actionsListPuzzle2D),	/* num actions */
+		resourcesPuzzle2D,	/* resources */
+		XtNumber(resourcesPuzzle2D),	/* num resources */
 		NULLQUARK,	/* xrm class */
 		TRUE,		/* compress motion */
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
 		NULL,		/* destroy */
-		(XtWidgetProc) ResizeRubik2D,	/* resize */
-		(XtExposeProc) ExposeRubik2D,	/* expose */
-		(XtSetValuesFunc) SetValuesRubik2D,	/* set values */
+		(XtWidgetProc) ResizePuzzle2D,	/* resize */
+		(XtExposeProc) ExposePuzzle2D,	/* expose */
+		(XtSetValuesFunc) SetValuesPuzzle2D,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
 		XtInheritAcceptFocus,	/* accept focus */
 		XtVersion,	/* version */
 		NULL,		/* callback private */
-		defaultTranslationsRubik2D,	/* tm table */
+		defaultTranslationsPuzzle2D,	/* tm table */
 		NULL,		/* query geometry */
 		NULL,		/* display accelerator */
 		NULL		/* extension */
@@ -609,7 +613,7 @@ NarrowSelection2D(Rubik2DWidget w, int *face, int *position, int *direction)
 
 #ifndef WINVER
 static      Boolean
-SetValuesRubik2D(Widget current, Widget request, Widget renew)
+SetValuesPuzzle2D(Widget current, Widget request, Widget renew)
 {
 	Rubik2DWidget c = (Rubik2DWidget) current, w = (Rubik2DWidget) renew;
 	Boolean     redraw = False;
@@ -618,11 +622,11 @@ SetValuesRubik2D(Widget current, Widget request, Widget renew)
 	    w->rubik.sizey != c->rubik.sizey ||
 	    w->rubik.sizez != c->rubik.sizez) {
 		ResetPieces((RubikWidget) w);
-		ResizeRubik2D(w);
+		ResizePuzzle2D(w);
 		redraw = True;
 	}
 	if (w->rubik2d.cubeLength != c->rubik2d.cubeLength) {
-		ResizeRubik2D(w);
+		ResizePuzzle2D(w);
 		redraw = True;
 	}
 	return (redraw);
@@ -633,7 +637,7 @@ SetValuesRubik2D(Widget current, Widget request, Widget renew)
 static
 #endif
 void
-ResizeRubik2D(Rubik2DWidget w)
+ResizePuzzle2D(Rubik2DWidget w)
 {
 #ifdef WINVER
 	RECT        rect;
@@ -667,7 +671,7 @@ ResizeRubik2D(Rubik2DWidget w)
 static
 #endif
 void
-InitializeRubik2D(
+InitializePuzzle2D(
 #ifdef WINVER
 Rubik2DWidget w
 #else
@@ -681,18 +685,18 @@ Widget request, Widget renew
 	SetAllColors((RubikWidget) w);
 #endif
 	w->rubik.dim = 2;
-	ResizeRubik2D(w);
+	ResizePuzzle2D(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-ExposeRubik2D(
+ExposePuzzle2D(
 #ifdef WINVER
 Rubik2DWidget w
 #else
-Widget renew, XEvent * event, Region region
+Widget renew, XEvent *event, Region region
 #endif
 )
 {
@@ -710,30 +714,30 @@ Widget renew, XEvent * event, Region region
 
 #ifndef WINVER
 static void
-MoveRubik2DTop(Rubik2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DTop(Rubik2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, TOP,
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, TOP,
 		       (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubik2DLeft(Rubik2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DLeft(Rubik2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
 		       (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubik2DRight(Rubik2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DRight(Rubik2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
 		       (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubik2DBottom(Rubik2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DBottom(Rubik2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
 		       (int) (event->xkey.state & ControlMask));
 }
 #endif
