@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1994 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -72,9 +72,9 @@ extern Boolean NarrowSelection2D(Rubik2DWidget w,
 extern void DrawFrame2D(Rubik2DWidget w, Boolean focus);
 extern void DrawSquare2D(Rubik2DWidget w, int face, int position, int offset);
 #ifdef WINVER
-extern void ResizeRubik2D(Rubik2DWidget w);
-extern void InitializeRubik2D(Rubik2DWidget w);
-extern void ExposeRubik2D(Rubik2DWidget w);
+extern void ResizePuzzle2D(Rubik2DWidget w);
+extern void InitializePuzzle2D(Rubik2DWidget w);
+extern void ExposePuzzle2D(Rubik2DWidget w);
 #endif
 
 #endif /* _Rubik2dP_h */
