@@ -29,20 +29,28 @@
 #include "Rubik3dP.h"
 
 #ifndef WINVER
-static Boolean SetValuesRubik3D(Widget current, Widget request, Widget renew);
-static void ResizeRubik3D(Rubik3DWidget w);
-static void InitializeRubik3D(Widget request, Widget renew);
-static void ExposeRubik3D(Widget renew, XEvent * event, Region region);
-static void MoveRubik3DTl(Rubik3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubik3DTop(Rubik3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubik3DTr(Rubik3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubik3DLeft(Rubik3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubik3DRight(Rubik3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubik3DBl(Rubik3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubik3DBottom(Rubik3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubik3DBr(Rubik3DWidget w, XEvent * event, char **args, int nArgs);
+static Boolean SetValuesPuzzle3D(Widget current, Widget request, Widget renew);
+static void ResizePuzzle3D(Rubik3DWidget w);
+static void InitializePuzzle3D(Widget request, Widget renew);
+static void ExposePuzzle3D(Widget renew, XEvent *event, Region region);
+static void MovePuzzle3DTl(Rubik3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DTop(Rubik3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DTr(Rubik3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DLeft(Rubik3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DRight(Rubik3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DBl(Rubik3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DBottom(Rubik3DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DBr(Rubik3DWidget w,
+	XEvent *event, char **args, int nArgs);
 
-static char defaultTranslationsRubik3D[] =
+static char defaultTranslationsPuzzle3D[] =
 "<KeyPress>q: Quit()\n\
  Ctrl<KeyPress>C: Quit()\n\
  <KeyPress>osfCancel: Hide()\n\
@@ -89,66 +97,70 @@ static char defaultTranslationsRubik3D[] =
  <KeyPress>R15: MoveBr()\n\
  <Btn1Down>: Select()\n\
  <Btn1Up>: Release()\n\
- <KeyPress>p: Practice()\n\
  <Btn2Down>: PracticeMaybe()\n\
  <Btn2Down>(2+): Practice2()\n\
- <KeyPress>r: Randomize()\n\
  <Btn3Down>: RandomizeMaybe()\n\
  <Btn3Down>(2+): Randomize2()\n\
  <Btn4Down>: MoveTop()\n\
  <Btn5Down>: MoveBottom()\n\
  <KeyPress>g: Get()\n\
  <KeyPress>w: Write()\n\
- <KeyPress>c: Clear()\n\
  <KeyPress>u: Undo()\n\
+ <KeyPress>r: Redo()\n\
+ <KeyPress>c: Clear()\n\
+ <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
+ <KeyPress>p: Practice()\n\
+ <KeyPress>o: Orientize()\n\
  <KeyPress>d: Decrement()\n\
  <KeyPress>i: Increment()\n\
  <KeyPress>x: IncrementX()\n\
  <KeyPress>y: IncrementY()\n\
- <KeyPress>z: IncrementZ()\n\
- <KeyPress>o: Orientize()\n\
+ <KeyPress>f: IncrementZ()\n\
+ <KeyPress>v: View()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
-static XtActionsRec actionsListRubik3D[] =
+static XtActionsRec actionsListPuzzle3D[] =
 {
-	{(char *) "Quit", (XtActionProc) QuitRubik},
-	{(char *) "Hide", (XtActionProc) HideRubik},
-	{(char *) "MoveCcw", (XtActionProc) MoveRubikCcw},
-	{(char *) "MoveTl", (XtActionProc) MoveRubik3DTl},
-	{(char *) "MoveTop", (XtActionProc) MoveRubik3DTop},
-	{(char *) "MoveTr", (XtActionProc) MoveRubik3DTr},
-	{(char *) "MoveLeft", (XtActionProc) MoveRubik3DLeft},
-	{(char *) "MoveCw", (XtActionProc) MoveRubikCw},
-	{(char *) "MoveRight", (XtActionProc) MoveRubik3DRight},
-	{(char *) "MoveBl", (XtActionProc) MoveRubik3DBl},
-	{(char *) "MoveBottom", (XtActionProc) MoveRubik3DBottom},
-	{(char *) "MoveBr", (XtActionProc) MoveRubik3DBr},
-	{(char *) "Select", (XtActionProc) SelectRubik},
-	{(char *) "Release", (XtActionProc) ReleaseRubik},
-	{(char *) "Practice", (XtActionProc) PracticeRubik},
-	{(char *) "PracticeMaybe", (XtActionProc) PracticeRubikMaybe},
-	{(char *) "Practice2", (XtActionProc) PracticeRubik2},
-	{(char *) "Randomize", (XtActionProc) RandomizeRubik},
-	{(char *) "RandomizeMaybe", (XtActionProc) RandomizeRubikMaybe},
-	{(char *) "Randomize2", (XtActionProc) RandomizeRubik2},
-	{(char *) "Get", (XtActionProc) GetRubik},
-	{(char *) "Write", (XtActionProc) WriteRubik},
-	{(char *) "Clear", (XtActionProc) ClearRubik},
-	{(char *) "Undo", (XtActionProc) UndoRubik},
-	{(char *) "Solve", (XtActionProc) SolveRubik},
-	{(char *) "Decrement", (XtActionProc) DecrementRubik},
-	{(char *) "Increment", (XtActionProc) IncrementRubik},
-	{(char *) "IncrementX", (XtActionProc) IncrementXRubik},
-	{(char *) "IncrementY", (XtActionProc) IncrementYRubik},
-	{(char *) "IncrementZ", (XtActionProc) IncrementZRubik},
-	{(char *) "Orientize", (XtActionProc) OrientizeRubik},
-	{(char *) "Enter", (XtActionProc) EnterRubik},
-	{(char *) "Leave", (XtActionProc) LeaveRubik}
+	{(char *) "Quit", (XtActionProc) QuitPuzzle},
+	{(char *) "Hide", (XtActionProc) HidePuzzle},
+	{(char *) "MoveCcw", (XtActionProc) MovePuzzleCcw},
+	{(char *) "MoveTl", (XtActionProc) MovePuzzle3DTl},
+	{(char *) "MoveTop", (XtActionProc) MovePuzzle3DTop},
+	{(char *) "MoveTr", (XtActionProc) MovePuzzle3DTr},
+	{(char *) "MoveLeft", (XtActionProc) MovePuzzle3DLeft},
+	{(char *) "MoveCw", (XtActionProc) MovePuzzleCw},
+	{(char *) "MoveRight", (XtActionProc) MovePuzzle3DRight},
+	{(char *) "MoveBl", (XtActionProc) MovePuzzle3DBl},
+	{(char *) "MoveBottom", (XtActionProc) MovePuzzle3DBottom},
+	{(char *) "MoveBr", (XtActionProc) MovePuzzle3DBr},
+	{(char *) "Select", (XtActionProc) SelectPuzzle},
+	{(char *) "Release", (XtActionProc) ReleasePuzzle},
+	{(char *) "PracticeMaybe", (XtActionProc) PracticePuzzleMaybe},
+	{(char *) "Practice2", (XtActionProc) PracticePuzzle2},
+	{(char *) "RandomizeMaybe", (XtActionProc) RandomizePuzzleMaybe},
+	{(char *) "Randomize2", (XtActionProc) RandomizePuzzle2},
+	{(char *) "Get", (XtActionProc) GetPuzzle},
+	{(char *) "Write", (XtActionProc) WritePuzzle},
+	{(char *) "Undo", (XtActionProc) UndoPuzzle},
+	{(char *) "Redo", (XtActionProc) RedoPuzzle},
+	{(char *) "Clear", (XtActionProc) ClearPuzzle},
+	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
+	{(char *) "Solve", (XtActionProc) SolvePuzzle},
+	{(char *) "Practice", (XtActionProc) PracticePuzzle},
+	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
+	{(char *) "Decrement", (XtActionProc) DecrementPuzzle},
+	{(char *) "Increment", (XtActionProc) IncrementPuzzle},
+	{(char *) "IncrementX", (XtActionProc) IncrementXPuzzle},
+	{(char *) "IncrementY", (XtActionProc) IncrementYPuzzle},
+	{(char *) "IncrementZ", (XtActionProc) IncrementZPuzzle},
+	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Enter", (XtActionProc) EnterPuzzle},
+	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
 
-static XtResource resourcesRubik3D[] =
+static XtResource resourcesPuzzle3D[] =
 {
 	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
 	 XtOffset(RubikWidget, core.width),
@@ -263,29 +275,29 @@ Rubik3DClassRec rubik3dClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeRubik3D,		/* initialize */
+		(XtInitProc) InitializePuzzle3D,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
-		actionsListRubik3D,	/* actions */
-		XtNumber(actionsListRubik3D),	/* num actions */
-		resourcesRubik3D,	/* resources */
-		XtNumber(resourcesRubik3D),	/* num resources */
+		actionsListPuzzle3D,	/* actions */
+		XtNumber(actionsListPuzzle3D),	/* num actions */
+		resourcesPuzzle3D,	/* resources */
+		XtNumber(resourcesPuzzle3D),	/* num resources */
 		NULLQUARK,	/* xrm class */
 		TRUE,		/* compress motion */
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
 		NULL,		/* destroy */
-		(XtWidgetProc) ResizeRubik3D,	/* resize */
-		(XtExposeProc) ExposeRubik3D,	/* expose */
-		(XtSetValuesFunc) SetValuesRubik3D,	/* set values */
+		(XtWidgetProc) ResizePuzzle3D,	/* resize */
+		(XtExposeProc) ExposePuzzle3D,	/* expose */
+		(XtSetValuesFunc) SetValuesPuzzle3D,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
 		XtInheritAcceptFocus,	/* accept focus */
 		XtVersion,	/* version */
 		NULL,		/* callback private */
-		defaultTranslationsRubik3D,	/* tm table */
+		defaultTranslationsPuzzle3D,	/* tm table */
 		NULL,		/* query geometry */
 		NULL,		/* display accelerator */
 		NULL		/* extension */
@@ -1166,7 +1178,7 @@ NarrowSelection3D(Rubik3DWidget w, int *face, int *position, int *direction)
 
 #ifndef WINVER
 static      Boolean
-SetValuesRubik3D(Widget current, Widget request, Widget renew)
+SetValuesPuzzle3D(Widget current, Widget request, Widget renew)
 {
 	Rubik3DWidget c = (Rubik3DWidget) current, w = (Rubik3DWidget) renew;
 	Boolean     redraw = False;
@@ -1175,7 +1187,7 @@ SetValuesRubik3D(Widget current, Widget request, Widget renew)
 	    w->rubik.sizey != c->rubik.sizey ||
 	    w->rubik.sizez != c->rubik.sizez) {
 		ResetPieces((RubikWidget) w);
-		ResizeRubik3D(w);
+		ResizePuzzle3D(w);
 		redraw = True;
 	}
 	if (w->rubik3d.cubeSizex.x != c->rubik3d.cubeSizex.x ||
@@ -1184,7 +1196,7 @@ SetValuesRubik3D(Widget current, Widget request, Widget renew)
 	    w->rubik3d.cubeSizex.y != c->rubik3d.cubeSizex.y ||
 	    w->rubik3d.cubeSizey.y != c->rubik3d.cubeSizey.y ||
 	    w->rubik3d.cubeSizez.y != c->rubik3d.cubeSizez.y) {
-		ResizeRubik3D(w);
+		ResizePuzzle3D(w);
 		redraw = True;
 	}
 	return (redraw);
@@ -1195,7 +1207,7 @@ SetValuesRubik3D(Widget current, Widget request, Widget renew)
 static
 #endif
 void
-ResizeRubik3D(Rubik3DWidget w)
+ResizePuzzle3D(Rubik3DWidget w)
 {
 #ifdef WINVER
 	RECT        rect;
@@ -1249,7 +1261,7 @@ ResizeRubik3D(Rubik3DWidget w)
 static
 #endif
 void
-InitializeRubik3D(
+InitializePuzzle3D(
 #ifdef WINVER
 Rubik3DWidget w
 #else
@@ -1263,18 +1275,18 @@ Widget request, Widget renew
 	SetAllColors((RubikWidget) w);
 #endif
 	w->rubik.dim = 3;
-	ResizeRubik3D(w);
+	ResizePuzzle3D(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-ExposeRubik3D(
+ExposePuzzle3D(
 #ifdef WINVER
 Rubik3DWidget w
 #else
-Widget renew, XEvent * event, Region region
+Widget renew, XEvent *event, Region region
 #endif
 )
 {
@@ -1292,58 +1304,58 @@ Widget renew, XEvent * event, Region region
 
 #ifndef WINVER
 static void
-MoveRubik3DTl(Rubik3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DTl(Rubik3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, TL,
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, TL,
 		       (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubik3DTop(Rubik3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DTop(Rubik3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, TOP,
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, TOP,
 		       (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubik3DTr(Rubik3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DTr(Rubik3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, TR,
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, TR,
 		       (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubik3DLeft(Rubik3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DLeft(Rubik3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
 		       (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubik3DRight(Rubik3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DRight(Rubik3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
 		       (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubik3DBl(Rubik3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DBl(Rubik3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, BL,
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, BL,
 		       (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubik3DBottom(Rubik3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DBottom(Rubik3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
 		       (int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubik3DBr(Rubik3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DBr(Rubik3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, BR,
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, BR,
 		       (int) (event->xkey.state & ControlMask));
 }
 #endif
