@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1994 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -54,7 +54,6 @@ extern Rubik3DClassRec rubik3dClassRec;
 #define DOWN 0
 #define UP 1
 #define COORD3D 12
-#define MAXVIEWS 2
 
 #define SQRT_3 1.732050808
 #define MULTIPLY(a) ((int)((double)a*SQRT_3/2.0))
@@ -86,9 +85,9 @@ extern Boolean NarrowSelection3D(Rubik3DWidget w,
 extern void DrawFrame3D(Rubik3DWidget w, Boolean focus);
 extern void DrawSquare3D(Rubik3DWidget w, int face, int position, int offset);
 #ifdef WINVER
-extern void ResizeRubik3D(Rubik3DWidget w);
-extern void InitializeRubik3D(Rubik3DWidget w);
-extern void ExposeRubik3D(Rubik3DWidget w);
+extern void ResizePuzzle3D(Rubik3DWidget w);
+extern void InitializePuzzle3D(Rubik3DWidget w);
+extern void ExposePuzzle3D(Rubik3DWidget w);
 #endif
 
 #endif /* _Rubik3dP_h */
