@@ -33,7 +33,7 @@
  * This file is provided AS IS with no warranties of any kind.  The author
  * shall have no liability with respect to the infringement of copyrights,
  * trade secrets or any patents by this file or any part thereof.  In no
- * event will the author be liable for any lost revenue or profits or
+ *event will the author be liable for any lost revenue or profits or
  * other special, indirect and consequential damages.
  *
  * This mode shows an auto-solving rubik's cube "puzzle". If somebody
@@ -59,7 +59,7 @@
 #include "RubikP.h"
 #include "RubikGLP.h"
 
-static GLXContext glXContext;
+static GLXContext *glXContext = (GLXContext *) NULL;
 static Boolean madeCurrent = False;
 
 static float front_shininess[] =
@@ -168,20 +168,28 @@ static int rotateDir[MAXFACES][MAXORIENT] =
  *  BOTTOM => X, Z
  */
 
-static Boolean SetValuesRubikGL(Widget current, Widget request, Widget renew);
-static void ResizeRubikGL(RubikGLWidget w);
-static void InitializeRubikGL(Widget request, Widget renew);
-static void ExposeRubikGL(Widget renew, XEvent * event, Region region);
-static void MoveRubikGLTl(RubikGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubikGLTop(RubikGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubikGLTr(RubikGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubikGLLeft(RubikGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubikGLRight(RubikGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubikGLBl(RubikGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubikGLBottom(RubikGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRubikGLBr(RubikGLWidget w, XEvent * event, char **args, int nArgs);
+static Boolean SetValuesPuzzleGL(Widget current, Widget request, Widget renew);
+static void ResizePuzzleGL(RubikGLWidget w);
+static void InitializePuzzleGL(Widget request, Widget renew);
+static void ExposePuzzleGL(Widget renew, XEvent *event, Region region);
+static void MovePuzzleGLTl(RubikGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLTop(RubikGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLTr(RubikGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLLeft(RubikGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLRight(RubikGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLBl(RubikGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLBottom(RubikGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLBr(RubikGLWidget w,
+	XEvent *event, char **args, int nArgs);
 
-static char defaultTranslationsRubikGL[] =
+static char defaultTranslationsPuzzleGL[] =
 "<KeyPress>q: Quit()\n\
  Ctrl<KeyPress>C: Quit()\n\
  <KeyPress>osfCancel: Hide()\n\
@@ -228,64 +236,68 @@ static char defaultTranslationsRubikGL[] =
  <KeyPress>R15: MoveBr()\n\
  <Btn1Down>: Select()\n\
  <Btn1Up>: Release()\n\
- <KeyPress>p: Practice()\n\
  <Btn2Down>: PracticeMaybe()\n\
  <Btn2Down>(2+): Practice2()\n\
- <KeyPress>r: Randomize()\n\
  <Btn3Down>: RandomizeMaybe()\n\
  <Btn3Down>(2+): Randomize2()\n\
  <KeyPress>g: Get()\n\
  <KeyPress>w: Write()\n\
- <KeyPress>c: Clear()\n\
  <KeyPress>u: Undo()\n\
+ <KeyPress>r: Redo()\n\
+ <KeyPress>c: Clear()\n\
+ <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
+ <KeyPress>p: Practice()\n\
+ <KeyPress>o: Orientize()\n\
  <KeyPress>d: Decrement()\n\
  <KeyPress>i: Increment()\n\
  <KeyPress>x: IncrementX()\n\
  <KeyPress>y: IncrementY()\n\
- <KeyPress>z: IncrementZ()\n\
- <KeyPress>o: Orientize()\n\
+ <KeyPress>f: IncrementZ()\n\
+ <KeyPress>v: View()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
-static XtActionsRec actionsListRubikGL[] =
+static XtActionsRec actionsListPuzzleGL[] =
 {
-	{(char *) "Quit", (XtActionProc) QuitRubik},
-	{(char *) "Hide", (XtActionProc) HideRubik},
-	{(char *) "MoveCcw", (XtActionProc) MoveRubikCcw},
-	{(char *) "MoveTl", (XtActionProc) MoveRubikGLTl},
-	{(char *) "MoveTop", (XtActionProc) MoveRubikGLTop},
-	{(char *) "MoveTr", (XtActionProc) MoveRubikGLTr},
-	{(char *) "MoveLeft", (XtActionProc) MoveRubikGLLeft},
-	{(char *) "MoveCw", (XtActionProc) MoveRubikCw},
-	{(char *) "MoveRight", (XtActionProc) MoveRubikGLRight},
-	{(char *) "MoveBl", (XtActionProc) MoveRubikGLBl},
-	{(char *) "MoveBottom", (XtActionProc) MoveRubikGLBottom},
-	{(char *) "MoveBr", (XtActionProc) MoveRubikGLBr},
-	{(char *) "Select", (XtActionProc) SelectRubik},
-	{(char *) "Release", (XtActionProc) ReleaseRubik},
-	{(char *) "Practice", (XtActionProc) PracticeRubik},
-	{(char *) "PracticeMaybe", (XtActionProc) PracticeRubikMaybe},
-	{(char *) "Practice2", (XtActionProc) PracticeRubik2},
-	{(char *) "Randomize", (XtActionProc) RandomizeRubik},
-	{(char *) "RandomizeMaybe", (XtActionProc) RandomizeRubikMaybe},
-	{(char *) "Randomize2", (XtActionProc) RandomizeRubik2},
-	{(char *) "Get", (XtActionProc) GetRubik},
-	{(char *) "Write", (XtActionProc) WriteRubik},
-	{(char *) "Clear", (XtActionProc) ClearRubik},
-	{(char *) "Undo", (XtActionProc) UndoRubik},
-	{(char *) "Solve", (XtActionProc) SolveRubik},
-	{(char *) "Decrement", (XtActionProc) DecrementRubik},
-	{(char *) "Increment", (XtActionProc) IncrementRubik},
-	{(char *) "IncrementX", (XtActionProc) IncrementXRubik},
-	{(char *) "IncrementY", (XtActionProc) IncrementYRubik},
-	{(char *) "IncrementZ", (XtActionProc) IncrementZRubik},
-	{(char *) "Orientize", (XtActionProc) OrientizeRubik},
-	{(char *) "Enter", (XtActionProc) EnterRubik},
-	{(char *) "Leave", (XtActionProc) LeaveRubik}
+	{(char *) "Quit", (XtActionProc) QuitPuzzle},
+	{(char *) "Hide", (XtActionProc) HidePuzzle},
+	{(char *) "MoveCcw", (XtActionProc) MovePuzzleCcw},
+	{(char *) "MoveTl", (XtActionProc) MovePuzzleGLTl},
+	{(char *) "MoveTop", (XtActionProc) MovePuzzleGLTop},
+	{(char *) "MoveTr", (XtActionProc) MovePuzzleGLTr},
+	{(char *) "MoveLeft", (XtActionProc) MovePuzzleGLLeft},
+	{(char *) "MoveCw", (XtActionProc) MovePuzzleCw},
+	{(char *) "MoveRight", (XtActionProc) MovePuzzleGLRight},
+	{(char *) "MoveBl", (XtActionProc) MovePuzzleGLBl},
+	{(char *) "MoveBottom", (XtActionProc) MovePuzzleGLBottom},
+	{(char *) "MoveBr", (XtActionProc) MovePuzzleGLBr},
+	{(char *) "Select", (XtActionProc) SelectPuzzle},
+	{(char *) "Release", (XtActionProc) ReleasePuzzle},
+	{(char *) "PracticeMaybe", (XtActionProc) PracticePuzzleMaybe},
+	{(char *) "Practice2", (XtActionProc) PracticePuzzle2},
+	{(char *) "RandomizeMaybe", (XtActionProc) RandomizePuzzleMaybe},
+	{(char *) "Randomize2", (XtActionProc) RandomizePuzzle2},
+	{(char *) "Get", (XtActionProc) GetPuzzle},
+	{(char *) "Write", (XtActionProc) WritePuzzle},
+	{(char *) "Undo", (XtActionProc) UndoPuzzle},
+	{(char *) "Redo", (XtActionProc) RedoPuzzle},
+	{(char *) "Clear", (XtActionProc) ClearPuzzle},
+	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
+	{(char *) "Solve", (XtActionProc) SolvePuzzle},
+	{(char *) "Practice", (XtActionProc) PracticePuzzle},
+	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
+	{(char *) "Decrement", (XtActionProc) DecrementPuzzle},
+	{(char *) "Increment", (XtActionProc) IncrementPuzzle},
+	{(char *) "IncrementX", (XtActionProc) IncrementXPuzzle},
+	{(char *) "IncrementY", (XtActionProc) IncrementYPuzzle},
+	{(char *) "IncrementZ", (XtActionProc) IncrementZPuzzle},
+	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Enter", (XtActionProc) EnterPuzzle},
+	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
 
-static XtResource resourcesRubikGL[] =
+static XtResource resourcesPuzzleGL[] =
 {
 	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
 	 XtOffset(RubikWidget, core.width),
@@ -310,22 +322,22 @@ static XtResource resourcesRubikGL[] =
 	 XtRString, (caddr_t) "cyan" /*XtDefaultForeground*/},
 	{XtNfaceColor0, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(RubikWidget, rubik.faceName[0]),
-	 XtRString, (caddr_t) "red"},
+	 XtRString, (caddr_t) "Red"},
 	{XtNfaceColor1, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(RubikWidget, rubik.faceName[1]),
-	 XtRString, (caddr_t) "yellow"},
+	 XtRString, (caddr_t) "Yellow"},
 	{XtNfaceColor2, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(RubikWidget, rubik.faceName[2]),
-	 XtRString, (caddr_t) "white"},
+	 XtRString, (caddr_t) "White"},
 	{XtNfaceColor3, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(RubikWidget, rubik.faceName[3]),
-	 XtRString, (caddr_t) "green"},
+	 XtRString, (caddr_t) "Green"},
 	{XtNfaceColor4, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(RubikWidget, rubik.faceName[4]),
-	 XtRString, (caddr_t) "orange"},
+	 XtRString, (caddr_t) "Orange"},
 	{XtNfaceColor5, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(RubikWidget, rubik.faceName[5]),
-	 XtRString, (caddr_t) "blue"},
+	 XtRString, (caddr_t) "Blue"},
 	{XtNpieceBorder, XtCColor, XtRPixel, sizeof (Pixel),
 	 XtOffset(RubikWidget, rubik.borderColor),
 	 XtRString, (caddr_t) "gray25" /*XtDefaultForeground*/},
@@ -335,6 +347,9 @@ static XtResource resourcesRubikGL[] =
 	{XtNfont, XtCFont, XtRString, sizeof (String),
 	 XtOffset(RubikWidget, rubik.font),
 	 XtRString, (caddr_t) "9x15bold"},
+	{XtNview, XtCView, XtRInt, sizeof (int),
+	 XtOffset(RubikWidget, rubik.view),
+	 XtRString, (caddr_t) "1"},
 	{XtNsizex, XtCSizeX, XtRInt, sizeof (int),
 	 XtOffset(RubikWidget, rubik.sizex),
 	 XtRString, (caddr_t) "3"},	/* DEFAULTFACETS */
@@ -400,29 +415,29 @@ RubikGLClassRec rubikGLClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeRubikGL,		/* initialize */
+		(XtInitProc) InitializePuzzleGL,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
-		actionsListRubikGL,	/* actions */
-		XtNumber(actionsListRubikGL),	/* num actions */
-		resourcesRubikGL,	/* resources */
-		XtNumber(resourcesRubikGL),	/* num resources */
+		actionsListPuzzleGL,	/* actions */
+		XtNumber(actionsListPuzzleGL),	/* num actions */
+		resourcesPuzzleGL,	/* resources */
+		XtNumber(resourcesPuzzleGL),	/* num resources */
 		NULLQUARK,	/* xrm class */
 		TRUE,		/* compress motion */
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
 		NULL,		/* destroy */
-		(XtWidgetProc) ResizeRubikGL,	/* resize */
-		(XtExposeProc) ExposeRubikGL,	/* expose */
-		(XtSetValuesFunc) SetValuesRubikGL,	/* set values */
+		(XtWidgetProc) ResizePuzzleGL,	/* resize */
+		(XtExposeProc) ExposePuzzleGL,	/* expose */
+		(XtSetValuesFunc) SetValuesPuzzleGL,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
 		XtInheritAcceptFocus,	/* accept focus */
 		XtVersion,	/* version */
 		NULL,		/* callback private */
-		defaultTranslationsRubikGL,	/* tm table */
+		defaultTranslationsPuzzleGL,	/* tm table */
 		NULL,		/* query geometry */
 		NULL,		/* display accelerator */
 		NULL		/* extension */
@@ -441,13 +456,14 @@ WidgetClass rubikGLWidgetClass = (WidgetClass) & rubikGLClassRec;
 void
 printCube(RubikGLWidget w)
 {
-	int         face, position, sizeOfRow, sizeOfColumn;
+	int face, position, sizeOfRow, sizeOfColumn;
 
 	for (face = 0; face < MAXFACES; face++) {
 		faceSizes(w, face, &sizeOfRow, &sizeOfColumn);
 		for (position = 0; position < sizeOfRow * sizeOfColumn; position++) {
-			(void) printf("%d %d  ", w->rubik.cubeLoc[face][position].face,
-				      w->rubik.cubeLoc[face][position].rotation);
+			(void) printf("%d %d  ",
+				w->rubik.cubeLoc[face][position].face,
+				w->rubik.cubeLoc[face][position].rotation);
 			if (!((position + 1) % sizeOfRow))
 				(void) printf("\n");
 		}
@@ -461,9 +477,9 @@ printCube(RubikGLWidget w)
 static void
 pickColor(RubikGLWidget w, int C, int mono)
 {
-       float Material[] = {1.0, 1.0, 1.0, 1.0};
-
+	float Material[] = {1.0, 1.0, 1.0, 1.0};
 	XColor color;
+
 	switch (C) {
 		case TOP_FACE:
 		case LEFT_FACE:
@@ -473,21 +489,26 @@ pickColor(RubikGLWidget w, int C, int mono)
 		case BACK_FACE:
 			break;
 		case TEST_FACE:
-			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, MaterialMagenta);
+			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,
+				MaterialMagenta);
 			return;
 		default:
-			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, MaterialCyan);
+			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,
+				MaterialCyan);
 			return;
 	}
 	color.pixel = w->rubik.faceColor[C];
 	XQueryColor(XtDisplay(w), DefaultColormapOfScreen(XtScreen(w)),
 		&color);
-	/*(void) printf("i%d %d %d\n", color.red, color.green, color.blue);*/
+#ifdef DEBUG
+	(void) printf("i%d %d %d\n", color.red, color.green, color.blue);
+#endif
 	if (mono) {
 		float intensity = 0.3 * color.red + 0.59 * color.green +
 			0.11 * color.blue;
-
-		/*(void) printf("m%g\n", intensity);*/
+#ifdef DEBUG
+		(void) printf("m%g\n", intensity);
+#endif
 		Material[0] *= (float) intensity / 0xFFFF;
 		Material[1] *= (float) intensity / 0xFFFF;
 		Material[2] *= (float) intensity / 0xFFFF;
@@ -496,7 +517,9 @@ pickColor(RubikGLWidget w, int C, int mono)
 		Material[1] *= (float) color.green / 0xFFFF;
 		Material[2] *= (float) color.blue / 0xFFFF;
 	}
-	/*(void) printf("f%g %g %g\n", Material[0], Material[1], Material[2]);*/
+#ifdef DEBUG
+	(void) printf("f%g %g %g\n", Material[0], Material[1], Material[2]);
+#endif
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, Material);
 }
 
@@ -645,10 +668,9 @@ DrawCubit(RubikGLWidget w,
 	Boolean mono = w->rubik.mono;
 	char *buf1, *buf2;
 
-#define CUTDEPTH (STICKERDEPTH+0.001)
 	if (!draw_stickerless_cubit())
 		return False;
-	if (back != NO_POSITION) {
+	if (back != NO_POSITION) { /* Blue */
 		glBegin(GL_POLYGON);
 		pickColor(w, w->rubik.cubeLoc[BACK_FACE][back].face, mono);
 		glNormal3f(0.0, 0.0, -1.0);
@@ -704,7 +726,7 @@ DrawCubit(RubikGLWidget w,
 			glEnd();
 		}
 	}
-	if (front != NO_POSITION) {
+	if (front != NO_POSITION) { /* White */
 		glBegin(GL_POLYGON);
 		glNormal3f(0.0, 0.0, 1.0);
 		pickColor(w, w->rubik.cubeLoc[FRONT_FACE][front].face, mono);
@@ -760,7 +782,7 @@ DrawCubit(RubikGLWidget w,
 			glEnd();
 		}
 	}
-	if (left != NO_POSITION) {
+	if (left != NO_POSITION) { /* Yellow */
 		glBegin(GL_POLYGON);
 		pickColor(w, w->rubik.cubeLoc[LEFT_FACE][left].face, mono);
 		glNormal3f(-1.0, 0.0, 0.0);
@@ -816,7 +838,7 @@ DrawCubit(RubikGLWidget w,
 			glEnd();
 		}
 	}
-	if (right != NO_POSITION) {
+	if (right != NO_POSITION) { /* Green */
 		glBegin(GL_POLYGON);
 		pickColor(w, w->rubik.cubeLoc[RIGHT_FACE][right].face, mono);
 		glNormal3f(1.0, 0.0, 0.0);
@@ -872,7 +894,7 @@ DrawCubit(RubikGLWidget w,
 			glEnd();
 		}
 	}
-	if (bottom != NO_POSITION) {
+	if (bottom != NO_POSITION) { /* Orange */
 		glBegin(GL_POLYGON);
 		pickColor(w, w->rubik.cubeLoc[BOTTOM_FACE][bottom].face, mono);
 		glNormal3f(0.0, -1.0, 0.0);
@@ -928,7 +950,7 @@ DrawCubit(RubikGLWidget w,
 			glEnd();
 		}
 	}
-	if (top != NO_POSITION) {
+	if (top != NO_POSITION) { /* Red */
 		glBegin(GL_POLYGON);
 		pickColor(w, w->rubik.cubeLoc[TOP_FACE][top].face, mono);
 		glNormal3f(0.0, 1.0, 0.0);
@@ -991,8 +1013,8 @@ DrawCubit(RubikGLWidget w,
 static void
 convertMove(RubikGLWidget w, RubikMove move, RubikSlice * slice)
 {
-	RubikLoc    plane;
-	int         sizeOfRow, sizeOfColumn;
+	RubikLoc plane;
+	int sizeOfRow, sizeOfColumn;
 
 	plane = rotateSlice[(int) move.face][move.direction % 2];
 	(*slice).face = plane.face;
@@ -1034,9 +1056,9 @@ DrawCube(RubikGLWidget w)
 #define MIDY(a) (((GLfloat)(2*a-MAXSIZEY+1))/2.0)
 #define MIDZ(a) (((GLfloat)(2*a-MAXSIZEZ+1))/2.0)
 #define DRAW_CUBIT(w,b,f,l,r,bm,t) if (!DrawCubit(w,b,f,l,r,bm,t)) return False
-	RubikSlice  slice;
-	GLfloat     rotateStep;
-	int         i, j, k;
+	RubikSlice slice;
+	GLfloat rotateStep;
+	int i, j, k;
 
 	if (w->rubikGL.movement.face == NO_FACE) {
 		slice.face = NO_FACE;
@@ -1059,7 +1081,7 @@ DrawCube(RubikGLWidget w)
 #if 0
 slice.face = 0; /* DAB */
 slice.face = FRONT_FACE; /* DAB */
-printf("slice.face %d\n", slice.face);
+(void) printf("slice.face %d\n", slice.face);
 #endif
 	switch (slice.face) {
 		case NO_FACE:
@@ -1302,9 +1324,9 @@ printf("slice.face %d\n", slice.face);
 				glTranslatef(0, S1, 0);
 				DRAW_CUBIT(w,
 					   FIRSTX + MAXSIZEX * j, (MAXSIZEZ == 1) ?
-				           FIRSTX + MAXSIZEX * REVY(j) : NO_POSITION,
+					   FIRSTX + MAXSIZEX * REVY(j) : NO_POSITION,
 					   FIRSTZ + MAXSIZEZ * REVY(j), (MAXSIZEX == 1) ?
-				           LASTZ + MAXSIZEZ * REVY(j) : NO_POSITION,
+					   LASTZ + MAXSIZEZ * REVY(j) : NO_POSITION,
 					   NO_POSITION, NO_POSITION);
 			}
 			if (MAXSIZEY > 1) {
@@ -1321,15 +1343,15 @@ printf("slice.face %d\n", slice.face);
 				DRAW_CUBIT(w,
 					   NO_POSITION, NO_POSITION,
 					   k + MAXSIZEZ * LASTY, (MAXSIZEX == 1) ?
-				           REVZ(k) + MAXSIZEZ * LASTY : NO_POSITION,
+					   REVZ(k) + MAXSIZEZ * LASTY : NO_POSITION,
 					   FIRSTX + MAXSIZEX * REVZ(k), (MAXSIZEY == 1) ?
-				           FIRSTX + MAXSIZEX * k : NO_POSITION);
+					   FIRSTX + MAXSIZEX * k : NO_POSITION);
 				for (j = 1; j < MAXSIZEY - 1; j++) {
 					glTranslatef(0, S1, 0);
 					DRAW_CUBIT(w,
 						   NO_POSITION, NO_POSITION,
 						   k + MAXSIZEZ * REVY(j), (MAXSIZEX == 1) ?
-				                   REVZ(k) + MAXSIZEZ * REVY(j) : NO_POSITION,
+						   REVZ(k) + MAXSIZEZ * REVY(j) : NO_POSITION,
 						   NO_POSITION, NO_POSITION);
 				}
 				if (MAXSIZEY > 1) {
@@ -1337,7 +1359,7 @@ printf("slice.face %d\n", slice.face);
 				DRAW_CUBIT(w,
 					   NO_POSITION, NO_POSITION,
 					   k + MAXSIZEZ * FIRSTY, (MAXSIZEX == 1) ?
-				           REVZ(k) + MAXSIZEZ * FIRSTY : NO_POSITION,
+					   REVZ(k) + MAXSIZEZ * FIRSTY : NO_POSITION,
 					   NO_POSITION, FIRSTX + MAXSIZEX * k);
 				}
 			}
@@ -1348,14 +1370,14 @@ printf("slice.face %d\n", slice.face);
 				   LASTZ + MAXSIZEZ * LASTY, (MAXSIZEX == 1) ?
 				   FIRSTZ + MAXSIZEZ * LASTY : NO_POSITION,
 				   FIRSTX + MAXSIZEX * FIRSTZ, (MAXSIZEY == 1) ?
-				           FIRSTX + MAXSIZEX * LASTZ : NO_POSITION);
+					   FIRSTX + MAXSIZEX * LASTZ : NO_POSITION);
 			if (MAXSIZEY > 1) {
 			for (j = 1; j < MAXSIZEY - 1; j++) {
 				glTranslatef(0, S1, 0);
 				DRAW_CUBIT(w,
 					   NO_POSITION, FIRSTX + MAXSIZEX * REVY(j),
 					   LASTZ + MAXSIZEZ * REVY(j), (MAXSIZEX == 1) ?
-				           FIRSTZ + MAXSIZEZ * REVY(j) : NO_POSITION,
+					   FIRSTZ + MAXSIZEZ * REVY(j) : NO_POSITION,
 					   NO_POSITION, NO_POSITION);
 			}
 			glTranslatef(0, S1, 0);
@@ -1374,15 +1396,15 @@ printf("slice.face %d\n", slice.face);
 				glTranslatef(MIDX(i), -HALFY, -HALFZ);
 				DRAW_CUBIT(w,
 					   i + MAXSIZEX * FIRSTY, (MAXSIZEZ == 1) ?
-				           i + MAXSIZEX * LASTY : NO_POSITION,
+					   i + MAXSIZEX * LASTY : NO_POSITION,
 					   NO_POSITION, NO_POSITION,
 					   i + MAXSIZEX * LASTZ, (MAXSIZEY == 1) ?
-				           i + MAXSIZEX * FIRSTZ : NO_POSITION);
+					   i + MAXSIZEX * FIRSTZ : NO_POSITION);
 				for (j = 1; j < MAXSIZEY - 1; j++) {
 					glTranslatef(0, S1, 0);
 					DRAW_CUBIT(w,
 						   i + MAXSIZEX * j, (MAXSIZEZ == 1) ?
-				                   i + MAXSIZEX * REVY(j) : NO_POSITION,
+						   i + MAXSIZEX * REVY(j) : NO_POSITION,
 						   NO_POSITION, NO_POSITION,
 						   NO_POSITION, NO_POSITION);
 				}
@@ -1390,7 +1412,7 @@ printf("slice.face %d\n", slice.face);
 				glTranslatef(0, S1, 0);
 				DRAW_CUBIT(w,
 					   i + MAXSIZEX * LASTY, (MAXSIZEZ == 1) ?
-				           i + MAXSIZEX * FIRSTY : NO_POSITION,
+					   i + MAXSIZEX * FIRSTY : NO_POSITION,
 					   NO_POSITION, NO_POSITION,
 					   NO_POSITION, i + MAXSIZEX * FIRSTZ);
 				}
@@ -1400,7 +1422,7 @@ printf("slice.face %d\n", slice.face);
 						   NO_POSITION, NO_POSITION,
 						   NO_POSITION, NO_POSITION,
 						   i + MAXSIZEX * REVZ(k), (MAXSIZEY == 1) ?
-					           i + MAXSIZEX * k : NO_POSITION);
+						   i + MAXSIZEX * k : NO_POSITION);
 					/* Center */
 					if (MAXSIZEY > 1) {
 					glTranslatef(0, SY, 0);
@@ -1448,7 +1470,7 @@ printf("slice.face %d\n", slice.face);
 				glTranslatef(0, S1, 0);
 				DRAW_CUBIT(w,
 					   LASTX + MAXSIZEX * j, (MAXSIZEZ == 1) ?
-				           LASTX + MAXSIZEX * REVY(j) : NO_POSITION,
+					   LASTX + MAXSIZEX * REVY(j) : NO_POSITION,
 					   NO_POSITION, LASTZ + MAXSIZEZ * REVY(j),
 					   NO_POSITION, NO_POSITION);
 			}
@@ -1810,7 +1832,7 @@ MovePiecesGL(RubikGLWidget w, int face, int position, int direction,
 void
 DrawAllPiecesGL(RubikGLWidget w)
 {
-	if (!glXMakeCurrent(XtDisplay(w), XtWindow(w), glXContext)) {
+	if (!glXMakeCurrent(XtDisplay(w), XtWindow(w), *glXContext)) {
 		XtWarning("GLX error");
 	}
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
@@ -1820,23 +1842,27 @@ DrawAllPiecesGL(RubikGLWidget w)
 		glScalef(SCALE, SCALE * w->core.width / w->core.height, SCALE);
 	else
 		glScalef(SCALE * w->core.height / w->core.width, SCALE, SCALE);
-#if 0
-	glRotatef(w->rubikGL.step / 2, 0, 1, 0); /* >-  vertical */
-	glRotatef(w->rubikGL.step / 2, 1, 0, 0);
-#endif
-#if 1
-	glRotatef(w->rubikGL.step / 2, 1, 0, 0); /* Y  !vertical */
-	glRotatef(w->rubikGL.step / 2, 0, 1, 0);
-#endif
-#if 0
-	glRotatef(2 * w->rubikGL.step, 0, 0, 1); /* -<  vertical */
-	glRotatef(5 * w->rubikGL.step / 2, 0, 1, 0);
-	glRotatef(w->rubikGL.step / 2, 1, 0, 0);
-#endif
-#if 0
-	glRotatef(5 * w->rubikGL.step / 2, 0, 1, 0); /* ^  !vertical */
-	glRotatef(w->rubikGL.step / 2, 1, 0, 1);
-#endif
+	switch (w->rubik.view) {
+		case 0: /* >-  vertical */
+			glRotatef(w->rubikGL.step / 2, 0, 1, 0);
+			glRotatef(w->rubikGL.step / 2, 1, 0, 0);
+			break;
+		case 1: /* Y  !vertical */
+			glRotatef(w->rubikGL.step / 2, 1, 0, 0);
+			glRotatef(w->rubikGL.step / 2, 0, 1, 0);
+			break;
+		case 2: /* -<  vertical */
+			glRotatef(2 * w->rubikGL.step, 0, 0, 1);
+			glRotatef(5 * w->rubikGL.step / 2, 0, 1, 0);
+			glRotatef(w->rubikGL.step / 2, 1, 0, 0);
+			break;
+		case 3: /* ^  !vertical */
+			glRotatef(5 * w->rubikGL.step / 2, 0, 1, 0);
+			glRotatef(w->rubikGL.step / 2, 1, 0, 1);
+			break;
+		default:
+			break;
+	}
 	glRotatef(w->rubikGL.angleStepX, 0, 1, 0);
 	glRotatef(w->rubikGL.angleStepY, 1, 0, 0);
 	glRotatef(w->rubikGL.angleStepZ, 0, 0, 1);
@@ -1862,27 +1888,33 @@ SelectPiecesGL(RubikGLWidget w, int x, int y, int *face, int *position)
 }
 
 static      Boolean
-SetValuesRubikGL(Widget current, Widget request, Widget renew)
+SetValuesPuzzleGL(Widget current, Widget request, Widget renew)
 {
 	RubikGLWidget c = (RubikGLWidget) current, w = (RubikGLWidget) renew;
 	Boolean     redraw = False;
 
+	if (w->rubik.view != c->rubik.view) {
+		ResizePuzzleGL(w);
+		redraw = True;
+	}
 	if (w->rubik.sizex != c->rubik.sizex ||
 	    w->rubik.sizey != c->rubik.sizey ||
 	    w->rubik.sizez != c->rubik.sizez) {
 		ResetPieces((RubikWidget) w);
-		ResizeRubikGL(w);
+		ResizePuzzleGL(w);
 		redraw = True;
 	}
+#if 0
 	if (w->rubikGL.cubeSizex.x != c->rubikGL.cubeSizex.x ||
 	    w->rubikGL.cubeSizey.x != c->rubikGL.cubeSizey.x ||
 	    w->rubikGL.cubeSizez.x != c->rubikGL.cubeSizez.x ||
 	    w->rubikGL.cubeSizex.y != c->rubikGL.cubeSizex.y ||
 	    w->rubikGL.cubeSizey.y != c->rubikGL.cubeSizey.y ||
 	    w->rubikGL.cubeSizez.y != c->rubikGL.cubeSizez.y) {
-		ResizeRubikGL(w);
+		ResizePuzzleGL(w);
 		redraw = True;
 	}
+#endif
 	if (w->rubik.delay != c->rubik.delay) {
 		w->rubikGL.numTurnInc = ((w->rubik.delay < MAXTURNINC) ?
 			w->rubik.delay + 1 : MAXTURNINC);
@@ -1897,21 +1929,27 @@ initGL(RubikGLWidget w) {
 
 	/*XGetWindowAttributes(XtDisplay(w), XtWindow(w), &xgwa); */
 	xviIn.screen = DefaultScreen(XtDisplay(w));
-        xviIn.visualid = XVisualIDFromVisual(DefaultVisual(XtDisplay(w),
+	xviIn.visualid = XVisualIDFromVisual(DefaultVisual(XtDisplay(w),
 		xviIn.screen));
 	xviOut = XGetVisualInfo(XtDisplay(w), VisualScreenMask | VisualIDMask,
 		&xviIn, &numVis);
 	if (!xviOut) {
 		XtWarning("Could not get XVisualInfo");
 		return (GLXContext *) NULL;
-        }
-	glXContext = glXCreateContext(XtDisplay(w), xviOut, 0, GL_TRUE);
+	}
+	if (glXContext)
+		free(glXContext);
+	if ((glXContext = (GLXContext *) malloc(sizeof (GLXContext))) ==
+			NULL) {
+		DISPLAY_ERROR("Not enough memory for glx info, exiting.");
+	}
+	*glXContext = glXCreateContext(XtDisplay(w), xviOut, 0, GL_TRUE);
 	(void) XFree((char *) xviOut);
-	if (!glXContext) {
+	if (!*glXContext) {
 		XtWarning("Could not create rendering context");
 		return (GLXContext *) NULL;
-        }
-	return &glXContext;
+	}
+	return glXContext;
 }
 
 static void
@@ -1919,7 +1957,7 @@ makeCurrentGL(RubikGLWidget w)
 {
 	GLboolean   rgbaMode;
 
-	if (!glXMakeCurrent(XtDisplay(w), XtWindow(w), glXContext)) {
+	if (!glXMakeCurrent(XtDisplay(w), XtWindow(w), *glXContext)) {
 		XtWarning("GLX error");
 	} else {
 		madeCurrent = True;
@@ -1933,7 +1971,7 @@ makeCurrentGL(RubikGLWidget w)
 			DefaultScreen(XtDisplay(w))));
 	}
 
-	ResizeRubikGL(w);
+	ResizePuzzleGL(w);
 
 	glDrawBuffer(GL_BACK);
 	glClearDepth(1.0);
@@ -1960,18 +1998,18 @@ makeCurrentGL(RubikGLWidget w)
 
 	w->rubikGL.step = 90;
 #if 0
-        w->rubikGL.vx = 0.05;
+	w->rubikGL.vx = 0.05;
 	if (LRAND() & 1)
 		w->rubikGL.vx *= -1.0;
-        w->rubikGL.vy = 0.05;
+	w->rubikGL.vy = 0.05;
 	if (LRAND() & 1)
 		w->rubikGL.vy *= -1.0;
 #endif
-        w->rubikGL.rotateStep = 0.0;
+	w->rubikGL.rotateStep = 0.0;
 }
 
 static void
-ResizeRubikGL(RubikGLWidget w)
+ResizePuzzleGL(RubikGLWidget w)
 {
 	glViewport(0, 0, (GLint) w->core.width,
 		(GLint) w->core.height);
@@ -1983,7 +2021,7 @@ ResizeRubikGL(RubikGLWidget w)
 }
 
 static void
-InitializeRubikGL(Widget request, Widget renew)
+InitializePuzzleGL(Widget request, Widget renew)
 {
 	RubikGLWidget w = (RubikGLWidget) renew;
 
@@ -1992,14 +2030,18 @@ InitializeRubikGL(Widget request, Widget renew)
 	w->rubikGL.numTurnInc = ((w->rubik.delay < MAXTURNINC) ?
 		w->rubik.delay + 1 : MAXTURNINC);
 	(void) initGL(w);
-	ResizeRubikGL(w);
+	ResizePuzzleGL(w);
 	w->rubikGL.movement.face = NO_FACE;
 	w->rubikGL.movement.position = NO_POSITION;
 	w->rubikGL.movement.direction = NO_DEPTH;
+	w->rubikGL.rotateStep = 0.0;
+	w->rubikGL.angleStepX = 0.0;
+	w->rubikGL.angleStepY = 0.0;
+	w->rubikGL.angleStepZ = 0.0;
 }
 
 static void
-ExposeRubikGL(Widget renew, XEvent * event, Region region)
+ExposePuzzleGL(Widget renew, XEvent *event, Region region)
 {
 	RubikGLWidget w = (RubikGLWidget) renew;
 
@@ -2014,59 +2056,67 @@ ExposeRubikGL(Widget renew, XEvent * event, Region region)
 
 #ifndef WINVER
 static void
-MoveRubikGLTl(RubikGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLTl(RubikGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, TL,
-		       (int) (event->xkey.state & ControlMask));
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y,
+		TL,
+		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubikGLTop(RubikGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLTop(RubikGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, TOP,
-		       (int) (event->xkey.state & ControlMask));
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y,
+		TOP,
+		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubikGLTr(RubikGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLTr(RubikGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, TR,
-		       (int) (event->xkey.state & ControlMask));
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y,
+		TR,
+		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubikGLLeft(RubikGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLLeft(RubikGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
-		       (int) (event->xkey.state & ControlMask));
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y,
+		LEFT,
+		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubikGLRight(RubikGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLRight(RubikGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
-		       (int) (event->xkey.state & ControlMask));
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y,
+		RIGHT,
+		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubikGLBl(RubikGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLBl(RubikGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, BL,
-		       (int) (event->xkey.state & ControlMask));
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y,
+		BL,
+		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubikGLBottom(RubikGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLBottom(RubikGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
-		       (int) (event->xkey.state & ControlMask));
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y,
+		BOTTOM,
+		(int) (event->xkey.state & ControlMask));
 }
 
 static void
-MoveRubikGLBr(RubikGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLBr(RubikGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveRubikInput((RubikWidget) w, event->xbutton.x, event->xbutton.y, BR,
-		       (int) (event->xkey.state & ControlMask));
+	MovePuzzleInput((RubikWidget) w, event->xbutton.x, event->xbutton.y,
+		BR,
+		(int) (event->xkey.state & ControlMask));
 }
 #endif
 #endif
