@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1994 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -33,15 +33,15 @@
 #include "Rubik.h"
 
 #ifdef WINVER
-extern void DestroyRubik(HBRUSH brush);
-extern void ResizeRubik(RubikWidget w);
-extern void SizeRubik(RubikWidget w);
-extern void InitializeRubik(RubikWidget w, HBRUSH brush);
-extern void ExposeRubik(RubikWidget w);
-extern void IncrementAxisRubik(RubikWidget w, char axis);
-extern void DimRubik(RubikWidget w);
-extern int  MoveRubikDir(RubikWidget w, const int direction);
-extern void SetRubikMove(RubikWidget w, int reason, int face, int position,
+extern void DestroyPuzzle(HBRUSH brush);
+extern void ResizePuzzle(RubikWidget w);
+extern void SizePuzzle(RubikWidget w);
+extern void InitializePuzzle(RubikWidget w, HBRUSH brush);
+extern void ExposePuzzle(RubikWidget w);
+extern void IncrementAxisPuzzle(RubikWidget w, char axis);
+extern void DimPuzzle(RubikWidget w);
+extern int  MovePuzzleDir(RubikWidget w, const int direction);
+extern void setPuzzleMove(RubikWidget w, int reason, int face, int position,
 	int direction, Boolean control, int fast);
 
 #else
@@ -58,9 +58,9 @@ typedef struct _RubikClassRec {
 
 extern RubikClassRec rubikClassRec;
 
-extern void QuitRubik(RubikWidget w, XEvent * event, char **args, int nArgs);
-extern void MoveRubikCw(RubikWidget w, XEvent * event, char **args, int nArgs);
-extern void MoveRubikCcw(RubikWidget w, XEvent * event, char **args,
+extern void QuitPuzzle(RubikWidget w, XEvent *event, char **args, int nArgs);
+extern void MovePuzzleCw(RubikWidget w, XEvent *event, char **args, int nArgs);
+extern void MovePuzzleCcw(RubikWidget w, XEvent *event, char **args,
 	int nArgs);
 #endif
 
@@ -82,8 +82,26 @@ extern void MoveRubikCcw(RubikWidget w, XEvent * event, char **args,
 #define DOUBLE 2
 #define INSTANT 3
 
+typedef struct _MoveRecord {
+#if 0
+	int face, direction, control;
+#endif
+	unsigned short int packed;  /* This makes assumptions on the data. */
+	int position;		    /* Do not make assumptions on this one. */
+} MoveRecord;
+
+typedef struct _MoveStack {
+	MoveRecord move;
+	struct _MoveStack *previous, *next;
+} MoveStack;
+
+typedef struct _RubikStack {
+	MoveStack *currMove, *lastMove, *firstMove;
+	int count;
+} RubikStack;
+
 typedef struct _RubikLoc {
-	char        face, rotation;
+	char face, rotation;
 } RubikLoc;
 
 typedef struct _RubikPart {
@@ -108,7 +126,7 @@ typedef struct _RubikPart {
 	char        userName[81], scoreFile[81];
 	char        faceChar[MAXFACES];
 #else
-	int         menu;
+	int         menu, view;
 	char       *userName, *scoreFile, *font;
 	XFontStruct *fontInfo;
 	char       *faceName[MAXFACES];
@@ -130,126 +148,134 @@ typedef struct _RowNext {
 
 extern RubikLoc *startLoc[MAXFACES];
 
-extern void HideRubik(
+extern void HidePuzzle(
 #ifdef WINVER
 const RubikWidget w
 #else
-RubikWidget w, XEvent * event, char **args, int nArgs
+RubikWidget w, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void SelectRubik(RubikWidget w
+extern void SelectPuzzle(RubikWidget w
 #ifdef WINVER
 , const int x, const int y, const int control
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void ReleaseRubik(RubikWidget w
+extern void ReleasePuzzle(RubikWidget w
 #ifdef WINVER
 , const int x, const int y, const int control
 #else
-, XEvent * event, char **args, int nArgs
-#endif
-);
-extern void PracticeRubik(RubikWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
 #ifndef WINVER
-extern void PracticeRubikMaybe(RubikWidget w
-, XEvent * event, char **args, int nArgs
+extern void PracticePuzzleMaybe(RubikWidget w
+, XEvent *event, char **args, int nArgs
 );
-extern void PracticeRubik2(RubikWidget w
-, XEvent * event, char **args, int nArgs
+extern void PracticePuzzle2(RubikWidget w
+, XEvent *event, char **args, int nArgs
+);
+extern void RandomizePuzzleMaybe(RubikWidget w
+, XEvent *event, char **args, int nArgs
+);
+extern void RandomizePuzzle2(RubikWidget w
+, XEvent *event, char **args, int nArgs
 );
 #endif
-extern void RandomizeRubik(RubikWidget w
+extern void GetPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
+extern void WritePuzzle(RubikWidget w
 #ifndef WINVER
-extern void RandomizeRubikMaybe(RubikWidget w
-, XEvent * event, char **args, int nArgs
-);
-extern void RandomizeRubik2(RubikWidget w
-, XEvent * event, char **args, int nArgs
-);
-#endif
-extern void GetRubik(RubikWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void WriteRubik(RubikWidget w
+extern void ClearPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void ClearRubik(RubikWidget w
+extern void UndoPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void UndoRubik(RubikWidget w
+extern void RedoPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void SolveRubik(RubikWidget w
+extern void RandomizePuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void OrientizeRubik(RubikWidget w
+extern void SolvePuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void DecrementRubik(RubikWidget w
+extern void PracticePuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void IncrementRubik(RubikWidget w
+extern void OrientizePuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void IncrementXRubik(RubikWidget w
+extern void DecrementPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void IncrementYRubik(RubikWidget w
+extern void IncrementPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void IncrementZRubik(RubikWidget w
+extern void IncrementXPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void EnterRubik(RubikWidget w
+extern void IncrementYPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 );
-extern void LeaveRubik(RubikWidget w
+extern void IncrementZPuzzle(RubikWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void ViewPuzzle(RubikWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void EnterPuzzle(RubikWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+);
+extern void LeavePuzzle(RubikWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 );
 
-extern void MoveRubikInput(RubikWidget w,
+extern void MovePuzzleInput(RubikWidget w,
 	int x, int y, int direction, int control);
-extern void SetRubik(RubikWidget w, int reason);
-extern void MoveRubik(RubikWidget w,
+extern void setPuzzle(RubikWidget w, int reason);
+extern void MovePuzzle(RubikWidget w,
 	const int face, const int position, const int direction,
 	const Boolean control, int fast);
-extern void MoveRubikDelay(RubikWidget w,
+extern void MovePuzzleDelay(RubikWidget w,
 	const int face, const int position, const int direction,
 	const Boolean control);
 #if (!defined(WINVER) || (WINVER <= 0x030a))
@@ -260,17 +286,20 @@ extern void ResetPieces(RubikWidget w);
 extern void SolveSomePieces(RubikWidget w);
 extern void DrawAllPieces(const RubikWidget w);
 extern Boolean CheckSolved(const RubikWidget w, Boolean rotation);
-extern void InitMoves(void);
-extern void PutMove(int face, int position, int direction, int control);
-extern void GetMove(int *face, int *position, int *direction, int *control);
-extern int  MadeMoves(void);
-extern void FlushMoves(RubikWidget w);
-extern int  NumMoves(void);
-extern void ScanMoves(FILE * fp, RubikWidget w, int moves);
-extern void PrintMoves(FILE * fp);
-extern void ScanStartPosition(FILE * fp, RubikWidget w);
-extern void PrintStartPosition(FILE * fp, RubikWidget w);
-extern void SetStartPosition(RubikWidget w);
+extern void newMoves(RubikStack *s);
+extern void deleteMoves(RubikStack *s);
+extern void setMove(RubikStack *s,
+	int face, int position, int direction, int control);
+extern void getMove(RubikStack *s,
+	int *face, int *position, int *direction, int *control);
+extern int madeMoves(RubikStack *s);
+extern void flushMoves(RubikWidget w, RubikStack *s, Boolean undo);
+extern int numMoves(RubikStack *s);
+extern void scanMoves(FILE *fp, RubikWidget w, int moves);
+extern void printMoves(FILE *fp, RubikStack *s);
+extern void scanStartPosition(FILE *fp, RubikWidget w);
+extern void printStartPosition(FILE *fp, RubikWidget w);
+extern void setStartPosition(RubikWidget w);
 extern void faceSizes(const RubikWidget w, const int face, int *sizeOfRow, int *sizeOfColumn);
 extern int  sizeFace(const RubikWidget w, const int face);
 extern int  sizeRow(const RubikWidget w, const int face);
