@@ -50,27 +50,27 @@
 #include "RubikP.h"
 
 /* Mappings of Ideal notation to my General nxnxn cube notation */
-#define RotateLeft(w)	MoveRubikPiece(w,2,0,LEFT,TRUE)
-#define RotateRight(w)	MoveRubikPiece(w,2,0,RIGHT,TRUE)
-#define RotateUp(w)	MoveRubikPiece(w,2,0,TOP,TRUE)
-#define RotateDown(w)	MoveRubikPiece(w,2,0,BOTTOM,TRUE)
-#define RotateCw(w)	MoveRubikPiece(w,0,0,RIGHT,TRUE)
-#define RotateCcw(w)	MoveRubikPiece(w,0,0,LEFT,TRUE)
+#define RotateLeft(w)	MovePuzzlePiece(w,2,0,LEFT,TRUE)
+#define RotateRight(w)	MovePuzzlePiece(w,2,0,RIGHT,TRUE)
+#define RotateUp(w)	MovePuzzlePiece(w,2,0,TOP,TRUE)
+#define RotateDown(w)	MovePuzzlePiece(w,2,0,BOTTOM,TRUE)
+#define RotateCw(w)	MovePuzzlePiece(w,0,0,RIGHT,TRUE)
+#define RotateCcw(w)	MovePuzzlePiece(w,0,0,LEFT,TRUE)
 
-#define RotateTopLeft(w)	MoveRubikPiece(w,2,0,LEFT,FALSE)
-#define RotateCenterLeft(w)	MoveRubikPiece(w,2,w->rubik.sizex,LEFT,FALSE)
-#define RotateBottomLeft(w)	MoveRubikPiece(w,2,w->rubik.sizex*(w->rubik.sizex-1),LEFT,FALSE)
-#define RotateTopRight(w)	MoveRubikPiece(w,2,0,RIGHT,FALSE)
-#define RotateCenterRight(w)	MoveRubikPiece(w,2,w->rubik.sizex,RIGHT,FALSE)
-#define RotateBottomRight(w)	MoveRubikPiece(w,2,w->rubik.sizex*(w->rubik.sizex-1),RIGHT,FALSE)
-#define RotateLeftUp(w)		MoveRubikPiece(w,2,0,TOP,FALSE)
-#define RotateCenterUp(w)	MoveRubikPiece(w,2,1,TOP,FALSE)
-#define RotateRightUp(w)	MoveRubikPiece(w,2,w->rubik.sizex-1,TOP,FALSE)
-#define RotateLeftDown(w)	MoveRubikPiece(w,2,0,BOTTOM,FALSE)
-#define RotateCenterDown(w)	MoveRubikPiece(w,2,1,BOTTOM,FALSE)
-#define RotateRightDown(w)	MoveRubikPiece(w,2,w->rubik.sizex-1,BOTTOM,FALSE)
-#define RotateFrontCw(w)	MoveRubikPiece(w,0,w->rubik.sizex*(w->rubik.sizex-1),RIGHT,FALSE)
-#define RotateFrontCcw(w)	MoveRubikPiece(w,0,w->rubik.sizex*(w->rubik.sizex-1),LEFT,FALSE)
+#define RotateTopLeft(w)	MovePuzzlePiece(w,2,0,LEFT,FALSE)
+#define RotateCenterLeft(w)	MovePuzzlePiece(w,2,w->rubik.sizex,LEFT,FALSE)
+#define RotateBottomLeft(w)	MovePuzzlePiece(w,2,w->rubik.sizex*(w->rubik.sizex-1),LEFT,FALSE)
+#define RotateTopRight(w)	MovePuzzlePiece(w,2,0,RIGHT,FALSE)
+#define RotateCenterRight(w)	MovePuzzlePiece(w,2,w->rubik.sizex,RIGHT,FALSE)
+#define RotateBottomRight(w)	MovePuzzlePiece(w,2,w->rubik.sizex*(w->rubik.sizex-1),RIGHT,FALSE)
+#define RotateLeftUp(w)		MovePuzzlePiece(w,2,0,TOP,FALSE)
+#define RotateCenterUp(w)	MovePuzzlePiece(w,2,1,TOP,FALSE)
+#define RotateRightUp(w)	MovePuzzlePiece(w,2,w->rubik.sizex-1,TOP,FALSE)
+#define RotateLeftDown(w)	MovePuzzlePiece(w,2,0,BOTTOM,FALSE)
+#define RotateCenterDown(w)	MovePuzzlePiece(w,2,1,BOTTOM,FALSE)
+#define RotateRightDown(w)	MovePuzzlePiece(w,2,w->rubik.sizex-1,BOTTOM,FALSE)
+#define RotateFrontCw(w)	MovePuzzlePiece(w,0,w->rubik.sizex*(w->rubik.sizex-1),RIGHT,FALSE)
+#define RotateFrontCcw(w)	MovePuzzlePiece(w,0,w->rubik.sizex*(w->rubik.sizex-1),LEFT,FALSE)
 
 #define BLUE 0
 #define WHITE 1
@@ -149,7 +149,7 @@ ProcessEvents(RubikWidget w)
 #endif
 
 static void
-MoveRubikPiece(RubikWidget w,
+MovePuzzlePiece(RubikWidget w,
 	const int face, const int position, const int direction,
 	const int control)
 {
@@ -159,7 +159,7 @@ MoveRubikPiece(RubikWidget w,
 	if (SolvingFlag && AbortSolvingFlag)
 		longjmp(solve_env, 1);
 #endif
-	MoveRubikDelay(w, face, position, direction, control);
+	MovePuzzleDelay(w, face, position, direction, control);
 }
 
 static int
@@ -1333,7 +1333,7 @@ SolveCenters(RubikWidget w)
 void
 SolveSomePieces(RubikWidget w)
 {
-	SetRubik(w, RUBIK_RESET);
+	setPuzzle(w, PUZZLE_RESET);
 	if (SolvingFlag)
 		return;
 #ifdef JMP
@@ -1363,5 +1363,5 @@ SolveSomePieces(RubikWidget w)
 #endif
 	SolvingFlag = False;
 	w->rubik.cheat = True; /* Assume the worst. */
-	SetRubik(w, RUBIK_COMPUTED);
+	setPuzzle(w, PUZZLE_COMPUTED);
 }
