@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1996 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1996 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -29,23 +29,24 @@
 #define IDM_WRITE	101
 #define IDM_EXIT	102
 #define IDM_HIDE	103
-#define IDM_CLEAR	200
-#define IDM_UNDO	201
-#define IDM_RANDOMIZE	202
-#define IDM_PRACTICE	203
-#define IDM_SOLVE	204
-#define IDM_ORIENT	205
-#define IDM_DIM	206
-#define IDM_TOP	300
+#define IDM_UNDO	200
+#define IDM_REDO	201
+#define IDM_CLEAR	202
+#define IDM_RANDOMIZE	203
+#define IDM_PRACTICE	204
+#define IDM_SOLVE	205
+#define IDM_ORIENTIZE	206
+#define IDM_DIM		207
+#define IDM_TOP		300
 #define IDM_RIGHT	301
 #define IDM_BOTTOM	302
 #define IDM_LEFT	303
-#define IDM_CW	305
-#define IDM_CCW	307
-#define IDM_TR	308
-#define IDM_BR	309
-#define IDM_BL	310
-#define IDM_TL	311
+#define IDM_CW		305
+#define IDM_CCW		307
+#define IDM_TR		308
+#define IDM_BR		309
+#define IDM_BL		310
+#define IDM_TL		311
 #define IDM_CONTROL_TOP	340
 #define IDM_CONTROL_RIGHT	341
 #define IDM_CONTROL_BOTTOM	342
@@ -56,8 +57,8 @@
 #define IDM_CONTROL_BR	349
 #define IDM_CONTROL_BL	350
 #define IDM_CONTROL_TL	351
-#define IDM_DEC	400
-#define IDM_INC	401
+#define IDM_DEC		400
+#define IDM_INC		401
 #define IDM_DECX	402
 #define IDM_INCX	403
 #define IDM_DECY	404
