@@ -94,7 +94,7 @@ static RubikRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Rubik Version 7.2.1\n"
+"Rubik Version 7.2.4\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -106,7 +106,7 @@ static const char optionsHelp[] = {
 "[-display [{host}]:[{vs}]] [-[no]mono] [-[no]{reverse|rv}]\n"
 "[-{foreground|fg} {color}] [-{background|bg} {color}]\n"
 "[-face{0|1|2|3|4|5} {color}] [-{border|bd} {color}]\n"
-"[-delay msecs] [-{font|fn} {fontname}]\n"
+"[-delay msecs] [-{font|fn} {fontname}] [-view {int}]\n"
 "[-size{x|y|z} {int}] [-[no]orient] [-[no]practice]\n"
 "[-userName {string}] [-scoreFile {filename}] [-scores]\n"
 "[-version]\n"
@@ -147,10 +147,32 @@ static const char featuresHelp[] = {
 "the record should say \"practice\").  This is good for "
 "learning moves and experimenting.\n"
 "\n"
-"Click \"mouse-right\" button, or press \"R\" or \"r\" "
+"Click \"mouse-right\" button, or press \"Z\" or \"z\" "
 "keys, to randomize the puzzle (this must be done first\n"
 "to set a new record).\n"
 "\n"
+"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
+"\n"
+"Press \"W\" or \"w\" keys to save (write) a puzzle.\n"
+"\n"
+"Press \"U\" or \"u\" keys to undo a move.\n"
+"\n"
+"Press \"R\" or \"r\" keys to redo a move.\n"
+"\n"
+"Press \"C\" or \"c\" keys to clear the puzzle.\n"
+"\n"
+"Press \"S\" or \"s\" keys to start auto-solver.  Only works "
+"on 1x1x1, 2x2x2, and 3x3x3 cubes.\n"
+"\n"
+"Press \"O\" or \"o\" keys to toggle  the orient mode.  One "
+"has to  orient the faces in orient mode, besides\n"
+"getting all  the faces to be the  same color.  To do this "
+"one  has to get the  lines to be  oriented in\n"
+"the same direction,  this only matters  with center "
+"\"cubies\",  if at all (i.e.  those \"cubies\" not on a\n"
+"corner or edge).  This does add complexity so there are "
+"2 sets of records.\n"
+"\n"
 "Press \"I\" or \"i\" keys to increase the number of \"cubies\".\n"
 "\n"
 "Press \"D\" or \"d\" keys to decrease the number of \"cubies\".\n"
@@ -167,32 +189,13 @@ static const char featuresHelp[] = {
 "Press \"Y\" key to decrease the number of \"cubies\" along "
 "the y axis.\n"
 "\n"
-"Press \"z\" key to increase the number of \"cubies\" along "
+"Press \"f\" key to increase the number of \"cubies\" along "
 "the z axis.\n"
 "\n"
-"Press \"Z\" key to decrease the number of \"cubies\" along "
+"Press \"F\" key to decrease the number of \"cubies\" along "
 "the z axis.\n"
 "\n"
-"Press \"O\" or \"o\" keys to toggle  the orient mode.  One "
-"has to  orient the faces in orient mode, besides\n"
-"getting all  the faces to be the  same color.  To do this "
-"one  has to get the  lines to be  oriented in\n"
-"the same direction,  this only matters  with center "
-"\"cubies\",  if at all (i.e.  those \"cubies\" not on a\n"
-"corner or edge).  This does add complexity so there are "
-"2 sets of records.\n"
-"\n"
-"Press \"S\" or \"s\" keys to start auto-solver.  Only works "
-"on 1x1x1, 2x2x2, and 3x3x3 cubes (3x3x3 cube in\n"
-"non-orient mode).\n"
-"\n"
-"Press \"U\" or \"u\" keys to undo a move.\n"
-"\n"
-"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
-"\n"
-"Press \"W\" or \"w\" keys to save a puzzle.\n"
-"\n"
-"Press \"C\" or \"c\" keys to clear a puzzle.\n"
+"Press \"V\" or \"v\" keys to change the view of the cube.\n"
 "\n"
 "Press \"Esc\" key to hide program.\n"
 "\n"
@@ -262,6 +265,7 @@ static char titleDsp[TITLELEN] = "";
 #endif
 static char scoreFileName[FILENAMELEN] = SCOREFILE;
 static char fileName[FILENAMELEN];
+static Boolean randomized = False;
 #ifdef WINVER
 #define MAXPROGNAME 80
 static char progDsp[MAXPROGNAME] = TITLE;
@@ -283,7 +287,7 @@ static Widget shell;
 static Widget shellGL;
 #endif
 #endif
-static Pixmap rubikIcon = None;
+static Pixmap icon = None;
 static Widget topLevel, rubik2d, rubik3d;
 #ifdef HAVE_OPENGL
 static Widget rubikGL;
@@ -551,7 +555,7 @@ MakeEquivalent(String *userName, String *scoreFile,
 		exit(0);
 	}
 	XtVaSetValues(rubik2d,
-		XtNdirection, RUBIK_IGNORE,
+		XtNdirection, PUZZLE_IGNORE,
 		XtNstart, False, NULL);
 	XtVaSetValues(rubik3d,
 		XtNuserName, *userName,
@@ -562,7 +566,7 @@ MakeEquivalent(String *userName, String *scoreFile,
 		XtNmono, mono,
 		XtNreverse, reverse,
 		XtNdelay, *delay,
-		XtNdirection, RUBIK_IGNORE,
+		XtNdirection, PUZZLE_IGNORE,
 		XtNpractice, *practice,
 		XtNstart, False,
 		XtNforeground, foreground,
@@ -584,7 +588,7 @@ MakeEquivalent(String *userName, String *scoreFile,
 		XtNmono, mono,
 		XtNreverse, reverse,
 		XtNdelay, *delay,
-		XtNdirection, RUBIK_IGNORE,
+		XtNdirection, PUZZLE_IGNORE,
 		XtNpractice, *practice,
 		XtNstart, False,
 		XtNforeground, foreground,
@@ -679,7 +683,7 @@ void
 #ifdef WINVER
 	int dim;
 
-	InitializeRubik(w, brush);
+	InitializePuzzle(w, brush);
 
 	sizex = w->rubik.sizex;
 	sizey = w->rubik.sizey;
@@ -723,7 +727,7 @@ void
 #ifdef WINVER
 		(void) strncpy(userNameDsp, userName, USERNAMELEN);
 #else
-		char * login = getlogin();
+		char *login = getlogin();
 
 		if (login == NULL) {
 			(void) strcpy(userNameDsp, "");
@@ -756,10 +760,10 @@ void
 
 #ifdef WINVER
 void
-SetRubik(RubikWidget w, int reason)
+setPuzzle(RubikWidget w, int reason)
 #else
 static void
-CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
+CallbackPuzzle(Widget w, caddr_t clientData, rubikCallbackStruct *callData)
 #endif
 {
 	int sizex, sizey, sizez;
@@ -837,7 +841,7 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-		case RUBIK_HIDE:
+		case PUZZLE_HIDE:
 #ifdef WINVER
 			ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -857,24 +861,22 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 #endif
 			break;
 #ifndef WINVER
-		case RUBIK_PRACTICE_QUERY:
+		case PUZZLE_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(practiceDialog);
 #else
-			XtVaSetValues(rubik2d,
-				XtNmenu, 8, NULL); /* menu choice */
+			XtVaSetValues(rubik2d, XtNmenu, MENU_PRACTICE, NULL);
 #endif
 			break;
-		case RUBIK_RANDOMIZE_QUERY:
+		case PUZZLE_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(randomizeDialog);
 #else
-			XtVaSetValues(rubik2d,
-				XtNmenu, 5, NULL); /* menu choice */
+			XtVaSetValues(rubik2d, XtNmenu, MENU_RANDOMIZE, NULL);
 #endif
 			break;
 #endif
-		case RUBIK_SOLVE_MESSAGE:
+		case PUZZLE_SOLVE_MESSAGE:
 #ifdef WINVER
 			(void) MessageBox(w->core.hWnd, solveHelp,
 				"Auto-Solve", MB_OK);
@@ -886,7 +888,7 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case RUBIK_RESTORE:
+		case PUZZLE_RESTORE:
 			if (practice) {
 				(void) strncpy(recordDsp, "practice",
 					MESSAGELEN);
@@ -895,43 +897,50 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 #endif
 			}
 			movesDsp = 0;
+			randomized = False;
 #ifndef WINVER
 #ifdef HAVE_OPENGL
 			XtVaSetValues(otherw2,
-				XtNdirection, RUBIK_RESTORE, NULL);
+				XtNdirection, PUZZLE_RESTORE, NULL);
 #endif
 			XtVaSetValues(otherw1,
-				XtNdirection, RUBIK_RESTORE, NULL);
+				XtNdirection, PUZZLE_RESTORE, NULL);
 			XtVaSetValues(w,
-				XtNdirection, RUBIK_RESTORE, NULL);
+				XtNdirection, PUZZLE_RESTORE, NULL);
 #endif
 			break;
-		case RUBIK_CLEAR:
+		case PUZZLE_CLEAR:
 			movesDsp = 0;
+			randomized = False;
 #ifndef WINVER
 #ifdef HAVE_OPENGL
 			XtVaSetValues(otherw2,
-				XtNdirection, RUBIK_CLEAR, NULL);
+				XtNdirection, PUZZLE_CLEAR, NULL);
 #endif
 			XtVaSetValues(otherw1,
-				XtNdirection, RUBIK_CLEAR, NULL);
+				XtNdirection, PUZZLE_CLEAR, NULL);
 			XtVaSetValues(w,
-				XtNdirection, RUBIK_CLEAR, NULL);
+				XtNdirection, PUZZLE_CLEAR, NULL);
 #endif
 			break;
-		case RUBIK_RESET:
+		case PUZZLE_RESET:
 			movesDsp = 0;
+			randomized = False;
 			break;
-		case RUBIK_ILLEGAL:
-			if (practice || start)
+		case PUZZLE_AMBIGUOUS:
+			(void) strncpy(messageDsp, "Ambiguous move",
+				MESSAGELEN);
+			break;
+		case PUZZLE_ILLEGAL:
+			if (practice || randomized)
 				(void) strncpy(messageDsp, "Illegal move",
 					MESSAGELEN);
 			else
-				(void) strncpy(messageDsp, "Randomize to start",
-					MESSAGELEN);
+				(void) strncpy(messageDsp,
+					"Randomize to start", MESSAGELEN);
 			break;
 #ifndef WINVER
-		case RUBIK_MOVED:
+		case PUZZLE_MOVED:
 			if (callData->control) {
 #ifdef HAVE_OPENGL
 				XtVaSetValues(otherw2,
@@ -965,10 +974,11 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 					XtNdirection, callData->direction,
 					XtNcontrol, callData->control,
 					XtNfast, callData->fast, NULL);
+				SET_STARTED(w, True);
 			}
 			break;
 #endif
-		case RUBIK_SOLVED:
+		case PUZZLE_SOLVED:
 			if (practice)
 				movesDsp = 0;
 			else if (cheat)
@@ -981,18 +991,18 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 			else
 				(void) strncpy(messageDsp, "Solved!",
 					MESSAGELEN);
-#ifdef WINVER
-			SET_STARTED(w, False);
-#else
 			SET_STARTED(w, False);
+#ifndef WINVER
 #ifdef HAVE_OPENGL
 			SET_STARTED(otherw2, False);
 #endif
 			SET_STARTED(otherw1, False);
 #endif
+			randomized = False;
 			break;
-		case RUBIK_PRACTICE:
+		case PUZZLE_PRACTICE:
 			movesDsp = 0;
+			randomized = False;
 			practice = !practice;
 			if (!practice)
 				(void) strncpy(messageDsp, "Randomize to start",
@@ -1018,8 +1028,9 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case RUBIK_RANDOMIZE:
+		case PUZZLE_RANDOMIZE:
 			movesDsp = 0;
+			randomized = True;
 #ifdef WINVER
 			w->rubik.practice = False;
 			w->rubik.started = False;
@@ -1037,7 +1048,20 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 				XtNstart, False, NULL);
 #endif
 			break;
-		case RUBIK_ORIENT:
+#ifdef HAVE_OPENGL
+		case PUZZLE_VIEW:
+			{
+				int view;
+
+				XtVaGetValues(rubikGL,
+					XtNview, &view, NULL);
+				view = (view + 1) % (2 * MAXVIEWS);
+				XtVaSetValues(rubikGL,
+					XtNview, view, NULL);
+			}
+			break;
+#endif
+		case PUZZLE_ORIENTIZE:
 			movesDsp = 0;
 			orient = !orient;
 			PrintRecord(sizex, sizey, sizez, orient, practice);
@@ -1054,7 +1078,7 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case RUBIK_DECX:
+		case PUZZLE_DECX:
 			movesDsp = 0;
 			sizex--;
 			PrintRecord(sizex, sizey, sizez, orient, practice);
@@ -1073,7 +1097,7 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case RUBIK_INCX:
+		case PUZZLE_INCX:
 			movesDsp = 0;
 			sizex++;
 			PrintRecord(sizex, sizey, sizez, orient, practice);
@@ -1092,7 +1116,7 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case RUBIK_DECY:
+		case PUZZLE_DECY:
 			movesDsp = 0;
 			sizey--;
 			PrintRecord(sizex, sizey, sizez, orient, practice);
@@ -1111,7 +1135,7 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case RUBIK_INCY:
+		case PUZZLE_INCY:
 			movesDsp = 0;
 			sizey++;
 			PrintRecord(sizex, sizey, sizez, orient, practice);
@@ -1130,7 +1154,7 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case RUBIK_DECZ:
+		case PUZZLE_DECZ:
 			movesDsp = 0;
 			sizez--;
 			PrintRecord(sizex, sizey, sizez, orient, practice);
@@ -1149,7 +1173,7 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case RUBIK_INCZ:
+		case PUZZLE_INCZ:
 			movesDsp = 0;
 			sizez++;
 			PrintRecord(sizex, sizey, sizez, orient, practice);
@@ -1168,7 +1192,7 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case RUBIK_COMPUTED:
+		case PUZZLE_COMPUTED:
 #ifdef WINVER
 			SET_STARTED(w, False);
 #else
@@ -1180,7 +1204,7 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 #endif
 			break;
 #ifndef WINVER
-		case RUBIK_UNDO:
+		case PUZZLE_UNDO:
 			if (callData->control) {
 
 #ifdef HAVE_OPENGL
@@ -1218,8 +1242,47 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 			}
 			break;
 #endif
+#ifndef WINVER
+		case PUZZLE_REDO:
+			if (callData->control) {
+
+#ifdef HAVE_OPENGL
+				XtVaSetValues(otherw2,
+					XtNface, callData->face,
+					XtNpos, callData->position,
+					XtNdirection, callData->direction,
+					XtNcontrol, callData->control,
+					XtNfast, callData->fast,
+					XtNstart, True, NULL);
+#endif
+				XtVaSetValues(otherw1,
+					XtNface, callData->face,
+					XtNpos, callData->position,
+					XtNdirection, callData->direction,
+					XtNcontrol, callData->control,
+					XtNfast, callData->fast,
+					XtNstart, True, NULL);
+			} else {
+				movesDsp++;
+#ifdef HAVE_OPENGL
+				XtVaSetValues(otherw2,
+					XtNface, callData->face,
+					XtNpos, callData->position,
+					XtNdirection, callData->direction,
+					XtNcontrol, callData->control,
+					XtNfast, callData->fast, NULL);
+#endif
+				XtVaSetValues(otherw1,
+					XtNface, callData->face,
+					XtNpos, callData->position,
+					XtNdirection, callData->direction,
+					XtNcontrol, callData->control,
+					XtNfast, callData->fast, NULL);
+			}
+			break;
+#endif
 #ifdef WINVER
-		case RUBIK_DIM:
+		case PUZZLE_DIM:
 			dim++;
 #ifdef HAVE_OPENGL
 			if (dim > 4)
@@ -1248,8 +1311,8 @@ CallbackRubik(Widget w, caddr_t clientData, rubikCallbackStruct * callData)
 
 #ifdef WINVER
 void
-SetRubikMove(RubikWidget w, int reason, int face, int position, int direction,
-	Boolean control, int fast)
+setPuzzleMove(RubikWidget w, int reason, int face, int position, int direction,
+		Boolean control, int fast)
 {
 	int sizex, sizey, sizez;
 	/* Boolean orient, practice, cheat; */
@@ -1266,18 +1329,24 @@ SetRubikMove(RubikWidget w, int reason, int face, int position, int direction,
 #endif
 	dim = w->rubik.dim;
 	switch (reason) {
-		case RUBIK_MOVED:
+		case PUZZLE_MOVED:
 			if (!control) {
 				movesDsp++;
 				SET_STARTED(w, True);
 			}
 			break;
-		case RUBIK_UNDO:
+		case PUZZLE_UNDO:
 			if (!control) {
 				movesDsp--;
 				SET_STARTED(w, True);
 			}
 			break;
+		case PUZZLE_REDO:
+			if (!control) {
+				movesDsp++;
+				SET_STARTED(w, True);
+			}
+			break;
 	}
 	PrintStatus(messageDsp, movesDsp, dim, sizex, sizey, sizez);
 }
@@ -1306,15 +1375,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	if (GetFocus()) {
 		if (!widget.rubik.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			EnterRubik(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			EnterPuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	} else {
 		if (widget.rubik.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			LeaveRubik(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			LeavePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	}
@@ -1323,42 +1394,47 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			Initialize(&widget, brush);
 			break;
 		case WM_DESTROY:
-			DestroyRubik(brush);
+			DestroyPuzzle(brush);
 			break;
 		case WM_SIZE:
-			ResizeRubik(&widget);
+			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
 		case WM_PAINT:
 			widget.core.hDC = BeginPaint(hWnd, &paint);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ExposeRubik(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ExposePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 			break;
 		case WM_RBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			RandomizeRubik(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			SelectRubik(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			SelectPuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONUP:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ReleaseRubik(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ReleasePuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 #if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
 		case WM_MOUSEWHEEL:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
 			{
 				int zDelta = ((short) HIWORD(wParam));
 				POINT cursor, origin;
@@ -1367,12 +1443,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				ClientToScreen(hWnd, &origin);
 				(void) GetCursorPos(&cursor);
 				if (zDelta > (WHEEL_DELTA >> 1)) {
-					MoveRubikInput(&widget,
+					MovePuzzleInput(&widget,
 						cursor.x - origin.x, cursor.y - origin.y,
 						TOP,
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
 				} else if (zDelta < -(WHEEL_DELTA >> 1)) {
-					MoveRubikInput(&widget,
+					MovePuzzleInput(&widget,
 						cursor.x - origin.x, cursor.y - origin.y,
 						BOTTOM,
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0);
@@ -1384,55 +1460,65 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		case WM_COMMAND:
 			switch (LOWORD(wParam)) {
 				case IDM_GET:
-					GetRubik(&widget);
-					ResizeRubik(&widget);
+					GetPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_WRITE:
-					WriteRubik(&widget);
+					WritePuzzle(&widget);
 					break;
 				case IDM_EXIT:
-					DestroyRubik(brush);
+					DestroyPuzzle(brush);
 					break;
 				case IDM_HIDE:
-					HideRubik(&widget);
-					break;
-				case IDM_CLEAR:
-					ClearRubik(&widget);
-					SizeRubik(&widget);
-					(void) InvalidateRect(hWnd, NULL, TRUE);
+					HidePuzzle(&widget);
 					break;
 				case IDM_UNDO:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					UndoRubik(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					UndoPuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
+				case IDM_REDO:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RedoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_CLEAR:
+					ClearPuzzle(&widget);
+					SizePuzzle(&widget);
+					(void) InvalidateRect(hWnd, NULL, TRUE);
+					break;
 				case IDM_RANDOMIZE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					RandomizeRubik(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RandomizePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_PRACTICE:
-					PracticeRubik(&widget);
-					SizeRubik(&widget);
+					PracticePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SOLVE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					SolveRubik(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					SolvePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
-				case IDM_ORIENT:
-					OrientizeRubik(&widget);
-					SizeRubik(&widget);
+				case IDM_ORIENTIZE:
+					OrientizePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_DIM:
-					(void) DimRubik(&widget);
-					ResizeRubik(&widget);
+					(void) DimPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_TOP:
@@ -1449,11 +1535,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
-						(void) MoveRubikInput(&widget,
+						(void) MovePuzzleInput(&widget,
 							cursor.x - origin.x, cursor.y - origin.y,
 							(int) LOWORD(wParam) - IDM_TOP,
 							FALSE);
@@ -1473,11 +1560,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
-						(void) MoveRubikInput(&widget,
+						(void) MovePuzzleInput(&widget,
 							cursor.x - origin.x, cursor.y - origin.y,
 							(int) LOWORD(wParam) - IDM_CONTROL_TOP,
 							TRUE);
@@ -1486,43 +1574,43 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 
 					break;
 				case IDM_DEC:
-					DecrementRubik(&widget);
-					SizeRubik(&widget);
+					DecrementPuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_INC:
-					IncrementRubik(&widget);
-					SizeRubik(&widget);
+					IncrementPuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_DECX:
-					IncrementAxisRubik(&widget, 'X');
-					SizeRubik(&widget);
+					IncrementAxisPuzzle(&widget, 'X');
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_INCX:
-					IncrementAxisRubik(&widget, 'x');
-					SizeRubik(&widget);
+					IncrementAxisPuzzle(&widget, 'x');
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_DECY:
-					IncrementAxisRubik(&widget, 'Y');
-					SizeRubik(&widget);
+					IncrementAxisPuzzle(&widget, 'Y');
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_INCY:
-					IncrementAxisRubik(&widget, 'y');
-					SizeRubik(&widget);
+					IncrementAxisPuzzle(&widget, 'y');
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_DECZ:
-					IncrementAxisRubik(&widget, 'Z');
-					SizeRubik(&widget);
+					IncrementAxisPuzzle(&widget, 'Z');
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_INCZ:
-					IncrementAxisRubik(&widget, 'z');
-					SizeRubik(&widget);
+					IncrementAxisPuzzle(&widget, 'z');
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_ABOUT:
@@ -1599,7 +1687,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
 
 #else
 static void
-Usage(char * programName)
+Usage(char *programName)
 {
 	unsigned int i;
 
@@ -1632,6 +1720,11 @@ static XrmOptionDescRec options[] =
 	{(char *) "-face3", (char *) "*rubik.faceColor3", XrmoptionSepArg, NULL},
 	{(char *) "-face4", (char *) "*rubik.faceColor4", XrmoptionSepArg, NULL},
 	{(char *) "-face5", (char *) "*rubik.faceColor5", XrmoptionSepArg, NULL},
+	{(char *) "-delay", (char *) "*rubik.delay", XrmoptionSepArg, NULL},
+	{(char *) "-fn", (char *) "*rubik.font", XrmoptionSepArg, NULL},
+	{(char *) "-font", (char *) "*rubik.font", XrmoptionSepArg, NULL},
+	{(char *) "-view", (char *) "*rubik.view", XrmoptionSepArg, NULL},
+	{(char *) "-sizex", (char *) "*rubik.sizex", XrmoptionSepArg, NULL},
 	{(char *) "-sizex", (char *) "*rubik.sizex", XrmoptionSepArg, NULL},
 	{(char *) "-sizey", (char *) "*rubik.sizey", XrmoptionSepArg, NULL},
 	{(char *) "-sizez", (char *) "*rubik.sizez", XrmoptionSepArg, NULL},
@@ -1639,9 +1732,6 @@ static XrmOptionDescRec options[] =
 	{(char *) "-noorient", (char *) "*rubik.orient", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-practice", (char *) "*rubik.practice", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-nopractice", (char *) "*rubik.practice", XrmoptionNoArg, (char *) "FALSE"},
-	{(char *) "-delay", (char *) "*rubik.delay", XrmoptionSepArg, NULL},
-	{(char *) "-fn", (char *) "*rubik.font", XrmoptionSepArg, NULL},
-	{(char *) "-font", (char *) "*rubik.font", XrmoptionSepArg, NULL},
 	{(char *) "-userName", (char *) "*rubik.userName", XrmoptionSepArg, NULL},
 	{(char *) "-scoreFile", (char *) "*rubik.scoreFile", XrmoptionSepArg, NULL},
 	{(char *) "-scores", (char *) "*rubik.scoreOnly", XrmoptionNoArg, (char *) "TRUE"},
@@ -1650,23 +1740,23 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackRubikPractice(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzlePractice(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(rubik2d, XtNmenu, 8, NULL); /* menu choice */
+		XtVaSetValues(rubik2d, XtNmenu, MENU_PRACTICE, NULL);
 	}
 }
 
 static void
-CallbackRubikRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzleRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(rubik2d, XtNmenu, 5, NULL); /* menu choice */
+		XtVaSetValues(rubik2d, XtNmenu, MENU_RANDOMIZE, NULL);
 	}
 }
 
 static void
-CubesXSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+CubesXSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int sizex = cbs->value, old, sizey, sizez;
 	Boolean orient, practice;
@@ -1696,7 +1786,7 @@ CubesXSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-CubesYSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+CubesYSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int sizey = cbs->value, old, sizex, sizez;
 	Boolean orient, practice;
@@ -1726,7 +1816,7 @@ CubesYSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-CubesZSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+CubesZSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int sizez = cbs->value, old, sizex, sizey;
 	Boolean orient, practice;
@@ -1756,7 +1846,7 @@ CubesZSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 }
 
 static void
-OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	int sizex, sizey, sizez;
 	Boolean orient = cbs->set, practice;
@@ -1783,7 +1873,7 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
 }
 
 static void
-PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	int sizex, sizey, sizez;
 	Boolean orient, practice = cbs->set;
@@ -1816,7 +1906,7 @@ PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cb
 }
 
 static void
-SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MAXSPEED + MINSPEED - cbs->value - 1, oldDelay;
 
@@ -1836,9 +1926,9 @@ fileCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	if (val == 2)
+	if (val == MENU_EXIT)
 		exit(0);
-	XtVaSetValues(rubik2d, XtNmenu, val, NULL);
+	XtVaSetValues(rubik2d, XtNmenu, val, NULL); /* GWQ */
 }
 
 static void
@@ -1846,7 +1936,7 @@ playCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	XtVaSetValues(rubik2d, XtNmenu, val + 3, NULL); /* GWQ */
+	XtVaSetValues(rubik2d, XtNmenu, val + MENU_EXIT + 1, NULL);
 }
 
 static Widget
@@ -1944,8 +2034,9 @@ main(int argc, char **argv)
 	Widget movesRowCol, cubesRowCol, switchRowCol, messageRowCol;
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
-	XmString clearString, undoString, randomizeString, solveString;
-	XmString orientString, practiceString;
+	XmString undoString, redoString;
+	XmString clearString, randomizeString, practiceString, solveString;
+	XmString orientString, incrementString, decrementString, viewString;
 #endif
 
 	progDsp = argv[0];
@@ -1954,7 +2045,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#if HAVE_XPM
+#ifdef HAVE_XPM
 	{
 		XpmAttributes xpmAttributes;
 		XpmColorSymbol transparentColor[1] = {{NULL,
@@ -1969,18 +2060,18 @@ main(int argc, char **argv)
 		transparentColor[0].pixel = bg;
 		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) rubik_xpm, &rubikIcon, NULL,
+			(char **) rubik_xpm, &icon, NULL,
 			&xpmAttributes);
 	}
-	if (rubikIcon == (Pixmap) NULL)
+	if (icon == (Pixmap) NULL)
 #endif
-		rubikIcon = XCreateBitmapFromData(XtDisplay(topLevel),
+		icon = XCreateBitmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
 			(char *) rubik_bits,
 			rubik_width, rubik_height);
 #ifdef HAVE_MOTIF
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, rubikIcon,
+		XtNiconPixmap, icon,
 		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
@@ -2007,26 +2098,38 @@ main(int argc, char **argv)
 	XmStringFree(getString);
 	XmStringFree(writeString);
 	XmStringFree(quitString);
-	clearString = XmStringCreateSimple((char *) "Clear");
 	undoString = XmStringCreateSimple((char *) "Undo");
-	randomizeString = XmStringCreateSimple((char *) "Randomize");
+	redoString = XmStringCreateSimple((char *) "Redo");
+	clearString = XmStringCreateSimple((char *) "Clear");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
+	practiceString = XmStringCreateSimple((char *) "Practice");
 	solveString = XmStringCreateSimple((char *) "Solve");
 	orientString = XmStringCreateSimple((char *) "Orientize");
-	practiceString = XmStringCreateSimple((char *) "Practice");
+	incrementString = XmStringCreateSimple((char *) "Increment");
+        decrementString = XmStringCreateSimple((char *) "Decrement");
+	viewString = XmStringCreateSimple((char *) "View");
 	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
-		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
+		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
 		XmVaPUSHBUTTON, orientString, 'O', NULL, NULL,
-		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
+		XmVaPUSHBUTTON, incrementString, 'I', NULL, NULL,
+		XmVaPUSHBUTTON, decrementString, 'D', NULL, NULL,
+		XmVaPUSHBUTTON, viewString, 'V', NULL, NULL,
 		NULL);
-	XmStringFree(clearString);
 	XmStringFree(undoString);
+	XmStringFree(redoString);
+	XmStringFree(clearString);
 	XmStringFree(randomizeString);
+	XmStringFree(practiceString);
 	XmStringFree(solveString);
 	XmStringFree(orientString);
-	XmStringFree(practiceString);
+	XmStringFree(incrementString);
+	XmStringFree(decrementString);
+	XmStringFree(viewString);
 	pullDownMenu = XmCreatePulldownMenu(menuBar,
 		(char *) "helpPullDown", NULL, 0);
 	widget = XtVaCreateManagedWidget("Help",
@@ -2034,29 +2137,27 @@ main(int argc, char **argv)
 		XmNsubMenuId, pullDownMenu,
 		XmNmnemonic, 'H', NULL);
 	XtVaSetValues(menuBar, XmNmenuHelpWidget, widget, NULL);
-	widget = XtVaCreateManagedWidget("About",
-		xmPushButtonGadgetClass, pullDownMenu,
-		XmNmnemonic, 'A', NULL);
-	XtAddCallback(widget, XmNactivateCallback, helpCB, (char *) 0);
 	widget = XtVaCreateManagedWidget("Description",
 		xmPushButtonGadgetClass, pullDownMenu,
 		XmNmnemonic, 'D', NULL);
-	XtAddCallback(widget, XmNactivateCallback, helpCB, (char *) 1);
+	XtAddCallback(widget, XmNactivateCallback, helpCB, (char *) 0);
 	widget = XtVaCreateManagedWidget("FeaTures",
 		xmPushButtonGadgetClass, pullDownMenu,
 		XmNmnemonic, 'F', NULL);
+	XtAddCallback(widget, XmNactivateCallback, helpCB, (char *) 1);
+	widget = XtVaCreateManagedWidget("Options",
+		xmPushButtonGadgetClass, pullDownMenu,
+		XmNmnemonic, 'O', NULL);
 	XtAddCallback(widget, XmNactivateCallback, helpCB, (char *) 2);
 	widget = XtVaCreateManagedWidget("References",
 		xmPushButtonGadgetClass, pullDownMenu,
 		XmNmnemonic, 'R', NULL);
 	XtAddCallback(widget, XmNactivateCallback, helpCB, (char *) 3);
-	widget = XtVaCreateManagedWidget("Options",
+	widget = XtVaCreateManagedWidget("About",
 		xmPushButtonGadgetClass, pullDownMenu,
-		XmNmnemonic, 'O', NULL);
+		XmNmnemonic, 'A', NULL);
 	XtAddCallback(widget, XmNactivateCallback, helpCB, (char *) 4);
 	XtManageChild(menuBar);
-	aboutDialog = createHelp(menuBar, (char *) aboutHelp,
-		(char *) "About");
 	descriptionDialog = createHelp(menuBar, (char *) descriptionHelp,
 		(char *) "Description");
 	featuresDialog = createHelp(menuBar, (char *) featuresHelp,
@@ -2065,16 +2166,18 @@ main(int argc, char **argv)
 		(char *) "Options");
 	referencesDialog = createHelp(menuBar, (char *) referencesHelp,
 		(char *) "References");
+	aboutDialog = createHelp(menuBar, (char *) aboutHelp,
+		(char *) "About");
 	solveDialog = createHelp(menuBar, (char *) solveHelp,
 		(char *) "Solve");
 	practiceDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to toggle the practice mode?",
 		(char *) "Practice Query",
-		(XtCallbackProc) CallbackRubikPractice);
+		(XtCallbackProc) CallbackPuzzlePractice);
 	randomizeDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to randomize?",
 		(char *) "Randomize Query",
-		(XtCallbackProc) CallbackRubikRandomize);
+		(XtCallbackProc) CallbackPuzzleRandomize);
 	mainPanel = XtCreateManagedWidget("mainPanel",
 		xmPanedWindowWidgetClass, menuBarPanel,
 		NULL, 0);
@@ -2206,7 +2309,7 @@ main(int argc, char **argv)
 	XtVaSetValues(rubik2d,
 		XtNheight, 200, NULL);
 	XtAddCallback(rubik2d, XtNselectCallback,
-		(XtCallbackProc) CallbackRubik, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	rubik3d = XtCreateManagedWidget("rubik",
 		rubik3dWidgetClass, mainPanel, NULL, 0);
 	XtVaSetValues(rubik3d,
@@ -2219,24 +2322,24 @@ main(int argc, char **argv)
 #endif
 #else
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, rubikIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 	shell = XtCreateApplicationShell(argv[0],
 		topLevelShellWidgetClass, NULL, 0);
 	XtVaSetValues(shell,
-		XtNiconPixmap, rubikIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 #ifdef HAVE_OPENGL
 	shellGL = XtCreateApplicationShell(argv[0],
 		topLevelShellWidgetClass, NULL, 0);
 	XtVaSetValues(shellGL,
-		XtNiconPixmap, rubikIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 #endif
 	rubik2d = XtCreateManagedWidget("rubik",
 		rubik2dWidgetClass, topLevel, NULL, 0);
 	XtAddCallback(rubik2d, XtNselectCallback,
-		(XtCallbackProc) CallbackRubik, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	rubik3d = XtCreateManagedWidget("rubik",
 		rubik3dWidgetClass, shell, NULL, 0);
 #ifdef HAVE_OPENGL
@@ -2245,10 +2348,10 @@ main(int argc, char **argv)
 #endif
 #endif
 	XtAddCallback(rubik3d, XtNselectCallback,
-		(XtCallbackProc) CallbackRubik, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 #ifdef HAVE_OPENGL
 	XtAddCallback(rubikGL, XtNselectCallback,
-		(XtCallbackProc) CallbackRubik, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 #endif
 	Initialize();
 	XtRealizeWidget(topLevel);
