@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1994 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -46,12 +46,12 @@
 
 static const char *faceColorString[MAXFACES] =
 {
-        "255 0 0",
-        "0 0 255",
-        "255 255 255",
-        "0 255 0",
-        "255 192 203",
-        "255 255 0"
+	"255 0 0",
+	"0 0 255",
+	"255 255 255",
+	"0 255 0",
+	"255 192 203",
+	"255 255 0"
 };
 
 static const char faceColorChar[MAXFACES] =
@@ -66,9 +66,9 @@ static const char faceColorChar[MAXFACES] =
 #endif
 #endif
 
-static Boolean SetValuesSkewb(Widget current, Widget request, Widget renew);
-static void DestroySkewb(Widget old);
-static void InitializeSkewb(Widget request, Widget renew);
+static Boolean SetValuesPuzzle(Widget current, Widget request, Widget renew);
+static void DestroyPuzzle(Widget old);
+static void InitializePuzzle(Widget request, Widget renew);
 
 SkewbClassRec skewbClassRec =
 {
@@ -79,7 +79,7 @@ SkewbClassRec skewbClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeSkewb,	/* initialize */
+		(XtInitProc) InitializePuzzle,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
 		NULL,		/* actions */
@@ -91,10 +91,10 @@ SkewbClassRec skewbClassRec =
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
-		(XtWidgetProc) DestroySkewb,	/* destroy */
+		(XtWidgetProc) DestroyPuzzle,	/* destroy */
 		NULL,		/* resize */
 		NULL,		/* expose */
-		(XtSetValuesFunc) SetValuesSkewb,	/* set values */
+		(XtSetValuesFunc) SetValuesPuzzle,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
@@ -114,7 +114,7 @@ SkewbClassRec skewbClassRec =
 WidgetClass skewbWidgetClass = (WidgetClass) & skewbClassRec;
 
 void
-SetSkewb(SkewbWidget w, int reason)
+setPuzzle(SkewbWidget w, int reason)
 {
 	skewbCallbackStruct cb;
 
@@ -123,7 +123,7 @@ SetSkewb(SkewbWidget w, int reason)
 }
 
 static void
-SetSkewbMove(SkewbWidget w, int reason, int face, int position, int direction)
+setPuzzleMove(SkewbWidget w, int reason, int face, int position, int direction)
 {
 	skewbCallbackStruct cb;
 
@@ -147,8 +147,8 @@ loadFont(SkewbWidget w)
 		XUnloadFont(XtDisplay(w), w->skewb.fontInfo->fid);
 		XFreeFont(XtDisplay(w), w->skewb.fontInfo);
 	}
-	if ((w->skewb.fontInfo = XLoadQueryFont(display,
-			w->skewb.font)) == NULL) {
+	if (w->skewb.font && (w->skewb.fontInfo =
+			XLoadQueryFont(display, w->skewb.font)) == NULL) {
 		(void) sprintf(buf,
 			"Can not open %s font.\nAttempting %s font as alternate\n",
 			w->skewb.font, altfontname);
@@ -178,77 +178,157 @@ loadFont(SkewbWidget w)
 #define LOGFILE "skewb.log"
 #endif
 
+static int facesToDirection[MAXFACES][MAXFACES] =
+{
+	{-1, BOTTOM, BOTTOM, BOTTOM, -1, TOP},
+	{RIGHT, -1, RIGHT, -1, RIGHT, RIGHT},
+	{TOP, LEFT, -1, RIGHT, BOTTOM, -1},
+	{LEFT, -1, LEFT, -1, LEFT, LEFT},
+	{-1, TOP, TOP, TOP, -1, BOTTOM},
+	{BOTTOM, RIGHT, -1, LEFT, TOP, -1}
+};
+static int facesToPosition[MAXFACES][MAXFACES][MAXORIENT] =
+{
+	{
+		{-1, -1, -1, -1},
+		{-1, -1, 0, 3},
+		{-1, 0, 3, -1},
+		{0, 3, -1, -1},
+		{-1, -1, -1, -1},
+		{1, -1, -1, 2}
+	},
+	{
+		{2, -1, -1, 3},
+		{-1, -1, -1, -1},
+		{3, 2, -1, -1},
+		{-1, -1, -1, -1},
+		{-1, 3, 2, -1},
+		{-1, -1, 3, 2}
+	},
+	{
+		{1, -1, -1, 2},
+		{-1, -1, 1, 0},
+		{-1, -1, -1, -1},
+		{3, 2, -1, -1},
+		{-1, 0, 3, -1},
+		{-1, -1, -1, -1}
+	},
+	{
+		{0, -1, -1, 1},
+		{-1, -1, -1, -1},
+		{-1, -1, 1, 0},
+		{-1, -1, -1, -1},
+		{-1, 1, 0, -1},
+		{1, 0, -1, -1}
+	},
+	{
+		{-1, -1, -1, -1},
+		{-1, -1, 2, 1},
+		{1, -1, -1, 2},
+		{2, 1, -1, -1},
+		{-1, -1, -1, -1},
+		{-1, 0, 3, -1}
+	},
+	{
+		{-1, 0, 3, -1},
+		{-1, -1, 3, 2},
+		{-1, -1, -1, -1},
+		{1, 0, -1, -1},
+		{1, -1, -1, 2},
+		{-1, -1, -1, -1}
+	}
+};
 static SkewbLoc slideNextRow[MAXFACES][MAXORIENT][MAXORIENT / 2] =
 {
 	{
 		{
 			{2, CW},
-			{1, HALF}},
+			{1, HALF}
+		},
 		{
 			{5, CCW},
-			{1, STRT}},
+			{1, STRT}
+		},
 		{
 			{3, STRT},
-			{5, CW}},
+			{5, CW}
+		},
 		{
 			{3, HALF},
-			{2, CCW}}
+			{2, CCW}
+		}
 	},
 	{
 		{
 			{4, STRT},
-			{5, CW}},
+			{5, CW}
+		},
 		{
 			{0, STRT},
-			{5, CCW}},
+			{5, CCW}
+		},
 		{
 			{2, CCW},
-			{0, HALF}},
+			{0, HALF}
+		},
 		{
 			{2, CW},
-			{4, HALF}}
+			{4, HALF}
+		}
 	},
 	{
 		{
 			{4, CW},
-			{1, CCW}},
+			{1, CCW}
+		},
 		{
 			{0, CCW},
-			{1, CW}},
+			{1, CW}
+		},
 		{
 			{3, CCW},
-			{0, CW}},
+			{0, CW}
+		},
 		{
 			{3, CW},
-			{4, CCW}}
+			{4, CCW}
+		}
 	},
 	{
 		{
 			{4, HALF},
-			{2, CCW}},
+			{2, CCW}
+		},
 		{
 			{0, HALF},
-			{2, CW}},
+			{2, CW}
+		},
 		{
 			{5, CW},
-			{0, STRT}},
+			{0, STRT}
+		},
 		{
 			{5, CCW},
-			{4, STRT}}
+			{4, STRT}
+		}
 	},
 	{
 		{
 			{5, CW},
-			{1, STRT}},
+			{1, STRT}
+		},
 		{
 			{2, CCW},
-			{1, HALF}},
+			{1, HALF}
+		},
 		{
 			{3, HALF},
-			{2, CW}},
+			{2, CW}
+		},
 		{
 			{3, STRT},
-			{5, CCW}}
+			{5, CCW}
+		}
 	},
 	{
 		{
@@ -256,13 +336,16 @@ static SkewbLoc slideNextRow[MAXFACES][MAXORIENT][MAXORIENT / 2] =
 			{1, CW}},
 		{
 			{4, CCW},
-			{1, CCW}},
+			{1, CCW}
+		},
 		{
 			{3, CW},
-			{4, CW}},
+			{4, CW}
+		},
 		{
 			{3, CCW},
-			{0, CCW}}
+			{0, CCW}
+		}
 	}
 };
 static SkewbLoc minToMaj[MAXFACES][MAXORIENT] =
@@ -271,32 +354,38 @@ static SkewbLoc minToMaj[MAXFACES][MAXORIENT] =
 		{3, CW},
 		{2, STRT},
 		{1, CCW},
-		{5, STRT}},
+		{5, STRT}
+	},
 	{
 		{2, STRT},
 		{4, CCW},
 		{5, HALF},
-		{0, CW}},
+		{0, CW}
+	},
 	{
 		{3, STRT},
 		{4, STRT},
 		{1, STRT},
-		{0, STRT}},
+		{0, STRT}
+	},
 	{
 		{5, HALF},
 		{4, CW},
 		{2, STRT},
-		{0, CCW}},
+		{0, CCW}
+	},
 	{
 		{3, CCW},
 		{5, STRT},
 		{1, CW},
-		{2, STRT}},
+		{2, STRT}
+	},
 	{
 		{3, HALF},
 		{0, STRT},
 		{1, HALF},
-		{4, STRT}}
+		{4, STRT}
+	}
 };
 
 static SkewbLoc slideNextFace[MAXFACES][MAXORIENT] =
@@ -305,35 +394,41 @@ static SkewbLoc slideNextFace[MAXFACES][MAXORIENT] =
 		{5, STRT},
 		{3, CW},
 		{2, STRT},
-		{1, CCW}},
+		{1, CCW}
+	},
 	{
 		{0, CW},
 		{2, STRT},
 		{4, CCW},
-		{5, HALF}},
+		{5, HALF}
+	},
 	{
 		{0, STRT},
 		{3, STRT},
 		{4, STRT},
-		{1, STRT}},
+		{1, STRT}
+	},
 	{
 		{0, CCW},
 		{5, HALF},
 		{4, CW},
-		{2, STRT}},
+		{2, STRT}
+	},
 	{
 		{2, STRT},
 		{3, CCW},
 		{5, STRT},
-		{1, CW}},
+		{1, CW}
+	},
 	{
 		{4, STRT},
 		{3, HALF},
 		{0, STRT},
-		{1, HALF}}
+		{1, HALF}
+	}
 };
 
-static int  faceToRotate[MAXFACES][MAXORIENT] =
+static int faceToRotate[MAXFACES][MAXORIENT] =
 {
 	{3, 2, 1, 5},
 	{2, 4, 5, 0},
@@ -350,140 +445,166 @@ static SkewbLocPos orthToDiag[MAXFACES][MAXORIENT][MAXORIENT] =
 			{3, 0, 1},
 			{5, 1, 0},
 			{3, 0, 3},
-			{5, 1, 2}},
+			{5, 1, 2}
+		},
 		{
 			{3, 3, 0},
 			{2, 0, 1},
 			{3, 3, 2},
-			{2, 0, 3}},
+			{2, 0, 3}
+		},
 		{
 			{1, 0, 3},
 			{2, 3, 0},
 			{1, 0, 1},
-			{2, 3, 2}},
+			{2, 3, 2}
+		},
 		{
 			{1, 3, 2},
 			{5, 2, 1},
 			{1, 3, 0},
-			{5, 2, 3}}
+			{5, 2, 3}
+		}
 	},
 	{
 		{
 			{2, 3, 0},
 			{0, 2, 1},
 			{2, 3, 2},
-			{0, 2, 3}},
+			{0, 2, 3}
+		},
 		{
 			{2, 2, 3},
 			{4, 3, 0},
 			{2, 2, 1},
-			{4, 3, 2}},
+			{4, 3, 2}
+		},
 		{
 			{5, 3, 2},
 			{4, 2, 3},
 			{5, 3, 0},
-			{4, 2, 1}},
+			{4, 2, 1}
+		},
 		{
 			{5, 2, 1},
 			{0, 3, 2},
 			{5, 2, 3},
-			{0, 3, 0}}
+			{0, 3, 0}
+		}
 	},
 	{
 		{
 			{3, 3, 0},
 			{0, 1, 0},
 			{3, 3, 2},
-			{0, 1, 2}},
+			{0, 1, 2}
+		},
 		{
 			{3, 2, 3},
 			{4, 0, 1},
 			{3, 2, 1},
-			{4, 0, 3}},
+			{4, 0, 3}
+		},
 		{
 			{1, 1, 0},
 			{4, 3, 0},
 			{1, 1, 2},
-			{4, 3, 2}},
+			{4, 3, 2}
+		},
 		{
 			{1, 0, 3},
 			{0, 2, 1},
 			{1, 0, 1},
-			{0, 2, 3}}
+			{0, 2, 3}
+		}
 	},
 	{
 		{
 			{5, 1, 2},
 			{0, 0, 3},
 			{5, 1, 0},
-			{0, 0, 1}},
+			{0, 0, 1}
+		},
 		{
 			{5, 0, 1},
 			{4, 1, 2},
 			{5, 0, 3},
-			{4, 1, 0}},
+			{4, 1, 0}
+		},
 		{
 			{2, 1, 0},
 			{4, 0, 1},
 			{2, 1, 2},
-			{4, 0, 3}},
+			{4, 0, 3}
+		},
 		{
 			{2, 0, 3},
 			{0, 1, 0},
 			{2, 0, 1},
-			{0, 1, 2}}
+			{0, 1, 2}
+		}
 	},
 	{
 		{
 			{3, 2, 3},
 			{2, 1, 0},
 			{3, 2, 1},
-			{2, 1, 2}},
+			{2, 1, 2}
+		},
 		{
 			{3, 1, 2},
 			{5, 0, 1},
 			{3, 1, 0},
-			{5, 0, 3}},
+			{5, 0, 3}
+		},
 		{
 			{1, 2, 1},
 			{5, 3, 0},
 			{1, 2, 3},
-			{5, 3, 2}},
+			{5, 3, 2}
+		},
 		{
 			{1, 1, 0},
 			{2, 2, 1},
 			{1, 1, 2},
-			{2, 2, 3}}
+			{2, 2, 3}
+		}
 	},
 	{
 		{
 			{3, 1, 2},
 			{4, 1, 0},
 			{3, 1, 0},
-			{4, 1, 2}},
+			{4, 1, 2}
+		},
 		{
 			{3, 0, 1},
 			{0, 0, 1},
 			{3, 0, 3},
-			{0, 0, 3}},
+			{0, 0, 3}
+		},
 		{
 			{1, 3, 2},
 			{0, 3, 0},
 			{1, 3, 0},
-			{0, 3, 2}},
+			{0, 3, 2}
+		},
 		{
 			{1, 2, 1},
 			{4, 2, 1},
 			{1, 2, 3},
-			{4, 2, 3}}
+			{4, 2, 3}
+		}
 	}
 };
 
+static SkewbStack undo, redo;
+
 Boolean
 CheckSolved(SkewbWidget w)
 {
-	int         face, position;
-	SkewbLoc    test;
+	int face, position;
+	SkewbLoc test;
 
 	for (face = 0; face < MAXFACES; face++)
 		for (position = 0; position < MAXCUBES; position++) {
@@ -491,8 +612,8 @@ CheckSolved(SkewbWidget w)
 				test.face = w->skewb.cubeLoc[face][position].face;
 				test.rotation = w->skewb.cubeLoc[face][position].rotation;
 			} else if (test.face !=		/*face */
-				   w->skewb.cubeLoc[face][position].face ||
-				   (w->skewb.orient && test.rotation !=		/*STRT - MAXORIENT */
+				  w->skewb.cubeLoc[face][position].face ||
+				  (w->skewb.orient && test.rotation !=		/*STRT - MAXORIENT */
 				  w->skewb.cubeLoc[face][position].rotation))
 				return False;
 		}
@@ -504,12 +625,12 @@ CheckSolved(SkewbWidget w)
 void
 PrintCube(SkewbWidget w)
 {
-	int         face, position;
+	int face, position;
 
 	for (face = 0; face < MAXFACES; face++) {
 		for (position = 0; position < MAXCUBES; position++)
 			(void) printf("%d %d  ", w->skewb.cubeLoc[face][position].face,
-				  w->skewb.cubeLoc[face][position].rotation);
+				w->skewb.cubeLoc[face][position].rotation);
 		(void) printf("\n");
 	}
 	(void) printf("\n");
@@ -538,7 +659,7 @@ DrawTriangle(SkewbWidget w, int face, int position, int offset)
 void
 DrawAllPieces(SkewbWidget w)
 {
-	int         face, position;
+	int face, position;
 
 #ifdef HAVE_OPENGL
 	if (w->skewb.dim == 4) {
@@ -568,13 +689,13 @@ DrawFrame(const SkewbWidget w, const Boolean focus)
 static void
 MoveNoPieces(SkewbWidget w)
 {
-	SetSkewb(w, SKEWB_ILLEGAL);
+	setPuzzle(w, PUZZLE_ILLEGAL);
 }
 
 static void
 RotateFace(SkewbWidget w, int face, int direction)
 {
-	int         corner;
+	int corner;
 
 	/* Read Face */
 	for (corner = 0; corner < MAXORIENT; corner++)
@@ -601,7 +722,7 @@ RotateFace(SkewbWidget w, int face, int direction)
 static void
 ReadFace(SkewbWidget w, int face, int h)
 {
-	int         position;
+	int position;
 
 	for (position = 0; position < MAXCUBES; position++)
 		w->skewb.rowLoc[h][position] = w->skewb.cubeLoc[face][position];
@@ -610,7 +731,7 @@ ReadFace(SkewbWidget w, int face, int h)
 static void
 WriteFace(SkewbWidget w, int face, int rotate, int h)
 {
-	int         corner, newCorner;
+	int corner, newCorner;
 
 	for (corner = 0; corner < MAXORIENT; corner++) {
 		newCorner = (corner + rotate) % MAXORIENT;
@@ -623,12 +744,17 @@ WriteFace(SkewbWidget w, int face, int rotate, int h)
 	w->skewb.cubeLoc[face][MAXORIENT].rotation =
 		(w->skewb.cubeLoc[face][MAXORIENT].rotation + rotate) % MAXORIENT;
 	DrawDiamond(w, face, FALSE);
+#ifdef HAVE_OPENGL
+	if (w->skewb.dim == 4) {
+		DrawAllPiecesGL((SkewbGLWidget) w);
+	}
+#endif
 }
 
 static void
 ReadDiagonal(SkewbWidget w, int face, int corner, int orient, int size)
 {
-	int         g;
+	int g;
 
 	if (size == MINOR)
 		w->skewb.minorLoc[orient] = w->skewb.cubeLoc[face][corner];
@@ -644,7 +770,7 @@ ReadDiagonal(SkewbWidget w, int face, int corner, int orient, int size)
 static void
 RotateDiagonal(SkewbWidget w, int rotate, int orient, int size)
 {
-	int         g;
+	int g;
 
 	if (size == MINOR)
 		w->skewb.minorLoc[orient].rotation =
@@ -658,7 +784,7 @@ RotateDiagonal(SkewbWidget w, int rotate, int orient, int size)
 static void
 WriteDiagonal(SkewbWidget w, int face, int corner, int orient, int size)
 {
-	int         g, h;
+	int g, h;
 
 	if (size == MINOR) {
 		w->skewb.cubeLoc[face][corner] = w->skewb.minorLoc[orient];
@@ -684,7 +810,7 @@ WriteDiagonal(SkewbWidget w, int face, int corner, int orient, int size)
 static void
 MovePieces(SkewbWidget w, int face, int position, int direction)
 {
-	int         newFace, newDirection, newCorner, k, size, rotate;
+	int newFace, newDirection, newCorner, k, size, rotate;
 
 	if (direction < 2 * MAXORIENT) {
 		/* position as MAXORIENT is ambiguous */
@@ -732,7 +858,7 @@ MovePieces(SkewbWidget w, int face, int position, int direction)
 static void
 MoveControlCb(SkewbWidget w, int face, int position, int direction)
 {
-	int         newFace, rotate;
+	int newFace, rotate;
 
 	if (direction >= 2 * MAXORIENT) {
 		SkewbLocPos newpos;
@@ -743,24 +869,24 @@ MoveControlCb(SkewbWidget w, int face, int position, int direction)
 		direction = newpos.direction;
 	}
 	MovePieces(w, face, position, direction);
-	SetSkewbMove(w, SKEWB_CONTROL, face, position, direction);
+	setPuzzleMove(w, PUZZLE_CONTROL, face, position, direction);
 	newFace = minToMaj[face][position].face;
 	rotate = minToMaj[face][position].rotation % MAXORIENT;
 	direction = (rotate + direction) % MAXORIENT;
 	position = (position + rotate + 2) % MAXORIENT;
 	MovePieces(w, newFace, position, direction);
-	SetSkewbMove(w, SKEWB_CONTROL, newFace, position, direction);
+	setPuzzleMove(w, PUZZLE_CONTROL, newFace, position, direction);
 }
 
 static void
 MoveAltCb(SkewbWidget w, int face, int position, int direction)
 {
 	MovePieces(w, face, position, direction);
-	SetSkewbMove(w, SKEWB_CONTROL, face, position, direction);
+	setPuzzleMove(w, PUZZLE_CONTROL, face, position, direction);
 }
 
 void
-MoveSkewb(SkewbWidget w, int face, int position, int direction, int control)
+MovePuzzle(SkewbWidget w, int face, int position, int direction, int control)
 {
 	if (control == 2)
 		MoveAltCb(w, face, position, direction);
@@ -775,29 +901,32 @@ MoveSkewb(SkewbWidget w, int face, int position, int direction, int control)
 		if (direction >= 2 * MAXORIENT)
 			newpos = orthToDiag[face][position][direction % MAXORIENT];
 		MovePieces(w, newpos.face, newpos.position, newpos.direction);
-		SetSkewbMove(w, SKEWB_MOVED, newpos.face, newpos.position,
+		setPuzzleMove(w, PUZZLE_MOVED, newpos.face, newpos.position,
 			newpos.direction);
 	}
-	PutMove(face, position, direction, control);
+	setMove(&undo, face, position, direction, control);
+	flushMoves(w, &redo, FALSE);
 }
 
-static      Boolean
+static Boolean
 SelectPieces(SkewbWidget w, int x, int y, int *face, int *position)
 {
 	if (w->skewb.dim == 2)
 		return SelectPieces2D((Skewb2DWidget) w, x, y,
-					   face, position);
+			face, position);
 	else if (w->skewb.dim == 3)
 		return SelectPieces3D((Skewb3DWidget) w, x, y,
-					   face, position);
+			face, position);
 	return False;
 }
 
-static      Boolean
+static int
 CheckMoveDir(int position1, int position2, int *direction)
 {
+	if (position1 == position2)
+		return 2;
 	if (!((position1 - position2 + MAXORIENT) % 2))
-		return False;
+		return 0;
 	switch (position1) {
 		case 0:
 			*direction = (position2 == 1) ? 2 : 3;
@@ -812,13 +941,13 @@ CheckMoveDir(int position1, int position2, int *direction)
 			*direction = (position2 == 0) ? 1 : 2;
 			break;
 		default:
-			return False;
+			return 0;
 	}
 	*direction += 2 * MAXORIENT;
-	return True;
+	return 1;
 }
 
-static      Boolean
+static Boolean
 NarrowSelection(SkewbWidget w, int *face, int *position, int *direction)
 {
 	if (w->skewb.dim == 2)
@@ -828,7 +957,7 @@ NarrowSelection(SkewbWidget w, int *face, int *position, int *direction)
 	return False;
 }
 
-static      Boolean
+static Boolean
 PositionPieces(SkewbWidget w, int x, int y, int *face, int *position, int *direction)
 {
 	if (!SelectPieces(w, x, y, face, position))
@@ -837,9 +966,10 @@ PositionPieces(SkewbWidget w, int x, int y, int *face, int *position, int *direc
 }
 
 void
-MoveSkewbInput(SkewbWidget w, int x, int y, int direction, int control, int alt)
+MovePuzzleInput(SkewbWidget w, int x, int y, int direction, int control,
+		int alt)
 {
-	int         face, position;
+	int face, position;
 
 	if (!w->skewb.practice && !control && !alt && CheckSolved(w)) {
 		MoveNoPieces(w);
@@ -853,31 +983,34 @@ MoveSkewbInput(SkewbWidget w, int x, int y, int direction, int control, int alt)
 		control = (control) ? 1 : 0;
 	if (position == MAXORIENT)
 		return;
-	MoveSkewb(w, face, position, direction, control);
+	MovePuzzle(w, face, position, direction, control);
 	if (!control && CheckSolved(w)) {
-		SetSkewb(w, SKEWB_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
 static void
 ResetPieces(SkewbWidget w)
 {
-	int         face, position;
+	int face, position;
 
 	for (face = 0; face < MAXFACES; face++)
 		for (position = 0; position < MAXCUBES; position++) {
 			w->skewb.cubeLoc[face][position].face = face;
 			w->skewb.cubeLoc[face][position].rotation = STRT - MAXORIENT;
 		}
-	FlushMoves(w);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	w->skewb.started = False;
+	w->skewb.currentFace = PUZZLE_IGNORE;
+	w->skewb.currentDirection = PUZZLE_IGNORE;
 }
 
 static void
 GetPieces(SkewbWidget w)
 {
-	FILE       *fp;
-	int         c, orient, practice, moves;
+	FILE *fp;
+	int c, orient, practice, moves;
 	char *buf1 = NULL, *buf2 = NULL;
 	char *fname, *lname, *name;
 
@@ -916,28 +1049,29 @@ GetPieces(SkewbWidget w)
 		}
 #endif
 	}
-	FlushMoves(w);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &orient);
 	if (w->skewb.orient != (Boolean) orient) {
-		SetSkewb(w, SKEWB_ORIENT);
+		setPuzzle(w, PUZZLE_ORIENTIZE);
 	}
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &practice);
 	if (w->skewb.practice != (Boolean) practice) {
-		SetSkewb(w, SKEWB_PRACTICE);
+		setPuzzle(w, PUZZLE_PRACTICE);
 	}
 #ifdef WINVER
 	ResetPieces(w);
 #endif
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
-	ScanStartPosition(fp, w);
-	SetSkewb(w, SKEWB_RESTORE);
+	scanStartPosition(fp, w);
+	setPuzzle(w, PUZZLE_RESTORE);
 #ifdef WINVER
-	SetStartPosition(w);
+	setStartPosition(w);
 #endif
-	ScanMoves(fp, w, moves);
+	scanMoves(fp, w, moves);
 	(void) fclose(fp);
 	(void) printf("%s: orient %d, practice %d, moves %d.\n",
 		name, orient, practice, moves);
@@ -949,7 +1083,7 @@ GetPieces(SkewbWidget w)
 static void
 WritePieces(SkewbWidget w)
 {
-	FILE       *fp;
+	FILE *fp;
 	char *buf1 = NULL, *buf2 = NULL;
 	char *fname, *lname, *name;
 
@@ -992,35 +1126,33 @@ WritePieces(SkewbWidget w)
 		(w->skewb.orient) ? 1 : 0);
 	(void) fprintf(fp, "practice%c %d\n", SYMBOL,
 		(w->skewb.practice) ? 1 : 0);
-	(void) fprintf(fp, "moves%c %d\n", SYMBOL, NumMoves());
-	PrintStartPosition(fp, w);
-	PrintMoves(fp);
+	(void) fprintf(fp, "moves%c %d\n", SYMBOL,
+		 numMoves(&undo));
+	printStartPosition(fp, w);
+	printMoves(fp, &undo);
 	(void) fclose(fp);
 	(void) printf("Saved to %s.\n", name);
 	free(lname);
 	free(fname);
 }
 
-static void
-ClearPieces(SkewbWidget w)
-{
-	SetSkewb(w, SKEWB_CLEAR);
-}
-
 static void
 UndoPieces(SkewbWidget w)
 {
-	if (MadeMoves()) {
-		int         face, position, direction, control;
+	if (madeMoves(&undo)) {
+		int face, position, direction, control;
 
-		GetMove(&face, &position, &direction, &control);
+		getMove(&undo,
+			&face, &position, &direction, &control);
+		setMove(&redo,
+			face, position, direction, control);
 		if (direction < 2 * MAXORIENT) {
 			direction = (direction < MAXORIENT) ?
 				(direction + MAXORIENT / 2) % MAXORIENT :
 				3 * MAXORIENT - direction;
 		} else {
 			direction = (direction + MAXORIENT / 2) % MAXORIENT +
-			       2 * MAXORIENT;
+				2 * MAXORIENT;
 		}
 		if (control == 2)
 			MoveAltCb(w, face, position, direction);
@@ -1036,30 +1168,68 @@ UndoPieces(SkewbWidget w)
 				direction = newpos.direction;
 			}
 			MovePieces(w, face, position, direction);
-			SetSkewbMove(w, SKEWB_UNDO, face, position, direction);
+			setPuzzleMove(w, PUZZLE_UNDO, face, position, direction);
 			if (CheckSolved(w)) {
-				SetSkewb(w, SKEWB_SOLVED);
+				setPuzzle(w, PUZZLE_SOLVED);
 			}
 		}
 	}
 }
 
+static void
+RedoPieces(SkewbWidget w)
+{
+	if (madeMoves(&redo)) {
+		int face, position, direction, control;
+
+		getMove(&redo,
+			&face, &position, &direction, &control);
+		setMove(&undo,
+			face, position, direction, control);
+		if (control == 2)
+			MoveAltCb(w, face, position, direction);
+		else if (control)
+			MoveControlCb(w, face, position, direction);
+		else {
+			if (direction >= 2 * MAXORIENT) {
+				SkewbLocPos newpos;
+
+				newpos = orthToDiag[face][position][direction % MAXORIENT];
+				face = newpos.face;
+				position = newpos.position;
+				direction = newpos.direction;
+			}
+			MovePieces(w, face, position, direction);
+			setPuzzleMove(w, PUZZLE_REDO, face, position, direction);
+			if (CheckSolved(w)) {
+				setPuzzle(w, PUZZLE_SOLVED);
+			}
+		}
+	}
+}
+
+static void
+ClearPieces(SkewbWidget w)
+{
+	setPuzzle(w, PUZZLE_CLEAR);
+}
+
 static void
 PracticePieces(SkewbWidget w)
 {
-	SetSkewb(w, SKEWB_PRACTICE);
+	setPuzzle(w, PUZZLE_PRACTICE);
 }
 
 static void
 RandomizePieces(SkewbWidget w)
 {
-	int         face, position, direction;
-	int         big = MAXCUBES * 3 + NRAND(2);
+	int face, position, direction;
+	int big = MAXCUBES * 3 + NRAND(2);
 
 	w->skewb.cheat = False;
 	if (w->skewb.practice)
 		PracticePieces(w);
-	SetSkewb(w, SKEWB_RESET);
+	setPuzzle(w, PUZZLE_RESET);
 
 #ifdef DEBUG
 	big = 3;
@@ -1069,12 +1239,13 @@ RandomizePieces(SkewbWidget w)
 		face = NRAND(MAXFACES);
 		position = NRAND(MAXORIENT);
 		direction = ((NRAND(2)) ? position + 1 : position + 3) % MAXORIENT;
-		MoveSkewb(w, face, position, direction, FALSE);
+		MovePuzzle(w, face, position, direction, FALSE);
 	}
-	FlushMoves(w);
-	SetSkewb(w, SKEWB_RANDOMIZE);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
+	setPuzzle(w, PUZZLE_RANDOMIZE);
 	if (CheckSolved(w)) {
-		SetSkewb(w, SKEWB_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
@@ -1084,25 +1255,31 @@ SolvePieces(SkewbWidget w)
 	if (CheckSolved(w))
 		return;
 	{
-		SetSkewb(w, SKEWB_SOLVE_MESSAGE);
+		setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
 	}
 }
 
 static void
 OrientizePieces(SkewbWidget w)
 {
-	SetSkewb(w, SKEWB_ORIENT);
+	setPuzzle(w, PUZZLE_ORIENTIZE);
+}
+
+static void
+ViewPieces(SkewbWidget w)
+{
+	setPuzzle(w, PUZZLE_VIEW);
 }
 
 #ifdef WINVER
 static void
-SetValuesSkewb(SkewbWidget w)
+SetValuesPuzzle(SkewbWidget w)
 {
 	struct tagColor {
-		int         red, green, blue;
+		int red, green, blue;
 	} color;
-	char        szBuf[80], buf[20], charbuf[2];
-	int         face;
+	char szBuf[80], buf[20], charbuf[2];
+	int face;
 
 	w->skewb.orient = (BOOL) GetPrivateProfileInt(SECTION, "orient",
 		DEFAULTORIENT, INIFILE);
@@ -1158,44 +1335,46 @@ SetValuesSkewb(SkewbWidget w)
 }
 
 void
-DestroySkewb(HBRUSH brush)
+DestroyPuzzle(HBRUSH brush)
 {
+	deleteMoves(&undo);
+	deleteMoves(&redo);
 	(void) DeleteObject(brush);
 	PostQuitMessage(0);
 }
 
 void
-ResizeSkewb(SkewbWidget w)
+ResizePuzzle(SkewbWidget w)
 {
 	if (w->skewb.dim == 2)
-		ResizeSkewb2D((Skewb2DWidget) w);
+		ResizePuzzle2D((Skewb2DWidget) w);
 	else if (w->skewb.dim == 3)
-		ResizeSkewb3D((Skewb3DWidget) w);
+		ResizePuzzle3D((Skewb3DWidget) w);
 }
 
 void
-SizeSkewb(SkewbWidget w)
+SizePuzzle(SkewbWidget w)
 {
 	ResetPieces(w);
-	ResizeSkewb(w);
+	ResizePuzzle(w);
 }
 
 void
-ExposeSkewb(SkewbWidget w)
+ExposePuzzle(SkewbWidget w)
 {
 	if (w->skewb.dim == 2)
-		ExposeSkewb2D((Skewb2DWidget) w);
+		ExposePuzzle2D((Skewb2DWidget) w);
 	else if (w->skewb.dim == 3)
-		ExposeSkewb3D((Skewb3DWidget) w);
+		ExposePuzzle3D((Skewb3DWidget) w);
 }
 
 #else
 static void
 GetColor(SkewbWidget w, int face)
 {
-	XGCValues   values;
-	XtGCMask    valueMask;
-	XColor      colorCell, rgb;
+	XGCValues values;
+	XtGCMask valueMask;
+	XColor colorCell, rgb;
 
 	valueMask = GCForeground | GCBackground;
 	if (w->skewb.reverse) {
@@ -1203,9 +1382,15 @@ GetColor(SkewbWidget w, int face)
 	} else {
 		values.background = w->skewb.background;
 	}
-	if (!w->skewb.mono) {
-		if (XAllocNamedColor(XtDisplay(w),
-				  DefaultColormapOfScreen(XtScreen(w)),
+	if (!w->skewb.mono || w->skewb.dim == 4) {
+		if (!w->skewb.faceName[face]) {
+			char *buf1;
+
+			intCat(&buf1, "Color name null for face ", face);
+			DISPLAY_WARNING(buf1);
+			free(buf1);
+		} else if (XAllocNamedColor(XtDisplay(w),
+				DefaultColormapOfScreen(XtScreen(w)),
 				w->skewb.faceName[face], &colorCell, &rgb)) {
 			values.foreground = w->skewb.faceColor[face] = colorCell.pixel;
 			if (w->skewb.faceGC[face])
@@ -1240,9 +1425,9 @@ GetColor(SkewbWidget w, int face)
 void
 SetAllColors(SkewbWidget w)
 {
-	XGCValues   values;
-	XtGCMask    valueMask;
-	int         face;
+	XGCValues values;
+	XtGCMask valueMask;
+	int face;
 
 	valueMask = GCForeground | GCBackground;
 
@@ -1293,12 +1478,12 @@ SetAllColors(SkewbWidget w)
 			w->skewb.fontInfo->fid);
 }
 
-static      Boolean
-SetValuesSkewb(Widget current, Widget request, Widget renew)
+static Boolean
+SetValuesPuzzle(Widget current, Widget request, Widget renew)
 {
 	SkewbWidget c = (SkewbWidget) current, w = (SkewbWidget) renew;
-	Boolean     redraw = False, setColors = False;
-	int         face;
+	Boolean redraw = False, setColors = False;
+	int face;
 
 	for (face = 0; face < MAXFACES; face++) {
 		if (strcmp(w->skewb.faceName[face], c->skewb.faceName[face])) {
@@ -1327,64 +1512,65 @@ SetValuesSkewb(Widget current, Widget request, Widget renew)
 		redraw = True;
 	}
 	if (w->skewb.menu != -1) {
-		switch (w->skewb.menu) {
-		case 0:
-			w->skewb.menu = -1;
+		int menu = w->skewb.menu;
+
+		w->skewb.menu = -1;
+		switch (menu) {
+		case MENU_GET:
 			GetPieces(w);
 			break;
-		case 1:
-			w->skewb.menu = -1;
+		case MENU_WRITE:
 			WritePieces(w);
 			break;
-		case 3:
-			w->skewb.menu = -1;
-			ClearPieces(w);
-			break;
-		case 4:
-			w->skewb.menu = -1;
+		case MENU_UNDO:
 			UndoPieces(w);
 			break;
-		case 5:
-			w->skewb.menu = -1;
+		case MENU_REDO:
+			RedoPieces(w);
+			break;
+		case MENU_CLEAR:
+			ClearPieces(w);
+			break;
+		case MENU_RANDOMIZE:
 			RandomizePieces(w);
 			break;
-		case 6:
-			w->skewb.menu = -1;
-			SolvePieces(w);
-			break;
-		case 7:
-			w->skewb.menu = -1;
-			OrientizePieces(w);
-			break;
-		case 8:
-			w->skewb.menu = -1;
+		case MENU_PRACTICE:
 			PracticePieces(w);
 			break;
+		case MENU_SOLVE:
+			SolvePieces(w);
+			break;
+		case MENU_ORIENTIZE:
+			OrientizePieces(w);
+			break;
+		case MENU_VIEW:
+			ViewPieces(w);
+			break;
 		default:
-			w->skewb.menu = -1;
 			break;
 		}
 	}
-	if (w->skewb.currentDirection == SKEWB_RESTORE) {
-		SetStartPosition(w);
-		w->skewb.currentDirection = SKEWB_IGNORE;
-	} else if (w->skewb.currentDirection == SKEWB_CLEAR) {
+	if (w->skewb.currentDirection == PUZZLE_RESTORE) {
+		setStartPosition(w);
+		w->skewb.currentDirection = PUZZLE_IGNORE;
+	} else if (w->skewb.currentDirection == PUZZLE_CLEAR) {
 		ResetPieces(w);
 		redraw = True;
-		w->skewb.currentDirection = SKEWB_IGNORE;
-	} else if (w->skewb.currentDirection != SKEWB_IGNORE) {
+		w->skewb.currentDirection = PUZZLE_IGNORE;
+	} else if (w->skewb.currentDirection > PUZZLE_IGNORE) {
 		MovePieces(w, w->skewb.currentFace, w->skewb.currentPosition,
-				w->skewb.currentDirection);
-		w->skewb.currentDirection = SKEWB_IGNORE;
+			w->skewb.currentDirection);
+		w->skewb.currentDirection = PUZZLE_IGNORE;
 	}
 	return redraw;
 }
 
 static void
-DestroySkewb(Widget old)
+DestroyPuzzle(Widget old)
 {
 	SkewbWidget w = (SkewbWidget) old;
-	int         face;
+	Display *display = XtDisplay(w);
+	int face;
 
 	for (face = 0; face < MAXFACES; face++)
 		XtReleaseGC(old, w->skewb.faceGC[face]);
@@ -1392,18 +1578,18 @@ DestroySkewb(Widget old)
 	XtReleaseGC(old, w->skewb.frameGC);
 	XtReleaseGC(old, w->skewb.inverseGC);
 	XtRemoveCallbacks(old, XtNselectCallback, w->skewb.select);
-}
-
-void
-QuitSkewb(SkewbWidget w, XEvent * event, char **args, int nArgs)
-{
-	Display *display = XtDisplay(w);
-
 	if (w->skewb.fontInfo) {
 		XUnloadFont(display, w->skewb.fontInfo->fid);
 		XFreeFont(display, w->skewb.fontInfo);
 	}
-	XtCloseDisplay(display);
+	deleteMoves(&undo);
+	deleteMoves(&redo);
+}
+
+void
+QuitPuzzle(SkewbWidget w, XEvent *event, char **args, int nArgs)
+{
+	XtCloseDisplay(XtDisplay(w));
 	exit(0);
 }
 #endif
@@ -1412,7 +1598,7 @@ QuitSkewb(SkewbWidget w, XEvent * event, char **args, int nArgs)
 static
 #endif
 void
-InitializeSkewb(
+InitializePuzzle(
 #ifdef WINVER
 SkewbWidget w, HBRUSH brush
 #else
@@ -1421,7 +1607,7 @@ Widget request, Widget renew
 )
 {
 #ifdef WINVER
-	SetValuesSkewb(w);
+	SetValuesPuzzle(w);
 #else
 	SkewbWidget w = (SkewbWidget) renew;
 	int face;
@@ -1437,7 +1623,8 @@ Widget request, Widget renew
 #endif
 	w->skewb.focus = False;
 	loadFont(w);
-	InitMoves();
+	newMoves(&undo);
+	newMoves(&redo);
 	w->skewb.cheat = False;
 	ResetPieces(w);
 #ifdef WINVER
@@ -1450,127 +1637,132 @@ Widget request, Widget renew
 }
 
 void
-HideSkewb(SkewbWidget w
+HidePuzzle(SkewbWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
-	SetSkewb(w, SKEWB_HIDE);
+	setPuzzle(w, PUZZLE_HIDE);
 }
 
 void
-SelectSkewb(SkewbWidget w
+SelectPuzzle(SkewbWidget w
 #ifdef WINVER
 , const int x, const int y, const int control, const int alt
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
 #ifndef WINVER
-	int  x = event->xbutton.x, y = event->xbutton.y;
+	int x = event->xbutton.x, y = event->xbutton.y;
 	int control = (int) (event->xkey.state & ControlMask);
-	int alt = (int) (event->xkey.state &
-		(Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask));
+	int alt = (int) (event->xkey.state & Mod1Mask);
 #endif
 
-	if (SelectPieces(w, x, y,
-		     &(w->skewb.currentFace), &(w->skewb.currentPosition))) {
+	if (SelectPieces(w, x, y, &(w->skewb.currentFace),
+			&(w->skewb.currentPosition))) {
 		if (w->skewb.currentPosition == MAXORIENT) {
 			if (control || alt || w->skewb.practice || !CheckSolved(w)) {
 				DrawDiamond(w, w->skewb.currentFace, TRUE);
 				FLUSH(w);
 				Sleep(100);
 				DrawDiamond(w, w->skewb.currentFace, FALSE);
-				w->skewb.currentFace = SKEWB_IGNORE;
-				w->skewb.currentDirection = SKEWB_IGNORE;
+				w->skewb.currentFace = PUZZLE_IGNORE;
+				w->skewb.currentDirection = PUZZLE_IGNORE;
 			}
 		} else if (control || alt || w->skewb.practice || !CheckSolved(w)) {
 			DrawTriangle(w, w->skewb.currentFace, w->skewb.currentPosition,
-				     TRUE);
+				TRUE);
 		}
 	} else {
-		w->skewb.currentFace = SKEWB_IGNORE;
-		w->skewb.currentDirection = SKEWB_IGNORE;
+		w->skewb.currentFace = PUZZLE_IGNORE;
+		w->skewb.currentDirection = PUZZLE_IGNORE;
 	}
 }
 
 void
-ReleaseSkewb(SkewbWidget w
+ReleasePuzzle(SkewbWidget w
 #ifdef WINVER
 , const int x, const int y, const int control, const int alt
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
 #ifndef WINVER
-	int  x = event->xbutton.x, y = event->xbutton.y;
+	int x = event->xbutton.x, y = event->xbutton.y;
 	int control = (int) (event->xkey.state & ControlMask);
-	int alt = (int) (event->xkey.state &
-		(Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask));
+	int alt = (int) (event->xkey.state & Mod1Mask);
 #endif
-	int         face, position, count = -1, direction = 0, ctrl = control;
+	int face, position, count = -1, direction = 0, ctrl = control;
 
-	if (w->skewb.currentFace == SKEWB_IGNORE)
+	if (w->skewb.currentFace <= PUZZLE_IGNORE)
 		return;
 	if (w->skewb.currentPosition != MAXORIENT)
 		DrawTriangle(w, w->skewb.currentFace, w->skewb.currentPosition,
 			FALSE);
 	if (!control && !alt && !w->skewb.practice && CheckSolved(w))
 		MoveNoPieces(w);
-	else if (SelectPieces(w, x, y,
-				&face, &position) && position != MAXORIENT &&
-		 position != w->skewb.currentPosition) {
+	else if (SelectPieces(w, x, y, &face, &position)) {
 		if (alt)
 			ctrl = 2;
 		else
 			ctrl = (ctrl) ? 1 : 0;
-		if (face == w->skewb.currentFace)
-			count = CheckMoveDir(w->skewb.currentPosition,
-				position, &direction);
+		if (face == w->skewb.currentFace) {
+			if (position == MAXORIENT)
+				count = -1;
+			else
+				count = CheckMoveDir(w->skewb.currentPosition,
+					position, &direction);
+			/* count == 2 too annoying */
+			if (count == 2)
+				count = -1;
+		} else {
+			direction = facesToDirection[w->skewb.currentFace][face];
+			w->skewb.currentPosition =
+				facesToPosition[w->skewb.currentFace][face][w->skewb.currentPosition];
+			count = face;
+			face = w->skewb.currentFace;
+			w->skewb.currentFace = count;
+			if (w->skewb.currentPosition >= 0)
+				count = 1;
+			else
+				count = 0;
+		}
 		if (count == 1) {
-			MoveSkewb(w, face, w->skewb.currentPosition, direction,
+			MovePuzzle(w, w->skewb.currentFace,
+				w->skewb.currentPosition, direction,
 				ctrl);
 			if (!ctrl && CheckSolved(w)) {
-				SetSkewb(w, SKEWB_SOLVED);
+				setPuzzle(w, PUZZLE_SOLVED);
 			}
 		} else if (count == 0)
 			MoveNoPieces(w);
 	}
-	w->skewb.currentFace = SKEWB_IGNORE;
-	w->skewb.currentDirection = SKEWB_IGNORE;
-}
-
-void
-PracticeSkewb(SkewbWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	PracticePieces(w);
+	w->skewb.currentFace = PUZZLE_IGNORE;
+	w->skewb.currentDirection = PUZZLE_IGNORE;
 }
 
 #ifndef WINVER
 void
-PracticeSkewbMaybe(SkewbWidget w
-, XEvent * event, char **args, int nArgs
+PracticePuzzleMaybe(SkewbWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 	if (!w->skewb.started)
 		PracticePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		SetSkewb(w, SKEWB_PRACTICE_QUERY);
+		setPuzzle(w, PUZZLE_PRACTICE_QUERY);
 	}
 #endif
 }
 
 void
-PracticeSkewb2(SkewbWidget w
-, XEvent * event, char **args, int nArgs
+PracticePuzzle2(SkewbWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 #ifdef HAVE_MOTIF
@@ -1578,36 +1770,24 @@ PracticeSkewb2(SkewbWidget w
 #endif
 		PracticePieces(w);
 }
-#endif
 
 void
-RandomizeSkewb(SkewbWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	RandomizePieces(w);
-}
-
-#ifndef WINVER
-void
-RandomizeSkewbMaybe(SkewbWidget w
-, XEvent * event, char **args, int nArgs
+RandomizePuzzleMaybe(SkewbWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 	if (!w->skewb.started)
 		RandomizePieces(w);
 #ifdef HAVE_MOTIF
 	else {
-		SetSkewb(w, SKEWB_RANDOMIZE_QUERY);
+		setPuzzle(w, PUZZLE_RANDOMIZE_QUERY);
 	}
 #endif
 }
 
 void
-RandomizeSkewb2(SkewbWidget w
-, XEvent * event, char **args, int nArgs
+RandomizePuzzle2(SkewbWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 #ifdef HAVE_MOTIF
@@ -1618,9 +1798,9 @@ RandomizeSkewb2(SkewbWidget w
 #endif
 
 void
-GetSkewb(SkewbWidget w
+GetPuzzle(SkewbWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1628,9 +1808,9 @@ GetSkewb(SkewbWidget w
 }
 
 void
-WriteSkewb(SkewbWidget w
+WritePuzzle(SkewbWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1638,19 +1818,9 @@ WriteSkewb(SkewbWidget w
 }
 
 void
-ClearSkewb(SkewbWidget w
+UndoPuzzle(SkewbWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	ClearPieces(w);
-}
-
-void
-UndoSkewb(SkewbWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1658,9 +1828,39 @@ UndoSkewb(SkewbWidget w
 }
 
 void
-SolveSkewb(SkewbWidget w
+RedoPuzzle(SkewbWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RedoPieces(w);
+}
+
+void
+ClearPuzzle(SkewbWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	ClearPieces(w);
+}
+
+void
+RandomizePuzzle(SkewbWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RandomizePieces(w);
+}
+
+void
+SolvePuzzle(SkewbWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1668,9 +1868,19 @@ SolveSkewb(SkewbWidget w
 }
 
 void
-OrientizeSkewb(SkewbWidget w
+PracticePuzzle(SkewbWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	PracticePieces(w);
+}
+
+void
+OrientizePuzzle(SkewbWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1678,9 +1888,19 @@ OrientizeSkewb(SkewbWidget w
 }
 
 void
-EnterSkewb(SkewbWidget w
+ViewPuzzle(SkewbWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	ViewPieces(w);
+}
+
+void
+EnterPuzzle(SkewbWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1689,9 +1909,9 @@ EnterSkewb(SkewbWidget w
 }
 
 void
-LeaveSkewb(SkewbWidget w
+LeavePuzzle(SkewbWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1701,23 +1921,23 @@ LeaveSkewb(SkewbWidget w
 
 #ifdef WINVER
 void
-DimSkewb(SkewbWidget w)
+DimPuzzle(SkewbWidget w)
 {
-	SetSkewb(w, SKEWB_DIM);
+	setPuzzle(w, PUZZLE_DIM);
 }
 
 #else
 void
-MoveSkewbCcw(SkewbWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleCcw(SkewbWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput(w, event->xbutton.x, event->xbutton.y, CCW,
-		       (int) (event->xbutton.state & ControlMask), FALSE);
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, CCW,
+		(int) (event->xbutton.state & ControlMask), FALSE);
 }
 
 void
-MoveSkewbCw(SkewbWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleCw(SkewbWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput(w, event->xbutton.x, event->xbutton.y, CW,
-		       (int) (event->xkey.state & ControlMask), FALSE);
+	MovePuzzleInput(w, event->xbutton.x, event->xbutton.y, CW,
+		(int) (event->xkey.state & ControlMask), FALSE);
 }
 #endif
