@@ -46,6 +46,7 @@
 #define XtNfaceColor5 ((char *) "faceColor5")
 #define XtNpieceBorder ((char *) "pieceBorder")
 #define XtNdelay ((char *) "delay")
+#define XtNview ((char *) "view")
 #define XtNorient ((char *) "orient")
 #define XtNpractice ((char *) "practice")
 #define XtNuserName ((char *) "userName")
@@ -62,6 +63,7 @@
 #define XtCMono ((char *) "Mono")
 #define XtCReverse ((char *) "Reverse")
 #define XtCDelay ((char *) "Delay")
+#define XtCView ((char *) "View")
 #define XtCOrient ((char *) "Orient")
 #define XtCPractice ((char *) "Practice")
 #define XtCUserName ((char *) "UserName")
@@ -82,28 +84,43 @@ typedef struct {
 } skewbCallbackStruct;
 #endif
 
-#define SKEWB_HIDE (-9)
-#define SKEWB_PRACTICE_QUERY (-8)
-#define SKEWB_RANDOMIZE_QUERY (-7)
-#define SKEWB_SOLVE_MESSAGE (-6)
-#define SKEWB_RESTORE (-5)
-#define SKEWB_CLEAR (-4)
-#define SKEWB_RESET (-3)
-#define SKEWB_ILLEGAL (-1)
-#define SKEWB_IGNORE (-1)
-#define SKEWB_MOVED 0
-#define SKEWB_CONTROL 1
-#define SKEWB_SOLVED 2
-#define SKEWB_PRACTICE 3
-#define SKEWB_RANDOMIZE 4
-#define SKEWB_DEC 5
-#define SKEWB_ORIENT 6
-#define SKEWB_INC 7
-#define SKEWB_COMPUTED 8
-#define SKEWB_UNDO 9
-#define SKEWB_DIM 10
+#define PUZZLE_HIDE (-9)
+#define PUZZLE_PRACTICE_QUERY (-8)
+#define PUZZLE_RANDOMIZE_QUERY (-7)
+#define PUZZLE_SOLVE_MESSAGE (-6)
+#define PUZZLE_RESTORE (-5)
+#define PUZZLE_CLEAR (-4)
+#define PUZZLE_RESET (-3)
+#define PUZZLE_ILLEGAL (-2)
+#define PUZZLE_IGNORE (-1)
+#define PUZZLE_MOVED 0
+#define PUZZLE_CONTROL 1
+#define PUZZLE_SOLVED 2
+#define PUZZLE_PRACTICE 3
+#define PUZZLE_RANDOMIZE 4
+#define PUZZLE_DEC 5
+#define PUZZLE_ORIENTIZE 6
+#define PUZZLE_INC 7
+#define PUZZLE_COMPUTED 8
+#define PUZZLE_UNDO 9
+#define PUZZLE_REDO 10
+#define PUZZLE_DIM 11
+#define PUZZLE_VIEW 12
+
+#define MENU_GET 0
+#define MENU_WRITE 1
+#define MENU_EXIT 2
+#define MENU_UNDO 3
+#define MENU_REDO 4
+#define MENU_CLEAR 5
+#define MENU_RANDOMIZE 6
+#define MENU_PRACTICE 7
+#define MENU_SOLVE 8
+#define MENU_ORIENTIZE 9
+#define MENU_VIEW 10
 
 #define MAXFACES 6
+#define MAXVIEWS 2
 #define DEFAULTORIENT FALSE
 #define DEFAULTPRACTICE FALSE
 #ifdef MONOTEST
