@@ -29,20 +29,28 @@
 #include "Skewb2dP.h"
 
 #ifndef WINVER
-static Boolean SetValuesSkewb2D(Widget current, Widget request, Widget renew);
-static void ResizeSkewb2D(Skewb2DWidget w);
-static void InitializeSkewb2D(Widget request, Widget renew);
-static void ExposeSkewb2D(Widget renew, XEvent * event, Region region);
-static void MoveSkewb2DTl(Skewb2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb2DTop(Skewb2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb2DTr(Skewb2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb2DLeft(Skewb2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb2dRight(Skewb2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb2DBl(Skewb2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb2DBottom(Skewb2DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb2DBr(Skewb2DWidget w, XEvent * event, char **args, int nArgs);
+static Boolean SetValuesPuzzle2D(Widget current, Widget request, Widget renew);
+static void ResizePuzzle2D(Skewb2DWidget w);
+static void InitializePuzzle2D(Widget request, Widget renew);
+static void ExposePuzzle2D(Widget renew, XEvent *event, Region region);
+static void MovePuzzle2DTl(Skewb2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DTop(Skewb2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DTr(Skewb2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DLeft(Skewb2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2dRight(Skewb2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DBl(Skewb2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DBottom(Skewb2DWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzle2DBr(Skewb2DWidget w,
+	XEvent *event, char **args, int nArgs);
 
-static char defaultTranslationsSkewb2D[] =
+static char defaultTranslationsPuzzle2D[] =
 "<KeyPress>q: Quit()\n\
  Ctrl<KeyPress>C: Quit()\n\
  <KeyPress>osfCancel: Hide()\n\
@@ -89,56 +97,60 @@ static char defaultTranslationsSkewb2D[] =
  <KeyPress>R3: MoveBr()\n\
  <Btn1Down>: Select()\n\
  <Btn1Up>: Release()\n\
- <KeyPress>p: Practice()\n\
  <Btn2Down>: PracticeMaybe()\n\
  <Btn2Down>(2+): Practice2()\n\
- <KeyPress>r: Randomize()\n\
  <Btn3Down>: RandomizeMaybe()\n\
  <Btn3Down>(2+): Randomize2()\n\
  <Btn4Down>: MoveTop()\n\
  <Btn5Down>: MoveBottom()\n\
  <KeyPress>g: Get()\n\
  <KeyPress>w: Write()\n\
- <KeyPress>c: Clear()\n\
  <KeyPress>u: Undo()\n\
+ <KeyPress>r: Redo()\n\
+ <KeyPress>c: Clear()\n\
+ <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
+ <KeyPress>p: Practice()\n\
  <KeyPress>o: Orientize()\n\
+ <KeyPress>v: View()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
-static XtActionsRec actionsListSkewb2D[] =
+static XtActionsRec actionsListPuzzle2D[] =
 {
-	{(char *) "Quit", (XtActionProc) QuitSkewb},
-	{(char *) "Hide", (XtActionProc) HideSkewb},
-	{(char *) "MoveCcw", (XtActionProc) MoveSkewbCcw},
-	{(char *) "MoveTl", (XtActionProc) MoveSkewb2DTl},
-	{(char *) "MoveTop", (XtActionProc) MoveSkewb2DTop},
-	{(char *) "MoveTr", (XtActionProc) MoveSkewb2DTr},
-	{(char *) "MoveLeft", (XtActionProc) MoveSkewb2DLeft},
-	{(char *) "MoveCw", (XtActionProc) MoveSkewbCw},
-	{(char *) "MoveRight", (XtActionProc) MoveSkewb2dRight},
-	{(char *) "MoveBl", (XtActionProc) MoveSkewb2DBl},
-	{(char *) "MoveBottom", (XtActionProc) MoveSkewb2DBottom},
-	{(char *) "MoveBr", (XtActionProc) MoveSkewb2DBr},
-	{(char *) "Select", (XtActionProc) SelectSkewb},
-	{(char *) "Release", (XtActionProc) ReleaseSkewb},
-	{(char *) "Practice", (XtActionProc) PracticeSkewb},
-	{(char *) "PracticeMaybe", (XtActionProc) PracticeSkewbMaybe},
-	{(char *) "Practice2", (XtActionProc) PracticeSkewb2},
-	{(char *) "Randomize", (XtActionProc) RandomizeSkewb},
-	{(char *) "RandomizeMaybe", (XtActionProc) RandomizeSkewbMaybe},
-	{(char *) "Randomize2", (XtActionProc) RandomizeSkewb2},
-	{(char *) "Get", (XtActionProc) GetSkewb},
-	{(char *) "Write", (XtActionProc) WriteSkewb},
-	{(char *) "Clear", (XtActionProc) ClearSkewb},
-	{(char *) "Undo", (XtActionProc) UndoSkewb},
-	{(char *) "Solve", (XtActionProc) SolveSkewb},
-	{(char *) "Orientize", (XtActionProc) OrientizeSkewb},
-	{(char *) "Enter", (XtActionProc) EnterSkewb},
-	{(char *) "Leave", (XtActionProc) LeaveSkewb}
+	{(char *) "Quit", (XtActionProc) QuitPuzzle},
+	{(char *) "Hide", (XtActionProc) HidePuzzle},
+	{(char *) "MoveCcw", (XtActionProc) MovePuzzleCcw},
+	{(char *) "MoveTl", (XtActionProc) MovePuzzle2DTl},
+	{(char *) "MoveTop", (XtActionProc) MovePuzzle2DTop},
+	{(char *) "MoveTr", (XtActionProc) MovePuzzle2DTr},
+	{(char *) "MoveLeft", (XtActionProc) MovePuzzle2DLeft},
+	{(char *) "MoveCw", (XtActionProc) MovePuzzleCw},
+	{(char *) "MoveRight", (XtActionProc) MovePuzzle2dRight},
+	{(char *) "MoveBl", (XtActionProc) MovePuzzle2DBl},
+	{(char *) "MoveBottom", (XtActionProc) MovePuzzle2DBottom},
+	{(char *) "MoveBr", (XtActionProc) MovePuzzle2DBr},
+	{(char *) "Select", (XtActionProc) SelectPuzzle},
+	{(char *) "Release", (XtActionProc) ReleasePuzzle},
+	{(char *) "PracticeMaybe", (XtActionProc) PracticePuzzleMaybe},
+	{(char *) "Practice2", (XtActionProc) PracticePuzzle2},
+	{(char *) "RandomizeMaybe", (XtActionProc) RandomizePuzzleMaybe},
+	{(char *) "Randomize2", (XtActionProc) RandomizePuzzle2},
+	{(char *) "Get", (XtActionProc) GetPuzzle},
+	{(char *) "Write", (XtActionProc) WritePuzzle},
+	{(char *) "Undo", (XtActionProc) UndoPuzzle},
+	{(char *) "Redo", (XtActionProc) RedoPuzzle},
+	{(char *) "Clear", (XtActionProc) ClearPuzzle},
+	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
+	{(char *) "Solve", (XtActionProc) SolvePuzzle},
+	{(char *) "Practice", (XtActionProc) PracticePuzzle},
+	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
+	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Enter", (XtActionProc) EnterPuzzle},
+	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
 
-static XtResource resourcesSkewb2D[] =
+static XtResource resourcesPuzzle2D[] =
 {
 	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
 	 XtOffset(SkewbWidget, core.width),
@@ -235,29 +247,29 @@ Skewb2DClassRec skewb2dClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeSkewb2D,		/* initialize */
+		(XtInitProc) InitializePuzzle2D,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
-		actionsListSkewb2D,	/* actions */
-		XtNumber(actionsListSkewb2D),	/* num actions */
-		resourcesSkewb2D,	/* resources */
-		XtNumber(resourcesSkewb2D),	/* num resources */
+		actionsListPuzzle2D,	/* actions */
+		XtNumber(actionsListPuzzle2D),	/* num actions */
+		resourcesPuzzle2D,	/* resources */
+		XtNumber(resourcesPuzzle2D),	/* num resources */
 		NULLQUARK,	/* xrm class */
 		TRUE,		/* compress motion */
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
 		NULL,		/* destroy */
-		(XtWidgetProc) ResizeSkewb2D,	/* resize */
-		(XtExposeProc) ExposeSkewb2D,	/* expose */
-		(XtSetValuesFunc) SetValuesSkewb2D,	/* set values */
+		(XtWidgetProc) ResizePuzzle2D,	/* resize */
+		(XtExposeProc) ExposePuzzle2D,	/* expose */
+		(XtSetValuesFunc) SetValuesPuzzle2D,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
 		XtInheritAcceptFocus,	/* accept focus */
 		XtVersion,	/* version */
 		NULL,		/* callback private */
-		defaultTranslationsSkewb2D,	/* tm table */
+		defaultTranslationsPuzzle2D,	/* tm table */
 		NULL,		/* query geometry */
 		NULL,		/* display accelerator */
 		NULL		/* extension */
@@ -714,13 +726,13 @@ NarrowSelection2D(Skewb2DWidget w, int *face, int *position, int *direction)
 
 #ifndef WINVER
 static      Boolean
-SetValuesSkewb2D(Widget current, Widget request, Widget renew)
+SetValuesPuzzle2D(Widget current, Widget request, Widget renew)
 {
 	Skewb2DWidget c = (Skewb2DWidget) current, w = (Skewb2DWidget) renew;
 	Boolean     redraw = False;
 
 	if (w->skewb2d.diamondLength != c->skewb2d.diamondLength) {
-		ResizeSkewb2D(w);
+		ResizePuzzle2D(w);
 		redraw = True;
 	}
 	return (redraw);
@@ -731,7 +743,7 @@ SetValuesSkewb2D(Widget current, Widget request, Widget renew)
 static
 #endif
 void
-ResizeSkewb2D(Skewb2DWidget w)
+ResizePuzzle2D(Skewb2DWidget w)
 {
 #ifdef WINVER
 	RECT        rect;
@@ -775,7 +787,7 @@ ResizeSkewb2D(Skewb2DWidget w)
 static
 #endif
 void
-InitializeSkewb2D(
+InitializePuzzle2D(
 #ifdef WINVER
 Skewb2DWidget w
 #else
@@ -789,18 +801,18 @@ Widget request, Widget renew
 	SetAllColors((SkewbWidget) w);
 #endif
 	w->skewb.dim = 2;
-	ResizeSkewb2D(w);
+	ResizePuzzle2D(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-ExposeSkewb2D(
+ExposePuzzle2D(
 #ifdef WINVER
 Skewb2DWidget w
 #else
-Widget renew, XEvent * event, Region region
+Widget renew, XEvent *event, Region region
 #endif
 )
 {
@@ -818,66 +830,62 @@ Widget renew, XEvent * event, Region region
 
 #ifndef WINVER
 static void
-MoveSkewb2DTl(Skewb2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DTl(Skewb2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TL,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TL,
 		       (int) (event->xkey.state & ControlMask), FALSE);
 }
 
 static void
-MoveSkewb2DTop(Skewb2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DTop(Skewb2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TOP,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TOP,
 		       (int) (event->xkey.state & ControlMask),
-		       (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		       (int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewb2DTr(Skewb2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DTr(Skewb2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TR,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TR,
 		       (int) (event->xkey.state & ControlMask), FALSE);
 }
 
 static void
-MoveSkewb2DLeft(Skewb2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DLeft(Skewb2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
 		       (int) (event->xkey.state & ControlMask),
-		       (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		       (int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewb2dRight(Skewb2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2dRight(Skewb2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
 		       (int) (event->xkey.state & ControlMask),
-		       (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		       (int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewb2DBl(Skewb2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DBl(Skewb2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BL,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BL,
 		       (int) (event->xkey.state & ControlMask), FALSE);
 }
 
 static void
-MoveSkewb2DBottom(Skewb2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DBottom(Skewb2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
 		       (int) (event->xkey.state & ControlMask),
-		       (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		       (int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewb2DBr(Skewb2DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle2DBr(Skewb2DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BR,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BR,
 		       (int) (event->xkey.state & ControlMask), FALSE);
 }
 #endif
