@@ -29,20 +29,20 @@
 #include "Skewb3dP.h"
 
 #ifndef WINVER
-static Boolean SetValuesSkewb3D(Widget current, Widget request, Widget renew);
-static void ResizeSkewb3D(Skewb3DWidget w);
-static void InitializeSkewb3D(Widget request, Widget renew);
-static void ExposeSkewb3D(Widget renew, XEvent * event, Region region);
-static void MoveSkewb3DTl(Skewb3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb3DTop(Skewb3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb3DTr(Skewb3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb3DLeft(Skewb3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb3DRight(Skewb3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb3DBl(Skewb3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb3DBottom(Skewb3DWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewb3DBr(Skewb3DWidget w, XEvent * event, char **args, int nArgs);
+static Boolean SetValuesPuzzle3D(Widget current, Widget request, Widget renew);
+static void ResizePuzzle3D(Skewb3DWidget w);
+static void InitializePuzzle3D(Widget request, Widget renew);
+static void ExposePuzzle3D(Widget renew, XEvent *event, Region region);
+static void MovePuzzle3DTl(Skewb3DWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DTop(Skewb3DWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DTr(Skewb3DWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DLeft(Skewb3DWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DRight(Skewb3DWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DBl(Skewb3DWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DBottom(Skewb3DWidget w, XEvent *event, char **args, int nArgs);
+static void MovePuzzle3DBr(Skewb3DWidget w, XEvent *event, char **args, int nArgs);
 
-static char defaultTranslationsSkewb3D[] =
+static char defaultTranslationsPuzzle3D[] =
 "<KeyPress>q: Quit()\n\
  Ctrl<KeyPress>C: Quit()\n\
  <KeyPress>osfCancel: Hide()\n\
@@ -89,56 +89,60 @@ static char defaultTranslationsSkewb3D[] =
  <KeyPress>R15: MoveBr()\n\
  <Btn1Down>: Select()\n\
  <Btn1Up>: Release()\n\
- <KeyPress>p: Practice()\n\
  <Btn2Down>: PracticeMaybe()\n\
  <Btn2Down>(2+): Practice2()\n\
- <KeyPress>r: Randomize()\n\
  <Btn3Down>: RandomizeMaybe()\n\
  <Btn3Down>(2+): Randomize2()\n\
  <Btn4Down>: MoveTop()\n\
  <Btn5Down>: MoveBottom()\n\
  <KeyPress>g: Get()\n\
  <KeyPress>w: Write()\n\
- <KeyPress>c: Clear()\n\
  <KeyPress>u: Undo()\n\
+ <KeyPress>r: Redo()\n\
+ <KeyPress>c: Clear()\n\
+ <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
+ <KeyPress>p: Practice()\n\
  <KeyPress>o: Orientize()\n\
+ <KeyPress>v: View()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
-static XtActionsRec actionsListSkewb3D[] =
+static XtActionsRec actionsListPuzzle3D[] =
 {
-	{(char *) "Quit", (XtActionProc) QuitSkewb},
-	{(char *) "Hide", (XtActionProc) HideSkewb},
-	{(char *) "MoveCcw", (XtActionProc) MoveSkewbCcw},
-	{(char *) "MoveTl", (XtActionProc) MoveSkewb3DTl},
-	{(char *) "MoveTop", (XtActionProc) MoveSkewb3DTop},
-	{(char *) "MoveTr", (XtActionProc) MoveSkewb3DTr},
-	{(char *) "MoveLeft", (XtActionProc) MoveSkewb3DLeft},
-	{(char *) "MoveCw", (XtActionProc) MoveSkewbCw},
-	{(char *) "MoveRight", (XtActionProc) MoveSkewb3DRight},
-	{(char *) "MoveBl", (XtActionProc) MoveSkewb3DBl},
-	{(char *) "MoveBottom", (XtActionProc) MoveSkewb3DBottom},
-	{(char *) "MoveBr", (XtActionProc) MoveSkewb3DBr},
-	{(char *) "Select", (XtActionProc) SelectSkewb},
-	{(char *) "Release", (XtActionProc) ReleaseSkewb},
-	{(char *) "Practice", (XtActionProc) PracticeSkewb},
-	{(char *) "PracticeMaybe", (XtActionProc) PracticeSkewbMaybe},
-	{(char *) "Practice2", (XtActionProc) PracticeSkewb2},
-	{(char *) "Randomize", (XtActionProc) RandomizeSkewb},
-	{(char *) "RandomizeMaybe", (XtActionProc) RandomizeSkewbMaybe},
-	{(char *) "Randomize2", (XtActionProc) RandomizeSkewb2},
-	{(char *) "Get", (XtActionProc) GetSkewb},
-	{(char *) "Write", (XtActionProc) WriteSkewb},
-	{(char *) "Clear", (XtActionProc) ClearSkewb},
-	{(char *) "Undo", (XtActionProc) UndoSkewb},
-	{(char *) "Solve", (XtActionProc) SolveSkewb},
-	{(char *) "Orientize", (XtActionProc) OrientizeSkewb},
-	{(char *) "Enter", (XtActionProc) EnterSkewb},
-	{(char *) "Leave", (XtActionProc) LeaveSkewb}
+	{(char *) "Quit", (XtActionProc) QuitPuzzle},
+	{(char *) "Hide", (XtActionProc) HidePuzzle},
+	{(char *) "MoveCcw", (XtActionProc) MovePuzzleCcw},
+	{(char *) "MoveTl", (XtActionProc) MovePuzzle3DTl},
+	{(char *) "MoveTop", (XtActionProc) MovePuzzle3DTop},
+	{(char *) "MoveTr", (XtActionProc) MovePuzzle3DTr},
+	{(char *) "MoveLeft", (XtActionProc) MovePuzzle3DLeft},
+	{(char *) "MoveCw", (XtActionProc) MovePuzzleCw},
+	{(char *) "MoveRight", (XtActionProc) MovePuzzle3DRight},
+	{(char *) "MoveBl", (XtActionProc) MovePuzzle3DBl},
+	{(char *) "MoveBottom", (XtActionProc) MovePuzzle3DBottom},
+	{(char *) "MoveBr", (XtActionProc) MovePuzzle3DBr},
+	{(char *) "Select", (XtActionProc) SelectPuzzle},
+	{(char *) "Release", (XtActionProc) ReleasePuzzle},
+	{(char *) "PracticeMaybe", (XtActionProc) PracticePuzzleMaybe},
+	{(char *) "Practice2", (XtActionProc) PracticePuzzle2},
+	{(char *) "RandomizeMaybe", (XtActionProc) RandomizePuzzleMaybe},
+	{(char *) "Randomize2", (XtActionProc) RandomizePuzzle2},
+	{(char *) "Get", (XtActionProc) GetPuzzle},
+	{(char *) "Write", (XtActionProc) WritePuzzle},
+	{(char *) "Undo", (XtActionProc) UndoPuzzle},
+	{(char *) "Redo", (XtActionProc) RedoPuzzle},
+	{(char *) "Clear", (XtActionProc) ClearPuzzle},
+	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
+	{(char *) "Solve", (XtActionProc) SolvePuzzle},
+	{(char *) "Practice", (XtActionProc) PracticePuzzle},
+	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
+	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Enter", (XtActionProc) EnterPuzzle},
+	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
 
-static XtResource resourcesSkewb3D[] =
+static XtResource resourcesPuzzle3D[] =
 {
 	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
 	 XtOffset(SkewbWidget, core.width),
@@ -235,29 +239,29 @@ Skewb3DClassRec skewb3dClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeSkewb3D,		/* initialize */
+		(XtInitProc) InitializePuzzle3D,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
-		actionsListSkewb3D,	/* actions */
-		XtNumber(actionsListSkewb3D),	/* num actions */
-		resourcesSkewb3D,	/* resources */
-		XtNumber(resourcesSkewb3D),	/* num resources */
+		actionsListPuzzle3D,	/* actions */
+		XtNumber(actionsListPuzzle3D),	/* num actions */
+		resourcesPuzzle3D,	/* resources */
+		XtNumber(resourcesPuzzle3D),	/* num resources */
 		NULLQUARK,	/* xrm class */
 		TRUE,		/* compress motion */
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
 		NULL,		/* destroy */
-		(XtWidgetProc) ResizeSkewb3D,	/* resize */
-		(XtExposeProc) ExposeSkewb3D,	/* expose */
-		(XtSetValuesFunc) SetValuesSkewb3D,	/* set values */
+		(XtWidgetProc) ResizePuzzle3D,	/* resize */
+		(XtExposeProc) ExposePuzzle3D,	/* expose */
+		(XtSetValuesFunc) SetValuesPuzzle3D,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
 		XtInheritAcceptFocus,	/* accept focus */
 		XtVersion,	/* version */
 		NULL,		/* callback private */
-		defaultTranslationsSkewb3D,	/* tm table */
+		defaultTranslationsPuzzle3D,	/* tm table */
 		NULL,		/* query geometry */
 		NULL,		/* display accelerator */
 		NULL		/* extension */
@@ -1008,13 +1012,13 @@ NarrowSelection3D(Skewb3DWidget w, int *face, int *position, int *direction)
 
 #ifndef WINVER
 static      Boolean
-SetValuesSkewb3D(Widget current, Widget request, Widget renew)
+SetValuesPuzzle3D(Widget current, Widget request, Widget renew)
 {
 	Skewb3DWidget c = (Skewb3DWidget) current, w = (Skewb3DWidget) renew;
 	Boolean     redraw = False;
 
 	if (w->skewb3d.cubeSize.x != c->skewb3d.cubeSize.x) {
-		ResizeSkewb3D(w);
+		ResizePuzzle3D(w);
 		redraw = True;
 	}
 	return (redraw);
@@ -1025,7 +1029,7 @@ SetValuesSkewb3D(Widget current, Widget request, Widget renew)
 static
 #endif
 void
-ResizeSkewb3D(Skewb3DWidget w)
+ResizePuzzle3D(Skewb3DWidget w)
 {
 	Point      tempSize;
 #ifdef WINVER
@@ -1090,7 +1094,7 @@ ResizeSkewb3D(Skewb3DWidget w)
 static
 #endif
 void
-InitializeSkewb3D(
+InitializePuzzle3D(
 #ifdef WINVER
 Skewb3DWidget w
 #else
@@ -1104,18 +1108,18 @@ Widget request, Widget renew
 	SetAllColors((SkewbWidget) w);
 #endif
 	w->skewb.dim = 3;
-	ResizeSkewb3D(w);
+	ResizePuzzle3D(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-ExposeSkewb3D(
+ExposePuzzle3D(
 #ifdef WINVER
 Skewb3DWidget w
 #else
-Widget renew, XEvent * event, Region region
+Widget renew, XEvent *event, Region region
 #endif
 )
 {
@@ -1133,74 +1137,66 @@ Widget renew, XEvent * event, Region region
 
 #ifndef WINVER
 static void
-MoveSkewb3DTl(Skewb3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DTl(Skewb3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TL,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TL,
 		       (int) (event->xkey.state & ControlMask),
-		       (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		       (int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewb3DTop(Skewb3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DTop(Skewb3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TOP,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TOP,
 		       (int) (event->xkey.state & ControlMask),
-		       (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		       (int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewb3DTr(Skewb3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DTr(Skewb3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TR,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TR,
 		       (int) (event->xkey.state & ControlMask),
-		       (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		       (int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewb3DLeft(Skewb3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DLeft(Skewb3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
 		       (int) (event->xkey.state & ControlMask),
-		       (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		       (int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewb3DRight(Skewb3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DRight(Skewb3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
 		       (int) (event->xkey.state & ControlMask),
-		       (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		       (int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewb3DBl(Skewb3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DBl(Skewb3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BL,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BL,
 		       (int) (event->xkey.state & ControlMask),
-		       (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		       (int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewb3DBottom(Skewb3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DBottom(Skewb3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
 		       (int) (event->xkey.state & ControlMask),
-		       (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		       (int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewb3DBr(Skewb3DWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzle3DBr(Skewb3DWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BR,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BR,
 		       (int) (event->xkey.state & ControlMask),
-		       (int) (event->xkey.state &
-		    (Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		       (int) (event->xkey.state & Mod1Mask));
 }
 #endif
