@@ -7,7 +7,7 @@
 #
 #  Copyright (c) 2003 - 2006	David Albert Bagley, bagleyd@tux.org
 #
-#                   All Rights Reserved
+#		   All Rights Reserved
 #
 #  Permission to use, copy, modify, and distribute this software and
 #  its documentation for any purpose and without fee is hereby granted,
@@ -33,11 +33,9 @@
  * This file is provided AS IS with no warranties of any kind.  The author
  * shall have no liability with respect to the infringement of copyrights,
  * trade secrets or any patents by this file or any part thereof.  In no
- * event will the author be liable for any lost revenue or profits or
+ *event will the author be liable for any lost revenue or profits or
  * other special, indirect and consequential damages.
  *
- * This mode shows an auto-solving a skewb "puzzle".
- *
  * Thanks goes also to Brian Paul for making it possible and inexpensive
  * to use OpenGL at home.
  *
@@ -51,7 +49,7 @@
 #include "SkewbP.h"
 #include "SkewbGLP.h"
 
-static GLXContext glXContext;
+static GLXContext *glXContext = (GLXContext *) NULL;
 static Boolean madeCurrent = False;
 
 static float front_shininess[] =
@@ -220,13 +218,13 @@ static SkewbLocPos orthToDiag[MAXFACES][MAXORIENT][MAXORIENT] =
  * Color labels mapping:
  * =====================
  *
- *        +------+
- *        |3    0|
- *        |      |
- *        | TOP  |
- *        | (0)  |
- *        | RED  |
- *        |2    1|
+ *	+------+
+ *	|3    0|
+ *	|      |
+ *	| TOP  |
+ *	| (0)  |
+ *	| RED  |
+ *	|2    1|
  * +------+------+------+
  * |3    0|3    0|3    0|
  * |      |      |      |
@@ -235,20 +233,20 @@ static SkewbLocPos orthToDiag[MAXFACES][MAXORIENT][MAXORIENT] =
  * | BLUE |WHITE |GREEN |
  * |2    1|2    1|2    1|
  * +------+------+------+
- *        |3    0|
- *        |      |
- *        |BOTTOM|
- *        | (4)  |
- *        | PINK |
- *        |2    1|
- *        +------+         +------+
- *        |3    0|         |3 /\ 0|
- *        |      |         | /  \ |
- *        | BACK |         |/xxxx\|
- *        | (5)  |         |\(N) /|
- *        |YELLOW|         | \  / |
- *        |2    1|         |2 \/ 1|
- *        +------+         +------+
+ *	|3    0|
+ *	|      |
+ *	|BOTTOM|
+ *	| (4)  |
+ *	| PINK |
+ *	|2    1|
+ *	+------+	 +------+
+ *	|3    0|	 |3 /\ 0|
+ *	|      |	 | /  \ |
+ *	| BACK |	 |/xxxx\|
+ *	| (5)  |	 |\(N) /|
+ *	|YELLOW|	 | \  / |
+ *	|2    1|	 |2 \/ 1|
+ *	+------+	 +------+
  *
  *  Map to 3d
  *  FRONT  => X, Y
@@ -259,20 +257,28 @@ static SkewbLocPos orthToDiag[MAXFACES][MAXORIENT][MAXORIENT] =
  *  BOTTOM => X, Z
  */
 
-static Boolean SetValuesSkewbGL(Widget current, Widget request, Widget renew);
-static void ResizeSkewbGL(SkewbGLWidget w);
-static void InitializeSkewbGL(Widget request, Widget renew);
-static void ExposeSkewbGL(Widget renew, XEvent * event, Region region);
-static void MoveSkewbGLTl(SkewbGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewbGLTop(SkewbGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewbGLTr(SkewbGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewbGLLeft(SkewbGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewbGLRight(SkewbGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewbGLBl(SkewbGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewbGLBottom(SkewbGLWidget w, XEvent * event, char **args, int nArgs);
-static void MoveSkewbGLBr(SkewbGLWidget w, XEvent * event, char **args, int nArgs);
+static Boolean SetValuesPuzzleGL(Widget current, Widget request, Widget renew);
+static void ResizePuzzleGL(SkewbGLWidget w);
+static void InitializePuzzleGL(Widget request, Widget renew);
+static void ExposePuzzleGL(Widget renew, XEvent *event, Region region);
+static void MovePuzzleGLTl(SkewbGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLTop(SkewbGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLTr(SkewbGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLLeft(SkewbGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLRight(SkewbGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLBl(SkewbGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLBottom(SkewbGLWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleGLBr(SkewbGLWidget w,
+	XEvent *event, char **args, int nArgs);
 
-static char defaultTranslationsSkewbGL[] =
+static char defaultTranslationsPuzzleGL[] =
 "<KeyPress>q: Quit()\n\
  Ctrl<KeyPress>C: Quit()\n\
  <KeyPress>osfCancel: Hide()\n\
@@ -319,55 +325,65 @@ static char defaultTranslationsSkewbGL[] =
  <KeyPress>R15: MoveBr()\n\
  <Btn1Down>: Select()\n\
  <Btn1Up>: Release()\n\
- <KeyPress>p: Practice()\n\
  <Btn2Down>: PracticeMaybe()\n\
  <Btn2Down>(2+): Practice2()\n\
- <KeyPress>r: Randomize()\n\
  <Btn3Down>: RandomizeMaybe()\n\
  <Btn3Down>(2+): Randomize2()\n\
  <KeyPress>g: Get()\n\
  <KeyPress>w: Write()\n\
- <KeyPress>c: Clear()\n\
  <KeyPress>u: Undo()\n\
+ <KeyPress>r: Redo()\n\
+ <KeyPress>c: Clear()\n\
+ <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
+ <KeyPress>p: Practice()\n\
  <KeyPress>o: Orientize()\n\
+ <KeyPress>v: View()\n\
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
-static XtActionsRec actionsListSkewbGL[] =
+static XtActionsRec actionsListPuzzleGL[] =
 {
-	{(char *) "Quit", (XtActionProc) QuitSkewb},
-	{(char *) "Hide", (XtActionProc) HideSkewb},
-	{(char *) "MoveCcw", (XtActionProc) MoveSkewbCcw},
-	{(char *) "MoveTl", (XtActionProc) MoveSkewbGLTl},
-	{(char *) "MoveTop", (XtActionProc) MoveSkewbGLTop},
-	{(char *) "MoveTr", (XtActionProc) MoveSkewbGLTr},
-	{(char *) "MoveLeft", (XtActionProc) MoveSkewbGLLeft},
-	{(char *) "MoveCw", (XtActionProc) MoveSkewbCw},
-	{(char *) "MoveRight", (XtActionProc) MoveSkewbGLRight},
-	{(char *) "MoveBl", (XtActionProc) MoveSkewbGLBl},
-	{(char *) "MoveBottom", (XtActionProc) MoveSkewbGLBottom},
-	{(char *) "MoveBr", (XtActionProc) MoveSkewbGLBr},
-	{(char *) "Select", (XtActionProc) SelectSkewb},
-	{(char *) "Release", (XtActionProc) ReleaseSkewb},
-	{(char *) "Practice", (XtActionProc) PracticeSkewb},
-	{(char *) "PracticeMaybe", (XtActionProc) PracticeSkewbMaybe},
-	{(char *) "Practice2", (XtActionProc) PracticeSkewb2},
-	{(char *) "Randomize", (XtActionProc) RandomizeSkewb},
-	{(char *) "RandomizeMaybe", (XtActionProc) RandomizeSkewbMaybe},
-	{(char *) "Randomize2", (XtActionProc) RandomizeSkewb2},
-	{(char *) "Get", (XtActionProc) GetSkewb},
-	{(char *) "Write", (XtActionProc) WriteSkewb},
-	{(char *) "Clear", (XtActionProc) ClearSkewb},
-	{(char *) "Undo", (XtActionProc) UndoSkewb},
-	{(char *) "Solve", (XtActionProc) SolveSkewb},
-	{(char *) "Orientize", (XtActionProc) OrientizeSkewb},
-	{(char *) "Enter", (XtActionProc) EnterSkewb},
-	{(char *) "Leave", (XtActionProc) LeaveSkewb}
+	{(char *) "Quit", (XtActionProc) QuitPuzzle},
+	{(char *) "Hide", (XtActionProc) HidePuzzle},
+	{(char *) "MoveCcw", (XtActionProc) MovePuzzleCcw},
+	{(char *) "MoveTl", (XtActionProc) MovePuzzleGLTl},
+	{(char *) "MoveTop", (XtActionProc) MovePuzzleGLTop},
+	{(char *) "MoveTr", (XtActionProc) MovePuzzleGLTr},
+	{(char *) "MoveLeft", (XtActionProc) MovePuzzleGLLeft},
+	{(char *) "MoveCw", (XtActionProc) MovePuzzleCw},
+	{(char *) "MoveRight", (XtActionProc) MovePuzzleGLRight},
+	{(char *) "MoveBl", (XtActionProc) MovePuzzleGLBl},
+	{(char *) "MoveBottom", (XtActionProc) MovePuzzleGLBottom},
+	{(char *) "MoveBr", (XtActionProc) MovePuzzleGLBr},
+	{(char *) "Select", (XtActionProc) SelectPuzzle},
+	{(char *) "Release", (XtActionProc) ReleasePuzzle},
+	{(char *) "PracticeMaybe", (XtActionProc) PracticePuzzleMaybe},
+	{(char *) "Practice2", (XtActionProc) PracticePuzzle2},
+	{(char *) "RandomizeMaybe", (XtActionProc) RandomizePuzzleMaybe},
+	{(char *) "Randomize2", (XtActionProc) RandomizePuzzle2},
+	{(char *) "Get", (XtActionProc) GetPuzzle},
+	{(char *) "Write", (XtActionProc) WritePuzzle},
+	{(char *) "Undo", (XtActionProc) UndoPuzzle},
+	{(char *) "Redo", (XtActionProc) RedoPuzzle},
+	{(char *) "Clear", (XtActionProc) ClearPuzzle},
+	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
+	{(char *) "Solve", (XtActionProc) SolvePuzzle},
+	{(char *) "Practice", (XtActionProc) PracticePuzzle},
+	{(char *) "Orientize", (XtActionProc) OrientizePuzzle},
+	{(char *) "View", (XtActionProc) ViewPuzzle},
+	{(char *) "Enter", (XtActionProc) EnterPuzzle},
+	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
 
-static XtResource resourcesSkewbGL[] =
+static XtResource resourcesPuzzleGL[] =
 {
+	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
+	 XtOffset(SkewbWidget, core.width),
+	 XtRString, (caddr_t) "250"},
+	{XtNheight, XtCHeight, XtRDimension, sizeof (Dimension),
+	 XtOffset(SkewbWidget, core.height),
+	 XtRString, (caddr_t) "400"},
 	{XtNmono, XtCMono, XtRBoolean, sizeof (Boolean),
 	 XtOffset(SkewbWidget, skewb.mono),
 	 XtRString, (caddr_t) "FALSE"},
@@ -382,40 +398,37 @@ static XtResource resourcesSkewbGL[] =
 	 XtRString, (caddr_t) XtDefaultBackground},
 	{XtNframeColor, XtCColor, XtRPixel, sizeof (Pixel),
 	 XtOffset(SkewbWidget, skewb.frameColor),
-	 XtRString, (caddr_t) "cyan" /*XtDefaultForeground*/},
+	 XtRString, (caddr_t) "Cyan" /*XtDefaultForeground*/},
 	{XtNfaceColor0, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(SkewbWidget, skewb.faceName[0]),
-	 XtRString, (caddr_t) "red"},
+	 XtRString, (caddr_t) "Red"},
 	{XtNfaceColor1, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(SkewbWidget, skewb.faceName[1]),
-	 XtRString, (caddr_t) "blue"},
+	 XtRString, (caddr_t) "Blue"},
 	{XtNfaceColor2, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(SkewbWidget, skewb.faceName[2]),
-	 XtRString, (caddr_t) "white"},
+	 XtRString, (caddr_t) "White"},
 	{XtNfaceColor3, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(SkewbWidget, skewb.faceName[3]),
-	 XtRString, (caddr_t) "green"},
+	 XtRString, (caddr_t) "Green"},
 	{XtNfaceColor4, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(SkewbWidget, skewb.faceName[4]),
-	 XtRString, (caddr_t) "pink"},
+	 XtRString, (caddr_t) "Pink"},
 	{XtNfaceColor5, XtCLabel, XtRString, sizeof (String),
 	 XtOffset(SkewbWidget, skewb.faceName[5]),
-	 XtRString, (caddr_t) "yellow"},
+	 XtRString, (caddr_t) "Yellow"},
 	{XtNpieceBorder, XtCColor, XtRPixel, sizeof (Pixel),
 	 XtOffset(SkewbWidget, skewb.borderColor),
 	 XtRString, (caddr_t) "gray25" /*XtDefaultForeground*/},
 	{XtNdelay, XtCDelay, XtRInt, sizeof (int),
 	 XtOffset(SkewbWidget, skewb.delay),
-	 XtRString, (caddr_t) "300"},
+	 XtRString, (caddr_t) "10"},
 	{XtNfont, XtCFont, XtRString, sizeof (String),
 	 XtOffset(SkewbWidget, skewb.font),
 	 XtRString, (caddr_t) "9x15bold"},
-	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
-	 XtOffset(SkewbWidget, core.width),
-	 XtRString, (caddr_t) "250"},
-	{XtNheight, XtCHeight, XtRDimension, sizeof (Dimension),
-	 XtOffset(SkewbWidget, core.height),
-	 XtRString, (caddr_t) "400"},
+	{XtNview, XtCView, XtRInt, sizeof (int),
+	 XtOffset(SkewbWidget, skewb.view),
+	 XtRString, (caddr_t) "1"},
 	{XtNorient, XtCOrient, XtRBoolean, sizeof (Boolean),
 	 XtOffset(SkewbWidget, skewb.orient),
 	 XtRString, (caddr_t) "FALSE"},	/* DEFAULTORIENT */
@@ -466,29 +479,29 @@ SkewbGLClassRec skewbGLClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeSkewbGL,		/* initialize */
+		(XtInitProc) InitializePuzzleGL,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
-		actionsListSkewbGL,	/* actions */
-		XtNumber(actionsListSkewbGL),	/* num actions */
-		resourcesSkewbGL,	/* resources */
-		XtNumber(resourcesSkewbGL),	/* num resources */
+		actionsListPuzzleGL,	/* actions */
+		XtNumber(actionsListPuzzleGL),	/* num actions */
+		resourcesPuzzleGL,	/* resources */
+		XtNumber(resourcesPuzzleGL),	/* num resources */
 		NULLQUARK,	/* xrm class */
 		TRUE,		/* compress motion */
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
 		NULL,		/* destroy */
-		(XtWidgetProc) ResizeSkewbGL,	/* resize */
-		(XtExposeProc) ExposeSkewbGL,	/* expose */
-		(XtSetValuesFunc) SetValuesSkewbGL,	/* set values */
+		(XtWidgetProc) ResizePuzzleGL,	/* resize */
+		(XtExposeProc) ExposePuzzleGL,	/* expose */
+		(XtSetValuesFunc) SetValuesPuzzleGL,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
 		XtInheritAcceptFocus,	/* accept focus */
 		XtVersion,	/* version */
 		NULL,		/* callback private */
-		defaultTranslationsSkewbGL,	/* tm table */
+		defaultTranslationsPuzzleGL,	/* tm table */
 		NULL,		/* query geometry */
 		NULL,		/* display accelerator */
 		NULL		/* extension */
@@ -507,11 +520,12 @@ WidgetClass skewbGLWidgetClass = (WidgetClass) & skewbGLClassRec;
 void
 printCube(SkewbGLWidget w)
 {
-	int      face, position;
+	int face, position;
 
 	for (face = 0; face < MAXFACES; face++) {
 		for (position = 0; position < MAXCUBES; position++) {
-			(void) printf("%d %d  ", w->skewb.cubeLoc[face][position].face,
+			(void) printf("%d %d  ",
+				w->skewb.cubeLoc[face][position].face,
 				w->skewb.cubeLoc[face][position].rotation);
 		}
 		(void) printf("\n");
@@ -523,38 +537,39 @@ printCube(SkewbGLWidget w)
 static void
 pickColor(SkewbGLWidget w, int C, int mono)
 {
-       float Material[] = {1.0, 1.0, 1.0, 1.0};
-
+	float Material[] = {1.0, 1.0, 1.0, 1.0};
 	XColor color;
+
 	switch (C) {
 		case TOP_FACE:
-			break;
 		case LEFT_FACE:
-			break;
 		case FRONT_FACE:
-			break;
 		case RIGHT_FACE:
-			break;
 		case BOTTOM_FACE:
-			break;
 		case BACK_FACE:
 			break;
 		case TEST_FACE:
-			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, MaterialMagenta);
+			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,
+				MaterialMagenta);
 			return;
 		default:
-			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, MaterialCyan);
+			glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE,
+				MaterialCyan);
 			return;
 	}
 	color.pixel = w->skewb.faceColor[C];
 	XQueryColor(XtDisplay(w), DefaultColormapOfScreen(XtScreen(w)),
 		&color);
-	/*(void) printf("i%d %d %d\n", color.red, color.green, color.blue);*/
+#ifdef DEBUG
+	(void) printf("i%d %d %d\n", color.red, color.green, color.blue);
+#endif
 	if (mono) {
 		float intensity = 0.3 * color.red + 0.59 * color.green +
 			0.11 * color.blue;
 
-		/*(void) printf("m%g\n", intensity);*/
+#ifdef DEBUG
+		(void) printf("m%g\n", intensity);
+#endif
 		Material[0] *= (float) intensity / 0xFFFF;
 		Material[1] *= (float) intensity / 0xFFFF;
 		Material[2] *= (float) intensity / 0xFFFF;
@@ -563,7 +578,9 @@ pickColor(SkewbGLWidget w, int C, int mono)
 		Material[1] *= (float) color.green / 0xFFFF;
 		Material[2] *= (float) color.blue / 0xFFFF;
 	}
-	/*(void) printf("f%g %g %g\n", Material[0], Material[1], Material[2]);*/
+#ifdef DEBUG
+	(void) printf("f%g %g %g\n", Material[0], Material[1], Material[2]);
+#endif
 	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, Material);
 }
 
@@ -666,58 +683,310 @@ draw_stickerless_facit(void)
 	return True;
 }
 
+static void
+draw_orient_line(int face, int rotation, float size, float x, float y, float z)
+{
+	int err = 0;
+	char *buf1, *buf2;
+	float stickerLong = STICKERLONG * size;
+
+	glTranslatef(x, y, z);
+	glBegin(GL_QUADS);
+	glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, MaterialGray);
+	switch (face) {
+	case 0:
+		switch (rotation) {
+		case 0:
+			glVertex3f(-CUT, 0.0, -CUT);
+			glVertex3f(CUT, 0.0, -CUT);
+			glVertex3f(CUT, 0.0, -stickerLong);
+			glVertex3f(-CUT, 0.0, -stickerLong);
+			break;
+		case 1:
+			glVertex3f(CUT, 0.0, -CUT);
+			glVertex3f(CUT, 0.0, CUT);
+			glVertex3f(stickerLong, 0.0, CUT);
+			glVertex3f(stickerLong, 0.0, -CUT);
+			break;
+		case 2:
+			glVertex3f(CUT, 0.0, CUT);
+			glVertex3f(-CUT, 0.0, CUT);
+			glVertex3f(-CUT, 0.0, stickerLong);
+			glVertex3f(CUT, 0.0, stickerLong);
+			break;
+		case 3:
+			glVertex3f(-CUT, 0.0, CUT);
+			glVertex3f(-CUT, 0.0, -CUT);
+			glVertex3f(-stickerLong, 0.0, -CUT);
+			glVertex3f(-stickerLong, 0.0, CUT);
+			break;
+		default:
+			err = 1;
+		}
+		break;
+	case 1:
+		switch (rotation) {
+		case 0:
+			glVertex3f(0.0, CUT, -CUT);
+			glVertex3f(0.0, CUT, CUT);
+			glVertex3f(0.0, stickerLong, CUT);
+			glVertex3f(0.0, stickerLong, -CUT);
+			break;
+		case 1:
+			glVertex3f(0.0, CUT, CUT);
+			glVertex3f(0.0, -CUT, CUT);
+			glVertex3f(0.0, -CUT, stickerLong);
+			glVertex3f(0.0, CUT, stickerLong);
+			break;
+		case 2:
+			glVertex3f(0.0, -CUT, CUT);
+			glVertex3f(0.0, -CUT, -CUT);
+			glVertex3f(0.0, -stickerLong, -CUT);
+			glVertex3f(0.0, -stickerLong, CUT);
+			break;
+		case 3:
+			glVertex3f(0.0, -CUT, -CUT);
+			glVertex3f(0.0, CUT, -CUT);
+			glVertex3f(0.0, CUT, -stickerLong);
+			glVertex3f(0.0, -CUT, -stickerLong);
+			break;
+		default:
+			err = 1;
+		}
+		break;
+	case 2:
+		switch (rotation) {
+		case 0:
+			glVertex3f(-CUT, CUT, 0.0);
+			glVertex3f(CUT, CUT, 0.0);
+			glVertex3f(CUT, stickerLong, 0.0);
+			glVertex3f(-CUT, stickerLong, 0.0);
+			break;
+		case 1:
+			glVertex3f(CUT, CUT, 0.0);
+			glVertex3f(CUT, -CUT, 0.0);
+			glVertex3f(stickerLong, -CUT, 0.0);
+			glVertex3f(stickerLong, CUT, 0.0);
+			break;
+		case 2:
+			glVertex3f(CUT, -CUT, 0.0);
+			glVertex3f(-CUT, -CUT, 0.0);
+			glVertex3f(-CUT, -stickerLong, 0.0);
+			glVertex3f(CUT, -stickerLong, 0.0);
+			break;
+		case 3:
+			glVertex3f(-CUT, -CUT, 0.0);
+			glVertex3f(-CUT, CUT, 0.0);
+			glVertex3f(-stickerLong, CUT, 0.0);
+			glVertex3f(-stickerLong, -CUT, 0.0);
+			break;
+		default:
+			err = 1;
+		}
+		break;
+	case 3:
+		switch (rotation) {
+		case 0:
+			glVertex3f(0.0, CUT, -CUT);
+			glVertex3f(0.0, stickerLong, -CUT);
+			glVertex3f(0.0, stickerLong, CUT);
+			glVertex3f(0.0, CUT, CUT);
+			break;
+		case 1:
+			glVertex3f(0.0, -CUT, -CUT);
+			glVertex3f(0.0, -CUT, -stickerLong);
+			glVertex3f(0.0, CUT, -stickerLong);
+			glVertex3f(0.0, CUT, -CUT);
+			break;
+		case 2:
+			glVertex3f(0.0, -CUT, CUT);
+			glVertex3f(0.0, -stickerLong, CUT);
+			glVertex3f(0.0, -stickerLong, -CUT);
+			glVertex3f(0.0, -CUT, -CUT);
+			break;
+		case 3:
+			glVertex3f(0.0, CUT, CUT);
+			glVertex3f(0.0, CUT, stickerLong);
+			glVertex3f(0.0, -CUT, stickerLong);
+			glVertex3f(0.0, -CUT, CUT);
+			break;
+		default:
+			err = 1;
+		}
+		break;
+	case 4:
+		switch (rotation) {
+		case 0:
+			glVertex3f(CUT, 0.0, CUT);
+			glVertex3f(CUT, 0.0, stickerLong);
+			glVertex3f(-CUT, 0.0, stickerLong);
+			glVertex3f(-CUT, 0.0, CUT);
+			break;
+		case 1:
+			glVertex3f(CUT, 0.0, -CUT);
+			glVertex3f(stickerLong, 0.0, -CUT);
+			glVertex3f(stickerLong, 0.0, CUT);
+			glVertex3f(CUT, 0.0, CUT);
+			break;
+		case 2:
+			glVertex3f(-CUT, 0.0, -CUT);
+			glVertex3f(-CUT, 0.0, -stickerLong);
+			glVertex3f(CUT, 0.0, -stickerLong);
+			glVertex3f(CUT, 0.0, -CUT);
+			break;
+		case 3:
+			glVertex3f(-CUT, 0.0, CUT);
+			glVertex3f(-stickerLong, 0.0, CUT);
+			glVertex3f(-stickerLong, 0.0, -CUT);
+			glVertex3f(-CUT, 0.0, -CUT);
+			break;
+		default:
+			err = 1;
+		}
+		break;
+	case 5:
+		switch (rotation) {
+		case 0:
+			glVertex3f(CUT, -CUT, 0.0);
+			glVertex3f(CUT, -stickerLong, 0.0);
+			glVertex3f(-CUT, -stickerLong, 0.0);
+			glVertex3f(-CUT, -CUT, 0.0);
+			break;
+		case 1:
+			glVertex3f(CUT, CUT, 0.0);
+			glVertex3f(stickerLong, CUT, 0.0);
+			glVertex3f(stickerLong, -CUT, 0.0);
+			glVertex3f(CUT, -CUT, 0.0);
+			break;
+		case 2:
+			glVertex3f(-CUT, CUT, 0.0);
+			glVertex3f(-CUT, stickerLong, 0.0);
+			glVertex3f(CUT, stickerLong, 0.0);
+			glVertex3f(CUT, CUT, 0.0);
+			break;
+		case 3:
+			glVertex3f(-CUT, -CUT, 0.0);
+			glVertex3f(-stickerLong, -CUT, 0.0);
+			glVertex3f(-stickerLong, CUT, 0.0);
+			glVertex3f(-CUT, CUT, 0.0);
+			break;
+		default:
+			err = 1;
+		}
+		break;
+	}
+	glEnd();
+	if (err == 1) {
+		intCat(&buf1, "draw_orient_line: face ", face);
+		stringCat(&buf2, buf1, ", rotation ");
+		free(buf1);
+		intCat(&buf1, buf2, rotation);
+		free(buf2);
+		XtWarning(buf1);
+		free(buf1);
+	}
+}
+
 static void
 DrawFacit(SkewbGLWidget w, int face)
 {
 	Boolean mono = w->skewb.mono;
 
-	glBegin(GL_QUADS);
+	glPushMatrix();
 	pickColor(w, w->skewb.cubeLoc[face][MAXORIENT].face, mono);
 	switch (face) {
 	case TOP_FACE:
+		glBegin(GL_QUADS);
 		glNormal3f(0.0, 1.0, 0.0);
 		glVertex3f(0.0, FSTICKERDEPTH, -FSTICKERLONG);
 		glVertex3f(-FSTICKERLONG, FSTICKERDEPTH, 0.0);
 		glVertex3f(0.0, FSTICKERDEPTH, FSTICKERLONG);
 		glVertex3f(FSTICKERLONG, FSTICKERDEPTH, 0.0);
+		glEnd();
+		if (w->skewb.orient) {
+			draw_orient_line(face,
+				w->skewb.cubeLoc[face][MAXORIENT].rotation,
+				2.0,
+				0.0, FCUTDEPTH, 0.0);
+		}
 		break;
 	case LEFT_FACE:
+		glBegin(GL_QUADS);
 		glNormal3f(-1.0, 0.0, 0.0);
 		glVertex3f(-FSTICKERDEPTH, 0.0, -FSTICKERLONG);
 		glVertex3f(-FSTICKERDEPTH, -FSTICKERLONG, 0.0);
 		glVertex3f(-FSTICKERDEPTH, 0.0, FSTICKERLONG);
 		glVertex3f(-FSTICKERDEPTH, FSTICKERLONG, 0.0);
+		glEnd();
+		if (w->skewb.orient) {
+			draw_orient_line(face,
+				w->skewb.cubeLoc[face][MAXORIENT].rotation,
+				2.0,
+				-FCUTDEPTH, 0.0, 0.0);
+		}
 		break;
 	case FRONT_FACE:
+		glBegin(GL_QUADS);
 		glNormal3f(0.0, 0.0, 1.0);
 		glVertex3f(-FSTICKERLONG, 0.0, FSTICKERDEPTH);
 		glVertex3f(0.0, -FSTICKERLONG, FSTICKERDEPTH);
 		glVertex3f(FSTICKERLONG, 0.0, FSTICKERDEPTH);
 		glVertex3f(0.0, FSTICKERLONG, FSTICKERDEPTH);
+		glEnd();
+		if (w->skewb.orient) {
+			draw_orient_line(face,
+				w->skewb.cubeLoc[face][MAXORIENT].rotation,
+				2.0,
+				0.0, 0.0, FCUTDEPTH);
+		}
 		break;
 	case RIGHT_FACE:
+		glBegin(GL_QUADS);
 		glNormal3f(1.0, 0.0, 0.0);
 		glVertex3f(FSTICKERDEPTH, FSTICKERLONG, 0.0);
 		glVertex3f(FSTICKERDEPTH, 0.0, FSTICKERLONG);
 		glVertex3f(FSTICKERDEPTH, -FSTICKERLONG, 0.0);
 		glVertex3f(FSTICKERDEPTH, 0.0, -FSTICKERLONG);
+		glEnd();
+		if (w->skewb.orient) {
+			draw_orient_line(face,
+				w->skewb.cubeLoc[face][MAXORIENT].rotation,
+				2.0,
+				FCUTDEPTH, 0.0, 0.0);
+		}
 		break;
 	case BOTTOM_FACE:
+		glBegin(GL_QUADS);
 		glNormal3f(0.0, -1.0, 0.0);
 		glVertex3f(FSTICKERLONG, -FSTICKERDEPTH, 0.0);
 		glVertex3f(0.0, -FSTICKERDEPTH, FSTICKERLONG);
 		glVertex3f(-FSTICKERLONG, -FSTICKERDEPTH, 0.0);
 		glVertex3f(0.0, -FSTICKERDEPTH, -FSTICKERLONG);
+		glEnd();
+		if (w->skewb.orient) {
+			draw_orient_line(face,
+				w->skewb.cubeLoc[face][MAXORIENT].rotation,
+				2.0,
+				0.0, -FCUTDEPTH, 0.0);
+		}
 		break;
 	case BACK_FACE:
+		glBegin(GL_QUADS);
 		glNormal3f(0.0, 0.0, -1.0);
 		glVertex3f(0.0, FSTICKERLONG, -FSTICKERDEPTH);
 		glVertex3f(FSTICKERLONG, 0.0, -FSTICKERDEPTH);
 		glVertex3f(0.0, -FSTICKERLONG, -FSTICKERDEPTH);
 		glVertex3f(-FSTICKERLONG, 0.0, -FSTICKERDEPTH);
+		glEnd();
+		if (w->skewb.orient) {
+			draw_orient_line(face,
+				w->skewb.cubeLoc[face][MAXORIENT].rotation,
+				2.0,
+				0.0, 0.0, -FCUTDEPTH);
+		}
 		break;
 	}
-	glEnd();
+	glPopMatrix();
 }
 
 static void
@@ -726,7 +995,8 @@ DrawCubit(SkewbGLWidget w,
 {
 	Boolean mono = w->skewb.mono;
 
-	if (back != NO_FACE) {
+	if (back != NO_FACE) { /* Yellow */
+		glPushMatrix();
 		glBegin(GL_POLYGON);
 		pickColor(w, back, mono);
 		glNormal3f(0.0, 0.0, -1.0);
@@ -747,8 +1017,36 @@ DrawCubit(SkewbGLWidget w,
 			glVertex3f(STICKERLONG, -STICKERSHORT, -STICKERDEPTH);
 		}
 		glEnd();
+		if (w->skewb.orient) {
+			if (right != NO_FACE && top != NO_FACE) {
+				draw_orient_line(BACK_FACE,
+					w->skewb.cubeLoc[BACK_FACE][1].rotation,
+					0.5,
+					0.1, 0.5 * STICKERSHORT, -CUTDEPTH);
+			}
+			if (left != NO_FACE && top != NO_FACE) {
+				draw_orient_line(BACK_FACE,
+					w->skewb.cubeLoc[BACK_FACE][2].rotation,
+					0.5,
+					-0.5 * STICKERSHORT, 0.1, -CUTDEPTH);
+			}
+			if (right != NO_FACE && bottom != NO_FACE) {
+				draw_orient_line(BACK_FACE,
+					w->skewb.cubeLoc[BACK_FACE][0].rotation,
+					0.5,
+					0.1, -0.5 * STICKERSHORT, -CUTDEPTH);
+			}
+			if (left != NO_FACE && bottom != NO_FACE) {
+				draw_orient_line(BACK_FACE,
+					w->skewb.cubeLoc[BACK_FACE][3].rotation,
+					0.5,
+					-0.5 * STICKERSHORT, -0.1, -CUTDEPTH);
+			}
+		}
+		glPopMatrix();
 	}
-	if (front != NO_FACE) {
+	if (front != NO_FACE) { /* White */
+		glPushMatrix();
 		glBegin(GL_POLYGON);
 		pickColor(w, front, mono);
 		glNormal3f(0.0, 0.0, 1.0);
@@ -769,8 +1067,36 @@ DrawCubit(SkewbGLWidget w,
 			glVertex3f(STICKERLONG, STICKERSHORT, STICKERDEPTH);
 		}
 		glEnd();
+		if (w->skewb.orient) {
+			if (right != NO_FACE && top != NO_FACE) {
+				draw_orient_line(FRONT_FACE,
+					w->skewb.cubeLoc[FRONT_FACE][0].rotation,
+					0.5,
+					0.1, 0.5 * STICKERSHORT, CUTDEPTH);
+			}
+			if (left != NO_FACE && top != NO_FACE) {
+				draw_orient_line(FRONT_FACE,
+					w->skewb.cubeLoc[FRONT_FACE][3].rotation,
+					0.5,
+					-0.5 * STICKERSHORT, 0.1, CUTDEPTH);
+			}
+			if (right != NO_FACE && bottom != NO_FACE) {
+				draw_orient_line(FRONT_FACE,
+					w->skewb.cubeLoc[FRONT_FACE][1].rotation,
+					0.5,
+					0.1, -0.5 * STICKERSHORT, CUTDEPTH);
+			}
+			if (left != NO_FACE && bottom != NO_FACE) {
+				draw_orient_line(FRONT_FACE,
+					w->skewb.cubeLoc[FRONT_FACE][2].rotation,
+					0.5,
+					-0.5 * STICKERSHORT, -0.1, CUTDEPTH);
+			}
+		}
+		glPopMatrix();
 	}
-	if (left != NO_FACE) {
+	if (left != NO_FACE) { /* Blue */
+		glPushMatrix();
 		glBegin(GL_POLYGON);
 		pickColor(w, left, mono);
 		glNormal3f(-1.0, 0.0, 0.0);
@@ -791,8 +1117,36 @@ DrawCubit(SkewbGLWidget w,
 			glVertex3f(-STICKERDEPTH, -STICKERLONG, STICKERSHORT);
 		}
 		glEnd();
+		if (w->skewb.orient) {
+			if (front != NO_FACE && top != NO_FACE) {
+				draw_orient_line(LEFT_FACE,
+					w->skewb.cubeLoc[LEFT_FACE][0].rotation,
+					0.5,
+					-CUTDEPTH, 0.1, 0.5 * STICKERSHORT);
+			}
+			if (back != NO_FACE && top != NO_FACE) {
+				draw_orient_line(LEFT_FACE,
+					w->skewb.cubeLoc[LEFT_FACE][3].rotation,
+					0.5,
+					-CUTDEPTH, 0.5 * STICKERSHORT, -0.1);
+			}
+			if (front != NO_FACE && bottom != NO_FACE) {
+				draw_orient_line(LEFT_FACE,
+					w->skewb.cubeLoc[LEFT_FACE][1].rotation,
+					0.5,
+					-CUTDEPTH, -0.1, 0.5 * STICKERSHORT);
+			}
+			if (back != NO_FACE && bottom != NO_FACE) {
+				draw_orient_line(LEFT_FACE,
+					w->skewb.cubeLoc[LEFT_FACE][2].rotation,
+					0.5,
+					-CUTDEPTH, -0.5 * STICKERSHORT, -0.1);
+			}
+		}
+		glPopMatrix();
 	}
 	if (right != NO_FACE) { /* Green */
+		glPushMatrix();
 		glBegin(GL_POLYGON);
 		pickColor(w, right, mono);
 		glNormal3f(1.0, 0.0, 0.0);
@@ -813,8 +1167,36 @@ DrawCubit(SkewbGLWidget w,
 			glVertex3f(STICKERDEPTH, -STICKERLONG, -STICKERSHORT);
 		}
 		glEnd();
+		if (w->skewb.orient) {
+			if (front != NO_FACE && top != NO_FACE) {
+				draw_orient_line(RIGHT_FACE,
+					w->skewb.cubeLoc[RIGHT_FACE][3].rotation,
+					0.5,
+					CUTDEPTH, 0.1, 0.5 * STICKERSHORT);
+			}
+			if (back != NO_FACE && top != NO_FACE) {
+				draw_orient_line(RIGHT_FACE,
+					w->skewb.cubeLoc[RIGHT_FACE][0].rotation,
+					0.5,
+					CUTDEPTH, 0.5 * STICKERSHORT, -0.1);
+			}
+			if (front != NO_FACE && bottom != NO_FACE) {
+				draw_orient_line(RIGHT_FACE,
+					w->skewb.cubeLoc[RIGHT_FACE][2].rotation,
+					0.5,
+					CUTDEPTH, -0.1, 0.5 * STICKERSHORT);
+			}
+			if (back != NO_FACE && bottom != NO_FACE) {
+				draw_orient_line(RIGHT_FACE,
+					w->skewb.cubeLoc[RIGHT_FACE][1].rotation,
+					0.5,
+					CUTDEPTH, -0.5 * STICKERSHORT, -0.1);
+			}
+		}
+		glPopMatrix();
 	}
 	if (bottom != NO_FACE) { /* Pink */
+		glPushMatrix();
 		glBegin(GL_POLYGON);
 		pickColor(w, bottom, mono);
 		glNormal3f(0.0, -1.0, 0.0);
@@ -835,8 +1217,36 @@ DrawCubit(SkewbGLWidget w,
 			glVertex3f(STICKERSHORT, -STICKERDEPTH, -STICKERLONG);
 		}
 		glEnd();
+		if (w->skewb.orient) {
+			if (right != NO_FACE && back != NO_FACE) {
+				draw_orient_line(BOTTOM_FACE,
+					w->skewb.cubeLoc[BOTTOM_FACE][1].rotation,
+					0.5,
+					0.5 * STICKERSHORT, -CUTDEPTH, -0.1);
+			}
+			if (left != NO_FACE && back != NO_FACE) {
+				draw_orient_line(BOTTOM_FACE,
+					w->skewb.cubeLoc[BOTTOM_FACE][2].rotation,
+					0.5,
+					-0.5 * STICKERSHORT, -CUTDEPTH, -0.1);
+			}
+			if (right != NO_FACE && front != NO_FACE) {
+				draw_orient_line(BOTTOM_FACE,
+					w->skewb.cubeLoc[BOTTOM_FACE][0].rotation,
+					0.5,
+					0.5 * STICKERSHORT, -CUTDEPTH, 0.1);
+			}
+			if (left != NO_FACE && front != NO_FACE) {
+				draw_orient_line(BOTTOM_FACE,
+					w->skewb.cubeLoc[BOTTOM_FACE][3].rotation,
+					0.5,
+					-0.5 * STICKERSHORT, -CUTDEPTH, 0.1);
+			}
+		}
+		glPopMatrix();
 	}
-	if (top != NO_FACE) {
+	if (top != NO_FACE) { /* Red */
+		glPushMatrix();
 		glBegin(GL_POLYGON);
 		pickColor(w, top, mono);
 		glNormal3f(0.0, 1.0, 0.0);
@@ -857,18 +1267,44 @@ DrawCubit(SkewbGLWidget w,
 			glVertex3f(-STICKERSHORT, STICKERDEPTH, -STICKERLONG);
 		}
 		glEnd();
+		if (w->skewb.orient) {
+			if (right != NO_FACE && back != NO_FACE) {
+				draw_orient_line(TOP_FACE,
+					w->skewb.cubeLoc[TOP_FACE][0].rotation,
+					0.5,
+					0.5 * STICKERSHORT, CUTDEPTH, -0.1);
+			}
+			if (left != NO_FACE && back != NO_FACE) {
+				draw_orient_line(TOP_FACE,
+					w->skewb.cubeLoc[TOP_FACE][3].rotation,
+					0.5,
+					-0.5 * STICKERSHORT, CUTDEPTH, -0.1);
+			}
+			if (right != NO_FACE && front != NO_FACE) {
+				draw_orient_line(TOP_FACE,
+					w->skewb.cubeLoc[TOP_FACE][1].rotation,
+					0.5,
+					0.5 * STICKERSHORT, CUTDEPTH, 0.1);
+			}
+			if (left != NO_FACE && front != NO_FACE) {
+				draw_orient_line(TOP_FACE,
+					w->skewb.cubeLoc[TOP_FACE][2].rotation,
+					0.5,
+					-0.5 * STICKERSHORT, CUTDEPTH, 0.1);
+			}
+		}
+		glPopMatrix();
 	}
 }
 
-/* Assume the size is at least 2, or its just not challenging... */
 static Bool
 DrawCube(SkewbGLWidget w)
 {
 #define S1 1
 #define DRAW_STICKERLESS_FACIT() if (!draw_stickerless_facit()) return False
 #define DRAW_STICKERLESS_CUBIT() if (!draw_stickerless_cubit()) return False
-	SkewbSlice  slice;
-	GLfloat     rotateStep;
+	SkewbSlice slice;
+	GLfloat rotateStep;
 	int i;
 
 	/*if (w->skewbGL.movement.face == NO_FACE) */ {
@@ -1023,7 +1459,7 @@ DrawCube(SkewbGLWidget w)
 void
 DrawAllPiecesGL(SkewbGLWidget w)
 {
-	if (!glXMakeCurrent(XtDisplay(w), XtWindow(w), glXContext)) {
+	if (!glXMakeCurrent(XtDisplay(w), XtWindow(w), *glXContext)) {
 		XtWarning("GLX error");
 	}
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
@@ -1033,23 +1469,27 @@ DrawAllPiecesGL(SkewbGLWidget w)
 		glScalef(SCALE, SCALE * w->core.width / w->core.height, SCALE);
 	else
 		glScalef(SCALE * w->core.height / w->core.width, SCALE, SCALE);
-#if 0
-	glRotatef(w->skewbGL.step / 2, 0, 1, 0); /* >-  vertical */
-	glRotatef(w->skewbGL.step / 2, 1, 0, 0);
-#endif
-#if 1
-	glRotatef(w->skewbGL.step / 2, 1, 0, 0); /* Y  !vertical */
-	glRotatef(w->skewbGL.step / 2, 0, 1, 0);
-#endif
-#if 0
-	glRotatef(2 * w->skewbGL.step, 0, 0, 1); /* -<  vertical */
-	glRotatef(5 * w->skewbGL.step / 2, 0, 1, 0);
-	glRotatef(w->skewbGL.step / 2, 1, 0, 0);
-#endif
-#if 0
-	glRotatef(5 * w->skewbGL.step / 2, 0, 1, 0); /* ^  !vertical */
-	glRotatef(w->skewbGL.step / 2, 1, 0, 1);
-#endif
+	switch (w->skewb.view) {
+		case 0: /* >-  vertical */
+			glRotatef(w->skewbGL.step / 2, 0, 1, 0);
+			glRotatef(w->skewbGL.step / 2, 1, 0, 0);
+			break;
+		case 1: /* Y  !vertical */
+			glRotatef(w->skewbGL.step / 2, 1, 0, 0);
+			glRotatef(w->skewbGL.step / 2, 0, 1, 0);
+			break;
+		case 2: /* -<  vertical */
+			glRotatef(2 * w->skewbGL.step, 0, 0, 1);
+			glRotatef(5 * w->skewbGL.step / 2, 0, 1, 0);
+			glRotatef(w->skewbGL.step / 2, 1, 0, 0);
+			break;
+		case 3: /* ^  !vertical */
+			glRotatef(5 * w->skewbGL.step / 2, 0, 1, 0);
+			glRotatef(w->skewbGL.step / 2, 1, 0, 1);
+			break;
+		default:
+			break;
+	}
 	if (!DrawCube(w)) {
 		XtWarning("DrawCube error");
 	}
@@ -1078,21 +1518,27 @@ initGL(SkewbGLWidget w) {
 
 	/*XGetWindowAttributes(XtDisplay(w), XtWindow(w), &xgwa); */
 	xviIn.screen = DefaultScreen(XtDisplay(w));
-        xviIn.visualid = XVisualIDFromVisual(DefaultVisual(XtDisplay(w),
+	xviIn.visualid = XVisualIDFromVisual(DefaultVisual(XtDisplay(w),
 		xviIn.screen));
 	xviOut = XGetVisualInfo(XtDisplay(w), VisualScreenMask | VisualIDMask,
 		&xviIn, &numVis);
 	if (!xviOut) {
 		XtWarning("Could not get XVisualInfo");
 		return (GLXContext *) NULL;
-        }
-	glXContext = glXCreateContext(XtDisplay(w), xviOut, 0, GL_TRUE);
+	}
+	if (glXContext)
+		free(glXContext);
+	if ((glXContext = (GLXContext *) malloc(sizeof (GLXContext))) ==
+			NULL) {
+		DISPLAY_ERROR("Not enough memory for glx info, exiting.");
+	}
+	*glXContext = glXCreateContext(XtDisplay(w), xviOut, 0, GL_TRUE);
 	(void) XFree((char *) xviOut);
-	if (!glXContext) {
+	if (!*glXContext) {
 		XtWarning("Could not create rendering context");
 		return (GLXContext *) NULL;
-        }
-	return &glXContext;
+	}
+	return glXContext;
 }
 
 static void
@@ -1100,7 +1546,7 @@ makeCurrentGL(SkewbGLWidget w)
 {
 	GLboolean   rgbaMode;
 
-	if (!glXMakeCurrent(XtDisplay(w), XtWindow(w), glXContext)) {
+	if (!glXMakeCurrent(XtDisplay(w), XtWindow(w), *glXContext)) {
 		XtWarning("GLX error");
 	} else {
 		madeCurrent = True;
@@ -1114,7 +1560,7 @@ makeCurrentGL(SkewbGLWidget w)
 			DefaultScreen(XtDisplay(w))));
 	}
 
-	ResizeSkewbGL(w);
+	ResizePuzzleGL(w);
 
 	glDrawBuffer(GL_BACK);
 	glClearDepth(1.0);
@@ -1142,36 +1588,42 @@ makeCurrentGL(SkewbGLWidget w)
 	w->skewbGL.angleStep = 90.0;
 	w->skewbGL.step = 90;
 #if 0
-        w->skewbGL.vx = 0.05;
+	w->skewbGL.vx = 0.05;
 	if (LRAND() & 1)
 		w->skewbGL.vx *= -1.0;
-        w->skewbGL.vy = 0.05;
+	w->skewbGL.vy = 0.05;
 	if (LRAND() & 1)
 		w->skewbGL.vy *= -1.0;
 #endif
-        w->skewbGL.rotateStep = 0.0;
+	w->skewbGL.rotateStep = 0.0;
 }
 
-static      Boolean
-SetValuesSkewbGL(Widget current, Widget request, Widget renew)
+static Boolean
+SetValuesPuzzleGL(Widget current, Widget request, Widget renew)
 {
 	SkewbGLWidget c = (SkewbGLWidget) current, w = (SkewbGLWidget) renew;
-	Boolean     redraw = False;
+	Boolean redraw = False;
 
+	if (w->skewb.view != c->skewb.view) {
+		ResizePuzzleGL(w);
+		redraw = True;
+	}
+#if 0
 	if (w->skewbGL.cubeSizex.x != c->skewbGL.cubeSizex.x ||
 	    w->skewbGL.cubeSizey.x != c->skewbGL.cubeSizey.x ||
 	    w->skewbGL.cubeSizez.x != c->skewbGL.cubeSizez.x ||
 	    w->skewbGL.cubeSizex.y != c->skewbGL.cubeSizex.y ||
 	    w->skewbGL.cubeSizey.y != c->skewbGL.cubeSizey.y ||
 	    w->skewbGL.cubeSizez.y != c->skewbGL.cubeSizez.y) {
-		ResizeSkewbGL(w);
+		ResizePuzzleGL(w);
 		redraw = True;
 	}
+#endif
 	return (redraw);
 }
 
 static void
-ResizeSkewbGL(SkewbGLWidget w)
+ResizePuzzleGL(SkewbGLWidget w)
 {
 	glViewport(0, 0, (GLint) w->core.width,
 		(GLint) w->core.height);
@@ -1182,18 +1634,18 @@ ResizeSkewbGL(SkewbGLWidget w)
 }
 
 static void
-InitializeSkewbGL(Widget request, Widget renew)
+InitializePuzzleGL(Widget request, Widget renew)
 {
 	SkewbGLWidget w = (SkewbGLWidget) renew;
 
 	w->skewb.dim = 4; /* 2 and 3 already taken */
 	SetAllColors((SkewbWidget) w);
 	(void) initGL(w);
-	ResizeSkewbGL(w);
+	ResizePuzzleGL(w);
 }
 
 static void
-ExposeSkewbGL(Widget renew, XEvent * event, Region region)
+ExposePuzzleGL(Widget renew, XEvent *event, Region region)
 {
 	SkewbGLWidget w = (SkewbGLWidget) renew;
 
@@ -1208,75 +1660,75 @@ ExposeSkewbGL(Widget renew, XEvent * event, Region region)
 
 #ifndef WINVER
 static void
-MoveSkewbGLTl(SkewbGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLTl(SkewbGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TL,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y,
+		TL,
 		(int) (event->xkey.state & ControlMask),
-		(int) (event->xkey.state &
-		(Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewbGLTop(SkewbGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLTop(SkewbGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TOP,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y,
+		TOP,
 		(int) (event->xkey.state & ControlMask),
-		(int) (event->xkey.state &
-		(Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewbGLTr(SkewbGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLTr(SkewbGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, TR,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y,
+		TR,
 		(int) (event->xkey.state & ControlMask),
-		(int) (event->xkey.state &
-		(Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewbGLLeft(SkewbGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLLeft(SkewbGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, LEFT,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y,
+		LEFT,
 		(int) (event->xkey.state & ControlMask),
-		(int) (event->xkey.state &
-		(Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewbGLRight(SkewbGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLRight(SkewbGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, RIGHT,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y,
+		RIGHT,
 		(int) (event->xkey.state & ControlMask),
-		(int) (event->xkey.state &
-		(Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewbGLBl(SkewbGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLBl(SkewbGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BL,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y,
+		BL,
 		(int) (event->xkey.state & ControlMask),
-		(int) (event->xkey.state &
-		(Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewbGLBottom(SkewbGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLBottom(SkewbGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BOTTOM,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y,
+		BOTTOM,
 		(int) (event->xkey.state & ControlMask),
-		(int) (event->xkey.state &
-		(Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		(int) (event->xkey.state & Mod1Mask));
 }
 
 static void
-MoveSkewbGLBr(SkewbGLWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleGLBr(SkewbGLWidget w, XEvent *event, char **args, int nArgs)
 {
-	MoveSkewbInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y, BR,
+	MovePuzzleInput((SkewbWidget) w, event->xbutton.x, event->xbutton.y,
+		BR,
 		(int) (event->xkey.state & ControlMask),
-		(int) (event->xkey.state &
-		(Mod1Mask | Mod2Mask | Mod3Mask | Mod4Mask | Mod5Mask)));
+		(int) (event->xkey.state & Mod1Mask));
 }
 #endif
 #endif
