@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 2003 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 2003 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -53,10 +53,8 @@ extern SkewbGLClassRec skewbGLClassRec;
 #define BOTTOM 10
 #define LEFT 11
 #define MAXROTATE 3
-#define MAXCUBES (MAXORIENT+1)
 #define MINOR 0
 #define MAJOR 1
-#define MAXFACES 6
 
 #define TOP_FACE 0
 #define LEFT_FACE 1
@@ -79,6 +77,9 @@ extern SkewbGLClassRec skewbGLClassRec;
 #define FSTICKERLONG (FACEROUND-0.05)
 #define FSTICKERSHORT (FSTICKERLONG-0.05)
 #define FSTICKERDEPTH (FACELEN+0.01)
+#define CUT 0.04
+#define CUTDEPTH (STICKERDEPTH+0.001)
+#define FCUTDEPTH (FSTICKERDEPTH+0.001)
 
 #define SCALE (1.1)
 
@@ -99,7 +100,7 @@ typedef struct _SkewbGLPart {
 
 	Position    faceDiagonal;
 	Position    cubeDiagonalx, cubeDiagonaly, cubeDiagonalz;
-	Point       cubeSizex, cubeSizey, cubeSizez;
+	/*Point       cubeSizex, cubeSizey, cubeSizez;*/
 	Point       faceSize, viewSize;
 	Point       viewMiddle;
 	SkewbMove   movement;
