@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1996 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1996 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -29,23 +29,24 @@
 #define IDM_WRITE	101
 #define IDM_EXIT	102
 #define IDM_HIDE	103
-#define IDM_CLEAR	200
-#define IDM_UNDO	201
-#define IDM_RANDOMIZE	202
-#define IDM_PRACTICE	203
-#define IDM_SOLVE	204
-#define IDM_ORIENT	205
-#define IDM_DIM 206
-#define IDM_TR  300
-#define IDM_BR  301
-#define IDM_BL  302
-#define IDM_TL  303
-#define IDM_CW  305
-#define IDM_CCW 307
-#define IDM_TOP 308
-#define IDM_RIGHT 309
-#define IDM_BOTTOM  310
-#define IDM_LEFT  311
+#define IDM_UNDO	200
+#define IDM_REDO	201
+#define IDM_CLEAR	202
+#define IDM_RANDOMIZE	203
+#define IDM_PRACTICE	204
+#define IDM_SOLVE	205
+#define IDM_ORIENTIZE	206
+#define IDM_DIM		207
+#define IDM_TR		300
+#define IDM_BR		301
+#define IDM_BL		302
+#define IDM_TL		303
+#define IDM_CW		305
+#define IDM_CCW		307
+#define IDM_TOP		308
+#define IDM_RIGHT	309
+#define IDM_BOTTOM	310
+#define IDM_LEFT	311
 #define IDM_CONTROL_TR  340
 #define IDM_CONTROL_BR  341
 #define IDM_CONTROL_BL  342
@@ -53,9 +54,9 @@
 #define IDM_CONTROL_CW  345
 #define IDM_CONTROL_CCW 347
 #define IDM_CONTROL_TOP 348
-#define IDM_CONTROL_RIGHT 349
-#define IDM_CONTROL_BOTTOM  350
-#define IDM_CONTROL_LEFT  351
+#define IDM_CONTROL_RIGHT	349
+#define IDM_CONTROL_BOTTOM	350
+#define IDM_CONTROL_LEFT	351
 #define IDM_ABOUT	500
 #define IDM_DESCRIPTION	501
 #define IDM_FEATURES	502
