@@ -93,7 +93,7 @@ static SkewbRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Skewb Version 7.2.1\n"
+"Skewb Version 7.2.4\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -105,9 +105,9 @@ static const char optionsHelp[] = {
 "[-display [{host}]:[{vs}]] [-[no]mono] [-[no]{reverse|rv}]\n"
 "[-{foreground|fg} {color}] [-{background|bg} {color}]\n"
 "[-face{0|1|2|3|4|5} {color}] [-{border|bd} {color}]\n"
-"[-{font|fn} {fontname}] [-[no]orient] [-[no]practice]\n"
-"[-userName {string}] [-scoreFile {filename}] [-scores]\n"
-"[-version]\n"
+"[-{font|fn} {fontname}] [-view {int}] [-[no]orient]\n"
+"[-[no]practice] [-userName {string}]\n"
+"[-scoreFile {filename}] [-scores] [-version]\n"
 };
 #endif
 
@@ -148,10 +148,23 @@ static const char featuresHelp[] = {
 "the record should say \"practice\").  This is good for "
 "learning moves and experimenting.\n"
 "\n"
-"Click \"mouse-right\" button,  or press \"R\" or \"r\" "
+"Click \"mouse-right\" button,  or press \"Z\" or \"z\" "
 "keys,  to randomize the puzzle  (this must be done first\n"
 "to set a new record).\n"
 "\n"
+"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
+"\n"
+"Press \"W\" or \"w\" keys to save (write) a puzzle.\n"
+"\n"
+"Press \"U\" or \"u\" keys to undo a move.\n"
+"\n"
+"Press \"R\" or \"r\" keys to redo a move.\n"
+"\n"
+"Press \"C\" or \"c\" keys to clear the puzzle.\n"
+"\n"
+"\"S\" or \"s\" keys reserved for the auto-solver "
+"(not implemented).\n"
+"\n"
 "Press \"O\" or \"o\" keys to toggle  the orient mode.  One "
 "has  to orient the faces  in orient mode,  besides\n"
 "getting  all the faces to be the  same color.  To do this "
@@ -160,16 +173,7 @@ static const char featuresHelp[] = {
 "piece.  This does add complexity  so there are 2\n"
 "sets of records.\n"
 "\n"
-"\"S\" or \"s\" keys reserved for the auto-solver "
-"(not implemented).\n"
-"\n"
-"Press \"U\" or \"u\" keys to undo a move.\n"
-"\n"
-"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
-"\n"
-"Press \"W\" or \"w\" keys to save a puzzle.\n"
-"\n"
-"Press \"C\" or \"c\" keys to clear a puzzle.\n"
+"Press \"V\" or \"v\" keys to change the view of the cube.\n"
 "\n"
 "Press \"Esc\" key to hide program.\n"
 "\n"
@@ -247,6 +251,7 @@ static char titleDsp[TITLELEN] = "";
 #endif
 static char scoreFileName[FILENAMELEN] = SCOREFILE;
 static char fileName[FILENAMELEN];
+static Boolean randomized = False;
 #ifdef WINVER
 #define MAXPROGNAME 80
 static char progDsp[MAXPROGNAME] = TITLE;
@@ -265,7 +270,7 @@ static Widget shell;
 static Widget shellGL;
 #endif
 #endif
-static Pixmap skewbIcon = None;
+static Pixmap icon = None;
 static Widget topLevel, skewb2d, skewb3d;
 #ifdef HAVE_OPENGL
 static Widget skewbGL;
@@ -478,15 +483,15 @@ MakeEquivalent(String *userName, String *scoreFile,
 		exit(0);
 	}
 	XtVaSetValues(skewb2d,
-		XtNdirection, SKEWB_IGNORE,
+		XtNdirection, PUZZLE_IGNORE,
 		XtNstart, False, NULL);
 	XtVaSetValues(skewb3d,
 		XtNuserName, *userName,
 		XtNorient, *orient,
+		XtNpractice, *practice,
 		XtNmono, mono,
 		XtNreverse, reverse,
-		XtNdirection, SKEWB_IGNORE,
-		XtNpractice, *practice,
+		XtNdirection, PUZZLE_IGNORE,
 		XtNstart, False,
 		XtNforeground, foreground,
 		XtNbackground, background,
@@ -501,10 +506,10 @@ MakeEquivalent(String *userName, String *scoreFile,
 	XtVaSetValues(skewbGL,
 		XtNuserName, *userName,
 		XtNorient, *orient,
+		XtNpractice, *practice,
 		XtNmono, mono,
 		XtNreverse, reverse,
-		XtNdirection, SKEWB_IGNORE,
-		XtNpractice, *practice,
+		XtNdirection, PUZZLE_IGNORE,
 		XtNstart, False,
 		XtNforeground, foreground,
 		XtNbackground, background,
@@ -589,7 +594,7 @@ void
 #ifdef WINVER
 	int dim;
 
-	InitializeSkewb(w, brush);
+	InitializePuzzle(w, brush);
 
 	orient = w->skewb.orient;
 	practice = w->skewb.practice;
@@ -617,7 +622,7 @@ void
 #ifdef WINVER
 		(void) strncpy(userNameDsp, userName, USERNAMELEN);
 #else
-		char * login = getlogin();
+		char *login = getlogin();
 
 		if (login == NULL) {
 			(void) strcpy(userNameDsp, "");
@@ -650,10 +655,10 @@ void
 
 #ifdef WINVER
 void
-SetSkewb(SkewbWidget w, int reason)
+setPuzzle(SkewbWidget w, int reason)
 #else
 static void
-CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
+CallbackPuzzle(Widget w, caddr_t clientData, skewbCallbackStruct *callData)
 #endif
 {
 	Boolean orient, practice, start, cheat;
@@ -724,7 +729,7 @@ CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-		case SKEWB_HIDE:
+		case PUZZLE_HIDE:
 #ifdef WINVER
 			ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -744,24 +749,22 @@ CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
 #endif
 			break;
 #ifndef WINVER
-		case SKEWB_PRACTICE_QUERY:
+		case PUZZLE_PRACTICE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(practiceDialog);
 #else
-			XtVaSetValues(skewb2d,
-				XtNmenu, 8, NULL); /* menu choice */
+			XtVaSetValues(skewb2d, XtNmenu, MENU_PRACTICE, NULL);
 #endif
 			break;
-		case SKEWB_RANDOMIZE_QUERY:
+		case PUZZLE_RANDOMIZE_QUERY:
 #ifdef HAVE_MOTIF
 			XtManageChild(randomizeDialog);
 #else
-			XtVaSetValues(skewb2d,
-				XtNmenu, 5, NULL); /* menu choice */
+			XtVaSetValues(skewb2d, XtNmenu, MENU_RANDOMIZE, NULL);
 #endif
 			break;
 #endif
-		case SKEWB_SOLVE_MESSAGE:
+		case PUZZLE_SOLVE_MESSAGE:
 #ifdef WINVER
 			(void) MessageBox(w->core.hWnd, solveHelp,
 				"Auto-Solve", MB_OK);
@@ -773,7 +776,7 @@ CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case SKEWB_RESTORE:
+		case PUZZLE_RESTORE:
 			if (practice) {
 				(void) strncpy(recordDsp, "practice",
 					MESSAGELEN);
@@ -782,43 +785,46 @@ CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
 #endif
 			}
 			movesDsp = 0;
+			randomized = False;
 #ifndef WINVER
 #ifdef HAVE_OPENGL
 			XtVaSetValues(otherw2,
-				XtNdirection, SKEWB_RESTORE, NULL);
+				XtNdirection, PUZZLE_RESTORE, NULL);
 #endif
 			XtVaSetValues(otherw1,
-				XtNdirection, SKEWB_RESTORE, NULL);
+				XtNdirection, PUZZLE_RESTORE, NULL);
 			XtVaSetValues(w,
-				XtNdirection, SKEWB_RESTORE, NULL);
+				XtNdirection, PUZZLE_RESTORE, NULL);
 #endif
 			break;
-		case SKEWB_CLEAR:
+		case PUZZLE_CLEAR:
 			movesDsp = 0;
+			randomized = False;
 #ifndef WINVER
 #ifdef HAVE_OPENGL
 			XtVaSetValues(otherw2,
-				XtNdirection, SKEWB_CLEAR, NULL);
+				XtNdirection, PUZZLE_CLEAR, NULL);
 #endif
 			XtVaSetValues(otherw1,
-				XtNdirection, SKEWB_CLEAR, NULL);
+				XtNdirection, PUZZLE_CLEAR, NULL);
 			XtVaSetValues(w,
-				XtNdirection, SKEWB_CLEAR, NULL);
+				XtNdirection, PUZZLE_CLEAR, NULL);
 #endif
 			break;
-		case SKEWB_RESET:
+		case PUZZLE_RESET:
 			movesDsp = 0;
+			randomized = False;
 			break;
-		case SKEWB_ILLEGAL:
-			if (practice || start)
+		case PUZZLE_ILLEGAL:
+			if (practice || randomized)
 				(void) strncpy(messageDsp, "Illegal move",
 					MESSAGELEN);
 			else
-				(void) strncpy(messageDsp, "Randomize to start",
-					MESSAGELEN);
+				(void) strncpy(messageDsp,
+					"Randomize to start", MESSAGELEN);
 			break;
 #ifndef WINVER
-		case SKEWB_MOVED:
+		case PUZZLE_MOVED:
 			movesDsp++;
 #ifdef HAVE_OPENGL
 			XtVaSetValues(otherw2,
@@ -834,7 +840,7 @@ CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
 				XtNdirection, callData->direction, NULL);
 			SET_STARTED(w, True);
 			break;
-		case SKEWB_CONTROL:
+		case PUZZLE_CONTROL:
 #ifdef HAVE_OPENGL
 			XtVaSetValues(otherw2,
 				XtNface, callData->face,
@@ -845,9 +851,9 @@ CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
 				XtNface, callData->face,
 				XtNpos, callData->position,
 				XtNdirection, callData->direction, NULL);
-			return;
+			break;
 #endif
-		case SKEWB_SOLVED:
+		case PUZZLE_SOLVED:
 			if (practice)
 				movesDsp = 0;
 			else if (cheat)
@@ -859,18 +865,18 @@ CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
 			else
 				(void) strncpy(messageDsp, "Solved!",
 					MESSAGELEN);
-#ifdef WINVER
-			SET_STARTED(w, False);
-#else
 			SET_STARTED(w, False);
+#ifndef WINVER
 #ifdef HAVE_OPENGL
 			SET_STARTED(otherw2, False);
 #endif
 			SET_STARTED(otherw1, False);
 #endif
+			randomized = False;
 			break;
-		case SKEWB_PRACTICE:
+		case PUZZLE_PRACTICE:
 			movesDsp = 0;
+			randomized = False;
 			practice = !practice;
 			if (!practice)
 				(void) strncpy(messageDsp, "Randomize to start",
@@ -897,8 +903,9 @@ CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case SKEWB_RANDOMIZE:
+		case PUZZLE_RANDOMIZE:
 			movesDsp = 0;
+			randomized = True;
 #ifdef WINVER
 			w->skewb.practice = False;
 			w->skewb.started = False;
@@ -916,7 +923,20 @@ CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
 				XtNstart, False, NULL);
 #endif
 			break;
-		case SKEWB_ORIENT:
+#ifdef HAVE_OPENGL
+		case PUZZLE_VIEW:
+			{
+				int view;
+
+				XtVaGetValues(skewbGL,
+					XtNview, &view, NULL);
+				view = (view + 1) % (2 * MAXVIEWS);
+				XtVaSetValues(skewbGL,
+					XtNview, view, NULL);
+			}
+			break;
+#endif
+		case PUZZLE_ORIENTIZE:
 			movesDsp = 0;
 			orient = !orient;
 			PrintRecord(orient, practice);
@@ -936,7 +956,7 @@ CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
 #endif
 #endif
 			break;
-		case SKEWB_COMPUTED:
+		case PUZZLE_COMPUTED:
 #ifdef WINVER
 			SET_STARTED(w, False);
 #else
@@ -948,7 +968,7 @@ CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
 #endif
 			break;
 #ifndef WINVER
-		case SKEWB_UNDO:
+		case PUZZLE_UNDO:
 			movesDsp--;
 #ifdef HAVE_OPENGL
 			XtVaSetValues(otherw2,
@@ -965,8 +985,26 @@ CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
 			SET_STARTED(w, True);
 			break;
 #endif
+#ifndef WINVER
+		case PUZZLE_REDO:
+			movesDsp++;
+#ifdef HAVE_OPENGL
+			XtVaSetValues(otherw2,
+				XtNstart, True,
+				XtNface, callData->face,
+				XtNpos, callData->position,
+				XtNdirection, callData->direction, NULL);
+#endif
+			XtVaSetValues(otherw1,
+				XtNstart, True,
+				XtNface, callData->face,
+				XtNpos, callData->position,
+				XtNdirection, callData->direction, NULL);
+			SET_STARTED(w, True);
+			break;
+#endif
 #ifdef WINVER
-		case SKEWB_DIM:
+		case PUZZLE_DIM:
 			dim++;
 #ifdef HAVE_OPENGL
 			if (dim > 4)
@@ -996,7 +1034,7 @@ CallbackSkewb(Widget w, caddr_t clientData, skewbCallbackStruct * callData)
 
 #ifdef WINVER
 void
-SetSkewbMove(SkewbWidget w, int reason, int face, int position, int direction)
+setPuzzleMove(SkewbWidget w, int reason, int face, int position, int direction)
 {
 	/*Boolean orient, practice, cheat; */
 	int dim;
@@ -1009,16 +1047,20 @@ SetSkewbMove(SkewbWidget w, int reason, int face, int position, int direction)
 #endif
 	dim = w->skewb.dim;
 	switch (reason) {
-		case SKEWB_MOVED:
+		case PUZZLE_MOVED:
 			movesDsp++;
 			SET_STARTED(w, True);
 			break;
-		case SKEWB_CONTROL:
+		case PUZZLE_CONTROL:
 			break;
-		case SKEWB_UNDO:
+		case PUZZLE_UNDO:
 			movesDsp--;
 			SET_STARTED(w, True);
 			break;
+		case PUZZLE_REDO:
+			movesDsp++;
+			SET_STARTED(w, True);
+			break;
 	}
 	PrintStatus(messageDsp, movesDsp, dim);
 }
@@ -1047,15 +1089,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	if (GetFocus()) {
 		if (!widget.skewb.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			EnterSkewb(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			EnterPuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	} else {
 		if (widget.skewb.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			LeaveSkewb(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			LeavePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	}
@@ -1064,36 +1108,40 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			Initialize(&widget, brush);
 			break;
 		case WM_DESTROY:
-			DestroySkewb(brush);
+			DestroyPuzzle(brush);
 			break;
 		case WM_SIZE:
-			ResizeSkewb(&widget);
+			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
 		case WM_PAINT:
 			widget.core.hDC = BeginPaint(hWnd, &paint);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ExposeSkewb(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ExposePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 			break;
 		case WM_RBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			RandomizeSkewb(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			RandomizePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			SelectSkewb(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			SelectPuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0,
 				(GetKeyState(VK_TAB) >> 1) ? 1 : 0);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONUP:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ReleaseSkewb(&widget, LOWORD(lParam), HIWORD(lParam),
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ReleasePuzzle(&widget, LOWORD(lParam), HIWORD(lParam),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0,
 				(GetKeyState(VK_TAB) >> 1) ? 1 : 0);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
@@ -1101,7 +1149,8 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
 		case WM_MOUSEWHEEL:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
 			{
 				int zDelta = ((short) HIWORD(wParam));
 				POINT cursor, origin;
@@ -1110,13 +1159,13 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				ClientToScreen(hWnd, &origin);
 				(void) GetCursorPos(&cursor);
 				if (zDelta > (WHEEL_DELTA >> 1)) {
-					MoveSkewbInput(&widget,
+					MovePuzzleInput(&widget,
 						cursor.x - origin.x, cursor.y - origin.y,
 						TOP,
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0,
 				(GetKeyState(VK_TAB) >> 1) ? 1 : 0);
 				} else if (zDelta < -(WHEEL_DELTA >> 1)) {
-					MoveSkewbInput(&widget,
+					MovePuzzleInput(&widget,
 						cursor.x - origin.x, cursor.y - origin.y,
 						BOTTOM,
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0,
@@ -1129,55 +1178,65 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		case WM_COMMAND:
 			switch (LOWORD(wParam)) {
 				case IDM_GET:
-					GetSkewb(&widget);
-					ResizeSkewb(&widget);
+					GetPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_WRITE:
-					WriteSkewb(&widget);
+					WritePuzzle(&widget);
 					break;
 				case IDM_EXIT:
-					DestroySkewb(brush);
+					DestroyPuzzle(brush);
 					break;
 				case IDM_HIDE:
-					HideSkewb(&widget);
-					break;
-				case IDM_CLEAR:
-					ClearSkewb(&widget);
-					SizeSkewb(&widget);
-					(void) InvalidateRect(hWnd, NULL, TRUE);
+					HidePuzzle(&widget);
 					break;
 				case IDM_UNDO:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					UndoSkewb(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					UndoPuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
+				case IDM_REDO:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RedoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_CLEAR:
+					ClearPuzzle(&widget);
+					SizePuzzle(&widget);
+					(void) InvalidateRect(hWnd, NULL, TRUE);
+					break;
 				case IDM_RANDOMIZE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					RandomizeSkewb(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RandomizePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_PRACTICE:
-					PracticeSkewb(&widget);
-					SizeSkewb(&widget);
+					PracticePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_SOLVE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					SolveSkewb(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					SolvePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
-				case IDM_ORIENT:
-					OrientizeSkewb(&widget);
-					SizeSkewb(&widget);
+				case IDM_ORIENTIZE:
+					OrientizePuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_DIM:
-					(void) DimSkewb(&widget);
-					ResizeSkewb(&widget);
+					(void) DimPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_TR:
@@ -1194,11 +1253,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
-						(void) MoveSkewbInput(&widget,
+						(void) MovePuzzleInput(&widget,
 							cursor.x - origin.x, cursor.y - origin.y,
 							(int) LOWORD(wParam) - IDM_TR,
 							FALSE, FALSE);
@@ -1220,11 +1280,12 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						POINT cursor, origin;
 
 						widget.core.hDC = GetDC(hWnd);
-						(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+						(void) SelectObject(widget.core.hDC,
+							GetStockObject(NULL_PEN));
 						origin.x = 0, origin.y = 0;
 						ClientToScreen(hWnd, &origin);
 						(void) GetCursorPos(&cursor);
-						(void) MoveSkewbInput(&widget,
+						(void) MovePuzzleInput(&widget,
 							cursor.x - origin.x, cursor.y - origin.y,
 							(int) LOWORD(wParam) - IDM_CONTROL_TR,
 							TRUE, FALSE);
@@ -1306,7 +1367,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
 
 #else
 static void
-Usage(char * programName)
+Usage(char *programName)
 {
 	unsigned int i;
 
@@ -1339,12 +1400,13 @@ static XrmOptionDescRec options[] =
 	{(char *) "-face3", (char *) "*skewb.faceColor3", XrmoptionSepArg, NULL},
 	{(char *) "-face4", (char *) "*skewb.faceColor4", XrmoptionSepArg, NULL},
 	{(char *) "-face5", (char *) "*skewb.faceColor5", XrmoptionSepArg, NULL},
+	{(char *) "-fn", (char *) "*skewb.font", XrmoptionSepArg, NULL},
+	{(char *) "-font", (char *) "*skewb.font", XrmoptionSepArg, NULL},
+	{(char *) "-view", (char *) "*skewb.view", XrmoptionSepArg, NULL},
 	{(char *) "-orient", (char *) "*skewb.orient", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-noorient", (char *) "*skewb.orient", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-practice", (char *) "*skewb.practice", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-nopractice", (char *) "*skewb.practice", XrmoptionNoArg, (char *) "FALSE"},
-	{(char *) "-fn", (char *) "*skewb.font", XrmoptionSepArg, NULL},
-	{(char *) "-font", (char *) "*skewb.font", XrmoptionSepArg, NULL},
 	{(char *) "-userName", (char *) "*skewb.userName", XrmoptionSepArg, NULL},
 	{(char *) "-scoreFile", (char *) "*skewb.scoreFile", XrmoptionSepArg, NULL},
 	{(char *) "-scores", (char *) "*skewb.scoreOnly", XrmoptionNoArg, (char *) "TRUE"},
@@ -1353,23 +1415,23 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackSkewbPractice(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzlePractice(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(skewb2d, XtNmenu, 8, NULL); /* menu choice */
+		XtVaSetValues(skewb2d, XtNmenu, MENU_PRACTICE, NULL);
 	}
 }
 
 static void
-CallbackSkewbRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct * cbs)
+CallbackPuzzleRandomize(Widget w, XtPointer clientData, XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(skewb2d, XtNmenu, 5, NULL); /* menu choice */
+		XtVaSetValues(skewb2d, XtNmenu, MENU_RANDOMIZE, NULL);
 	}
 }
 
 static void
-OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	Boolean orient = cbs->set, practice;
 
@@ -1392,7 +1454,7 @@ OrientToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
 }
 
 static void
-PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct * cbs)
+PracticeToggle(Widget w, XtPointer clientData, XmToggleButtonCallbackStruct *cbs)
 {
 	Boolean orient, practice = cbs->set;
 
@@ -1425,9 +1487,9 @@ fileCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	if (val == 2)
+	if (val == MENU_EXIT)
 		exit(0);
-	XtVaSetValues(skewb2d, XtNmenu, val, NULL);
+	XtVaSetValues(skewb2d, XtNmenu, val, NULL); /* GWQ */
 }
 
 static void
@@ -1435,7 +1497,7 @@ playCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	XtVaSetValues(skewb2d, XtNmenu, val + 3, NULL); /* GWQ */
+	XtVaSetValues(skewb2d, XtNmenu, val + MENU_EXIT + 1, NULL);
 }
 
 static Widget
@@ -1533,8 +1595,9 @@ main(int argc, char **argv)
 	Widget movesRowCol, switchRowCol, messageRowCol;
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
-	XmString clearString, undoString, randomizeString, solveString;
-	XmString orientString, practiceString;
+	XmString undoString, redoString;
+	XmString clearString, randomizeString, practiceString, solveString;
+	XmString orientString, viewString;
 #endif
 
 	progDsp = argv[0];
@@ -1543,7 +1606,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#if HAVE_XPM
+#ifdef HAVE_XPM
 	{
 		XpmAttributes xpmAttributes;
 		XpmColorSymbol transparentColor[1] = {{NULL,
@@ -1558,18 +1621,18 @@ main(int argc, char **argv)
 		transparentColor[0].pixel = bg;
 		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) skewb_xpm, &skewbIcon, NULL,
+			(char **) skewb_xpm, &icon, NULL,
 			&xpmAttributes);
 	}
-	if (skewbIcon == (Pixmap) NULL)
+	if (icon == (Pixmap) NULL)
 #endif
-		skewbIcon = XCreateBitmapFromData(XtDisplay(topLevel),
+		icon = XCreateBitmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
 			(char *) skewb_bits,
 			skewb_width, skewb_height);
 #ifdef HAVE_MOTIF
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, skewbIcon,
+		XtNiconPixmap, icon,
 		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
@@ -1596,26 +1659,32 @@ main(int argc, char **argv)
 	XmStringFree(getString);
 	XmStringFree(writeString);
 	XmStringFree(quitString);
-	clearString = XmStringCreateSimple((char *) "Clear");
 	undoString = XmStringCreateSimple((char *) "Undo");
-	randomizeString = XmStringCreateSimple((char *) "Randomize");
+	redoString = XmStringCreateSimple((char *) "Redo");
+	clearString = XmStringCreateSimple((char *) "Clear");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
+	practiceString = XmStringCreateSimple((char *) "Practice");
 	solveString = XmStringCreateSimple((char *) "(Solve)");
 	orientString = XmStringCreateSimple((char *) "Orientize");
-	practiceString = XmStringCreateSimple((char *) "Practice");
+	viewString = XmStringCreateSimple((char *) "View");
 	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
-		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
+		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
 		XmVaPUSHBUTTON, orientString, 'O', NULL, NULL,
-		XmVaPUSHBUTTON, practiceString, 'P', NULL, NULL,
+		XmVaPUSHBUTTON, viewString, 'V', NULL, NULL,
 		NULL);
-	XmStringFree(clearString);
 	XmStringFree(undoString);
+	XmStringFree(redoString);
+	XmStringFree(clearString);
 	XmStringFree(randomizeString);
+	XmStringFree(practiceString);
 	XmStringFree(solveString);
 	XmStringFree(orientString);
-	XmStringFree(practiceString);
+	XmStringFree(viewString);
 	pullDownMenu = XmCreatePulldownMenu(menuBar,
 		(char *) "helpPullDown", NULL, 0);
 	widget = XtVaCreateManagedWidget("Help",
@@ -1626,7 +1695,7 @@ main(int argc, char **argv)
 	widget = XtVaCreateManagedWidget("Description",
 		xmPushButtonGadgetClass, pullDownMenu,
 		XmNmnemonic, 'D', NULL);
-		XtAddCallback(widget, XmNactivateCallback, helpCB, (char *) 0);
+	XtAddCallback(widget, XmNactivateCallback, helpCB, (char *) 0);
 	widget = XtVaCreateManagedWidget("Features",
 		xmPushButtonGadgetClass, pullDownMenu,
 		XmNmnemonic, 'F', NULL);
@@ -1659,11 +1728,11 @@ main(int argc, char **argv)
 	practiceDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to toggle the practice mode?",
 		(char *) "Practice Query",
-		(XtCallbackProc) CallbackSkewbPractice);
+		(XtCallbackProc) CallbackPuzzlePractice);
 	randomizeDialog = createQuery(topLevel,
 		(char *) "Are you sure you want to randomize?",
 		(char *) "Randomize Query",
-		(XtCallbackProc) CallbackSkewbRandomize);
+		(XtCallbackProc) CallbackPuzzleRandomize);
 	mainPanel = XtCreateManagedWidget("mainPanel",
 		xmPanedWindowWidgetClass, menuBarPanel,
 		NULL, 0);
@@ -1750,7 +1819,7 @@ main(int argc, char **argv)
 	XtVaSetValues(skewb2d,
 		XtNheight, 200, NULL);
 	XtAddCallback(skewb2d, XtNselectCallback,
-		(XtCallbackProc) CallbackSkewb, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	skewb3d = XtCreateManagedWidget("skewb",
 		skewb3dWidgetClass, mainPanel, NULL, 0);
 	XtVaSetValues(skewb3d,
@@ -1763,24 +1832,24 @@ main(int argc, char **argv)
 #endif
 #else
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, skewbIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 	shell = XtCreateApplicationShell(argv[0],
 		topLevelShellWidgetClass, NULL, 0);
 	XtVaSetValues(shell,
-		XtNiconPixmap, skewbIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 #ifdef HAVE_OPENGL
 	shellGL = XtCreateApplicationShell(argv[0],
 		topLevelShellWidgetClass, NULL, 0);
 	XtVaSetValues(shellGL,
-		XtNiconPixmap, skewbIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 #endif
 	skewb2d = XtCreateManagedWidget("skewb",
 		skewb2dWidgetClass, topLevel, NULL, 0);
 	XtAddCallback(skewb2d, XtNselectCallback,
-		(XtCallbackProc) CallbackSkewb, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	skewb3d = XtCreateManagedWidget("skewb",
 		skewb3dWidgetClass, shell, NULL, 0);
 #ifdef HAVE_OPENGL
@@ -1789,10 +1858,10 @@ main(int argc, char **argv)
 #endif
 #endif
 	XtAddCallback(skewb3d, XtNselectCallback,
-		(XtCallbackProc) CallbackSkewb, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 #ifdef HAVE_OPENGL
 	XtAddCallback(skewbGL, XtNselectCallback,
-		(XtCallbackProc) CallbackSkewb, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 #endif
 	Initialize();
 	XtRealizeWidget(topLevel);
