@@ -48,61 +48,61 @@
 #endif
 
 static void InitializeThreeD(Widget request, Widget renew);
-static void ExposeThreeD(Widget renew, XEvent * event, Region region);
+static void ExposeThreeD(Widget renew, XEvent *event, Region region);
 static void ResizeThreeD(ThreeDWidget w);
 static void DestroyThreeD(Widget old);
 static Boolean SetValuesThreeD(Widget current, Widget request, Widget renew);
-static void QuitThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void QuitThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void HideThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void HideThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void SurfaceThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void SurfaceThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void ObjectThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void ObjectThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void EnterThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void EnterThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int nArgs);
-static void LeaveThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void LeaveThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int nArgs);
-static void MoveRaisePsiThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MoveRaisePsiThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void MoveRaisePhiThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MoveRaisePhiThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void MoveLowerPhiThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MoveLowerPhiThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void MoveLowerThetaThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MoveLowerThetaThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void MoveLowerPsiThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MoveLowerPsiThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void MoveRaiseThetaThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MoveRaiseThetaThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void MoveRaisePhiThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MoveRaisePhiThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void MoveLowerThetaLowerPhiThreeD(ThreeDWidget w, XEvent * event,
+static void MoveLowerThetaLowerPhiThreeD(ThreeDWidget w, XEvent *event,
 	char **args, int n_args);
-static void MoveLowerThetaRaisePhiThreeD(ThreeDWidget w, XEvent * event,
+static void MoveLowerThetaRaisePhiThreeD(ThreeDWidget w, XEvent *event,
 	char **args, int n_args);
-static void MoveRaiseThetaLowerPhiThreeD(ThreeDWidget w, XEvent * event,
+static void MoveRaiseThetaLowerPhiThreeD(ThreeDWidget w, XEvent *event,
 	char **args, int n_args);
-static void MoveRaiseThetaRaisePhiThreeD(ThreeDWidget w, XEvent * event,
+static void MoveRaiseThetaRaisePhiThreeD(ThreeDWidget w, XEvent *event,
 	char **args, int n_args);
-static void MoveLeftThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MoveLeftThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void MoveRightThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MoveRightThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void MoveUpThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MoveUpThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void MoveDownThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MoveDownThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void MoveOutThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MoveOutThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void MoveInThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MoveInThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int n_args);
-static void SelectThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void SelectThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int nArgs);
-static void MotionThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void MotionThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int nArgs);
-static void ReleaseThreeD(ThreeDWidget w, XEvent * event, char **args,
+static void ReleaseThreeD(ThreeDWidget w, XEvent *event, char **args,
 	int nArgs);
 
 static char defaultTranslationsThreeD[] =
@@ -325,7 +325,7 @@ ThreeDClassRec threedClassRec = {
 WidgetClass threedWidgetClass = (WidgetClass) & threedClassRec;
 
 static void
-SetThreeD(ThreeDWidget w, int reason)
+setThreed(ThreeDWidget w, int reason)
 {
 	threedCallbackStruct cb;
 
@@ -695,7 +695,7 @@ readLine(FILE *fp, char *string, char special)
 				parsed = True;
 			} else {
 				string[j] = buffer[i];
-				if (string[j] == '\0')	
+				if (string[j] == '\0')
 					parsed = True;
 				i++;
 				j++;
@@ -1159,7 +1159,7 @@ SetAllColors(ThreeDWidget w)
 }
 
 static void
-QuitThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
+QuitThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	XtCloseDisplay(XtDisplay(w));
 	exit(0);
@@ -1338,7 +1338,7 @@ ExposeThreeD(
 #ifdef WINVER
 ThreeDWidget w
 #else
-Widget renew, XEvent * event, Region region
+Widget renew, XEvent *event, Region region
 #endif
 )
 {
@@ -1358,11 +1358,11 @@ static
 void
 HideThreeD(ThreeDWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
-	SetThreeD(w, THREED_HIDE);
+	setThreed(w, THREED_HIDE);
 }
 
 #ifndef WINVER
@@ -1373,7 +1373,7 @@ SelectThreeD(ThreeDWidget w
 #ifdef WINVER
 , const int x, const int y/*, const int control*/
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1394,7 +1394,7 @@ MotionThreeD(ThreeDWidget w
 #ifdef WINVER
 , const int x, const int y/*, const int shift, const int control*/
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1414,7 +1414,7 @@ MotionThreeD(ThreeDWidget w
 		NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngleTheta;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAnglePhi;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 	w->threed.currentPosition.x = x;
 	w->threed.currentPosition.y = y;
@@ -1426,7 +1426,7 @@ static
 void
 ReleaseThreeD(ThreeDWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1437,13 +1437,13 @@ ReleaseThreeD(ThreeDWidget w
 
 #ifdef WINVER
 void
-SetSurfaceThreeD(ThreeDWidget w , Boolean surface)
+setSurfaceThreeD(ThreeDWidget w , Boolean surface)
 {
 	w->threed.surface = surface;
 }
 
 void
-SetObjectThreeD(ThreeDWidget w, int object3D)
+setObjectThreeD(ThreeDWidget w, int object3D)
 {
 	w->threed.object = object3D;
 }
@@ -1453,25 +1453,25 @@ SetObjectThreeD(ThreeDWidget w, int object3D)
 static
 void
 SurfaceThreeD(ThreeDWidget w
-, XEvent * event, char **args, int n_args
+, XEvent *event, char **args, int n_args
 )
 {
-	SetThreeD(w, THREED_SURFACE);
+	setThreed(w, THREED_SURFACE);
 }
 
 static
 void
 ObjectThreeD(ThreeDWidget w
-, XEvent * event, char **args, int n_args
+, XEvent *event, char **args, int n_args
 )
 {
-	SetThreeD(w, THREED_OBJECT);
+	setThreed(w, THREED_OBJECT);
 }
 
 void
 EnterThreeD(ThreeDWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1482,7 +1482,7 @@ EnterThreeD(ThreeDWidget w
 void
 LeaveThreeD(ThreeDWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -1491,7 +1491,7 @@ LeaveThreeD(ThreeDWidget w
 }
 
 static void
-MoveLowerPhiThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
+MoveLowerPhiThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	int oldAngle = w->threed.angle.phi;
 
@@ -1499,12 +1499,12 @@ MoveLowerPhiThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
 	if (w->threed.angle.phi < MINDEGREES)
 		w->threed.angle.phi += NUM_DEGREES;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAngle;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveLowerThetaThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
+MoveLowerThetaThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	int oldAngle = w->threed.angle.theta;
 
@@ -1512,12 +1512,12 @@ MoveLowerThetaThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
 	if (w->threed.angle.theta < MINDEGREES)
 		w->threed.angle.theta += NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngle;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveRaiseThetaThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
+MoveRaiseThetaThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	int oldAngle = w->threed.angle.theta;
 
@@ -1525,12 +1525,12 @@ MoveRaiseThetaThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
 	if (w->threed.angle.theta > MAXDEGREES)
 		w->threed.angle.theta -= NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngle;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveRaisePhiThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
+MoveRaisePhiThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	int oldAngle = w->threed.angle.phi;
 
@@ -1538,12 +1538,12 @@ MoveRaisePhiThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
 	if (w->threed.angle.phi > MAXDEGREES)
 		w->threed.angle.phi -= NUM_DEGREES;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAngle;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveLowerPsiThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
+MoveLowerPsiThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	int oldAngle = w->threed.angle.psi;
 
@@ -1551,12 +1551,12 @@ MoveLowerPsiThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
 	if (w->threed.angle.psi < MINDEGREES)
 		w->threed.angle.psi += NUM_DEGREES;
 	w->threed.deltaAngle.psi = w->threed.angle.psi - oldAngle;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveRaisePsiThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
+MoveRaisePsiThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	int oldAngle = w->threed.angle.psi;
 
@@ -1564,12 +1564,12 @@ MoveRaisePsiThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
 	if (w->threed.angle.psi > MAXDEGREES)
 		w->threed.angle.psi -= NUM_DEGREES;
 	w->threed.deltaAngle.psi = w->threed.angle.psi - oldAngle;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveLowerThetaLowerPhiThreeD(ThreeDWidget w, XEvent * event,
+MoveLowerThetaLowerPhiThreeD(ThreeDWidget w, XEvent *event,
 		char **args, int n_args)
 {
 	int oldAngleTheta = w->threed.angle.theta;
@@ -1583,12 +1583,12 @@ MoveLowerThetaLowerPhiThreeD(ThreeDWidget w, XEvent * event,
 		w->threed.angle.phi += NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngleTheta;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAnglePhi;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveLowerThetaRaisePhiThreeD(ThreeDWidget w, XEvent * event,
+MoveLowerThetaRaisePhiThreeD(ThreeDWidget w, XEvent *event,
 		char **args, int n_args)
 {
 	int oldAngleTheta = w->threed.angle.theta;
@@ -1602,12 +1602,12 @@ MoveLowerThetaRaisePhiThreeD(ThreeDWidget w, XEvent * event,
 		w->threed.angle.phi -= NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngleTheta;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAnglePhi;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveRaiseThetaLowerPhiThreeD(ThreeDWidget w, XEvent * event,
+MoveRaiseThetaLowerPhiThreeD(ThreeDWidget w, XEvent *event,
 		char **args, int n_args)
 {
 	int oldAngleTheta = w->threed.angle.theta;
@@ -1621,12 +1621,12 @@ MoveRaiseThetaLowerPhiThreeD(ThreeDWidget w, XEvent * event,
 		w->threed.angle.phi += NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngleTheta;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAnglePhi;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveRaiseThetaRaisePhiThreeD(ThreeDWidget w, XEvent * event,
+MoveRaiseThetaRaisePhiThreeD(ThreeDWidget w, XEvent *event,
 		char **args, int n_args)
 {
 	int oldAngleTheta = w->threed.angle.theta;
@@ -1640,61 +1640,61 @@ MoveRaiseThetaRaisePhiThreeD(ThreeDWidget w, XEvent * event,
 		w->threed.angle.phi -= NUM_DEGREES;
 	w->threed.deltaAngle.theta = w->threed.angle.theta - oldAngleTheta;
 	w->threed.deltaAngle.phi = w->threed.angle.phi - oldAnglePhi;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveRightThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
+MoveRightThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	if (w->threed.distance.x < MAXDISTANCE)
 		w->threed.distance.x += DELTADISTANCE;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveLeftThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
+MoveLeftThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	if (w->threed.distance.x > MINDISTANCE)
 		w->threed.distance.x -= DELTADISTANCE;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveUpThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
+MoveUpThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	if (w->threed.distance.y < MAXDISTANCE)
 		w->threed.distance.y += DELTADISTANCE;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveDownThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
+MoveDownThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	if (w->threed.distance.y > MINDISTANCE)
 		w->threed.distance.y -= DELTADISTANCE;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveOutThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
+MoveOutThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	if (w->threed.distance.z < MAXDEPTH)
 		w->threed.distance.z += DELTADEPTH;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 
 static void
-MoveInThreeD(ThreeDWidget w, XEvent * event, char **args, int n_args)
+MoveInThreeD(ThreeDWidget w, XEvent *event, char **args, int n_args)
 {
 	if (w->threed.distance.z > MINDEPTH)
 		w->threed.distance.z -= DELTADEPTH;
-	SetThreeD(w, THREED_SET);
+	setThreed(w, THREED_SET);
 	RepaintGraphicsThreeD(w);
 }
 #endif
