@@ -34,7 +34,7 @@
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Threed Version 7.2\n"
+"Threed Version 7.2.2\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -119,7 +119,7 @@ Initialize(ThreeDWidget w, HBRUSH brush)
 }
 
 void
-SetThreeD(ThreeDWidget w, int reason)
+setThreed(ThreeDWidget w, int reason)
 {
 	switch (reason) {
 		case THREED_HIDE:
@@ -173,14 +173,16 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			break;
 		case WM_PAINT:
 			widget.core.hDC = BeginPaint(widget.core.hWnd, &paint);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
 			ExposeThreeD(&widget);
 			(void) EndPaint(hWnd, &paint);
 			break;
 		case WM_LBUTTONDOWN:
 			mousePressed = True;
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
 			SelectThreeD(&widget, LOWORD(lParam), HIWORD(lParam)
 				/*, (GetKeyState(VK_CONTROL) >> 1) ? 1 : 0*/);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
@@ -190,7 +192,8 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				if (mousePressed) {
 					mousePressed = False;
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
 					ReleaseThreeD(&widget/*, LOWORD(lParam), HIWORD(lParam)
 					((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
 					(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0*/);
@@ -199,7 +202,8 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				break;
 			}
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
 			MotionThreeD(&widget, LOWORD(lParam), HIWORD(lParam)
 				/*, ((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0*/);
@@ -208,7 +212,8 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 		case WM_LBUTTONUP:
 			mousePressed = False;
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
 			ReleaseThreeD(&widget/*, LOWORD(lParam), HIWORD(lParam)
 				((GetKeyState(VK_SHIFT) >> 1) || (GetKeyState(VK_CAPITAL) & 1)),
 				(GetKeyState(VK_CONTROL) >> 1) ? 1 : 0*/);
@@ -217,7 +222,8 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 #if (_WIN32_WINNT >= 0x0400) || (_WIN32_WINDOWS > 0x0400)
 		case WM_MOUSEWHEEL:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
 			{
 				int zDelta = ((short) HIWORD(wParam));
 				POINT cursor, origin;
@@ -266,7 +272,7 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						MF_UNCHECKED);
 					(void) CheckMenuItem(hMenu,
 						LOWORD(wParam), MF_CHECKED);
-					SetObjectThreeD(&widget, LOWORD(wParam) - IDM_TETRA);
+					setObjectThreeD(&widget, LOWORD(wParam) - IDM_TETRA);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_HIDE:
@@ -279,7 +285,7 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 						(unsigned int) widget.threed.surface + IDM_WIRE,
 						MF_UNCHECKED);
 					(void) CheckMenuItem(hMenu, LOWORD(wParam), MF_CHECKED);
-					SetSurfaceThreeD(&widget, (BOOL) (LOWORD(wParam) - IDM_WIRE));
+					setSurfaceThreeD(&widget, (BOOL) (LOWORD(wParam) - IDM_WIRE));
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_DISTXD:
@@ -617,7 +623,7 @@ static Pixmap threedIcon = None;
 static Widget topLevel, threed;
 
 static void
-Usage(char * programName)
+Usage(char *programName)
 {
 	unsigned int i;
 
@@ -1058,7 +1064,7 @@ int main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#if HAVE_XPM
+#ifdef HAVE_XPM
 	{
 		XpmAttributes xpmAttributes;
 		XpmColorSymbol transparentColor[1] = {{NULL,
