@@ -66,39 +66,66 @@
 #endif
 #endif
 
-static Boolean SetValuesTriangles(Widget current, Widget request, Widget renew);
-static void QuitTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void DestroyTriangles(Widget old);
-static void ResizeTriangles(TrianglesWidget w);
-static void SizeTriangles(TrianglesWidget w);
-static void InitializeTriangles(Widget request, Widget renew);
-static void ExposeTriangles(Widget renew, XEvent * event, Region region);
-static void HideTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void SelectTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void ReleaseTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void RandomizeTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void RandomizeTrianglesMaybe(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void RandomizeTriangles2(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void GetTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void WriteTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void ClearTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void UndoTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void SolveTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void IncrementTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void DecrementTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void SpeedTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void SlowTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void SoundTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void EnterTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void LeaveTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void MoveTrianglesTl(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void MoveTrianglesTr(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void MoveTrianglesLeft(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void MoveRightTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void MoveTrianglesBl(TrianglesWidget w, XEvent * event, char **args, int nArgs);
-static void MoveTrianglesBr(TrianglesWidget w, XEvent * event, char **args, int nArgs);
+static Boolean SetValuesPuzzle(Widget current, Widget request, Widget renew);
+static void QuitPuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void DestroyPuzzle(Widget old);
+static void ResizePuzzle(TrianglesWidget w);
+static void SizePuzzle(TrianglesWidget w);
+static void InitializePuzzle(Widget request, Widget renew);
+static void ExposePuzzle(Widget renew, XEvent *event, Region region);
+static void HidePuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SelectPuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ReleasePuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ClearPuzzleMaybe(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ClearPuzzle2(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void GetPuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void WritePuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void UndoPuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void RedoPuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void ClearPuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void RandomizePuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SolvePuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void IncrementPuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void DecrementPuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SpeedPuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SlowPuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void SoundPuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void EnterPuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void LeavePuzzle(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleTl(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleTr(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleLeft(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleRight(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleBl(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
+static void MovePuzzleBr(TrianglesWidget w,
+	XEvent *event, char **args, int nArgs);
 
-static char defaultTranslationsTriangles[] =
+static char defaultTranslationsPuzzle[] =
 "<KeyPress>q: Quit()\n\
  Ctrl<KeyPress>C: Quit()\n\
  <KeyPress>osfCancel: Hide()\n\
@@ -130,13 +157,14 @@ static char defaultTranslationsTriangles[] =
  <KeyPress>R15: MoveBr()\n\
  <Btn1Down>: Select()\n\
  <Btn1Up>: Release()\n\
- <KeyPress>r: Randomize()\n\
- <Btn3Down>: RandomizeMaybe()\n\
- <Btn3Down>(2+): Randomize2()\n\
+ <Btn3Down>: ClearMaybe()\n\
+ <Btn3Down>(2+): Clear2()\n\
  <KeyPress>g: Get()\n\
  <KeyPress>w: Write()\n\
- <KeyPress>c: Clear()\n\
  <KeyPress>u: Undo()\n\
+ <KeyPress>r: Redo()\n\
+ <KeyPress>c: Clear()\n\
+ <KeyPress>z: Randomize()\n\
  <KeyPress>s: Solve()\n\
  <KeyPress>i: Increment()\n\
  <KeyPress>d: Decrement()\n\
@@ -148,36 +176,37 @@ static char defaultTranslationsTriangles[] =
  <EnterWindow>: Enter()\n\
  <LeaveWindow>: Leave()";
 
-static XtActionsRec actionsListTriangles[] =
+static XtActionsRec actionsListPuzzle[] =
 {
-	{(char *) "Quit", (XtActionProc) QuitTriangles},
-	{(char *) "Hide", (XtActionProc) HideTriangles},
-	{(char *) "MoveTl", (XtActionProc) MoveTrianglesTl},
-	{(char *) "MoveTr", (XtActionProc) MoveTrianglesTr},
-	{(char *) "MoveLeft", (XtActionProc) MoveTrianglesLeft},
-	{(char *) "MoveRight", (XtActionProc) MoveRightTriangles},
-	{(char *) "MoveBl", (XtActionProc) MoveTrianglesBl},
-	{(char *) "MoveBr", (XtActionProc) MoveTrianglesBr},
-	{(char *) "Select", (XtActionProc) SelectTriangles},
-	{(char *) "Release", (XtActionProc) ReleaseTriangles},
-	{(char *) "Randomize", (XtActionProc) RandomizeTriangles},
-	{(char *) "RandomizeMaybe", (XtActionProc) RandomizeTrianglesMaybe},
-	{(char *) "Randomize2", (XtActionProc) RandomizeTriangles2},
-	{(char *) "Get", (XtActionProc) GetTriangles},
-	{(char *) "Write", (XtActionProc) WriteTriangles},
-	{(char *) "Clear", (XtActionProc) ClearTriangles},
-	{(char *) "Undo", (XtActionProc) UndoTriangles},
-	{(char *) "Solve", (XtActionProc) SolveTriangles},
-	{(char *) "Increment", (XtActionProc) IncrementTriangles},
-	{(char *) "Decrement", (XtActionProc) DecrementTriangles},
-	{(char *) "Speed", (XtActionProc) SpeedTriangles},
-	{(char *) "Slow", (XtActionProc) SlowTriangles},
-	{(char *) "Sound", (XtActionProc) SoundTriangles},
-	{(char *) "Enter", (XtActionProc) EnterTriangles},
-	{(char *) "Leave", (XtActionProc) LeaveTriangles}
+	{(char *) "Quit", (XtActionProc) QuitPuzzle},
+	{(char *) "Hide", (XtActionProc) HidePuzzle},
+	{(char *) "MoveTl", (XtActionProc) MovePuzzleTl},
+	{(char *) "MoveTr", (XtActionProc) MovePuzzleTr},
+	{(char *) "MoveLeft", (XtActionProc) MovePuzzleLeft},
+	{(char *) "MoveRight", (XtActionProc) MovePuzzleRight},
+	{(char *) "MoveBl", (XtActionProc) MovePuzzleBl},
+	{(char *) "MoveBr", (XtActionProc) MovePuzzleBr},
+	{(char *) "Select", (XtActionProc) SelectPuzzle},
+	{(char *) "Release", (XtActionProc) ReleasePuzzle},
+	{(char *) "ClearMaybe", (XtActionProc) ClearPuzzleMaybe},
+	{(char *) "Clear2", (XtActionProc) ClearPuzzle2},
+	{(char *) "Get", (XtActionProc) GetPuzzle},
+	{(char *) "Write", (XtActionProc) WritePuzzle},
+	{(char *) "Undo", (XtActionProc) UndoPuzzle},
+	{(char *) "Redo", (XtActionProc) RedoPuzzle},
+	{(char *) "Clear", (XtActionProc) ClearPuzzle},
+	{(char *) "Randomize", (XtActionProc) RandomizePuzzle},
+	{(char *) "Solve", (XtActionProc) SolvePuzzle},
+	{(char *) "Increment", (XtActionProc) IncrementPuzzle},
+	{(char *) "Decrement", (XtActionProc) DecrementPuzzle},
+	{(char *) "Speed", (XtActionProc) SpeedPuzzle},
+	{(char *) "Slow", (XtActionProc) SlowPuzzle},
+	{(char *) "Sound", (XtActionProc) SoundPuzzle},
+	{(char *) "Enter", (XtActionProc) EnterPuzzle},
+	{(char *) "Leave", (XtActionProc) LeavePuzzle}
 };
 
-static XtResource resourcesTriangles[] =
+static XtResource resourcesPuzzle[] =
 {
 	{XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
 	 XtOffset(TrianglesWidget, core.width),
@@ -265,29 +294,29 @@ TrianglesClassRec trianglesClassRec =
 		NULL,		/* class initialize */
 		NULL,		/* class part initialize */
 		FALSE,		/* class inited */
-		(XtInitProc) InitializeTriangles,	/* initialize */
+		(XtInitProc) InitializePuzzle,	/* initialize */
 		NULL,		/* initialize hook */
 		XtInheritRealize,	/* realize */
-		actionsListTriangles,	/* actions */
-		XtNumber(actionsListTriangles),		/* num actions */
-		resourcesTriangles,	/* resources */
-		XtNumber(resourcesTriangles),	/* num resources */
+		actionsListPuzzle,	/* actions */
+		XtNumber(actionsListPuzzle),	/* num actions */
+		resourcesPuzzle,	/* resources */
+		XtNumber(resourcesPuzzle),	/* num resources */
 		NULLQUARK,	/* xrm class */
 		TRUE,		/* compress motion */
 		TRUE,		/* compress exposure */
 		TRUE,		/* compress enterleave */
 		TRUE,		/* visible interest */
-		(XtWidgetProc) DestroyTriangles,	/* destroy */
-		(XtWidgetProc) ResizeTriangles,		/* resize */
-		(XtExposeProc) ExposeTriangles,		/* expose */
-		(XtSetValuesFunc) SetValuesTriangles,	/* set values */
+		(XtWidgetProc) DestroyPuzzle,	/* destroy */
+		(XtWidgetProc) ResizePuzzle,	/* resize */
+		(XtExposeProc) ExposePuzzle,	/* expose */
+		(XtSetValuesFunc) SetValuesPuzzle,	/* set values */
 		NULL,		/* set values hook */
 		XtInheritSetValuesAlmost,	/* set values almost */
 		NULL,		/* get values hook */
 		NULL,		/* accept focus */
 		XtVersion,	/* version */
 		NULL,		/* callback private */
-		defaultTranslationsTriangles,	/* tm table */
+		defaultTranslationsPuzzle,	/* tm table */
 		NULL,		/* query geometry */
 		NULL,		/* display accelerator */
 		NULL		/* extension */
@@ -300,7 +329,7 @@ TrianglesClassRec trianglesClassRec =
 WidgetClass trianglesWidgetClass = (WidgetClass) & trianglesClassRec;
 
 void
-SetTriangles(TrianglesWidget w, int reason)
+setPuzzle(TrianglesWidget w, int reason)
 {
 	trianglesCallbackStruct cb;
 
@@ -321,8 +350,8 @@ loadFont(TrianglesWidget w)
 		XUnloadFont(XtDisplay(w), w->triangles.fontInfo->fid);
 		XFreeFont(XtDisplay(w), w->triangles.fontInfo);
 	}
-	if ((w->triangles.fontInfo = XLoadQueryFont(display,
-			w->triangles.font)) == NULL) {
+	if (w->triangles.font && (w->triangles.fontInfo =
+			XLoadQueryFont(display, w->triangles.font)) == NULL) {
 		(void) sprintf(buf,
 			"Can not open %s font.\nAttempting %s font as alternate\n",
 			w->triangles.font, altfontname);
@@ -391,6 +420,9 @@ static Point rhombusUnit[ROWTYPES][ROWTYPES + 2] =
 static Point rhombusList[ROWTYPES][ROWTYPES + 2];
 #endif
 
+static TrianglesStack undo = {NULL, NULL, NULL, 0};
+static TrianglesStack redo = {NULL, NULL, NULL, 0};
+
 static void
 CheckTiles(TrianglesWidget w)
 {
@@ -798,17 +830,17 @@ DrawAllTiles(const TrianglesWidget w)
 static int
 MovableTile(TrianglesWidget w)
 {
-	int rowType = TRIANGLES_BLOCKED, l;
+	int rowType = PUZZLE_BLOCKED, l;
 
 	/* Are the spaces in a "row" with the mouse click?
 	   (If two, then one clicked on a space). */
 	for (l = 0; l < ROWTYPES; l++) {
 		if (w->triangles.currentRow[l] == w->triangles.spaceRow[DOWN][l] &&
 		w->triangles.currentRow[l] == w->triangles.spaceRow[UP][l]) {
-			if (rowType == TRIANGLES_BLOCKED) {
+			if (rowType == PUZZLE_BLOCKED) {
 				rowType = l;
 			} else {
-				return TRIANGLES_SPACE;
+				return PUZZLE_SPACE;
 			}
 		}
 	}
@@ -1034,8 +1066,9 @@ AnimateSlide(TrianglesWidget w, int dir, int fast, Boolean logMoves)
 #endif
 	for (aTile = 0; aTile < numTiles; aTile++) {
 		if (logMoves) {
-			SetTriangles(w, TRIANGLES_MOVED);
-			PutMove(direction);
+			setPuzzle(w, PUZZLE_MOVED);
+			setMove(&undo, direction);
+			flushMoves(w, &redo, FALSE);
 		}
 		posNext = TileNFromSpace(w, aTile + 1, rowType, orient,
 			(dir == UP) ? DOWN : UP);
@@ -1088,6 +1121,7 @@ ResetTiles(TrianglesWidget w)
 {
 	int i;
 
+	w->triangles.cw = !w->triangles.cw;
 	w->triangles.sizeSize = w->triangles.size * w->triangles.size;
 	if (w->triangles.tileOfPosition)
 		free(w->triangles.tileOfPosition);
@@ -1101,22 +1135,65 @@ ResetTiles(TrianglesWidget w)
 			malloc(sizeof (int) * w->triangles.sizeSize))) {
 		DISPLAY_ERROR("Not enough memory, exiting.");
 	}
-
+	/*for (i = 0; i < w->triangles.sizeSize - 2; i++)
+		w->triangles.tileOfPosition[i] = i + 1;
 	w->triangles.spacePosition[UP] = w->triangles.sizeSize - 1;
-	w->triangles.spaceRow[UP][TRBL] = w->triangles.size - 1; /* i */
-	w->triangles.spaceRow[UP][TLBR] = 0; /* j */
-	w->triangles.spaceRow[UP][ROW] = w->triangles.size - 1; /* k */
+	w->triangles.spaceRow[UP][TRBL] = w->triangles.size - 1; *//* i *//*
+	w->triangles.spaceRow[UP][TLBR] = 0; *//* j *//*
+	w->triangles.spaceRow[UP][ROW] = w->triangles.size - 1; *//* k *//*
+	w->triangles.tileOfPosition[w->triangles.sizeSize - 1] = 0;
 	if (w->triangles.size > 1) {
 		w->triangles.spacePosition[DOWN] = w->triangles.sizeSize - 2;
-		w->triangles.spaceRow[DOWN][TRBL] = w->triangles.size - 2; /* i */
-		w->triangles.spaceRow[DOWN][TLBR] = 0; /* j */
-		w->triangles.spaceRow[DOWN][ROW] = w->triangles.size - 1; /* k */
+		w->triangles.spaceRow[DOWN][TRBL] = w->triangles.size - 2; *//* i *//*
+		w->triangles.spaceRow[DOWN][TLBR] = 0; *//* j *//*
+		w->triangles.spaceRow[DOWN][ROW] = w->triangles.size - 1; *//* k *//*
 		w->triangles.tileOfPosition[w->triangles.sizeSize - 2] = -1;
+	}*/
+	for (i = 0; i < w->triangles.sizeSize; i++) {
+		int row, trbl, tlbr;
+
+		row = Row(i);
+		trbl = TrBl(i, row);
+		tlbr = TlBr(i, row);
+		if (w->triangles.cw) {
+			w->triangles.tileOfPosition[i] =
+				ToPosition(w->triangles.size - 1 - trbl,
+				tlbr, w->triangles.size - 1 - row) + 1;
+		} else {
+			w->triangles.tileOfPosition[i] =
+				ToPosition(w->triangles.size - 1 - tlbr,
+				w->triangles.size - 1 - row, trbl) + 1;
+		}
 	}
-	w->triangles.tileOfPosition[w->triangles.sizeSize - 1] = 0;
-	for (i = 1; i < w->triangles.sizeSize - 1; i++)
-		w->triangles.tileOfPosition[i - 1] = i;
-	FlushMoves(w);
+	if (w->triangles.cw) {
+		w->triangles.spacePosition[UP] = w->triangles.sizeSize - 2 * w->triangles.size + 1;
+		w->triangles.spaceRow[UP][TRBL] = 0; /* i */
+		w->triangles.spaceRow[UP][TLBR] = w->triangles.size - 1; /* j */
+		w->triangles.spaceRow[UP][ROW] = w->triangles.size - 1; /* k */
+		w->triangles.tileOfPosition[w->triangles.sizeSize - 2 * w->triangles.size + 1] = 0;
+		if (w->triangles.size > 1) {
+			w->triangles.spacePosition[DOWN] = w->triangles.sizeSize - 2 * w->triangles.size + 2;
+			w->triangles.spaceRow[DOWN][TRBL] = 0; /* i */
+			w->triangles.spaceRow[DOWN][TLBR] = w->triangles.size - 2; /* j */
+			w->triangles.spaceRow[DOWN][ROW] = w->triangles.size - 1; /* k */
+			w->triangles.tileOfPosition[w->triangles.sizeSize - 2 * w->triangles.size + 2] = -1;
+		}
+	} else {
+		w->triangles.spacePosition[UP] = 0;
+		w->triangles.spaceRow[UP][TRBL] = 0; /* i */
+		w->triangles.spaceRow[UP][TLBR] = 0; /* j */
+		w->triangles.spaceRow[UP][ROW] = 0; /* k */
+		w->triangles.tileOfPosition[0] = 0;
+		if (w->triangles.size > 1) {
+			w->triangles.spacePosition[DOWN] = 2;
+			w->triangles.spaceRow[DOWN][TRBL] = 0; /* i */
+			w->triangles.spaceRow[DOWN][TLBR] = 0; /* j */
+			w->triangles.spaceRow[DOWN][ROW] = 1; /* k */
+			w->triangles.tileOfPosition[2] = -1;
+		}
+	}
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	w->triangles.currentPosition = -1;
 	w->triangles.started = FALSE;
 }
@@ -1192,7 +1269,7 @@ DrawFrame(TrianglesWidget w, Pixmap dr, Boolean focus)
 static void
 MoveNoTiles(TrianglesWidget w)
 {
-	SetTriangles(w, TRIANGLES_IGNORE);
+	setPuzzle(w, PUZZLE_IGNORE);
 }
 
 static void
@@ -1325,11 +1402,12 @@ MoveTilesDir(TrianglesWidget w, int direction, int fast)
 }
 
 int
-MoveTrianglesDir(TrianglesWidget w, const int direction, const int fast)
+MovePuzzleDir(TrianglesWidget w, const int direction, const int fast)
 {
 	if (MoveTilesDir(w, direction, fast)) {
-		SetTriangles(w, TRIANGLES_MOVED);
-		PutMove(direction);
+		setPuzzle(w, PUZZLE_MOVED);
+		setMove(&undo, direction);
+		flushMoves(w, &redo, FALSE);
 		return TRUE;
 	}
 	return FALSE;
@@ -1339,18 +1417,18 @@ MoveTrianglesDir(TrianglesWidget w, const int direction, const int fast)
 static
 #endif
 int
-MoveTriangles(TrianglesWidget w, const int direction)
+MovePuzzle(TrianglesWidget w, const int direction)
 {
 	if (CheckSolved(w)) {
 		MoveNoTiles(w);
 		return FALSE;
 	}
-	if (!MoveTrianglesDir(w, direction, NORMAL)) {
-		SetTriangles(w, TRIANGLES_BLOCKED);
+	if (!MovePuzzleDir(w, direction, NORMAL)) {
+		setPuzzle(w, PUZZLE_BLOCKED);
 		return FALSE;
 	}
 	if (CheckSolved(w)) {
-		SetTriangles(w, TRIANGLES_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 	return TRUE;
 }
@@ -1390,16 +1468,19 @@ DiscreteMoves(TrianglesWidget w, int dir, int fast)
 		next = TileNFromSpace(w, 1, rowType, orient, UP);
 		orient = (orient == UP) ? DOWN : UP;
 		MoveTiles(w, next, orient, fast);
-		SetTriangles(w, TRIANGLES_MOVED);
+		setPuzzle(w, PUZZLE_MOVED);
 		switch (rowType) {
 		case TLBR:
-			PutMove(BR);
+			setMove(&undo, BR);
+			flushMoves(w, &redo, FALSE);
 			return;
 		case TRBL:
-			PutMove(BL);
+			setMove(&undo, BL);
+			flushMoves(w, &redo, FALSE);
 			return;
 		case ROW:
-			PutMove(RIGHT);
+			setMove(&undo, RIGHT);
+			flushMoves(w, &redo, FALSE);
 			return;
 		default:
 			{
@@ -1418,16 +1499,19 @@ DiscreteMoves(TrianglesWidget w, int dir, int fast)
 		next = TileNFromSpace(w, 1, rowType, orient, DOWN);
 		orient = (orient == UP) ? DOWN : UP;
 		MoveTiles(w, next, orient, fast);
-		SetTriangles(w, TRIANGLES_MOVED);
+		setPuzzle(w, PUZZLE_MOVED);
 		switch (rowType) {
 		case TLBR:
-			PutMove(TL);
+			setMove(&undo, TL);
+			flushMoves(w, &redo, FALSE);
 			return;
 		case TRBL:
-			PutMove(TR);
+			setMove(&undo, TR);
+			flushMoves(w, &redo, FALSE);
 			return;
 		case ROW:
-			PutMove(LEFT);
+			setMove(&undo, LEFT);
+			flushMoves(w, &redo, FALSE);
 			return;
 		default:
 			{
@@ -1520,7 +1604,7 @@ SelectTiles(TrianglesWidget w)
 		}
 	}
 	if (CheckSolved(w)) {
-		SetTriangles(w, TRIANGLES_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
@@ -1561,7 +1645,7 @@ RandomizeTiles(TrianglesWidget w)
 		int lastDirection = -1;
 		int randomDirection;
 
-		SetTriangles(w, TRIANGLES_RESET);
+		setPuzzle(w, PUZZLE_RESET);
 
 		if (w->triangles.size == 2)
 			big *= big;
@@ -1580,17 +1664,18 @@ RandomizeTiles(TrianglesWidget w)
 #endif
 
 			if ((randomDirection + COORD / 2) % COORD != lastDirection) {
-				if (MoveTrianglesDir(w, randomDirection, INSTANT))
+				if (MovePuzzleDir(w, randomDirection, INSTANT))
 					lastDirection = randomDirection;
 				else
 					big++;
 			}
 		}
-		FlushMoves(w);
-		SetTriangles(w, TRIANGLES_RANDOMIZE);
+		flushMoves(w, &undo, TRUE);
+		flushMoves(w, &redo, FALSE);
+		setPuzzle(w, PUZZLE_RANDOMIZE);
 	}
 	if (CheckSolved(w)) {
-		SetTriangles(w, TRIANGLES_SOLVED);
+		setPuzzle(w, PUZZLE_SOLVED);
 	}
 }
 
@@ -1637,15 +1722,16 @@ GetTiles(TrianglesWidget w)
 		}
 #endif
 	}
-	FlushMoves(w);
+	flushMoves(w, &undo, TRUE);
+	flushMoves(w, &redo, FALSE);
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &size);
 	if (size >= MINTILES) {
 		for (i = w->triangles.size; i < size; i++) {
-			SetTriangles(w, TRIANGLES_INC);
+			setPuzzle(w, PUZZLE_INC);
 		}
 		for (i = w->triangles.size; i > size; i--) {
-			SetTriangles(w, TRIANGLES_DEC);
+			setPuzzle(w, PUZZLE_DEC);
 		}
 	} else {
 		stringCat(&buf1, name, " corrupted: size ");
@@ -1665,10 +1751,10 @@ GetTiles(TrianglesWidget w)
 #endif
 	while ((c = getc(fp)) != EOF && c != SYMBOL);
 	(void) fscanf(fp, "%d", &moves);
-	ScanStartPosition(fp, w);
-	SetTriangles(w, TRIANGLES_RESTORE);
-	SetStartPosition(w);
-	ScanMoves(fp, w, moves);
+	scanStartPosition(fp, w);
+	setPuzzle(w, PUZZLE_RESTORE);
+	setStartPosition(w);
+	scanMoves(fp, w, moves);
 	(void) fclose(fp);
 	(void) printf("%s: size %d, moves %d.\n", name, size, moves);
 	free(lname);
@@ -1719,35 +1805,28 @@ WriteTiles(TrianglesWidget w)
 #endif
 	}
 	(void) fprintf(fp, "size%c %d\n", SYMBOL, w->triangles.size);
-	(void) fprintf(fp, "moves%c %d\n", SYMBOL, NumMoves());
-	PrintStartPosition(fp, w);
-	PrintMoves(fp);
+	(void) fprintf(fp, "moves%c %d\n", SYMBOL,
+		numMoves(&undo));
+	printStartPosition(fp, w);
+	printMoves(fp, &undo);
 	(void) fclose(fp);
 	(void) printf("Saved to %s.\n", name);
 	free(lname);
 	free(fname);
 }
 
-static void
-ClearTiles(TrianglesWidget w)
-{
-	if (w->triangles.currentPosition >= 0)
-		return;
-	ResetTiles(w);
-	DrawAllTiles(w);
-	SetTriangles(w, TRIANGLES_RESET);
-}
-
 static void
 UndoTiles(TrianglesWidget w)
 {
-	if (MadeMoves() && w->triangles.currentPosition < 0) {
+	if (madeMoves(&undo) &&
+			w->triangles.currentPosition < 0) {
 		int direction;
 
-		GetMove(&direction);
+		getMove(&undo, &direction);
+		setMove(&redo, direction);
 		direction = (direction + (COORD / 2)) % COORD;
 		if (MoveTilesDir(w, direction, DOUBLE)) {
-			SetTriangles(w, TRIANGLES_UNDO);
+			setPuzzle(w, PUZZLE_UNDO);
 		} else {
 			char *buf1, *buf2;
 
@@ -1760,18 +1839,51 @@ UndoTiles(TrianglesWidget w)
 	}
 }
 
+static void
+RedoTiles(TrianglesWidget w)
+{
+	if (madeMoves(&redo) &&
+			w->triangles.currentPosition < 0) {
+		int direction;
+
+		getMove(&redo, &direction);
+		setMove(&undo, direction);
+		if (MoveTilesDir(w, direction, DOUBLE)) {
+			setPuzzle(w, PUZZLE_REDO);
+		} else {
+			char *buf1, *buf2;
+
+			intCat(&buf1, "Move ", direction);
+			stringCat(&buf2, buf1, " can not be made");
+			free(buf1);
+			DISPLAY_WARNING(buf2);
+			free(buf2);
+		}
+	}
+}
+
+static void
+ClearTiles(TrianglesWidget w)
+{
+	if (w->triangles.currentPosition >= 0)
+		return;
+	ResetTiles(w);
+	DrawAllTiles(w);
+	setPuzzle(w, PUZZLE_RESET);
+}
+
 static void
 SolveTiles(TrianglesWidget w)
 {
 	if (CheckSolved(w) || w->triangles.currentPosition >= 0)
 		return;
-	SetTriangles(w, TRIANGLES_SOLVE_MESSAGE);
+	setPuzzle(w, PUZZLE_SOLVE_MESSAGE);
 }
 
 static void
 IncrementTiles(TrianglesWidget w)
 {
-	SetTriangles(w, TRIANGLES_INC);
+	setPuzzle(w, PUZZLE_INC);
 }
 
 static void
@@ -1779,7 +1891,7 @@ DecrementTiles(TrianglesWidget w)
 {
 	if (w->triangles.size <= MINTILES)
 		return;
-	SetTriangles(w, TRIANGLES_DEC);
+	setPuzzle(w, PUZZLE_DEC);
 }
 
 static void
@@ -1789,7 +1901,7 @@ SpeedTiles(TrianglesWidget w)
 	if (w->triangles.delay < 0)
 		w->triangles.delay = 0;
 #ifdef HAVE_MOTIF
-	SetTriangles(w, TRIANGLES_SPEED);
+	setPuzzle(w, PUZZLE_SPEED);
 #endif
 }
 
@@ -1798,7 +1910,7 @@ SlowTiles(TrianglesWidget w)
 {
 	w->triangles.delay += 5;
 #ifdef HAVE_MOTIF
-	SetTriangles(w, TRIANGLES_SPEED);
+	setPuzzle(w, PUZZLE_SPEED);
 #endif
 }
 
@@ -1830,7 +1942,7 @@ darker(const int light)
 
 
 static void
-SetValuesTriangles(TrianglesWidget w)
+SetValuesPuzzle(TrianglesWidget w)
 {
 	struct tagColor {
 		int red, green, blue;
@@ -1895,8 +2007,17 @@ SetValuesTriangles(TrianglesWidget w)
 }
 
 void
-DestroyTriangles(HBRUSH brush)
+DestroyPuzzle(TrianglesWidget w, HBRUSH brush)
 {
+	if (w->core.memDC) {
+		if (w->triangles.bufferTiles != NULL) {
+			DeleteObject(w->triangles.bufferTiles);
+		}
+		DeleteDC(w->core.memDC);
+		w->core.memDC = NULL;
+	}
+	deleteMoves(&undo);
+	deleteMoves(&redo);
 	(void) DeleteObject(brush);
 	PostQuitMessage(0);
 }
@@ -2026,7 +2147,7 @@ SetAllColors(TrianglesWidget w)
 }
 
 static Boolean
-SetValuesTriangles(Widget current, Widget request, Widget renew)
+SetValuesPuzzle(Widget current, Widget request, Widget renew)
 {
 	TrianglesWidget c = (TrianglesWidget) current, w = (TrianglesWidget) renew;
 	Boolean redraw = FALSE;
@@ -2049,11 +2170,11 @@ SetValuesTriangles(Widget current, Widget request, Widget renew)
 	}
 	if (w->triangles.size != c->triangles.size ||
 			w->triangles.base != c->triangles.base) {
-		SizeTriangles(w);
+		SizePuzzle(w);
 		redraw = True;
 	} else if (w->triangles.offset.x != c->triangles.offset.x ||
 			w->triangles.offset.y != c->triangles.offset.y) {
-		ResizeTriangles(w);
+		ResizePuzzle(w);
 		redraw = True;
 	}
 	if (w->triangles.delay != c->triangles.delay) {
@@ -2065,37 +2186,40 @@ SetValuesTriangles(Widget current, Widget request, Widget renew)
 
 		w->triangles.menu = -1;
 		switch (menu) {
-		case 0:
+		case MENU_GET:
 			GetTiles(w);
 			break;
-		case 1:
+		case MENU_WRITE:
 			WriteTiles(w);
 			break;
-		case 3:
-			ClearTiles(w);
-			break;
-		case 4:
+		case MENU_UNDO:
 			UndoTiles(w);
 			break;
-		case 5:
+		case MENU_REDO:
+			RedoTiles(w);
+			break;
+		case MENU_CLEAR:
+			ClearTiles(w);
+			break;
+		case MENU_RANDOMIZE:
 			RandomizeTiles(w);
 			break;
-		case 6:
+		case MENU_SOLVE:
 			SolveTiles(w);
 			break;
-		case 7:
+		case MENU_INCREMENT:
 			IncrementTiles(w);
 			break;
-		case 8:
+		case MENU_DECREMENT:
 			DecrementTiles(w);
 			break;
-		case 9:
+		case MENU_SPEED:
 			SpeedTiles(w);
 			break;
-		case 10:
+		case MENU_SLOW:
 			SlowTiles(w);
 			break;
-		case 11:
+		case MENU_SOUND:
 			SoundTiles(w);
 			break;
 		default:
@@ -2112,24 +2236,19 @@ SetValuesTriangles(Widget current, Widget request, Widget renew)
 }
 
 static void
-QuitTriangles(TrianglesWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
+DestroyPuzzle(Widget old)
 {
-#ifdef WINVER
-	if (w->triangles.freeDC) {
-		if (w->triangles.bufferTiles != NULL) {
-			DeleteObject(w->triangles.bufferTiles);
-		}
-		DeleteDC(w->core.memDC);
-		w->core.memDC = NULL;
-	}
-#else
+	TrianglesWidget w = (TrianglesWidget) old;
 	Display *display = XtDisplay(w);
 	int sel;
 
+	XtReleaseGC(old, w->triangles.tileGC);
+	XtReleaseGC(old, w->triangles.borderGC);
+	XtReleaseGC(old, w->triangles.tileBrighterGC);
+	XtReleaseGC(old, w->triangles.tileDarkerGC);
+	XtReleaseGC(old, w->triangles.frameGC);
+	XtReleaseGC(old, w->triangles.inverseGC);
+	XtRemoveCallbacks(old, XtNselectCallback, w->triangles.select);
 	if (w->triangles.colormap != None) {
 		XInstallColormap(display, w->triangles.oldColormap);
 		XFreeColormap(display, w->triangles.colormap);
@@ -2141,23 +2260,15 @@ QuitTriangles(TrianglesWidget w
 		XUnloadFont(display, w->triangles.fontInfo->fid);
 		XFreeFont(display, w->triangles.fontInfo);
 	}
-	XtCloseDisplay(display);
-#endif
-	exit(0);
+	deleteMoves(&undo);
+	deleteMoves(&redo);
 }
 
 static void
-DestroyTriangles(Widget old)
+QuitPuzzle(TrianglesWidget w, XEvent *event, char **args, int nArgs)
 {
-	TrianglesWidget w = (TrianglesWidget) old;
-
-	XtReleaseGC(old, w->triangles.tileGC);
-	XtReleaseGC(old, w->triangles.borderGC);
-	XtReleaseGC(old, w->triangles.tileBrighterGC);
-	XtReleaseGC(old, w->triangles.tileDarkerGC);
-	XtReleaseGC(old, w->triangles.frameGC);
-	XtReleaseGC(old, w->triangles.inverseGC);
-	XtRemoveCallbacks(old, XtNselectCallback, w->triangles.select);
+	XtCloseDisplay(XtDisplay(w));
+	exit(0);
 }
 #endif
 
@@ -2279,7 +2390,7 @@ ResizeTiles(TrianglesWidget w)
 static
 #endif
 void
-ResizeTriangles(TrianglesWidget w)
+ResizePuzzle(TrianglesWidget w)
 {
 	double sqrt_3 = 1.73205080756887729352744634150587237;
 	Point tempSize;
@@ -2321,17 +2432,17 @@ ResizeTriangles(TrianglesWidget w)
 static
 #endif
 void
-SizeTriangles(TrianglesWidget w)
+SizePuzzle(TrianglesWidget w)
 {
 	ResetTiles(w);
-	ResizeTriangles(w);
+	ResizePuzzle(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-InitializeTriangles(
+InitializePuzzle(
 #ifdef WINVER
 TrianglesWidget w, HBRUSH brush
 #else
@@ -2340,7 +2451,7 @@ Widget request, Widget renew
 )
 {
 #ifdef WINVER
-	SetValuesTriangles(w);
+	SetValuesPuzzle(w);
 	brush = CreateSolidBrush(w->triangles.inverseGC);
 	SETBACK(w->core.hWnd, brush);
 	(void) SRAND(time(NULL));
@@ -2361,27 +2472,30 @@ Widget request, Widget renew
 	w->triangles.tileDarkerGC = NULL;
 	w->triangles.frameGC = NULL;
 	w->triangles.inverseGC = NULL;
+	w->triangles.image = NULL;
 #endif
 	w->triangles.focus = False;
 	loadFont(w);
 	w->triangles.tileOfPosition = NULL;
 	CheckTiles(w);
-	InitMoves();
+	newMoves(&undo);
+	newMoves(&redo);
 	w->triangles.numSlices = ((w->triangles.delay < MAXSLICES) ?
 		w->triangles.delay + 1 : MAXSLICES);
 	w->triangles.cheat = False;
-	SizeTriangles(w);
+	w->triangles.cw = (NRAND(2) == 0);  /* they take the same # of moves */
+	SizePuzzle(w);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-ExposeTriangles(
+ExposePuzzle(
 #ifdef WINVER
 TrianglesWidget w
 #else
-Widget renew, XEvent * event, Region region
+Widget renew, XEvent *event, Region region
 #endif
 )
 {
@@ -2401,24 +2515,24 @@ Widget renew, XEvent * event, Region region
 static
 #endif
 void
-HideTriangles(TrianglesWidget w
+HidePuzzle(TrianglesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
-	SetTriangles(w, TRIANGLES_HIDE);
+	setPuzzle(w, PUZZLE_HIDE);
 }
 
 #ifndef WINVER
 static
 #endif
 void
-SelectTriangles(TrianglesWidget w
+SelectPuzzle(TrianglesWidget w
 #ifdef WINVER
 , const int x, const int y
 #else
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2443,17 +2557,17 @@ SelectTriangles(TrianglesWidget w
 		if (rowType < 0) {
 			DrawTile(w, w->triangles.currentPosition,
 				w->triangles.currentPositionOrient,
-				rowType == TRIANGLES_SPACE, False, TRUE, 0, 0);
+				rowType == PUZZLE_SPACE, False, TRUE, 0, 0);
 			FLUSH(w);
 			Sleep(100);
 			DrawTile(w, w->triangles.currentPosition,
 				w->triangles.currentPositionOrient,
 				True, True, TRUE, 0, 0);
-			if (rowType != TRIANGLES_SPACE)
+			if (rowType != PUZZLE_SPACE)
 				DrawTile(w, w->triangles.currentPosition,
 					w->triangles.currentPositionOrient,
 					False, False, FALSE, 0, 0);
-			SetTriangles(w, rowType);
+			setPuzzle(w, rowType);
 			w->triangles.currentPosition = -1;
 			return;
 		}
@@ -2468,9 +2582,9 @@ SelectTriangles(TrianglesWidget w
 static
 #endif
 void
-ReleaseTriangles(TrianglesWidget w
+ReleasePuzzle(TrianglesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2484,43 +2598,30 @@ ReleaseTriangles(TrianglesWidget w
 	w->triangles.currentPosition = -1;
 }
 
-#ifndef WINVER
-static
-#endif
-void
-RandomizeTriangles(TrianglesWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	RandomizeTiles(w);
-}
-
 #ifndef WINVER
 static void
-RandomizeTrianglesMaybe(TrianglesWidget w
-, XEvent * event, char **args, int nArgs
+ClearPuzzleMaybe(TrianglesWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 	if (!w->triangles.started)
-		RandomizeTiles(w);
+		ClearTiles(w);
 #ifdef HAVE_MOTIF
 	else {
-		SetTriangles(w, TRIANGLES_RANDOMIZE_QUERY);
+		setPuzzle(w, PUZZLE_CLEAR_QUERY);
 	}
 #endif
 }
 
 static void
-RandomizeTriangles2(TrianglesWidget w
-, XEvent * event, char **args, int nArgs
+ClearPuzzle2(TrianglesWidget w
+, XEvent *event, char **args, int nArgs
 )
 {
 #ifdef HAVE_MOTIF
 	if (!w->triangles.started)
 #endif
-		RandomizeTiles(w);
+		ClearTiles(w);
 }
 #endif
 
@@ -2528,9 +2629,9 @@ RandomizeTriangles2(TrianglesWidget w
 static
 #endif
 void
-GetTriangles(TrianglesWidget w
+GetPuzzle(TrianglesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2541,9 +2642,9 @@ GetTriangles(TrianglesWidget w
 static
 #endif
 void
-WriteTriangles(TrianglesWidget w
+WritePuzzle(TrianglesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2554,22 +2655,9 @@ WriteTriangles(TrianglesWidget w
 static
 #endif
 void
-ClearTriangles(TrianglesWidget w
+UndoPuzzle(TrianglesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
-#endif
-)
-{
-	ClearTiles(w);
-}
-
-#ifndef WINVER
-static
-#endif
-void
-UndoTriangles(TrianglesWidget w
-#ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2580,9 +2668,48 @@ UndoTriangles(TrianglesWidget w
 static
 #endif
 void
-SolveTriangles(TrianglesWidget w
+RedoPuzzle(TrianglesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RedoTiles(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+ClearPuzzle(TrianglesWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	ClearTiles(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+RandomizePuzzle(TrianglesWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
+#endif
+)
+{
+	RandomizeTiles(w);
+}
+
+#ifndef WINVER
+static
+#endif
+void
+SolvePuzzle(TrianglesWidget w
+#ifndef WINVER
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2593,9 +2720,9 @@ SolveTriangles(TrianglesWidget w
 static
 #endif
 void
-IncrementTriangles(TrianglesWidget w
+IncrementPuzzle(TrianglesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2606,9 +2733,9 @@ IncrementTriangles(TrianglesWidget w
 static
 #endif
 void
-DecrementTriangles(TrianglesWidget w
+DecrementPuzzle(TrianglesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2619,9 +2746,9 @@ DecrementTriangles(TrianglesWidget w
 static
 #endif
 void
-SpeedTriangles(TrianglesWidget w
+SpeedPuzzle(TrianglesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2632,9 +2759,9 @@ SpeedTriangles(TrianglesWidget w
 static
 #endif
 void
-SlowTriangles(TrianglesWidget w
+SlowPuzzle(TrianglesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2645,9 +2772,9 @@ SlowTriangles(TrianglesWidget w
 static
 #endif
 void
-SoundTriangles(TrianglesWidget w
+SoundPuzzle(TrianglesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2658,9 +2785,9 @@ SoundTriangles(TrianglesWidget w
 static
 #endif
 void
-EnterTriangles(TrianglesWidget w
+EnterPuzzle(TrianglesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2672,9 +2799,9 @@ EnterTriangles(TrianglesWidget w
 static
 #endif
 void
-LeaveTriangles(TrianglesWidget w
+LeavePuzzle(TrianglesWidget w
 #ifndef WINVER
-, XEvent * event, char **args, int nArgs
+, XEvent *event, char **args, int nArgs
 #endif
 )
 {
@@ -2684,38 +2811,38 @@ LeaveTriangles(TrianglesWidget w
 
 #ifndef WINVER
 static void
-MoveTrianglesTl(TrianglesWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleTl(TrianglesWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveTriangles(w, TL);
+	(void) MovePuzzle(w, TL);
 }
 
 static void
-MoveTrianglesTr(TrianglesWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleTr(TrianglesWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveTriangles(w, TR);
+	(void) MovePuzzle(w, TR);
 }
 
 static void
-MoveTrianglesLeft(TrianglesWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleLeft(TrianglesWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveTriangles(w, LEFT);
+	(void) MovePuzzle(w, LEFT);
 }
 
 static void
-MoveRightTriangles(TrianglesWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleRight(TrianglesWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveTriangles(w, RIGHT);
+	(void) MovePuzzle(w, RIGHT);
 }
 
 static void
-MoveTrianglesBl(TrianglesWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleBl(TrianglesWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveTriangles(w, BL);
+	(void) MovePuzzle(w, BL);
 }
 
 static void
-MoveTrianglesBr(TrianglesWidget w, XEvent * event, char **args, int nArgs)
+MovePuzzleBr(TrianglesWidget w, XEvent *event, char **args, int nArgs)
 {
-	(void) MoveTriangles(w, BR);
+	(void) MovePuzzle(w, BR);
 }
 #endif
