@@ -81,23 +81,38 @@ typedef struct {
 } trianglesCallbackStruct;
 #endif
 
-#define TRIANGLES_HIDE (-8)
-#define TRIANGLES_RANDOMIZE_QUERY (-7)
-#define TRIANGLES_SOLVE_MESSAGE (-6)
-#define TRIANGLES_RESTORE (-5)
-#define TRIANGLES_RESET (-4)
-#define TRIANGLES_BLOCKED (-3)
-#define TRIANGLES_SPACE (-2)
-#define TRIANGLES_IGNORE (-1)
-#define TRIANGLES_MOVED 0
-#define TRIANGLES_SOLVED 1
-#define TRIANGLES_PRACTICE 2
-#define TRIANGLES_RANDOMIZE 3
-#define TRIANGLES_DEC 4
-#define TRIANGLES_INC 5
-#define TRIANGLES_COMPUTED 6
-#define TRIANGLES_UNDO 7
-#define TRIANGLES_SPEED 8
+#define PUZZLE_HIDE (-8)
+#define PUZZLE_CLEAR_QUERY (-7)
+#define PUZZLE_SOLVE_MESSAGE (-6)
+#define PUZZLE_RESTORE (-5)
+#define PUZZLE_RESET (-4)
+#define PUZZLE_BLOCKED (-3)
+#define PUZZLE_SPACE (-2)
+#define PUZZLE_IGNORE (-1)
+#define PUZZLE_MOVED 0
+#define PUZZLE_SOLVED 1
+#define PUZZLE_PRACTICE 2
+#define PUZZLE_RANDOMIZE 3
+#define PUZZLE_DEC 4
+#define PUZZLE_INC 5
+#define PUZZLE_COMPUTED 6
+#define PUZZLE_UNDO 7
+#define PUZZLE_REDO 8
+#define PUZZLE_SPEED 9
+
+#define MENU_GET 0
+#define MENU_WRITE 1
+#define MENU_EXIT 2
+#define MENU_UNDO 3
+#define MENU_REDO 4
+#define MENU_CLEAR 5
+#define MENU_RANDOMIZE 6
+#define MENU_SOLVE 7
+#define MENU_INCREMENT 8
+#define MENU_DECREMENT 9
+#define MENU_SPEED 10
+#define MENU_SLOW 11
+#define MENU_SOUND 12
 
 #define MINTILES 1
 #define DEFAULTTILES 4
