@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 2005	David Albert Bagley, bagleyd@tux.org
+#  Copyright (c) 1994 - 2006	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -33,30 +33,29 @@
 #include "Triangles.h"
 
 #ifdef WINVER
-extern void DestroyTriangles(HBRUSH brush);
-extern void ResizeTriangles(TrianglesWidget w);
-extern void SizeTriangles(TrianglesWidget w);
-extern void InitializeTriangles(TrianglesWidget w, HBRUSH brush);
-extern void ExposeTriangles(TrianglesWidget w);
-extern void HideTriangles(TrianglesWidget w);
-extern void SelectTriangles(TrianglesWidget w, const int x, const int y);
-extern void ReleaseTriangles(TrianglesWidget w);
-extern void RandomizeTriangles(TrianglesWidget w);
-extern void RandomizeTrianglesMaybe(TrianglesWidget w);
-extern void RandomizeTriangles2(TrianglesWidget w);
-extern void ClearTriangles(TrianglesWidget w);
-extern void GetTriangles(TrianglesWidget w);
-extern void WriteTriangles(TrianglesWidget w);
-extern void UndoTriangles(TrianglesWidget w);
-extern void SolveTriangles(TrianglesWidget w);
-extern void IncrementTriangles(TrianglesWidget w);
-extern void DecrementTriangles(TrianglesWidget w);
-extern void SpeedTriangles(TrianglesWidget w);
-extern void SlowTriangles(TrianglesWidget w);
-extern void SoundTriangles(TrianglesWidget w);
-extern void EnterTriangles(TrianglesWidget w);
-extern void LeaveTriangles(TrianglesWidget w);
-extern int MoveTriangles(TrianglesWidget w, const int direction);
+extern void DestroyPuzzle(TrianglesWidget w, HBRUSH brush);
+extern void ResizePuzzle(TrianglesWidget w);
+extern void SizePuzzle(TrianglesWidget w);
+extern void InitializePuzzle(TrianglesWidget w, HBRUSH brush);
+extern void ExposePuzzle(TrianglesWidget w);
+extern void HidePuzzle(TrianglesWidget w);
+extern void SelectPuzzle(TrianglesWidget w, const int x, const int y);
+extern void ReleasePuzzle(TrianglesWidget w);
+extern void GetPuzzle(TrianglesWidget w);
+extern void WritePuzzle(TrianglesWidget w);
+extern void UndoPuzzle(TrianglesWidget w);
+extern void RedoPuzzle(TrianglesWidget w);
+extern void ClearPuzzle(TrianglesWidget w);
+extern void RandomizePuzzle(TrianglesWidget w);
+extern void SolvePuzzle(TrianglesWidget w);
+extern void IncrementPuzzle(TrianglesWidget w);
+extern void DecrementPuzzle(TrianglesWidget w);
+extern void SpeedPuzzle(TrianglesWidget w);
+extern void SlowPuzzle(TrianglesWidget w);
+extern void SoundPuzzle(TrianglesWidget w);
+extern void EnterPuzzle(TrianglesWidget w);
+extern void LeavePuzzle(TrianglesWidget w);
+extern int MovePuzzle(TrianglesWidget w, const int direction);
 
 #else
 
@@ -100,12 +99,29 @@ extern TrianglesClassRec trianglesClassRec;
 #define DOUBLE 2
 #define INSTANT 3
 
+typedef struct _MoveRecord {
+#if 0
+	int direction;
+#endif
+	unsigned char packed;	/* This makes assumptions on the data. */
+} MoveRecord;
+
+typedef struct _MoveStack {
+	MoveRecord move;
+	struct _MoveStack *previous, *next;
+} MoveStack;
+
+typedef struct _TrianglesStack {
+	MoveStack *currMove, *lastMove, *firstMove;
+	int count;
+} TrianglesStack;
+
 typedef struct _TrianglesPart {
 	int        *tileOfPosition, spacePosition[MAXORIENT];
 	int         spaceRow[MAXORIENT][ROWTYPES];
 	int         currentPosition, currentRow[ROWTYPES];
 	int         currentPositionOrient;
-	Boolean     started, cheat, sound;
+	Boolean     started, cheat, sound, cw;
 	Boolean     mono, reverse, focus;
 	Boolean     scoreOnly, versionOnly;
 	int         delay, numSlices;
@@ -140,8 +156,8 @@ typedef struct _TrianglesRec {
 } TrianglesRec;
 
 extern int *startPosition;
-extern void SetTriangles(TrianglesWidget w, int reason);
-extern int MoveTrianglesDir(TrianglesWidget w, const int direction,
+extern void setPuzzle(TrianglesWidget w, int reason);
+extern int MovePuzzleDir(TrianglesWidget w, const int direction,
 	const int fast);
 
 #if 0
@@ -150,17 +166,18 @@ extern void SolveSomeTiles(TrianglesWidget w);
 #endif
 extern void DrawAllTiles(const TrianglesWidget w);
 extern Boolean CheckSolved(const TrianglesWidget w);
-extern void InitMoves(void);
-extern void PutMove(int direction);
-extern void GetMove(int *direction);
-extern int MadeMoves(void);
-extern void FlushMoves(TrianglesWidget w);
-extern int NumMoves(void);
-extern void ScanMoves(FILE * fp, TrianglesWidget w, int moves);
-extern void PrintMoves(FILE * fp);
-extern void ScanStartPosition(FILE * fp, TrianglesWidget w);
-extern void PrintStartPosition(FILE * fp, TrianglesWidget w);
-extern void SetStartPosition(TrianglesWidget w);
+extern void newMoves(TrianglesStack *s);
+extern void deleteMoves(TrianglesStack *s);
+extern void setMove(TrianglesStack *s, int direction);
+extern void getMove(TrianglesStack *s, int *direction);
+extern int madeMoves(TrianglesStack *s);
+extern void flushMoves(TrianglesWidget w, TrianglesStack *s, Boolean undo);
+extern int numMoves(TrianglesStack *s);
+extern void scanMoves(FILE *fp, TrianglesWidget w, int moves);
+extern void printMoves(FILE *fp, TrianglesStack *s);
+extern void scanStartPosition(FILE *fp, TrianglesWidget w);
+extern void printStartPosition(FILE *fp, TrianglesWidget w);
+extern void setStartPosition(TrianglesWidget w);
 extern int Row(const int pos);
 extern int TrBl(const int pos, const int posRow);
 extern int TlBr(const int pos, const int posRow);
