@@ -88,7 +88,7 @@ static TrianglesRec widget;
 
 #ifndef WINVER
 static const char aboutHelp[] = {
-"Triangles Version 7.2\n"
+"Triangles Version 7.2.4\n"
 "Send bugs (reports or fixes) to the author: "
 "David Bagley <bagleyd@tux.org>\n"
 "The latest version is at: "
@@ -125,23 +125,26 @@ static const char featuresHelp[] = {
 "the tile farthest from the spaces but still in line with the "
 "spaces.\n"
 "\n"
-"Click \"mouse-right\" button, or press \"R\" or \"r\" "
-"keys, to randomize the puzzle.\n"
+"Click \"mouse-right\" button, or press \"C\" or \"c\" "
+"keys, to clear the puzzle.\n"
 "\n"
-"Press \"I\" or \"i\" keys to increase the number of tiles.\n"
+"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
 "\n"
-"Press \"O\" or \"o\" keys to decrease the number of tiles.\n"
+"Press \"W\" or \"w\" keys to save (write) a puzzle.\n"
+"\n"
+"Press \"U\" or \"u\" keys to undo a move.\n"
+"\n"
+"Press \"R\" or \"r\" keys to redo a move.\n"
+"\n"
+"Press \"Z\" or \"z\" keys to randomize the puzzle.  New records "
+"can only be set from a cleared state.\n"
 "\n"
 "\"S\" or \"s\" keys reserved for the auto-solver "
 "(not implemented).\n"
 "\n"
-"Press \"U\" or \"u\" keys to undo a move.\n"
+"Press \"I\" or \"i\" keys to increase the number of tiles.\n"
 "\n"
-"Press \"G\" or \"g\" keys to get a saved puzzle.\n"
-"\n"
-"Press \"W\" or \"w\" keys to save a puzzle.\n"
-"\n"
-"Press \"C\" or \"c\" keys to clear a puzzle.\n"
+"Press \"O\" or \"o\" keys to decrease the number of tiles.\n"
 "\n"
 "Press \">\" or \".\" keys to speed up the movement of tiles.\n"
 "\n"
@@ -209,19 +212,20 @@ static Widget moves, record, message, tile;
 static char buff[21];
 static Widget descriptionDialog, featuresDialog;
 static Widget optionsDialog, referencesDialog, aboutDialog;
-static Widget solveDialog, randomizeDialog;
+static Widget solveDialog, clearDialog;
 static Arg arg[3];
 #else
 static char titleDsp[TITLELEN] = "";
 #endif
 static char scoreFileName[FILENAMELEN] = SCOREFILE;
 static char fileName[FILENAMELEN];
+static Boolean randomized = False;
 #ifdef WINVER
 #define MAXPROGNAME 80
 static char progDsp[MAXPROGNAME] = TITLE;
 static char userNameDsp[USERNAMELEN] = "Guest";
 #else
-static Pixmap trianglesIcon = None;
+static Pixmap icon = None;
 static Widget topLevel, triangles;
 static char *progDsp;
 static char userNameDsp[USERNAMELEN] = "";
@@ -442,7 +446,7 @@ Widget w
 	char *userName, *scoreFile;
 
 #ifdef WINVER
-	InitializeTriangles(w, brush);
+	InitializePuzzle(w, brush);
 
 	size = w->triangles.size;
 	userName = w->triangles.userName;
@@ -496,7 +500,7 @@ Widget w
 #ifdef WINVER
 		(void) strncpy(userNameDsp, userName, USERNAMELEN);
 #else
-		char * login = getlogin();
+		char *login = getlogin();
 
 		if (login == NULL) {
 			(void) strcpy(userNameDsp, "");
@@ -521,11 +525,11 @@ Widget w
 
 #ifdef WINVER
 void
-SetTriangles(TrianglesWidget w, int reason)
+setPuzzle(TrianglesWidget w, int reason)
 #else
 static void
-CallbackTriangles(Widget w, caddr_t clientData,
-		trianglesCallbackStruct * callData)
+CallbackPuzzle(Widget w, caddr_t clientData,
+		trianglesCallbackStruct *callData)
 #endif
 {
 #ifndef WINVER
@@ -544,7 +548,7 @@ CallbackTriangles(Widget w, caddr_t clientData,
 		XtNcheat, &cheat, NULL);
 #endif
 	switch (reason) {
-		case TRIANGLES_HIDE:
+		case PUZZLE_HIDE:
 #ifdef WINVER
 			ShowWindow(w->core.hWnd, SW_SHOWMINIMIZED);
 #else
@@ -554,16 +558,15 @@ CallbackTriangles(Widget w, caddr_t clientData,
 #endif
 			break;
 #ifndef WINVER
-		case TRIANGLES_RANDOMIZE_QUERY:
+		case PUZZLE_CLEAR_QUERY:
 #ifdef HAVE_MOTIF
-			XtManageChild(randomizeDialog);
+			XtManageChild(clearDialog);
 #else
-			XtVaSetValues(triangles,
-				XtNmenu, 5, NULL); /* menu choice */
+			XtVaSetValues(triangles, XtNmenu, MENU_CLEAR, NULL);
 #endif
 			break;
 #endif
-		case TRIANGLES_SOLVE_MESSAGE:
+		case PUZZLE_SOLVE_MESSAGE:
 #ifdef WINVER
 			(void) MessageBox(w->core.hWnd, solveHelp,
 				"Auto-Solve", MB_OK);
@@ -575,33 +578,34 @@ CallbackTriangles(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case TRIANGLES_RESTORE:
-		case TRIANGLES_RESET:
+		case PUZZLE_RESTORE:
+		case PUZZLE_RESET:
 			movesDsp = 0;
+			randomized = False;
 			break;
-		case TRIANGLES_BLOCKED:
+		case PUZZLE_BLOCKED:
 			(void) strncpy(messageDsp, "Blocked", MESSAGELEN);
 			break;
-		case TRIANGLES_SPACE:
+		case PUZZLE_SPACE:
 #if 0
 			/* Too annoying */
 			(void) strncpy(messageDsp, "Spaces can not slide",
 				MESSAGELEN);
 #endif
 			break;
-		case TRIANGLES_IGNORE:
-			(void) strncpy(messageDsp, "Randomize to start",
+		case PUZZLE_IGNORE:
+			(void) strncpy(messageDsp, "Clear to start",
 				MESSAGELEN);
 			break;
-		case TRIANGLES_MOVED:
+		case PUZZLE_MOVED:
 			movesDsp++;
 			SET_STARTED(w, True);
 			break;
-		case TRIANGLES_SOLVED:
+		case PUZZLE_SOLVED:
 			if (cheat)
 				(void) sprintf(messageDsp,
 					"No cheating %s!!", userNameDsp);
-			else if (HandleSolved(movesDsp, size))
+			else if (!randomized && HandleSolved(movesDsp, size))
 				(void) sprintf(messageDsp,
 					"Congratulations %s!!", userNameDsp);
 			else
@@ -609,11 +613,12 @@ CallbackTriangles(Widget w, caddr_t clientData,
 					MESSAGELEN);
 			SET_STARTED(w, False);
 			break;
-		case TRIANGLES_RANDOMIZE:
+		case PUZZLE_RANDOMIZE:
 			movesDsp = 0;
+			randomized = True;
 			SET_STARTED(w, False);
 			break;
-		case TRIANGLES_DEC:
+		case PUZZLE_DEC:
 			movesDsp = 0;
 			size--;
 			PrintRecord(size);
@@ -628,7 +633,7 @@ CallbackTriangles(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case TRIANGLES_INC:
+		case PUZZLE_INC:
 			movesDsp = 0;
 			size++;
 			PrintRecord(size);
@@ -643,15 +648,19 @@ CallbackTriangles(Widget w, caddr_t clientData,
 #endif
 #endif
 			break;
-		case TRIANGLES_COMPUTED:
+		case PUZZLE_COMPUTED:
 			SET_STARTED(w, False);
 			break;
-		case TRIANGLES_UNDO:
+		case PUZZLE_UNDO:
 			movesDsp--;
 			SET_STARTED(w, True);
 			break;
+		case PUZZLE_REDO:
+			movesDsp++;
+			SET_STARTED(w, True);
+			break;
 #if defined(HAVE_MOTIF) && defined(ANIMATE)
-		case TRIANGLES_SPEED:
+		case PUZZLE_SPEED:
 			{
 				int oldDelay, delay;
 
@@ -704,15 +713,17 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	if (GetFocus()) {
 		if (!widget.triangles.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			EnterTriangles(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			EnterPuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	} else {
 		if (widget.triangles.focus) {
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_BRUSH));
-			LeaveTriangles(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_BRUSH));
+			LeavePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 		}
 	}
@@ -721,84 +732,99 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			Initialize(&widget, brush);
 			break;
 		case WM_DESTROY:
-			DestroyTriangles(brush);
+			DestroyPuzzle(&widget, brush);
 			break;
 		case WM_SIZE:
-			ResizeTriangles(&widget);
+			ResizePuzzle(&widget);
 			(void) InvalidateRect(hWnd, NULL, TRUE);
 			break;
 		case WM_PAINT:
 			widget.core.hDC = BeginPaint(hWnd, &paint);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ExposeTriangles(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ExposePuzzle(&widget);
 			(void) EndPaint(hWnd, &paint);
 			break;
 		case WM_RBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			RandomizeTriangles(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ClearPuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONDOWN:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			SelectTriangles(&widget, LOWORD(lParam), HIWORD(lParam));
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			SelectPuzzle(&widget, LOWORD(lParam), HIWORD(lParam));
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_LBUTTONUP:
 			widget.core.hDC = GetDC(hWnd);
-			(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-			ReleaseTriangles(&widget);
+			(void) SelectObject(widget.core.hDC,
+				GetStockObject(NULL_PEN));
+			ReleasePuzzle(&widget);
 			(void) ReleaseDC(hWnd, widget.core.hDC);
 			break;
 		case WM_COMMAND:
 			switch (LOWORD(wParam)) {
 				case IDM_GET:
-					GetTriangles(&widget);
-					ResizeTriangles(&widget);
+					GetPuzzle(&widget);
+					ResizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_WRITE:
-					WriteTriangles(&widget);
+					WritePuzzle(&widget);
 					break;
 				case IDM_EXIT:
-					DestroyTriangles(brush);
+					DestroyPuzzle(&widget, brush);
 					break;
 				case IDM_HIDE:
-					HideTriangles(&widget);
-					break;
-				case IDM_CLEAR:
-					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					ClearTriangles(&widget);
-					(void) ReleaseDC(hWnd, widget.core.hDC);
+					HidePuzzle(&widget);
 					break;
 				case IDM_UNDO:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					UndoTriangles(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					UndoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_REDO:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RedoPuzzle(&widget);
+					(void) ReleaseDC(hWnd, widget.core.hDC);
+					break;
+				case IDM_CLEAR:
+					widget.core.hDC = GetDC(hWnd);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					ClearPuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_RANDOMIZE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					RandomizeTriangles(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					RandomizePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_SOLVE:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					SolveTriangles(&widget);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					SolvePuzzle(&widget);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_SPEED:
-					SpeedTriangles(&widget);
+					SpeedPuzzle(&widget);
 					break;
 				case IDM_SLOW:
-					SlowTriangles(&widget);
+					SlowPuzzle(&widget);
 					break;
 				case IDM_SOUND:
-					SoundTriangles(&widget);
+					SoundPuzzle(&widget);
 					break;
 				case IDM_TR:
 				case IDM_RIGHT:
@@ -807,18 +833,19 @@ WindowProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				case IDM_LEFT:
 				case IDM_TL:
 					widget.core.hDC = GetDC(hWnd);
-					(void) SelectObject(widget.core.hDC, GetStockObject(NULL_PEN));
-					(void) MoveTriangles(&widget, (int) LOWORD(wParam) - IDM_TR);
+					(void) SelectObject(widget.core.hDC,
+						GetStockObject(NULL_PEN));
+					(void) MovePuzzle(&widget, (int) LOWORD(wParam) - IDM_TR);
 					(void) ReleaseDC(hWnd, widget.core.hDC);
 					break;
 				case IDM_DEC:
-					DecrementTriangles(&widget);
-					SizeTriangles(&widget);
+					DecrementPuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_INC:
-					IncrementTriangles(&widget);
-					SizeTriangles(&widget);
+					IncrementPuzzle(&widget);
+					SizePuzzle(&widget);
 					(void) InvalidateRect(hWnd, NULL, TRUE);
 					break;
 				case IDM_ABOUT:
@@ -896,7 +923,7 @@ WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine,
 #else
 
 static void
-Usage(char * programName)
+Usage(char *programName)
 {
 	unsigned int i;
 
@@ -924,7 +951,6 @@ static XrmOptionDescRec options[] =
 	{(char *) "-bd", (char *) "*triangles.tileBorder", XrmoptionSepArg, NULL},
 	{(char *) "-border", (char *) "*triangles.tileBorder", XrmoptionSepArg, NULL},
 	{(char *) "-tile", (char *) "*triangles.tileColor", XrmoptionSepArg, NULL},
-	{(char *) "-size", (char *) "*triangles.size", XrmoptionSepArg, NULL},
 	{(char *) "-install", (char *) "*triangles.install", XrmoptionNoArg, (char *) "TRUE"},
 	{(char *) "-noinstall", (char *) "*triangles.install", XrmoptionNoArg, (char *) "FALSE"},
 	{(char *) "-picture", (char *) "*triangles.picture", XrmoptionSepArg, NULL},
@@ -935,6 +961,7 @@ static XrmOptionDescRec options[] =
 	{(char *) "-base", (char *) "*triangles.base", XrmoptionSepArg, NULL},
 	{(char *) "-fn", (char *) "*triangles.font", XrmoptionSepArg, NULL},
 	{(char *) "-font", (char *) "*triangles.font", XrmoptionSepArg, NULL},
+	{(char *) "-size", (char *) "*triangles.size", XrmoptionSepArg, NULL},
 	{(char *) "-userName", (char *) "*triangles.userName", XrmoptionSepArg, NULL},
 	{(char *) "-scoreFile", (char *) "*triangles.scoreFile", XrmoptionSepArg, NULL},
 	{(char *) "-scores", (char *) "*triangles.scoreOnly", XrmoptionNoArg, (char *) "TRUE"},
@@ -943,16 +970,16 @@ static XrmOptionDescRec options[] =
 
 #ifdef HAVE_MOTIF
 static void
-CallbackTrianglesRandomize(Widget w, XtPointer clientData,
-		XmAnyCallbackStruct * cbs)
+CallbackPuzzleClear(Widget w, XtPointer clientData,
+		XmAnyCallbackStruct *cbs)
 {
 	if (cbs->reason == XmCR_OK) {
-		XtVaSetValues(triangles, XtNmenu, 5, NULL); /* menu choice */
+		XtVaSetValues(triangles, XtNmenu, MENU_CLEAR, NULL);
 	}
 }
 
 static void
-TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int size = cbs->value, old;
 
@@ -972,7 +999,7 @@ TileSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
 
 #ifdef ANIMATE
 static void
-SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct * cbs)
+SpeedSlider(Widget w, XtPointer clientData, XmScaleCallbackStruct *cbs)
 {
 	int delay = MAXSPEED + MINSPEED - cbs->value - 1, oldDelay;
 
@@ -989,9 +1016,9 @@ fileCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	if (val == 2)
+	if (val == MENU_EXIT)
 		exit(0);
-	XtVaSetValues(triangles, XtNmenu, val, NULL);
+	XtVaSetValues(triangles, XtNmenu, val, NULL); /* GWQ */
 }
 
 static void
@@ -999,7 +1026,7 @@ playCB(Widget w, void *value, void *clientData)
 {
 	int val = (int) value;
 
-	XtVaSetValues(triangles, XtNmenu, val + 3, NULL); /* GWQ */
+	XtVaSetValues(triangles, XtNmenu, val + MENU_EXIT + 1, NULL);
 }
 
 static Widget
@@ -1097,7 +1124,8 @@ main(int argc, char **argv)
 	Widget movesRowCol, tilesRowCol, messageRowCol;
 	XmString fileString, playString;
 	XmString getString, writeString, quitString;
-	XmString clearString, undoString, randomizeString, solveString;
+	XmString undoString, redoString;
+	XmString clearString, randomizeString, solveString;
 	XmString incrementString, decrementString;
 	XmString speedString, slowString, soundString;
 #endif
@@ -1108,7 +1136,7 @@ main(int argc, char **argv)
 	if (argc != 1)
 		Usage(argv[0]);
 
-#if HAVE_XPM
+#ifdef HAVE_XPM
 	{
 		XpmAttributes xpmAttributes;
 		XpmColorSymbol transparentColor[1] = {{NULL,
@@ -1123,18 +1151,18 @@ main(int argc, char **argv)
 		transparentColor[0].pixel = bg;
 		(void) XpmCreatePixmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
-			(char **) triangles_xpm, &trianglesIcon, NULL,
+			(char **) triangles_xpm, &icon, NULL,
 			&xpmAttributes);
 	}
-	if (trianglesIcon == (Pixmap) NULL)
+	if (icon == (Pixmap) NULL)
 #endif
-		trianglesIcon = XCreateBitmapFromData(XtDisplay(topLevel),
+		icon = XCreateBitmapFromData(XtDisplay(topLevel),
 			RootWindowOfScreen(XtScreen(topLevel)),
 			(char *) triangles_bits,
 			triangles_width, triangles_height);
 #ifdef HAVE_MOTIF
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, trianglesIcon,
+		XtNiconPixmap, icon,
 		XmNkeyboardFocusPolicy, XmPOINTER, NULL); /* not XmEXPLICIT */
 	menuBarPanel = XtVaCreateManagedWidget("menuBarPanel",
 		xmPanedWindowWidgetClass, topLevel,
@@ -1161,9 +1189,10 @@ main(int argc, char **argv)
 	XmStringFree(getString);
 	XmStringFree(writeString);
 	XmStringFree(quitString);
-	clearString = XmStringCreateSimple((char *) "Clear");
 	undoString = XmStringCreateSimple((char *) "Undo");
-	randomizeString = XmStringCreateSimple((char *) "Randomize");
+	redoString = XmStringCreateSimple((char *) "Redo");
+	clearString = XmStringCreateSimple((char *) "Clear");
+	randomizeString = XmStringCreateSimple((char *) "RandomiZe");
 	solveString = XmStringCreateSimple((char *) "(Solve)");
 	incrementString = XmStringCreateSimple((char *) "Increment");
 	decrementString = XmStringCreateSimple((char *) "Decrement");
@@ -1171,9 +1200,10 @@ main(int argc, char **argv)
 	slowString = XmStringCreateSimple((char *) "<Slow");
 	soundString = XmStringCreateSimple((char *) "@Sound");
 	XmVaCreateSimplePulldownMenu(menuBar, (char *) "play_menu", 1, playCB,
-		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
 		XmVaPUSHBUTTON, undoString, 'U', NULL, NULL,
-		XmVaPUSHBUTTON, randomizeString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, redoString, 'R', NULL, NULL,
+		XmVaPUSHBUTTON, clearString, 'C', NULL, NULL,
+		XmVaPUSHBUTTON, randomizeString, 'Z', NULL, NULL,
 		XmVaPUSHBUTTON, solveString, 'S', NULL, NULL,
 		XmVaPUSHBUTTON, incrementString, 'I', NULL, NULL,
 		XmVaPUSHBUTTON, decrementString, 'D', NULL, NULL,
@@ -1181,8 +1211,9 @@ main(int argc, char **argv)
 		XmVaPUSHBUTTON, slowString, '<', NULL, NULL,
 		XmVaPUSHBUTTON, soundString, '@', NULL, NULL,
 		NULL);
-	XmStringFree(clearString);
 	XmStringFree(undoString);
+	XmStringFree(redoString);
+	XmStringFree(clearString);
 	XmStringFree(randomizeString);
 	XmStringFree(solveString);
 	XmStringFree(incrementString);
@@ -1230,10 +1261,10 @@ main(int argc, char **argv)
 		(char *) "About");
 	solveDialog = createHelp(menuBar, (char *) solveHelp,
 		(char *) "Solve");
-	randomizeDialog = createQuery(topLevel,
-		(char *) "Are you sure you want to randomize?",
-		(char *) "Randomize Query",
-		(XtCallbackProc) CallbackTrianglesRandomize);
+	clearDialog = createQuery(topLevel,
+		(char *) "Are you sure you want to start over?",
+		(char *) "Clear Query",
+		(XtCallbackProc) CallbackPuzzleClear);
 	mainPanel = XtCreateManagedWidget("mainPanel",
 		xmPanedWindowWidgetClass, menuBarPanel,
 		NULL, 0);
@@ -1279,7 +1310,7 @@ main(int argc, char **argv)
 		(void) XtVaCreateManagedWidget("mouseRightText",
 			xmLabelGadgetClass, movesRowCol,
 			XtVaTypedArg, XmNlabelString,
-			XmRString, "Randomize", 10, NULL);
+			XmRString, "Clear", 6, NULL);
 		(void) XtVaCreateManagedWidget("mouseRight",
 			xmLabelGadgetClass, movesRowCol,
 			XmNlabelType, XmPIXMAP,
@@ -1332,13 +1363,13 @@ main(int argc, char **argv)
 		trianglesWidgetClass, mainPanel, NULL, 0);
 #else
 	XtVaSetValues(topLevel,
-		XtNiconPixmap, trianglesIcon,
+		XtNiconPixmap, icon,
 		XtNinput, True, NULL);
 	triangles = XtCreateManagedWidget("triangles",
 		trianglesWidgetClass, topLevel, NULL, 0);
 #endif
 	XtAddCallback(triangles, XtNselectCallback,
-		(XtCallbackProc) CallbackTriangles, (XtPointer) NULL);
+		(XtCallbackProc) CallbackPuzzle, (XtPointer) NULL);
 	Initialize(triangles);
 	XtRealizeWidget(topLevel);
 	XGrabButton(XtDisplay(triangles), (unsigned int) AnyButton, AnyModifier,
